#!/usr/bin/env node --no-warnings
import os from "node:os";

import fs, { rmSync, readdirSync, statSync } from "fs";

import path, { join, dirname, resolve, relative } from "path";

import * as childProcess from "child_process";

import { execSync as execSync$1 } from "child_process";

import require$$1, { spawn, execSync } from "node:child_process";

import require$$0$1 from "node:events";

import require$$2 from "node:path";

import require$$3 from "node:fs";

import process$2 from "node:process";

import require$$0$2 from "stream";

import require$$2$1, { homedir } from "os";

import tty from "node:tty";

import * as readline$1 from "node:readline";

import { AsyncLocalStorage, AsyncResource } from "node:async_hooks";

import require$$0$3 from "tty";

import require$$0$4 from "util";

const MIN_NODE_VERSION = "22.0.0";

function isVersionNewer(currentVersion, requiredVersion) {
    if (currentVersion == requiredVersion) return !1;
    const [major, minor, patch] = currentVersion.split(".").map(Number), [reqMajor, reqMinor, reqPatch] = requiredVersion.split(".").map(Number);
    return major > reqMajor || major === reqMajor && minor >= reqMinor || major === reqMajor && minor == reqMinor && patch >= reqPatch;
}

function assertRuntimeVersion() {
    "undefined" != typeof Bun || isVersionNewer(process.versions.node, MIN_NODE_VERSION) || (console.error(`This tool requires Node.js v${MIN_NODE_VERSION} or higher.`, `You are using v${process.versions.node}.`), 
    process.exit(1));
}

var name = "mucho", version = "0.5.0", description = "", license = "MIT", type = "module", publishConfig = {
    access: "public"
}, main$2 = "./bin/index.mjs", scripts = {
    dev: "pnpm build && pnpm mucho",
    build: "rollup -c",
    watch: "rollup -c --watch",
    test: "pnpm test:unit",
    "test:unit": "jest -c ./jest-unit.config.ts --rootDir . --silent",
    "test:unit:watch": "jest -c ./jest-unit.config.ts --rootDir . --watch",
    mucho: "node ./bin/index.mjs",
    "changeset:version": 'changeset version && git add -A && git commit -m "chore: version"',
    "changeset:publish": "pnpm build && changeset publish",
    "style:fix": "prettier --write 'src/{*,**/*}.{ts,tsx,js,jsx,css,json,md}'"
}, bin = {
    mucho: "bin/index.mjs"
}, files = [ "/bin" ], dependencies = {
    "@commander-js/extra-typings": "^12.1.0",
    "@iarna/toml": "^2.2.5",
    "@inquirer/prompts": "^7.0.0",
    "@solana-program/compute-budget": "^0.6.1",
    "@solana/buffer-layout": "^4.0.1",
    "@solana/buffer-layout-utils": "^0.2.0",
    "@solana/web3.js": "^2.0.0",
    "cli-table3": "^0.6.5",
    commander: "^12.1.0",
    dotenv: "^16.4.5",
    inquirer: "^12.0.0",
    ora: "^8.1.0",
    picocolors: "^1.1.1",
    punycode: "^2.3.1",
    "shell-exec": "^1.1.2",
    yaml: "^2.6.0"
}, devDependencies = {
    "@changesets/changelog-github": "^0.5.0",
    "@changesets/cli": "^2.27.9",
    "@jest/types": "^29.6.3",
    "@rollup/plugin-commonjs": "^28.0.1",
    "@rollup/plugin-json": "^6.1.0",
    "@rollup/plugin-node-resolve": "^15.3.0",
    "@rollup/plugin-terser": "^0.4.4",
    "@rollup/plugin-typescript": "^12.1.1",
    "@swc/core": "^1.9.3",
    "@swc/jest": "^0.2.37",
    "@types/bn.js": "^5.1.6",
    "@types/inquirer": "^9.0.7",
    "@types/jest": "^29.5.14",
    "@types/node": "^22.7.6",
    "@types/prompts": "^2.4.9",
    jest: "^29.7.0",
    rollup: "^4.24.0",
    "ts-jest": "^29.2.5",
    "ts-node": "^10.9.2",
    tslib: "^2.8.0",
    typescript: "^5.6.3"
}, packageManager = "pnpm@9.1.0", keywords = [], homepage = "https://github.com/solana-developers/mucho#readme", bugs = {
    url: "https://github.com/solana-developers/mucho/issues"
}, repository = {
    name: "solana-developers/mucho",
    type: "git",
    url: "git+https://github.com/solana-developers/mucho.git"
}, author = "nickfrosty", contributors = [ {
    name: "Nick Frostbutter",
    url: "https://github.com/nickfrosty"
} ], packageJson = {
    name: name,
    version: version,
    description: description,
    license: license,
    type: type,
    publishConfig: publishConfig,
    main: main$2,
    scripts: scripts,
    bin: bin,
    files: files,
    dependencies: dependencies,
    devDependencies: devDependencies,
    packageManager: packageManager,
    keywords: keywords,
    homepage: homepage,
    bugs: bugs,
    repository: repository,
    author: author,
    contributors: contributors
};

function getAppInfo() {
    return {
        version: packageJson.version,
        name: packageJson.name
    };
}

var commonjsGlobal = "undefined" != typeof globalThis ? globalThis : "undefined" != typeof window ? window : "undefined" != typeof global ? global : "undefined" != typeof self ? self : {};

function getDefaultExportFromCjs(x) {
    return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, "default") ? x.default : x;
}

var picocolors$1 = {
    exports: {}
}, hasRequiredPicocolors;

function requirePicocolors() {
    if (hasRequiredPicocolors) return picocolors$1.exports;
    hasRequiredPicocolors = 1;
    let p = process || {}, argv = p.argv || [], env = p.env || {}, isColorSupported = !(env.NO_COLOR || argv.includes("--no-color")) && (!!env.FORCE_COLOR || argv.includes("--color") || "win32" === p.platform || (p.stdout || {}).isTTY && "dumb" !== env.TERM || !!env.CI), formatter = (open, close, replace = open) => input => {
        let string = "" + input, index = string.indexOf(close, open.length);
        return ~index ? open + replaceClose(string, close, replace, index) + close : open + string + close;
    }, replaceClose = (string, close, replace, index) => {
        let result = "", cursor = 0;
        do {
            result += string.substring(cursor, index) + replace, cursor = index + close.length, 
            index = string.indexOf(close, cursor);
        } while (~index);
        return result + string.substring(cursor);
    }, createColors = (enabled = isColorSupported) => {
        let f = enabled ? formatter : () => String;
        return {
            isColorSupported: enabled,
            reset: f("[0m", "[0m"),
            bold: f("[1m", "[22m", "[22m[1m"),
            dim: f("[2m", "[22m", "[22m[2m"),
            italic: f("[3m", "[23m"),
            underline: f("[4m", "[24m"),
            inverse: f("[7m", "[27m"),
            hidden: f("[8m", "[28m"),
            strikethrough: f("[9m", "[29m"),
            black: f("[30m", "[39m"),
            red: f("[31m", "[39m"),
            green: f("[32m", "[39m"),
            yellow: f("[33m", "[39m"),
            blue: f("[34m", "[39m"),
            magenta: f("[35m", "[39m"),
            cyan: f("[36m", "[39m"),
            white: f("[37m", "[39m"),
            gray: f("[90m", "[39m"),
            bgBlack: f("[40m", "[49m"),
            bgRed: f("[41m", "[49m"),
            bgGreen: f("[42m", "[49m"),
            bgYellow: f("[43m", "[49m"),
            bgBlue: f("[44m", "[49m"),
            bgMagenta: f("[45m", "[49m"),
            bgCyan: f("[46m", "[49m"),
            bgWhite: f("[47m", "[49m"),
            blackBright: f("[90m", "[39m"),
            redBright: f("[91m", "[39m"),
            greenBright: f("[92m", "[39m"),
            yellowBright: f("[93m", "[39m"),
            blueBright: f("[94m", "[39m"),
            magentaBright: f("[95m", "[39m"),
            cyanBright: f("[96m", "[39m"),
            whiteBright: f("[97m", "[39m"),
            bgBlackBright: f("[100m", "[49m"),
            bgRedBright: f("[101m", "[49m"),
            bgGreenBright: f("[102m", "[49m"),
            bgYellowBright: f("[103m", "[49m"),
            bgBlueBright: f("[104m", "[49m"),
            bgMagentaBright: f("[105m", "[49m"),
            bgCyanBright: f("[106m", "[49m"),
            bgWhiteBright: f("[107m", "[49m")
        };
    };
    return picocolors$1.exports = createColors(), picocolors$1.exports.createColors = createColors, 
    picocolors$1.exports;
}

var picocolorsExports = requirePicocolors(), picocolors = getDefaultExportFromCjs(picocolorsExports);

function titleMessage(msg, colorFn = picocolors.inverse) {
    console.log(colorFn(` ${msg} `));
}

function warnMessage(msg, highlight = !1) {
    console.log(picocolors.yellow(msg));
}

function cancelMessage(msg = "Operation canceled") {
    console.log(msg), process.exit();
}

function warningOutro(msg = "Operation canceled") {
    warnMessage(msg), process.exit();
}

function errorOutro(msg, title = "An error occurred") {
    errorMessage(msg, title), process.exit(1);
}

function errorMessage(err, title = null) {
    let message = "Unknown error";
    "string" == typeof err ? message = err : err instanceof Error && (message = err.message), 
    title ? (console.log(picocolors.bgRed(title)), console.log(message, "\n")) : console.log(picocolors.bgRed(` ${message} `), "\n");
}

var __defProp = Object.defineProperty, __defProps = Object.defineProperties, __getOwnPropDescs = Object.getOwnPropertyDescriptors, __getOwnPropSymbols = Object.getOwnPropertySymbols, __hasOwnProp = Object.prototype.hasOwnProperty, __propIsEnum = Object.prototype.propertyIsEnumerable, __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, {
    enumerable: !0,
    configurable: !0,
    writable: !0,
    value: value
}) : obj[key] = value, __spreadValues = (a, b) => {
    for (var prop in b || (b = {})) __hasOwnProp.call(b, prop) && __defNormalProp(a, prop, b[prop]);
    if (__getOwnPropSymbols) for (var prop of __getOwnPropSymbols(b)) __propIsEnum.call(b, prop) && __defNormalProp(a, prop, b[prop]);
    return a;
}, __spreadProps = (a, b) => __defProps(a, __getOwnPropDescs(b));

function shellExec(cmd, opts) {
    const executable = Array.isArray(cmd) ? cmd.join(";") : cmd, options = __spreadProps(__spreadValues({}, opts), {
        stdio: "pipe",
        cwd: process.cwd()
    }), {platform: platform} = process;
    try {
        const cmd2 = "win32" === platform ? "cmd" : "sh", arg = "win32" === platform ? "/C" : "-c", child = childProcess.spawn(cmd2, [ arg, executable ], options);
        return new Promise((resolve => {
            const stdoutList = [], stderrList = [];
            child.stdout && child.stdout.on("data", (data => {
                if (Buffer.isBuffer(data)) return stdoutList.push(data.toString());
                stdoutList.push(data);
            })), child.stderr && child.stderr.on("data", (data => {
                if (Buffer.isBuffer(data)) return stderrList.push(data.toString());
                stderrList.push(JSON.stringify(data));
            }));
            const getDefaultResult = () => {
                const stderr = stderrList.join("\n");
                return {
                    stdout: stdoutList.join("\n"),
                    stderr: stderr,
                    cmd: executable
                };
            };
            child.on("error", (error => resolve(__spreadProps(__spreadValues({}, getDefaultResult()), {
                error: error
            })))), child.on("close", (code => resolve(__spreadProps(__spreadValues({}, getDefaultResult()), {
                code: code
            }))));
        }));
    } catch (error) {
        return Promise.reject(error);
    }
}

var src_default$1 = shellExec;

const TOOL_CONFIG = {
    node: {
        version: "node --version"
    },
    rust: {
        pathSource: "$HOME/.cargo/env",
        version: "rustc --version"
    },
    solana: {
        dependencies: [ "rust" ],
        pathSource: "$HOME/.local/share/solana/install/active_release/bin",
        version: "solana --version"
    },
    avm: {
        dependencies: [ "rust" ],
        version: "avm --version"
    },
    anchor: {
        dependencies: [ "avm" ],
        version: "anchor --version"
    },
    yarn: {
        version: "yarn --version"
    },
    trident: {
        dependencies: [ "rust" ],
        version: "cargo install --list | grep trident-cli"
    },
    zest: {
        dependencies: [ "rust" ],
        version: "cargo install --list | grep zest"
    },
    "cargo-update": {
        dependencies: [ "rust" ],
        version: "cargo install --list | grep cargo-update"
    },
    verify: {
        dependencies: [ "rust" ],
        version: "solana-verify --version"
    }
};

var PathSourceStatus;

!function(PathSourceStatus) {
    PathSourceStatus[PathSourceStatus.SUCCESS = 0] = "SUCCESS", PathSourceStatus[PathSourceStatus.FAILED = 1] = "FAILED", 
    PathSourceStatus[PathSourceStatus.OUTPUT_MISMATCH = 2] = "OUTPUT_MISMATCH", PathSourceStatus[PathSourceStatus.MISSING_PATH = 3] = "MISSING_PATH";
}(PathSourceStatus || (PathSourceStatus = {}));

const VERSION_REGEX = /(?:[\w-]+\s+)?(\d+\.\d+\.?\d+?)/;

async function installedToolVersion(name) {
    let command = "";
    if (Object.prototype.hasOwnProperty.call(TOOL_CONFIG, name) && (command = TOOL_CONFIG[name].version, 
    TOOL_CONFIG[name].pathSource && (command = `export PATH="${TOOL_CONFIG[name].pathSource}:$PATH" && ${command}`)), 
    !command) return !1;
    const res = await checkCommand(command);
    return res && VERSION_REGEX.exec(res)[1] || res;
}

async function checkCommand(cmd, errorOptions = null) {
    try {
        const {stdout: stdout} = await src_default$1(cmd);
        if (stdout) return stdout.trim();
        if (errorOptions) throw "Command not found";
        return !1;
    } catch (err) {
        if (errorOptions) if ("function" == typeof errorOptions?.onError) {
            const res = await errorOptions.onError();
            if (errorOptions.doubleCheck) return await checkCommand(cmd, {
                exit: errorOptions.exit
            });
            !res && errorOptions?.exit && process.exit(1);
        } else warnMessage(errorOptions.message || `Unable to execute command: ${cmd.split(" ")[0]}`), 
        errorOptions.exit && process.exit(1);
        return !1;
    }
}

function detectOperatingSystem() {
    switch (os.platform()) {
      case "darwin":
        return "mac";

      case "win32":
        return "windows";

      case "linux":
        return "linux";

      default:
        return "unknown";
    }
}

function appendPathToRCFiles(newPath, name = void 0) {
    const homeDir = process.env.HOME || process.env.USERPROFILE;
    if (!homeDir) return void console.error("[appendPathToRCFiles]", "Unable to find the user home directory.");
    const rcFiles = [ ".bashrc", ".zshrc" ].map((file => path.join(homeDir, file)));
    let exportLine = `export PATH=${newPath}:$PATH\n`;
    if (name) {
        const exportName = `${name.toUpperCase()}_HOME`;
        exportLine = `\n# ${name}\nexport ${exportName}="${newPath}"\ncase ":$PATH:" in\n  *":$${exportName}:"*) ;;\n  *) export PATH="$${exportName}:$PATH" ;;\nesac\n# ${name} end\n\n`;
    }
    rcFiles.forEach((rcFile => {
        if (fs.existsSync(rcFile)) {
            fs.readFileSync(rcFile, "utf-8").includes(newPath) || fs.appendFileSync(rcFile, exportLine);
        }
    }));
}

function shellExecInSession({command: command, args: args, outputOnly: outputOnly}) {
    return console.log(""), outputOnly ? ((args = args || []).unshift(command), console.log(args.join(" "))) : spawn(command, args, {
        detached: !1,
        stdio: "inherit",
        shell: !0
    });
}

async function getCommandOutput(cmd) {
    try {
        return (await src_default$1(cmd)).stdout.toString().trim();
    } catch (error) {
        return !1;
    }
}

function getCommandOutputSync(cmd) {
    try {
        return execSync(cmd, {
            encoding: "utf-8",
            stdio: "pipe"
        }).toString().trim();
    } catch (error) {
        return !1;
    }
}

async function getCurrentNpmPackageVersion(packageName, global = !1) {
    return await getCommandOutput(`npm list -g ${packageName}`).then((res => res ? res.match(new RegExp(`${packageName}@(.*)`, "gi"))?.[0].match(VERSION_REGEX)?.[1] || !1 : res));
}

async function getNpmRegistryPackageVersion(packageName) {
    try {
        const response = await fetch(`https://registry.npmjs.org/${encodeURIComponent(packageName)}`, {
            headers: {
                "Cache-Control": "no-cache, no-store, must-revalidate",
                Pragma: "no-cache",
                Expires: "0"
            }
        });
        if (!response.ok) {
            if (404 === response.status) throw new Error(`Package "${packageName}" not found`);
            throw new Error(`HTTP error! status: ${response.status}`);
        }
        const data = await response.json(), allVersions = Object.keys(data.versions).sort(((a, b) => {
            const aParts = a.split(".").map(Number), bParts = b.split(".").map(Number);
            for (let i = 0; i < 3; i++) if (aParts[i] !== bParts[i]) return aParts[i] - bParts[i];
            return 0;
        }));
        return {
            latest: data["dist-tags"].latest,
            allVersions: allVersions
        };
    } catch (error) {
        return {
            error: error instanceof Error ? error.message : "Unknown error occurred"
        };
    }
}

async function checkForSelfUpdate() {
    const registry = await getNpmRegistryPackageVersion(getAppInfo().name), current = await getCurrentNpmPackageVersion(getAppInfo().name, !0);
    "error" in registry || registry.latest != current && (current && !isVersionNewer(registry.latest, current) || (titleMessage(`${getAppInfo().name} update available - v${registry.latest}`, (val => picocolors.inverse(picocolors.green(val)))), 
    console.log("  ", "To install the latest version, run the following command:"), 
    console.log("  ", picocolors.green(`npx ${getAppInfo().name}@latest install`)), 
    console.log()));
}

var extraTypings = {
    exports: {}
}, commander = {}, argument = {}, error = {}, hasRequiredError, hasRequiredArgument;

function requireError() {
    if (hasRequiredError) return error;
    hasRequiredError = 1;
    class CommanderError extends Error {
        constructor(exitCode, code, message) {
            super(message), Error.captureStackTrace(this, this.constructor), this.name = this.constructor.name, 
            this.code = code, this.exitCode = exitCode, this.nestedError = void 0;
        }
    }
    return error.CommanderError = CommanderError, error.InvalidArgumentError = class extends CommanderError {
        constructor(message) {
            super(1, "commander.invalidArgument", message), Error.captureStackTrace(this, this.constructor), 
            this.name = this.constructor.name;
        }
    }, error;
}

function requireArgument() {
    if (hasRequiredArgument) return argument;
    hasRequiredArgument = 1;
    const {InvalidArgumentError: InvalidArgumentError} = requireError();
    return argument.Argument = class {
        constructor(name, description) {
            switch (this.description = description || "", this.variadic = !1, this.parseArg = void 0, 
            this.defaultValue = void 0, this.defaultValueDescription = void 0, this.argChoices = void 0, 
            name[0]) {
              case "<":
                this.required = !0, this._name = name.slice(1, -1);
                break;

              case "[":
                this.required = !1, this._name = name.slice(1, -1);
                break;

              default:
                this.required = !0, this._name = name;
            }
            this._name.length > 3 && "..." === this._name.slice(-3) && (this.variadic = !0, 
            this._name = this._name.slice(0, -3));
        }
        name() {
            return this._name;
        }
        _concatValue(value, previous) {
            return previous !== this.defaultValue && Array.isArray(previous) ? previous.concat(value) : [ value ];
        }
        default(value, description) {
            return this.defaultValue = value, this.defaultValueDescription = description, this;
        }
        argParser(fn) {
            return this.parseArg = fn, this;
        }
        choices(values) {
            return this.argChoices = values.slice(), this.parseArg = (arg, previous) => {
                if (!this.argChoices.includes(arg)) throw new InvalidArgumentError(`Allowed choices are ${this.argChoices.join(", ")}.`);
                return this.variadic ? this._concatValue(arg, previous) : arg;
            }, this;
        }
        argRequired() {
            return this.required = !0, this;
        }
        argOptional() {
            return this.required = !1, this;
        }
    }, argument.humanReadableArgName = function(arg) {
        const nameOutput = arg.name() + (!0 === arg.variadic ? "..." : "");
        return arg.required ? "<" + nameOutput + ">" : "[" + nameOutput + "]";
    }, argument;
}

var command$1 = {}, help = {}, hasRequiredHelp;

function requireHelp() {
    if (hasRequiredHelp) return help;
    hasRequiredHelp = 1;
    const {humanReadableArgName: humanReadableArgName} = requireArgument();
    return help.Help = class {
        constructor() {
            this.helpWidth = void 0, this.sortSubcommands = !1, this.sortOptions = !1, this.showGlobalOptions = !1;
        }
        visibleCommands(cmd) {
            const visibleCommands = cmd.commands.filter((cmd => !cmd._hidden)), helpCommand = cmd._getHelpCommand();
            return helpCommand && !helpCommand._hidden && visibleCommands.push(helpCommand), 
            this.sortSubcommands && visibleCommands.sort(((a, b) => a.name().localeCompare(b.name()))), 
            visibleCommands;
        }
        compareOptions(a, b) {
            const getSortKey = option => option.short ? option.short.replace(/^-/, "") : option.long.replace(/^--/, "");
            return getSortKey(a).localeCompare(getSortKey(b));
        }
        visibleOptions(cmd) {
            const visibleOptions = cmd.options.filter((option => !option.hidden)), helpOption = cmd._getHelpOption();
            if (helpOption && !helpOption.hidden) {
                const removeShort = helpOption.short && cmd._findOption(helpOption.short), removeLong = helpOption.long && cmd._findOption(helpOption.long);
                removeShort || removeLong ? helpOption.long && !removeLong ? visibleOptions.push(cmd.createOption(helpOption.long, helpOption.description)) : helpOption.short && !removeShort && visibleOptions.push(cmd.createOption(helpOption.short, helpOption.description)) : visibleOptions.push(helpOption);
            }
            return this.sortOptions && visibleOptions.sort(this.compareOptions), visibleOptions;
        }
        visibleGlobalOptions(cmd) {
            if (!this.showGlobalOptions) return [];
            const globalOptions = [];
            for (let ancestorCmd = cmd.parent; ancestorCmd; ancestorCmd = ancestorCmd.parent) {
                const visibleOptions = ancestorCmd.options.filter((option => !option.hidden));
                globalOptions.push(...visibleOptions);
            }
            return this.sortOptions && globalOptions.sort(this.compareOptions), globalOptions;
        }
        visibleArguments(cmd) {
            return cmd._argsDescription && cmd.registeredArguments.forEach((argument => {
                argument.description = argument.description || cmd._argsDescription[argument.name()] || "";
            })), cmd.registeredArguments.find((argument => argument.description)) ? cmd.registeredArguments : [];
        }
        subcommandTerm(cmd) {
            const args = cmd.registeredArguments.map((arg => humanReadableArgName(arg))).join(" ");
            return cmd._name + (cmd._aliases[0] ? "|" + cmd._aliases[0] : "") + (cmd.options.length ? " [options]" : "") + (args ? " " + args : "");
        }
        optionTerm(option) {
            return option.flags;
        }
        argumentTerm(argument) {
            return argument.name();
        }
        longestSubcommandTermLength(cmd, helper) {
            return helper.visibleCommands(cmd).reduce(((max, command) => Math.max(max, helper.subcommandTerm(command).length)), 0);
        }
        longestOptionTermLength(cmd, helper) {
            return helper.visibleOptions(cmd).reduce(((max, option) => Math.max(max, helper.optionTerm(option).length)), 0);
        }
        longestGlobalOptionTermLength(cmd, helper) {
            return helper.visibleGlobalOptions(cmd).reduce(((max, option) => Math.max(max, helper.optionTerm(option).length)), 0);
        }
        longestArgumentTermLength(cmd, helper) {
            return helper.visibleArguments(cmd).reduce(((max, argument) => Math.max(max, helper.argumentTerm(argument).length)), 0);
        }
        commandUsage(cmd) {
            let cmdName = cmd._name;
            cmd._aliases[0] && (cmdName = cmdName + "|" + cmd._aliases[0]);
            let ancestorCmdNames = "";
            for (let ancestorCmd = cmd.parent; ancestorCmd; ancestorCmd = ancestorCmd.parent) ancestorCmdNames = ancestorCmd.name() + " " + ancestorCmdNames;
            return ancestorCmdNames + cmdName + " " + cmd.usage();
        }
        commandDescription(cmd) {
            return cmd.description();
        }
        subcommandDescription(cmd) {
            return cmd.summary() || cmd.description();
        }
        optionDescription(option) {
            const extraInfo = [];
            if (option.argChoices && extraInfo.push(`choices: ${option.argChoices.map((choice => JSON.stringify(choice))).join(", ")}`), 
            void 0 !== option.defaultValue) {
                (option.required || option.optional || option.isBoolean() && "boolean" == typeof option.defaultValue) && extraInfo.push(`default: ${option.defaultValueDescription || JSON.stringify(option.defaultValue)}`);
            }
            return void 0 !== option.presetArg && option.optional && extraInfo.push(`preset: ${JSON.stringify(option.presetArg)}`), 
            void 0 !== option.envVar && extraInfo.push(`env: ${option.envVar}`), extraInfo.length > 0 ? `${option.description} (${extraInfo.join(", ")})` : option.description;
        }
        argumentDescription(argument) {
            const extraInfo = [];
            if (argument.argChoices && extraInfo.push(`choices: ${argument.argChoices.map((choice => JSON.stringify(choice))).join(", ")}`), 
            void 0 !== argument.defaultValue && extraInfo.push(`default: ${argument.defaultValueDescription || JSON.stringify(argument.defaultValue)}`), 
            extraInfo.length > 0) {
                const extraDescripton = `(${extraInfo.join(", ")})`;
                return argument.description ? `${argument.description} ${extraDescripton}` : extraDescripton;
            }
            return argument.description;
        }
        formatHelp(cmd, helper) {
            const termWidth = helper.padWidth(cmd, helper), helpWidth = helper.helpWidth || 80;
            function formatItem(term, description) {
                if (description) {
                    const fullText = `${term.padEnd(termWidth + 2)}${description}`;
                    return helper.wrap(fullText, helpWidth - 2, termWidth + 2);
                }
                return term;
            }
            function formatList(textArray) {
                return textArray.join("\n").replace(/^/gm, " ".repeat(2));
            }
            let output = [ `Usage: ${helper.commandUsage(cmd)}`, "" ];
            const commandDescription = helper.commandDescription(cmd);
            commandDescription.length > 0 && (output = output.concat([ helper.wrap(commandDescription, helpWidth, 0), "" ]));
            const argumentList = helper.visibleArguments(cmd).map((argument => formatItem(helper.argumentTerm(argument), helper.argumentDescription(argument))));
            argumentList.length > 0 && (output = output.concat([ "Arguments:", formatList(argumentList), "" ]));
            const optionList = helper.visibleOptions(cmd).map((option => formatItem(helper.optionTerm(option), helper.optionDescription(option))));
            if (optionList.length > 0 && (output = output.concat([ "Options:", formatList(optionList), "" ])), 
            this.showGlobalOptions) {
                const globalOptionList = helper.visibleGlobalOptions(cmd).map((option => formatItem(helper.optionTerm(option), helper.optionDescription(option))));
                globalOptionList.length > 0 && (output = output.concat([ "Global Options:", formatList(globalOptionList), "" ]));
            }
            const commandList = helper.visibleCommands(cmd).map((cmd => formatItem(helper.subcommandTerm(cmd), helper.subcommandDescription(cmd))));
            return commandList.length > 0 && (output = output.concat([ "Commands:", formatList(commandList), "" ])), 
            output.join("\n");
        }
        padWidth(cmd, helper) {
            return Math.max(helper.longestOptionTermLength(cmd, helper), helper.longestGlobalOptionTermLength(cmd, helper), helper.longestSubcommandTermLength(cmd, helper), helper.longestArgumentTermLength(cmd, helper));
        }
        wrap(str, width, indent, minColumnWidth = 40) {
            const manualIndent = new RegExp("[\\n][ \\f\\t\\v   -   　\ufeff]+");
            if (str.match(manualIndent)) return str;
            const columnWidth = width - indent;
            if (columnWidth < minColumnWidth) return str;
            const leadingStr = str.slice(0, indent), columnText = str.slice(indent).replace("\r\n", "\n"), indentString = " ".repeat(indent), regex = new RegExp(`\n|.{1,${columnWidth - 1}}([\\s​]|$)|[^\\s​]+?([\\s​]|$)`, "g");
            return leadingStr + (columnText.match(regex) || []).map(((line, i) => "\n" === line ? "" : (i > 0 ? indentString : "") + line.trimEnd())).join("\n");
        }
    }, help;
}

var option = {}, hasRequiredOption;

function requireOption() {
    if (hasRequiredOption) return option;
    hasRequiredOption = 1;
    const {InvalidArgumentError: InvalidArgumentError} = requireError();
    return option.Option = class {
        constructor(flags, description) {
            this.flags = flags, this.description = description || "", this.required = flags.includes("<"), 
            this.optional = flags.includes("["), this.variadic = /\w\.\.\.[>\]]$/.test(flags), 
            this.mandatory = !1;
            const optionFlags = function(flags) {
                let shortFlag, longFlag;
                const flagParts = flags.split(/[ |,]+/);
                flagParts.length > 1 && !/^[[<]/.test(flagParts[1]) && (shortFlag = flagParts.shift());
                longFlag = flagParts.shift(), !shortFlag && /^-[^-]$/.test(longFlag) && (shortFlag = longFlag, 
                longFlag = void 0);
                return {
                    shortFlag: shortFlag,
                    longFlag: longFlag
                };
            }(flags);
            this.short = optionFlags.shortFlag, this.long = optionFlags.longFlag, this.negate = !1, 
            this.long && (this.negate = this.long.startsWith("--no-")), this.defaultValue = void 0, 
            this.defaultValueDescription = void 0, this.presetArg = void 0, this.envVar = void 0, 
            this.parseArg = void 0, this.hidden = !1, this.argChoices = void 0, this.conflictsWith = [], 
            this.implied = void 0;
        }
        default(value, description) {
            return this.defaultValue = value, this.defaultValueDescription = description, this;
        }
        preset(arg) {
            return this.presetArg = arg, this;
        }
        conflicts(names) {
            return this.conflictsWith = this.conflictsWith.concat(names), this;
        }
        implies(impliedOptionValues) {
            let newImplied = impliedOptionValues;
            return "string" == typeof impliedOptionValues && (newImplied = {
                [impliedOptionValues]: !0
            }), this.implied = Object.assign(this.implied || {}, newImplied), this;
        }
        env(name) {
            return this.envVar = name, this;
        }
        argParser(fn) {
            return this.parseArg = fn, this;
        }
        makeOptionMandatory(mandatory = !0) {
            return this.mandatory = !!mandatory, this;
        }
        hideHelp(hide = !0) {
            return this.hidden = !!hide, this;
        }
        _concatValue(value, previous) {
            return previous !== this.defaultValue && Array.isArray(previous) ? previous.concat(value) : [ value ];
        }
        choices(values) {
            return this.argChoices = values.slice(), this.parseArg = (arg, previous) => {
                if (!this.argChoices.includes(arg)) throw new InvalidArgumentError(`Allowed choices are ${this.argChoices.join(", ")}.`);
                return this.variadic ? this._concatValue(arg, previous) : arg;
            }, this;
        }
        name() {
            return this.long ? this.long.replace(/^--/, "") : this.short.replace(/^-/, "");
        }
        attributeName() {
            return this.name().replace(/^no-/, "").split("-").reduce(((str, word) => str + word[0].toUpperCase() + word.slice(1)));
        }
        is(arg) {
            return this.short === arg || this.long === arg;
        }
        isBoolean() {
            return !this.required && !this.optional && !this.negate;
        }
    }, option.DualOptions = class {
        constructor(options) {
            this.positiveOptions = new Map, this.negativeOptions = new Map, this.dualOptions = new Set, 
            options.forEach((option => {
                option.negate ? this.negativeOptions.set(option.attributeName(), option) : this.positiveOptions.set(option.attributeName(), option);
            })), this.negativeOptions.forEach(((value, key) => {
                this.positiveOptions.has(key) && this.dualOptions.add(key);
            }));
        }
        valueFromOption(value, option) {
            const optionKey = option.attributeName();
            if (!this.dualOptions.has(optionKey)) return !0;
            const preset = this.negativeOptions.get(optionKey).presetArg, negativeValue = void 0 !== preset && preset;
            return option.negate === (negativeValue === value);
        }
    }, option;
}

var suggestSimilar = {}, hasRequiredSuggestSimilar, hasRequiredCommand, hasRequiredCommander, hasRequiredExtraTypings;

function requireSuggestSimilar() {
    if (hasRequiredSuggestSimilar) return suggestSimilar;
    hasRequiredSuggestSimilar = 1;
    return suggestSimilar.suggestSimilar = function(word, candidates) {
        if (!candidates || 0 === candidates.length) return "";
        candidates = Array.from(new Set(candidates));
        const searchingOptions = word.startsWith("--");
        searchingOptions && (word = word.slice(2), candidates = candidates.map((candidate => candidate.slice(2))));
        let similar = [], bestDistance = 3;
        return candidates.forEach((candidate => {
            if (candidate.length <= 1) return;
            const distance = function(a, b) {
                if (Math.abs(a.length - b.length) > 3) return Math.max(a.length, b.length);
                const d = [];
                for (let i = 0; i <= a.length; i++) d[i] = [ i ];
                for (let j = 0; j <= b.length; j++) d[0][j] = j;
                for (let j = 1; j <= b.length; j++) for (let i = 1; i <= a.length; i++) {
                    let cost = 1;
                    cost = a[i - 1] === b[j - 1] ? 0 : 1, d[i][j] = Math.min(d[i - 1][j] + 1, d[i][j - 1] + 1, d[i - 1][j - 1] + cost), 
                    i > 1 && j > 1 && a[i - 1] === b[j - 2] && a[i - 2] === b[j - 1] && (d[i][j] = Math.min(d[i][j], d[i - 2][j - 2] + 1));
                }
                return d[a.length][b.length];
            }(word, candidate), length = Math.max(word.length, candidate.length);
            (length - distance) / length > .4 && (distance < bestDistance ? (bestDistance = distance, 
            similar = [ candidate ]) : distance === bestDistance && similar.push(candidate));
        })), similar.sort(((a, b) => a.localeCompare(b))), searchingOptions && (similar = similar.map((candidate => `--${candidate}`))), 
        similar.length > 1 ? `\n(Did you mean one of ${similar.join(", ")}?)` : 1 === similar.length ? `\n(Did you mean ${similar[0]}?)` : "";
    }, suggestSimilar;
}

function requireCommand() {
    if (hasRequiredCommand) return command$1;
    hasRequiredCommand = 1;
    const EventEmitter = require$$0$1.EventEmitter, childProcess = require$$1, path = require$$2, fs = require$$3, process = process$2, {Argument: Argument, humanReadableArgName: humanReadableArgName} = requireArgument(), {CommanderError: CommanderError} = requireError(), {Help: Help} = requireHelp(), {Option: Option, DualOptions: DualOptions} = requireOption(), {suggestSimilar: suggestSimilar} = requireSuggestSimilar();
    class Command extends EventEmitter {
        constructor(name) {
            super(), this.commands = [], this.options = [], this.parent = null, this._allowUnknownOption = !1, 
            this._allowExcessArguments = !0, this.registeredArguments = [], this._args = this.registeredArguments, 
            this.args = [], this.rawArgs = [], this.processedArgs = [], this._scriptPath = null, 
            this._name = name || "", this._optionValues = {}, this._optionValueSources = {}, 
            this._storeOptionsAsProperties = !1, this._actionHandler = null, this._executableHandler = !1, 
            this._executableFile = null, this._executableDir = null, this._defaultCommandName = null, 
            this._exitCallback = null, this._aliases = [], this._combineFlagAndOptionalValue = !0, 
            this._description = "", this._summary = "", this._argsDescription = void 0, this._enablePositionalOptions = !1, 
            this._passThroughOptions = !1, this._lifeCycleHooks = {}, this._showHelpAfterError = !1, 
            this._showSuggestionAfterError = !0, this._outputConfiguration = {
                writeOut: str => process.stdout.write(str),
                writeErr: str => process.stderr.write(str),
                getOutHelpWidth: () => process.stdout.isTTY ? process.stdout.columns : void 0,
                getErrHelpWidth: () => process.stderr.isTTY ? process.stderr.columns : void 0,
                outputError: (str, write) => write(str)
            }, this._hidden = !1, this._helpOption = void 0, this._addImplicitHelpCommand = void 0, 
            this._helpCommand = void 0, this._helpConfiguration = {};
        }
        copyInheritedSettings(sourceCommand) {
            return this._outputConfiguration = sourceCommand._outputConfiguration, this._helpOption = sourceCommand._helpOption, 
            this._helpCommand = sourceCommand._helpCommand, this._helpConfiguration = sourceCommand._helpConfiguration, 
            this._exitCallback = sourceCommand._exitCallback, this._storeOptionsAsProperties = sourceCommand._storeOptionsAsProperties, 
            this._combineFlagAndOptionalValue = sourceCommand._combineFlagAndOptionalValue, 
            this._allowExcessArguments = sourceCommand._allowExcessArguments, this._enablePositionalOptions = sourceCommand._enablePositionalOptions, 
            this._showHelpAfterError = sourceCommand._showHelpAfterError, this._showSuggestionAfterError = sourceCommand._showSuggestionAfterError, 
            this;
        }
        _getCommandAndAncestors() {
            const result = [];
            for (let command = this; command; command = command.parent) result.push(command);
            return result;
        }
        command(nameAndArgs, actionOptsOrExecDesc, execOpts) {
            let desc = actionOptsOrExecDesc, opts = execOpts;
            "object" == typeof desc && null !== desc && (opts = desc, desc = null), opts = opts || {};
            const [, name, args] = nameAndArgs.match(/([^ ]+) *(.*)/), cmd = this.createCommand(name);
            return desc && (cmd.description(desc), cmd._executableHandler = !0), opts.isDefault && (this._defaultCommandName = cmd._name), 
            cmd._hidden = !(!opts.noHelp && !opts.hidden), cmd._executableFile = opts.executableFile || null, 
            args && cmd.arguments(args), this._registerCommand(cmd), cmd.parent = this, cmd.copyInheritedSettings(this), 
            desc ? this : cmd;
        }
        createCommand(name) {
            return new Command(name);
        }
        createHelp() {
            return Object.assign(new Help, this.configureHelp());
        }
        configureHelp(configuration) {
            return void 0 === configuration ? this._helpConfiguration : (this._helpConfiguration = configuration, 
            this);
        }
        configureOutput(configuration) {
            return void 0 === configuration ? this._outputConfiguration : (Object.assign(this._outputConfiguration, configuration), 
            this);
        }
        showHelpAfterError(displayHelp = !0) {
            return "string" != typeof displayHelp && (displayHelp = !!displayHelp), this._showHelpAfterError = displayHelp, 
            this;
        }
        showSuggestionAfterError(displaySuggestion = !0) {
            return this._showSuggestionAfterError = !!displaySuggestion, this;
        }
        addCommand(cmd, opts) {
            if (!cmd._name) throw new Error("Command passed to .addCommand() must have a name\n- specify the name in Command constructor or using .name()");
            return (opts = opts || {}).isDefault && (this._defaultCommandName = cmd._name), 
            (opts.noHelp || opts.hidden) && (cmd._hidden = !0), this._registerCommand(cmd), 
            cmd.parent = this, cmd._checkForBrokenPassThrough(), this;
        }
        createArgument(name, description) {
            return new Argument(name, description);
        }
        argument(name, description, fn, defaultValue) {
            const argument = this.createArgument(name, description);
            return "function" == typeof fn ? argument.default(defaultValue).argParser(fn) : argument.default(fn), 
            this.addArgument(argument), this;
        }
        arguments(names) {
            return names.trim().split(/ +/).forEach((detail => {
                this.argument(detail);
            })), this;
        }
        addArgument(argument) {
            const previousArgument = this.registeredArguments.slice(-1)[0];
            if (previousArgument && previousArgument.variadic) throw new Error(`only the last argument can be variadic '${previousArgument.name()}'`);
            if (argument.required && void 0 !== argument.defaultValue && void 0 === argument.parseArg) throw new Error(`a default value for a required argument is never used: '${argument.name()}'`);
            return this.registeredArguments.push(argument), this;
        }
        helpCommand(enableOrNameAndArgs, description) {
            if ("boolean" == typeof enableOrNameAndArgs) return this._addImplicitHelpCommand = enableOrNameAndArgs, 
            this;
            enableOrNameAndArgs = enableOrNameAndArgs ?? "help [command]";
            const [, helpName, helpArgs] = enableOrNameAndArgs.match(/([^ ]+) *(.*)/), helpDescription = description ?? "display help for command", helpCommand = this.createCommand(helpName);
            return helpCommand.helpOption(!1), helpArgs && helpCommand.arguments(helpArgs), 
            helpDescription && helpCommand.description(helpDescription), this._addImplicitHelpCommand = !0, 
            this._helpCommand = helpCommand, this;
        }
        addHelpCommand(helpCommand, deprecatedDescription) {
            return "object" != typeof helpCommand ? (this.helpCommand(helpCommand, deprecatedDescription), 
            this) : (this._addImplicitHelpCommand = !0, this._helpCommand = helpCommand, this);
        }
        _getHelpCommand() {
            return this._addImplicitHelpCommand ?? (this.commands.length && !this._actionHandler && !this._findCommand("help")) ? (void 0 === this._helpCommand && this.helpCommand(void 0, void 0), 
            this._helpCommand) : null;
        }
        hook(event, listener) {
            const allowedValues = [ "preSubcommand", "preAction", "postAction" ];
            if (!allowedValues.includes(event)) throw new Error(`Unexpected value for event passed to hook : '${event}'.\nExpecting one of '${allowedValues.join("', '")}'`);
            return this._lifeCycleHooks[event] ? this._lifeCycleHooks[event].push(listener) : this._lifeCycleHooks[event] = [ listener ], 
            this;
        }
        exitOverride(fn) {
            return this._exitCallback = fn || (err => {
                if ("commander.executeSubCommandAsync" !== err.code) throw err;
            }), this;
        }
        _exit(exitCode, code, message) {
            this._exitCallback && this._exitCallback(new CommanderError(exitCode, code, message)), 
            process.exit(exitCode);
        }
        action(fn) {
            return this._actionHandler = args => {
                const expectedArgsCount = this.registeredArguments.length, actionArgs = args.slice(0, expectedArgsCount);
                return this._storeOptionsAsProperties ? actionArgs[expectedArgsCount] = this : actionArgs[expectedArgsCount] = this.opts(), 
                actionArgs.push(this), fn.apply(this, actionArgs);
            }, this;
        }
        createOption(flags, description) {
            return new Option(flags, description);
        }
        _callParseArg(target, value, previous, invalidArgumentMessage) {
            try {
                return target.parseArg(value, previous);
            } catch (err) {
                if ("commander.invalidArgument" === err.code) {
                    const message = `${invalidArgumentMessage} ${err.message}`;
                    this.error(message, {
                        exitCode: err.exitCode,
                        code: err.code
                    });
                }
                throw err;
            }
        }
        _registerOption(option) {
            const matchingOption = option.short && this._findOption(option.short) || option.long && this._findOption(option.long);
            if (matchingOption) {
                const matchingFlag = option.long && this._findOption(option.long) ? option.long : option.short;
                throw new Error(`Cannot add option '${option.flags}'${this._name && ` to command '${this._name}'`} due to conflicting flag '${matchingFlag}'\n-  already used by option '${matchingOption.flags}'`);
            }
            this.options.push(option);
        }
        _registerCommand(command) {
            const knownBy = cmd => [ cmd.name() ].concat(cmd.aliases()), alreadyUsed = knownBy(command).find((name => this._findCommand(name)));
            if (alreadyUsed) {
                const existingCmd = knownBy(this._findCommand(alreadyUsed)).join("|"), newCmd = knownBy(command).join("|");
                throw new Error(`cannot add command '${newCmd}' as already have command '${existingCmd}'`);
            }
            this.commands.push(command);
        }
        addOption(option) {
            this._registerOption(option);
            const oname = option.name(), name = option.attributeName();
            if (option.negate) {
                const positiveLongFlag = option.long.replace(/^--no-/, "--");
                this._findOption(positiveLongFlag) || this.setOptionValueWithSource(name, void 0 === option.defaultValue || option.defaultValue, "default");
            } else void 0 !== option.defaultValue && this.setOptionValueWithSource(name, option.defaultValue, "default");
            const handleOptionValue = (val, invalidValueMessage, valueSource) => {
                null == val && void 0 !== option.presetArg && (val = option.presetArg);
                const oldValue = this.getOptionValue(name);
                null !== val && option.parseArg ? val = this._callParseArg(option, val, oldValue, invalidValueMessage) : null !== val && option.variadic && (val = option._concatValue(val, oldValue)), 
                null == val && (val = !option.negate && (!(!option.isBoolean() && !option.optional) || "")), 
                this.setOptionValueWithSource(name, val, valueSource);
            };
            return this.on("option:" + oname, (val => {
                const invalidValueMessage = `error: option '${option.flags}' argument '${val}' is invalid.`;
                handleOptionValue(val, invalidValueMessage, "cli");
            })), option.envVar && this.on("optionEnv:" + oname, (val => {
                const invalidValueMessage = `error: option '${option.flags}' value '${val}' from env '${option.envVar}' is invalid.`;
                handleOptionValue(val, invalidValueMessage, "env");
            })), this;
        }
        _optionEx(config, flags, description, fn, defaultValue) {
            if ("object" == typeof flags && flags instanceof Option) throw new Error("To add an Option object use addOption() instead of option() or requiredOption()");
            const option = this.createOption(flags, description);
            if (option.makeOptionMandatory(!!config.mandatory), "function" == typeof fn) option.default(defaultValue).argParser(fn); else if (fn instanceof RegExp) {
                const regex = fn;
                fn = (val, def) => {
                    const m = regex.exec(val);
                    return m ? m[0] : def;
                }, option.default(defaultValue).argParser(fn);
            } else option.default(fn);
            return this.addOption(option);
        }
        option(flags, description, parseArg, defaultValue) {
            return this._optionEx({}, flags, description, parseArg, defaultValue);
        }
        requiredOption(flags, description, parseArg, defaultValue) {
            return this._optionEx({
                mandatory: !0
            }, flags, description, parseArg, defaultValue);
        }
        combineFlagAndOptionalValue(combine = !0) {
            return this._combineFlagAndOptionalValue = !!combine, this;
        }
        allowUnknownOption(allowUnknown = !0) {
            return this._allowUnknownOption = !!allowUnknown, this;
        }
        allowExcessArguments(allowExcess = !0) {
            return this._allowExcessArguments = !!allowExcess, this;
        }
        enablePositionalOptions(positional = !0) {
            return this._enablePositionalOptions = !!positional, this;
        }
        passThroughOptions(passThrough = !0) {
            return this._passThroughOptions = !!passThrough, this._checkForBrokenPassThrough(), 
            this;
        }
        _checkForBrokenPassThrough() {
            if (this.parent && this._passThroughOptions && !this.parent._enablePositionalOptions) throw new Error(`passThroughOptions cannot be used for '${this._name}' without turning on enablePositionalOptions for parent command(s)`);
        }
        storeOptionsAsProperties(storeAsProperties = !0) {
            if (this.options.length) throw new Error("call .storeOptionsAsProperties() before adding options");
            if (Object.keys(this._optionValues).length) throw new Error("call .storeOptionsAsProperties() before setting option values");
            return this._storeOptionsAsProperties = !!storeAsProperties, this;
        }
        getOptionValue(key) {
            return this._storeOptionsAsProperties ? this[key] : this._optionValues[key];
        }
        setOptionValue(key, value) {
            return this.setOptionValueWithSource(key, value, void 0);
        }
        setOptionValueWithSource(key, value, source) {
            return this._storeOptionsAsProperties ? this[key] = value : this._optionValues[key] = value, 
            this._optionValueSources[key] = source, this;
        }
        getOptionValueSource(key) {
            return this._optionValueSources[key];
        }
        getOptionValueSourceWithGlobals(key) {
            let source;
            return this._getCommandAndAncestors().forEach((cmd => {
                void 0 !== cmd.getOptionValueSource(key) && (source = cmd.getOptionValueSource(key));
            })), source;
        }
        _prepareUserArgs(argv, parseOptions) {
            if (void 0 !== argv && !Array.isArray(argv)) throw new Error("first parameter to parse must be array or undefined");
            if (parseOptions = parseOptions || {}, void 0 === argv && void 0 === parseOptions.from) {
                process.versions?.electron && (parseOptions.from = "electron");
                const execArgv = process.execArgv ?? [];
                (execArgv.includes("-e") || execArgv.includes("--eval") || execArgv.includes("-p") || execArgv.includes("--print")) && (parseOptions.from = "eval");
            }
            let userArgs;
            switch (void 0 === argv && (argv = process.argv), this.rawArgs = argv.slice(), parseOptions.from) {
              case void 0:
              case "node":
                this._scriptPath = argv[1], userArgs = argv.slice(2);
                break;

              case "electron":
                process.defaultApp ? (this._scriptPath = argv[1], userArgs = argv.slice(2)) : userArgs = argv.slice(1);
                break;

              case "user":
                userArgs = argv.slice(0);
                break;

              case "eval":
                userArgs = argv.slice(1);
                break;

              default:
                throw new Error(`unexpected parse option { from: '${parseOptions.from}' }`);
            }
            return !this._name && this._scriptPath && this.nameFromFilename(this._scriptPath), 
            this._name = this._name || "program", userArgs;
        }
        parse(argv, parseOptions) {
            const userArgs = this._prepareUserArgs(argv, parseOptions);
            return this._parseCommand([], userArgs), this;
        }
        async parseAsync(argv, parseOptions) {
            const userArgs = this._prepareUserArgs(argv, parseOptions);
            return await this._parseCommand([], userArgs), this;
        }
        _executeSubCommand(subcommand, args) {
            args = args.slice();
            let launchWithNode = !1;
            const sourceExt = [ ".js", ".ts", ".tsx", ".mjs", ".cjs" ];
            function findFile(baseDir, baseName) {
                const localBin = path.resolve(baseDir, baseName);
                if (fs.existsSync(localBin)) return localBin;
                if (sourceExt.includes(path.extname(baseName))) return;
                const foundExt = sourceExt.find((ext => fs.existsSync(`${localBin}${ext}`)));
                return foundExt ? `${localBin}${foundExt}` : void 0;
            }
            this._checkForMissingMandatoryOptions(), this._checkForConflictingOptions();
            let proc, executableFile = subcommand._executableFile || `${this._name}-${subcommand._name}`, executableDir = this._executableDir || "";
            if (this._scriptPath) {
                let resolvedScriptPath;
                try {
                    resolvedScriptPath = fs.realpathSync(this._scriptPath);
                } catch (err) {
                    resolvedScriptPath = this._scriptPath;
                }
                executableDir = path.resolve(path.dirname(resolvedScriptPath), executableDir);
            }
            if (executableDir) {
                let localFile = findFile(executableDir, executableFile);
                if (!localFile && !subcommand._executableFile && this._scriptPath) {
                    const legacyName = path.basename(this._scriptPath, path.extname(this._scriptPath));
                    legacyName !== this._name && (localFile = findFile(executableDir, `${legacyName}-${subcommand._name}`));
                }
                executableFile = localFile || executableFile;
            }
            if (launchWithNode = sourceExt.includes(path.extname(executableFile)), "win32" !== process.platform ? launchWithNode ? (args.unshift(executableFile), 
            args = incrementNodeInspectorPort(process.execArgv).concat(args), proc = childProcess.spawn(process.argv[0], args, {
                stdio: "inherit"
            })) : proc = childProcess.spawn(executableFile, args, {
                stdio: "inherit"
            }) : (args.unshift(executableFile), args = incrementNodeInspectorPort(process.execArgv).concat(args), 
            proc = childProcess.spawn(process.execPath, args, {
                stdio: "inherit"
            })), !proc.killed) {
                [ "SIGUSR1", "SIGUSR2", "SIGTERM", "SIGINT", "SIGHUP" ].forEach((signal => {
                    process.on(signal, (() => {
                        !1 === proc.killed && null === proc.exitCode && proc.kill(signal);
                    }));
                }));
            }
            const exitCallback = this._exitCallback;
            proc.on("close", (code => {
                code = code ?? 1, exitCallback ? exitCallback(new CommanderError(code, "commander.executeSubCommandAsync", "(close)")) : process.exit(code);
            })), proc.on("error", (err => {
                if ("ENOENT" === err.code) {
                    const executableDirMessage = executableDir ? `searched for local subcommand relative to directory '${executableDir}'` : "no directory for search for local subcommand, use .executableDir() to supply a custom directory", executableMissing = `'${executableFile}' does not exist\n - if '${subcommand._name}' is not meant to be an executable command, remove description parameter from '.command()' and use '.description()' instead\n - if the default executable name is not suitable, use the executableFile option to supply a custom name or path\n - ${executableDirMessage}`;
                    throw new Error(executableMissing);
                }
                if ("EACCES" === err.code) throw new Error(`'${executableFile}' not executable`);
                if (exitCallback) {
                    const wrappedError = new CommanderError(1, "commander.executeSubCommandAsync", "(error)");
                    wrappedError.nestedError = err, exitCallback(wrappedError);
                } else process.exit(1);
            })), this.runningCommand = proc;
        }
        _dispatchSubcommand(commandName, operands, unknown) {
            const subCommand = this._findCommand(commandName);
            let promiseChain;
            return subCommand || this.help({
                error: !0
            }), promiseChain = this._chainOrCallSubCommandHook(promiseChain, subCommand, "preSubcommand"), 
            promiseChain = this._chainOrCall(promiseChain, (() => {
                if (!subCommand._executableHandler) return subCommand._parseCommand(operands, unknown);
                this._executeSubCommand(subCommand, operands.concat(unknown));
            })), promiseChain;
        }
        _dispatchHelpCommand(subcommandName) {
            subcommandName || this.help();
            const subCommand = this._findCommand(subcommandName);
            return subCommand && !subCommand._executableHandler && subCommand.help(), this._dispatchSubcommand(subcommandName, [], [ this._getHelpOption()?.long ?? this._getHelpOption()?.short ?? "--help" ]);
        }
        _checkNumberOfArguments() {
            this.registeredArguments.forEach(((arg, i) => {
                arg.required && null == this.args[i] && this.missingArgument(arg.name());
            })), this.registeredArguments.length > 0 && this.registeredArguments[this.registeredArguments.length - 1].variadic || this.args.length > this.registeredArguments.length && this._excessArguments(this.args);
        }
        _processArguments() {
            const myParseArg = (argument, value, previous) => {
                let parsedValue = value;
                if (null !== value && argument.parseArg) {
                    const invalidValueMessage = `error: command-argument value '${value}' is invalid for argument '${argument.name()}'.`;
                    parsedValue = this._callParseArg(argument, value, previous, invalidValueMessage);
                }
                return parsedValue;
            };
            this._checkNumberOfArguments();
            const processedArgs = [];
            this.registeredArguments.forEach(((declaredArg, index) => {
                let value = declaredArg.defaultValue;
                declaredArg.variadic ? index < this.args.length ? (value = this.args.slice(index), 
                declaredArg.parseArg && (value = value.reduce(((processed, v) => myParseArg(declaredArg, v, processed)), declaredArg.defaultValue))) : void 0 === value && (value = []) : index < this.args.length && (value = this.args[index], 
                declaredArg.parseArg && (value = myParseArg(declaredArg, value, declaredArg.defaultValue))), 
                processedArgs[index] = value;
            })), this.processedArgs = processedArgs;
        }
        _chainOrCall(promise, fn) {
            return promise && promise.then && "function" == typeof promise.then ? promise.then((() => fn())) : fn();
        }
        _chainOrCallHooks(promise, event) {
            let result = promise;
            const hooks = [];
            return this._getCommandAndAncestors().reverse().filter((cmd => void 0 !== cmd._lifeCycleHooks[event])).forEach((hookedCommand => {
                hookedCommand._lifeCycleHooks[event].forEach((callback => {
                    hooks.push({
                        hookedCommand: hookedCommand,
                        callback: callback
                    });
                }));
            })), "postAction" === event && hooks.reverse(), hooks.forEach((hookDetail => {
                result = this._chainOrCall(result, (() => hookDetail.callback(hookDetail.hookedCommand, this)));
            })), result;
        }
        _chainOrCallSubCommandHook(promise, subCommand, event) {
            let result = promise;
            return void 0 !== this._lifeCycleHooks[event] && this._lifeCycleHooks[event].forEach((hook => {
                result = this._chainOrCall(result, (() => hook(this, subCommand)));
            })), result;
        }
        _parseCommand(operands, unknown) {
            const parsed = this.parseOptions(unknown);
            if (this._parseOptionsEnv(), this._parseOptionsImplied(), operands = operands.concat(parsed.operands), 
            unknown = parsed.unknown, this.args = operands.concat(unknown), operands && this._findCommand(operands[0])) return this._dispatchSubcommand(operands[0], operands.slice(1), unknown);
            if (this._getHelpCommand() && operands[0] === this._getHelpCommand().name()) return this._dispatchHelpCommand(operands[1]);
            if (this._defaultCommandName) return this._outputHelpIfRequested(unknown), this._dispatchSubcommand(this._defaultCommandName, operands, unknown);
            !this.commands.length || 0 !== this.args.length || this._actionHandler || this._defaultCommandName || this.help({
                error: !0
            }), this._outputHelpIfRequested(parsed.unknown), this._checkForMissingMandatoryOptions(), 
            this._checkForConflictingOptions();
            const checkForUnknownOptions = () => {
                parsed.unknown.length > 0 && this.unknownOption(parsed.unknown[0]);
            }, commandEvent = `command:${this.name()}`;
            if (this._actionHandler) {
                let promiseChain;
                return checkForUnknownOptions(), this._processArguments(), promiseChain = this._chainOrCallHooks(promiseChain, "preAction"), 
                promiseChain = this._chainOrCall(promiseChain, (() => this._actionHandler(this.processedArgs))), 
                this.parent && (promiseChain = this._chainOrCall(promiseChain, (() => {
                    this.parent.emit(commandEvent, operands, unknown);
                }))), promiseChain = this._chainOrCallHooks(promiseChain, "postAction"), promiseChain;
            }
            if (this.parent && this.parent.listenerCount(commandEvent)) checkForUnknownOptions(), 
            this._processArguments(), this.parent.emit(commandEvent, operands, unknown); else if (operands.length) {
                if (this._findCommand("*")) return this._dispatchSubcommand("*", operands, unknown);
                this.listenerCount("command:*") ? this.emit("command:*", operands, unknown) : this.commands.length ? this.unknownCommand() : (checkForUnknownOptions(), 
                this._processArguments());
            } else this.commands.length ? (checkForUnknownOptions(), this.help({
                error: !0
            })) : (checkForUnknownOptions(), this._processArguments());
        }
        _findCommand(name) {
            if (name) return this.commands.find((cmd => cmd._name === name || cmd._aliases.includes(name)));
        }
        _findOption(arg) {
            return this.options.find((option => option.is(arg)));
        }
        _checkForMissingMandatoryOptions() {
            this._getCommandAndAncestors().forEach((cmd => {
                cmd.options.forEach((anOption => {
                    anOption.mandatory && void 0 === cmd.getOptionValue(anOption.attributeName()) && cmd.missingMandatoryOptionValue(anOption);
                }));
            }));
        }
        _checkForConflictingLocalOptions() {
            const definedNonDefaultOptions = this.options.filter((option => {
                const optionKey = option.attributeName();
                return void 0 !== this.getOptionValue(optionKey) && "default" !== this.getOptionValueSource(optionKey);
            })), optionsWithConflicting = definedNonDefaultOptions.filter((option => option.conflictsWith.length > 0));
            optionsWithConflicting.forEach((option => {
                const conflictingAndDefined = definedNonDefaultOptions.find((defined => option.conflictsWith.includes(defined.attributeName())));
                conflictingAndDefined && this._conflictingOption(option, conflictingAndDefined);
            }));
        }
        _checkForConflictingOptions() {
            this._getCommandAndAncestors().forEach((cmd => {
                cmd._checkForConflictingLocalOptions();
            }));
        }
        parseOptions(argv) {
            const operands = [], unknown = [];
            let dest = operands;
            const args = argv.slice();
            function maybeOption(arg) {
                return arg.length > 1 && "-" === arg[0];
            }
            let activeVariadicOption = null;
            for (;args.length; ) {
                const arg = args.shift();
                if ("--" === arg) {
                    dest === unknown && dest.push(arg), dest.push(...args);
                    break;
                }
                if (!activeVariadicOption || maybeOption(arg)) {
                    if (activeVariadicOption = null, maybeOption(arg)) {
                        const option = this._findOption(arg);
                        if (option) {
                            if (option.required) {
                                const value = args.shift();
                                void 0 === value && this.optionMissingArgument(option), this.emit(`option:${option.name()}`, value);
                            } else if (option.optional) {
                                let value = null;
                                args.length > 0 && !maybeOption(args[0]) && (value = args.shift()), this.emit(`option:${option.name()}`, value);
                            } else this.emit(`option:${option.name()}`);
                            activeVariadicOption = option.variadic ? option : null;
                            continue;
                        }
                    }
                    if (arg.length > 2 && "-" === arg[0] && "-" !== arg[1]) {
                        const option = this._findOption(`-${arg[1]}`);
                        if (option) {
                            option.required || option.optional && this._combineFlagAndOptionalValue ? this.emit(`option:${option.name()}`, arg.slice(2)) : (this.emit(`option:${option.name()}`), 
                            args.unshift(`-${arg.slice(2)}`));
                            continue;
                        }
                    }
                    if (/^--[^=]+=/.test(arg)) {
                        const index = arg.indexOf("="), option = this._findOption(arg.slice(0, index));
                        if (option && (option.required || option.optional)) {
                            this.emit(`option:${option.name()}`, arg.slice(index + 1));
                            continue;
                        }
                    }
                    if (maybeOption(arg) && (dest = unknown), (this._enablePositionalOptions || this._passThroughOptions) && 0 === operands.length && 0 === unknown.length) {
                        if (this._findCommand(arg)) {
                            operands.push(arg), args.length > 0 && unknown.push(...args);
                            break;
                        }
                        if (this._getHelpCommand() && arg === this._getHelpCommand().name()) {
                            operands.push(arg), args.length > 0 && operands.push(...args);
                            break;
                        }
                        if (this._defaultCommandName) {
                            unknown.push(arg), args.length > 0 && unknown.push(...args);
                            break;
                        }
                    }
                    if (this._passThroughOptions) {
                        dest.push(arg), args.length > 0 && dest.push(...args);
                        break;
                    }
                    dest.push(arg);
                } else this.emit(`option:${activeVariadicOption.name()}`, arg);
            }
            return {
                operands: operands,
                unknown: unknown
            };
        }
        opts() {
            if (this._storeOptionsAsProperties) {
                const result = {}, len = this.options.length;
                for (let i = 0; i < len; i++) {
                    const key = this.options[i].attributeName();
                    result[key] = key === this._versionOptionName ? this._version : this[key];
                }
                return result;
            }
            return this._optionValues;
        }
        optsWithGlobals() {
            return this._getCommandAndAncestors().reduce(((combinedOptions, cmd) => Object.assign(combinedOptions, cmd.opts())), {});
        }
        error(message, errorOptions) {
            this._outputConfiguration.outputError(`${message}\n`, this._outputConfiguration.writeErr), 
            "string" == typeof this._showHelpAfterError ? this._outputConfiguration.writeErr(`${this._showHelpAfterError}\n`) : this._showHelpAfterError && (this._outputConfiguration.writeErr("\n"), 
            this.outputHelp({
                error: !0
            }));
            const config = errorOptions || {}, exitCode = config.exitCode || 1, code = config.code || "commander.error";
            this._exit(exitCode, code, message);
        }
        _parseOptionsEnv() {
            this.options.forEach((option => {
                if (option.envVar && option.envVar in process.env) {
                    const optionKey = option.attributeName();
                    (void 0 === this.getOptionValue(optionKey) || [ "default", "config", "env" ].includes(this.getOptionValueSource(optionKey))) && (option.required || option.optional ? this.emit(`optionEnv:${option.name()}`, process.env[option.envVar]) : this.emit(`optionEnv:${option.name()}`));
                }
            }));
        }
        _parseOptionsImplied() {
            const dualHelper = new DualOptions(this.options), hasCustomOptionValue = optionKey => void 0 !== this.getOptionValue(optionKey) && ![ "default", "implied" ].includes(this.getOptionValueSource(optionKey));
            this.options.filter((option => void 0 !== option.implied && hasCustomOptionValue(option.attributeName()) && dualHelper.valueFromOption(this.getOptionValue(option.attributeName()), option))).forEach((option => {
                Object.keys(option.implied).filter((impliedKey => !hasCustomOptionValue(impliedKey))).forEach((impliedKey => {
                    this.setOptionValueWithSource(impliedKey, option.implied[impliedKey], "implied");
                }));
            }));
        }
        missingArgument(name) {
            const message = `error: missing required argument '${name}'`;
            this.error(message, {
                code: "commander.missingArgument"
            });
        }
        optionMissingArgument(option) {
            const message = `error: option '${option.flags}' argument missing`;
            this.error(message, {
                code: "commander.optionMissingArgument"
            });
        }
        missingMandatoryOptionValue(option) {
            const message = `error: required option '${option.flags}' not specified`;
            this.error(message, {
                code: "commander.missingMandatoryOptionValue"
            });
        }
        _conflictingOption(option, conflictingOption) {
            const findBestOptionFromValue = option => {
                const optionKey = option.attributeName(), optionValue = this.getOptionValue(optionKey), negativeOption = this.options.find((target => target.negate && optionKey === target.attributeName())), positiveOption = this.options.find((target => !target.negate && optionKey === target.attributeName()));
                return negativeOption && (void 0 === negativeOption.presetArg && !1 === optionValue || void 0 !== negativeOption.presetArg && optionValue === negativeOption.presetArg) ? negativeOption : positiveOption || option;
            }, getErrorMessage = option => {
                const bestOption = findBestOptionFromValue(option), optionKey = bestOption.attributeName();
                return "env" === this.getOptionValueSource(optionKey) ? `environment variable '${bestOption.envVar}'` : `option '${bestOption.flags}'`;
            }, message = `error: ${getErrorMessage(option)} cannot be used with ${getErrorMessage(conflictingOption)}`;
            this.error(message, {
                code: "commander.conflictingOption"
            });
        }
        unknownOption(flag) {
            if (this._allowUnknownOption) return;
            let suggestion = "";
            if (flag.startsWith("--") && this._showSuggestionAfterError) {
                let candidateFlags = [], command = this;
                do {
                    const moreFlags = command.createHelp().visibleOptions(command).filter((option => option.long)).map((option => option.long));
                    candidateFlags = candidateFlags.concat(moreFlags), command = command.parent;
                } while (command && !command._enablePositionalOptions);
                suggestion = suggestSimilar(flag, candidateFlags);
            }
            const message = `error: unknown option '${flag}'${suggestion}`;
            this.error(message, {
                code: "commander.unknownOption"
            });
        }
        _excessArguments(receivedArgs) {
            if (this._allowExcessArguments) return;
            const expected = this.registeredArguments.length, s = 1 === expected ? "" : "s", message = `error: too many arguments${this.parent ? ` for '${this.name()}'` : ""}. Expected ${expected} argument${s} but got ${receivedArgs.length}.`;
            this.error(message, {
                code: "commander.excessArguments"
            });
        }
        unknownCommand() {
            const unknownName = this.args[0];
            let suggestion = "";
            if (this._showSuggestionAfterError) {
                const candidateNames = [];
                this.createHelp().visibleCommands(this).forEach((command => {
                    candidateNames.push(command.name()), command.alias() && candidateNames.push(command.alias());
                })), suggestion = suggestSimilar(unknownName, candidateNames);
            }
            const message = `error: unknown command '${unknownName}'${suggestion}`;
            this.error(message, {
                code: "commander.unknownCommand"
            });
        }
        version(str, flags, description) {
            if (void 0 === str) return this._version;
            this._version = str, flags = flags || "-V, --version", description = description || "output the version number";
            const versionOption = this.createOption(flags, description);
            return this._versionOptionName = versionOption.attributeName(), this._registerOption(versionOption), 
            this.on("option:" + versionOption.name(), (() => {
                this._outputConfiguration.writeOut(`${str}\n`), this._exit(0, "commander.version", str);
            })), this;
        }
        description(str, argsDescription) {
            return void 0 === str && void 0 === argsDescription ? this._description : (this._description = str, 
            argsDescription && (this._argsDescription = argsDescription), this);
        }
        summary(str) {
            return void 0 === str ? this._summary : (this._summary = str, this);
        }
        alias(alias) {
            if (void 0 === alias) return this._aliases[0];
            let command = this;
            if (0 !== this.commands.length && this.commands[this.commands.length - 1]._executableHandler && (command = this.commands[this.commands.length - 1]), 
            alias === command._name) throw new Error("Command alias can't be the same as its name");
            const matchingCommand = this.parent?._findCommand(alias);
            if (matchingCommand) {
                const existingCmd = [ matchingCommand.name() ].concat(matchingCommand.aliases()).join("|");
                throw new Error(`cannot add alias '${alias}' to command '${this.name()}' as already have command '${existingCmd}'`);
            }
            return command._aliases.push(alias), this;
        }
        aliases(aliases) {
            return void 0 === aliases ? this._aliases : (aliases.forEach((alias => this.alias(alias))), 
            this);
        }
        usage(str) {
            if (void 0 === str) {
                if (this._usage) return this._usage;
                const args = this.registeredArguments.map((arg => humanReadableArgName(arg)));
                return [].concat(this.options.length || null !== this._helpOption ? "[options]" : [], this.commands.length ? "[command]" : [], this.registeredArguments.length ? args : []).join(" ");
            }
            return this._usage = str, this;
        }
        name(str) {
            return void 0 === str ? this._name : (this._name = str, this);
        }
        nameFromFilename(filename) {
            return this._name = path.basename(filename, path.extname(filename)), this;
        }
        executableDir(path) {
            return void 0 === path ? this._executableDir : (this._executableDir = path, this);
        }
        helpInformation(contextOptions) {
            const helper = this.createHelp();
            return void 0 === helper.helpWidth && (helper.helpWidth = contextOptions && contextOptions.error ? this._outputConfiguration.getErrHelpWidth() : this._outputConfiguration.getOutHelpWidth()), 
            helper.formatHelp(this, helper);
        }
        _getHelpContext(contextOptions) {
            const context = {
                error: !!(contextOptions = contextOptions || {}).error
            };
            let write;
            return write = context.error ? arg => this._outputConfiguration.writeErr(arg) : arg => this._outputConfiguration.writeOut(arg), 
            context.write = contextOptions.write || write, context.command = this, context;
        }
        outputHelp(contextOptions) {
            let deprecatedCallback;
            "function" == typeof contextOptions && (deprecatedCallback = contextOptions, contextOptions = void 0);
            const context = this._getHelpContext(contextOptions);
            this._getCommandAndAncestors().reverse().forEach((command => command.emit("beforeAllHelp", context))), 
            this.emit("beforeHelp", context);
            let helpInformation = this.helpInformation(context);
            if (deprecatedCallback && (helpInformation = deprecatedCallback(helpInformation), 
            "string" != typeof helpInformation && !Buffer.isBuffer(helpInformation))) throw new Error("outputHelp callback must return a string or a Buffer");
            context.write(helpInformation), this._getHelpOption()?.long && this.emit(this._getHelpOption().long), 
            this.emit("afterHelp", context), this._getCommandAndAncestors().forEach((command => command.emit("afterAllHelp", context)));
        }
        helpOption(flags, description) {
            return "boolean" == typeof flags ? (this._helpOption = flags ? this._helpOption ?? void 0 : null, 
            this) : (flags = flags ?? "-h, --help", description = description ?? "display help for command", 
            this._helpOption = this.createOption(flags, description), this);
        }
        _getHelpOption() {
            return void 0 === this._helpOption && this.helpOption(void 0, void 0), this._helpOption;
        }
        addHelpOption(option) {
            return this._helpOption = option, this;
        }
        help(contextOptions) {
            this.outputHelp(contextOptions);
            let exitCode = process.exitCode || 0;
            0 === exitCode && contextOptions && "function" != typeof contextOptions && contextOptions.error && (exitCode = 1), 
            this._exit(exitCode, "commander.help", "(outputHelp)");
        }
        addHelpText(position, text) {
            const allowedValues = [ "beforeAll", "before", "after", "afterAll" ];
            if (!allowedValues.includes(position)) throw new Error(`Unexpected value for position to addHelpText.\nExpecting one of '${allowedValues.join("', '")}'`);
            const helpEvent = `${position}Help`;
            return this.on(helpEvent, (context => {
                let helpStr;
                helpStr = "function" == typeof text ? text({
                    error: context.error,
                    command: context.command
                }) : text, helpStr && context.write(`${helpStr}\n`);
            })), this;
        }
        _outputHelpIfRequested(args) {
            const helpOption = this._getHelpOption();
            helpOption && args.find((arg => helpOption.is(arg))) && (this.outputHelp(), this._exit(0, "commander.helpDisplayed", "(outputHelp)"));
        }
    }
    function incrementNodeInspectorPort(args) {
        return args.map((arg => {
            if (!arg.startsWith("--inspect")) return arg;
            let debugOption, match, debugHost = "127.0.0.1", debugPort = "9229";
            return null !== (match = arg.match(/^(--inspect(-brk)?)$/)) ? debugOption = match[1] : null !== (match = arg.match(/^(--inspect(-brk|-port)?)=([^:]+)$/)) ? (debugOption = match[1], 
            /^\d+$/.test(match[3]) ? debugPort = match[3] : debugHost = match[3]) : null !== (match = arg.match(/^(--inspect(-brk|-port)?)=([^:]+):(\d+)$/)) && (debugOption = match[1], 
            debugHost = match[3], debugPort = match[4]), debugOption && "0" !== debugPort ? `${debugOption}=${debugHost}:${parseInt(debugPort) + 1}` : arg;
        }));
    }
    return command$1.Command = Command, command$1;
}

function requireCommander() {
    if (hasRequiredCommander) return commander;
    hasRequiredCommander = 1;
    const {Argument: Argument} = requireArgument(), {Command: Command} = requireCommand(), {CommanderError: CommanderError, InvalidArgumentError: InvalidArgumentError} = requireError(), {Help: Help} = requireHelp(), {Option: Option} = requireOption();
    return commander.program = new Command, commander.createCommand = name => new Command(name), 
    commander.createOption = (flags, description) => new Option(flags, description), 
    commander.createArgument = (name, description) => new Argument(name, description), 
    commander.Command = Command, commander.Option = Option, commander.Argument = Argument, 
    commander.Help = Help, commander.CommanderError = CommanderError, commander.InvalidArgumentError = InvalidArgumentError, 
    commander.InvalidOptionArgumentError = InvalidArgumentError, commander;
}

function requireExtraTypings() {
    return hasRequiredExtraTypings || (hasRequiredExtraTypings = 1, function(module, exports) {
        const commander = requireCommander();
        (exports = module.exports = {}).program = new commander.Command, exports.Argument = commander.Argument, 
        exports.Command = commander.Command, exports.CommanderError = commander.CommanderError, 
        exports.Help = commander.Help, exports.InvalidArgumentError = commander.InvalidArgumentError, 
        exports.InvalidOptionArgumentError = commander.InvalidArgumentError, exports.Option = commander.Option, 
        exports.createCommand = name => new commander.Command(name), exports.createOption = (flags, description) => new commander.Option(flags, description), 
        exports.createArgument = (name, description) => new commander.Argument(name, description);
    }(extraTypings, extraTypings.exports)), extraTypings.exports;
}

var extraTypingsExports = requireExtraTypings(), extraTypingsCommander = getDefaultExportFromCjs(extraTypingsExports);

const {program: program, createCommand: createCommand, createArgument: createArgument, createOption: createOption, CommanderError: CommanderError, InvalidArgumentError: InvalidArgumentError, InvalidOptionArgumentError: InvalidOptionArgumentError, Command: Command, Argument: Argument, Option: Option, Help: Help} = extraTypingsCommander;

var toml = {}, parse = {
    exports: {}
}, tomlParser = {
    exports: {}
}, parser$1, hasRequiredParser$1, createDatetime, hasRequiredCreateDatetime, formatNum, hasRequiredFormatNum, createDatetimeFloat, hasRequiredCreateDatetimeFloat, createDate, hasRequiredCreateDate, createTime, hasRequiredCreateTime, hasRequiredTomlParser, parsePrettyError, hasRequiredParsePrettyError, parseString_1, hasRequiredParseString, parseAsync_1, hasRequiredParseAsync, parseStream_1, hasRequiredParseStream, hasRequiredParse;

function requireParser$1() {
    if (hasRequiredParser$1) return parser$1;
    hasRequiredParser$1 = 1;
    class ParserError extends Error {
        constructor(msg, filename, linenumber) {
            super("[ParserError] " + msg, filename, linenumber), this.name = "ParserError", 
            this.code = "ParserError", Error.captureStackTrace && Error.captureStackTrace(this, ParserError);
        }
    }
    class State {
        constructor(parser) {
            this.parser = parser, this.buf = "", this.returned = null, this.result = null, this.resultTable = null, 
            this.resultArr = null;
        }
    }
    class Parser {
        constructor() {
            this.pos = 0, this.col = 0, this.line = 0, this.obj = {}, this.ctx = this.obj, this.stack = [], 
            this._buf = "", this.char = null, this.ii = 0, this.state = new State(this.parseStart);
        }
        parse(str) {
            if (0 === str.length || null == str.length) return;
            let getNext;
            for (this._buf = String(str), this.ii = -1, this.char = -1; !1 === getNext || this.nextChar(); ) getNext = this.runOne();
            this._buf = null;
        }
        nextChar() {
            return 10 === this.char && (++this.line, this.col = -1), ++this.ii, this.char = this._buf.codePointAt(this.ii), 
            ++this.pos, ++this.col, this.haveBuffer();
        }
        haveBuffer() {
            return this.ii < this._buf.length;
        }
        runOne() {
            return this.state.parser.call(this, this.state.returned);
        }
        finish() {
            let last;
            this.char = 1114112;
            do {
                last = this.state.parser, this.runOne();
            } while (this.state.parser !== last);
            return this.ctx = null, this.state = null, this._buf = null, this.obj;
        }
        next(fn) {
            if ("function" != typeof fn) throw new ParserError("Tried to set state to non-existent state: " + JSON.stringify(fn));
            this.state.parser = fn;
        }
        goto(fn) {
            return this.next(fn), this.runOne();
        }
        call(fn, returnWith) {
            returnWith && this.next(returnWith), this.stack.push(this.state), this.state = new State(fn);
        }
        callNow(fn, returnWith) {
            return this.call(fn, returnWith), this.runOne();
        }
        return(value) {
            if (0 === this.stack.length) throw this.error(new ParserError("Stack underflow"));
            void 0 === value && (value = this.state.buf), this.state = this.stack.pop(), this.state.returned = value;
        }
        returnNow(value) {
            return this.return(value), this.runOne();
        }
        consume() {
            if (1114112 === this.char) throw this.error(new ParserError("Unexpected end-of-buffer"));
            this.state.buf += this._buf[this.ii];
        }
        error(err) {
            return err.line = this.line, err.col = this.col, err.pos = this.pos, err;
        }
        parseStart() {
            throw new ParserError("Must declare a parseStart method");
        }
    }
    return Parser.END = 1114112, Parser.Error = ParserError, parser$1 = Parser;
}

function requireCreateDatetime() {
    return hasRequiredCreateDatetime ? createDatetime : (hasRequiredCreateDatetime = 1, 
    createDatetime = value => {
        const date = new Date(value);
        if (isNaN(date)) throw new TypeError("Invalid Datetime");
        return date;
    });
}

function requireFormatNum() {
    return hasRequiredFormatNum ? formatNum : (hasRequiredFormatNum = 1, formatNum = (d, num) => {
        for (num = String(num); num.length < d; ) num = "0" + num;
        return num;
    });
}

function requireCreateDatetimeFloat() {
    if (hasRequiredCreateDatetimeFloat) return createDatetimeFloat;
    hasRequiredCreateDatetimeFloat = 1;
    const f = requireFormatNum();
    class FloatingDateTime extends Date {
        constructor(value) {
            super(value + "Z"), this.isFloating = !0;
        }
        toISOString() {
            return `${`${this.getUTCFullYear()}-${f(2, this.getUTCMonth() + 1)}-${f(2, this.getUTCDate())}`}T${`${f(2, this.getUTCHours())}:${f(2, this.getUTCMinutes())}:${f(2, this.getUTCSeconds())}.${f(3, this.getUTCMilliseconds())}`}`;
        }
    }
    return createDatetimeFloat = value => {
        const date = new FloatingDateTime(value);
        if (isNaN(date)) throw new TypeError("Invalid Datetime");
        return date;
    };
}

function requireCreateDate() {
    if (hasRequiredCreateDate) return createDate;
    hasRequiredCreateDate = 1;
    const f = requireFormatNum(), DateTime = commonjsGlobal.Date;
    class Date extends DateTime {
        constructor(value) {
            super(value), this.isDate = !0;
        }
        toISOString() {
            return `${this.getUTCFullYear()}-${f(2, this.getUTCMonth() + 1)}-${f(2, this.getUTCDate())}`;
        }
    }
    return createDate = value => {
        const date = new Date(value);
        if (isNaN(date)) throw new TypeError("Invalid Datetime");
        return date;
    };
}

function requireCreateTime() {
    if (hasRequiredCreateTime) return createTime;
    hasRequiredCreateTime = 1;
    const f = requireFormatNum();
    class Time extends Date {
        constructor(value) {
            super(`0000-01-01T${value}Z`), this.isTime = !0;
        }
        toISOString() {
            return `${f(2, this.getUTCHours())}:${f(2, this.getUTCMinutes())}:${f(2, this.getUTCSeconds())}.${f(3, this.getUTCMilliseconds())}`;
        }
    }
    return createTime = value => {
        const date = new Time(value);
        if (isNaN(date)) throw new TypeError("Invalid Datetime");
        return date;
    };
}

function requireTomlParser() {
    if (hasRequiredTomlParser) return tomlParser.exports;
    hasRequiredTomlParser = 1, tomlParser.exports = makeParserClass(requireParser$1()), 
    tomlParser.exports.makeParserClass = makeParserClass;
    class TomlError extends Error {
        constructor(msg) {
            super(msg), this.name = "TomlError", Error.captureStackTrace && Error.captureStackTrace(this, TomlError), 
            this.fromTOML = !0, this.wrapped = null;
        }
    }
    TomlError.wrap = err => {
        const terr = new TomlError(err.message);
        return terr.code = err.code, terr.wrapped = err, terr;
    }, tomlParser.exports.TomlError = TomlError;
    const createDateTime = requireCreateDatetime(), createDateTimeFloat = requireCreateDatetimeFloat(), createDate = requireCreateDate(), createTime = requireCreateTime(), CTRL_I = 9, CTRL_J = 10, CTRL_M = 13, CTRL_CHAR_BOUNDARY = 31, CHAR_SP = 32, CHAR_QUOT = 34, CHAR_NUM = 35, CHAR_APOS = 39, CHAR_PLUS = 43, CHAR_COMMA = 44, CHAR_HYPHEN = 45, CHAR_PERIOD = 46, CHAR_0 = 48, CHAR_1 = 49, CHAR_7 = 55, CHAR_9 = 57, CHAR_COLON = 58, CHAR_EQUALS = 61, CHAR_A = 65, CHAR_E = 69, CHAR_F = 70, CHAR_T = 84, CHAR_U = 85, CHAR_Z = 90, CHAR_LOWBAR = 95, CHAR_a = 97, CHAR_b = 98, CHAR_e = 101, CHAR_f = 102, CHAR_i = 105, CHAR_l = 108, CHAR_n = 110, CHAR_o = 111, CHAR_r = 114, CHAR_s = 115, CHAR_t = 116, CHAR_u = 117, CHAR_x = 120, CHAR_z = 122, CHAR_LCUB = 123, CHAR_RCUB = 125, CHAR_LSQB = 91, CHAR_BSOL = 92, CHAR_RSQB = 93, CHAR_DEL = 127, SURROGATE_FIRST = 55296, SURROGATE_LAST = 57343, escapes = {
        [CHAR_b]: "\b",
        [CHAR_t]: "\t",
        [CHAR_n]: "\n",
        [CHAR_f]: "\f",
        [CHAR_r]: "\r",
        [CHAR_QUOT]: '"',
        [CHAR_BSOL]: "\\"
    };
    function isDigit(cp) {
        return cp >= CHAR_0 && cp <= CHAR_9;
    }
    function isHexit(cp) {
        return cp >= CHAR_A && cp <= CHAR_F || cp >= CHAR_a && cp <= CHAR_f || cp >= CHAR_0 && cp <= CHAR_9;
    }
    function isBit(cp) {
        return cp === CHAR_1 || cp === CHAR_0;
    }
    function isOctit(cp) {
        return cp >= CHAR_0 && cp <= CHAR_7;
    }
    function isAlphaNumQuoteHyphen(cp) {
        return cp >= CHAR_A && cp <= CHAR_Z || cp >= CHAR_a && cp <= CHAR_z || cp >= CHAR_0 && cp <= CHAR_9 || cp === CHAR_APOS || cp === CHAR_QUOT || cp === CHAR_LOWBAR || cp === CHAR_HYPHEN;
    }
    function isAlphaNumHyphen(cp) {
        return cp >= CHAR_A && cp <= CHAR_Z || cp >= CHAR_a && cp <= CHAR_z || cp >= CHAR_0 && cp <= CHAR_9 || cp === CHAR_LOWBAR || cp === CHAR_HYPHEN;
    }
    const _type = Symbol("type"), _declared = Symbol("declared"), hasOwnProperty = Object.prototype.hasOwnProperty, defineProperty = Object.defineProperty, descriptor = {
        configurable: !0,
        enumerable: !0,
        writable: !0,
        value: void 0
    };
    function hasKey(obj, key) {
        return !!hasOwnProperty.call(obj, key) || ("__proto__" === key && defineProperty(obj, "__proto__", descriptor), 
        !1);
    }
    const INLINE_TABLE = Symbol("inline-table");
    function InlineTable() {
        return Object.defineProperties({}, {
            [_type]: {
                value: INLINE_TABLE
            }
        });
    }
    function isInlineTable(obj) {
        return null !== obj && "object" == typeof obj && obj[_type] === INLINE_TABLE;
    }
    const TABLE = Symbol("table");
    function Table() {
        return Object.defineProperties({}, {
            [_type]: {
                value: TABLE
            },
            [_declared]: {
                value: !1,
                writable: !0
            }
        });
    }
    function isTable(obj) {
        return null !== obj && "object" == typeof obj && obj[_type] === TABLE;
    }
    const _contentType = Symbol("content-type"), INLINE_LIST = Symbol("inline-list");
    function InlineList(type) {
        return Object.defineProperties([], {
            [_type]: {
                value: INLINE_LIST
            },
            [_contentType]: {
                value: type
            }
        });
    }
    function isInlineList(obj) {
        return null !== obj && "object" == typeof obj && obj[_type] === INLINE_LIST;
    }
    const LIST = Symbol("list");
    function List() {
        return Object.defineProperties([], {
            [_type]: {
                value: LIST
            }
        });
    }
    function isList(obj) {
        return null !== obj && "object" == typeof obj && obj[_type] === LIST;
    }
    let _custom;
    try {
        const utilInspect = eval("require('util').inspect");
        _custom = utilInspect.custom;
    } catch (_) {}
    const _inspect = _custom || "inspect";
    class BoxedBigInt {
        constructor(value) {
            try {
                this.value = commonjsGlobal.BigInt.asIntN(64, value);
            } catch (_) {
                this.value = null;
            }
            Object.defineProperty(this, _type, {
                value: INTEGER
            });
        }
        isNaN() {
            return null === this.value;
        }
        toString() {
            return String(this.value);
        }
        [_inspect]() {
            return `[BigInt: ${this.toString()}]}`;
        }
        valueOf() {
            return this.value;
        }
    }
    const INTEGER = Symbol("integer");
    function Integer(value) {
        let num = Number(value);
        return Object.is(num, -0) && (num = 0), commonjsGlobal.BigInt && !Number.isSafeInteger(num) ? new BoxedBigInt(value) : Object.defineProperties(new Number(num), {
            isNaN: {
                value: function() {
                    return isNaN(this);
                }
            },
            [_type]: {
                value: INTEGER
            },
            [_inspect]: {
                value: () => `[Integer: ${value}]`
            }
        });
    }
    function isInteger(obj) {
        return null !== obj && "object" == typeof obj && obj[_type] === INTEGER;
    }
    const FLOAT = Symbol("float");
    function Float(value) {
        return Object.defineProperties(new Number(value), {
            [_type]: {
                value: FLOAT
            },
            [_inspect]: {
                value: () => `[Float: ${value}]`
            }
        });
    }
    function isFloat(obj) {
        return null !== obj && "object" == typeof obj && obj[_type] === FLOAT;
    }
    function tomlType(value) {
        const type = typeof value;
        if ("object" === type) {
            if (null === value) return "null";
            if (value instanceof Date) return "datetime";
            if (_type in value) switch (value[_type]) {
              case INLINE_TABLE:
                return "inline-table";

              case INLINE_LIST:
                return "inline-list";

              case TABLE:
                return "table";

              case LIST:
                return "list";

              case FLOAT:
                return "float";

              case INTEGER:
                return "integer";
            }
        }
        return type;
    }
    function makeParserClass(Parser) {
        return class extends Parser {
            constructor() {
                super(), this.ctx = this.obj = Table();
            }
            atEndOfWord() {
                return this.char === CHAR_NUM || this.char === CTRL_I || this.char === CHAR_SP || this.atEndOfLine();
            }
            atEndOfLine() {
                return this.char === Parser.END || this.char === CTRL_J || this.char === CTRL_M;
            }
            parseStart() {
                if (this.char === Parser.END) return null;
                if (this.char === CHAR_LSQB) return this.call(this.parseTableOrList);
                if (this.char === CHAR_NUM) return this.call(this.parseComment);
                if (this.char === CTRL_J || this.char === CHAR_SP || this.char === CTRL_I || this.char === CTRL_M) return null;
                if (isAlphaNumQuoteHyphen(this.char)) return this.callNow(this.parseAssignStatement);
                throw this.error(new TomlError(`Unknown character "${this.char}"`));
            }
            parseWhitespaceToEOL() {
                if (this.char === CHAR_SP || this.char === CTRL_I || this.char === CTRL_M) return null;
                if (this.char === CHAR_NUM) return this.goto(this.parseComment);
                if (this.char === Parser.END || this.char === CTRL_J) return this.return();
                throw this.error(new TomlError("Unexpected character, expected only whitespace or comments till end of line"));
            }
            parseAssignStatement() {
                return this.callNow(this.parseAssign, this.recordAssignStatement);
            }
            recordAssignStatement(kv) {
                let target = this.ctx, finalKey = kv.key.pop();
                for (let kw of kv.key) {
                    if (hasKey(target, kw) && (!isTable(target[kw]) || target[kw][_declared])) throw this.error(new TomlError("Can't redefine existing key"));
                    target = target[kw] = target[kw] || Table();
                }
                if (hasKey(target, finalKey)) throw this.error(new TomlError("Can't redefine existing key"));
                return isInteger(kv.value) || isFloat(kv.value) ? target[finalKey] = kv.value.valueOf() : target[finalKey] = kv.value, 
                this.goto(this.parseWhitespaceToEOL);
            }
            parseAssign() {
                return this.callNow(this.parseKeyword, this.recordAssignKeyword);
            }
            recordAssignKeyword(key) {
                return this.state.resultTable ? this.state.resultTable.push(key) : this.state.resultTable = [ key ], 
                this.goto(this.parseAssignKeywordPreDot);
            }
            parseAssignKeywordPreDot() {
                return this.char === CHAR_PERIOD ? this.next(this.parseAssignKeywordPostDot) : this.char !== CHAR_SP && this.char !== CTRL_I ? this.goto(this.parseAssignEqual) : void 0;
            }
            parseAssignKeywordPostDot() {
                if (this.char !== CHAR_SP && this.char !== CTRL_I) return this.callNow(this.parseKeyword, this.recordAssignKeyword);
            }
            parseAssignEqual() {
                if (this.char === CHAR_EQUALS) return this.next(this.parseAssignPreValue);
                throw this.error(new TomlError('Invalid character, expected "="'));
            }
            parseAssignPreValue() {
                return this.char === CHAR_SP || this.char === CTRL_I ? null : this.callNow(this.parseValue, this.recordAssignValue);
            }
            recordAssignValue(value) {
                return this.returnNow({
                    key: this.state.resultTable,
                    value: value
                });
            }
            parseComment() {
                do {
                    if (this.char === Parser.END || this.char === CTRL_J) return this.return();
                } while (this.nextChar());
            }
            parseTableOrList() {
                if (this.char !== CHAR_LSQB) return this.goto(this.parseTable);
                this.next(this.parseList);
            }
            parseTable() {
                return this.ctx = this.obj, this.goto(this.parseTableNext);
            }
            parseTableNext() {
                return this.char === CHAR_SP || this.char === CTRL_I ? null : this.callNow(this.parseKeyword, this.parseTableMore);
            }
            parseTableMore(keyword) {
                if (this.char === CHAR_SP || this.char === CTRL_I) return null;
                if (this.char === CHAR_RSQB) {
                    if (hasKey(this.ctx, keyword) && (!isTable(this.ctx[keyword]) || this.ctx[keyword][_declared])) throw this.error(new TomlError("Can't redefine existing key"));
                    return this.ctx = this.ctx[keyword] = this.ctx[keyword] || Table(), this.ctx[_declared] = !0, 
                    this.next(this.parseWhitespaceToEOL);
                }
                if (this.char === CHAR_PERIOD) {
                    if (hasKey(this.ctx, keyword)) if (isTable(this.ctx[keyword])) this.ctx = this.ctx[keyword]; else {
                        if (!isList(this.ctx[keyword])) throw this.error(new TomlError("Can't redefine existing key"));
                        this.ctx = this.ctx[keyword][this.ctx[keyword].length - 1];
                    } else this.ctx = this.ctx[keyword] = Table();
                    return this.next(this.parseTableNext);
                }
                throw this.error(new TomlError("Unexpected character, expected whitespace, . or ]"));
            }
            parseList() {
                return this.ctx = this.obj, this.goto(this.parseListNext);
            }
            parseListNext() {
                return this.char === CHAR_SP || this.char === CTRL_I ? null : this.callNow(this.parseKeyword, this.parseListMore);
            }
            parseListMore(keyword) {
                if (this.char === CHAR_SP || this.char === CTRL_I) return null;
                if (this.char === CHAR_RSQB) {
                    if (hasKey(this.ctx, keyword) || (this.ctx[keyword] = List()), isInlineList(this.ctx[keyword])) throw this.error(new TomlError("Can't extend an inline array"));
                    if (!isList(this.ctx[keyword])) throw this.error(new TomlError("Can't redefine an existing key"));
                    {
                        const next = Table();
                        this.ctx[keyword].push(next), this.ctx = next;
                    }
                    return this.next(this.parseListEnd);
                }
                if (this.char === CHAR_PERIOD) {
                    if (hasKey(this.ctx, keyword)) {
                        if (isInlineList(this.ctx[keyword])) throw this.error(new TomlError("Can't extend an inline array"));
                        if (isInlineTable(this.ctx[keyword])) throw this.error(new TomlError("Can't extend an inline table"));
                        if (isList(this.ctx[keyword])) this.ctx = this.ctx[keyword][this.ctx[keyword].length - 1]; else {
                            if (!isTable(this.ctx[keyword])) throw this.error(new TomlError("Can't redefine an existing key"));
                            this.ctx = this.ctx[keyword];
                        }
                    } else this.ctx = this.ctx[keyword] = Table();
                    return this.next(this.parseListNext);
                }
                throw this.error(new TomlError("Unexpected character, expected whitespace, . or ]"));
            }
            parseListEnd(keyword) {
                if (this.char === CHAR_RSQB) return this.next(this.parseWhitespaceToEOL);
                throw this.error(new TomlError("Unexpected character, expected whitespace, . or ]"));
            }
            parseValue() {
                if (this.char === Parser.END) throw this.error(new TomlError("Key without value"));
                if (this.char === CHAR_QUOT) return this.next(this.parseDoubleString);
                if (this.char === CHAR_APOS) return this.next(this.parseSingleString);
                if (this.char === CHAR_HYPHEN || this.char === CHAR_PLUS) return this.goto(this.parseNumberSign);
                if (this.char === CHAR_i) return this.next(this.parseInf);
                if (this.char === CHAR_n) return this.next(this.parseNan);
                if (isDigit(this.char)) return this.goto(this.parseNumberOrDateTime);
                if (this.char === CHAR_t || this.char === CHAR_f) return this.goto(this.parseBoolean);
                if (this.char === CHAR_LSQB) return this.call(this.parseInlineList, this.recordValue);
                if (this.char === CHAR_LCUB) return this.call(this.parseInlineTable, this.recordValue);
                throw this.error(new TomlError("Unexpected character, expecting string, number, datetime, boolean, inline array or inline table"));
            }
            recordValue(value) {
                return this.returnNow(value);
            }
            parseInf() {
                if (this.char === CHAR_n) return this.next(this.parseInf2);
                throw this.error(new TomlError('Unexpected character, expected "inf", "+inf" or "-inf"'));
            }
            parseInf2() {
                if (this.char === CHAR_f) return "-" === this.state.buf ? this.return(-1 / 0) : this.return(1 / 0);
                throw this.error(new TomlError('Unexpected character, expected "inf", "+inf" or "-inf"'));
            }
            parseNan() {
                if (this.char === CHAR_a) return this.next(this.parseNan2);
                throw this.error(new TomlError('Unexpected character, expected "nan"'));
            }
            parseNan2() {
                if (this.char === CHAR_n) return this.return(NaN);
                throw this.error(new TomlError('Unexpected character, expected "nan"'));
            }
            parseKeyword() {
                return this.char === CHAR_QUOT ? this.next(this.parseBasicString) : this.char === CHAR_APOS ? this.next(this.parseLiteralString) : this.goto(this.parseBareKey);
            }
            parseBareKey() {
                do {
                    if (this.char === Parser.END) throw this.error(new TomlError("Key ended without value"));
                    if (!isAlphaNumHyphen(this.char)) {
                        if (0 === this.state.buf.length) throw this.error(new TomlError("Empty bare keys are not allowed"));
                        return this.returnNow();
                    }
                    this.consume();
                } while (this.nextChar());
            }
            parseSingleString() {
                return this.char === CHAR_APOS ? this.next(this.parseLiteralMultiStringMaybe) : this.goto(this.parseLiteralString);
            }
            parseLiteralString() {
                do {
                    if (this.char === CHAR_APOS) return this.return();
                    if (this.atEndOfLine()) throw this.error(new TomlError("Unterminated string"));
                    if (this.char === CHAR_DEL || this.char <= CTRL_CHAR_BOUNDARY && this.char !== CTRL_I) throw this.errorControlCharInString();
                    this.consume();
                } while (this.nextChar());
            }
            parseLiteralMultiStringMaybe() {
                return this.char === CHAR_APOS ? this.next(this.parseLiteralMultiString) : this.returnNow();
            }
            parseLiteralMultiString() {
                return this.char === CTRL_M ? null : this.char === CTRL_J ? this.next(this.parseLiteralMultiStringContent) : this.goto(this.parseLiteralMultiStringContent);
            }
            parseLiteralMultiStringContent() {
                do {
                    if (this.char === CHAR_APOS) return this.next(this.parseLiteralMultiEnd);
                    if (this.char === Parser.END) throw this.error(new TomlError("Unterminated multi-line string"));
                    if (this.char === CHAR_DEL || this.char <= CTRL_CHAR_BOUNDARY && this.char !== CTRL_I && this.char !== CTRL_J && this.char !== CTRL_M) throw this.errorControlCharInString();
                    this.consume();
                } while (this.nextChar());
            }
            parseLiteralMultiEnd() {
                return this.char === CHAR_APOS ? this.next(this.parseLiteralMultiEnd2) : (this.state.buf += "'", 
                this.goto(this.parseLiteralMultiStringContent));
            }
            parseLiteralMultiEnd2() {
                return this.char === CHAR_APOS ? this.return() : (this.state.buf += "''", this.goto(this.parseLiteralMultiStringContent));
            }
            parseDoubleString() {
                return this.char === CHAR_QUOT ? this.next(this.parseMultiStringMaybe) : this.goto(this.parseBasicString);
            }
            parseBasicString() {
                do {
                    if (this.char === CHAR_BSOL) return this.call(this.parseEscape, this.recordEscapeReplacement);
                    if (this.char === CHAR_QUOT) return this.return();
                    if (this.atEndOfLine()) throw this.error(new TomlError("Unterminated string"));
                    if (this.char === CHAR_DEL || this.char <= CTRL_CHAR_BOUNDARY && this.char !== CTRL_I) throw this.errorControlCharInString();
                    this.consume();
                } while (this.nextChar());
            }
            recordEscapeReplacement(replacement) {
                return this.state.buf += replacement, this.goto(this.parseBasicString);
            }
            parseMultiStringMaybe() {
                return this.char === CHAR_QUOT ? this.next(this.parseMultiString) : this.returnNow();
            }
            parseMultiString() {
                return this.char === CTRL_M ? null : this.char === CTRL_J ? this.next(this.parseMultiStringContent) : this.goto(this.parseMultiStringContent);
            }
            parseMultiStringContent() {
                do {
                    if (this.char === CHAR_BSOL) return this.call(this.parseMultiEscape, this.recordMultiEscapeReplacement);
                    if (this.char === CHAR_QUOT) return this.next(this.parseMultiEnd);
                    if (this.char === Parser.END) throw this.error(new TomlError("Unterminated multi-line string"));
                    if (this.char === CHAR_DEL || this.char <= CTRL_CHAR_BOUNDARY && this.char !== CTRL_I && this.char !== CTRL_J && this.char !== CTRL_M) throw this.errorControlCharInString();
                    this.consume();
                } while (this.nextChar());
            }
            errorControlCharInString() {
                let displayCode = "\\u00";
                return this.char < 16 && (displayCode += "0"), displayCode += this.char.toString(16), 
                this.error(new TomlError(`Control characters (codes < 0x1f and 0x7f) are not allowed in strings, use ${displayCode} instead`));
            }
            recordMultiEscapeReplacement(replacement) {
                return this.state.buf += replacement, this.goto(this.parseMultiStringContent);
            }
            parseMultiEnd() {
                return this.char === CHAR_QUOT ? this.next(this.parseMultiEnd2) : (this.state.buf += '"', 
                this.goto(this.parseMultiStringContent));
            }
            parseMultiEnd2() {
                return this.char === CHAR_QUOT ? this.return() : (this.state.buf += '""', this.goto(this.parseMultiStringContent));
            }
            parseMultiEscape() {
                return this.char === CTRL_M || this.char === CTRL_J ? this.next(this.parseMultiTrim) : this.char === CHAR_SP || this.char === CTRL_I ? this.next(this.parsePreMultiTrim) : this.goto(this.parseEscape);
            }
            parsePreMultiTrim() {
                if (this.char === CHAR_SP || this.char === CTRL_I) return null;
                if (this.char === CTRL_M || this.char === CTRL_J) return this.next(this.parseMultiTrim);
                throw this.error(new TomlError("Can't escape whitespace"));
            }
            parseMultiTrim() {
                return this.char === CTRL_J || this.char === CHAR_SP || this.char === CTRL_I || this.char === CTRL_M ? null : this.returnNow();
            }
            parseEscape() {
                if (this.char in escapes) return this.return(escapes[this.char]);
                if (this.char === CHAR_u) return this.call(this.parseSmallUnicode, this.parseUnicodeReturn);
                if (this.char === CHAR_U) return this.call(this.parseLargeUnicode, this.parseUnicodeReturn);
                throw this.error(new TomlError("Unknown escape character: " + this.char));
            }
            parseUnicodeReturn(char) {
                try {
                    const codePoint = parseInt(char, 16);
                    if (codePoint >= SURROGATE_FIRST && codePoint <= SURROGATE_LAST) throw this.error(new TomlError("Invalid unicode, character in range 0xD800 - 0xDFFF is reserved"));
                    return this.returnNow(String.fromCodePoint(codePoint));
                } catch (err) {
                    throw this.error(TomlError.wrap(err));
                }
            }
            parseSmallUnicode() {
                if (!isHexit(this.char)) throw this.error(new TomlError("Invalid character in unicode sequence, expected hex"));
                if (this.consume(), this.state.buf.length >= 4) return this.return();
            }
            parseLargeUnicode() {
                if (!isHexit(this.char)) throw this.error(new TomlError("Invalid character in unicode sequence, expected hex"));
                if (this.consume(), this.state.buf.length >= 8) return this.return();
            }
            parseNumberSign() {
                return this.consume(), this.next(this.parseMaybeSignedInfOrNan);
            }
            parseMaybeSignedInfOrNan() {
                return this.char === CHAR_i ? this.next(this.parseInf) : this.char === CHAR_n ? this.next(this.parseNan) : this.callNow(this.parseNoUnder, this.parseNumberIntegerStart);
            }
            parseNumberIntegerStart() {
                return this.char === CHAR_0 ? (this.consume(), this.next(this.parseNumberIntegerExponentOrDecimal)) : this.goto(this.parseNumberInteger);
            }
            parseNumberIntegerExponentOrDecimal() {
                return this.char === CHAR_PERIOD ? (this.consume(), this.call(this.parseNoUnder, this.parseNumberFloat)) : this.char === CHAR_E || this.char === CHAR_e ? (this.consume(), 
                this.next(this.parseNumberExponentSign)) : this.returnNow(Integer(this.state.buf));
            }
            parseNumberInteger() {
                if (!isDigit(this.char)) {
                    if (this.char === CHAR_LOWBAR) return this.call(this.parseNoUnder);
                    if (this.char === CHAR_E || this.char === CHAR_e) return this.consume(), this.next(this.parseNumberExponentSign);
                    if (this.char === CHAR_PERIOD) return this.consume(), this.call(this.parseNoUnder, this.parseNumberFloat);
                    {
                        const result = Integer(this.state.buf);
                        if (result.isNaN()) throw this.error(new TomlError("Invalid number"));
                        return this.returnNow(result);
                    }
                }
                this.consume();
            }
            parseNoUnder() {
                if (this.char === CHAR_LOWBAR || this.char === CHAR_PERIOD || this.char === CHAR_E || this.char === CHAR_e) throw this.error(new TomlError("Unexpected character, expected digit"));
                if (this.atEndOfWord()) throw this.error(new TomlError("Incomplete number"));
                return this.returnNow();
            }
            parseNoUnderHexOctBinLiteral() {
                if (this.char === CHAR_LOWBAR || this.char === CHAR_PERIOD) throw this.error(new TomlError("Unexpected character, expected digit"));
                if (this.atEndOfWord()) throw this.error(new TomlError("Incomplete number"));
                return this.returnNow();
            }
            parseNumberFloat() {
                return this.char === CHAR_LOWBAR ? this.call(this.parseNoUnder, this.parseNumberFloat) : isDigit(this.char) ? void this.consume() : this.char === CHAR_E || this.char === CHAR_e ? (this.consume(), 
                this.next(this.parseNumberExponentSign)) : this.returnNow(Float(this.state.buf));
            }
            parseNumberExponentSign() {
                if (isDigit(this.char)) return this.goto(this.parseNumberExponent);
                if (this.char !== CHAR_HYPHEN && this.char !== CHAR_PLUS) throw this.error(new TomlError("Unexpected character, expected -, + or digit"));
                this.consume(), this.call(this.parseNoUnder, this.parseNumberExponent);
            }
            parseNumberExponent() {
                if (!isDigit(this.char)) return this.char === CHAR_LOWBAR ? this.call(this.parseNoUnder) : this.returnNow(Float(this.state.buf));
                this.consume();
            }
            parseNumberOrDateTime() {
                return this.char === CHAR_0 ? (this.consume(), this.next(this.parseNumberBaseOrDateTime)) : this.goto(this.parseNumberOrDateTimeOnly);
            }
            parseNumberOrDateTimeOnly() {
                return this.char === CHAR_LOWBAR ? this.call(this.parseNoUnder, this.parseNumberInteger) : isDigit(this.char) ? (this.consume(), 
                void (this.state.buf.length > 4 && this.next(this.parseNumberInteger))) : this.char === CHAR_E || this.char === CHAR_e ? (this.consume(), 
                this.next(this.parseNumberExponentSign)) : this.char === CHAR_PERIOD ? (this.consume(), 
                this.call(this.parseNoUnder, this.parseNumberFloat)) : this.char === CHAR_HYPHEN ? this.goto(this.parseDateTime) : this.char === CHAR_COLON ? this.goto(this.parseOnlyTimeHour) : this.returnNow(Integer(this.state.buf));
            }
            parseDateTimeOnly() {
                if (this.state.buf.length < 4) {
                    if (isDigit(this.char)) return this.consume();
                    if (this.char === CHAR_COLON) return this.goto(this.parseOnlyTimeHour);
                    throw this.error(new TomlError("Expected digit while parsing year part of a date"));
                }
                if (this.char === CHAR_HYPHEN) return this.goto(this.parseDateTime);
                throw this.error(new TomlError("Expected hyphen (-) while parsing year part of date"));
            }
            parseNumberBaseOrDateTime() {
                return this.char === CHAR_b ? (this.consume(), this.call(this.parseNoUnderHexOctBinLiteral, this.parseIntegerBin)) : this.char === CHAR_o ? (this.consume(), 
                this.call(this.parseNoUnderHexOctBinLiteral, this.parseIntegerOct)) : this.char === CHAR_x ? (this.consume(), 
                this.call(this.parseNoUnderHexOctBinLiteral, this.parseIntegerHex)) : this.char === CHAR_PERIOD ? this.goto(this.parseNumberInteger) : isDigit(this.char) ? this.goto(this.parseDateTimeOnly) : this.returnNow(Integer(this.state.buf));
            }
            parseIntegerHex() {
                if (!isHexit(this.char)) {
                    if (this.char === CHAR_LOWBAR) return this.call(this.parseNoUnderHexOctBinLiteral);
                    {
                        const result = Integer(this.state.buf);
                        if (result.isNaN()) throw this.error(new TomlError("Invalid number"));
                        return this.returnNow(result);
                    }
                }
                this.consume();
            }
            parseIntegerOct() {
                if (!isOctit(this.char)) {
                    if (this.char === CHAR_LOWBAR) return this.call(this.parseNoUnderHexOctBinLiteral);
                    {
                        const result = Integer(this.state.buf);
                        if (result.isNaN()) throw this.error(new TomlError("Invalid number"));
                        return this.returnNow(result);
                    }
                }
                this.consume();
            }
            parseIntegerBin() {
                if (!isBit(this.char)) {
                    if (this.char === CHAR_LOWBAR) return this.call(this.parseNoUnderHexOctBinLiteral);
                    {
                        const result = Integer(this.state.buf);
                        if (result.isNaN()) throw this.error(new TomlError("Invalid number"));
                        return this.returnNow(result);
                    }
                }
                this.consume();
            }
            parseDateTime() {
                if (this.state.buf.length < 4) throw this.error(new TomlError("Years less than 1000 must be zero padded to four characters"));
                return this.state.result = this.state.buf, this.state.buf = "", this.next(this.parseDateMonth);
            }
            parseDateMonth() {
                if (this.char === CHAR_HYPHEN) {
                    if (this.state.buf.length < 2) throw this.error(new TomlError("Months less than 10 must be zero padded to two characters"));
                    return this.state.result += "-" + this.state.buf, this.state.buf = "", this.next(this.parseDateDay);
                }
                if (!isDigit(this.char)) throw this.error(new TomlError("Incomplete datetime"));
                this.consume();
            }
            parseDateDay() {
                if (this.char === CHAR_T || this.char === CHAR_SP) {
                    if (this.state.buf.length < 2) throw this.error(new TomlError("Days less than 10 must be zero padded to two characters"));
                    return this.state.result += "-" + this.state.buf, this.state.buf = "", this.next(this.parseStartTimeHour);
                }
                if (this.atEndOfWord()) return this.returnNow(createDate(this.state.result + "-" + this.state.buf));
                if (!isDigit(this.char)) throw this.error(new TomlError("Incomplete datetime"));
                this.consume();
            }
            parseStartTimeHour() {
                return this.atEndOfWord() ? this.returnNow(createDate(this.state.result)) : this.goto(this.parseTimeHour);
            }
            parseTimeHour() {
                if (this.char === CHAR_COLON) {
                    if (this.state.buf.length < 2) throw this.error(new TomlError("Hours less than 10 must be zero padded to two characters"));
                    return this.state.result += "T" + this.state.buf, this.state.buf = "", this.next(this.parseTimeMin);
                }
                if (!isDigit(this.char)) throw this.error(new TomlError("Incomplete datetime"));
                this.consume();
            }
            parseTimeMin() {
                if (!(this.state.buf.length < 2 && isDigit(this.char))) {
                    if (2 === this.state.buf.length && this.char === CHAR_COLON) return this.state.result += ":" + this.state.buf, 
                    this.state.buf = "", this.next(this.parseTimeSec);
                    throw this.error(new TomlError("Incomplete datetime"));
                }
                this.consume();
            }
            parseTimeSec() {
                if (!isDigit(this.char)) throw this.error(new TomlError("Incomplete datetime"));
                if (this.consume(), 2 === this.state.buf.length) return this.state.result += ":" + this.state.buf, 
                this.state.buf = "", this.next(this.parseTimeZoneOrFraction);
            }
            parseOnlyTimeHour() {
                if (this.char === CHAR_COLON) {
                    if (this.state.buf.length < 2) throw this.error(new TomlError("Hours less than 10 must be zero padded to two characters"));
                    return this.state.result = this.state.buf, this.state.buf = "", this.next(this.parseOnlyTimeMin);
                }
                throw this.error(new TomlError("Incomplete time"));
            }
            parseOnlyTimeMin() {
                if (!(this.state.buf.length < 2 && isDigit(this.char))) {
                    if (2 === this.state.buf.length && this.char === CHAR_COLON) return this.state.result += ":" + this.state.buf, 
                    this.state.buf = "", this.next(this.parseOnlyTimeSec);
                    throw this.error(new TomlError("Incomplete time"));
                }
                this.consume();
            }
            parseOnlyTimeSec() {
                if (!isDigit(this.char)) throw this.error(new TomlError("Incomplete time"));
                if (this.consume(), 2 === this.state.buf.length) return this.next(this.parseOnlyTimeFractionMaybe);
            }
            parseOnlyTimeFractionMaybe() {
                if (this.state.result += ":" + this.state.buf, this.char !== CHAR_PERIOD) return this.return(createTime(this.state.result));
                this.state.buf = "", this.next(this.parseOnlyTimeFraction);
            }
            parseOnlyTimeFraction() {
                if (!isDigit(this.char)) {
                    if (this.atEndOfWord()) {
                        if (0 === this.state.buf.length) throw this.error(new TomlError("Expected digit in milliseconds"));
                        return this.returnNow(createTime(this.state.result + "." + this.state.buf));
                    }
                    throw this.error(new TomlError("Unexpected character in datetime, expected period (.), minus (-), plus (+) or Z"));
                }
                this.consume();
            }
            parseTimeZoneOrFraction() {
                if (this.char === CHAR_PERIOD) this.consume(), this.next(this.parseDateTimeFraction); else {
                    if (this.char !== CHAR_HYPHEN && this.char !== CHAR_PLUS) {
                        if (this.char === CHAR_Z) return this.consume(), this.return(createDateTime(this.state.result + this.state.buf));
                        if (this.atEndOfWord()) return this.returnNow(createDateTimeFloat(this.state.result + this.state.buf));
                        throw this.error(new TomlError("Unexpected character in datetime, expected period (.), minus (-), plus (+) or Z"));
                    }
                    this.consume(), this.next(this.parseTimeZoneHour);
                }
            }
            parseDateTimeFraction() {
                if (isDigit(this.char)) this.consume(); else {
                    if (1 === this.state.buf.length) throw this.error(new TomlError("Expected digit in milliseconds"));
                    if (this.char !== CHAR_HYPHEN && this.char !== CHAR_PLUS) {
                        if (this.char === CHAR_Z) return this.consume(), this.return(createDateTime(this.state.result + this.state.buf));
                        if (this.atEndOfWord()) return this.returnNow(createDateTimeFloat(this.state.result + this.state.buf));
                        throw this.error(new TomlError("Unexpected character in datetime, expected period (.), minus (-), plus (+) or Z"));
                    }
                    this.consume(), this.next(this.parseTimeZoneHour);
                }
            }
            parseTimeZoneHour() {
                if (!isDigit(this.char)) throw this.error(new TomlError("Unexpected character in datetime, expected digit"));
                if (this.consume(), /\d\d$/.test(this.state.buf)) return this.next(this.parseTimeZoneSep);
            }
            parseTimeZoneSep() {
                if (this.char !== CHAR_COLON) throw this.error(new TomlError("Unexpected character in datetime, expected colon"));
                this.consume(), this.next(this.parseTimeZoneMin);
            }
            parseTimeZoneMin() {
                if (!isDigit(this.char)) throw this.error(new TomlError("Unexpected character in datetime, expected digit"));
                if (this.consume(), /\d\d$/.test(this.state.buf)) return this.return(createDateTime(this.state.result + this.state.buf));
            }
            parseBoolean() {
                return this.char === CHAR_t ? (this.consume(), this.next(this.parseTrue_r)) : this.char === CHAR_f ? (this.consume(), 
                this.next(this.parseFalse_a)) : void 0;
            }
            parseTrue_r() {
                if (this.char === CHAR_r) return this.consume(), this.next(this.parseTrue_u);
                throw this.error(new TomlError("Invalid boolean, expected true or false"));
            }
            parseTrue_u() {
                if (this.char === CHAR_u) return this.consume(), this.next(this.parseTrue_e);
                throw this.error(new TomlError("Invalid boolean, expected true or false"));
            }
            parseTrue_e() {
                if (this.char === CHAR_e) return this.return(!0);
                throw this.error(new TomlError("Invalid boolean, expected true or false"));
            }
            parseFalse_a() {
                if (this.char === CHAR_a) return this.consume(), this.next(this.parseFalse_l);
                throw this.error(new TomlError("Invalid boolean, expected true or false"));
            }
            parseFalse_l() {
                if (this.char === CHAR_l) return this.consume(), this.next(this.parseFalse_s);
                throw this.error(new TomlError("Invalid boolean, expected true or false"));
            }
            parseFalse_s() {
                if (this.char === CHAR_s) return this.consume(), this.next(this.parseFalse_e);
                throw this.error(new TomlError("Invalid boolean, expected true or false"));
            }
            parseFalse_e() {
                if (this.char === CHAR_e) return this.return(!1);
                throw this.error(new TomlError("Invalid boolean, expected true or false"));
            }
            parseInlineList() {
                if (this.char === CHAR_SP || this.char === CTRL_I || this.char === CTRL_M || this.char === CTRL_J) return null;
                if (this.char === Parser.END) throw this.error(new TomlError("Unterminated inline array"));
                return this.char === CHAR_NUM ? this.call(this.parseComment) : this.char === CHAR_RSQB ? this.return(this.state.resultArr || InlineList()) : this.callNow(this.parseValue, this.recordInlineListValue);
            }
            recordInlineListValue(value) {
                if (this.state.resultArr) {
                    const listType = this.state.resultArr[_contentType], valueType = tomlType(value);
                    if (listType !== valueType) throw this.error(new TomlError(`Inline lists must be a single type, not a mix of ${listType} and ${valueType}`));
                } else this.state.resultArr = InlineList(tomlType(value));
                return isFloat(value) || isInteger(value) ? this.state.resultArr.push(value.valueOf()) : this.state.resultArr.push(value), 
                this.goto(this.parseInlineListNext);
            }
            parseInlineListNext() {
                if (this.char === CHAR_SP || this.char === CTRL_I || this.char === CTRL_M || this.char === CTRL_J) return null;
                if (this.char === CHAR_NUM) return this.call(this.parseComment);
                if (this.char === CHAR_COMMA) return this.next(this.parseInlineList);
                if (this.char === CHAR_RSQB) return this.goto(this.parseInlineList);
                throw this.error(new TomlError("Invalid character, expected whitespace, comma (,) or close bracket (])"));
            }
            parseInlineTable() {
                if (this.char === CHAR_SP || this.char === CTRL_I) return null;
                if (this.char === Parser.END || this.char === CHAR_NUM || this.char === CTRL_J || this.char === CTRL_M) throw this.error(new TomlError("Unterminated inline array"));
                return this.char === CHAR_RCUB ? this.return(this.state.resultTable || InlineTable()) : (this.state.resultTable || (this.state.resultTable = InlineTable()), 
                this.callNow(this.parseAssign, this.recordInlineTableValue));
            }
            recordInlineTableValue(kv) {
                let target = this.state.resultTable, finalKey = kv.key.pop();
                for (let kw of kv.key) {
                    if (hasKey(target, kw) && (!isTable(target[kw]) || target[kw][_declared])) throw this.error(new TomlError("Can't redefine existing key"));
                    target = target[kw] = target[kw] || Table();
                }
                if (hasKey(target, finalKey)) throw this.error(new TomlError("Can't redefine existing key"));
                return isInteger(kv.value) || isFloat(kv.value) ? target[finalKey] = kv.value.valueOf() : target[finalKey] = kv.value, 
                this.goto(this.parseInlineTableNext);
            }
            parseInlineTableNext() {
                if (this.char === CHAR_SP || this.char === CTRL_I) return null;
                if (this.char === Parser.END || this.char === CHAR_NUM || this.char === CTRL_J || this.char === CTRL_M) throw this.error(new TomlError("Unterminated inline array"));
                if (this.char === CHAR_COMMA) return this.next(this.parseInlineTable);
                if (this.char === CHAR_RCUB) return this.goto(this.parseInlineTable);
                throw this.error(new TomlError("Invalid character, expected whitespace, comma (,) or close bracket (])"));
            }
        };
    }
    return tomlParser.exports;
}

function requireParsePrettyError() {
    if (hasRequiredParsePrettyError) return parsePrettyError;
    return hasRequiredParsePrettyError = 1, parsePrettyError = function(err, buf) {
        if (null == err.pos || null == err.line) return err;
        let msg = err.message;
        if (msg += ` at row ${err.line + 1}, col ${err.col + 1}, pos ${err.pos}:\n`, buf && buf.split) {
            const lines = buf.split(/\n/), lineNumWidth = String(Math.min(lines.length, err.line + 3)).length;
            let linePadding = " ";
            for (;linePadding.length < lineNumWidth; ) linePadding += " ";
            for (let ii = Math.max(0, err.line - 1); ii < Math.min(lines.length, err.line + 2); ++ii) {
                let lineNum = String(ii + 1);
                if (lineNum.length < lineNumWidth && (lineNum = " " + lineNum), err.line === ii) {
                    msg += lineNum + "> " + lines[ii] + "\n", msg += linePadding + "  ";
                    for (let hh = 0; hh < err.col; ++hh) msg += " ";
                    msg += "^\n";
                } else msg += lineNum + ": " + lines[ii] + "\n";
            }
        }
        return err.message = msg + "\n", err;
    }, parsePrettyError;
}

function requireParseString() {
    if (hasRequiredParseString) return parseString_1;
    hasRequiredParseString = 1, parseString_1 = function(str) {
        commonjsGlobal.Buffer && commonjsGlobal.Buffer.isBuffer(str) && (str = str.toString("utf8"));
        const parser = new TOMLParser;
        try {
            return parser.parse(str), parser.finish();
        } catch (err) {
            throw prettyError(err, str);
        }
    };
    const TOMLParser = requireTomlParser(), prettyError = requireParsePrettyError();
    return parseString_1;
}

function requireParseAsync() {
    if (hasRequiredParseAsync) return parseAsync_1;
    hasRequiredParseAsync = 1, parseAsync_1 = function(str, opts) {
        opts || (opts = {});
        const blocksize = opts.blocksize || 40960, parser = new TOMLParser;
        return new Promise(((resolve, reject) => {
            setImmediate(parseAsyncNext, 0, blocksize, resolve, reject);
        }));
        function parseAsyncNext(index, blocksize, resolve, reject) {
            if (index >= str.length) try {
                return resolve(parser.finish());
            } catch (err) {
                return reject(prettyError(err, str));
            }
            try {
                parser.parse(str.slice(index, index + blocksize)), setImmediate(parseAsyncNext, index + blocksize, blocksize, resolve, reject);
            } catch (err) {
                reject(prettyError(err, str));
            }
        }
    };
    const TOMLParser = requireTomlParser(), prettyError = requireParsePrettyError();
    return parseAsync_1;
}

function requireParseStream() {
    if (hasRequiredParseStream) return parseStream_1;
    hasRequiredParseStream = 1, parseStream_1 = function(stm) {
        return stm ? function(stm) {
            const parser = new TOMLParser;
            return stm.setEncoding("utf8"), new Promise(((resolve, reject) => {
                let readable, ended = !1, errored = !1;
                function finish() {
                    if (ended = !0, !readable) try {
                        resolve(parser.finish());
                    } catch (err) {
                        reject(err);
                    }
                }
                function error(err) {
                    errored = !0, reject(err);
                }
                function readNext() {
                    let data;
                    for (readable = !0; null !== (data = stm.read()); ) try {
                        parser.parse(data);
                    } catch (err) {
                        return error(err);
                    }
                    if (readable = !1, ended) return finish();
                    errored || stm.once("readable", readNext);
                }
                stm.once("end", finish), stm.once("error", error), readNext();
            }));
        }(stm) : function() {
            const parser = new TOMLParser;
            return new stream.Transform({
                objectMode: !0,
                transform(chunk, encoding, cb) {
                    try {
                        parser.parse(chunk.toString(encoding));
                    } catch (err) {
                        this.emit("error", err);
                    }
                    cb();
                },
                flush(cb) {
                    try {
                        this.push(parser.finish());
                    } catch (err) {
                        this.emit("error", err);
                    }
                    cb();
                }
            });
        }();
    };
    const stream = require$$0$2, TOMLParser = requireTomlParser();
    return parseStream_1;
}

function requireParse() {
    return hasRequiredParse || (hasRequiredParse = 1, parse.exports = requireParseString(), 
    parse.exports.async = requireParseAsync(), parse.exports.stream = requireParseStream(), 
    parse.exports.prettyError = requireParsePrettyError()), parse.exports;
}

var stringify$2 = {
    exports: {}
}, hasRequiredStringify$1, hasRequiredToml;

function requireStringify$1() {
    if (hasRequiredStringify$1) return stringify$2.exports;
    function typeError(type) {
        return new Error("Can only stringify objects, not " + type);
    }
    function getInlineKeys(obj) {
        return Object.keys(obj).filter((key => isInline(obj[key])));
    }
    function toJSON(obj) {
        let nobj = Array.isArray(obj) ? [] : Object.prototype.hasOwnProperty.call(obj, "__proto__") ? {
            ["__proto__"]: void 0
        } : {};
        for (let prop of Object.keys(obj)) obj[prop] && "function" == typeof obj[prop].toJSON && !("toISOString" in obj[prop]) ? nobj[prop] = obj[prop].toJSON() : nobj[prop] = obj[prop];
        return nobj;
    }
    function stringifyObject(prefix, indent, obj) {
        var inlineKeys, complexKeys;
        inlineKeys = getInlineKeys(obj = toJSON(obj)), complexKeys = function(obj) {
            return Object.keys(obj).filter((key => !isInline(obj[key])));
        }(obj);
        var result = [], inlineIndent = indent || "";
        inlineKeys.forEach((key => {
            var type = tomlType(obj[key]);
            "undefined" !== type && "null" !== type && result.push(inlineIndent + stringifyKey(key) + " = " + stringifyAnyInline(obj[key], !0));
        })), result.length > 0 && result.push("");
        var complexIndent = prefix && inlineKeys.length > 0 ? indent + "  " : "";
        return complexKeys.forEach((key => {
            result.push(function(prefix, indent, key, value) {
                var valueType = tomlType(value);
                if ("array" === valueType) return function(prefix, indent, key, values) {
                    validateArray(values = toJSON(values));
                    var firstValueType = tomlType(values[0]);
                    if ("table" !== firstValueType) throw typeError(firstValueType);
                    var fullKey = prefix + stringifyKey(key), result = "";
                    return values.forEach((table => {
                        result.length > 0 && (result += "\n"), result += indent + "[[" + fullKey + "]]\n", 
                        result += stringifyObject(fullKey + ".", indent, table);
                    })), result;
                }(prefix, indent, key, value);
                if ("table" === valueType) return function(prefix, indent, key, value) {
                    var fullKey = prefix + stringifyKey(key), result = "";
                    getInlineKeys(value).length > 0 && (result += indent + "[" + fullKey + "]\n");
                    return result + stringifyObject(fullKey + ".", indent, value);
                }(prefix, indent, key, value);
                throw typeError(valueType);
            }(prefix, complexIndent, key, obj[key]));
        })), result.join("\n");
    }
    function isInline(value) {
        switch (tomlType(value)) {
          case "undefined":
          case "null":
          case "integer":
          case "nan":
          case "float":
          case "boolean":
          case "string":
          case "datetime":
            return !0;

          case "array":
            return 0 === value.length || "table" !== tomlType(value[0]);

          case "table":
            return 0 === Object.keys(value).length;

          default:
            return !1;
        }
    }
    function tomlType(value) {
        return void 0 === value ? "undefined" : null === value ? "null" : "bigint" == typeof value || Number.isInteger(value) && !Object.is(value, -0) ? "integer" : "number" == typeof value ? "float" : "boolean" == typeof value ? "boolean" : "string" == typeof value ? "string" : "toISOString" in value ? isNaN(value) ? "undefined" : "datetime" : Array.isArray(value) ? "array" : "table";
    }
    function stringifyKey(key) {
        var keyStr = String(key);
        return /^[-A-Za-z0-9_]+$/.test(keyStr) ? keyStr : stringifyBasicString(keyStr);
    }
    function stringifyBasicString(str) {
        return '"' + escapeString(str).replace(/"/g, '\\"') + '"';
    }
    function escapeString(str) {
        return str.replace(/\\/g, "\\\\").replace(/[\b]/g, "\\b").replace(/\t/g, "\\t").replace(/\n/g, "\\n").replace(/\f/g, "\\f").replace(/\r/g, "\\r").replace(/([\u0000-\u001f\u007f])/, (c => "\\u" + function(num, str) {
            for (;str.length < num; ) str = "0" + str;
            return str;
        }(4, c.codePointAt(0).toString(16))));
    }
    function stringifyAnyInline(value, multilineOk) {
        let type = tomlType(value);
        return "string" === type && (multilineOk && /\n/.test(value) ? type = "string-multiline" : !/[\b\t\n\f\r']/.test(value) && /"/.test(value) && (type = "string-literal")), 
        stringifyInline(value, type);
    }
    function stringifyInline(value, type) {
        switch (type || (type = tomlType(value)), type) {
          case "string-multiline":
            return function(str) {
                let escaped = str.split(/\n/).map((str => escapeString(str).replace(/"(?="")/g, '\\"'))).join("\n");
                return '"' === escaped.slice(-1) && (escaped += "\\\n"), '"""\n' + escaped + '"""';
            }(value);

          case "string":
            return stringifyBasicString(value);

          case "string-literal":
            return "'" + value + "'";

          case "integer":
            return stringifyInteger(value);

          case "float":
            return function(value) {
                if (value === 1 / 0) return "inf";
                if (value === -1 / 0) return "-inf";
                if (Object.is(value, NaN)) return "nan";
                if (Object.is(value, -0)) return "-0.0";
                var chunks = String(value).split("."), int = chunks[0], dec = chunks[1] || 0;
                return stringifyInteger(int) + "." + dec;
            }(value);

          case "boolean":
            return function(value) {
                return String(value);
            }(value);

          case "datetime":
            return function(value) {
                return value.toISOString();
            }(value);

          case "array":
            return function(values) {
                const type = validateArray(values = toJSON(values));
                var result = "[", stringified = values.map((_ => stringifyInline(_, type)));
                stringified.join(", ").length > 60 || /\n/.test(stringified) ? result += "\n  " + stringified.join(",\n  ") + "\n" : result += " " + stringified.join(", ") + (stringified.length > 0 ? " " : "");
                return result + "]";
            }(value.filter((_ => "null" !== tomlType(_) && "undefined" !== tomlType(_) && "nan" !== tomlType(_))));

          case "table":
            return function(value) {
                value = toJSON(value);
                var result = [];
                return Object.keys(value).forEach((key => {
                    result.push(stringifyKey(key) + " = " + stringifyAnyInline(value[key], !1));
                })), "{ " + result.join(", ") + (result.length > 0 ? " " : "") + "}";
            }(value);

          default:
            throw typeError(type);
        }
    }
    function stringifyInteger(value) {
        return String(value).replace(/\B(?=(\d{3})+(?!\d))/g, "_");
    }
    function validateArray(values) {
        const type = function(values) {
            var contentType = tomlType(values[0]);
            return values.every((_ => tomlType(_) === contentType)) ? contentType : values.every((_ => function(type) {
                return "float" === type || "integer" === type;
            }(tomlType(_)))) ? "float" : "mixed";
        }(values);
        if ("mixed" === type) throw new Error("Array values can't have mixed types");
        return type;
    }
    return hasRequiredStringify$1 = 1, stringify$2.exports = function(obj) {
        if (null === obj) throw typeError("null");
        if (void 0 === obj) throw typeError("undefined");
        if ("object" != typeof obj) throw typeError(typeof obj);
        "function" == typeof obj.toJSON && (obj = obj.toJSON());
        if (null == obj) return null;
        const type = tomlType(obj);
        if ("table" !== type) throw typeError(type);
        return stringifyObject("", "", obj);
    }, stringify$2.exports.value = stringifyInline, stringify$2.exports;
}

function requireToml() {
    return hasRequiredToml || (hasRequiredToml = 1, toml.parse = requireParse(), toml.stringify = requireStringify$1()), 
    toml;
}

var tomlExports = requireToml(), dist = {}, composer = {}, directives = {}, identity = {}, hasRequiredIdentity;

function requireIdentity() {
    if (hasRequiredIdentity) return identity;
    hasRequiredIdentity = 1;
    const ALIAS = Symbol.for("yaml.alias"), DOC = Symbol.for("yaml.document"), MAP = Symbol.for("yaml.map"), PAIR = Symbol.for("yaml.pair"), SCALAR = Symbol.for("yaml.scalar"), SEQ = Symbol.for("yaml.seq"), NODE_TYPE = Symbol.for("yaml.node.type"), isScalar = node => !!node && "object" == typeof node && node[NODE_TYPE] === SCALAR;
    function isCollection(node) {
        if (node && "object" == typeof node) switch (node[NODE_TYPE]) {
          case MAP:
          case SEQ:
            return !0;
        }
        return !1;
    }
    return identity.ALIAS = ALIAS, identity.DOC = DOC, identity.MAP = MAP, identity.NODE_TYPE = NODE_TYPE, 
    identity.PAIR = PAIR, identity.SCALAR = SCALAR, identity.SEQ = SEQ, identity.hasAnchor = node => (isScalar(node) || isCollection(node)) && !!node.anchor, 
    identity.isAlias = node => !!node && "object" == typeof node && node[NODE_TYPE] === ALIAS, 
    identity.isCollection = isCollection, identity.isDocument = node => !!node && "object" == typeof node && node[NODE_TYPE] === DOC, 
    identity.isMap = node => !!node && "object" == typeof node && node[NODE_TYPE] === MAP, 
    identity.isNode = function(node) {
        if (node && "object" == typeof node) switch (node[NODE_TYPE]) {
          case ALIAS:
          case MAP:
          case SCALAR:
          case SEQ:
            return !0;
        }
        return !1;
    }, identity.isPair = node => !!node && "object" == typeof node && node[NODE_TYPE] === PAIR, 
    identity.isScalar = isScalar, identity.isSeq = node => !!node && "object" == typeof node && node[NODE_TYPE] === SEQ, 
    identity;
}

var visit = {}, hasRequiredVisit, hasRequiredDirectives;

function requireVisit() {
    if (hasRequiredVisit) return visit;
    hasRequiredVisit = 1;
    var identity = requireIdentity();
    const BREAK = Symbol("break visit"), SKIP = Symbol("skip children"), REMOVE = Symbol("remove node");
    function visit$1(node, visitor) {
        const visitor_ = initVisitor(visitor);
        if (identity.isDocument(node)) {
            visit_(null, node.contents, visitor_, Object.freeze([ node ])) === REMOVE && (node.contents = null);
        } else visit_(null, node, visitor_, Object.freeze([]));
    }
    function visit_(key, node, visitor, path) {
        const ctrl = callVisitor(key, node, visitor, path);
        if (identity.isNode(ctrl) || identity.isPair(ctrl)) return replaceNode(key, path, ctrl), 
        visit_(key, ctrl, visitor, path);
        if ("symbol" != typeof ctrl) if (identity.isCollection(node)) {
            path = Object.freeze(path.concat(node));
            for (let i = 0; i < node.items.length; ++i) {
                const ci = visit_(i, node.items[i], visitor, path);
                if ("number" == typeof ci) i = ci - 1; else {
                    if (ci === BREAK) return BREAK;
                    ci === REMOVE && (node.items.splice(i, 1), i -= 1);
                }
            }
        } else if (identity.isPair(node)) {
            path = Object.freeze(path.concat(node));
            const ck = visit_("key", node.key, visitor, path);
            if (ck === BREAK) return BREAK;
            ck === REMOVE && (node.key = null);
            const cv = visit_("value", node.value, visitor, path);
            if (cv === BREAK) return BREAK;
            cv === REMOVE && (node.value = null);
        }
        return ctrl;
    }
    async function visitAsync(node, visitor) {
        const visitor_ = initVisitor(visitor);
        if (identity.isDocument(node)) {
            await visitAsync_(null, node.contents, visitor_, Object.freeze([ node ])) === REMOVE && (node.contents = null);
        } else await visitAsync_(null, node, visitor_, Object.freeze([]));
    }
    async function visitAsync_(key, node, visitor, path) {
        const ctrl = await callVisitor(key, node, visitor, path);
        if (identity.isNode(ctrl) || identity.isPair(ctrl)) return replaceNode(key, path, ctrl), 
        visitAsync_(key, ctrl, visitor, path);
        if ("symbol" != typeof ctrl) if (identity.isCollection(node)) {
            path = Object.freeze(path.concat(node));
            for (let i = 0; i < node.items.length; ++i) {
                const ci = await visitAsync_(i, node.items[i], visitor, path);
                if ("number" == typeof ci) i = ci - 1; else {
                    if (ci === BREAK) return BREAK;
                    ci === REMOVE && (node.items.splice(i, 1), i -= 1);
                }
            }
        } else if (identity.isPair(node)) {
            path = Object.freeze(path.concat(node));
            const ck = await visitAsync_("key", node.key, visitor, path);
            if (ck === BREAK) return BREAK;
            ck === REMOVE && (node.key = null);
            const cv = await visitAsync_("value", node.value, visitor, path);
            if (cv === BREAK) return BREAK;
            cv === REMOVE && (node.value = null);
        }
        return ctrl;
    }
    function initVisitor(visitor) {
        return "object" == typeof visitor && (visitor.Collection || visitor.Node || visitor.Value) ? Object.assign({
            Alias: visitor.Node,
            Map: visitor.Node,
            Scalar: visitor.Node,
            Seq: visitor.Node
        }, visitor.Value && {
            Map: visitor.Value,
            Scalar: visitor.Value,
            Seq: visitor.Value
        }, visitor.Collection && {
            Map: visitor.Collection,
            Seq: visitor.Collection
        }, visitor) : visitor;
    }
    function callVisitor(key, node, visitor, path) {
        return "function" == typeof visitor ? visitor(key, node, path) : identity.isMap(node) ? visitor.Map?.(key, node, path) : identity.isSeq(node) ? visitor.Seq?.(key, node, path) : identity.isPair(node) ? visitor.Pair?.(key, node, path) : identity.isScalar(node) ? visitor.Scalar?.(key, node, path) : identity.isAlias(node) ? visitor.Alias?.(key, node, path) : void 0;
    }
    function replaceNode(key, path, node) {
        const parent = path[path.length - 1];
        if (identity.isCollection(parent)) parent.items[key] = node; else if (identity.isPair(parent)) "key" === key ? parent.key = node : parent.value = node; else {
            if (!identity.isDocument(parent)) {
                const pt = identity.isAlias(parent) ? "alias" : "scalar";
                throw new Error(`Cannot replace node with ${pt} parent`);
            }
            parent.contents = node;
        }
    }
    return visit$1.BREAK = BREAK, visit$1.SKIP = SKIP, visit$1.REMOVE = REMOVE, visitAsync.BREAK = BREAK, 
    visitAsync.SKIP = SKIP, visitAsync.REMOVE = REMOVE, visit.visit = visit$1, visit.visitAsync = visitAsync, 
    visit;
}

function requireDirectives() {
    if (hasRequiredDirectives) return directives;
    hasRequiredDirectives = 1;
    var identity = requireIdentity(), visit = requireVisit();
    const escapeChars = {
        "!": "%21",
        ",": "%2C",
        "[": "%5B",
        "]": "%5D",
        "{": "%7B",
        "}": "%7D"
    };
    class Directives {
        constructor(yaml, tags) {
            this.docStart = null, this.docEnd = !1, this.yaml = Object.assign({}, Directives.defaultYaml, yaml), 
            this.tags = Object.assign({}, Directives.defaultTags, tags);
        }
        clone() {
            const copy = new Directives(this.yaml, this.tags);
            return copy.docStart = this.docStart, copy;
        }
        atDocument() {
            const res = new Directives(this.yaml, this.tags);
            switch (this.yaml.version) {
              case "1.1":
                this.atNextDocument = !0;
                break;

              case "1.2":
                this.atNextDocument = !1, this.yaml = {
                    explicit: Directives.defaultYaml.explicit,
                    version: "1.2"
                }, this.tags = Object.assign({}, Directives.defaultTags);
            }
            return res;
        }
        add(line, onError) {
            this.atNextDocument && (this.yaml = {
                explicit: Directives.defaultYaml.explicit,
                version: "1.1"
            }, this.tags = Object.assign({}, Directives.defaultTags), this.atNextDocument = !1);
            const parts = line.trim().split(/[ \t]+/), name = parts.shift();
            switch (name) {
              case "%TAG":
                {
                    if (2 !== parts.length && (onError(0, "%TAG directive should contain exactly two parts"), 
                    parts.length < 2)) return !1;
                    const [handle, prefix] = parts;
                    return this.tags[handle] = prefix, !0;
                }

              case "%YAML":
                {
                    if (this.yaml.explicit = !0, 1 !== parts.length) return onError(0, "%YAML directive should contain exactly one part"), 
                    !1;
                    const [version] = parts;
                    if ("1.1" === version || "1.2" === version) return this.yaml.version = version, 
                    !0;
                    return onError(6, `Unsupported YAML version ${version}`, /^\d+\.\d+$/.test(version)), 
                    !1;
                }

              default:
                return onError(0, `Unknown directive ${name}`, !0), !1;
            }
        }
        tagName(source, onError) {
            if ("!" === source) return "!";
            if ("!" !== source[0]) return onError(`Not a valid tag: ${source}`), null;
            if ("<" === source[1]) {
                const verbatim = source.slice(2, -1);
                return "!" === verbatim || "!!" === verbatim ? (onError(`Verbatim tags aren't resolved, so ${source} is invalid.`), 
                null) : (">" !== source[source.length - 1] && onError("Verbatim tags must end with a >"), 
                verbatim);
            }
            const [, handle, suffix] = source.match(/^(.*!)([^!]*)$/s);
            suffix || onError(`The ${source} tag has no suffix`);
            const prefix = this.tags[handle];
            if (prefix) try {
                return prefix + decodeURIComponent(suffix);
            } catch (error) {
                return onError(String(error)), null;
            }
            return "!" === handle ? source : (onError(`Could not resolve tag: ${source}`), null);
        }
        tagString(tag) {
            for (const [handle, prefix] of Object.entries(this.tags)) if (tag.startsWith(prefix)) return handle + tag.substring(prefix.length).replace(/[!,[\]{}]/g, (ch => escapeChars[ch]));
            return "!" === tag[0] ? tag : `!<${tag}>`;
        }
        toString(doc) {
            const lines = this.yaml.explicit ? [ `%YAML ${this.yaml.version || "1.2"}` ] : [], tagEntries = Object.entries(this.tags);
            let tagNames;
            if (doc && tagEntries.length > 0 && identity.isNode(doc.contents)) {
                const tags = {};
                visit.visit(doc.contents, ((_key, node) => {
                    identity.isNode(node) && node.tag && (tags[node.tag] = !0);
                })), tagNames = Object.keys(tags);
            } else tagNames = [];
            for (const [handle, prefix] of tagEntries) "!!" === handle && "tag:yaml.org,2002:" === prefix || doc && !tagNames.some((tn => tn.startsWith(prefix))) || lines.push(`%TAG ${handle} ${prefix}`);
            return lines.join("\n");
        }
    }
    return Directives.defaultYaml = {
        explicit: !1,
        version: "1.2"
    }, Directives.defaultTags = {
        "!!": "tag:yaml.org,2002:"
    }, directives.Directives = Directives, directives;
}

var Document = {}, Alias = {}, anchors = {}, hasRequiredAnchors;

function requireAnchors() {
    if (hasRequiredAnchors) return anchors;
    hasRequiredAnchors = 1;
    var identity = requireIdentity(), visit = requireVisit();
    function anchorNames(root) {
        const anchors = new Set;
        return visit.visit(root, {
            Value(_key, node) {
                node.anchor && anchors.add(node.anchor);
            }
        }), anchors;
    }
    function findNewAnchor(prefix, exclude) {
        for (let i = 1; ;++i) {
            const name = `${prefix}${i}`;
            if (!exclude.has(name)) return name;
        }
    }
    return anchors.anchorIsValid = function(anchor) {
        if (/[\x00-\x19\s,[\]{}]/.test(anchor)) {
            const sa = JSON.stringify(anchor);
            throw new Error(`Anchor must not contain whitespace or control characters: ${sa}`);
        }
        return !0;
    }, anchors.anchorNames = anchorNames, anchors.createNodeAnchors = function(doc, prefix) {
        const aliasObjects = [], sourceObjects = new Map;
        let prevAnchors = null;
        return {
            onAnchor: source => {
                aliasObjects.push(source), prevAnchors || (prevAnchors = anchorNames(doc));
                const anchor = findNewAnchor(prefix, prevAnchors);
                return prevAnchors.add(anchor), anchor;
            },
            setAnchors: () => {
                for (const source of aliasObjects) {
                    const ref = sourceObjects.get(source);
                    if ("object" != typeof ref || !ref.anchor || !identity.isScalar(ref.node) && !identity.isCollection(ref.node)) {
                        const error = new Error("Failed to resolve repeated object (this should not happen)");
                        throw error.source = source, error;
                    }
                    ref.node.anchor = ref.anchor;
                }
            },
            sourceObjects: sourceObjects
        };
    }, anchors.findNewAnchor = findNewAnchor, anchors;
}

var Node = {}, applyReviver = {}, hasRequiredApplyReviver;

function requireApplyReviver() {
    if (hasRequiredApplyReviver) return applyReviver;
    return hasRequiredApplyReviver = 1, applyReviver.applyReviver = function applyReviver$1(reviver, obj, key, val) {
        if (val && "object" == typeof val) if (Array.isArray(val)) for (let i = 0, len = val.length; i < len; ++i) {
            const v0 = val[i], v1 = applyReviver$1(reviver, val, String(i), v0);
            void 0 === v1 ? delete val[i] : v1 !== v0 && (val[i] = v1);
        } else if (val instanceof Map) for (const k of Array.from(val.keys())) {
            const v0 = val.get(k), v1 = applyReviver$1(reviver, val, k, v0);
            void 0 === v1 ? val.delete(k) : v1 !== v0 && val.set(k, v1);
        } else if (val instanceof Set) for (const v0 of Array.from(val)) {
            const v1 = applyReviver$1(reviver, val, v0, v0);
            void 0 === v1 ? val.delete(v0) : v1 !== v0 && (val.delete(v0), val.add(v1));
        } else for (const [k, v0] of Object.entries(val)) {
            const v1 = applyReviver$1(reviver, val, k, v0);
            void 0 === v1 ? delete val[k] : v1 !== v0 && (val[k] = v1);
        }
        return reviver.call(obj, key, val);
    }, applyReviver;
}

var toJS = {}, hasRequiredToJS, hasRequiredNode, hasRequiredAlias;

function requireToJS() {
    if (hasRequiredToJS) return toJS;
    hasRequiredToJS = 1;
    var identity = requireIdentity();
    return toJS.toJS = function toJS$1(value, arg, ctx) {
        if (Array.isArray(value)) return value.map(((v, i) => toJS$1(v, String(i), ctx)));
        if (value && "function" == typeof value.toJSON) {
            if (!ctx || !identity.hasAnchor(value)) return value.toJSON(arg, ctx);
            const data = {
                aliasCount: 0,
                count: 1,
                res: void 0
            };
            ctx.anchors.set(value, data), ctx.onCreate = res => {
                data.res = res, delete ctx.onCreate;
            };
            const res = value.toJSON(arg, ctx);
            return ctx.onCreate && ctx.onCreate(res), res;
        }
        return "bigint" != typeof value || ctx?.keep ? value : Number(value);
    }, toJS;
}

function requireNode() {
    if (hasRequiredNode) return Node;
    hasRequiredNode = 1;
    var applyReviver = requireApplyReviver(), identity = requireIdentity(), toJS = requireToJS();
    return Node.NodeBase = class {
        constructor(type) {
            Object.defineProperty(this, identity.NODE_TYPE, {
                value: type
            });
        }
        clone() {
            const copy = Object.create(Object.getPrototypeOf(this), Object.getOwnPropertyDescriptors(this));
            return this.range && (copy.range = this.range.slice()), copy;
        }
        toJS(doc, {mapAsMap: mapAsMap, maxAliasCount: maxAliasCount, onAnchor: onAnchor, reviver: reviver} = {}) {
            if (!identity.isDocument(doc)) throw new TypeError("A document argument is required");
            const ctx = {
                anchors: new Map,
                doc: doc,
                keep: !0,
                mapAsMap: !0 === mapAsMap,
                mapKeyWarned: !1,
                maxAliasCount: "number" == typeof maxAliasCount ? maxAliasCount : 100
            }, res = toJS.toJS(this, "", ctx);
            if ("function" == typeof onAnchor) for (const {count: count, res: res} of ctx.anchors.values()) onAnchor(res, count);
            return "function" == typeof reviver ? applyReviver.applyReviver(reviver, {
                "": res
            }, "", res) : res;
        }
    }, Node;
}

function requireAlias() {
    if (hasRequiredAlias) return Alias;
    hasRequiredAlias = 1;
    var anchors = requireAnchors(), visit = requireVisit(), identity = requireIdentity(), Node = requireNode(), toJS = requireToJS();
    let Alias$1 = class extends Node.NodeBase {
        constructor(source) {
            super(identity.ALIAS), this.source = source, Object.defineProperty(this, "tag", {
                set() {
                    throw new Error("Alias nodes cannot have tags");
                }
            });
        }
        resolve(doc) {
            let found;
            return visit.visit(doc, {
                Node: (_key, node) => {
                    if (node === this) return visit.visit.BREAK;
                    node.anchor === this.source && (found = node);
                }
            }), found;
        }
        toJSON(_arg, ctx) {
            if (!ctx) return {
                source: this.source
            };
            const {anchors: anchors, doc: doc, maxAliasCount: maxAliasCount} = ctx, source = this.resolve(doc);
            if (!source) {
                const msg = `Unresolved alias (the anchor must be set before the alias): ${this.source}`;
                throw new ReferenceError(msg);
            }
            let data = anchors.get(source);
            if (data || (toJS.toJS(source, null, ctx), data = anchors.get(source)), !data || void 0 === data.res) {
                throw new ReferenceError("This should not happen: Alias anchor was not resolved?");
            }
            if (maxAliasCount >= 0 && (data.count += 1, 0 === data.aliasCount && (data.aliasCount = getAliasCount(doc, source, anchors)), 
            data.count * data.aliasCount > maxAliasCount)) {
                throw new ReferenceError("Excessive alias count indicates a resource exhaustion attack");
            }
            return data.res;
        }
        toString(ctx, _onComment, _onChompKeep) {
            const src = `*${this.source}`;
            if (ctx) {
                if (anchors.anchorIsValid(this.source), ctx.options.verifyAliasOrder && !ctx.anchors.has(this.source)) {
                    const msg = `Unresolved alias (the anchor must be set before the alias): ${this.source}`;
                    throw new Error(msg);
                }
                if (ctx.implicitKey) return `${src} `;
            }
            return src;
        }
    };
    function getAliasCount(doc, node, anchors) {
        if (identity.isAlias(node)) {
            const source = node.resolve(doc), anchor = anchors && source && anchors.get(source);
            return anchor ? anchor.count * anchor.aliasCount : 0;
        }
        if (identity.isCollection(node)) {
            let count = 0;
            for (const item of node.items) {
                const c = getAliasCount(doc, item, anchors);
                c > count && (count = c);
            }
            return count;
        }
        if (identity.isPair(node)) {
            const kc = getAliasCount(doc, node.key, anchors), vc = getAliasCount(doc, node.value, anchors);
            return Math.max(kc, vc);
        }
        return 1;
    }
    return Alias.Alias = Alias$1, Alias;
}

var Collection = {}, createNode = {}, Scalar = {}, hasRequiredScalar, hasRequiredCreateNode, hasRequiredCollection;

function requireScalar() {
    if (hasRequiredScalar) return Scalar;
    hasRequiredScalar = 1;
    var identity = requireIdentity(), Node = requireNode(), toJS = requireToJS();
    let Scalar$1 = class extends Node.NodeBase {
        constructor(value) {
            super(identity.SCALAR), this.value = value;
        }
        toJSON(arg, ctx) {
            return ctx?.keep ? this.value : toJS.toJS(this.value, arg, ctx);
        }
        toString() {
            return String(this.value);
        }
    };
    return Scalar$1.BLOCK_FOLDED = "BLOCK_FOLDED", Scalar$1.BLOCK_LITERAL = "BLOCK_LITERAL", 
    Scalar$1.PLAIN = "PLAIN", Scalar$1.QUOTE_DOUBLE = "QUOTE_DOUBLE", Scalar$1.QUOTE_SINGLE = "QUOTE_SINGLE", 
    Scalar.Scalar = Scalar$1, Scalar.isScalarValue = value => !value || "function" != typeof value && "object" != typeof value, 
    Scalar;
}

function requireCreateNode() {
    if (hasRequiredCreateNode) return createNode;
    hasRequiredCreateNode = 1;
    var Alias = requireAlias(), identity = requireIdentity(), Scalar = requireScalar();
    return createNode.createNode = function(value, tagName, ctx) {
        if (identity.isDocument(value) && (value = value.contents), identity.isNode(value)) return value;
        if (identity.isPair(value)) {
            const map = ctx.schema[identity.MAP].createNode?.(ctx.schema, null, ctx);
            return map.items.push(value), map;
        }
        (value instanceof String || value instanceof Number || value instanceof Boolean || "undefined" != typeof BigInt && value instanceof BigInt) && (value = value.valueOf());
        const {aliasDuplicateObjects: aliasDuplicateObjects, onAnchor: onAnchor, onTagObj: onTagObj, schema: schema, sourceObjects: sourceObjects} = ctx;
        let ref;
        if (aliasDuplicateObjects && value && "object" == typeof value) {
            if (ref = sourceObjects.get(value), ref) return ref.anchor || (ref.anchor = onAnchor(value)), 
            new Alias.Alias(ref.anchor);
            ref = {
                anchor: null,
                node: null
            }, sourceObjects.set(value, ref);
        }
        tagName?.startsWith("!!") && (tagName = "tag:yaml.org,2002:" + tagName.slice(2));
        let tagObj = function(value, tagName, tags) {
            if (tagName) {
                const match = tags.filter((t => t.tag === tagName)), tagObj = match.find((t => !t.format)) ?? match[0];
                if (!tagObj) throw new Error(`Tag ${tagName} not found`);
                return tagObj;
            }
            return tags.find((t => t.identify?.(value) && !t.format));
        }(value, tagName, schema.tags);
        if (!tagObj) {
            if (value && "function" == typeof value.toJSON && (value = value.toJSON()), !value || "object" != typeof value) {
                const node = new Scalar.Scalar(value);
                return ref && (ref.node = node), node;
            }
            tagObj = value instanceof Map ? schema[identity.MAP] : Symbol.iterator in Object(value) ? schema[identity.SEQ] : schema[identity.MAP];
        }
        onTagObj && (onTagObj(tagObj), delete ctx.onTagObj);
        const node = tagObj?.createNode ? tagObj.createNode(ctx.schema, value, ctx) : "function" == typeof tagObj?.nodeClass?.from ? tagObj.nodeClass.from(ctx.schema, value, ctx) : new Scalar.Scalar(value);
        return tagName ? node.tag = tagName : tagObj.default || (node.tag = tagObj.tag), 
        ref && (ref.node = node), node;
    }, createNode;
}

function requireCollection() {
    if (hasRequiredCollection) return Collection;
    hasRequiredCollection = 1;
    var createNode = requireCreateNode(), identity = requireIdentity(), Node = requireNode();
    function collectionFromPath(schema, path, value) {
        let v = value;
        for (let i = path.length - 1; i >= 0; --i) {
            const k = path[i];
            if ("number" == typeof k && Number.isInteger(k) && k >= 0) {
                const a = [];
                a[k] = v, v = a;
            } else v = new Map([ [ k, v ] ]);
        }
        return createNode.createNode(v, void 0, {
            aliasDuplicateObjects: !1,
            keepUndefined: !1,
            onAnchor: () => {
                throw new Error("This should not happen, please report a bug.");
            },
            schema: schema,
            sourceObjects: new Map
        });
    }
    const isEmptyPath = path => null == path || "object" == typeof path && !!path[Symbol.iterator]().next().done;
    let Collection$1 = class extends Node.NodeBase {
        constructor(type, schema) {
            super(type), Object.defineProperty(this, "schema", {
                value: schema,
                configurable: !0,
                enumerable: !1,
                writable: !0
            });
        }
        clone(schema) {
            const copy = Object.create(Object.getPrototypeOf(this), Object.getOwnPropertyDescriptors(this));
            return schema && (copy.schema = schema), copy.items = copy.items.map((it => identity.isNode(it) || identity.isPair(it) ? it.clone(schema) : it)), 
            this.range && (copy.range = this.range.slice()), copy;
        }
        addIn(path, value) {
            if (isEmptyPath(path)) this.add(value); else {
                const [key, ...rest] = path, node = this.get(key, !0);
                if (identity.isCollection(node)) node.addIn(rest, value); else {
                    if (void 0 !== node || !this.schema) throw new Error(`Expected YAML collection at ${key}. Remaining path: ${rest}`);
                    this.set(key, collectionFromPath(this.schema, rest, value));
                }
            }
        }
        deleteIn(path) {
            const [key, ...rest] = path;
            if (0 === rest.length) return this.delete(key);
            const node = this.get(key, !0);
            if (identity.isCollection(node)) return node.deleteIn(rest);
            throw new Error(`Expected YAML collection at ${key}. Remaining path: ${rest}`);
        }
        getIn(path, keepScalar) {
            const [key, ...rest] = path, node = this.get(key, !0);
            return 0 === rest.length ? !keepScalar && identity.isScalar(node) ? node.value : node : identity.isCollection(node) ? node.getIn(rest, keepScalar) : void 0;
        }
        hasAllNullValues(allowScalar) {
            return this.items.every((node => {
                if (!identity.isPair(node)) return !1;
                const n = node.value;
                return null == n || allowScalar && identity.isScalar(n) && null == n.value && !n.commentBefore && !n.comment && !n.tag;
            }));
        }
        hasIn(path) {
            const [key, ...rest] = path;
            if (0 === rest.length) return this.has(key);
            const node = this.get(key, !0);
            return !!identity.isCollection(node) && node.hasIn(rest);
        }
        setIn(path, value) {
            const [key, ...rest] = path;
            if (0 === rest.length) this.set(key, value); else {
                const node = this.get(key, !0);
                if (identity.isCollection(node)) node.setIn(rest, value); else {
                    if (void 0 !== node || !this.schema) throw new Error(`Expected YAML collection at ${key}. Remaining path: ${rest}`);
                    this.set(key, collectionFromPath(this.schema, rest, value));
                }
            }
        }
    };
    return Collection.Collection = Collection$1, Collection.collectionFromPath = collectionFromPath, 
    Collection.isEmptyPath = isEmptyPath, Collection;
}

var Pair = {}, stringifyPair = {}, stringify$1 = {}, stringifyComment = {}, hasRequiredStringifyComment;

function requireStringifyComment() {
    if (hasRequiredStringifyComment) return stringifyComment;
    hasRequiredStringifyComment = 1;
    function indentComment(comment, indent) {
        return /^\n+$/.test(comment) ? comment.substring(1) : indent ? comment.replace(/^(?! *$)/gm, indent) : comment;
    }
    return stringifyComment.indentComment = indentComment, stringifyComment.lineComment = (str, indent, comment) => str.endsWith("\n") ? indentComment(comment, indent) : comment.includes("\n") ? "\n" + indentComment(comment, indent) : (str.endsWith(" ") ? "" : " ") + comment, 
    stringifyComment.stringifyComment = str => str.replace(/^(?!$)(?: $)?/gm, "#"), 
    stringifyComment;
}

var stringifyString = {}, foldFlowLines = {}, hasRequiredFoldFlowLines, hasRequiredStringifyString, hasRequiredStringify, hasRequiredStringifyPair;

function requireFoldFlowLines() {
    if (hasRequiredFoldFlowLines) return foldFlowLines;
    hasRequiredFoldFlowLines = 1;
    function consumeMoreIndentedLines(text, i, indent) {
        let end = i, start = i + 1, ch = text[start];
        for (;" " === ch || "\t" === ch; ) if (i < start + indent) ch = text[++i]; else {
            do {
                ch = text[++i];
            } while (ch && "\n" !== ch);
            end = i, start = i + 1, ch = text[start];
        }
        return end;
    }
    return foldFlowLines.FOLD_BLOCK = "block", foldFlowLines.FOLD_FLOW = "flow", foldFlowLines.FOLD_QUOTED = "quoted", 
    foldFlowLines.foldFlowLines = function(text, indent, mode = "flow", {indentAtStart: indentAtStart, lineWidth: lineWidth = 80, minContentWidth: minContentWidth = 20, onFold: onFold, onOverflow: onOverflow} = {}) {
        if (!lineWidth || lineWidth < 0) return text;
        lineWidth < minContentWidth && (minContentWidth = 0);
        const endStep = Math.max(1 + minContentWidth, 1 + lineWidth - indent.length);
        if (text.length <= endStep) return text;
        const folds = [], escapedFolds = {};
        let split, prev, end = lineWidth - indent.length;
        "number" == typeof indentAtStart && (indentAtStart > lineWidth - Math.max(2, minContentWidth) ? folds.push(0) : end = lineWidth - indentAtStart);
        let overflow = !1, i = -1, escStart = -1, escEnd = -1;
        "block" === mode && (i = consumeMoreIndentedLines(text, i, indent.length), -1 !== i && (end = i + endStep));
        for (let ch; ch = text[i += 1]; ) {
            if ("quoted" === mode && "\\" === ch) {
                switch (escStart = i, text[i + 1]) {
                  case "x":
                    i += 3;
                    break;

                  case "u":
                    i += 5;
                    break;

                  case "U":
                    i += 9;
                    break;

                  default:
                    i += 1;
                }
                escEnd = i;
            }
            if ("\n" === ch) "block" === mode && (i = consumeMoreIndentedLines(text, i, indent.length)), 
            end = i + indent.length + endStep, split = void 0; else {
                if (" " === ch && prev && " " !== prev && "\n" !== prev && "\t" !== prev) {
                    const next = text[i + 1];
                    next && " " !== next && "\n" !== next && "\t" !== next && (split = i);
                }
                if (i >= end) if (split) folds.push(split), end = split + endStep, split = void 0; else if ("quoted" === mode) {
                    for (;" " === prev || "\t" === prev; ) prev = ch, ch = text[i += 1], overflow = !0;
                    const j = i > escEnd + 1 ? i - 2 : escStart - 1;
                    if (escapedFolds[j]) return text;
                    folds.push(j), escapedFolds[j] = !0, end = j + endStep, split = void 0;
                } else overflow = !0;
            }
            prev = ch;
        }
        if (overflow && onOverflow && onOverflow(), 0 === folds.length) return text;
        onFold && onFold();
        let res = text.slice(0, folds[0]);
        for (let i = 0; i < folds.length; ++i) {
            const fold = folds[i], end = folds[i + 1] || text.length;
            0 === fold ? res = `\n${indent}${text.slice(0, end)}` : ("quoted" === mode && escapedFolds[fold] && (res += `${text[fold]}\\`), 
            res += `\n${indent}${text.slice(fold + 1, end)}`);
        }
        return res;
    }, foldFlowLines;
}

function requireStringifyString() {
    if (hasRequiredStringifyString) return stringifyString;
    hasRequiredStringifyString = 1;
    var Scalar = requireScalar(), foldFlowLines = requireFoldFlowLines();
    const getFoldOptions = (ctx, isBlock) => ({
        indentAtStart: isBlock ? ctx.indent.length : ctx.indentAtStart,
        lineWidth: ctx.options.lineWidth,
        minContentWidth: ctx.options.minContentWidth
    }), containsDocumentMarker = str => /^(%|---|\.\.\.)/m.test(str);
    function doubleQuotedString(value, ctx) {
        const json = JSON.stringify(value);
        if (ctx.options.doubleQuotedAsJSON) return json;
        const {implicitKey: implicitKey} = ctx, minMultiLineLength = ctx.options.doubleQuotedMinMultiLineLength, indent = ctx.indent || (containsDocumentMarker(value) ? "  " : "");
        let str = "", start = 0;
        for (let i = 0, ch = json[i]; ch; ch = json[++i]) if (" " === ch && "\\" === json[i + 1] && "n" === json[i + 2] && (str += json.slice(start, i) + "\\ ", 
        i += 1, start = i, ch = "\\"), "\\" === ch) switch (json[i + 1]) {
          case "u":
            {
                str += json.slice(start, i);
                const code = json.substr(i + 2, 4);
                switch (code) {
                  case "0000":
                    str += "\\0";
                    break;

                  case "0007":
                    str += "\\a";
                    break;

                  case "000b":
                    str += "\\v";
                    break;

                  case "001b":
                    str += "\\e";
                    break;

                  case "0085":
                    str += "\\N";
                    break;

                  case "00a0":
                    str += "\\_";
                    break;

                  case "2028":
                    str += "\\L";
                    break;

                  case "2029":
                    str += "\\P";
                    break;

                  default:
                    "00" === code.substr(0, 2) ? str += "\\x" + code.substr(2) : str += json.substr(i, 6);
                }
                i += 5, start = i + 1;
            }
            break;

          case "n":
            if (implicitKey || '"' === json[i + 2] || json.length < minMultiLineLength) i += 1; else {
                for (str += json.slice(start, i) + "\n\n"; "\\" === json[i + 2] && "n" === json[i + 3] && '"' !== json[i + 4]; ) str += "\n", 
                i += 2;
                str += indent, " " === json[i + 2] && (str += "\\"), i += 1, start = i + 1;
            }
            break;

          default:
            i += 1;
        }
        return str = start ? str + json.slice(start) : json, implicitKey ? str : foldFlowLines.foldFlowLines(str, indent, foldFlowLines.FOLD_QUOTED, getFoldOptions(ctx, !1));
    }
    function singleQuotedString(value, ctx) {
        if (!1 === ctx.options.singleQuote || ctx.implicitKey && value.includes("\n") || /[ \t]\n|\n[ \t]/.test(value)) return doubleQuotedString(value, ctx);
        const indent = ctx.indent || (containsDocumentMarker(value) ? "  " : ""), res = "'" + value.replace(/'/g, "''").replace(/\n+/g, `$&\n${indent}`) + "'";
        return ctx.implicitKey ? res : foldFlowLines.foldFlowLines(res, indent, foldFlowLines.FOLD_FLOW, getFoldOptions(ctx, !1));
    }
    function quotedString(value, ctx) {
        const {singleQuote: singleQuote} = ctx.options;
        let qs;
        if (!1 === singleQuote) qs = doubleQuotedString; else {
            const hasDouble = value.includes('"'), hasSingle = value.includes("'");
            qs = hasDouble && !hasSingle ? singleQuotedString : hasSingle && !hasDouble ? doubleQuotedString : singleQuote ? singleQuotedString : doubleQuotedString;
        }
        return qs(value, ctx);
    }
    let blockEndNewlines;
    try {
        blockEndNewlines = new RegExp("(^|(?<!\n))\n+(?!\n|$)", "g");
    } catch {
        blockEndNewlines = /\n+(?!\n|$)/g;
    }
    function blockString({comment: comment, type: type, value: value}, ctx, onComment, onChompKeep) {
        const {blockQuote: blockQuote, commentString: commentString, lineWidth: lineWidth} = ctx.options;
        if (!blockQuote || /\n[\t ]+$/.test(value) || /^\s*$/.test(value)) return quotedString(value, ctx);
        const indent = ctx.indent || (ctx.forceBlockIndent || containsDocumentMarker(value) ? "  " : ""), literal = "literal" === blockQuote || "folded" !== blockQuote && type !== Scalar.Scalar.BLOCK_FOLDED && (type === Scalar.Scalar.BLOCK_LITERAL || !function(str, lineWidth, indentLength) {
            if (!lineWidth || lineWidth < 0) return !1;
            const limit = lineWidth - indentLength, strLen = str.length;
            if (strLen <= limit) return !1;
            for (let i = 0, start = 0; i < strLen; ++i) if ("\n" === str[i]) {
                if (i - start > limit) return !0;
                if (start = i + 1, strLen - start <= limit) return !1;
            }
            return !0;
        }(value, lineWidth, indent.length));
        if (!value) return literal ? "|\n" : ">\n";
        let chomp, endStart;
        for (endStart = value.length; endStart > 0; --endStart) {
            const ch = value[endStart - 1];
            if ("\n" !== ch && "\t" !== ch && " " !== ch) break;
        }
        let end = value.substring(endStart);
        const endNlPos = end.indexOf("\n");
        -1 === endNlPos ? chomp = "-" : value === end || endNlPos !== end.length - 1 ? (chomp = "+", 
        onChompKeep && onChompKeep()) : chomp = "", end && (value = value.slice(0, -end.length), 
        "\n" === end[end.length - 1] && (end = end.slice(0, -1)), end = end.replace(blockEndNewlines, `$&${indent}`));
        let startEnd, startWithSpace = !1, startNlPos = -1;
        for (startEnd = 0; startEnd < value.length; ++startEnd) {
            const ch = value[startEnd];
            if (" " === ch) startWithSpace = !0; else {
                if ("\n" !== ch) break;
                startNlPos = startEnd;
            }
        }
        let start = value.substring(0, startNlPos < startEnd ? startNlPos + 1 : startEnd);
        start && (value = value.substring(start.length), start = start.replace(/\n+/g, `$&${indent}`));
        let header = (literal ? "|" : ">") + (startWithSpace ? indent ? "2" : "1" : "") + chomp;
        if (comment && (header += " " + commentString(comment.replace(/ ?[\r\n]+/g, " ")), 
        onComment && onComment()), literal) return `${header}\n${indent}${start}${value = value.replace(/\n+/g, `$&${indent}`)}${end}`;
        value = value.replace(/\n+/g, "\n$&").replace(/(?:^|\n)([\t ].*)(?:([\n\t ]*)\n(?![\n\t ]))?/g, "$1$2").replace(/\n+/g, `$&${indent}`);
        return `${header}\n${indent}${foldFlowLines.foldFlowLines(`${start}${value}${end}`, indent, foldFlowLines.FOLD_BLOCK, getFoldOptions(ctx, !0))}`;
    }
    return stringifyString.stringifyString = function(item, ctx, onComment, onChompKeep) {
        const {implicitKey: implicitKey, inFlow: inFlow} = ctx, ss = "string" == typeof item.value ? item : Object.assign({}, item, {
            value: String(item.value)
        });
        let {type: type} = item;
        type !== Scalar.Scalar.QUOTE_DOUBLE && /[\x00-\x08\x0b-\x1f\x7f-\x9f\u{D800}-\u{DFFF}]/u.test(ss.value) && (type = Scalar.Scalar.QUOTE_DOUBLE);
        const _stringify = _type => {
            switch (_type) {
              case Scalar.Scalar.BLOCK_FOLDED:
              case Scalar.Scalar.BLOCK_LITERAL:
                return implicitKey || inFlow ? quotedString(ss.value, ctx) : blockString(ss, ctx, onComment, onChompKeep);

              case Scalar.Scalar.QUOTE_DOUBLE:
                return doubleQuotedString(ss.value, ctx);

              case Scalar.Scalar.QUOTE_SINGLE:
                return singleQuotedString(ss.value, ctx);

              case Scalar.Scalar.PLAIN:
                return function(item, ctx, onComment, onChompKeep) {
                    const {type: type, value: value} = item, {actualString: actualString, implicitKey: implicitKey, indent: indent, indentStep: indentStep, inFlow: inFlow} = ctx;
                    if (implicitKey && value.includes("\n") || inFlow && /[[\]{},]/.test(value)) return quotedString(value, ctx);
                    if (!value || /^[\n\t ,[\]{}#&*!|>'"%@`]|^[?-]$|^[?-][ \t]|[\n:][ \t]|[ \t]\n|[\n\t ]#|[\n\t :]$/.test(value)) return implicitKey || inFlow || !value.includes("\n") ? quotedString(value, ctx) : blockString(item, ctx, onComment, onChompKeep);
                    if (!implicitKey && !inFlow && type !== Scalar.Scalar.PLAIN && value.includes("\n")) return blockString(item, ctx, onComment, onChompKeep);
                    if (containsDocumentMarker(value)) {
                        if ("" === indent) return ctx.forceBlockIndent = !0, blockString(item, ctx, onComment, onChompKeep);
                        if (implicitKey && indent === indentStep) return quotedString(value, ctx);
                    }
                    const str = value.replace(/\n+/g, `$&\n${indent}`);
                    if (actualString) {
                        const test = tag => tag.default && "tag:yaml.org,2002:str" !== tag.tag && tag.test?.test(str), {compat: compat, tags: tags} = ctx.doc.schema;
                        if (tags.some(test) || compat?.some(test)) return quotedString(value, ctx);
                    }
                    return implicitKey ? str : foldFlowLines.foldFlowLines(str, indent, foldFlowLines.FOLD_FLOW, getFoldOptions(ctx, !1));
                }(ss, ctx, onComment, onChompKeep);

              default:
                return null;
            }
        };
        let res = _stringify(type);
        if (null === res) {
            const {defaultKeyType: defaultKeyType, defaultStringType: defaultStringType} = ctx.options, t = implicitKey && defaultKeyType || defaultStringType;
            if (res = _stringify(t), null === res) throw new Error(`Unsupported default string type ${t}`);
        }
        return res;
    }, stringifyString;
}

function requireStringify() {
    if (hasRequiredStringify) return stringify$1;
    hasRequiredStringify = 1;
    var anchors = requireAnchors(), identity = requireIdentity(), stringifyComment = requireStringifyComment(), stringifyString = requireStringifyString();
    return stringify$1.createStringifyContext = function(doc, options) {
        const opt = Object.assign({
            blockQuote: !0,
            commentString: stringifyComment.stringifyComment,
            defaultKeyType: null,
            defaultStringType: "PLAIN",
            directives: null,
            doubleQuotedAsJSON: !1,
            doubleQuotedMinMultiLineLength: 40,
            falseStr: "false",
            flowCollectionPadding: !0,
            indentSeq: !0,
            lineWidth: 80,
            minContentWidth: 20,
            nullStr: "null",
            simpleKeys: !1,
            singleQuote: null,
            trueStr: "true",
            verifyAliasOrder: !0
        }, doc.schema.toStringOptions, options);
        let inFlow;
        switch (opt.collectionStyle) {
          case "block":
            inFlow = !1;
            break;

          case "flow":
            inFlow = !0;
            break;

          default:
            inFlow = null;
        }
        return {
            anchors: new Set,
            doc: doc,
            flowCollectionPadding: opt.flowCollectionPadding ? " " : "",
            indent: "",
            indentStep: "number" == typeof opt.indent ? " ".repeat(opt.indent) : "  ",
            inFlow: inFlow,
            options: opt
        };
    }, stringify$1.stringify = function(item, ctx, onComment, onChompKeep) {
        if (identity.isPair(item)) return item.toString(ctx, onComment, onChompKeep);
        if (identity.isAlias(item)) {
            if (ctx.doc.directives) return item.toString(ctx);
            if (ctx.resolvedAliases?.has(item)) throw new TypeError("Cannot stringify circular structure without alias nodes");
            ctx.resolvedAliases ? ctx.resolvedAliases.add(item) : ctx.resolvedAliases = new Set([ item ]), 
            item = item.resolve(ctx.doc);
        }
        let tagObj;
        const node = identity.isNode(item) ? item : ctx.doc.createNode(item, {
            onTagObj: o => tagObj = o
        });
        tagObj || (tagObj = function(tags, item) {
            if (item.tag) {
                const match = tags.filter((t => t.tag === item.tag));
                if (match.length > 0) return match.find((t => t.format === item.format)) ?? match[0];
            }
            let tagObj, obj;
            if (identity.isScalar(item)) {
                obj = item.value;
                let match = tags.filter((t => t.identify?.(obj)));
                if (match.length > 1) {
                    const testMatch = match.filter((t => t.test));
                    testMatch.length > 0 && (match = testMatch);
                }
                tagObj = match.find((t => t.format === item.format)) ?? match.find((t => !t.format));
            } else obj = item, tagObj = tags.find((t => t.nodeClass && obj instanceof t.nodeClass));
            if (!tagObj) throw new Error(`Tag not resolved for ${obj?.constructor?.name ?? typeof obj} value`);
            return tagObj;
        }(ctx.doc.schema.tags, node));
        const props = function(node, tagObj, {anchors: anchors$1, doc: doc}) {
            if (!doc.directives) return "";
            const props = [], anchor = (identity.isScalar(node) || identity.isCollection(node)) && node.anchor;
            anchor && anchors.anchorIsValid(anchor) && (anchors$1.add(anchor), props.push(`&${anchor}`));
            const tag = node.tag ? node.tag : tagObj.default ? null : tagObj.tag;
            return tag && props.push(doc.directives.tagString(tag)), props.join(" ");
        }(node, tagObj, ctx);
        props.length > 0 && (ctx.indentAtStart = (ctx.indentAtStart ?? 0) + props.length + 1);
        const str = "function" == typeof tagObj.stringify ? tagObj.stringify(node, ctx, onComment, onChompKeep) : identity.isScalar(node) ? stringifyString.stringifyString(node, ctx, onComment, onChompKeep) : node.toString(ctx, onComment, onChompKeep);
        return props ? identity.isScalar(node) || "{" === str[0] || "[" === str[0] ? `${props} ${str}` : `${props}\n${ctx.indent}${str}` : str;
    }, stringify$1;
}

function requireStringifyPair() {
    if (hasRequiredStringifyPair) return stringifyPair;
    hasRequiredStringifyPair = 1;
    var identity = requireIdentity(), Scalar = requireScalar(), stringify = requireStringify(), stringifyComment = requireStringifyComment();
    return stringifyPair.stringifyPair = function({key: key, value: value}, ctx, onComment, onChompKeep) {
        const {allNullValues: allNullValues, doc: doc, indent: indent, indentStep: indentStep, options: {commentString: commentString, indentSeq: indentSeq, simpleKeys: simpleKeys}} = ctx;
        let keyComment = identity.isNode(key) && key.comment || null;
        if (simpleKeys) {
            if (keyComment) throw new Error("With simple keys, key nodes cannot have comments");
            if (identity.isCollection(key) || !identity.isNode(key) && "object" == typeof key) {
                throw new Error("With simple keys, collection cannot be used as a key value");
            }
        }
        let explicitKey = !simpleKeys && (!key || keyComment && null == value && !ctx.inFlow || identity.isCollection(key) || (identity.isScalar(key) ? key.type === Scalar.Scalar.BLOCK_FOLDED || key.type === Scalar.Scalar.BLOCK_LITERAL : "object" == typeof key));
        ctx = Object.assign({}, ctx, {
            allNullValues: !1,
            implicitKey: !explicitKey && (simpleKeys || !allNullValues),
            indent: indent + indentStep
        });
        let vsb, vcb, valueComment, keyCommentDone = !1, chompKeep = !1, str = stringify.stringify(key, ctx, (() => keyCommentDone = !0), (() => chompKeep = !0));
        if (!explicitKey && !ctx.inFlow && str.length > 1024) {
            if (simpleKeys) throw new Error("With simple keys, single line scalar must not span more than 1024 characters");
            explicitKey = !0;
        }
        if (ctx.inFlow) {
            if (allNullValues || null == value) return keyCommentDone && onComment && onComment(), 
            "" === str ? "?" : explicitKey ? `? ${str}` : str;
        } else if (allNullValues && !simpleKeys || null == value && explicitKey) return str = `? ${str}`, 
        keyComment && !keyCommentDone ? str += stringifyComment.lineComment(str, ctx.indent, commentString(keyComment)) : chompKeep && onChompKeep && onChompKeep(), 
        str;
        keyCommentDone && (keyComment = null), explicitKey ? (keyComment && (str += stringifyComment.lineComment(str, ctx.indent, commentString(keyComment))), 
        str = `? ${str}\n${indent}:`) : (str = `${str}:`, keyComment && (str += stringifyComment.lineComment(str, ctx.indent, commentString(keyComment)))), 
        identity.isNode(value) ? (vsb = !!value.spaceBefore, vcb = value.commentBefore, 
        valueComment = value.comment) : (vsb = !1, vcb = null, valueComment = null, value && "object" == typeof value && (value = doc.createNode(value))), 
        ctx.implicitKey = !1, explicitKey || keyComment || !identity.isScalar(value) || (ctx.indentAtStart = str.length + 1), 
        chompKeep = !1, indentSeq || !(indentStep.length >= 2) || ctx.inFlow || explicitKey || !identity.isSeq(value) || value.flow || value.tag || value.anchor || (ctx.indent = ctx.indent.substring(2));
        let valueCommentDone = !1;
        const valueStr = stringify.stringify(value, ctx, (() => valueCommentDone = !0), (() => chompKeep = !0));
        let ws = " ";
        if (keyComment || vsb || vcb) {
            if (ws = vsb ? "\n" : "", vcb) {
                const cs = commentString(vcb);
                ws += `\n${stringifyComment.indentComment(cs, ctx.indent)}`;
            }
            "" !== valueStr || ctx.inFlow ? ws += `\n${ctx.indent}` : "\n" === ws && (ws = "\n\n");
        } else if (!explicitKey && identity.isCollection(value)) {
            const vs0 = valueStr[0], nl0 = valueStr.indexOf("\n"), hasNewline = -1 !== nl0, flow = ctx.inFlow ?? value.flow ?? 0 === value.items.length;
            if (hasNewline || !flow) {
                let hasPropsLine = !1;
                if (hasNewline && ("&" === vs0 || "!" === vs0)) {
                    let sp0 = valueStr.indexOf(" ");
                    "&" === vs0 && -1 !== sp0 && sp0 < nl0 && "!" === valueStr[sp0 + 1] && (sp0 = valueStr.indexOf(" ", sp0 + 1)), 
                    (-1 === sp0 || nl0 < sp0) && (hasPropsLine = !0);
                }
                hasPropsLine || (ws = `\n${ctx.indent}`);
            }
        } else "" !== valueStr && "\n" !== valueStr[0] || (ws = "");
        return str += ws + valueStr, ctx.inFlow ? valueCommentDone && onComment && onComment() : valueComment && !valueCommentDone ? str += stringifyComment.lineComment(str, ctx.indent, commentString(valueComment)) : chompKeep && onChompKeep && onChompKeep(), 
        str;
    }, stringifyPair;
}

var addPairToJSMap = {}, log = {}, hasRequiredLog;

function requireLog() {
    if (hasRequiredLog) return log;
    return hasRequiredLog = 1, log.debug = function(logLevel, ...messages) {
        "debug" === logLevel && console.log(...messages);
    }, log.warn = function(logLevel, warning) {
        "debug" !== logLevel && "warn" !== logLevel || "undefined" != typeof process && process.emitWarning && process.emitWarning(warning);
    }, log;
}

var merge = {}, hasRequiredMerge, hasRequiredAddPairToJSMap, hasRequiredPair;

function requireMerge() {
    if (hasRequiredMerge) return merge;
    hasRequiredMerge = 1;
    var identity = requireIdentity(), Scalar = requireScalar();
    const merge$1 = {
        identify: value => "<<" === value || "symbol" == typeof value && "<<" === value.description,
        default: "key",
        tag: "tag:yaml.org,2002:merge",
        test: /^<<$/,
        resolve: () => Object.assign(new Scalar.Scalar(Symbol("<<")), {
            addToJSMap: addMergeToJSMap
        }),
        stringify: () => "<<"
    };
    function addMergeToJSMap(ctx, map, value) {
        if (value = ctx && identity.isAlias(value) ? value.resolve(ctx.doc) : value, identity.isSeq(value)) for (const it of value.items) mergeValue(ctx, map, it); else if (Array.isArray(value)) for (const it of value) mergeValue(ctx, map, it); else mergeValue(ctx, map, value);
    }
    function mergeValue(ctx, map, value) {
        const source = ctx && identity.isAlias(value) ? value.resolve(ctx.doc) : value;
        if (!identity.isMap(source)) throw new Error("Merge sources must be maps or map aliases");
        const srcMap = source.toJSON(null, ctx, Map);
        for (const [key, value] of srcMap) map instanceof Map ? map.has(key) || map.set(key, value) : map instanceof Set ? map.add(key) : Object.prototype.hasOwnProperty.call(map, key) || Object.defineProperty(map, key, {
            value: value,
            writable: !0,
            enumerable: !0,
            configurable: !0
        });
        return map;
    }
    return merge.addMergeToJSMap = addMergeToJSMap, merge.isMergeKey = (ctx, key) => (merge$1.identify(key) || identity.isScalar(key) && (!key.type || key.type === Scalar.Scalar.PLAIN) && merge$1.identify(key.value)) && ctx?.doc.schema.tags.some((tag => tag.tag === merge$1.tag && tag.default)), 
    merge.merge = merge$1, merge;
}

function requireAddPairToJSMap() {
    if (hasRequiredAddPairToJSMap) return addPairToJSMap;
    hasRequiredAddPairToJSMap = 1;
    var log = requireLog(), merge = requireMerge(), stringify = requireStringify(), identity = requireIdentity(), toJS = requireToJS();
    return addPairToJSMap.addPairToJSMap = function(ctx, map, {key: key, value: value}) {
        if (identity.isNode(key) && key.addToJSMap) key.addToJSMap(ctx, map, value); else if (merge.isMergeKey(ctx, key)) merge.addMergeToJSMap(ctx, map, value); else {
            const jsKey = toJS.toJS(key, "", ctx);
            if (map instanceof Map) map.set(jsKey, toJS.toJS(value, jsKey, ctx)); else if (map instanceof Set) map.add(jsKey); else {
                const stringKey = function(key, jsKey, ctx) {
                    if (null === jsKey) return "";
                    if ("object" != typeof jsKey) return String(jsKey);
                    if (identity.isNode(key) && ctx?.doc) {
                        const strCtx = stringify.createStringifyContext(ctx.doc, {});
                        strCtx.anchors = new Set;
                        for (const node of ctx.anchors.keys()) strCtx.anchors.add(node.anchor);
                        strCtx.inFlow = !0, strCtx.inStringifyKey = !0;
                        const strKey = key.toString(strCtx);
                        if (!ctx.mapKeyWarned) {
                            let jsonStr = JSON.stringify(strKey);
                            jsonStr.length > 40 && (jsonStr = jsonStr.substring(0, 36) + '..."'), log.warn(ctx.doc.options.logLevel, `Keys with collection values will be stringified due to JS Object restrictions: ${jsonStr}. Set mapAsMap: true to use object keys.`), 
                            ctx.mapKeyWarned = !0;
                        }
                        return strKey;
                    }
                    return JSON.stringify(jsKey);
                }(key, jsKey, ctx), jsValue = toJS.toJS(value, stringKey, ctx);
                stringKey in map ? Object.defineProperty(map, stringKey, {
                    value: jsValue,
                    writable: !0,
                    enumerable: !0,
                    configurable: !0
                }) : map[stringKey] = jsValue;
            }
        }
        return map;
    }, addPairToJSMap;
}

function requirePair() {
    if (hasRequiredPair) return Pair;
    hasRequiredPair = 1;
    var createNode = requireCreateNode(), stringifyPair = requireStringifyPair(), addPairToJSMap = requireAddPairToJSMap(), identity = requireIdentity();
    let Pair$1 = class Pair {
        constructor(key, value = null) {
            Object.defineProperty(this, identity.NODE_TYPE, {
                value: identity.PAIR
            }), this.key = key, this.value = value;
        }
        clone(schema) {
            let {key: key, value: value} = this;
            return identity.isNode(key) && (key = key.clone(schema)), identity.isNode(value) && (value = value.clone(schema)), 
            new Pair(key, value);
        }
        toJSON(_, ctx) {
            const pair = ctx?.mapAsMap ? new Map : {};
            return addPairToJSMap.addPairToJSMap(ctx, pair, this);
        }
        toString(ctx, onComment, onChompKeep) {
            return ctx?.doc ? stringifyPair.stringifyPair(this, ctx, onComment, onChompKeep) : JSON.stringify(this);
        }
    };
    return Pair.Pair = Pair$1, Pair.createPair = function(key, value, ctx) {
        const k = createNode.createNode(key, void 0, ctx), v = createNode.createNode(value, void 0, ctx);
        return new Pair$1(k, v);
    }, Pair;
}

var Schema = {}, map = {}, YAMLMap = {}, stringifyCollection = {}, hasRequiredStringifyCollection, hasRequiredYAMLMap, hasRequiredMap;

function requireStringifyCollection() {
    if (hasRequiredStringifyCollection) return stringifyCollection;
    hasRequiredStringifyCollection = 1;
    var identity = requireIdentity(), stringify = requireStringify(), stringifyComment = requireStringifyComment();
    function stringifyBlockCollection({comment: comment, items: items}, ctx, {blockItemPrefix: blockItemPrefix, flowChars: flowChars, itemIndent: itemIndent, onChompKeep: onChompKeep, onComment: onComment}) {
        const {indent: indent, options: {commentString: commentString}} = ctx, itemCtx = Object.assign({}, ctx, {
            indent: itemIndent,
            type: null
        });
        let chompKeep = !1;
        const lines = [];
        for (let i = 0; i < items.length; ++i) {
            const item = items[i];
            let comment = null;
            if (identity.isNode(item)) !chompKeep && item.spaceBefore && lines.push(""), addCommentBefore(ctx, lines, item.commentBefore, chompKeep), 
            item.comment && (comment = item.comment); else if (identity.isPair(item)) {
                const ik = identity.isNode(item.key) ? item.key : null;
                ik && (!chompKeep && ik.spaceBefore && lines.push(""), addCommentBefore(ctx, lines, ik.commentBefore, chompKeep));
            }
            chompKeep = !1;
            let str = stringify.stringify(item, itemCtx, (() => comment = null), (() => chompKeep = !0));
            comment && (str += stringifyComment.lineComment(str, itemIndent, commentString(comment))), 
            chompKeep && comment && (chompKeep = !1), lines.push(blockItemPrefix + str);
        }
        let str;
        if (0 === lines.length) str = flowChars.start + flowChars.end; else {
            str = lines[0];
            for (let i = 1; i < lines.length; ++i) {
                const line = lines[i];
                str += line ? `\n${indent}${line}` : "\n";
            }
        }
        return comment ? (str += "\n" + stringifyComment.indentComment(commentString(comment), indent), 
        onComment && onComment()) : chompKeep && onChompKeep && onChompKeep(), str;
    }
    function stringifyFlowCollection({items: items}, ctx, {flowChars: flowChars, itemIndent: itemIndent}) {
        const {indent: indent, indentStep: indentStep, flowCollectionPadding: fcPadding, options: {commentString: commentString}} = ctx;
        itemIndent += indentStep;
        const itemCtx = Object.assign({}, ctx, {
            indent: itemIndent,
            inFlow: !0,
            type: null
        });
        let reqNewline = !1, linesAtValue = 0;
        const lines = [];
        for (let i = 0; i < items.length; ++i) {
            const item = items[i];
            let comment = null;
            if (identity.isNode(item)) item.spaceBefore && lines.push(""), addCommentBefore(ctx, lines, item.commentBefore, !1), 
            item.comment && (comment = item.comment); else if (identity.isPair(item)) {
                const ik = identity.isNode(item.key) ? item.key : null;
                ik && (ik.spaceBefore && lines.push(""), addCommentBefore(ctx, lines, ik.commentBefore, !1), 
                ik.comment && (reqNewline = !0));
                const iv = identity.isNode(item.value) ? item.value : null;
                iv ? (iv.comment && (comment = iv.comment), iv.commentBefore && (reqNewline = !0)) : null == item.value && ik?.comment && (comment = ik.comment);
            }
            comment && (reqNewline = !0);
            let str = stringify.stringify(item, itemCtx, (() => comment = null));
            i < items.length - 1 && (str += ","), comment && (str += stringifyComment.lineComment(str, itemIndent, commentString(comment))), 
            !reqNewline && (lines.length > linesAtValue || str.includes("\n")) && (reqNewline = !0), 
            lines.push(str), linesAtValue = lines.length;
        }
        const {start: start, end: end} = flowChars;
        if (0 === lines.length) return start + end;
        if (!reqNewline) {
            const len = lines.reduce(((sum, line) => sum + line.length + 2), 2);
            reqNewline = ctx.options.lineWidth > 0 && len > ctx.options.lineWidth;
        }
        if (reqNewline) {
            let str = start;
            for (const line of lines) str += line ? `\n${indentStep}${indent}${line}` : "\n";
            return `${str}\n${indent}${end}`;
        }
        return `${start}${fcPadding}${lines.join(" ")}${fcPadding}${end}`;
    }
    function addCommentBefore({indent: indent, options: {commentString: commentString}}, lines, comment, chompKeep) {
        if (comment && chompKeep && (comment = comment.replace(/^\n+/, "")), comment) {
            const ic = stringifyComment.indentComment(commentString(comment), indent);
            lines.push(ic.trimStart());
        }
    }
    return stringifyCollection.stringifyCollection = function(collection, ctx, options) {
        return (ctx.inFlow ?? collection.flow ? stringifyFlowCollection : stringifyBlockCollection)(collection, ctx, options);
    }, stringifyCollection;
}

function requireYAMLMap() {
    if (hasRequiredYAMLMap) return YAMLMap;
    hasRequiredYAMLMap = 1;
    var stringifyCollection = requireStringifyCollection(), addPairToJSMap = requireAddPairToJSMap(), Collection = requireCollection(), identity = requireIdentity(), Pair = requirePair(), Scalar = requireScalar();
    function findPair(items, key) {
        const k = identity.isScalar(key) ? key.value : key;
        for (const it of items) if (identity.isPair(it)) {
            if (it.key === key || it.key === k) return it;
            if (identity.isScalar(it.key) && it.key.value === k) return it;
        }
    }
    let YAMLMap$1 = class extends Collection.Collection {
        static get tagName() {
            return "tag:yaml.org,2002:map";
        }
        constructor(schema) {
            super(identity.MAP, schema), this.items = [];
        }
        static from(schema, obj, ctx) {
            const {keepUndefined: keepUndefined, replacer: replacer} = ctx, map = new this(schema), add = (key, value) => {
                if ("function" == typeof replacer) value = replacer.call(obj, key, value); else if (Array.isArray(replacer) && !replacer.includes(key)) return;
                (void 0 !== value || keepUndefined) && map.items.push(Pair.createPair(key, value, ctx));
            };
            if (obj instanceof Map) for (const [key, value] of obj) add(key, value); else if (obj && "object" == typeof obj) for (const key of Object.keys(obj)) add(key, obj[key]);
            return "function" == typeof schema.sortMapEntries && map.items.sort(schema.sortMapEntries), 
            map;
        }
        add(pair, overwrite) {
            let _pair;
            _pair = identity.isPair(pair) ? pair : pair && "object" == typeof pair && "key" in pair ? new Pair.Pair(pair.key, pair.value) : new Pair.Pair(pair, pair?.value);
            const prev = findPair(this.items, _pair.key), sortEntries = this.schema?.sortMapEntries;
            if (prev) {
                if (!overwrite) throw new Error(`Key ${_pair.key} already set`);
                identity.isScalar(prev.value) && Scalar.isScalarValue(_pair.value) ? prev.value.value = _pair.value : prev.value = _pair.value;
            } else if (sortEntries) {
                const i = this.items.findIndex((item => sortEntries(_pair, item) < 0));
                -1 === i ? this.items.push(_pair) : this.items.splice(i, 0, _pair);
            } else this.items.push(_pair);
        }
        delete(key) {
            const it = findPair(this.items, key);
            if (!it) return !1;
            return this.items.splice(this.items.indexOf(it), 1).length > 0;
        }
        get(key, keepScalar) {
            const it = findPair(this.items, key), node = it?.value;
            return (!keepScalar && identity.isScalar(node) ? node.value : node) ?? void 0;
        }
        has(key) {
            return !!findPair(this.items, key);
        }
        set(key, value) {
            this.add(new Pair.Pair(key, value), !0);
        }
        toJSON(_, ctx, Type) {
            const map = Type ? new Type : ctx?.mapAsMap ? new Map : {};
            ctx?.onCreate && ctx.onCreate(map);
            for (const item of this.items) addPairToJSMap.addPairToJSMap(ctx, map, item);
            return map;
        }
        toString(ctx, onComment, onChompKeep) {
            if (!ctx) return JSON.stringify(this);
            for (const item of this.items) if (!identity.isPair(item)) throw new Error(`Map items must all be pairs; found ${JSON.stringify(item)} instead`);
            return !ctx.allNullValues && this.hasAllNullValues(!1) && (ctx = Object.assign({}, ctx, {
                allNullValues: !0
            })), stringifyCollection.stringifyCollection(this, ctx, {
                blockItemPrefix: "",
                flowChars: {
                    start: "{",
                    end: "}"
                },
                itemIndent: ctx.indent || "",
                onChompKeep: onChompKeep,
                onComment: onComment
            });
        }
    };
    return YAMLMap.YAMLMap = YAMLMap$1, YAMLMap.findPair = findPair, YAMLMap;
}

function requireMap() {
    if (hasRequiredMap) return map;
    hasRequiredMap = 1;
    var identity = requireIdentity(), YAMLMap = requireYAMLMap();
    const map$1 = {
        collection: "map",
        default: !0,
        nodeClass: YAMLMap.YAMLMap,
        tag: "tag:yaml.org,2002:map",
        resolve: (map, onError) => (identity.isMap(map) || onError("Expected a mapping for this tag"), 
        map),
        createNode: (schema, obj, ctx) => YAMLMap.YAMLMap.from(schema, obj, ctx)
    };
    return map.map = map$1, map;
}

var seq = {}, YAMLSeq = {}, hasRequiredYAMLSeq, hasRequiredSeq;

function requireYAMLSeq() {
    if (hasRequiredYAMLSeq) return YAMLSeq;
    hasRequiredYAMLSeq = 1;
    var createNode = requireCreateNode(), stringifyCollection = requireStringifyCollection(), Collection = requireCollection(), identity = requireIdentity(), Scalar = requireScalar(), toJS = requireToJS();
    let YAMLSeq$1 = class extends Collection.Collection {
        static get tagName() {
            return "tag:yaml.org,2002:seq";
        }
        constructor(schema) {
            super(identity.SEQ, schema), this.items = [];
        }
        add(value) {
            this.items.push(value);
        }
        delete(key) {
            const idx = asItemIndex(key);
            if ("number" != typeof idx) return !1;
            return this.items.splice(idx, 1).length > 0;
        }
        get(key, keepScalar) {
            const idx = asItemIndex(key);
            if ("number" != typeof idx) return;
            const it = this.items[idx];
            return !keepScalar && identity.isScalar(it) ? it.value : it;
        }
        has(key) {
            const idx = asItemIndex(key);
            return "number" == typeof idx && idx < this.items.length;
        }
        set(key, value) {
            const idx = asItemIndex(key);
            if ("number" != typeof idx) throw new Error(`Expected a valid index, not ${key}.`);
            const prev = this.items[idx];
            identity.isScalar(prev) && Scalar.isScalarValue(value) ? prev.value = value : this.items[idx] = value;
        }
        toJSON(_, ctx) {
            const seq = [];
            ctx?.onCreate && ctx.onCreate(seq);
            let i = 0;
            for (const item of this.items) seq.push(toJS.toJS(item, String(i++), ctx));
            return seq;
        }
        toString(ctx, onComment, onChompKeep) {
            return ctx ? stringifyCollection.stringifyCollection(this, ctx, {
                blockItemPrefix: "- ",
                flowChars: {
                    start: "[",
                    end: "]"
                },
                itemIndent: (ctx.indent || "") + "  ",
                onChompKeep: onChompKeep,
                onComment: onComment
            }) : JSON.stringify(this);
        }
        static from(schema, obj, ctx) {
            const {replacer: replacer} = ctx, seq = new this(schema);
            if (obj && Symbol.iterator in Object(obj)) {
                let i = 0;
                for (let it of obj) {
                    if ("function" == typeof replacer) {
                        const key = obj instanceof Set ? it : String(i++);
                        it = replacer.call(obj, key, it);
                    }
                    seq.items.push(createNode.createNode(it, void 0, ctx));
                }
            }
            return seq;
        }
    };
    function asItemIndex(key) {
        let idx = identity.isScalar(key) ? key.value : key;
        return idx && "string" == typeof idx && (idx = Number(idx)), "number" == typeof idx && Number.isInteger(idx) && idx >= 0 ? idx : null;
    }
    return YAMLSeq.YAMLSeq = YAMLSeq$1, YAMLSeq;
}

function requireSeq() {
    if (hasRequiredSeq) return seq;
    hasRequiredSeq = 1;
    var identity = requireIdentity(), YAMLSeq = requireYAMLSeq();
    const seq$1 = {
        collection: "seq",
        default: !0,
        nodeClass: YAMLSeq.YAMLSeq,
        tag: "tag:yaml.org,2002:seq",
        resolve: (seq, onError) => (identity.isSeq(seq) || onError("Expected a sequence for this tag"), 
        seq),
        createNode: (schema, obj, ctx) => YAMLSeq.YAMLSeq.from(schema, obj, ctx)
    };
    return seq.seq = seq$1, seq;
}

var string = {}, hasRequiredString;

function requireString() {
    if (hasRequiredString) return string;
    hasRequiredString = 1;
    var stringifyString = requireStringifyString();
    const string$1 = {
        identify: value => "string" == typeof value,
        default: !0,
        tag: "tag:yaml.org,2002:str",
        resolve: str => str,
        stringify: (item, ctx, onComment, onChompKeep) => (ctx = Object.assign({
            actualString: !0
        }, ctx), stringifyString.stringifyString(item, ctx, onComment, onChompKeep))
    };
    return string.string = string$1, string;
}

var tags = {}, _null = {}, hasRequired_null;

function require_null() {
    if (hasRequired_null) return _null;
    hasRequired_null = 1;
    var Scalar = requireScalar();
    const nullTag = {
        identify: value => null == value,
        createNode: () => new Scalar.Scalar(null),
        default: !0,
        tag: "tag:yaml.org,2002:null",
        test: /^(?:~|[Nn]ull|NULL)?$/,
        resolve: () => new Scalar.Scalar(null),
        stringify: ({source: source}, ctx) => "string" == typeof source && nullTag.test.test(source) ? source : ctx.options.nullStr
    };
    return _null.nullTag = nullTag, _null;
}

var bool$1 = {}, hasRequiredBool$1;

function requireBool$1() {
    if (hasRequiredBool$1) return bool$1;
    hasRequiredBool$1 = 1;
    var Scalar = requireScalar();
    const boolTag = {
        identify: value => "boolean" == typeof value,
        default: !0,
        tag: "tag:yaml.org,2002:bool",
        test: /^(?:[Tt]rue|TRUE|[Ff]alse|FALSE)$/,
        resolve: str => new Scalar.Scalar("t" === str[0] || "T" === str[0]),
        stringify({source: source, value: value}, ctx) {
            if (source && boolTag.test.test(source)) {
                if (value === ("t" === source[0] || "T" === source[0])) return source;
            }
            return value ? ctx.options.trueStr : ctx.options.falseStr;
        }
    };
    return bool$1.boolTag = boolTag, bool$1;
}

var float$1 = {}, stringifyNumber = {}, hasRequiredStringifyNumber, hasRequiredFloat$1;

function requireStringifyNumber() {
    if (hasRequiredStringifyNumber) return stringifyNumber;
    return hasRequiredStringifyNumber = 1, stringifyNumber.stringifyNumber = function({format: format, minFractionDigits: minFractionDigits, tag: tag, value: value}) {
        if ("bigint" == typeof value) return String(value);
        const num = "number" == typeof value ? value : Number(value);
        if (!isFinite(num)) return isNaN(num) ? ".nan" : num < 0 ? "-.inf" : ".inf";
        let n = JSON.stringify(value);
        if (!format && minFractionDigits && (!tag || "tag:yaml.org,2002:float" === tag) && /^\d/.test(n)) {
            let i = n.indexOf(".");
            i < 0 && (i = n.length, n += ".");
            let d = minFractionDigits - (n.length - i - 1);
            for (;d-- > 0; ) n += "0";
        }
        return n;
    }, stringifyNumber;
}

function requireFloat$1() {
    if (hasRequiredFloat$1) return float$1;
    hasRequiredFloat$1 = 1;
    var Scalar = requireScalar(), stringifyNumber = requireStringifyNumber();
    const floatNaN = {
        identify: value => "number" == typeof value,
        default: !0,
        tag: "tag:yaml.org,2002:float",
        test: /^(?:[-+]?\.(?:inf|Inf|INF)|\.nan|\.NaN|\.NAN)$/,
        resolve: str => "nan" === str.slice(-3).toLowerCase() ? NaN : "-" === str[0] ? Number.NEGATIVE_INFINITY : Number.POSITIVE_INFINITY,
        stringify: stringifyNumber.stringifyNumber
    }, floatExp = {
        identify: value => "number" == typeof value,
        default: !0,
        tag: "tag:yaml.org,2002:float",
        format: "EXP",
        test: /^[-+]?(?:\.[0-9]+|[0-9]+(?:\.[0-9]*)?)[eE][-+]?[0-9]+$/,
        resolve: str => parseFloat(str),
        stringify(node) {
            const num = Number(node.value);
            return isFinite(num) ? num.toExponential() : stringifyNumber.stringifyNumber(node);
        }
    }, float = {
        identify: value => "number" == typeof value,
        default: !0,
        tag: "tag:yaml.org,2002:float",
        test: /^[-+]?(?:\.[0-9]+|[0-9]+\.[0-9]*)$/,
        resolve(str) {
            const node = new Scalar.Scalar(parseFloat(str)), dot = str.indexOf(".");
            return -1 !== dot && "0" === str[str.length - 1] && (node.minFractionDigits = str.length - dot - 1), 
            node;
        },
        stringify: stringifyNumber.stringifyNumber
    };
    return float$1.float = float, float$1.floatExp = floatExp, float$1.floatNaN = floatNaN, 
    float$1;
}

var int$1 = {}, hasRequiredInt$1;

function requireInt$1() {
    if (hasRequiredInt$1) return int$1;
    hasRequiredInt$1 = 1;
    var stringifyNumber = requireStringifyNumber();
    const intIdentify = value => "bigint" == typeof value || Number.isInteger(value), intResolve = (str, offset, radix, {intAsBigInt: intAsBigInt}) => intAsBigInt ? BigInt(str) : parseInt(str.substring(offset), radix);
    function intStringify(node, radix, prefix) {
        const {value: value} = node;
        return intIdentify(value) && value >= 0 ? prefix + value.toString(radix) : stringifyNumber.stringifyNumber(node);
    }
    const intOct = {
        identify: value => intIdentify(value) && value >= 0,
        default: !0,
        tag: "tag:yaml.org,2002:int",
        format: "OCT",
        test: /^0o[0-7]+$/,
        resolve: (str, _onError, opt) => intResolve(str, 2, 8, opt),
        stringify: node => intStringify(node, 8, "0o")
    }, int = {
        identify: intIdentify,
        default: !0,
        tag: "tag:yaml.org,2002:int",
        test: /^[-+]?[0-9]+$/,
        resolve: (str, _onError, opt) => intResolve(str, 0, 10, opt),
        stringify: stringifyNumber.stringifyNumber
    }, intHex = {
        identify: value => intIdentify(value) && value >= 0,
        default: !0,
        tag: "tag:yaml.org,2002:int",
        format: "HEX",
        test: /^0x[0-9a-fA-F]+$/,
        resolve: (str, _onError, opt) => intResolve(str, 2, 16, opt),
        stringify: node => intStringify(node, 16, "0x")
    };
    return int$1.int = int, int$1.intHex = intHex, int$1.intOct = intOct, int$1;
}

var schema$2 = {}, hasRequiredSchema$3;

function requireSchema$3() {
    if (hasRequiredSchema$3) return schema$2;
    hasRequiredSchema$3 = 1;
    var map = requireMap(), _null = require_null(), seq = requireSeq(), string = requireString(), bool = requireBool$1(), float = requireFloat$1(), int = requireInt$1();
    const schema = [ map.map, seq.seq, string.string, _null.nullTag, bool.boolTag, int.intOct, int.int, int.intHex, float.floatNaN, float.floatExp, float.float ];
    return schema$2.schema = schema, schema$2;
}

var schema$1 = {}, hasRequiredSchema$2;

function requireSchema$2() {
    if (hasRequiredSchema$2) return schema$1;
    hasRequiredSchema$2 = 1;
    var Scalar = requireScalar(), map = requireMap(), seq = requireSeq();
    function intIdentify(value) {
        return "bigint" == typeof value || Number.isInteger(value);
    }
    const stringifyJSON = ({value: value}) => JSON.stringify(value), jsonScalars = [ {
        identify: value => "string" == typeof value,
        default: !0,
        tag: "tag:yaml.org,2002:str",
        resolve: str => str,
        stringify: stringifyJSON
    }, {
        identify: value => null == value,
        createNode: () => new Scalar.Scalar(null),
        default: !0,
        tag: "tag:yaml.org,2002:null",
        test: /^null$/,
        resolve: () => null,
        stringify: stringifyJSON
    }, {
        identify: value => "boolean" == typeof value,
        default: !0,
        tag: "tag:yaml.org,2002:bool",
        test: /^true|false$/,
        resolve: str => "true" === str,
        stringify: stringifyJSON
    }, {
        identify: intIdentify,
        default: !0,
        tag: "tag:yaml.org,2002:int",
        test: /^-?(?:0|[1-9][0-9]*)$/,
        resolve: (str, _onError, {intAsBigInt: intAsBigInt}) => intAsBigInt ? BigInt(str) : parseInt(str, 10),
        stringify: ({value: value}) => intIdentify(value) ? value.toString() : JSON.stringify(value)
    }, {
        identify: value => "number" == typeof value,
        default: !0,
        tag: "tag:yaml.org,2002:float",
        test: /^-?(?:0|[1-9][0-9]*)(?:\.[0-9]*)?(?:[eE][-+]?[0-9]+)?$/,
        resolve: str => parseFloat(str),
        stringify: stringifyJSON
    } ], jsonError = {
        default: !0,
        tag: "",
        test: /^/,
        resolve: (str, onError) => (onError(`Unresolved plain scalar ${JSON.stringify(str)}`), 
        str)
    }, schema = [ map.map, seq.seq ].concat(jsonScalars, jsonError);
    return schema$1.schema = schema, schema$1;
}

var binary$1 = {}, hasRequiredBinary;

function requireBinary() {
    if (hasRequiredBinary) return binary$1;
    hasRequiredBinary = 1;
    var Scalar = requireScalar(), stringifyString = requireStringifyString();
    const binary = {
        identify: value => value instanceof Uint8Array,
        default: !1,
        tag: "tag:yaml.org,2002:binary",
        resolve(src, onError) {
            if ("function" == typeof Buffer) return Buffer.from(src, "base64");
            if ("function" == typeof atob) {
                const str = atob(src.replace(/[\n\r]/g, "")), buffer = new Uint8Array(str.length);
                for (let i = 0; i < str.length; ++i) buffer[i] = str.charCodeAt(i);
                return buffer;
            }
            return onError("This environment does not support reading binary tags; either Buffer or atob is required"), 
            src;
        },
        stringify({comment: comment, type: type, value: value}, ctx, onComment, onChompKeep) {
            const buf = value;
            let str;
            if ("function" == typeof Buffer) str = buf instanceof Buffer ? buf.toString("base64") : Buffer.from(buf.buffer).toString("base64"); else {
                if ("function" != typeof btoa) throw new Error("This environment does not support writing binary tags; either Buffer or btoa is required");
                {
                    let s = "";
                    for (let i = 0; i < buf.length; ++i) s += String.fromCharCode(buf[i]);
                    str = btoa(s);
                }
            }
            if (type || (type = Scalar.Scalar.BLOCK_LITERAL), type !== Scalar.Scalar.QUOTE_DOUBLE) {
                const lineWidth = Math.max(ctx.options.lineWidth - ctx.indent.length, ctx.options.minContentWidth), n = Math.ceil(str.length / lineWidth), lines = new Array(n);
                for (let i = 0, o = 0; i < n; ++i, o += lineWidth) lines[i] = str.substr(o, lineWidth);
                str = lines.join(type === Scalar.Scalar.BLOCK_LITERAL ? "\n" : " ");
            }
            return stringifyString.stringifyString({
                comment: comment,
                type: type,
                value: str
            }, ctx, onComment, onChompKeep);
        }
    };
    return binary$1.binary = binary, binary$1;
}

var omap = {}, pairs = {}, hasRequiredPairs, hasRequiredOmap;

function requirePairs() {
    if (hasRequiredPairs) return pairs;
    hasRequiredPairs = 1;
    var identity = requireIdentity(), Pair = requirePair(), Scalar = requireScalar(), YAMLSeq = requireYAMLSeq();
    function resolvePairs(seq, onError) {
        if (identity.isSeq(seq)) for (let i = 0; i < seq.items.length; ++i) {
            let item = seq.items[i];
            if (!identity.isPair(item)) {
                if (identity.isMap(item)) {
                    item.items.length > 1 && onError("Each pair must have its own sequence indicator");
                    const pair = item.items[0] || new Pair.Pair(new Scalar.Scalar(null));
                    if (item.commentBefore && (pair.key.commentBefore = pair.key.commentBefore ? `${item.commentBefore}\n${pair.key.commentBefore}` : item.commentBefore), 
                    item.comment) {
                        const cn = pair.value ?? pair.key;
                        cn.comment = cn.comment ? `${item.comment}\n${cn.comment}` : item.comment;
                    }
                    item = pair;
                }
                seq.items[i] = identity.isPair(item) ? item : new Pair.Pair(item);
            }
        } else onError("Expected a sequence for this tag");
        return seq;
    }
    function createPairs(schema, iterable, ctx) {
        const {replacer: replacer} = ctx, pairs = new YAMLSeq.YAMLSeq(schema);
        pairs.tag = "tag:yaml.org,2002:pairs";
        let i = 0;
        if (iterable && Symbol.iterator in Object(iterable)) for (let it of iterable) {
            let key, value;
            if ("function" == typeof replacer && (it = replacer.call(iterable, String(i++), it)), 
            Array.isArray(it)) {
                if (2 !== it.length) throw new TypeError(`Expected [key, value] tuple: ${it}`);
                key = it[0], value = it[1];
            } else if (it && it instanceof Object) {
                const keys = Object.keys(it);
                if (1 !== keys.length) throw new TypeError(`Expected tuple with one key, not ${keys.length} keys`);
                key = keys[0], value = it[key];
            } else key = it;
            pairs.items.push(Pair.createPair(key, value, ctx));
        }
        return pairs;
    }
    const pairs$1 = {
        collection: "seq",
        default: !1,
        tag: "tag:yaml.org,2002:pairs",
        resolve: resolvePairs,
        createNode: createPairs
    };
    return pairs.createPairs = createPairs, pairs.pairs = pairs$1, pairs.resolvePairs = resolvePairs, 
    pairs;
}

function requireOmap() {
    if (hasRequiredOmap) return omap;
    hasRequiredOmap = 1;
    var identity = requireIdentity(), toJS = requireToJS(), YAMLMap = requireYAMLMap(), YAMLSeq = requireYAMLSeq(), pairs = requirePairs();
    class YAMLOMap extends YAMLSeq.YAMLSeq {
        constructor() {
            super(), this.add = YAMLMap.YAMLMap.prototype.add.bind(this), this.delete = YAMLMap.YAMLMap.prototype.delete.bind(this), 
            this.get = YAMLMap.YAMLMap.prototype.get.bind(this), this.has = YAMLMap.YAMLMap.prototype.has.bind(this), 
            this.set = YAMLMap.YAMLMap.prototype.set.bind(this), this.tag = YAMLOMap.tag;
        }
        toJSON(_, ctx) {
            if (!ctx) return super.toJSON(_);
            const map = new Map;
            ctx?.onCreate && ctx.onCreate(map);
            for (const pair of this.items) {
                let key, value;
                if (identity.isPair(pair) ? (key = toJS.toJS(pair.key, "", ctx), value = toJS.toJS(pair.value, key, ctx)) : key = toJS.toJS(pair, "", ctx), 
                map.has(key)) throw new Error("Ordered maps must not include duplicate keys");
                map.set(key, value);
            }
            return map;
        }
        static from(schema, iterable, ctx) {
            const pairs$1 = pairs.createPairs(schema, iterable, ctx), omap = new this;
            return omap.items = pairs$1.items, omap;
        }
    }
    YAMLOMap.tag = "tag:yaml.org,2002:omap";
    const omap$1 = {
        collection: "seq",
        identify: value => value instanceof Map,
        nodeClass: YAMLOMap,
        default: !1,
        tag: "tag:yaml.org,2002:omap",
        resolve(seq, onError) {
            const pairs$1 = pairs.resolvePairs(seq, onError), seenKeys = [];
            for (const {key: key} of pairs$1.items) identity.isScalar(key) && (seenKeys.includes(key.value) ? onError(`Ordered maps must not include duplicate keys: ${key.value}`) : seenKeys.push(key.value));
            return Object.assign(new YAMLOMap, pairs$1);
        },
        createNode: (schema, iterable, ctx) => YAMLOMap.from(schema, iterable, ctx)
    };
    return omap.YAMLOMap = YAMLOMap, omap.omap = omap$1, omap;
}

var schema = {}, bool = {}, hasRequiredBool;

function requireBool() {
    if (hasRequiredBool) return bool;
    hasRequiredBool = 1;
    var Scalar = requireScalar();
    function boolStringify({value: value, source: source}, ctx) {
        return source && (value ? trueTag : falseTag).test.test(source) ? source : value ? ctx.options.trueStr : ctx.options.falseStr;
    }
    const trueTag = {
        identify: value => !0 === value,
        default: !0,
        tag: "tag:yaml.org,2002:bool",
        test: /^(?:Y|y|[Yy]es|YES|[Tt]rue|TRUE|[Oo]n|ON)$/,
        resolve: () => new Scalar.Scalar(!0),
        stringify: boolStringify
    }, falseTag = {
        identify: value => !1 === value,
        default: !0,
        tag: "tag:yaml.org,2002:bool",
        test: /^(?:N|n|[Nn]o|NO|[Ff]alse|FALSE|[Oo]ff|OFF)$/,
        resolve: () => new Scalar.Scalar(!1),
        stringify: boolStringify
    };
    return bool.falseTag = falseTag, bool.trueTag = trueTag, bool;
}

var float = {}, hasRequiredFloat;

function requireFloat() {
    if (hasRequiredFloat) return float;
    hasRequiredFloat = 1;
    var Scalar = requireScalar(), stringifyNumber = requireStringifyNumber();
    const floatNaN = {
        identify: value => "number" == typeof value,
        default: !0,
        tag: "tag:yaml.org,2002:float",
        test: /^(?:[-+]?\.(?:inf|Inf|INF)|\.nan|\.NaN|\.NAN)$/,
        resolve: str => "nan" === str.slice(-3).toLowerCase() ? NaN : "-" === str[0] ? Number.NEGATIVE_INFINITY : Number.POSITIVE_INFINITY,
        stringify: stringifyNumber.stringifyNumber
    }, floatExp = {
        identify: value => "number" == typeof value,
        default: !0,
        tag: "tag:yaml.org,2002:float",
        format: "EXP",
        test: /^[-+]?(?:[0-9][0-9_]*)?(?:\.[0-9_]*)?[eE][-+]?[0-9]+$/,
        resolve: str => parseFloat(str.replace(/_/g, "")),
        stringify(node) {
            const num = Number(node.value);
            return isFinite(num) ? num.toExponential() : stringifyNumber.stringifyNumber(node);
        }
    }, float$1 = {
        identify: value => "number" == typeof value,
        default: !0,
        tag: "tag:yaml.org,2002:float",
        test: /^[-+]?(?:[0-9][0-9_]*)?\.[0-9_]*$/,
        resolve(str) {
            const node = new Scalar.Scalar(parseFloat(str.replace(/_/g, ""))), dot = str.indexOf(".");
            if (-1 !== dot) {
                const f = str.substring(dot + 1).replace(/_/g, "");
                "0" === f[f.length - 1] && (node.minFractionDigits = f.length);
            }
            return node;
        },
        stringify: stringifyNumber.stringifyNumber
    };
    return float.float = float$1, float.floatExp = floatExp, float.floatNaN = floatNaN, 
    float;
}

var int = {}, hasRequiredInt;

function requireInt() {
    if (hasRequiredInt) return int;
    hasRequiredInt = 1;
    var stringifyNumber = requireStringifyNumber();
    const intIdentify = value => "bigint" == typeof value || Number.isInteger(value);
    function intResolve(str, offset, radix, {intAsBigInt: intAsBigInt}) {
        const sign = str[0];
        if ("-" !== sign && "+" !== sign || (offset += 1), str = str.substring(offset).replace(/_/g, ""), 
        intAsBigInt) {
            switch (radix) {
              case 2:
                str = `0b${str}`;
                break;

              case 8:
                str = `0o${str}`;
                break;

              case 16:
                str = `0x${str}`;
            }
            const n = BigInt(str);
            return "-" === sign ? BigInt(-1) * n : n;
        }
        const n = parseInt(str, radix);
        return "-" === sign ? -1 * n : n;
    }
    function intStringify(node, radix, prefix) {
        const {value: value} = node;
        if (intIdentify(value)) {
            const str = value.toString(radix);
            return value < 0 ? "-" + prefix + str.substr(1) : prefix + str;
        }
        return stringifyNumber.stringifyNumber(node);
    }
    const intBin = {
        identify: intIdentify,
        default: !0,
        tag: "tag:yaml.org,2002:int",
        format: "BIN",
        test: /^[-+]?0b[0-1_]+$/,
        resolve: (str, _onError, opt) => intResolve(str, 2, 2, opt),
        stringify: node => intStringify(node, 2, "0b")
    }, intOct = {
        identify: intIdentify,
        default: !0,
        tag: "tag:yaml.org,2002:int",
        format: "OCT",
        test: /^[-+]?0[0-7_]+$/,
        resolve: (str, _onError, opt) => intResolve(str, 1, 8, opt),
        stringify: node => intStringify(node, 8, "0")
    }, int$1 = {
        identify: intIdentify,
        default: !0,
        tag: "tag:yaml.org,2002:int",
        test: /^[-+]?[0-9][0-9_]*$/,
        resolve: (str, _onError, opt) => intResolve(str, 0, 10, opt),
        stringify: stringifyNumber.stringifyNumber
    }, intHex = {
        identify: intIdentify,
        default: !0,
        tag: "tag:yaml.org,2002:int",
        format: "HEX",
        test: /^[-+]?0x[0-9a-fA-F_]+$/,
        resolve: (str, _onError, opt) => intResolve(str, 2, 16, opt),
        stringify: node => intStringify(node, 16, "0x")
    };
    return int.int = int$1, int.intBin = intBin, int.intHex = intHex, int.intOct = intOct, 
    int;
}

var set = {}, hasRequiredSet;

function requireSet() {
    if (hasRequiredSet) return set;
    hasRequiredSet = 1;
    var identity = requireIdentity(), Pair = requirePair(), YAMLMap = requireYAMLMap();
    class YAMLSet extends YAMLMap.YAMLMap {
        constructor(schema) {
            super(schema), this.tag = YAMLSet.tag;
        }
        add(key) {
            let pair;
            pair = identity.isPair(key) ? key : key && "object" == typeof key && "key" in key && "value" in key && null === key.value ? new Pair.Pair(key.key, null) : new Pair.Pair(key, null);
            YAMLMap.findPair(this.items, pair.key) || this.items.push(pair);
        }
        get(key, keepPair) {
            const pair = YAMLMap.findPair(this.items, key);
            return !keepPair && identity.isPair(pair) ? identity.isScalar(pair.key) ? pair.key.value : pair.key : pair;
        }
        set(key, value) {
            if ("boolean" != typeof value) throw new Error("Expected boolean value for set(key, value) in a YAML set, not " + typeof value);
            const prev = YAMLMap.findPair(this.items, key);
            prev && !value ? this.items.splice(this.items.indexOf(prev), 1) : !prev && value && this.items.push(new Pair.Pair(key));
        }
        toJSON(_, ctx) {
            return super.toJSON(_, ctx, Set);
        }
        toString(ctx, onComment, onChompKeep) {
            if (!ctx) return JSON.stringify(this);
            if (this.hasAllNullValues(!0)) return super.toString(Object.assign({}, ctx, {
                allNullValues: !0
            }), onComment, onChompKeep);
            throw new Error("Set items must all have null values");
        }
        static from(schema, iterable, ctx) {
            const {replacer: replacer} = ctx, set = new this(schema);
            if (iterable && Symbol.iterator in Object(iterable)) for (let value of iterable) "function" == typeof replacer && (value = replacer.call(iterable, value, value)), 
            set.items.push(Pair.createPair(value, null, ctx));
            return set;
        }
    }
    YAMLSet.tag = "tag:yaml.org,2002:set";
    const set$1 = {
        collection: "map",
        identify: value => value instanceof Set,
        nodeClass: YAMLSet,
        default: !1,
        tag: "tag:yaml.org,2002:set",
        createNode: (schema, iterable, ctx) => YAMLSet.from(schema, iterable, ctx),
        resolve(map, onError) {
            if (identity.isMap(map)) {
                if (map.hasAllNullValues(!0)) return Object.assign(new YAMLSet, map);
                onError("Set items must all have null values");
            } else onError("Expected a mapping for this tag");
            return map;
        }
    };
    return set.YAMLSet = YAMLSet, set.set = set$1, set;
}

var timestamp = {}, hasRequiredTimestamp, hasRequiredSchema$1, hasRequiredTags, hasRequiredSchema;

function requireTimestamp() {
    if (hasRequiredTimestamp) return timestamp;
    hasRequiredTimestamp = 1;
    var stringifyNumber = requireStringifyNumber();
    function parseSexagesimal(str, asBigInt) {
        const sign = str[0], parts = "-" === sign || "+" === sign ? str.substring(1) : str, num = n => asBigInt ? BigInt(n) : Number(n), res = parts.replace(/_/g, "").split(":").reduce(((res, p) => res * num(60) + num(p)), num(0));
        return "-" === sign ? num(-1) * res : res;
    }
    function stringifySexagesimal(node) {
        let {value: value} = node, num = n => n;
        if ("bigint" == typeof value) num = n => BigInt(n); else if (isNaN(value) || !isFinite(value)) return stringifyNumber.stringifyNumber(node);
        let sign = "";
        value < 0 && (sign = "-", value *= num(-1));
        const _60 = num(60), parts = [ value % _60 ];
        return value < 60 ? parts.unshift(0) : (value = (value - parts[0]) / _60, parts.unshift(value % _60), 
        value >= 60 && (value = (value - parts[0]) / _60, parts.unshift(value))), sign + parts.map((n => String(n).padStart(2, "0"))).join(":").replace(/000000\d*$/, "");
    }
    const intTime = {
        identify: value => "bigint" == typeof value || Number.isInteger(value),
        default: !0,
        tag: "tag:yaml.org,2002:int",
        format: "TIME",
        test: /^[-+]?[0-9][0-9_]*(?::[0-5]?[0-9])+$/,
        resolve: (str, _onError, {intAsBigInt: intAsBigInt}) => parseSexagesimal(str, intAsBigInt),
        stringify: stringifySexagesimal
    }, floatTime = {
        identify: value => "number" == typeof value,
        default: !0,
        tag: "tag:yaml.org,2002:float",
        format: "TIME",
        test: /^[-+]?[0-9][0-9_]*(?::[0-5]?[0-9])+\.[0-9_]*$/,
        resolve: str => parseSexagesimal(str, !1),
        stringify: stringifySexagesimal
    }, timestamp$1 = {
        identify: value => value instanceof Date,
        default: !0,
        tag: "tag:yaml.org,2002:timestamp",
        test: RegExp("^([0-9]{4})-([0-9]{1,2})-([0-9]{1,2})(?:(?:t|T|[ \\t]+)([0-9]{1,2}):([0-9]{1,2}):([0-9]{1,2}(\\.[0-9]+)?)(?:[ \\t]*(Z|[-+][012]?[0-9](?::[0-9]{2})?))?)?$"),
        resolve(str) {
            const match = str.match(timestamp$1.test);
            if (!match) throw new Error("!!timestamp expects a date, starting with yyyy-mm-dd");
            const [, year, month, day, hour, minute, second] = match.map(Number), millisec = match[7] ? Number((match[7] + "00").substr(1, 3)) : 0;
            let date = Date.UTC(year, month - 1, day, hour || 0, minute || 0, second || 0, millisec);
            const tz = match[8];
            if (tz && "Z" !== tz) {
                let d = parseSexagesimal(tz, !1);
                Math.abs(d) < 30 && (d *= 60), date -= 6e4 * d;
            }
            return new Date(date);
        },
        stringify: ({value: value}) => value.toISOString().replace(/((T00:00)?:00)?\.000Z$/, "")
    };
    return timestamp.floatTime = floatTime, timestamp.intTime = intTime, timestamp.timestamp = timestamp$1, 
    timestamp;
}

function requireSchema$1() {
    if (hasRequiredSchema$1) return schema;
    hasRequiredSchema$1 = 1;
    var map = requireMap(), _null = require_null(), seq = requireSeq(), string = requireString(), binary = requireBinary(), bool = requireBool(), float = requireFloat(), int = requireInt(), merge = requireMerge(), omap = requireOmap(), pairs = requirePairs(), set = requireSet(), timestamp = requireTimestamp();
    const schema$1 = [ map.map, seq.seq, string.string, _null.nullTag, bool.trueTag, bool.falseTag, int.intBin, int.intOct, int.int, int.intHex, float.floatNaN, float.floatExp, float.float, binary.binary, merge.merge, omap.omap, pairs.pairs, set.set, timestamp.intTime, timestamp.floatTime, timestamp.timestamp ];
    return schema.schema = schema$1, schema;
}

function requireTags() {
    if (hasRequiredTags) return tags;
    hasRequiredTags = 1;
    var map = requireMap(), _null = require_null(), seq = requireSeq(), string = requireString(), bool = requireBool$1(), float = requireFloat$1(), int = requireInt$1(), schema = requireSchema$3(), schema$1 = requireSchema$2(), binary = requireBinary(), merge = requireMerge(), omap = requireOmap(), pairs = requirePairs(), schema$2 = requireSchema$1(), set = requireSet(), timestamp = requireTimestamp();
    const schemas = new Map([ [ "core", schema.schema ], [ "failsafe", [ map.map, seq.seq, string.string ] ], [ "json", schema$1.schema ], [ "yaml11", schema$2.schema ], [ "yaml-1.1", schema$2.schema ] ]), tagsByName = {
        binary: binary.binary,
        bool: bool.boolTag,
        float: float.float,
        floatExp: float.floatExp,
        floatNaN: float.floatNaN,
        floatTime: timestamp.floatTime,
        int: int.int,
        intHex: int.intHex,
        intOct: int.intOct,
        intTime: timestamp.intTime,
        map: map.map,
        merge: merge.merge,
        null: _null.nullTag,
        omap: omap.omap,
        pairs: pairs.pairs,
        seq: seq.seq,
        set: set.set,
        timestamp: timestamp.timestamp
    }, coreKnownTags = {
        "tag:yaml.org,2002:binary": binary.binary,
        "tag:yaml.org,2002:merge": merge.merge,
        "tag:yaml.org,2002:omap": omap.omap,
        "tag:yaml.org,2002:pairs": pairs.pairs,
        "tag:yaml.org,2002:set": set.set,
        "tag:yaml.org,2002:timestamp": timestamp.timestamp
    };
    return tags.coreKnownTags = coreKnownTags, tags.getTags = function(customTags, schemaName, addMergeTag) {
        const schemaTags = schemas.get(schemaName);
        if (schemaTags && !customTags) return addMergeTag && !schemaTags.includes(merge.merge) ? schemaTags.concat(merge.merge) : schemaTags.slice();
        let tags = schemaTags;
        if (!tags) {
            if (!Array.isArray(customTags)) {
                const keys = Array.from(schemas.keys()).filter((key => "yaml11" !== key)).map((key => JSON.stringify(key))).join(", ");
                throw new Error(`Unknown schema "${schemaName}"; use one of ${keys} or define customTags array`);
            }
            tags = [];
        }
        if (Array.isArray(customTags)) for (const tag of customTags) tags = tags.concat(tag); else "function" == typeof customTags && (tags = customTags(tags.slice()));
        return addMergeTag && (tags = tags.concat(merge.merge)), tags.reduce(((tags, tag) => {
            const tagObj = "string" == typeof tag ? tagsByName[tag] : tag;
            if (!tagObj) {
                const tagName = JSON.stringify(tag), keys = Object.keys(tagsByName).map((key => JSON.stringify(key))).join(", ");
                throw new Error(`Unknown custom tag ${tagName}; use one of ${keys}`);
            }
            return tags.includes(tagObj) || tags.push(tagObj), tags;
        }), []);
    }, tags;
}

function requireSchema() {
    if (hasRequiredSchema) return Schema;
    hasRequiredSchema = 1;
    var identity = requireIdentity(), map = requireMap(), seq = requireSeq(), string = requireString(), tags = requireTags();
    const sortMapEntriesByKey = (a, b) => a.key < b.key ? -1 : a.key > b.key ? 1 : 0;
    return Schema.Schema = class Schema {
        constructor({compat: compat, customTags: customTags, merge: merge, resolveKnownTags: resolveKnownTags, schema: schema, sortMapEntries: sortMapEntries, toStringDefaults: toStringDefaults}) {
            this.compat = Array.isArray(compat) ? tags.getTags(compat, "compat") : compat ? tags.getTags(null, compat) : null, 
            this.name = "string" == typeof schema && schema || "core", this.knownTags = resolveKnownTags ? tags.coreKnownTags : {}, 
            this.tags = tags.getTags(customTags, this.name, merge), this.toStringOptions = toStringDefaults ?? null, 
            Object.defineProperty(this, identity.MAP, {
                value: map.map
            }), Object.defineProperty(this, identity.SCALAR, {
                value: string.string
            }), Object.defineProperty(this, identity.SEQ, {
                value: seq.seq
            }), this.sortMapEntries = "function" == typeof sortMapEntries ? sortMapEntries : !0 === sortMapEntries ? sortMapEntriesByKey : null;
        }
        clone() {
            const copy = Object.create(Schema.prototype, Object.getOwnPropertyDescriptors(this));
            return copy.tags = this.tags.slice(), copy;
        }
    }, Schema;
}

var stringifyDocument = {}, hasRequiredStringifyDocument, hasRequiredDocument;

function requireStringifyDocument() {
    if (hasRequiredStringifyDocument) return stringifyDocument;
    hasRequiredStringifyDocument = 1;
    var identity = requireIdentity(), stringify = requireStringify(), stringifyComment = requireStringifyComment();
    return stringifyDocument.stringifyDocument = function(doc, options) {
        const lines = [];
        let hasDirectives = !0 === options.directives;
        if (!1 !== options.directives && doc.directives) {
            const dir = doc.directives.toString(doc);
            dir ? (lines.push(dir), hasDirectives = !0) : doc.directives.docStart && (hasDirectives = !0);
        }
        hasDirectives && lines.push("---");
        const ctx = stringify.createStringifyContext(doc, options), {commentString: commentString} = ctx.options;
        if (doc.commentBefore) {
            1 !== lines.length && lines.unshift("");
            const cs = commentString(doc.commentBefore);
            lines.unshift(stringifyComment.indentComment(cs, ""));
        }
        let chompKeep = !1, contentComment = null;
        if (doc.contents) {
            if (identity.isNode(doc.contents)) {
                if (doc.contents.spaceBefore && hasDirectives && lines.push(""), doc.contents.commentBefore) {
                    const cs = commentString(doc.contents.commentBefore);
                    lines.push(stringifyComment.indentComment(cs, ""));
                }
                ctx.forceBlockIndent = !!doc.comment, contentComment = doc.contents.comment;
            }
            const onChompKeep = contentComment ? void 0 : () => chompKeep = !0;
            let body = stringify.stringify(doc.contents, ctx, (() => contentComment = null), onChompKeep);
            contentComment && (body += stringifyComment.lineComment(body, "", commentString(contentComment))), 
            "|" !== body[0] && ">" !== body[0] || "---" !== lines[lines.length - 1] ? lines.push(body) : lines[lines.length - 1] = `--- ${body}`;
        } else lines.push(stringify.stringify(doc.contents, ctx));
        if (doc.directives?.docEnd) if (doc.comment) {
            const cs = commentString(doc.comment);
            cs.includes("\n") ? (lines.push("..."), lines.push(stringifyComment.indentComment(cs, ""))) : lines.push(`... ${cs}`);
        } else lines.push("..."); else {
            let dc = doc.comment;
            dc && chompKeep && (dc = dc.replace(/^\n+/, "")), dc && (chompKeep && !contentComment || "" === lines[lines.length - 1] || lines.push(""), 
            lines.push(stringifyComment.indentComment(commentString(dc), "")));
        }
        return lines.join("\n") + "\n";
    }, stringifyDocument;
}

function requireDocument() {
    if (hasRequiredDocument) return Document;
    hasRequiredDocument = 1;
    var Alias = requireAlias(), Collection = requireCollection(), identity = requireIdentity(), Pair = requirePair(), toJS = requireToJS(), Schema = requireSchema(), stringifyDocument = requireStringifyDocument(), anchors = requireAnchors(), applyReviver = requireApplyReviver(), createNode = requireCreateNode(), directives = requireDirectives();
    function assertCollection(contents) {
        if (identity.isCollection(contents)) return !0;
        throw new Error("Expected a YAML collection as document contents");
    }
    return Document.Document = class Document {
        constructor(value, replacer, options) {
            this.commentBefore = null, this.comment = null, this.errors = [], this.warnings = [], 
            Object.defineProperty(this, identity.NODE_TYPE, {
                value: identity.DOC
            });
            let _replacer = null;
            "function" == typeof replacer || Array.isArray(replacer) ? _replacer = replacer : void 0 === options && replacer && (options = replacer, 
            replacer = void 0);
            const opt = Object.assign({
                intAsBigInt: !1,
                keepSourceTokens: !1,
                logLevel: "warn",
                prettyErrors: !0,
                strict: !0,
                stringKeys: !1,
                uniqueKeys: !0,
                version: "1.2"
            }, options);
            this.options = opt;
            let {version: version} = opt;
            options?._directives ? (this.directives = options._directives.atDocument(), this.directives.yaml.explicit && (version = this.directives.yaml.version)) : this.directives = new directives.Directives({
                version: version
            }), this.setSchema(version, options), this.contents = void 0 === value ? null : this.createNode(value, _replacer, options);
        }
        clone() {
            const copy = Object.create(Document.prototype, {
                [identity.NODE_TYPE]: {
                    value: identity.DOC
                }
            });
            return copy.commentBefore = this.commentBefore, copy.comment = this.comment, copy.errors = this.errors.slice(), 
            copy.warnings = this.warnings.slice(), copy.options = Object.assign({}, this.options), 
            this.directives && (copy.directives = this.directives.clone()), copy.schema = this.schema.clone(), 
            copy.contents = identity.isNode(this.contents) ? this.contents.clone(copy.schema) : this.contents, 
            this.range && (copy.range = this.range.slice()), copy;
        }
        add(value) {
            assertCollection(this.contents) && this.contents.add(value);
        }
        addIn(path, value) {
            assertCollection(this.contents) && this.contents.addIn(path, value);
        }
        createAlias(node, name) {
            if (!node.anchor) {
                const prev = anchors.anchorNames(this);
                node.anchor = !name || prev.has(name) ? anchors.findNewAnchor(name || "a", prev) : name;
            }
            return new Alias.Alias(node.anchor);
        }
        createNode(value, replacer, options) {
            let _replacer;
            if ("function" == typeof replacer) value = replacer.call({
                "": value
            }, "", value), _replacer = replacer; else if (Array.isArray(replacer)) {
                const keyToStr = v => "number" == typeof v || v instanceof String || v instanceof Number, asStr = replacer.filter(keyToStr).map(String);
                asStr.length > 0 && (replacer = replacer.concat(asStr)), _replacer = replacer;
            } else void 0 === options && replacer && (options = replacer, replacer = void 0);
            const {aliasDuplicateObjects: aliasDuplicateObjects, anchorPrefix: anchorPrefix, flow: flow, keepUndefined: keepUndefined, onTagObj: onTagObj, tag: tag} = options ?? {}, {onAnchor: onAnchor, setAnchors: setAnchors, sourceObjects: sourceObjects} = anchors.createNodeAnchors(this, anchorPrefix || "a"), ctx = {
                aliasDuplicateObjects: aliasDuplicateObjects ?? !0,
                keepUndefined: keepUndefined ?? !1,
                onAnchor: onAnchor,
                onTagObj: onTagObj,
                replacer: _replacer,
                schema: this.schema,
                sourceObjects: sourceObjects
            }, node = createNode.createNode(value, tag, ctx);
            return flow && identity.isCollection(node) && (node.flow = !0), setAnchors(), node;
        }
        createPair(key, value, options = {}) {
            const k = this.createNode(key, null, options), v = this.createNode(value, null, options);
            return new Pair.Pair(k, v);
        }
        delete(key) {
            return !!assertCollection(this.contents) && this.contents.delete(key);
        }
        deleteIn(path) {
            return Collection.isEmptyPath(path) ? null != this.contents && (this.contents = null, 
            !0) : !!assertCollection(this.contents) && this.contents.deleteIn(path);
        }
        get(key, keepScalar) {
            return identity.isCollection(this.contents) ? this.contents.get(key, keepScalar) : void 0;
        }
        getIn(path, keepScalar) {
            return Collection.isEmptyPath(path) ? !keepScalar && identity.isScalar(this.contents) ? this.contents.value : this.contents : identity.isCollection(this.contents) ? this.contents.getIn(path, keepScalar) : void 0;
        }
        has(key) {
            return !!identity.isCollection(this.contents) && this.contents.has(key);
        }
        hasIn(path) {
            return Collection.isEmptyPath(path) ? void 0 !== this.contents : !!identity.isCollection(this.contents) && this.contents.hasIn(path);
        }
        set(key, value) {
            null == this.contents ? this.contents = Collection.collectionFromPath(this.schema, [ key ], value) : assertCollection(this.contents) && this.contents.set(key, value);
        }
        setIn(path, value) {
            Collection.isEmptyPath(path) ? this.contents = value : null == this.contents ? this.contents = Collection.collectionFromPath(this.schema, Array.from(path), value) : assertCollection(this.contents) && this.contents.setIn(path, value);
        }
        setSchema(version, options = {}) {
            let opt;
            switch ("number" == typeof version && (version = String(version)), version) {
              case "1.1":
                this.directives ? this.directives.yaml.version = "1.1" : this.directives = new directives.Directives({
                    version: "1.1"
                }), opt = {
                    resolveKnownTags: !1,
                    schema: "yaml-1.1"
                };
                break;

              case "1.2":
              case "next":
                this.directives ? this.directives.yaml.version = version : this.directives = new directives.Directives({
                    version: version
                }), opt = {
                    resolveKnownTags: !0,
                    schema: "core"
                };
                break;

              case null:
                this.directives && delete this.directives, opt = null;
                break;

              default:
                {
                    const sv = JSON.stringify(version);
                    throw new Error(`Expected '1.1', '1.2' or null as first argument, but found: ${sv}`);
                }
            }
            if (options.schema instanceof Object) this.schema = options.schema; else {
                if (!opt) throw new Error("With a null YAML version, the { schema: Schema } option is required");
                this.schema = new Schema.Schema(Object.assign(opt, options));
            }
        }
        toJS({json: json, jsonArg: jsonArg, mapAsMap: mapAsMap, maxAliasCount: maxAliasCount, onAnchor: onAnchor, reviver: reviver} = {}) {
            const ctx = {
                anchors: new Map,
                doc: this,
                keep: !json,
                mapAsMap: !0 === mapAsMap,
                mapKeyWarned: !1,
                maxAliasCount: "number" == typeof maxAliasCount ? maxAliasCount : 100
            }, res = toJS.toJS(this.contents, jsonArg ?? "", ctx);
            if ("function" == typeof onAnchor) for (const {count: count, res: res} of ctx.anchors.values()) onAnchor(res, count);
            return "function" == typeof reviver ? applyReviver.applyReviver(reviver, {
                "": res
            }, "", res) : res;
        }
        toJSON(jsonArg, onAnchor) {
            return this.toJS({
                json: !0,
                jsonArg: jsonArg,
                mapAsMap: !1,
                onAnchor: onAnchor
            });
        }
        toString(options = {}) {
            if (this.errors.length > 0) throw new Error("Document with errors cannot be stringified");
            if ("indent" in options && (!Number.isInteger(options.indent) || Number(options.indent) <= 0)) {
                const s = JSON.stringify(options.indent);
                throw new Error(`"indent" option must be a positive integer, not ${s}`);
            }
            return stringifyDocument.stringifyDocument(this, options);
        }
    }, Document;
}

var errors = {}, hasRequiredErrors;

function requireErrors() {
    if (hasRequiredErrors) return errors;
    hasRequiredErrors = 1;
    class YAMLError extends Error {
        constructor(name, pos, code, message) {
            super(), this.name = name, this.code = code, this.message = message, this.pos = pos;
        }
    }
    return errors.YAMLError = YAMLError, errors.YAMLParseError = class extends YAMLError {
        constructor(pos, code, message) {
            super("YAMLParseError", pos, code, message);
        }
    }, errors.YAMLWarning = class extends YAMLError {
        constructor(pos, code, message) {
            super("YAMLWarning", pos, code, message);
        }
    }, errors.prettifyError = (src, lc) => error => {
        if (-1 === error.pos[0]) return;
        error.linePos = error.pos.map((pos => lc.linePos(pos)));
        const {line: line, col: col} = error.linePos[0];
        error.message += ` at line ${line}, column ${col}`;
        let ci = col - 1, lineStr = src.substring(lc.lineStarts[line - 1], lc.lineStarts[line]).replace(/[\n\r]+$/, "");
        if (ci >= 60 && lineStr.length > 80) {
            const trimStart = Math.min(ci - 39, lineStr.length - 79);
            lineStr = "…" + lineStr.substring(trimStart), ci -= trimStart - 1;
        }
        if (lineStr.length > 80 && (lineStr = lineStr.substring(0, 79) + "…"), line > 1 && /^ *$/.test(lineStr.substring(0, ci))) {
            let prev = src.substring(lc.lineStarts[line - 2], lc.lineStarts[line - 1]);
            prev.length > 80 && (prev = prev.substring(0, 79) + "…\n"), lineStr = prev + lineStr;
        }
        if (/[^ ]/.test(lineStr)) {
            let count = 1;
            const end = error.linePos[1];
            end && end.line === line && end.col > col && (count = Math.max(1, Math.min(end.col - col, 80 - ci)));
            const pointer = " ".repeat(ci) + "^".repeat(count);
            error.message += `:\n\n${lineStr}\n${pointer}\n`;
        }
    }, errors;
}

var composeDoc = {}, composeNode = {}, composeCollection = {}, resolveBlockMap = {}, resolveProps = {}, hasRequiredResolveProps;

function requireResolveProps() {
    if (hasRequiredResolveProps) return resolveProps;
    return hasRequiredResolveProps = 1, resolveProps.resolveProps = function(tokens, {flow: flow, indicator: indicator, next: next, offset: offset, onError: onError, parentIndent: parentIndent, startOnNewline: startOnNewline}) {
        let spaceBefore = !1, atNewline = startOnNewline, hasSpace = startOnNewline, comment = "", commentSep = "", hasNewline = !1, reqSpace = !1, tab = null, anchor = null, tag = null, newlineAfterProp = null, comma = null, found = null, start = null;
        for (const token of tokens) switch (reqSpace && ("space" !== token.type && "newline" !== token.type && "comma" !== token.type && onError(token.offset, "MISSING_CHAR", "Tags and anchors must be separated from the next token by white space"), 
        reqSpace = !1), tab && (atNewline && "comment" !== token.type && "newline" !== token.type && onError(tab, "TAB_AS_INDENT", "Tabs are not allowed as indentation"), 
        tab = null), token.type) {
          case "space":
            flow || "doc-start" === indicator && "flow-collection" === next?.type || !token.source.includes("\t") || (tab = token), 
            hasSpace = !0;
            break;

          case "comment":
            {
                hasSpace || onError(token, "MISSING_CHAR", "Comments must be separated from other tokens by white space characters");
                const cb = token.source.substring(1) || " ";
                comment ? comment += commentSep + cb : comment = cb, commentSep = "", atNewline = !1;
                break;
            }

          case "newline":
            atNewline ? comment ? comment += token.source : spaceBefore = !0 : commentSep += token.source, 
            atNewline = !0, hasNewline = !0, (anchor || tag) && (newlineAfterProp = token), 
            hasSpace = !0;
            break;

          case "anchor":
            anchor && onError(token, "MULTIPLE_ANCHORS", "A node can have at most one anchor"), 
            token.source.endsWith(":") && onError(token.offset + token.source.length - 1, "BAD_ALIAS", "Anchor ending in : is ambiguous", !0), 
            anchor = token, null === start && (start = token.offset), atNewline = !1, hasSpace = !1, 
            reqSpace = !0;
            break;

          case "tag":
            tag && onError(token, "MULTIPLE_TAGS", "A node can have at most one tag"), tag = token, 
            null === start && (start = token.offset), atNewline = !1, hasSpace = !1, reqSpace = !0;
            break;

          case indicator:
            (anchor || tag) && onError(token, "BAD_PROP_ORDER", `Anchors and tags must be after the ${token.source} indicator`), 
            found && onError(token, "UNEXPECTED_TOKEN", `Unexpected ${token.source} in ${flow ?? "collection"}`), 
            found = token, atNewline = "seq-item-ind" === indicator || "explicit-key-ind" === indicator, 
            hasSpace = !1;
            break;

          case "comma":
            if (flow) {
                comma && onError(token, "UNEXPECTED_TOKEN", `Unexpected , in ${flow}`), comma = token, 
                atNewline = !1, hasSpace = !1;
                break;
            }

          default:
            onError(token, "UNEXPECTED_TOKEN", `Unexpected ${token.type} token`), atNewline = !1, 
            hasSpace = !1;
        }
        const last = tokens[tokens.length - 1], end = last ? last.offset + last.source.length : offset;
        return reqSpace && next && "space" !== next.type && "newline" !== next.type && "comma" !== next.type && ("scalar" !== next.type || "" !== next.source) && onError(next.offset, "MISSING_CHAR", "Tags and anchors must be separated from the next token by white space"), 
        tab && (atNewline && tab.indent <= parentIndent || "block-map" === next?.type || "block-seq" === next?.type) && onError(tab, "TAB_AS_INDENT", "Tabs are not allowed as indentation"), 
        {
            comma: comma,
            found: found,
            spaceBefore: spaceBefore,
            comment: comment,
            hasNewline: hasNewline,
            anchor: anchor,
            tag: tag,
            newlineAfterProp: newlineAfterProp,
            end: end,
            start: start ?? end
        };
    }, resolveProps;
}

var utilContainsNewline = {}, hasRequiredUtilContainsNewline;

function requireUtilContainsNewline() {
    if (hasRequiredUtilContainsNewline) return utilContainsNewline;
    return hasRequiredUtilContainsNewline = 1, utilContainsNewline.containsNewline = function containsNewline(key) {
        if (!key) return null;
        switch (key.type) {
          case "alias":
          case "scalar":
          case "double-quoted-scalar":
          case "single-quoted-scalar":
            if (key.source.includes("\n")) return !0;
            if (key.end) for (const st of key.end) if ("newline" === st.type) return !0;
            return !1;

          case "flow-collection":
            for (const it of key.items) {
                for (const st of it.start) if ("newline" === st.type) return !0;
                if (it.sep) for (const st of it.sep) if ("newline" === st.type) return !0;
                if (containsNewline(it.key) || containsNewline(it.value)) return !0;
            }
            return !1;

          default:
            return !0;
        }
    }, utilContainsNewline;
}

var utilFlowIndentCheck = {}, hasRequiredUtilFlowIndentCheck;

function requireUtilFlowIndentCheck() {
    if (hasRequiredUtilFlowIndentCheck) return utilFlowIndentCheck;
    hasRequiredUtilFlowIndentCheck = 1;
    var utilContainsNewline = requireUtilContainsNewline();
    return utilFlowIndentCheck.flowIndentCheck = function(indent, fc, onError) {
        if ("flow-collection" === fc?.type) {
            const end = fc.end[0];
            if (end.indent === indent && ("]" === end.source || "}" === end.source) && utilContainsNewline.containsNewline(fc)) {
                onError(end, "BAD_INDENT", "Flow end indicator should be more indented than parent", !0);
            }
        }
    }, utilFlowIndentCheck;
}

var utilMapIncludes = {}, hasRequiredUtilMapIncludes, hasRequiredResolveBlockMap;

function requireUtilMapIncludes() {
    if (hasRequiredUtilMapIncludes) return utilMapIncludes;
    hasRequiredUtilMapIncludes = 1;
    var identity = requireIdentity();
    return utilMapIncludes.mapIncludes = function(ctx, items, search) {
        const {uniqueKeys: uniqueKeys} = ctx.options;
        if (!1 === uniqueKeys) return !1;
        const isEqual = "function" == typeof uniqueKeys ? uniqueKeys : (a, b) => a === b || identity.isScalar(a) && identity.isScalar(b) && a.value === b.value;
        return items.some((pair => isEqual(pair.key, search)));
    }, utilMapIncludes;
}

function requireResolveBlockMap() {
    if (hasRequiredResolveBlockMap) return resolveBlockMap;
    hasRequiredResolveBlockMap = 1;
    var Pair = requirePair(), YAMLMap = requireYAMLMap(), resolveProps = requireResolveProps(), utilContainsNewline = requireUtilContainsNewline(), utilFlowIndentCheck = requireUtilFlowIndentCheck(), utilMapIncludes = requireUtilMapIncludes();
    const startColMsg = "All mapping items must start at the same column";
    return resolveBlockMap.resolveBlockMap = function({composeNode: composeNode, composeEmptyNode: composeEmptyNode}, ctx, bm, onError, tag) {
        const map = new (tag?.nodeClass ?? YAMLMap.YAMLMap)(ctx.schema);
        ctx.atRoot && (ctx.atRoot = !1);
        let offset = bm.offset, commentEnd = null;
        for (const collItem of bm.items) {
            const {start: start, key: key, sep: sep, value: value} = collItem, keyProps = resolveProps.resolveProps(start, {
                indicator: "explicit-key-ind",
                next: key ?? sep?.[0],
                offset: offset,
                onError: onError,
                parentIndent: bm.indent,
                startOnNewline: !0
            }), implicitKey = !keyProps.found;
            if (implicitKey) {
                if (key && ("block-seq" === key.type ? onError(offset, "BLOCK_AS_IMPLICIT_KEY", "A block sequence may not be used as an implicit map key") : "indent" in key && key.indent !== bm.indent && onError(offset, "BAD_INDENT", startColMsg)), 
                !keyProps.anchor && !keyProps.tag && !sep) {
                    commentEnd = keyProps.end, keyProps.comment && (map.comment ? map.comment += "\n" + keyProps.comment : map.comment = keyProps.comment);
                    continue;
                }
                (keyProps.newlineAfterProp || utilContainsNewline.containsNewline(key)) && onError(key ?? start[start.length - 1], "MULTILINE_IMPLICIT_KEY", "Implicit keys need to be on a single line");
            } else keyProps.found?.indent !== bm.indent && onError(offset, "BAD_INDENT", startColMsg);
            ctx.atKey = !0;
            const keyStart = keyProps.end, keyNode = key ? composeNode(ctx, key, keyProps, onError) : composeEmptyNode(ctx, keyStart, start, null, keyProps, onError);
            ctx.schema.compat && utilFlowIndentCheck.flowIndentCheck(bm.indent, key, onError), 
            ctx.atKey = !1, utilMapIncludes.mapIncludes(ctx, map.items, keyNode) && onError(keyStart, "DUPLICATE_KEY", "Map keys must be unique");
            const valueProps = resolveProps.resolveProps(sep ?? [], {
                indicator: "map-value-ind",
                next: value,
                offset: keyNode.range[2],
                onError: onError,
                parentIndent: bm.indent,
                startOnNewline: !key || "block-scalar" === key.type
            });
            if (offset = valueProps.end, valueProps.found) {
                implicitKey && ("block-map" !== value?.type || valueProps.hasNewline || onError(offset, "BLOCK_AS_IMPLICIT_KEY", "Nested mappings are not allowed in compact mappings"), 
                ctx.options.strict && keyProps.start < valueProps.found.offset - 1024 && onError(keyNode.range, "KEY_OVER_1024_CHARS", "The : indicator must be at most 1024 chars after the start of an implicit block mapping key"));
                const valueNode = value ? composeNode(ctx, value, valueProps, onError) : composeEmptyNode(ctx, offset, sep, null, valueProps, onError);
                ctx.schema.compat && utilFlowIndentCheck.flowIndentCheck(bm.indent, value, onError), 
                offset = valueNode.range[2];
                const pair = new Pair.Pair(keyNode, valueNode);
                ctx.options.keepSourceTokens && (pair.srcToken = collItem), map.items.push(pair);
            } else {
                implicitKey && onError(keyNode.range, "MISSING_CHAR", "Implicit map keys need to be followed by map values"), 
                valueProps.comment && (keyNode.comment ? keyNode.comment += "\n" + valueProps.comment : keyNode.comment = valueProps.comment);
                const pair = new Pair.Pair(keyNode);
                ctx.options.keepSourceTokens && (pair.srcToken = collItem), map.items.push(pair);
            }
        }
        return commentEnd && commentEnd < offset && onError(commentEnd, "IMPOSSIBLE", "Map comment with trailing content"), 
        map.range = [ bm.offset, offset, commentEnd ?? offset ], map;
    }, resolveBlockMap;
}

var resolveBlockSeq = {}, hasRequiredResolveBlockSeq;

function requireResolveBlockSeq() {
    if (hasRequiredResolveBlockSeq) return resolveBlockSeq;
    hasRequiredResolveBlockSeq = 1;
    var YAMLSeq = requireYAMLSeq(), resolveProps = requireResolveProps(), utilFlowIndentCheck = requireUtilFlowIndentCheck();
    return resolveBlockSeq.resolveBlockSeq = function({composeNode: composeNode, composeEmptyNode: composeEmptyNode}, ctx, bs, onError, tag) {
        const seq = new (tag?.nodeClass ?? YAMLSeq.YAMLSeq)(ctx.schema);
        ctx.atRoot && (ctx.atRoot = !1), ctx.atKey && (ctx.atKey = !1);
        let offset = bs.offset, commentEnd = null;
        for (const {start: start, value: value} of bs.items) {
            const props = resolveProps.resolveProps(start, {
                indicator: "seq-item-ind",
                next: value,
                offset: offset,
                onError: onError,
                parentIndent: bs.indent,
                startOnNewline: !0
            });
            if (!props.found) {
                if (!(props.anchor || props.tag || value)) {
                    commentEnd = props.end, props.comment && (seq.comment = props.comment);
                    continue;
                }
                value && "block-seq" === value.type ? onError(props.end, "BAD_INDENT", "All sequence items must start at the same column") : onError(offset, "MISSING_CHAR", "Sequence item without - indicator");
            }
            const node = value ? composeNode(ctx, value, props, onError) : composeEmptyNode(ctx, props.end, start, null, props, onError);
            ctx.schema.compat && utilFlowIndentCheck.flowIndentCheck(bs.indent, value, onError), 
            offset = node.range[2], seq.items.push(node);
        }
        return seq.range = [ bs.offset, offset, commentEnd ?? offset ], seq;
    }, resolveBlockSeq;
}

var resolveFlowCollection = {}, resolveEnd = {}, hasRequiredResolveEnd, hasRequiredResolveFlowCollection, hasRequiredComposeCollection;

function requireResolveEnd() {
    if (hasRequiredResolveEnd) return resolveEnd;
    return hasRequiredResolveEnd = 1, resolveEnd.resolveEnd = function(end, offset, reqSpace, onError) {
        let comment = "";
        if (end) {
            let hasSpace = !1, sep = "";
            for (const token of end) {
                const {source: source, type: type} = token;
                switch (type) {
                  case "space":
                    hasSpace = !0;
                    break;

                  case "comment":
                    {
                        reqSpace && !hasSpace && onError(token, "MISSING_CHAR", "Comments must be separated from other tokens by white space characters");
                        const cb = source.substring(1) || " ";
                        comment ? comment += sep + cb : comment = cb, sep = "";
                        break;
                    }

                  case "newline":
                    comment && (sep += source), hasSpace = !0;
                    break;

                  default:
                    onError(token, "UNEXPECTED_TOKEN", `Unexpected ${type} at node end`);
                }
                offset += source.length;
            }
        }
        return {
            comment: comment,
            offset: offset
        };
    }, resolveEnd;
}

function requireResolveFlowCollection() {
    if (hasRequiredResolveFlowCollection) return resolveFlowCollection;
    hasRequiredResolveFlowCollection = 1;
    var identity = requireIdentity(), Pair = requirePair(), YAMLMap = requireYAMLMap(), YAMLSeq = requireYAMLSeq(), resolveEnd = requireResolveEnd(), resolveProps = requireResolveProps(), utilContainsNewline = requireUtilContainsNewline(), utilMapIncludes = requireUtilMapIncludes();
    const blockMsg = "Block collections are not allowed within flow collections", isBlock = token => token && ("block-map" === token.type || "block-seq" === token.type);
    return resolveFlowCollection.resolveFlowCollection = function({composeNode: composeNode, composeEmptyNode: composeEmptyNode}, ctx, fc, onError, tag) {
        const isMap = "{" === fc.start.source, fcName = isMap ? "flow map" : "flow sequence", coll = new (tag?.nodeClass ?? (isMap ? YAMLMap.YAMLMap : YAMLSeq.YAMLSeq))(ctx.schema);
        coll.flow = !0;
        const atRoot = ctx.atRoot;
        atRoot && (ctx.atRoot = !1), ctx.atKey && (ctx.atKey = !1);
        let offset = fc.offset + fc.start.source.length;
        for (let i = 0; i < fc.items.length; ++i) {
            const collItem = fc.items[i], {start: start, key: key, sep: sep, value: value} = collItem, props = resolveProps.resolveProps(start, {
                flow: fcName,
                indicator: "explicit-key-ind",
                next: key ?? sep?.[0],
                offset: offset,
                onError: onError,
                parentIndent: fc.indent,
                startOnNewline: !1
            });
            if (!props.found) {
                if (!(props.anchor || props.tag || sep || value)) {
                    0 === i && props.comma ? onError(props.comma, "UNEXPECTED_TOKEN", `Unexpected , in ${fcName}`) : i < fc.items.length - 1 && onError(props.start, "UNEXPECTED_TOKEN", `Unexpected empty item in ${fcName}`), 
                    props.comment && (coll.comment ? coll.comment += "\n" + props.comment : coll.comment = props.comment), 
                    offset = props.end;
                    continue;
                }
                !isMap && ctx.options.strict && utilContainsNewline.containsNewline(key) && onError(key, "MULTILINE_IMPLICIT_KEY", "Implicit keys of flow sequence pairs need to be on a single line");
            }
            if (0 === i) props.comma && onError(props.comma, "UNEXPECTED_TOKEN", `Unexpected , in ${fcName}`); else if (props.comma || onError(props.start, "MISSING_CHAR", `Missing , between ${fcName} items`), 
            props.comment) {
                let prevItemComment = "";
                loop: for (const st of start) switch (st.type) {
                  case "comma":
                  case "space":
                    break;

                  case "comment":
                    prevItemComment = st.source.substring(1);
                    break loop;

                  default:
                    break loop;
                }
                if (prevItemComment) {
                    let prev = coll.items[coll.items.length - 1];
                    identity.isPair(prev) && (prev = prev.value ?? prev.key), prev.comment ? prev.comment += "\n" + prevItemComment : prev.comment = prevItemComment, 
                    props.comment = props.comment.substring(prevItemComment.length + 1);
                }
            }
            if (isMap || sep || props.found) {
                ctx.atKey = !0;
                const keyStart = props.end, keyNode = key ? composeNode(ctx, key, props, onError) : composeEmptyNode(ctx, keyStart, start, null, props, onError);
                isBlock(key) && onError(keyNode.range, "BLOCK_IN_FLOW", blockMsg), ctx.atKey = !1;
                const valueProps = resolveProps.resolveProps(sep ?? [], {
                    flow: fcName,
                    indicator: "map-value-ind",
                    next: value,
                    offset: keyNode.range[2],
                    onError: onError,
                    parentIndent: fc.indent,
                    startOnNewline: !1
                });
                if (valueProps.found) {
                    if (!isMap && !props.found && ctx.options.strict) {
                        if (sep) for (const st of sep) {
                            if (st === valueProps.found) break;
                            if ("newline" === st.type) {
                                onError(st, "MULTILINE_IMPLICIT_KEY", "Implicit keys of flow sequence pairs need to be on a single line");
                                break;
                            }
                        }
                        props.start < valueProps.found.offset - 1024 && onError(valueProps.found, "KEY_OVER_1024_CHARS", "The : indicator must be at most 1024 chars after the start of an implicit flow sequence key");
                    }
                } else value && ("source" in value && value.source && ":" === value.source[0] ? onError(value, "MISSING_CHAR", `Missing space after : in ${fcName}`) : onError(valueProps.start, "MISSING_CHAR", `Missing , or : between ${fcName} items`));
                const valueNode = value ? composeNode(ctx, value, valueProps, onError) : valueProps.found ? composeEmptyNode(ctx, valueProps.end, sep, null, valueProps, onError) : null;
                valueNode ? isBlock(value) && onError(valueNode.range, "BLOCK_IN_FLOW", blockMsg) : valueProps.comment && (keyNode.comment ? keyNode.comment += "\n" + valueProps.comment : keyNode.comment = valueProps.comment);
                const pair = new Pair.Pair(keyNode, valueNode);
                if (ctx.options.keepSourceTokens && (pair.srcToken = collItem), isMap) {
                    const map = coll;
                    utilMapIncludes.mapIncludes(ctx, map.items, keyNode) && onError(keyStart, "DUPLICATE_KEY", "Map keys must be unique"), 
                    map.items.push(pair);
                } else {
                    const map = new YAMLMap.YAMLMap(ctx.schema);
                    map.flow = !0, map.items.push(pair);
                    const endRange = (valueNode ?? keyNode).range;
                    map.range = [ keyNode.range[0], endRange[1], endRange[2] ], coll.items.push(map);
                }
                offset = valueNode ? valueNode.range[2] : valueProps.end;
            } else {
                const valueNode = value ? composeNode(ctx, value, props, onError) : composeEmptyNode(ctx, props.end, sep, null, props, onError);
                coll.items.push(valueNode), offset = valueNode.range[2], isBlock(value) && onError(valueNode.range, "BLOCK_IN_FLOW", blockMsg);
            }
        }
        const expectedEnd = isMap ? "}" : "]", [ce, ...ee] = fc.end;
        let cePos = offset;
        if (ce && ce.source === expectedEnd) cePos = ce.offset + ce.source.length; else {
            const name = fcName[0].toUpperCase() + fcName.substring(1);
            onError(offset, atRoot ? "MISSING_CHAR" : "BAD_INDENT", atRoot ? `${name} must end with a ${expectedEnd}` : `${name} in block collection must be sufficiently indented and end with a ${expectedEnd}`), 
            ce && 1 !== ce.source.length && ee.unshift(ce);
        }
        if (ee.length > 0) {
            const end = resolveEnd.resolveEnd(ee, cePos, ctx.options.strict, onError);
            end.comment && (coll.comment ? coll.comment += "\n" + end.comment : coll.comment = end.comment), 
            coll.range = [ fc.offset, cePos, end.offset ];
        } else coll.range = [ fc.offset, cePos, cePos ];
        return coll;
    }, resolveFlowCollection;
}

function requireComposeCollection() {
    if (hasRequiredComposeCollection) return composeCollection;
    hasRequiredComposeCollection = 1;
    var identity = requireIdentity(), Scalar = requireScalar(), YAMLMap = requireYAMLMap(), YAMLSeq = requireYAMLSeq(), resolveBlockMap = requireResolveBlockMap(), resolveBlockSeq = requireResolveBlockSeq(), resolveFlowCollection = requireResolveFlowCollection();
    function resolveCollection(CN, ctx, token, onError, tagName, tag) {
        const coll = "block-map" === token.type ? resolveBlockMap.resolveBlockMap(CN, ctx, token, onError, tag) : "block-seq" === token.type ? resolveBlockSeq.resolveBlockSeq(CN, ctx, token, onError, tag) : resolveFlowCollection.resolveFlowCollection(CN, ctx, token, onError, tag), Coll = coll.constructor;
        return "!" === tagName || tagName === Coll.tagName ? (coll.tag = Coll.tagName, coll) : (tagName && (coll.tag = tagName), 
        coll);
    }
    return composeCollection.composeCollection = function(CN, ctx, token, props, onError) {
        const tagToken = props.tag, tagName = tagToken ? ctx.directives.tagName(tagToken.source, (msg => onError(tagToken, "TAG_RESOLVE_FAILED", msg))) : null;
        if ("block-seq" === token.type) {
            const {anchor: anchor, newlineAfterProp: nl} = props, lastProp = anchor && tagToken ? anchor.offset > tagToken.offset ? anchor : tagToken : anchor ?? tagToken;
            if (lastProp && (!nl || nl.offset < lastProp.offset)) {
                onError(lastProp, "MISSING_CHAR", "Missing newline after block sequence props");
            }
        }
        const expType = "block-map" === token.type ? "map" : "block-seq" === token.type ? "seq" : "{" === token.start.source ? "map" : "seq";
        if (!tagToken || !tagName || "!" === tagName || tagName === YAMLMap.YAMLMap.tagName && "map" === expType || tagName === YAMLSeq.YAMLSeq.tagName && "seq" === expType) return resolveCollection(CN, ctx, token, onError, tagName);
        let tag = ctx.schema.tags.find((t => t.tag === tagName && t.collection === expType));
        if (!tag) {
            const kt = ctx.schema.knownTags[tagName];
            if (!kt || kt.collection !== expType) return kt?.collection ? onError(tagToken, "BAD_COLLECTION_TYPE", `${kt.tag} used for ${expType} collection, but expects ${kt.collection}`, !0) : onError(tagToken, "TAG_RESOLVE_FAILED", `Unresolved tag: ${tagName}`, !0), 
            resolveCollection(CN, ctx, token, onError, tagName);
            ctx.schema.tags.push(Object.assign({}, kt, {
                default: !1
            })), tag = kt;
        }
        const coll = resolveCollection(CN, ctx, token, onError, tagName, tag), res = tag.resolve?.(coll, (msg => onError(tagToken, "TAG_RESOLVE_FAILED", msg)), ctx.options) ?? coll, node = identity.isNode(res) ? res : new Scalar.Scalar(res);
        return node.range = coll.range, node.tag = tagName, tag?.format && (node.format = tag.format), 
        node;
    }, composeCollection;
}

var composeScalar = {}, resolveBlockScalar = {}, hasRequiredResolveBlockScalar;

function requireResolveBlockScalar() {
    if (hasRequiredResolveBlockScalar) return resolveBlockScalar;
    hasRequiredResolveBlockScalar = 1;
    var Scalar = requireScalar();
    return resolveBlockScalar.resolveBlockScalar = function(ctx, scalar, onError) {
        const start = scalar.offset, header = function({offset: offset, props: props}, strict, onError) {
            if ("block-scalar-header" !== props[0].type) return onError(props[0], "IMPOSSIBLE", "Block scalar header not found"), 
            null;
            const {source: source} = props[0], mode = source[0];
            let indent = 0, chomp = "", error = -1;
            for (let i = 1; i < source.length; ++i) {
                const ch = source[i];
                if (chomp || "-" !== ch && "+" !== ch) {
                    const n = Number(ch);
                    !indent && n ? indent = n : -1 === error && (error = offset + i);
                } else chomp = ch;
            }
            -1 !== error && onError(error, "UNEXPECTED_TOKEN", `Block scalar header includes extra characters: ${source}`);
            let hasSpace = !1, comment = "", length = source.length;
            for (let i = 1; i < props.length; ++i) {
                const token = props[i];
                switch (token.type) {
                  case "space":
                    hasSpace = !0;

                  case "newline":
                    length += token.source.length;
                    break;

                  case "comment":
                    if (strict && !hasSpace) {
                        onError(token, "MISSING_CHAR", "Comments must be separated from other tokens by white space characters");
                    }
                    length += token.source.length, comment = token.source.substring(1);
                    break;

                  case "error":
                    onError(token, "UNEXPECTED_TOKEN", token.message), length += token.source.length;
                    break;

                  default:
                    {
                        onError(token, "UNEXPECTED_TOKEN", `Unexpected token in block scalar header: ${token.type}`);
                        const ts = token.source;
                        ts && "string" == typeof ts && (length += ts.length);
                    }
                }
            }
            return {
                mode: mode,
                indent: indent,
                chomp: chomp,
                comment: comment,
                length: length
            };
        }(scalar, ctx.options.strict, onError);
        if (!header) return {
            value: "",
            type: null,
            comment: "",
            range: [ start, start, start ]
        };
        const type = ">" === header.mode ? Scalar.Scalar.BLOCK_FOLDED : Scalar.Scalar.BLOCK_LITERAL, lines = scalar.source ? function(source) {
            const split = source.split(/\n( *)/), first = split[0], m = first.match(/^( *)/), lines = [ m?.[1] ? [ m[1], first.slice(m[1].length) ] : [ "", first ] ];
            for (let i = 1; i < split.length; i += 2) lines.push([ split[i], split[i + 1] ]);
            return lines;
        }(scalar.source) : [];
        let chompStart = lines.length;
        for (let i = lines.length - 1; i >= 0; --i) {
            const content = lines[i][1];
            if ("" !== content && "\r" !== content) break;
            chompStart = i;
        }
        if (0 === chompStart) {
            const value = "+" === header.chomp && lines.length > 0 ? "\n".repeat(Math.max(1, lines.length - 1)) : "";
            let end = start + header.length;
            return scalar.source && (end += scalar.source.length), {
                value: value,
                type: type,
                comment: header.comment,
                range: [ start, end, end ]
            };
        }
        let trimIndent = scalar.indent + header.indent, offset = scalar.offset + header.length, contentStart = 0;
        for (let i = 0; i < chompStart; ++i) {
            const [indent, content] = lines[i];
            if ("" !== content && "\r" !== content) {
                if (indent.length < trimIndent) {
                    const message = "Block scalars with more-indented leading empty lines must use an explicit indentation indicator";
                    onError(offset + indent.length, "MISSING_CHAR", message);
                }
                if (0 === header.indent && (trimIndent = indent.length), contentStart = i, 0 === trimIndent && !ctx.atRoot) {
                    onError(offset, "BAD_INDENT", "Block scalar values in collections must be indented");
                }
                break;
            }
            0 === header.indent && indent.length > trimIndent && (trimIndent = indent.length), 
            offset += indent.length + content.length + 1;
        }
        for (let i = lines.length - 1; i >= chompStart; --i) lines[i][0].length > trimIndent && (chompStart = i + 1);
        let value = "", sep = "", prevMoreIndented = !1;
        for (let i = 0; i < contentStart; ++i) value += lines[i][0].slice(trimIndent) + "\n";
        for (let i = contentStart; i < chompStart; ++i) {
            let [indent, content] = lines[i];
            offset += indent.length + content.length + 1;
            const crlf = "\r" === content[content.length - 1];
            if (crlf && (content = content.slice(0, -1)), content && indent.length < trimIndent) {
                const message = `Block scalar lines must not be less indented than their ${header.indent ? "explicit indentation indicator" : "first line"}`;
                onError(offset - content.length - (crlf ? 2 : 1), "BAD_INDENT", message), indent = "";
            }
            type === Scalar.Scalar.BLOCK_LITERAL ? (value += sep + indent.slice(trimIndent) + content, 
            sep = "\n") : indent.length > trimIndent || "\t" === content[0] ? (" " === sep ? sep = "\n" : prevMoreIndented || "\n" !== sep || (sep = "\n\n"), 
            value += sep + indent.slice(trimIndent) + content, sep = "\n", prevMoreIndented = !0) : "" === content ? "\n" === sep ? value += "\n" : sep = "\n" : (value += sep + content, 
            sep = " ", prevMoreIndented = !1);
        }
        switch (header.chomp) {
          case "-":
            break;

          case "+":
            for (let i = chompStart; i < lines.length; ++i) value += "\n" + lines[i][0].slice(trimIndent);
            "\n" !== value[value.length - 1] && (value += "\n");
            break;

          default:
            value += "\n";
        }
        const end = start + header.length + scalar.source.length;
        return {
            value: value,
            type: type,
            comment: header.comment,
            range: [ start, end, end ]
        };
    }, resolveBlockScalar;
}

var resolveFlowScalar = {}, hasRequiredResolveFlowScalar, hasRequiredComposeScalar;

function requireResolveFlowScalar() {
    if (hasRequiredResolveFlowScalar) return resolveFlowScalar;
    hasRequiredResolveFlowScalar = 1;
    var Scalar = requireScalar(), resolveEnd = requireResolveEnd();
    function foldLines(source) {
        let first, line;
        try {
            first = new RegExp("(.*?)(?<![ \t])[ \t]*\r?\n", "sy"), line = new RegExp("[ \t]*(.*?)(?:(?<![ \t])[ \t]*)?\r?\n", "sy");
        } catch {
            first = /(.*?)[ \t]*\r?\n/sy, line = /[ \t]*(.*?)[ \t]*\r?\n/sy;
        }
        let match = first.exec(source);
        if (!match) return source;
        let res = match[1], sep = " ", pos = first.lastIndex;
        for (line.lastIndex = pos; match = line.exec(source); ) "" === match[1] ? "\n" === sep ? res += sep : sep = "\n" : (res += sep + match[1], 
        sep = " "), pos = line.lastIndex;
        const last = /[ \t]*(.*)/sy;
        return last.lastIndex = pos, match = last.exec(source), res + sep + (match?.[1] ?? "");
    }
    function foldNewline(source, offset) {
        let fold = "", ch = source[offset + 1];
        for (;!(" " !== ch && "\t" !== ch && "\n" !== ch && "\r" !== ch || "\r" === ch && "\n" !== source[offset + 2]); ) "\n" === ch && (fold += "\n"), 
        ch = source[(offset += 1) + 1];
        return fold || (fold = " "), {
            fold: fold,
            offset: offset
        };
    }
    const escapeCodes = {
        0: "\0",
        a: "",
        b: "\b",
        e: "",
        f: "\f",
        n: "\n",
        r: "\r",
        t: "\t",
        v: "\v",
        N: "",
        _: " ",
        L: "\u2028",
        P: "\u2029",
        " ": " ",
        '"': '"',
        "/": "/",
        "\\": "\\",
        "\t": "\t"
    };
    function parseCharCode(source, offset, length, onError) {
        const cc = source.substr(offset, length), code = cc.length === length && /^[0-9a-fA-F]+$/.test(cc) ? parseInt(cc, 16) : NaN;
        if (isNaN(code)) {
            const raw = source.substr(offset - 2, length + 2);
            return onError(offset - 2, "BAD_DQ_ESCAPE", `Invalid escape sequence ${raw}`), raw;
        }
        return String.fromCodePoint(code);
    }
    return resolveFlowScalar.resolveFlowScalar = function(scalar, strict, onError) {
        const {offset: offset, type: type, source: source, end: end} = scalar;
        let _type, value;
        const _onError = (rel, code, msg) => onError(offset + rel, code, msg);
        switch (type) {
          case "scalar":
            _type = Scalar.Scalar.PLAIN, value = function(source, onError) {
                let badChar = "";
                switch (source[0]) {
                  case "\t":
                    badChar = "a tab character";
                    break;

                  case ",":
                    badChar = "flow indicator character ,";
                    break;

                  case "%":
                    badChar = "directive indicator character %";
                    break;

                  case "|":
                  case ">":
                    badChar = `block scalar indicator ${source[0]}`;
                    break;

                  case "@":
                  case "`":
                    badChar = `reserved character ${source[0]}`;
                }
                badChar && onError(0, "BAD_SCALAR_START", `Plain value cannot start with ${badChar}`);
                return foldLines(source);
            }(source, _onError);
            break;

          case "single-quoted-scalar":
            _type = Scalar.Scalar.QUOTE_SINGLE, value = function(source, onError) {
                "'" === source[source.length - 1] && 1 !== source.length || onError(source.length, "MISSING_CHAR", "Missing closing 'quote");
                return foldLines(source.slice(1, -1)).replace(/''/g, "'");
            }(source, _onError);
            break;

          case "double-quoted-scalar":
            _type = Scalar.Scalar.QUOTE_DOUBLE, value = function(source, onError) {
                let res = "";
                for (let i = 1; i < source.length - 1; ++i) {
                    const ch = source[i];
                    if ("\r" !== ch || "\n" !== source[i + 1]) if ("\n" === ch) {
                        const {fold: fold, offset: offset} = foldNewline(source, i);
                        res += fold, i = offset;
                    } else if ("\\" === ch) {
                        let next = source[++i];
                        const cc = escapeCodes[next];
                        if (cc) res += cc; else if ("\n" === next) for (next = source[i + 1]; " " === next || "\t" === next; ) next = source[1 + ++i]; else if ("\r" === next && "\n" === source[i + 1]) for (next = source[1 + ++i]; " " === next || "\t" === next; ) next = source[1 + ++i]; else if ("x" === next || "u" === next || "U" === next) {
                            const length = {
                                x: 2,
                                u: 4,
                                U: 8
                            }[next];
                            res += parseCharCode(source, i + 1, length, onError), i += length;
                        } else {
                            const raw = source.substr(i - 1, 2);
                            onError(i - 1, "BAD_DQ_ESCAPE", `Invalid escape sequence ${raw}`), res += raw;
                        }
                    } else if (" " === ch || "\t" === ch) {
                        const wsStart = i;
                        let next = source[i + 1];
                        for (;" " === next || "\t" === next; ) next = source[1 + ++i];
                        "\n" === next || "\r" === next && "\n" === source[i + 2] || (res += i > wsStart ? source.slice(wsStart, i + 1) : ch);
                    } else res += ch;
                }
                '"' === source[source.length - 1] && 1 !== source.length || onError(source.length, "MISSING_CHAR", 'Missing closing "quote');
                return res;
            }(source, _onError);
            break;

          default:
            return onError(scalar, "UNEXPECTED_TOKEN", `Expected a flow scalar value, but found: ${type}`), 
            {
                value: "",
                type: null,
                comment: "",
                range: [ offset, offset + source.length, offset + source.length ]
            };
        }
        const valueEnd = offset + source.length, re = resolveEnd.resolveEnd(end, valueEnd, strict, onError);
        return {
            value: value,
            type: _type,
            comment: re.comment,
            range: [ offset, valueEnd, re.offset ]
        };
    }, resolveFlowScalar;
}

function requireComposeScalar() {
    if (hasRequiredComposeScalar) return composeScalar;
    hasRequiredComposeScalar = 1;
    var identity = requireIdentity(), Scalar = requireScalar(), resolveBlockScalar = requireResolveBlockScalar(), resolveFlowScalar = requireResolveFlowScalar();
    return composeScalar.composeScalar = function(ctx, token, tagToken, onError) {
        const {value: value, type: type, comment: comment, range: range} = "block-scalar" === token.type ? resolveBlockScalar.resolveBlockScalar(ctx, token, onError) : resolveFlowScalar.resolveFlowScalar(token, ctx.options.strict, onError), tagName = tagToken ? ctx.directives.tagName(tagToken.source, (msg => onError(tagToken, "TAG_RESOLVE_FAILED", msg))) : null;
        let tag, scalar;
        tag = ctx.options.stringKeys && ctx.atKey ? ctx.schema[identity.SCALAR] : tagName ? function(schema, value, tagName, tagToken, onError) {
            if ("!" === tagName) return schema[identity.SCALAR];
            const matchWithTest = [];
            for (const tag of schema.tags) if (!tag.collection && tag.tag === tagName) {
                if (!tag.default || !tag.test) return tag;
                matchWithTest.push(tag);
            }
            for (const tag of matchWithTest) if (tag.test?.test(value)) return tag;
            const kt = schema.knownTags[tagName];
            if (kt && !kt.collection) return schema.tags.push(Object.assign({}, kt, {
                default: !1,
                test: void 0
            })), kt;
            return onError(tagToken, "TAG_RESOLVE_FAILED", `Unresolved tag: ${tagName}`, "tag:yaml.org,2002:str" !== tagName), 
            schema[identity.SCALAR];
        }(ctx.schema, value, tagName, tagToken, onError) : "scalar" === token.type ? function({atKey: atKey, directives: directives, schema: schema}, value, token, onError) {
            const tag = schema.tags.find((tag => (!0 === tag.default || atKey && "key" === tag.default) && tag.test?.test(value))) || schema[identity.SCALAR];
            if (schema.compat) {
                const compat = schema.compat.find((tag => tag.default && tag.test?.test(value))) ?? schema[identity.SCALAR];
                if (tag.tag !== compat.tag) {
                    onError(token, "TAG_RESOLVE_FAILED", `Value may be parsed as either ${directives.tagString(tag.tag)} or ${directives.tagString(compat.tag)}`, !0);
                }
            }
            return tag;
        }(ctx, value, token, onError) : ctx.schema[identity.SCALAR];
        try {
            const res = tag.resolve(value, (msg => onError(tagToken ?? token, "TAG_RESOLVE_FAILED", msg)), ctx.options);
            scalar = identity.isScalar(res) ? res : new Scalar.Scalar(res);
        } catch (error) {
            const msg = error instanceof Error ? error.message : String(error);
            onError(tagToken ?? token, "TAG_RESOLVE_FAILED", msg), scalar = new Scalar.Scalar(value);
        }
        return scalar.range = range, scalar.source = value, type && (scalar.type = type), 
        tagName && (scalar.tag = tagName), tag.format && (scalar.format = tag.format), comment && (scalar.comment = comment), 
        scalar;
    }, composeScalar;
}

var utilEmptyScalarPosition = {}, hasRequiredUtilEmptyScalarPosition, hasRequiredComposeNode, hasRequiredComposeDoc, hasRequiredComposer;

function requireUtilEmptyScalarPosition() {
    if (hasRequiredUtilEmptyScalarPosition) return utilEmptyScalarPosition;
    return hasRequiredUtilEmptyScalarPosition = 1, utilEmptyScalarPosition.emptyScalarPosition = function(offset, before, pos) {
        if (before) {
            null === pos && (pos = before.length);
            for (let i = pos - 1; i >= 0; --i) {
                let st = before[i];
                switch (st.type) {
                  case "space":
                  case "comment":
                  case "newline":
                    offset -= st.source.length;
                    continue;
                }
                for (st = before[++i]; "space" === st?.type; ) offset += st.source.length, st = before[++i];
                break;
            }
        }
        return offset;
    }, utilEmptyScalarPosition;
}

function requireComposeNode() {
    if (hasRequiredComposeNode) return composeNode;
    hasRequiredComposeNode = 1;
    var Alias = requireAlias(), identity = requireIdentity(), composeCollection = requireComposeCollection(), composeScalar = requireComposeScalar(), resolveEnd = requireResolveEnd(), utilEmptyScalarPosition = requireUtilEmptyScalarPosition();
    const CN = {
        composeNode: composeNode$1,
        composeEmptyNode: composeEmptyNode
    };
    function composeNode$1(ctx, token, props, onError) {
        const atKey = ctx.atKey, {spaceBefore: spaceBefore, comment: comment, anchor: anchor, tag: tag} = props;
        let node, isSrcToken = !0;
        switch (token.type) {
          case "alias":
            node = function({options: options}, {offset: offset, source: source, end: end}, onError) {
                const alias = new Alias.Alias(source.substring(1));
                "" === alias.source && onError(offset, "BAD_ALIAS", "Alias cannot be an empty string");
                alias.source.endsWith(":") && onError(offset + source.length - 1, "BAD_ALIAS", "Alias ending in : is ambiguous", !0);
                const valueEnd = offset + source.length, re = resolveEnd.resolveEnd(end, valueEnd, options.strict, onError);
                alias.range = [ offset, valueEnd, re.offset ], re.comment && (alias.comment = re.comment);
                return alias;
            }(ctx, token, onError), (anchor || tag) && onError(token, "ALIAS_PROPS", "An alias node must not specify any properties");
            break;

          case "scalar":
          case "single-quoted-scalar":
          case "double-quoted-scalar":
          case "block-scalar":
            node = composeScalar.composeScalar(ctx, token, tag, onError), anchor && (node.anchor = anchor.source.substring(1));
            break;

          case "block-map":
          case "block-seq":
          case "flow-collection":
            node = composeCollection.composeCollection(CN, ctx, token, props, onError), anchor && (node.anchor = anchor.source.substring(1));
            break;

          default:
            onError(token, "UNEXPECTED_TOKEN", "error" === token.type ? token.message : `Unsupported token (type: ${token.type})`), 
            node = composeEmptyNode(ctx, token.offset, void 0, null, props, onError), isSrcToken = !1;
        }
        if (anchor && "" === node.anchor && onError(anchor, "BAD_ALIAS", "Anchor cannot be an empty string"), 
        atKey && ctx.options.stringKeys && (!identity.isScalar(node) || "string" != typeof node.value || node.tag && "tag:yaml.org,2002:str" !== node.tag)) {
            onError(tag ?? token, "NON_STRING_KEY", "With stringKeys, all keys must be strings");
        }
        return spaceBefore && (node.spaceBefore = !0), comment && ("scalar" === token.type && "" === token.source ? node.comment = comment : node.commentBefore = comment), 
        ctx.options.keepSourceTokens && isSrcToken && (node.srcToken = token), node;
    }
    function composeEmptyNode(ctx, offset, before, pos, {spaceBefore: spaceBefore, comment: comment, anchor: anchor, tag: tag, end: end}, onError) {
        const token = {
            type: "scalar",
            offset: utilEmptyScalarPosition.emptyScalarPosition(offset, before, pos),
            indent: -1,
            source: ""
        }, node = composeScalar.composeScalar(ctx, token, tag, onError);
        return anchor && (node.anchor = anchor.source.substring(1), "" === node.anchor && onError(anchor, "BAD_ALIAS", "Anchor cannot be an empty string")), 
        spaceBefore && (node.spaceBefore = !0), comment && (node.comment = comment, node.range[2] = end), 
        node;
    }
    return composeNode.composeEmptyNode = composeEmptyNode, composeNode.composeNode = composeNode$1, 
    composeNode;
}

function requireComposeDoc() {
    if (hasRequiredComposeDoc) return composeDoc;
    hasRequiredComposeDoc = 1;
    var Document = requireDocument(), composeNode = requireComposeNode(), resolveEnd = requireResolveEnd(), resolveProps = requireResolveProps();
    return composeDoc.composeDoc = function(options, directives, {offset: offset, start: start, value: value, end: end}, onError) {
        const opts = Object.assign({
            _directives: directives
        }, options), doc = new Document.Document(void 0, opts), ctx = {
            atKey: !1,
            atRoot: !0,
            directives: doc.directives,
            options: doc.options,
            schema: doc.schema
        }, props = resolveProps.resolveProps(start, {
            indicator: "doc-start",
            next: value ?? end?.[0],
            offset: offset,
            onError: onError,
            parentIndent: 0,
            startOnNewline: !0
        });
        props.found && (doc.directives.docStart = !0, !value || "block-map" !== value.type && "block-seq" !== value.type || props.hasNewline || onError(props.end, "MISSING_CHAR", "Block collection cannot start on same line with directives-end marker")), 
        doc.contents = value ? composeNode.composeNode(ctx, value, props, onError) : composeNode.composeEmptyNode(ctx, props.end, start, null, props, onError);
        const contentEnd = doc.contents.range[2], re = resolveEnd.resolveEnd(end, contentEnd, !1, onError);
        return re.comment && (doc.comment = re.comment), doc.range = [ offset, contentEnd, re.offset ], 
        doc;
    }, composeDoc;
}

function requireComposer() {
    if (hasRequiredComposer) return composer;
    hasRequiredComposer = 1;
    var directives = requireDirectives(), Document = requireDocument(), errors = requireErrors(), identity = requireIdentity(), composeDoc = requireComposeDoc(), resolveEnd = requireResolveEnd();
    function getErrorPos(src) {
        if ("number" == typeof src) return [ src, src + 1 ];
        if (Array.isArray(src)) return 2 === src.length ? src : [ src[0], src[1] ];
        const {offset: offset, source: source} = src;
        return [ offset, offset + ("string" == typeof source ? source.length : 1) ];
    }
    function parsePrelude(prelude) {
        let comment = "", atComment = !1, afterEmptyLine = !1;
        for (let i = 0; i < prelude.length; ++i) {
            const source = prelude[i];
            switch (source[0]) {
              case "#":
                comment += ("" === comment ? "" : afterEmptyLine ? "\n\n" : "\n") + (source.substring(1) || " "), 
                atComment = !0, afterEmptyLine = !1;
                break;

              case "%":
                "#" !== prelude[i + 1]?.[0] && (i += 1), atComment = !1;
                break;

              default:
                atComment || (afterEmptyLine = !0), atComment = !1;
            }
        }
        return {
            comment: comment,
            afterEmptyLine: afterEmptyLine
        };
    }
    return composer.Composer = class {
        constructor(options = {}) {
            this.doc = null, this.atDirectives = !1, this.prelude = [], this.errors = [], this.warnings = [], 
            this.onError = (source, code, message, warning) => {
                const pos = getErrorPos(source);
                warning ? this.warnings.push(new errors.YAMLWarning(pos, code, message)) : this.errors.push(new errors.YAMLParseError(pos, code, message));
            }, this.directives = new directives.Directives({
                version: options.version || "1.2"
            }), this.options = options;
        }
        decorate(doc, afterDoc) {
            const {comment: comment, afterEmptyLine: afterEmptyLine} = parsePrelude(this.prelude);
            if (comment) {
                const dc = doc.contents;
                if (afterDoc) doc.comment = doc.comment ? `${doc.comment}\n${comment}` : comment; else if (afterEmptyLine || doc.directives.docStart || !dc) doc.commentBefore = comment; else if (identity.isCollection(dc) && !dc.flow && dc.items.length > 0) {
                    let it = dc.items[0];
                    identity.isPair(it) && (it = it.key);
                    const cb = it.commentBefore;
                    it.commentBefore = cb ? `${comment}\n${cb}` : comment;
                } else {
                    const cb = dc.commentBefore;
                    dc.commentBefore = cb ? `${comment}\n${cb}` : comment;
                }
            }
            afterDoc ? (Array.prototype.push.apply(doc.errors, this.errors), Array.prototype.push.apply(doc.warnings, this.warnings)) : (doc.errors = this.errors, 
            doc.warnings = this.warnings), this.prelude = [], this.errors = [], this.warnings = [];
        }
        streamInfo() {
            return {
                comment: parsePrelude(this.prelude).comment,
                directives: this.directives,
                errors: this.errors,
                warnings: this.warnings
            };
        }
        * compose(tokens, forceDoc = !1, endOffset = -1) {
            for (const token of tokens) yield* this.next(token);
            yield* this.end(forceDoc, endOffset);
        }
        * next(token) {
            switch (process.env.LOG_STREAM && console.dir(token, {
                depth: null
            }), token.type) {
              case "directive":
                this.directives.add(token.source, ((offset, message, warning) => {
                    const pos = getErrorPos(token);
                    pos[0] += offset, this.onError(pos, "BAD_DIRECTIVE", message, warning);
                })), this.prelude.push(token.source), this.atDirectives = !0;
                break;

              case "document":
                {
                    const doc = composeDoc.composeDoc(this.options, this.directives, token, this.onError);
                    this.atDirectives && !doc.directives.docStart && this.onError(token, "MISSING_CHAR", "Missing directives-end/doc-start indicator line"), 
                    this.decorate(doc, !1), this.doc && (yield this.doc), this.doc = doc, this.atDirectives = !1;
                    break;
                }

              case "byte-order-mark":
              case "space":
                break;

              case "comment":
              case "newline":
                this.prelude.push(token.source);
                break;

              case "error":
                {
                    const msg = token.source ? `${token.message}: ${JSON.stringify(token.source)}` : token.message, error = new errors.YAMLParseError(getErrorPos(token), "UNEXPECTED_TOKEN", msg);
                    this.atDirectives || !this.doc ? this.errors.push(error) : this.doc.errors.push(error);
                    break;
                }

              case "doc-end":
                {
                    if (!this.doc) {
                        const msg = "Unexpected doc-end without preceding document";
                        this.errors.push(new errors.YAMLParseError(getErrorPos(token), "UNEXPECTED_TOKEN", msg));
                        break;
                    }
                    this.doc.directives.docEnd = !0;
                    const end = resolveEnd.resolveEnd(token.end, token.offset + token.source.length, this.doc.options.strict, this.onError);
                    if (this.decorate(this.doc, !0), end.comment) {
                        const dc = this.doc.comment;
                        this.doc.comment = dc ? `${dc}\n${end.comment}` : end.comment;
                    }
                    this.doc.range[2] = end.offset;
                    break;
                }

              default:
                this.errors.push(new errors.YAMLParseError(getErrorPos(token), "UNEXPECTED_TOKEN", `Unsupported token ${token.type}`));
            }
        }
        * end(forceDoc = !1, endOffset = -1) {
            if (this.doc) this.decorate(this.doc, !0), yield this.doc, this.doc = null; else if (forceDoc) {
                const opts = Object.assign({
                    _directives: this.directives
                }, this.options), doc = new Document.Document(void 0, opts);
                this.atDirectives && this.onError(endOffset, "MISSING_CHAR", "Missing directives-end indicator line"), 
                doc.range = [ 0, endOffset, endOffset ], this.decorate(doc, !1), yield doc;
            }
        }
    }, composer;
}

var cst = {}, cstScalar = {}, hasRequiredCstScalar;

function requireCstScalar() {
    if (hasRequiredCstScalar) return cstScalar;
    hasRequiredCstScalar = 1;
    var resolveBlockScalar = requireResolveBlockScalar(), resolveFlowScalar = requireResolveFlowScalar(), errors = requireErrors(), stringifyString = requireStringifyString();
    function addEndtoBlockProps(props, end) {
        if (end) for (const st of end) switch (st.type) {
          case "space":
          case "comment":
            props.push(st);
            break;

          case "newline":
            return props.push(st), !0;
        }
        return !1;
    }
    function setFlowScalarValue(token, source, type) {
        switch (token.type) {
          case "scalar":
          case "double-quoted-scalar":
          case "single-quoted-scalar":
            token.type = type, token.source = source;
            break;

          case "block-scalar":
            {
                const end = token.props.slice(1);
                let oa = source.length;
                "block-scalar-header" === token.props[0].type && (oa -= token.props[0].source.length);
                for (const tok of end) tok.offset += oa;
                delete token.props, Object.assign(token, {
                    type: type,
                    source: source,
                    end: end
                });
                break;
            }

          case "block-map":
          case "block-seq":
            {
                const nl = {
                    type: "newline",
                    offset: token.offset + source.length,
                    indent: token.indent,
                    source: "\n"
                };
                delete token.items, Object.assign(token, {
                    type: type,
                    source: source,
                    end: [ nl ]
                });
                break;
            }

          default:
            {
                const indent = "indent" in token ? token.indent : -1, end = "end" in token && Array.isArray(token.end) ? token.end.filter((st => "space" === st.type || "comment" === st.type || "newline" === st.type)) : [];
                for (const key of Object.keys(token)) "type" !== key && "offset" !== key && delete token[key];
                Object.assign(token, {
                    type: type,
                    indent: indent,
                    source: source,
                    end: end
                });
            }
        }
    }
    return cstScalar.createScalarToken = function(value, context) {
        const {implicitKey: implicitKey = !1, indent: indent, inFlow: inFlow = !1, offset: offset = -1, type: type = "PLAIN"} = context, source = stringifyString.stringifyString({
            type: type,
            value: value
        }, {
            implicitKey: implicitKey,
            indent: indent > 0 ? " ".repeat(indent) : "",
            inFlow: inFlow,
            options: {
                blockQuote: !0,
                lineWidth: -1
            }
        }), end = context.end ?? [ {
            type: "newline",
            offset: -1,
            indent: indent,
            source: "\n"
        } ];
        switch (source[0]) {
          case "|":
          case ">":
            {
                const he = source.indexOf("\n"), head = source.substring(0, he), body = source.substring(he + 1) + "\n", props = [ {
                    type: "block-scalar-header",
                    offset: offset,
                    indent: indent,
                    source: head
                } ];
                return addEndtoBlockProps(props, end) || props.push({
                    type: "newline",
                    offset: -1,
                    indent: indent,
                    source: "\n"
                }), {
                    type: "block-scalar",
                    offset: offset,
                    indent: indent,
                    props: props,
                    source: body
                };
            }

          case '"':
            return {
                type: "double-quoted-scalar",
                offset: offset,
                indent: indent,
                source: source,
                end: end
            };

          case "'":
            return {
                type: "single-quoted-scalar",
                offset: offset,
                indent: indent,
                source: source,
                end: end
            };

          default:
            return {
                type: "scalar",
                offset: offset,
                indent: indent,
                source: source,
                end: end
            };
        }
    }, cstScalar.resolveAsScalar = function(token, strict = !0, onError) {
        if (token) {
            const _onError = (pos, code, message) => {
                const offset = "number" == typeof pos ? pos : Array.isArray(pos) ? pos[0] : pos.offset;
                if (!onError) throw new errors.YAMLParseError([ offset, offset + 1 ], code, message);
                onError(offset, code, message);
            };
            switch (token.type) {
              case "scalar":
              case "single-quoted-scalar":
              case "double-quoted-scalar":
                return resolveFlowScalar.resolveFlowScalar(token, strict, _onError);

              case "block-scalar":
                return resolveBlockScalar.resolveBlockScalar({
                    options: {
                        strict: strict
                    }
                }, token, _onError);
            }
        }
        return null;
    }, cstScalar.setScalarValue = function(token, value, context = {}) {
        let {afterKey: afterKey = !1, implicitKey: implicitKey = !1, inFlow: inFlow = !1, type: type} = context, indent = "indent" in token ? token.indent : null;
        if (afterKey && "number" == typeof indent && (indent += 2), !type) switch (token.type) {
          case "single-quoted-scalar":
            type = "QUOTE_SINGLE";
            break;

          case "double-quoted-scalar":
            type = "QUOTE_DOUBLE";
            break;

          case "block-scalar":
            {
                const header = token.props[0];
                if ("block-scalar-header" !== header.type) throw new Error("Invalid block scalar header");
                type = ">" === header.source[0] ? "BLOCK_FOLDED" : "BLOCK_LITERAL";
                break;
            }

          default:
            type = "PLAIN";
        }
        const source = stringifyString.stringifyString({
            type: type,
            value: value
        }, {
            implicitKey: implicitKey || null === indent,
            indent: null !== indent && indent > 0 ? " ".repeat(indent) : "",
            inFlow: inFlow,
            options: {
                blockQuote: !0,
                lineWidth: -1
            }
        });
        switch (source[0]) {
          case "|":
          case ">":
            !function(token, source) {
                const he = source.indexOf("\n"), head = source.substring(0, he), body = source.substring(he + 1) + "\n";
                if ("block-scalar" === token.type) {
                    const header = token.props[0];
                    if ("block-scalar-header" !== header.type) throw new Error("Invalid block scalar header");
                    header.source = head, token.source = body;
                } else {
                    const {offset: offset} = token, indent = "indent" in token ? token.indent : -1, props = [ {
                        type: "block-scalar-header",
                        offset: offset,
                        indent: indent,
                        source: head
                    } ];
                    addEndtoBlockProps(props, "end" in token ? token.end : void 0) || props.push({
                        type: "newline",
                        offset: -1,
                        indent: indent,
                        source: "\n"
                    });
                    for (const key of Object.keys(token)) "type" !== key && "offset" !== key && delete token[key];
                    Object.assign(token, {
                        type: "block-scalar",
                        indent: indent,
                        props: props,
                        source: body
                    });
                }
            }(token, source);
            break;

          case '"':
            setFlowScalarValue(token, source, "double-quoted-scalar");
            break;

          case "'":
            setFlowScalarValue(token, source, "single-quoted-scalar");
            break;

          default:
            setFlowScalarValue(token, source, "scalar");
        }
    }, cstScalar;
}

var cstStringify = {}, hasRequiredCstStringify;

function requireCstStringify() {
    if (hasRequiredCstStringify) return cstStringify;
    hasRequiredCstStringify = 1;
    function stringifyToken(token) {
        switch (token.type) {
          case "block-scalar":
            {
                let res = "";
                for (const tok of token.props) res += stringifyToken(tok);
                return res + token.source;
            }

          case "block-map":
          case "block-seq":
            {
                let res = "";
                for (const item of token.items) res += stringifyItem(item);
                return res;
            }

          case "flow-collection":
            {
                let res = token.start.source;
                for (const item of token.items) res += stringifyItem(item);
                for (const st of token.end) res += st.source;
                return res;
            }

          case "document":
            {
                let res = stringifyItem(token);
                if (token.end) for (const st of token.end) res += st.source;
                return res;
            }

          default:
            {
                let res = token.source;
                if ("end" in token && token.end) for (const st of token.end) res += st.source;
                return res;
            }
        }
    }
    function stringifyItem({start: start, key: key, sep: sep, value: value}) {
        let res = "";
        for (const st of start) res += st.source;
        if (key && (res += stringifyToken(key)), sep) for (const st of sep) res += st.source;
        return value && (res += stringifyToken(value)), res;
    }
    return cstStringify.stringify = cst => "type" in cst ? stringifyToken(cst) : stringifyItem(cst), 
    cstStringify;
}

var cstVisit = {}, hasRequiredCstVisit, hasRequiredCst;

function requireCstVisit() {
    if (hasRequiredCstVisit) return cstVisit;
    hasRequiredCstVisit = 1;
    const BREAK = Symbol("break visit"), SKIP = Symbol("skip children"), REMOVE = Symbol("remove item");
    function visit(cst, visitor) {
        "type" in cst && "document" === cst.type && (cst = {
            start: cst.start,
            value: cst.value
        }), _visit(Object.freeze([]), cst, visitor);
    }
    function _visit(path, item, visitor) {
        let ctrl = visitor(item, path);
        if ("symbol" == typeof ctrl) return ctrl;
        for (const field of [ "key", "value" ]) {
            const token = item[field];
            if (token && "items" in token) {
                for (let i = 0; i < token.items.length; ++i) {
                    const ci = _visit(Object.freeze(path.concat([ [ field, i ] ])), token.items[i], visitor);
                    if ("number" == typeof ci) i = ci - 1; else {
                        if (ci === BREAK) return BREAK;
                        ci === REMOVE && (token.items.splice(i, 1), i -= 1);
                    }
                }
                "function" == typeof ctrl && "key" === field && (ctrl = ctrl(item, path));
            }
        }
        return "function" == typeof ctrl ? ctrl(item, path) : ctrl;
    }
    return visit.BREAK = BREAK, visit.SKIP = SKIP, visit.REMOVE = REMOVE, visit.itemAtPath = (cst, path) => {
        let item = cst;
        for (const [field, index] of path) {
            const tok = item?.[field];
            if (!tok || !("items" in tok)) return;
            item = tok.items[index];
        }
        return item;
    }, visit.parentCollection = (cst, path) => {
        const parent = visit.itemAtPath(cst, path.slice(0, -1)), field = path[path.length - 1][0], coll = parent?.[field];
        if (coll && "items" in coll) return coll;
        throw new Error("Parent collection not found");
    }, cstVisit.visit = visit, cstVisit;
}

function requireCst() {
    if (hasRequiredCst) return cst;
    hasRequiredCst = 1;
    var cstScalar = requireCstScalar(), cstStringify = requireCstStringify(), cstVisit = requireCstVisit();
    return cst.createScalarToken = cstScalar.createScalarToken, cst.resolveAsScalar = cstScalar.resolveAsScalar, 
    cst.setScalarValue = cstScalar.setScalarValue, cst.stringify = cstStringify.stringify, 
    cst.visit = cstVisit.visit, cst.BOM = "\ufeff", cst.DOCUMENT = "", cst.FLOW_END = "", 
    cst.SCALAR = "", cst.isCollection = token => !!token && "items" in token, cst.isScalar = token => !!token && ("scalar" === token.type || "single-quoted-scalar" === token.type || "double-quoted-scalar" === token.type || "block-scalar" === token.type), 
    cst.prettyToken = function(token) {
        switch (token) {
          case "\ufeff":
            return "<BOM>";

          case "":
            return "<DOC>";

          case "":
            return "<FLOW_END>";

          case "":
            return "<SCALAR>";

          default:
            return JSON.stringify(token);
        }
    }, cst.tokenType = function(source) {
        switch (source) {
          case "\ufeff":
            return "byte-order-mark";

          case "":
            return "doc-mode";

          case "":
            return "flow-error-end";

          case "":
            return "scalar";

          case "---":
            return "doc-start";

          case "...":
            return "doc-end";

          case "":
          case "\n":
          case "\r\n":
            return "newline";

          case "-":
            return "seq-item-ind";

          case "?":
            return "explicit-key-ind";

          case ":":
            return "map-value-ind";

          case "{":
            return "flow-map-start";

          case "}":
            return "flow-map-end";

          case "[":
            return "flow-seq-start";

          case "]":
            return "flow-seq-end";

          case ",":
            return "comma";
        }
        switch (source[0]) {
          case " ":
          case "\t":
            return "space";

          case "#":
            return "comment";

          case "%":
            return "directive-line";

          case "*":
            return "alias";

          case "&":
            return "anchor";

          case "!":
            return "tag";

          case "'":
            return "single-quoted-scalar";

          case '"':
            return "double-quoted-scalar";

          case "|":
          case ">":
            return "block-scalar-header";
        }
        return null;
    }, cst;
}

var lexer = {}, hasRequiredLexer;

function requireLexer() {
    if (hasRequiredLexer) return lexer;
    hasRequiredLexer = 1;
    var cst = requireCst();
    function isEmpty(ch) {
        switch (ch) {
          case void 0:
          case " ":
          case "\n":
          case "\r":
          case "\t":
            return !0;

          default:
            return !1;
        }
    }
    const hexDigits = new Set("0123456789ABCDEFabcdef"), tagChars = new Set("0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz-#;/?:@&=+$_.!~*'()"), flowIndicatorChars = new Set(",[]{}"), invalidAnchorChars = new Set(" ,[]{}\n\r\t"), isNotAnchorChar = ch => !ch || invalidAnchorChars.has(ch);
    return lexer.Lexer = class {
        constructor() {
            this.atEnd = !1, this.blockScalarIndent = -1, this.blockScalarKeep = !1, this.buffer = "", 
            this.flowKey = !1, this.flowLevel = 0, this.indentNext = 0, this.indentValue = 0, 
            this.lineEndPos = null, this.next = null, this.pos = 0;
        }
        * lex(source, incomplete = !1) {
            if (source) {
                if ("string" != typeof source) throw TypeError("source is not a string");
                this.buffer = this.buffer ? this.buffer + source : source, this.lineEndPos = null;
            }
            this.atEnd = !incomplete;
            let next = this.next ?? "stream";
            for (;next && (incomplete || this.hasChars(1)); ) next = yield* this.parseNext(next);
        }
        atLineEnd() {
            let i = this.pos, ch = this.buffer[i];
            for (;" " === ch || "\t" === ch; ) ch = this.buffer[++i];
            return !ch || "#" === ch || "\n" === ch || "\r" === ch && "\n" === this.buffer[i + 1];
        }
        charAt(n) {
            return this.buffer[this.pos + n];
        }
        continueScalar(offset) {
            let ch = this.buffer[offset];
            if (this.indentNext > 0) {
                let indent = 0;
                for (;" " === ch; ) ch = this.buffer[++indent + offset];
                if ("\r" === ch) {
                    const next = this.buffer[indent + offset + 1];
                    if ("\n" === next || !next && !this.atEnd) return offset + indent + 1;
                }
                return "\n" === ch || indent >= this.indentNext || !ch && !this.atEnd ? offset + indent : -1;
            }
            if ("-" === ch || "." === ch) {
                const dt = this.buffer.substr(offset, 3);
                if (("---" === dt || "..." === dt) && isEmpty(this.buffer[offset + 3])) return -1;
            }
            return offset;
        }
        getLine() {
            let end = this.lineEndPos;
            return ("number" != typeof end || -1 !== end && end < this.pos) && (end = this.buffer.indexOf("\n", this.pos), 
            this.lineEndPos = end), -1 === end ? this.atEnd ? this.buffer.substring(this.pos) : null : ("\r" === this.buffer[end - 1] && (end -= 1), 
            this.buffer.substring(this.pos, end));
        }
        hasChars(n) {
            return this.pos + n <= this.buffer.length;
        }
        setNext(state) {
            return this.buffer = this.buffer.substring(this.pos), this.pos = 0, this.lineEndPos = null, 
            this.next = state, null;
        }
        peek(n) {
            return this.buffer.substr(this.pos, n);
        }
        * parseNext(next) {
            switch (next) {
              case "stream":
                return yield* this.parseStream();

              case "line-start":
                return yield* this.parseLineStart();

              case "block-start":
                return yield* this.parseBlockStart();

              case "doc":
                return yield* this.parseDocument();

              case "flow":
                return yield* this.parseFlowCollection();

              case "quoted-scalar":
                return yield* this.parseQuotedScalar();

              case "block-scalar":
                return yield* this.parseBlockScalar();

              case "plain-scalar":
                return yield* this.parsePlainScalar();
            }
        }
        * parseStream() {
            let line = this.getLine();
            if (null === line) return this.setNext("stream");
            if (line[0] === cst.BOM && (yield* this.pushCount(1), line = line.substring(1)), 
            "%" === line[0]) {
                let dirEnd = line.length, cs = line.indexOf("#");
                for (;-1 !== cs; ) {
                    const ch = line[cs - 1];
                    if (" " === ch || "\t" === ch) {
                        dirEnd = cs - 1;
                        break;
                    }
                    cs = line.indexOf("#", cs + 1);
                }
                for (;;) {
                    const ch = line[dirEnd - 1];
                    if (" " !== ch && "\t" !== ch) break;
                    dirEnd -= 1;
                }
                const n = (yield* this.pushCount(dirEnd)) + (yield* this.pushSpaces(!0));
                return yield* this.pushCount(line.length - n), this.pushNewline(), "stream";
            }
            if (this.atLineEnd()) {
                const sp = yield* this.pushSpaces(!0);
                return yield* this.pushCount(line.length - sp), yield* this.pushNewline(), "stream";
            }
            return yield cst.DOCUMENT, yield* this.parseLineStart();
        }
        * parseLineStart() {
            const ch = this.charAt(0);
            if (!ch && !this.atEnd) return this.setNext("line-start");
            if ("-" === ch || "." === ch) {
                if (!this.atEnd && !this.hasChars(4)) return this.setNext("line-start");
                const s = this.peek(3);
                if (("---" === s || "..." === s) && isEmpty(this.charAt(3))) return yield* this.pushCount(3), 
                this.indentValue = 0, this.indentNext = 0, "---" === s ? "doc" : "stream";
            }
            return this.indentValue = yield* this.pushSpaces(!1), this.indentNext > this.indentValue && !isEmpty(this.charAt(1)) && (this.indentNext = this.indentValue), 
            yield* this.parseBlockStart();
        }
        * parseBlockStart() {
            const [ch0, ch1] = this.peek(2);
            if (!ch1 && !this.atEnd) return this.setNext("block-start");
            if (("-" === ch0 || "?" === ch0 || ":" === ch0) && isEmpty(ch1)) {
                const n = (yield* this.pushCount(1)) + (yield* this.pushSpaces(!0));
                return this.indentNext = this.indentValue + 1, this.indentValue += n, yield* this.parseBlockStart();
            }
            return "doc";
        }
        * parseDocument() {
            yield* this.pushSpaces(!0);
            const line = this.getLine();
            if (null === line) return this.setNext("doc");
            let n = yield* this.pushIndicators();
            switch (line[n]) {
              case "#":
                yield* this.pushCount(line.length - n);

              case void 0:
                return yield* this.pushNewline(), yield* this.parseLineStart();

              case "{":
              case "[":
                return yield* this.pushCount(1), this.flowKey = !1, this.flowLevel = 1, "flow";

              case "}":
              case "]":
                return yield* this.pushCount(1), "doc";

              case "*":
                return yield* this.pushUntil(isNotAnchorChar), "doc";

              case '"':
              case "'":
                return yield* this.parseQuotedScalar();

              case "|":
              case ">":
                return n += (yield* this.parseBlockScalarHeader()), n += (yield* this.pushSpaces(!0)), 
                yield* this.pushCount(line.length - n), yield* this.pushNewline(), yield* this.parseBlockScalar();

              default:
                return yield* this.parsePlainScalar();
            }
        }
        * parseFlowCollection() {
            let nl, sp, indent = -1;
            do {
                nl = yield* this.pushNewline(), nl > 0 ? (sp = yield* this.pushSpaces(!1), this.indentValue = indent = sp) : sp = 0, 
                sp += (yield* this.pushSpaces(!0));
            } while (nl + sp > 0);
            const line = this.getLine();
            if (null === line) return this.setNext("flow");
            if (-1 !== indent && indent < this.indentNext && "#" !== line[0] || 0 === indent && (line.startsWith("---") || line.startsWith("...")) && isEmpty(line[3])) {
                if (!(indent === this.indentNext - 1 && 1 === this.flowLevel && ("]" === line[0] || "}" === line[0]))) return this.flowLevel = 0, 
                yield cst.FLOW_END, yield* this.parseLineStart();
            }
            let n = 0;
            for (;"," === line[n]; ) n += (yield* this.pushCount(1)), n += (yield* this.pushSpaces(!0)), 
            this.flowKey = !1;
            switch (n += (yield* this.pushIndicators()), line[n]) {
              case void 0:
                return "flow";

              case "#":
                return yield* this.pushCount(line.length - n), "flow";

              case "{":
              case "[":
                return yield* this.pushCount(1), this.flowKey = !1, this.flowLevel += 1, "flow";

              case "}":
              case "]":
                return yield* this.pushCount(1), this.flowKey = !0, this.flowLevel -= 1, this.flowLevel ? "flow" : "doc";

              case "*":
                return yield* this.pushUntil(isNotAnchorChar), "flow";

              case '"':
              case "'":
                return this.flowKey = !0, yield* this.parseQuotedScalar();

              case ":":
                {
                    const next = this.charAt(1);
                    if (this.flowKey || isEmpty(next) || "," === next) return this.flowKey = !1, yield* this.pushCount(1), 
                    yield* this.pushSpaces(!0), "flow";
                }

              default:
                return this.flowKey = !1, yield* this.parsePlainScalar();
            }
        }
        * parseQuotedScalar() {
            const quote = this.charAt(0);
            let end = this.buffer.indexOf(quote, this.pos + 1);
            if ("'" === quote) for (;-1 !== end && "'" === this.buffer[end + 1]; ) end = this.buffer.indexOf("'", end + 2); else for (;-1 !== end; ) {
                let n = 0;
                for (;"\\" === this.buffer[end - 1 - n]; ) n += 1;
                if (n % 2 == 0) break;
                end = this.buffer.indexOf('"', end + 1);
            }
            const qb = this.buffer.substring(0, end);
            let nl = qb.indexOf("\n", this.pos);
            if (-1 !== nl) {
                for (;-1 !== nl; ) {
                    const cs = this.continueScalar(nl + 1);
                    if (-1 === cs) break;
                    nl = qb.indexOf("\n", cs);
                }
                -1 !== nl && (end = nl - ("\r" === qb[nl - 1] ? 2 : 1));
            }
            if (-1 === end) {
                if (!this.atEnd) return this.setNext("quoted-scalar");
                end = this.buffer.length;
            }
            return yield* this.pushToIndex(end + 1, !1), this.flowLevel ? "flow" : "doc";
        }
        * parseBlockScalarHeader() {
            this.blockScalarIndent = -1, this.blockScalarKeep = !1;
            let i = this.pos;
            for (;;) {
                const ch = this.buffer[++i];
                if ("+" === ch) this.blockScalarKeep = !0; else if (ch > "0" && ch <= "9") this.blockScalarIndent = Number(ch) - 1; else if ("-" !== ch) break;
            }
            return yield* this.pushUntil((ch => isEmpty(ch) || "#" === ch));
        }
        * parseBlockScalar() {
            let ch, nl = this.pos - 1, indent = 0;
            loop: for (let i = this.pos; ch = this.buffer[i]; ++i) switch (ch) {
              case " ":
                indent += 1;
                break;

              case "\n":
                nl = i, indent = 0;
                break;

              case "\r":
                {
                    const next = this.buffer[i + 1];
                    if (!next && !this.atEnd) return this.setNext("block-scalar");
                    if ("\n" === next) break;
                }

              default:
                break loop;
            }
            if (!ch && !this.atEnd) return this.setNext("block-scalar");
            if (indent >= this.indentNext) {
                -1 === this.blockScalarIndent ? this.indentNext = indent : this.indentNext = this.blockScalarIndent + (0 === this.indentNext ? 1 : this.indentNext);
                do {
                    const cs = this.continueScalar(nl + 1);
                    if (-1 === cs) break;
                    nl = this.buffer.indexOf("\n", cs);
                } while (-1 !== nl);
                if (-1 === nl) {
                    if (!this.atEnd) return this.setNext("block-scalar");
                    nl = this.buffer.length;
                }
            }
            let i = nl + 1;
            for (ch = this.buffer[i]; " " === ch; ) ch = this.buffer[++i];
            if ("\t" === ch) {
                for (;"\t" === ch || " " === ch || "\r" === ch || "\n" === ch; ) ch = this.buffer[++i];
                nl = i - 1;
            } else if (!this.blockScalarKeep) for (;;) {
                let i = nl - 1, ch = this.buffer[i];
                "\r" === ch && (ch = this.buffer[--i]);
                const lastChar = i;
                for (;" " === ch; ) ch = this.buffer[--i];
                if (!("\n" === ch && i >= this.pos && i + 1 + indent > lastChar)) break;
                nl = i;
            }
            return yield cst.SCALAR, yield* this.pushToIndex(nl + 1, !0), yield* this.parseLineStart();
        }
        * parsePlainScalar() {
            const inFlow = this.flowLevel > 0;
            let ch, end = this.pos - 1, i = this.pos - 1;
            for (;ch = this.buffer[++i]; ) if (":" === ch) {
                const next = this.buffer[i + 1];
                if (isEmpty(next) || inFlow && flowIndicatorChars.has(next)) break;
                end = i;
            } else if (isEmpty(ch)) {
                let next = this.buffer[i + 1];
                if ("\r" === ch && ("\n" === next ? (i += 1, ch = "\n", next = this.buffer[i + 1]) : end = i), 
                "#" === next || inFlow && flowIndicatorChars.has(next)) break;
                if ("\n" === ch) {
                    const cs = this.continueScalar(i + 1);
                    if (-1 === cs) break;
                    i = Math.max(i, cs - 2);
                }
            } else {
                if (inFlow && flowIndicatorChars.has(ch)) break;
                end = i;
            }
            return ch || this.atEnd ? (yield cst.SCALAR, yield* this.pushToIndex(end + 1, !0), 
            inFlow ? "flow" : "doc") : this.setNext("plain-scalar");
        }
        * pushCount(n) {
            return n > 0 ? (yield this.buffer.substr(this.pos, n), this.pos += n, n) : 0;
        }
        * pushToIndex(i, allowEmpty) {
            const s = this.buffer.slice(this.pos, i);
            return s ? (yield s, this.pos += s.length, s.length) : (allowEmpty && (yield ""), 
            0);
        }
        * pushIndicators() {
            switch (this.charAt(0)) {
              case "!":
                return (yield* this.pushTag()) + (yield* this.pushSpaces(!0)) + (yield* this.pushIndicators());

              case "&":
                return (yield* this.pushUntil(isNotAnchorChar)) + (yield* this.pushSpaces(!0)) + (yield* this.pushIndicators());

              case "-":
              case "?":
              case ":":
                {
                    const inFlow = this.flowLevel > 0, ch1 = this.charAt(1);
                    if (isEmpty(ch1) || inFlow && flowIndicatorChars.has(ch1)) return inFlow ? this.flowKey && (this.flowKey = !1) : this.indentNext = this.indentValue + 1, 
                    (yield* this.pushCount(1)) + (yield* this.pushSpaces(!0)) + (yield* this.pushIndicators());
                }
            }
            return 0;
        }
        * pushTag() {
            if ("<" === this.charAt(1)) {
                let i = this.pos + 2, ch = this.buffer[i];
                for (;!isEmpty(ch) && ">" !== ch; ) ch = this.buffer[++i];
                return yield* this.pushToIndex(">" === ch ? i + 1 : i, !1);
            }
            {
                let i = this.pos + 1, ch = this.buffer[i];
                for (;ch; ) if (tagChars.has(ch)) ch = this.buffer[++i]; else {
                    if ("%" !== ch || !hexDigits.has(this.buffer[i + 1]) || !hexDigits.has(this.buffer[i + 2])) break;
                    ch = this.buffer[i += 3];
                }
                return yield* this.pushToIndex(i, !1);
            }
        }
        * pushNewline() {
            const ch = this.buffer[this.pos];
            return "\n" === ch ? yield* this.pushCount(1) : "\r" === ch && "\n" === this.charAt(1) ? yield* this.pushCount(2) : 0;
        }
        * pushSpaces(allowTabs) {
            let ch, i = this.pos - 1;
            do {
                ch = this.buffer[++i];
            } while (" " === ch || allowTabs && "\t" === ch);
            const n = i - this.pos;
            return n > 0 && (yield this.buffer.substr(this.pos, n), this.pos = i), n;
        }
        * pushUntil(test) {
            let i = this.pos, ch = this.buffer[i];
            for (;!test(ch); ) ch = this.buffer[++i];
            return yield* this.pushToIndex(i, !1);
        }
    }, lexer;
}

var lineCounter = {}, hasRequiredLineCounter;

function requireLineCounter() {
    if (hasRequiredLineCounter) return lineCounter;
    hasRequiredLineCounter = 1;
    return lineCounter.LineCounter = class {
        constructor() {
            this.lineStarts = [], this.addNewLine = offset => this.lineStarts.push(offset), 
            this.linePos = offset => {
                let low = 0, high = this.lineStarts.length;
                for (;low < high; ) {
                    const mid = low + high >> 1;
                    this.lineStarts[mid] < offset ? low = mid + 1 : high = mid;
                }
                if (this.lineStarts[low] === offset) return {
                    line: low + 1,
                    col: 1
                };
                if (0 === low) return {
                    line: 0,
                    col: offset
                };
                return {
                    line: low,
                    col: offset - this.lineStarts[low - 1] + 1
                };
            };
        }
    }, lineCounter;
}

var parser = {}, hasRequiredParser;

function requireParser() {
    if (hasRequiredParser) return parser;
    hasRequiredParser = 1;
    var cst = requireCst(), lexer = requireLexer();
    function includesToken(list, type) {
        for (let i = 0; i < list.length; ++i) if (list[i].type === type) return !0;
        return !1;
    }
    function findNonEmptyIndex(list) {
        for (let i = 0; i < list.length; ++i) switch (list[i].type) {
          case "space":
          case "comment":
          case "newline":
            break;

          default:
            return i;
        }
        return -1;
    }
    function isFlowToken(token) {
        switch (token?.type) {
          case "alias":
          case "scalar":
          case "single-quoted-scalar":
          case "double-quoted-scalar":
          case "flow-collection":
            return !0;

          default:
            return !1;
        }
    }
    function getPrevProps(parent) {
        switch (parent.type) {
          case "document":
            return parent.start;

          case "block-map":
            {
                const it = parent.items[parent.items.length - 1];
                return it.sep ?? it.start;
            }

          case "block-seq":
            return parent.items[parent.items.length - 1].start;

          default:
            return [];
        }
    }
    function getFirstKeyStartProps(prev) {
        if (0 === prev.length) return [];
        let i = prev.length;
        loop: for (;--i >= 0; ) switch (prev[i].type) {
          case "doc-start":
          case "explicit-key-ind":
          case "map-value-ind":
          case "seq-item-ind":
          case "newline":
            break loop;
        }
        for (;"space" === prev[++i]?.type; ) ;
        return prev.splice(i, prev.length);
    }
    function fixFlowSeqItems(fc) {
        if ("flow-seq-start" === fc.start.type) for (const it of fc.items) !it.sep || it.value || includesToken(it.start, "explicit-key-ind") || includesToken(it.sep, "map-value-ind") || (it.key && (it.value = it.key), 
        delete it.key, isFlowToken(it.value) ? it.value.end ? Array.prototype.push.apply(it.value.end, it.sep) : it.value.end = it.sep : Array.prototype.push.apply(it.start, it.sep), 
        delete it.sep);
    }
    return parser.Parser = class {
        constructor(onNewLine) {
            this.atNewLine = !0, this.atScalar = !1, this.indent = 0, this.offset = 0, this.onKeyLine = !1, 
            this.stack = [], this.source = "", this.type = "", this.lexer = new lexer.Lexer, 
            this.onNewLine = onNewLine;
        }
        * parse(source, incomplete = !1) {
            this.onNewLine && 0 === this.offset && this.onNewLine(0);
            for (const lexeme of this.lexer.lex(source, incomplete)) yield* this.next(lexeme);
            incomplete || (yield* this.end());
        }
        * next(source) {
            if (this.source = source, process.env.LOG_TOKENS && console.log("|", cst.prettyToken(source)), 
            this.atScalar) return this.atScalar = !1, yield* this.step(), void (this.offset += source.length);
            const type = cst.tokenType(source);
            if (type) if ("scalar" === type) this.atNewLine = !1, this.atScalar = !0, this.type = "scalar"; else {
                switch (this.type = type, yield* this.step(), type) {
                  case "newline":
                    this.atNewLine = !0, this.indent = 0, this.onNewLine && this.onNewLine(this.offset + source.length);
                    break;

                  case "space":
                    this.atNewLine && " " === source[0] && (this.indent += source.length);
                    break;

                  case "explicit-key-ind":
                  case "map-value-ind":
                  case "seq-item-ind":
                    this.atNewLine && (this.indent += source.length);
                    break;

                  case "doc-mode":
                  case "flow-error-end":
                    return;

                  default:
                    this.atNewLine = !1;
                }
                this.offset += source.length;
            } else {
                const message = `Not a YAML token: ${source}`;
                yield* this.pop({
                    type: "error",
                    offset: this.offset,
                    message: message,
                    source: source
                }), this.offset += source.length;
            }
        }
        * end() {
            for (;this.stack.length > 0; ) yield* this.pop();
        }
        get sourceToken() {
            return {
                type: this.type,
                offset: this.offset,
                indent: this.indent,
                source: this.source
            };
        }
        * step() {
            const top = this.peek(1);
            if ("doc-end" !== this.type || top && "doc-end" === top.type) {
                if (!top) return yield* this.stream();
                switch (top.type) {
                  case "document":
                    return yield* this.document(top);

                  case "alias":
                  case "scalar":
                  case "single-quoted-scalar":
                  case "double-quoted-scalar":
                    return yield* this.scalar(top);

                  case "block-scalar":
                    return yield* this.blockScalar(top);

                  case "block-map":
                    return yield* this.blockMap(top);

                  case "block-seq":
                    return yield* this.blockSequence(top);

                  case "flow-collection":
                    return yield* this.flowCollection(top);

                  case "doc-end":
                    return yield* this.documentEnd(top);
                }
                yield* this.pop();
            } else {
                for (;this.stack.length > 0; ) yield* this.pop();
                this.stack.push({
                    type: "doc-end",
                    offset: this.offset,
                    source: this.source
                });
            }
        }
        peek(n) {
            return this.stack[this.stack.length - n];
        }
        * pop(error) {
            const token = error ?? this.stack.pop();
            if (token) if (0 === this.stack.length) yield token; else {
                const top = this.peek(1);
                switch ("block-scalar" === token.type ? token.indent = "indent" in top ? top.indent : 0 : "flow-collection" === token.type && "document" === top.type && (token.indent = 0), 
                "flow-collection" === token.type && fixFlowSeqItems(token), top.type) {
                  case "document":
                    top.value = token;
                    break;

                  case "block-scalar":
                    top.props.push(token);
                    break;

                  case "block-map":
                    {
                        const it = top.items[top.items.length - 1];
                        if (it.value) return top.items.push({
                            start: [],
                            key: token,
                            sep: []
                        }), void (this.onKeyLine = !0);
                        if (!it.sep) return Object.assign(it, {
                            key: token,
                            sep: []
                        }), void (this.onKeyLine = !it.explicitKey);
                        it.value = token;
                        break;
                    }

                  case "block-seq":
                    {
                        const it = top.items[top.items.length - 1];
                        it.value ? top.items.push({
                            start: [],
                            value: token
                        }) : it.value = token;
                        break;
                    }

                  case "flow-collection":
                    {
                        const it = top.items[top.items.length - 1];
                        return void (!it || it.value ? top.items.push({
                            start: [],
                            key: token,
                            sep: []
                        }) : it.sep ? it.value = token : Object.assign(it, {
                            key: token,
                            sep: []
                        }));
                    }

                  default:
                    yield* this.pop(), yield* this.pop(token);
                }
                if (!("document" !== top.type && "block-map" !== top.type && "block-seq" !== top.type || "block-map" !== token.type && "block-seq" !== token.type)) {
                    const last = token.items[token.items.length - 1];
                    last && !last.sep && !last.value && last.start.length > 0 && -1 === findNonEmptyIndex(last.start) && (0 === token.indent || last.start.every((st => "comment" !== st.type || st.indent < token.indent))) && ("document" === top.type ? top.end = last.start : top.items.push({
                        start: last.start
                    }), token.items.splice(-1, 1));
                }
            } else {
                const message = "Tried to pop an empty stack";
                yield {
                    type: "error",
                    offset: this.offset,
                    source: "",
                    message: message
                };
            }
        }
        * stream() {
            switch (this.type) {
              case "directive-line":
                return void (yield {
                    type: "directive",
                    offset: this.offset,
                    source: this.source
                });

              case "byte-order-mark":
              case "space":
              case "comment":
              case "newline":
                return void (yield this.sourceToken);

              case "doc-mode":
              case "doc-start":
                {
                    const doc = {
                        type: "document",
                        offset: this.offset,
                        start: []
                    };
                    return "doc-start" === this.type && doc.start.push(this.sourceToken), void this.stack.push(doc);
                }
            }
            yield {
                type: "error",
                offset: this.offset,
                message: `Unexpected ${this.type} token in YAML stream`,
                source: this.source
            };
        }
        * document(doc) {
            if (doc.value) return yield* this.lineEnd(doc);
            switch (this.type) {
              case "doc-start":
                return void (-1 !== findNonEmptyIndex(doc.start) ? (yield* this.pop(), yield* this.step()) : doc.start.push(this.sourceToken));

              case "anchor":
              case "tag":
              case "space":
              case "comment":
              case "newline":
                return void doc.start.push(this.sourceToken);
            }
            const bv = this.startBlockValue(doc);
            bv ? this.stack.push(bv) : yield {
                type: "error",
                offset: this.offset,
                message: `Unexpected ${this.type} token in YAML document`,
                source: this.source
            };
        }
        * scalar(scalar) {
            if ("map-value-ind" === this.type) {
                const start = getFirstKeyStartProps(getPrevProps(this.peek(2)));
                let sep;
                scalar.end ? (sep = scalar.end, sep.push(this.sourceToken), delete scalar.end) : sep = [ this.sourceToken ];
                const map = {
                    type: "block-map",
                    offset: scalar.offset,
                    indent: scalar.indent,
                    items: [ {
                        start: start,
                        key: scalar,
                        sep: sep
                    } ]
                };
                this.onKeyLine = !0, this.stack[this.stack.length - 1] = map;
            } else yield* this.lineEnd(scalar);
        }
        * blockScalar(scalar) {
            switch (this.type) {
              case "space":
              case "comment":
              case "newline":
                return void scalar.props.push(this.sourceToken);

              case "scalar":
                if (scalar.source = this.source, this.atNewLine = !0, this.indent = 0, this.onNewLine) {
                    let nl = this.source.indexOf("\n") + 1;
                    for (;0 !== nl; ) this.onNewLine(this.offset + nl), nl = this.source.indexOf("\n", nl) + 1;
                }
                yield* this.pop();
                break;

              default:
                yield* this.pop(), yield* this.step();
            }
        }
        * blockMap(map) {
            const it = map.items[map.items.length - 1];
            switch (this.type) {
              case "newline":
                if (this.onKeyLine = !1, it.value) {
                    const end = "end" in it.value ? it.value.end : void 0, last = Array.isArray(end) ? end[end.length - 1] : void 0;
                    "comment" === last?.type ? end?.push(this.sourceToken) : map.items.push({
                        start: [ this.sourceToken ]
                    });
                } else it.sep ? it.sep.push(this.sourceToken) : it.start.push(this.sourceToken);
                return;

              case "space":
              case "comment":
                if (it.value) map.items.push({
                    start: [ this.sourceToken ]
                }); else if (it.sep) it.sep.push(this.sourceToken); else {
                    if (this.atIndentedComment(it.start, map.indent)) {
                        const prev = map.items[map.items.length - 2], end = prev?.value?.end;
                        if (Array.isArray(end)) return Array.prototype.push.apply(end, it.start), end.push(this.sourceToken), 
                        void map.items.pop();
                    }
                    it.start.push(this.sourceToken);
                }
                return;
            }
            if (this.indent >= map.indent) {
                const atMapIndent = !this.onKeyLine && this.indent === map.indent, atNextItem = atMapIndent && (it.sep || it.explicitKey) && "seq-item-ind" !== this.type;
                let start = [];
                if (atNextItem && it.sep && !it.value) {
                    const nl = [];
                    for (let i = 0; i < it.sep.length; ++i) {
                        const st = it.sep[i];
                        switch (st.type) {
                          case "newline":
                            nl.push(i);
                            break;

                          case "space":
                            break;

                          case "comment":
                            st.indent > map.indent && (nl.length = 0);
                            break;

                          default:
                            nl.length = 0;
                        }
                    }
                    nl.length >= 2 && (start = it.sep.splice(nl[1]));
                }
                switch (this.type) {
                  case "anchor":
                  case "tag":
                    return void (atNextItem || it.value ? (start.push(this.sourceToken), map.items.push({
                        start: start
                    }), this.onKeyLine = !0) : it.sep ? it.sep.push(this.sourceToken) : it.start.push(this.sourceToken));

                  case "explicit-key-ind":
                    return it.sep || it.explicitKey ? atNextItem || it.value ? (start.push(this.sourceToken), 
                    map.items.push({
                        start: start,
                        explicitKey: !0
                    })) : this.stack.push({
                        type: "block-map",
                        offset: this.offset,
                        indent: this.indent,
                        items: [ {
                            start: [ this.sourceToken ],
                            explicitKey: !0
                        } ]
                    }) : (it.start.push(this.sourceToken), it.explicitKey = !0), void (this.onKeyLine = !0);

                  case "map-value-ind":
                    if (it.explicitKey) if (it.sep) if (it.value) map.items.push({
                        start: [],
                        key: null,
                        sep: [ this.sourceToken ]
                    }); else if (includesToken(it.sep, "map-value-ind")) this.stack.push({
                        type: "block-map",
                        offset: this.offset,
                        indent: this.indent,
                        items: [ {
                            start: start,
                            key: null,
                            sep: [ this.sourceToken ]
                        } ]
                    }); else if (isFlowToken(it.key) && !includesToken(it.sep, "newline")) {
                        const start = getFirstKeyStartProps(it.start), key = it.key, sep = it.sep;
                        sep.push(this.sourceToken), delete it.key, delete it.sep, this.stack.push({
                            type: "block-map",
                            offset: this.offset,
                            indent: this.indent,
                            items: [ {
                                start: start,
                                key: key,
                                sep: sep
                            } ]
                        });
                    } else start.length > 0 ? it.sep = it.sep.concat(start, this.sourceToken) : it.sep.push(this.sourceToken); else if (includesToken(it.start, "newline")) Object.assign(it, {
                        key: null,
                        sep: [ this.sourceToken ]
                    }); else {
                        const start = getFirstKeyStartProps(it.start);
                        this.stack.push({
                            type: "block-map",
                            offset: this.offset,
                            indent: this.indent,
                            items: [ {
                                start: start,
                                key: null,
                                sep: [ this.sourceToken ]
                            } ]
                        });
                    } else it.sep ? it.value || atNextItem ? map.items.push({
                        start: start,
                        key: null,
                        sep: [ this.sourceToken ]
                    }) : includesToken(it.sep, "map-value-ind") ? this.stack.push({
                        type: "block-map",
                        offset: this.offset,
                        indent: this.indent,
                        items: [ {
                            start: [],
                            key: null,
                            sep: [ this.sourceToken ]
                        } ]
                    }) : it.sep.push(this.sourceToken) : Object.assign(it, {
                        key: null,
                        sep: [ this.sourceToken ]
                    });
                    return void (this.onKeyLine = !0);

                  case "alias":
                  case "scalar":
                  case "single-quoted-scalar":
                  case "double-quoted-scalar":
                    {
                        const fs = this.flowScalar(this.type);
                        return void (atNextItem || it.value ? (map.items.push({
                            start: start,
                            key: fs,
                            sep: []
                        }), this.onKeyLine = !0) : it.sep ? this.stack.push(fs) : (Object.assign(it, {
                            key: fs,
                            sep: []
                        }), this.onKeyLine = !0));
                    }

                  default:
                    {
                        const bv = this.startBlockValue(map);
                        if (bv) return atMapIndent && "block-seq" !== bv.type && map.items.push({
                            start: start
                        }), void this.stack.push(bv);
                    }
                }
            }
            yield* this.pop(), yield* this.step();
        }
        * blockSequence(seq) {
            const it = seq.items[seq.items.length - 1];
            switch (this.type) {
              case "newline":
                if (it.value) {
                    const end = "end" in it.value ? it.value.end : void 0, last = Array.isArray(end) ? end[end.length - 1] : void 0;
                    "comment" === last?.type ? end?.push(this.sourceToken) : seq.items.push({
                        start: [ this.sourceToken ]
                    });
                } else it.start.push(this.sourceToken);
                return;

              case "space":
              case "comment":
                if (it.value) seq.items.push({
                    start: [ this.sourceToken ]
                }); else {
                    if (this.atIndentedComment(it.start, seq.indent)) {
                        const prev = seq.items[seq.items.length - 2], end = prev?.value?.end;
                        if (Array.isArray(end)) return Array.prototype.push.apply(end, it.start), end.push(this.sourceToken), 
                        void seq.items.pop();
                    }
                    it.start.push(this.sourceToken);
                }
                return;

              case "anchor":
              case "tag":
                if (it.value || this.indent <= seq.indent) break;
                return void it.start.push(this.sourceToken);

              case "seq-item-ind":
                if (this.indent !== seq.indent) break;
                return void (it.value || includesToken(it.start, "seq-item-ind") ? seq.items.push({
                    start: [ this.sourceToken ]
                }) : it.start.push(this.sourceToken));
            }
            if (this.indent > seq.indent) {
                const bv = this.startBlockValue(seq);
                if (bv) return void this.stack.push(bv);
            }
            yield* this.pop(), yield* this.step();
        }
        * flowCollection(fc) {
            const it = fc.items[fc.items.length - 1];
            if ("flow-error-end" === this.type) {
                let top;
                do {
                    yield* this.pop(), top = this.peek(1);
                } while (top && "flow-collection" === top.type);
            } else if (0 === fc.end.length) {
                switch (this.type) {
                  case "comma":
                  case "explicit-key-ind":
                    return void (!it || it.sep ? fc.items.push({
                        start: [ this.sourceToken ]
                    }) : it.start.push(this.sourceToken));

                  case "map-value-ind":
                    return void (!it || it.value ? fc.items.push({
                        start: [],
                        key: null,
                        sep: [ this.sourceToken ]
                    }) : it.sep ? it.sep.push(this.sourceToken) : Object.assign(it, {
                        key: null,
                        sep: [ this.sourceToken ]
                    }));

                  case "space":
                  case "comment":
                  case "newline":
                  case "anchor":
                  case "tag":
                    return void (!it || it.value ? fc.items.push({
                        start: [ this.sourceToken ]
                    }) : it.sep ? it.sep.push(this.sourceToken) : it.start.push(this.sourceToken));

                  case "alias":
                  case "scalar":
                  case "single-quoted-scalar":
                  case "double-quoted-scalar":
                    {
                        const fs = this.flowScalar(this.type);
                        return void (!it || it.value ? fc.items.push({
                            start: [],
                            key: fs,
                            sep: []
                        }) : it.sep ? this.stack.push(fs) : Object.assign(it, {
                            key: fs,
                            sep: []
                        }));
                    }

                  case "flow-map-end":
                  case "flow-seq-end":
                    return void fc.end.push(this.sourceToken);
                }
                const bv = this.startBlockValue(fc);
                bv ? this.stack.push(bv) : (yield* this.pop(), yield* this.step());
            } else {
                const parent = this.peek(2);
                if ("block-map" === parent.type && ("map-value-ind" === this.type && parent.indent === fc.indent || "newline" === this.type && !parent.items[parent.items.length - 1].sep)) yield* this.pop(), 
                yield* this.step(); else if ("map-value-ind" === this.type && "flow-collection" !== parent.type) {
                    const start = getFirstKeyStartProps(getPrevProps(parent));
                    fixFlowSeqItems(fc);
                    const sep = fc.end.splice(1, fc.end.length);
                    sep.push(this.sourceToken);
                    const map = {
                        type: "block-map",
                        offset: fc.offset,
                        indent: fc.indent,
                        items: [ {
                            start: start,
                            key: fc,
                            sep: sep
                        } ]
                    };
                    this.onKeyLine = !0, this.stack[this.stack.length - 1] = map;
                } else yield* this.lineEnd(fc);
            }
        }
        flowScalar(type) {
            if (this.onNewLine) {
                let nl = this.source.indexOf("\n") + 1;
                for (;0 !== nl; ) this.onNewLine(this.offset + nl), nl = this.source.indexOf("\n", nl) + 1;
            }
            return {
                type: type,
                offset: this.offset,
                indent: this.indent,
                source: this.source
            };
        }
        startBlockValue(parent) {
            switch (this.type) {
              case "alias":
              case "scalar":
              case "single-quoted-scalar":
              case "double-quoted-scalar":
                return this.flowScalar(this.type);

              case "block-scalar-header":
                return {
                    type: "block-scalar",
                    offset: this.offset,
                    indent: this.indent,
                    props: [ this.sourceToken ],
                    source: ""
                };

              case "flow-map-start":
              case "flow-seq-start":
                return {
                    type: "flow-collection",
                    offset: this.offset,
                    indent: this.indent,
                    start: this.sourceToken,
                    items: [],
                    end: []
                };

              case "seq-item-ind":
                return {
                    type: "block-seq",
                    offset: this.offset,
                    indent: this.indent,
                    items: [ {
                        start: [ this.sourceToken ]
                    } ]
                };

              case "explicit-key-ind":
                {
                    this.onKeyLine = !0;
                    const start = getFirstKeyStartProps(getPrevProps(parent));
                    return start.push(this.sourceToken), {
                        type: "block-map",
                        offset: this.offset,
                        indent: this.indent,
                        items: [ {
                            start: start,
                            explicitKey: !0
                        } ]
                    };
                }

              case "map-value-ind":
                {
                    this.onKeyLine = !0;
                    const start = getFirstKeyStartProps(getPrevProps(parent));
                    return {
                        type: "block-map",
                        offset: this.offset,
                        indent: this.indent,
                        items: [ {
                            start: start,
                            key: null,
                            sep: [ this.sourceToken ]
                        } ]
                    };
                }
            }
            return null;
        }
        atIndentedComment(start, indent) {
            return "comment" === this.type && (!(this.indent <= indent) && start.every((st => "newline" === st.type || "space" === st.type)));
        }
        * documentEnd(docEnd) {
            "doc-mode" !== this.type && (docEnd.end ? docEnd.end.push(this.sourceToken) : docEnd.end = [ this.sourceToken ], 
            "newline" === this.type && (yield* this.pop()));
        }
        * lineEnd(token) {
            switch (this.type) {
              case "comma":
              case "doc-start":
              case "doc-end":
              case "flow-seq-end":
              case "flow-map-end":
              case "map-value-ind":
                yield* this.pop(), yield* this.step();
                break;

              case "newline":
                this.onKeyLine = !1;

              default:
                token.end ? token.end.push(this.sourceToken) : token.end = [ this.sourceToken ], 
                "newline" === this.type && (yield* this.pop());
            }
        }
    }, parser;
}

var publicApi = {}, hasRequiredPublicApi, hasRequiredDist;

function requirePublicApi() {
    if (hasRequiredPublicApi) return publicApi;
    hasRequiredPublicApi = 1;
    var composer = requireComposer(), Document = requireDocument(), errors = requireErrors(), log = requireLog(), identity = requireIdentity(), lineCounter = requireLineCounter(), parser = requireParser();
    function parseOptions(options) {
        const prettyErrors = !1 !== options.prettyErrors;
        return {
            lineCounter: options.lineCounter || prettyErrors && new lineCounter.LineCounter || null,
            prettyErrors: prettyErrors
        };
    }
    function parseDocument(source, options = {}) {
        const {lineCounter: lineCounter, prettyErrors: prettyErrors} = parseOptions(options), parser$1 = new parser.Parser(lineCounter?.addNewLine), composer$1 = new composer.Composer(options);
        let doc = null;
        for (const _doc of composer$1.compose(parser$1.parse(source), !0, source.length)) if (doc) {
            if ("silent" !== doc.options.logLevel) {
                doc.errors.push(new errors.YAMLParseError(_doc.range.slice(0, 2), "MULTIPLE_DOCS", "Source contains multiple documents; please use YAML.parseAllDocuments()"));
                break;
            }
        } else doc = _doc;
        return prettyErrors && lineCounter && (doc.errors.forEach(errors.prettifyError(source, lineCounter)), 
        doc.warnings.forEach(errors.prettifyError(source, lineCounter))), doc;
    }
    return publicApi.parse = function(src, reviver, options) {
        let _reviver;
        "function" == typeof reviver ? _reviver = reviver : void 0 === options && reviver && "object" == typeof reviver && (options = reviver);
        const doc = parseDocument(src, options);
        if (!doc) return null;
        if (doc.warnings.forEach((warning => log.warn(doc.options.logLevel, warning))), 
        doc.errors.length > 0) {
            if ("silent" !== doc.options.logLevel) throw doc.errors[0];
            doc.errors = [];
        }
        return doc.toJS(Object.assign({
            reviver: _reviver
        }, options));
    }, publicApi.parseAllDocuments = function(source, options = {}) {
        const {lineCounter: lineCounter, prettyErrors: prettyErrors} = parseOptions(options), parser$1 = new parser.Parser(lineCounter?.addNewLine), composer$1 = new composer.Composer(options), docs = Array.from(composer$1.compose(parser$1.parse(source)));
        if (prettyErrors && lineCounter) for (const doc of docs) doc.errors.forEach(errors.prettifyError(source, lineCounter)), 
        doc.warnings.forEach(errors.prettifyError(source, lineCounter));
        return docs.length > 0 ? docs : Object.assign([], {
            empty: !0
        }, composer$1.streamInfo());
    }, publicApi.parseDocument = parseDocument, publicApi.stringify = function(value, replacer, options) {
        let _replacer = null;
        if ("function" == typeof replacer || Array.isArray(replacer) ? _replacer = replacer : void 0 === options && replacer && (options = replacer), 
        "string" == typeof options && (options = options.length), "number" == typeof options) {
            const indent = Math.round(options);
            options = indent < 1 ? void 0 : indent > 8 ? {
                indent: 8
            } : {
                indent: indent
            };
        }
        if (void 0 === value) {
            const {keepUndefined: keepUndefined} = options ?? replacer ?? {};
            if (!keepUndefined) return;
        }
        return identity.isDocument(value) && !_replacer ? value.toString(options) : new Document.Document(value, _replacer, options).toString(options);
    }, publicApi;
}

function requireDist() {
    if (hasRequiredDist) return dist;
    hasRequiredDist = 1;
    var composer = requireComposer(), Document = requireDocument(), Schema = requireSchema(), errors = requireErrors(), Alias = requireAlias(), identity = requireIdentity(), Pair = requirePair(), Scalar = requireScalar(), YAMLMap = requireYAMLMap(), YAMLSeq = requireYAMLSeq(), cst = requireCst(), lexer = requireLexer(), lineCounter = requireLineCounter(), parser = requireParser(), publicApi = requirePublicApi(), visit = requireVisit();
    return dist.Composer = composer.Composer, dist.Document = Document.Document, dist.Schema = Schema.Schema, 
    dist.YAMLError = errors.YAMLError, dist.YAMLParseError = errors.YAMLParseError, 
    dist.YAMLWarning = errors.YAMLWarning, dist.Alias = Alias.Alias, dist.isAlias = identity.isAlias, 
    dist.isCollection = identity.isCollection, dist.isDocument = identity.isDocument, 
    dist.isMap = identity.isMap, dist.isNode = identity.isNode, dist.isPair = identity.isPair, 
    dist.isScalar = identity.isScalar, dist.isSeq = identity.isSeq, dist.Pair = Pair.Pair, 
    dist.Scalar = Scalar.Scalar, dist.YAMLMap = YAMLMap.YAMLMap, dist.YAMLSeq = YAMLSeq.YAMLSeq, 
    dist.CST = cst, dist.Lexer = lexer.Lexer, dist.LineCounter = lineCounter.LineCounter, 
    dist.Parser = parser.Parser, dist.parse = publicApi.parse, dist.parseAllDocuments = publicApi.parseAllDocuments, 
    dist.parseDocument = publicApi.parseDocument, dist.stringify = publicApi.stringify, 
    dist.visit = visit.visit, dist.visitAsync = visit.visitAsync, dist;
}

var distExports = requireDist();

function resolveTilde(filePath) {
    return filePath.startsWith("~") ? path.join(homedir(), filePath.slice(1)) : filePath;
}

function loadPlaintextFile(filePath) {
    try {
        return fs.readFileSync(resolveTilde(filePath), "utf-8");
    } catch (error) {
        throw "ENOENT" === error.code ? new Error(`File not found: ${filePath}`) : new Error(`Error reading file: ${error.message}`);
    }
}

function loadJsonFile(filePath) {
    try {
        return parseJson(loadPlaintextFile(filePath));
    } catch (error) {
        return null;
    }
}

function loadYamlFile(filePath) {
    try {
        return distExports.parse(loadPlaintextFile(filePath));
    } catch (error) {
        return null;
    }
}

function parseJson(input) {
    try {
        return JSON.parse(input);
    } catch (error) {
        return error instanceof SyntaxError && console.error("Error parsing JSON:", error.message), 
        null;
    }
}

function loadTomlFile(filePath, injectConfigPath = !0) {
    try {
        const data = loadPlaintextFile(filePath), parsedData = tomlExports.parse(data);
        return injectConfigPath && (parsedData.configPath = filePath), parsedData;
    } catch (error) {
        return error instanceof SyntaxError && console.error("Error parsing TOML:", error.message), 
        null;
    }
}

function createFolders(filePath, resolve = !0) {
    return filePath = resolveTilde(filePath), resolve && (filePath = path.resolve(filePath), 
    path.basename(filePath).includes(".") && (filePath = path.dirname(filePath))), fs.mkdirSync(filePath, {
        recursive: !0
    });
}

function doesFileExist(filePath, resolve = !0) {
    try {
        return filePath = resolveTilde(filePath), resolve && (filePath = path.resolve(filePath)), 
        fs.statfsSync(filePath), !0;
    } catch (err) {}
    return !1;
}

function directoryExists(directoryPath) {
    try {
        return fs.statSync(resolveTilde(directoryPath)).isDirectory();
    } catch (err) {
        return !1;
    }
}

function moveFiles(sourceDir, destinationDir, overwrite = !1) {
    if (!directoryExists(sourceDir)) return void warnMessage(`[moveFiles] Source directory does not exist: ${sourceDir}`);
    const files = fs.readdirSync(sourceDir);
    directoryExists(destinationDir) || createFolders(destinationDir), files.forEach((file => {
        const sourceFile = path.join(sourceDir, file), destinationFile = path.join(destinationDir, file);
        if (fs.existsSync(destinationFile)) {
            if (!overwrite) return;
            fs.unlinkSync(destinationFile);
        }
        fs.cpSync(sourceFile, destinationFile);
    }));
}

function loadFileNamesToMap(dir, extension) {
    const fileMap = new Map;
    if (!directoryExists(dir)) return fileMap;
    return fs.readdirSync(dir).forEach((file => {
        const filePath = path.join(dir, file);
        if (fs.statSync(filePath).isFile()) {
            const fileNameWithoutExt = path.basename(file, path.extname(file));
            extension && path.extname(file) !== extension || fileMap.set(fileNameWithoutExt, file);
        }
    })), fileMap;
}

function updateGitignore(items, gitignorePath = path.join(process.cwd(), ".gitignore")) {
    let gitignoreContent = "";
    doesFileExist(gitignorePath) && (gitignoreContent = loadPlaintextFile(gitignorePath));
    const gitignoreLines = gitignoreContent.split("\n").map((line => line.trim()));
    let updated = !1;
    items.forEach((item => {
        gitignoreLines.includes(item.trim()) || (gitignoreContent += `\n${item.trim()}`, 
        updated = !0);
    })), updated && fs.writeFileSync(gitignorePath, gitignoreContent.trim() + "\n", "utf-8");
}

function findFileInRepo(targetFilename, startDir = process.cwd(), maxDepth = 5, skipDirs = [ "node_modules", ".cache" ]) {
    let currentDir = startDir, depth = 0;
    for (;depth < maxDepth; ) {
        const filePath = findFileInDirectory(currentDir, targetFilename.toLowerCase(), skipDirs);
        if (filePath) return filePath;
        const gitDir = path.join(currentDir, ".git");
        if (fs.existsSync(gitDir) && fs.statSync(gitDir).isDirectory()) return null;
        const parentDir = path.dirname(currentDir);
        if (parentDir === currentDir) return null;
        currentDir = parentDir, depth++;
    }
    return null;
}

function findFileInDirectory(dir, targetFilename, skipDirs) {
    const files = fs.readdirSync(dir);
    for (const file of files) {
        const absolutePath = path.join(dir, file);
        try {
            if (fs.statSync(absolutePath).isDirectory()) {
                if (skipDirs.includes(file)) continue;
                const result = findFileInDirectory(absolutePath, targetFilename, skipDirs);
                if (result) return result;
            } else if (file.toLowerCase() === targetFilename) return absolutePath;
        } catch (err) {}
    }
    return null;
}

function isInCurrentDir(filePath) {
    return path.resolve(process.cwd()) === path.resolve(path.dirname(filePath));
}

function deepMerge$1(obj1, obj2) {
    const result = {
        ...obj1
    };
    for (const key in obj2) obj2.hasOwnProperty(key) && (isObject(obj2[key]) && isObject(result[key]) ? result[key] = deepMerge$1(result[key], obj2[key]) : result[key] = obj2[key]);
    return result;
}

function isObject(value) {
    return value && "object" == typeof value && !Array.isArray(value);
}

function timeAgo(date) {
    const now = new Date, past = new Date(date), elapsed = now.getTime() - past.getTime();
    if (elapsed < 0) return "in the future";
    if (elapsed < 6e4) {
        const seconds = Math.round(elapsed / 1e3);
        return `${seconds} ${1 === seconds ? "second" : "seconds"} ago`;
    }
    if (elapsed < 36e5) {
        const minutes = Math.round(elapsed / 6e4);
        return `${minutes} ${1 === minutes ? "minute" : "minutes"} ago`;
    }
    if (elapsed < 864e5) {
        const hours = Math.round(elapsed / 36e5);
        return `${hours} ${1 === hours ? "hour" : "hours"} ago`;
    }
    if (elapsed < 2592e6) {
        const days = Math.round(elapsed / 864e5);
        return `${days} ${1 === days ? "day" : "days"} ago`;
    }
    if (elapsed < 31536e6) {
        const months = Math.round(elapsed / 2592e6);
        return `${months} ${1 === months ? "month" : "months"} ago`;
    }
    {
        const years = Math.round(elapsed / 31536e6);
        return `${years} ${1 === years ? "year" : "years"} ago`;
    }
}

function numberStringToNumber(localeString) {
    return Number(localeString.replace(new RegExp(`[${(new Intl.NumberFormat).format(11111).replace(/\d/g, "")}]`, "g"), ""));
}

const DEFAULT_CONFIG_FILE = "Solana.toml", DEFAULT_ACCOUNTS_DIR = "fixtures", DEFAULT_CACHE_DIR = ".cache", DEFAULT_TEST_LEDGER_DIR = "test-ledger", DEFAULT_ACCOUNTS_DIR_TEMP = ".cache/staging/fixtures", DEFAULT_ACCOUNTS_DIR_LOADED = ".cache/loaded/fixtures", DEFAULT_KEYPAIR_PATH = "~/.config/solana/id.json", DEFAULT_CLI_YAML_PATH = "~/.config/solana/cli/config.yml", DEFAULT_BUILD_DIR = "target/deploy", cliConfig = loadSolanaCliConfig(), COMMON_OPTIONS = {
    config: new Option("-C --config <PATH>", "path to a Solana.toml config file").default(DEFAULT_CONFIG_FILE),
    keypair: new Option("--keypair <PATH>", "path to a keypair file").default(cliConfig?.keypair_path || DEFAULT_KEYPAIR_PATH),
    url: new Option("-u --url <URL_OR_MONIKER>", "URL for Solana's JSON RPC or moniker"),
    outputOnly: new Option("--output-only", "only output the generated command, do not execute it"),
    accountDir: new Option("--account-dir <ACCOUNT_DIR>", "local directory path to store any cloned accounts").default(DEFAULT_ACCOUNTS_DIR),
    manifestPath: new Option("--manifest-path <PATH>", "path to Cargo.toml").default(join(process.cwd(), "Cargo.toml"))
};

function loadSolanaCliConfig(filePath = DEFAULT_CLI_YAML_PATH) {
    try {
        const cliConfig = loadYamlFile(filePath);
        if (cliConfig?.json_rpc_url) switch (cliConfig.json_rpc_url) {
          case "https://api.devnet.solana.com":
            cliConfig.json_rpc_url = "devnet";
            break;

          case "https://api.testnet.solana.com":
            cliConfig.json_rpc_url = "testnet";
            break;

          case "https://api.mainnet-beta.solana.com":
            cliConfig.json_rpc_url = "mainnet";
            break;

          case "http://localhost:8899":
            cliConfig.json_rpc_url = "localhost";
        }
        return cliConfig;
    } catch (err) {
        return {
            keypair_path: DEFAULT_KEYPAIR_PATH
        };
    }
}

function loadConfigToml(configPath = DEFAULT_CONFIG_FILE, settings = {}, isConfigRequired = !1) {
    if (directoryExists(configPath) && (configPath = join(configPath, DEFAULT_CONFIG_FILE)), 
    configPath === DEFAULT_CONFIG_FILE) {
        const newPath = findFileInRepo(DEFAULT_CONFIG_FILE);
        newPath && (configPath = newPath, isInCurrentDir(newPath) || warnMessage(`Using closest Solana.toml located at: ${newPath}`));
    }
    let config = {
        configPath: configPath
    };
    doesFileExist(configPath, !0) ? config = loadTomlFile(configPath) || config : isConfigRequired ? warningOutro("No Solana.toml config file found. Operation canceled.") : warnMessage("No Solana.toml config file found. Skipping.");
    const defaultSettings = {
        cluster: COMMON_OPTIONS.url.defaultValue,
        accountDir: COMMON_OPTIONS.accountDir.defaultValue,
        keypair: COMMON_OPTIONS.keypair.defaultValue,
        ledgerDir: DEFAULT_TEST_LEDGER_DIR
    };
    return config.settings = Object.assign(defaultSettings, config.settings || {}), 
    config = deconflictSolanaTomlConfig(config, settings), config.configPath = configPath, 
    config;
}

function deconflictSolanaTomlConfig(config, args) {
    return args?.url && args.url !== COMMON_OPTIONS.url.defaultValue && (config.settings.cluster = args.url), 
    args?.accountDir && args.accountDir !== COMMON_OPTIONS.accountDir.defaultValue && (config.settings.accountDir = args.accountDir), 
    args?.keypair && args.keypair !== COMMON_OPTIONS.keypair.defaultValue && (config.settings.keypair = args.keypair), 
    config;
}

const cliOutputConfig = {
    writeErr(str) {
        console.log(str.trim() + "\n");
    },
    writeOut(str) {
        console.log(str.trim() + "\n");
    }
};

var SOLANA_ERROR__BLOCK_HEIGHT_EXCEEDED = 1, SOLANA_ERROR__INVALID_NONCE = 2, SOLANA_ERROR__NONCE_ACCOUNT_NOT_FOUND = 3, SOLANA_ERROR__BLOCKHASH_STRING_LENGTH_OUT_OF_RANGE = 4, SOLANA_ERROR__INVALID_BLOCKHASH_BYTE_LENGTH = 5, SOLANA_ERROR__LAMPORTS_OUT_OF_RANGE = 6, SOLANA_ERROR__MALFORMED_BIGINT_STRING = 7, SOLANA_ERROR__MALFORMED_NUMBER_STRING = 8, SOLANA_ERROR__TIMESTAMP_OUT_OF_RANGE = 9, SOLANA_ERROR__JSON_RPC__PARSE_ERROR = -32700, SOLANA_ERROR__JSON_RPC__INTERNAL_ERROR = -32603, SOLANA_ERROR__JSON_RPC__INVALID_PARAMS = -32602, SOLANA_ERROR__JSON_RPC__METHOD_NOT_FOUND = -32601, SOLANA_ERROR__JSON_RPC__INVALID_REQUEST = -32600, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_MIN_CONTEXT_SLOT_NOT_REACHED = -32016, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_UNSUPPORTED_TRANSACTION_VERSION = -32015, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_BLOCK_STATUS_NOT_AVAILABLE_YET = -32014, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_TRANSACTION_SIGNATURE_LEN_MISMATCH = -32013, SOLANA_ERROR__JSON_RPC__SCAN_ERROR = -32012, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_TRANSACTION_HISTORY_NOT_AVAILABLE = -32011, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_KEY_EXCLUDED_FROM_SECONDARY_INDEX = -32010, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_LONG_TERM_STORAGE_SLOT_SKIPPED = -32009, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_NO_SNAPSHOT = -32008, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_SLOT_SKIPPED = -32007, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_TRANSACTION_PRECOMPILE_VERIFICATION_FAILURE = -32006, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_NODE_UNHEALTHY = -32005, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_BLOCK_NOT_AVAILABLE = -32004, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_TRANSACTION_SIGNATURE_VERIFICATION_FAILURE = -32003, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_SEND_TRANSACTION_PREFLIGHT_FAILURE = -32002, SOLANA_ERROR__JSON_RPC__SERVER_ERROR_BLOCK_CLEANED_UP = -32001, SOLANA_ERROR__ADDRESSES__INVALID_BYTE_LENGTH = 28e5, SOLANA_ERROR__ADDRESSES__STRING_LENGTH_OUT_OF_RANGE = 2800001, SOLANA_ERROR__ADDRESSES__INVALID_BASE58_ENCODED_ADDRESS = 2800002, SOLANA_ERROR__ADDRESSES__INVALID_ED25519_PUBLIC_KEY = 2800003, SOLANA_ERROR__ADDRESSES__MALFORMED_PDA = 2800004, SOLANA_ERROR__ADDRESSES__PDA_BUMP_SEED_OUT_OF_RANGE = 2800005, SOLANA_ERROR__ADDRESSES__MAX_NUMBER_OF_PDA_SEEDS_EXCEEDED = 2800006, SOLANA_ERROR__ADDRESSES__MAX_PDA_SEED_LENGTH_EXCEEDED = 2800007, SOLANA_ERROR__ADDRESSES__INVALID_SEEDS_POINT_ON_CURVE = 2800008, SOLANA_ERROR__ADDRESSES__FAILED_TO_FIND_VIABLE_PDA_BUMP_SEED = 2800009, SOLANA_ERROR__ADDRESSES__PDA_ENDS_WITH_PDA_MARKER = 2800010, SOLANA_ERROR__ACCOUNTS__ACCOUNT_NOT_FOUND = 323e4, SOLANA_ERROR__ACCOUNTS__ONE_OR_MORE_ACCOUNTS_NOT_FOUND = 32300001, SOLANA_ERROR__ACCOUNTS__FAILED_TO_DECODE_ACCOUNT = 3230002, SOLANA_ERROR__ACCOUNTS__EXPECTED_DECODED_ACCOUNT = 3230003, SOLANA_ERROR__ACCOUNTS__EXPECTED_ALL_ACCOUNTS_TO_BE_DECODED = 3230004, SOLANA_ERROR__SUBTLE_CRYPTO__DISALLOWED_IN_INSECURE_CONTEXT = 361e4, SOLANA_ERROR__SUBTLE_CRYPTO__DIGEST_UNIMPLEMENTED = 3610001, SOLANA_ERROR__SUBTLE_CRYPTO__ED25519_ALGORITHM_UNIMPLEMENTED = 3610002, SOLANA_ERROR__SUBTLE_CRYPTO__EXPORT_FUNCTION_UNIMPLEMENTED = 3610003, SOLANA_ERROR__SUBTLE_CRYPTO__GENERATE_FUNCTION_UNIMPLEMENTED = 3610004, SOLANA_ERROR__SUBTLE_CRYPTO__SIGN_FUNCTION_UNIMPLEMENTED = 3610005, SOLANA_ERROR__SUBTLE_CRYPTO__VERIFY_FUNCTION_UNIMPLEMENTED = 3610006, SOLANA_ERROR__SUBTLE_CRYPTO__CANNOT_EXPORT_NON_EXTRACTABLE_KEY = 3610007, SOLANA_ERROR__CRYPTO__RANDOM_VALUES_FUNCTION_UNIMPLEMENTED = 3611e3, SOLANA_ERROR__KEYS__INVALID_KEY_PAIR_BYTE_LENGTH = 3704e3, SOLANA_ERROR__KEYS__INVALID_PRIVATE_KEY_BYTE_LENGTH = 3704001, SOLANA_ERROR__KEYS__INVALID_SIGNATURE_BYTE_LENGTH = 3704002, SOLANA_ERROR__KEYS__SIGNATURE_STRING_LENGTH_OUT_OF_RANGE = 3704003, SOLANA_ERROR__KEYS__PUBLIC_KEY_MUST_MATCH_PRIVATE_KEY = 3704004, SOLANA_ERROR__INSTRUCTION__EXPECTED_TO_HAVE_ACCOUNTS = 4128e3, SOLANA_ERROR__INSTRUCTION__EXPECTED_TO_HAVE_DATA = 4128001, SOLANA_ERROR__INSTRUCTION__PROGRAM_ID_MISMATCH = 4128002, SOLANA_ERROR__INSTRUCTION_ERROR__UNKNOWN = 4615e3, SOLANA_ERROR__INSTRUCTION_ERROR__GENERIC_ERROR = 4615001, SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_ARGUMENT = 4615002, SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_INSTRUCTION_DATA = 4615003, SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_ACCOUNT_DATA = 4615004, SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_DATA_TOO_SMALL = 4615005, SOLANA_ERROR__INSTRUCTION_ERROR__INSUFFICIENT_FUNDS = 4615006, SOLANA_ERROR__INSTRUCTION_ERROR__INCORRECT_PROGRAM_ID = 4615007, SOLANA_ERROR__INSTRUCTION_ERROR__MISSING_REQUIRED_SIGNATURE = 4615008, SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_ALREADY_INITIALIZED = 4615009, SOLANA_ERROR__INSTRUCTION_ERROR__UNINITIALIZED_ACCOUNT = 4615010, SOLANA_ERROR__INSTRUCTION_ERROR__UNBALANCED_INSTRUCTION = 4615011, SOLANA_ERROR__INSTRUCTION_ERROR__MODIFIED_PROGRAM_ID = 4615012, SOLANA_ERROR__INSTRUCTION_ERROR__EXTERNAL_ACCOUNT_LAMPORT_SPEND = 4615013, SOLANA_ERROR__INSTRUCTION_ERROR__EXTERNAL_ACCOUNT_DATA_MODIFIED = 4615014, SOLANA_ERROR__INSTRUCTION_ERROR__READONLY_LAMPORT_CHANGE = 4615015, SOLANA_ERROR__INSTRUCTION_ERROR__READONLY_DATA_MODIFIED = 4615016, SOLANA_ERROR__INSTRUCTION_ERROR__DUPLICATE_ACCOUNT_INDEX = 4615017, SOLANA_ERROR__INSTRUCTION_ERROR__EXECUTABLE_MODIFIED = 4615018, SOLANA_ERROR__INSTRUCTION_ERROR__RENT_EPOCH_MODIFIED = 4615019, SOLANA_ERROR__INSTRUCTION_ERROR__NOT_ENOUGH_ACCOUNT_KEYS = 4615020, SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_DATA_SIZE_CHANGED = 4615021, SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_NOT_EXECUTABLE = 4615022, SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_BORROW_FAILED = 4615023, SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_BORROW_OUTSTANDING = 4615024, SOLANA_ERROR__INSTRUCTION_ERROR__DUPLICATE_ACCOUNT_OUT_OF_SYNC = 4615025, SOLANA_ERROR__INSTRUCTION_ERROR__CUSTOM = 4615026, SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_ERROR = 4615027, SOLANA_ERROR__INSTRUCTION_ERROR__EXECUTABLE_DATA_MODIFIED = 4615028, SOLANA_ERROR__INSTRUCTION_ERROR__EXECUTABLE_LAMPORT_CHANGE = 4615029, SOLANA_ERROR__INSTRUCTION_ERROR__EXECUTABLE_ACCOUNT_NOT_RENT_EXEMPT = 4615030, SOLANA_ERROR__INSTRUCTION_ERROR__UNSUPPORTED_PROGRAM_ID = 4615031, SOLANA_ERROR__INSTRUCTION_ERROR__CALL_DEPTH = 4615032, SOLANA_ERROR__INSTRUCTION_ERROR__MISSING_ACCOUNT = 4615033, SOLANA_ERROR__INSTRUCTION_ERROR__REENTRANCY_NOT_ALLOWED = 4615034, SOLANA_ERROR__INSTRUCTION_ERROR__MAX_SEED_LENGTH_EXCEEDED = 4615035, SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_SEEDS = 4615036, SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_REALLOC = 4615037, SOLANA_ERROR__INSTRUCTION_ERROR__COMPUTATIONAL_BUDGET_EXCEEDED = 4615038, SOLANA_ERROR__INSTRUCTION_ERROR__PRIVILEGE_ESCALATION = 4615039, SOLANA_ERROR__INSTRUCTION_ERROR__PROGRAM_ENVIRONMENT_SETUP_FAILURE = 4615040, SOLANA_ERROR__INSTRUCTION_ERROR__PROGRAM_FAILED_TO_COMPLETE = 4615041, SOLANA_ERROR__INSTRUCTION_ERROR__PROGRAM_FAILED_TO_COMPILE = 4615042, SOLANA_ERROR__INSTRUCTION_ERROR__IMMUTABLE = 4615043, SOLANA_ERROR__INSTRUCTION_ERROR__INCORRECT_AUTHORITY = 4615044, SOLANA_ERROR__INSTRUCTION_ERROR__BORSH_IO_ERROR = 4615045, SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_NOT_RENT_EXEMPT = 4615046, SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_ACCOUNT_OWNER = 4615047, SOLANA_ERROR__INSTRUCTION_ERROR__ARITHMETIC_OVERFLOW = 4615048, SOLANA_ERROR__INSTRUCTION_ERROR__UNSUPPORTED_SYSVAR = 4615049, SOLANA_ERROR__INSTRUCTION_ERROR__ILLEGAL_OWNER = 4615050, SOLANA_ERROR__INSTRUCTION_ERROR__MAX_ACCOUNTS_DATA_ALLOCATIONS_EXCEEDED = 4615051, SOLANA_ERROR__INSTRUCTION_ERROR__MAX_ACCOUNTS_EXCEEDED = 4615052, SOLANA_ERROR__INSTRUCTION_ERROR__MAX_INSTRUCTION_TRACE_LENGTH_EXCEEDED = 4615053, SOLANA_ERROR__INSTRUCTION_ERROR__BUILTIN_PROGRAMS_MUST_CONSUME_COMPUTE_UNITS = 4615054, SOLANA_ERROR__SIGNER__ADDRESS_CANNOT_HAVE_MULTIPLE_SIGNERS = 5508e3, SOLANA_ERROR__SIGNER__EXPECTED_KEY_PAIR_SIGNER = 5508001, SOLANA_ERROR__SIGNER__EXPECTED_MESSAGE_SIGNER = 5508002, SOLANA_ERROR__SIGNER__EXPECTED_MESSAGE_MODIFYING_SIGNER = 5508003, SOLANA_ERROR__SIGNER__EXPECTED_MESSAGE_PARTIAL_SIGNER = 5508004, SOLANA_ERROR__SIGNER__EXPECTED_TRANSACTION_SIGNER = 5508005, SOLANA_ERROR__SIGNER__EXPECTED_TRANSACTION_MODIFYING_SIGNER = 5508006, SOLANA_ERROR__SIGNER__EXPECTED_TRANSACTION_PARTIAL_SIGNER = 5508007, SOLANA_ERROR__SIGNER__EXPECTED_TRANSACTION_SENDING_SIGNER = 5508008, SOLANA_ERROR__SIGNER__TRANSACTION_CANNOT_HAVE_MULTIPLE_SENDING_SIGNERS = 5508009, SOLANA_ERROR__SIGNER__TRANSACTION_SENDING_SIGNER_MISSING = 5508010, SOLANA_ERROR__SIGNER__WALLET_MULTISIGN_UNIMPLEMENTED = 5508011, SOLANA_ERROR__TRANSACTION__INVOKED_PROGRAMS_CANNOT_PAY_FEES = 5663e3, SOLANA_ERROR__TRANSACTION__INVOKED_PROGRAMS_MUST_NOT_BE_WRITABLE = 5663001, SOLANA_ERROR__TRANSACTION__EXPECTED_BLOCKHASH_LIFETIME = 5663002, SOLANA_ERROR__TRANSACTION__EXPECTED_NONCE_LIFETIME = 5663003, SOLANA_ERROR__TRANSACTION__VERSION_NUMBER_OUT_OF_RANGE = 5663004, SOLANA_ERROR__TRANSACTION__FAILED_TO_DECOMPILE_ADDRESS_LOOKUP_TABLE_CONTENTS_MISSING = 5663005, SOLANA_ERROR__TRANSACTION__FAILED_TO_DECOMPILE_ADDRESS_LOOKUP_TABLE_INDEX_OUT_OF_RANGE = 5663006, SOLANA_ERROR__TRANSACTION__FAILED_TO_DECOMPILE_INSTRUCTION_PROGRAM_ADDRESS_NOT_FOUND = 5663007, SOLANA_ERROR__TRANSACTION__FAILED_TO_DECOMPILE_FEE_PAYER_MISSING = 5663008, SOLANA_ERROR__TRANSACTION__SIGNATURES_MISSING = 5663009, SOLANA_ERROR__TRANSACTION__ADDRESS_MISSING = 5663010, SOLANA_ERROR__TRANSACTION__FEE_PAYER_MISSING = 5663011, SOLANA_ERROR__TRANSACTION__FEE_PAYER_SIGNATURE_MISSING = 5663012, SOLANA_ERROR__TRANSACTION__INVALID_NONCE_TRANSACTION_INSTRUCTIONS_MISSING = 5663013, SOLANA_ERROR__TRANSACTION__INVALID_NONCE_TRANSACTION_FIRST_INSTRUCTION_MUST_BE_ADVANCE_NONCE = 5663014, SOLANA_ERROR__TRANSACTION__ADDRESSES_CANNOT_SIGN_TRANSACTION = 5663015, SOLANA_ERROR__TRANSACTION__CANNOT_ENCODE_WITH_EMPTY_SIGNATURES = 5663016, SOLANA_ERROR__TRANSACTION__MESSAGE_SIGNATURES_MISMATCH = 5663017, SOLANA_ERROR__TRANSACTION__FAILED_TO_ESTIMATE_COMPUTE_LIMIT = 5663018, SOLANA_ERROR__TRANSACTION__FAILED_WHEN_SIMULATING_TO_ESTIMATE_COMPUTE_LIMIT = 5663019, SOLANA_ERROR__TRANSACTION_ERROR__UNKNOWN = 705e4, SOLANA_ERROR__TRANSACTION_ERROR__ACCOUNT_IN_USE = 7050001, SOLANA_ERROR__TRANSACTION_ERROR__ACCOUNT_LOADED_TWICE = 7050002, SOLANA_ERROR__TRANSACTION_ERROR__ACCOUNT_NOT_FOUND = 7050003, SOLANA_ERROR__TRANSACTION_ERROR__PROGRAM_ACCOUNT_NOT_FOUND = 7050004, SOLANA_ERROR__TRANSACTION_ERROR__INSUFFICIENT_FUNDS_FOR_FEE = 7050005, SOLANA_ERROR__TRANSACTION_ERROR__INVALID_ACCOUNT_FOR_FEE = 7050006, SOLANA_ERROR__TRANSACTION_ERROR__ALREADY_PROCESSED = 7050007, SOLANA_ERROR__TRANSACTION_ERROR__BLOCKHASH_NOT_FOUND = 7050008, SOLANA_ERROR__TRANSACTION_ERROR__CALL_CHAIN_TOO_DEEP = 7050009, SOLANA_ERROR__TRANSACTION_ERROR__MISSING_SIGNATURE_FOR_FEE = 7050010, SOLANA_ERROR__TRANSACTION_ERROR__INVALID_ACCOUNT_INDEX = 7050011, SOLANA_ERROR__TRANSACTION_ERROR__SIGNATURE_FAILURE = 7050012, SOLANA_ERROR__TRANSACTION_ERROR__INVALID_PROGRAM_FOR_EXECUTION = 7050013, SOLANA_ERROR__TRANSACTION_ERROR__SANITIZE_FAILURE = 7050014, SOLANA_ERROR__TRANSACTION_ERROR__CLUSTER_MAINTENANCE = 7050015, SOLANA_ERROR__TRANSACTION_ERROR__ACCOUNT_BORROW_OUTSTANDING = 7050016, SOLANA_ERROR__TRANSACTION_ERROR__WOULD_EXCEED_MAX_BLOCK_COST_LIMIT = 7050017, SOLANA_ERROR__TRANSACTION_ERROR__UNSUPPORTED_VERSION = 7050018, SOLANA_ERROR__TRANSACTION_ERROR__INVALID_WRITABLE_ACCOUNT = 7050019, SOLANA_ERROR__TRANSACTION_ERROR__WOULD_EXCEED_MAX_ACCOUNT_COST_LIMIT = 7050020, SOLANA_ERROR__TRANSACTION_ERROR__WOULD_EXCEED_ACCOUNT_DATA_BLOCK_LIMIT = 7050021, SOLANA_ERROR__TRANSACTION_ERROR__TOO_MANY_ACCOUNT_LOCKS = 7050022, SOLANA_ERROR__TRANSACTION_ERROR__ADDRESS_LOOKUP_TABLE_NOT_FOUND = 7050023, SOLANA_ERROR__TRANSACTION_ERROR__INVALID_ADDRESS_LOOKUP_TABLE_OWNER = 7050024, SOLANA_ERROR__TRANSACTION_ERROR__INVALID_ADDRESS_LOOKUP_TABLE_DATA = 7050025, SOLANA_ERROR__TRANSACTION_ERROR__INVALID_ADDRESS_LOOKUP_TABLE_INDEX = 7050026, SOLANA_ERROR__TRANSACTION_ERROR__INVALID_RENT_PAYING_ACCOUNT = 7050027, SOLANA_ERROR__TRANSACTION_ERROR__WOULD_EXCEED_MAX_VOTE_COST_LIMIT = 7050028, SOLANA_ERROR__TRANSACTION_ERROR__WOULD_EXCEED_ACCOUNT_DATA_TOTAL_LIMIT = 7050029, SOLANA_ERROR__TRANSACTION_ERROR__DUPLICATE_INSTRUCTION = 7050030, SOLANA_ERROR__TRANSACTION_ERROR__INSUFFICIENT_FUNDS_FOR_RENT = 7050031, SOLANA_ERROR__TRANSACTION_ERROR__MAX_LOADED_ACCOUNTS_DATA_SIZE_EXCEEDED = 7050032, SOLANA_ERROR__TRANSACTION_ERROR__INVALID_LOADED_ACCOUNTS_DATA_SIZE_LIMIT = 7050033, SOLANA_ERROR__TRANSACTION_ERROR__RESANITIZATION_NEEDED = 7050034, SOLANA_ERROR__TRANSACTION_ERROR__PROGRAM_EXECUTION_TEMPORARILY_RESTRICTED = 7050035, SOLANA_ERROR__TRANSACTION_ERROR__UNBALANCED_TRANSACTION = 7050036, SOLANA_ERROR__CODECS__CANNOT_DECODE_EMPTY_BYTE_ARRAY = 8078e3, SOLANA_ERROR__CODECS__INVALID_BYTE_LENGTH = 8078001, SOLANA_ERROR__CODECS__EXPECTED_FIXED_LENGTH = 8078002, SOLANA_ERROR__CODECS__EXPECTED_VARIABLE_LENGTH = 8078003, SOLANA_ERROR__CODECS__ENCODER_DECODER_SIZE_COMPATIBILITY_MISMATCH = 8078004, SOLANA_ERROR__CODECS__ENCODER_DECODER_FIXED_SIZE_MISMATCH = 8078005, SOLANA_ERROR__CODECS__ENCODER_DECODER_MAX_SIZE_MISMATCH = 8078006, SOLANA_ERROR__CODECS__INVALID_NUMBER_OF_ITEMS = 8078007, SOLANA_ERROR__CODECS__ENUM_DISCRIMINATOR_OUT_OF_RANGE = 8078008, SOLANA_ERROR__CODECS__INVALID_DISCRIMINATED_UNION_VARIANT = 8078009, SOLANA_ERROR__CODECS__INVALID_ENUM_VARIANT = 8078010, SOLANA_ERROR__CODECS__NUMBER_OUT_OF_RANGE = 8078011, SOLANA_ERROR__CODECS__INVALID_STRING_FOR_BASE = 8078012, SOLANA_ERROR__CODECS__EXPECTED_POSITIVE_BYTE_LENGTH = 8078013, SOLANA_ERROR__CODECS__OFFSET_OUT_OF_RANGE = 8078014, SOLANA_ERROR__CODECS__INVALID_LITERAL_UNION_VARIANT = 8078015, SOLANA_ERROR__CODECS__LITERAL_UNION_DISCRIMINATOR_OUT_OF_RANGE = 8078016, SOLANA_ERROR__CODECS__UNION_VARIANT_OUT_OF_RANGE = 8078017, SOLANA_ERROR__CODECS__INVALID_CONSTANT = 8078018, SOLANA_ERROR__CODECS__EXPECTED_ZERO_VALUE_TO_MATCH_ITEM_FIXED_SIZE = 8078019, SOLANA_ERROR__CODECS__ENCODED_BYTES_MUST_NOT_INCLUDE_SENTINEL = 8078020, SOLANA_ERROR__CODECS__SENTINEL_MISSING_IN_DECODED_BYTES = 8078021, SOLANA_ERROR__CODECS__CANNOT_USE_LEXICAL_VALUES_AS_ENUM_DISCRIMINATORS = 8078022, SOLANA_ERROR__RPC__INTEGER_OVERFLOW = 81e5, SOLANA_ERROR__RPC__TRANSPORT_HTTP_HEADER_FORBIDDEN = 8100001, SOLANA_ERROR__RPC__TRANSPORT_HTTP_ERROR = 8100002, SOLANA_ERROR__RPC__API_PLAN_MISSING_FOR_RPC_METHOD = 8100003, SOLANA_ERROR__RPC_SUBSCRIPTIONS__CANNOT_CREATE_SUBSCRIPTION_PLAN = 819e4, SOLANA_ERROR__RPC_SUBSCRIPTIONS__EXPECTED_SERVER_SUBSCRIPTION_ID = 8190001, SOLANA_ERROR__RPC_SUBSCRIPTIONS__CHANNEL_CLOSED_BEFORE_MESSAGE_BUFFERED = 8190002, SOLANA_ERROR__RPC_SUBSCRIPTIONS__CHANNEL_CONNECTION_CLOSED = 8190003, SOLANA_ERROR__RPC_SUBSCRIPTIONS__CHANNEL_FAILED_TO_CONNECT = 8190004, SOLANA_ERROR__INVARIANT_VIOLATION__SUBSCRIPTION_ITERATOR_STATE_MISSING = 99e5, SOLANA_ERROR__INVARIANT_VIOLATION__SUBSCRIPTION_ITERATOR_MUST_NOT_POLL_BEFORE_RESOLVING_EXISTING_MESSAGE_PROMISE = 9900001, SOLANA_ERROR__INVARIANT_VIOLATION__CACHED_ABORTABLE_ITERABLE_CACHE_ENTRY_MISSING = 9900002, SOLANA_ERROR__INVARIANT_VIOLATION__SWITCH_MUST_BE_EXHAUSTIVE = 9900003, SOLANA_ERROR__INVARIANT_VIOLATION__DATA_PUBLISHER_CHANNEL_UNIMPLEMENTED = 9900004;

function encodeValue(value) {
    if (Array.isArray(value)) {
        return "%5B" + value.map(encodeValue).join("%2C%20") + "%5D";
    }
    return "bigint" == typeof value ? `${value}n` : encodeURIComponent(String(null != value && null === Object.getPrototypeOf(value) ? {
        ...value
    } : value));
}

function encodeObjectContextEntry([key, value]) {
    return `${key}=${encodeValue(value)}`;
}

function encodeContextObject(context) {
    const searchParamsString = Object.entries(context).map(encodeObjectContextEntry).join("&");
    return Buffer.from(searchParamsString, "utf8").toString("base64");
}

var SolanaErrorMessages = {
    [SOLANA_ERROR__ACCOUNTS__ACCOUNT_NOT_FOUND]: "Account not found at address: $address",
    [SOLANA_ERROR__ACCOUNTS__EXPECTED_ALL_ACCOUNTS_TO_BE_DECODED]: "Not all accounts were decoded. Encoded accounts found at addresses: $addresses.",
    [SOLANA_ERROR__ACCOUNTS__EXPECTED_DECODED_ACCOUNT]: "Expected decoded account at address: $address",
    [SOLANA_ERROR__ACCOUNTS__FAILED_TO_DECODE_ACCOUNT]: "Failed to decode account data at address: $address",
    [SOLANA_ERROR__ACCOUNTS__ONE_OR_MORE_ACCOUNTS_NOT_FOUND]: "Accounts not found at addresses: $addresses",
    [SOLANA_ERROR__ADDRESSES__FAILED_TO_FIND_VIABLE_PDA_BUMP_SEED]: "Unable to find a viable program address bump seed.",
    [SOLANA_ERROR__ADDRESSES__INVALID_BASE58_ENCODED_ADDRESS]: "$putativeAddress is not a base58-encoded address.",
    [SOLANA_ERROR__ADDRESSES__INVALID_BYTE_LENGTH]: "Expected base58 encoded address to decode to a byte array of length 32. Actual length: $actualLength.",
    [SOLANA_ERROR__ADDRESSES__INVALID_ED25519_PUBLIC_KEY]: "The `CryptoKey` must be an `Ed25519` public key.",
    [SOLANA_ERROR__ADDRESSES__INVALID_SEEDS_POINT_ON_CURVE]: "Invalid seeds; point must fall off the Ed25519 curve.",
    [SOLANA_ERROR__ADDRESSES__MALFORMED_PDA]: "Expected given program derived address to have the following format: [Address, ProgramDerivedAddressBump].",
    [SOLANA_ERROR__ADDRESSES__MAX_NUMBER_OF_PDA_SEEDS_EXCEEDED]: "A maximum of $maxSeeds seeds, including the bump seed, may be supplied when creating an address. Received: $actual.",
    [SOLANA_ERROR__ADDRESSES__MAX_PDA_SEED_LENGTH_EXCEEDED]: "The seed at index $index with length $actual exceeds the maximum length of $maxSeedLength bytes.",
    [SOLANA_ERROR__ADDRESSES__PDA_BUMP_SEED_OUT_OF_RANGE]: "Expected program derived address bump to be in the range [0, 255], got: $bump.",
    [SOLANA_ERROR__ADDRESSES__PDA_ENDS_WITH_PDA_MARKER]: "Program address cannot end with PDA marker.",
    [SOLANA_ERROR__ADDRESSES__STRING_LENGTH_OUT_OF_RANGE]: "Expected base58-encoded address string of length in the range [32, 44]. Actual length: $actualLength.",
    [SOLANA_ERROR__BLOCKHASH_STRING_LENGTH_OUT_OF_RANGE]: "Expected base58-encoded blockash string of length in the range [32, 44]. Actual length: $actualLength.",
    [SOLANA_ERROR__BLOCK_HEIGHT_EXCEEDED]: "The network has progressed past the last block for which this transaction could have been committed.",
    [SOLANA_ERROR__CODECS__CANNOT_DECODE_EMPTY_BYTE_ARRAY]: "Codec [$codecDescription] cannot decode empty byte arrays.",
    [SOLANA_ERROR__CODECS__CANNOT_USE_LEXICAL_VALUES_AS_ENUM_DISCRIMINATORS]: "Enum codec cannot use lexical values [$stringValues] as discriminators. Either remove all lexical values or set `useValuesAsDiscriminators` to `false`.",
    [SOLANA_ERROR__CODECS__ENCODED_BYTES_MUST_NOT_INCLUDE_SENTINEL]: "Sentinel [$hexSentinel] must not be present in encoded bytes [$hexEncodedBytes].",
    [SOLANA_ERROR__CODECS__ENCODER_DECODER_FIXED_SIZE_MISMATCH]: "Encoder and decoder must have the same fixed size, got [$encoderFixedSize] and [$decoderFixedSize].",
    [SOLANA_ERROR__CODECS__ENCODER_DECODER_MAX_SIZE_MISMATCH]: "Encoder and decoder must have the same max size, got [$encoderMaxSize] and [$decoderMaxSize].",
    [SOLANA_ERROR__CODECS__ENCODER_DECODER_SIZE_COMPATIBILITY_MISMATCH]: "Encoder and decoder must either both be fixed-size or variable-size.",
    [SOLANA_ERROR__CODECS__ENUM_DISCRIMINATOR_OUT_OF_RANGE]: "Enum discriminator out of range. Expected a number in [$formattedValidDiscriminators], got $discriminator.",
    [SOLANA_ERROR__CODECS__EXPECTED_FIXED_LENGTH]: "Expected a fixed-size codec, got a variable-size one.",
    [SOLANA_ERROR__CODECS__EXPECTED_POSITIVE_BYTE_LENGTH]: "Codec [$codecDescription] expected a positive byte length, got $bytesLength.",
    [SOLANA_ERROR__CODECS__EXPECTED_VARIABLE_LENGTH]: "Expected a variable-size codec, got a fixed-size one.",
    [SOLANA_ERROR__CODECS__EXPECTED_ZERO_VALUE_TO_MATCH_ITEM_FIXED_SIZE]: "Codec [$codecDescription] expected zero-value [$hexZeroValue] to have the same size as the provided fixed-size item [$expectedSize bytes].",
    [SOLANA_ERROR__CODECS__INVALID_BYTE_LENGTH]: "Codec [$codecDescription] expected $expected bytes, got $bytesLength.",
    [SOLANA_ERROR__CODECS__INVALID_CONSTANT]: "Expected byte array constant [$hexConstant] to be present in data [$hexData] at offset [$offset].",
    [SOLANA_ERROR__CODECS__INVALID_DISCRIMINATED_UNION_VARIANT]: "Invalid discriminated union variant. Expected one of [$variants], got $value.",
    [SOLANA_ERROR__CODECS__INVALID_ENUM_VARIANT]: "Invalid enum variant. Expected one of [$stringValues] or a number in [$formattedNumericalValues], got $variant.",
    [SOLANA_ERROR__CODECS__INVALID_LITERAL_UNION_VARIANT]: "Invalid literal union variant. Expected one of [$variants], got $value.",
    [SOLANA_ERROR__CODECS__INVALID_NUMBER_OF_ITEMS]: "Expected [$codecDescription] to have $expected items, got $actual.",
    [SOLANA_ERROR__CODECS__INVALID_STRING_FOR_BASE]: "Invalid value $value for base $base with alphabet $alphabet.",
    [SOLANA_ERROR__CODECS__LITERAL_UNION_DISCRIMINATOR_OUT_OF_RANGE]: "Literal union discriminator out of range. Expected a number between $minRange and $maxRange, got $discriminator.",
    [SOLANA_ERROR__CODECS__NUMBER_OUT_OF_RANGE]: "Codec [$codecDescription] expected number to be in the range [$min, $max], got $value.",
    [SOLANA_ERROR__CODECS__OFFSET_OUT_OF_RANGE]: "Codec [$codecDescription] expected offset to be in the range [0, $bytesLength], got $offset.",
    [SOLANA_ERROR__CODECS__SENTINEL_MISSING_IN_DECODED_BYTES]: "Expected sentinel [$hexSentinel] to be present in decoded bytes [$hexDecodedBytes].",
    [SOLANA_ERROR__CODECS__UNION_VARIANT_OUT_OF_RANGE]: "Union variant out of range. Expected an index between $minRange and $maxRange, got $variant.",
    [SOLANA_ERROR__CRYPTO__RANDOM_VALUES_FUNCTION_UNIMPLEMENTED]: "No random values implementation could be found.",
    [SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_ALREADY_INITIALIZED]: "instruction requires an uninitialized account",
    [SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_BORROW_FAILED]: "instruction tries to borrow reference for an account which is already borrowed",
    [SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_BORROW_OUTSTANDING]: "instruction left account with an outstanding borrowed reference",
    [SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_DATA_SIZE_CHANGED]: "program other than the account's owner changed the size of the account data",
    [SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_DATA_TOO_SMALL]: "account data too small for instruction",
    [SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_NOT_EXECUTABLE]: "instruction expected an executable account",
    [SOLANA_ERROR__INSTRUCTION_ERROR__ACCOUNT_NOT_RENT_EXEMPT]: "An account does not have enough lamports to be rent-exempt",
    [SOLANA_ERROR__INSTRUCTION_ERROR__ARITHMETIC_OVERFLOW]: "Program arithmetic overflowed",
    [SOLANA_ERROR__INSTRUCTION_ERROR__BORSH_IO_ERROR]: "Failed to serialize or deserialize account data: $encodedData",
    [SOLANA_ERROR__INSTRUCTION_ERROR__BUILTIN_PROGRAMS_MUST_CONSUME_COMPUTE_UNITS]: "Builtin programs must consume compute units",
    [SOLANA_ERROR__INSTRUCTION_ERROR__CALL_DEPTH]: "Cross-program invocation call depth too deep",
    [SOLANA_ERROR__INSTRUCTION_ERROR__COMPUTATIONAL_BUDGET_EXCEEDED]: "Computational budget exceeded",
    [SOLANA_ERROR__INSTRUCTION_ERROR__CUSTOM]: "custom program error: #$code",
    [SOLANA_ERROR__INSTRUCTION_ERROR__DUPLICATE_ACCOUNT_INDEX]: "instruction contains duplicate accounts",
    [SOLANA_ERROR__INSTRUCTION_ERROR__DUPLICATE_ACCOUNT_OUT_OF_SYNC]: "instruction modifications of multiply-passed account differ",
    [SOLANA_ERROR__INSTRUCTION_ERROR__EXECUTABLE_ACCOUNT_NOT_RENT_EXEMPT]: "executable accounts must be rent exempt",
    [SOLANA_ERROR__INSTRUCTION_ERROR__EXECUTABLE_DATA_MODIFIED]: "instruction changed executable accounts data",
    [SOLANA_ERROR__INSTRUCTION_ERROR__EXECUTABLE_LAMPORT_CHANGE]: "instruction changed the balance of an executable account",
    [SOLANA_ERROR__INSTRUCTION_ERROR__EXECUTABLE_MODIFIED]: "instruction changed executable bit of an account",
    [SOLANA_ERROR__INSTRUCTION_ERROR__EXTERNAL_ACCOUNT_DATA_MODIFIED]: "instruction modified data of an account it does not own",
    [SOLANA_ERROR__INSTRUCTION_ERROR__EXTERNAL_ACCOUNT_LAMPORT_SPEND]: "instruction spent from the balance of an account it does not own",
    [SOLANA_ERROR__INSTRUCTION_ERROR__GENERIC_ERROR]: "generic instruction error",
    [SOLANA_ERROR__INSTRUCTION_ERROR__ILLEGAL_OWNER]: "Provided owner is not allowed",
    [SOLANA_ERROR__INSTRUCTION_ERROR__IMMUTABLE]: "Account is immutable",
    [SOLANA_ERROR__INSTRUCTION_ERROR__INCORRECT_AUTHORITY]: "Incorrect authority provided",
    [SOLANA_ERROR__INSTRUCTION_ERROR__INCORRECT_PROGRAM_ID]: "incorrect program id for instruction",
    [SOLANA_ERROR__INSTRUCTION_ERROR__INSUFFICIENT_FUNDS]: "insufficient funds for instruction",
    [SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_ACCOUNT_DATA]: "invalid account data for instruction",
    [SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_ACCOUNT_OWNER]: "Invalid account owner",
    [SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_ARGUMENT]: "invalid program argument",
    [SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_ERROR]: "program returned invalid error code",
    [SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_INSTRUCTION_DATA]: "invalid instruction data",
    [SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_REALLOC]: "Failed to reallocate account data",
    [SOLANA_ERROR__INSTRUCTION_ERROR__INVALID_SEEDS]: "Provided seeds do not result in a valid address",
    [SOLANA_ERROR__INSTRUCTION_ERROR__MAX_ACCOUNTS_DATA_ALLOCATIONS_EXCEEDED]: "Accounts data allocations exceeded the maximum allowed per transaction",
    [SOLANA_ERROR__INSTRUCTION_ERROR__MAX_ACCOUNTS_EXCEEDED]: "Max accounts exceeded",
    [SOLANA_ERROR__INSTRUCTION_ERROR__MAX_INSTRUCTION_TRACE_LENGTH_EXCEEDED]: "Max instruction trace length exceeded",
    [SOLANA_ERROR__INSTRUCTION_ERROR__MAX_SEED_LENGTH_EXCEEDED]: "Length of the seed is too long for address generation",
    [SOLANA_ERROR__INSTRUCTION_ERROR__MISSING_ACCOUNT]: "An account required by the instruction is missing",
    [SOLANA_ERROR__INSTRUCTION_ERROR__MISSING_REQUIRED_SIGNATURE]: "missing required signature for instruction",
    [SOLANA_ERROR__INSTRUCTION_ERROR__MODIFIED_PROGRAM_ID]: "instruction illegally modified the program id of an account",
    [SOLANA_ERROR__INSTRUCTION_ERROR__NOT_ENOUGH_ACCOUNT_KEYS]: "insufficient account keys for instruction",
    [SOLANA_ERROR__INSTRUCTION_ERROR__PRIVILEGE_ESCALATION]: "Cross-program invocation with unauthorized signer or writable account",
    [SOLANA_ERROR__INSTRUCTION_ERROR__PROGRAM_ENVIRONMENT_SETUP_FAILURE]: "Failed to create program execution environment",
    [SOLANA_ERROR__INSTRUCTION_ERROR__PROGRAM_FAILED_TO_COMPILE]: "Program failed to compile",
    [SOLANA_ERROR__INSTRUCTION_ERROR__PROGRAM_FAILED_TO_COMPLETE]: "Program failed to complete",
    [SOLANA_ERROR__INSTRUCTION_ERROR__READONLY_DATA_MODIFIED]: "instruction modified data of a read-only account",
    [SOLANA_ERROR__INSTRUCTION_ERROR__READONLY_LAMPORT_CHANGE]: "instruction changed the balance of a read-only account",
    [SOLANA_ERROR__INSTRUCTION_ERROR__REENTRANCY_NOT_ALLOWED]: "Cross-program invocation reentrancy not allowed for this instruction",
    [SOLANA_ERROR__INSTRUCTION_ERROR__RENT_EPOCH_MODIFIED]: "instruction modified rent epoch of an account",
    [SOLANA_ERROR__INSTRUCTION_ERROR__UNBALANCED_INSTRUCTION]: "sum of account balances before and after instruction do not match",
    [SOLANA_ERROR__INSTRUCTION_ERROR__UNINITIALIZED_ACCOUNT]: "instruction requires an initialized account",
    [SOLANA_ERROR__INSTRUCTION_ERROR__UNKNOWN]: "",
    [SOLANA_ERROR__INSTRUCTION_ERROR__UNSUPPORTED_PROGRAM_ID]: "Unsupported program id",
    [SOLANA_ERROR__INSTRUCTION_ERROR__UNSUPPORTED_SYSVAR]: "Unsupported sysvar",
    [SOLANA_ERROR__INSTRUCTION__EXPECTED_TO_HAVE_ACCOUNTS]: "The instruction does not have any accounts.",
    [SOLANA_ERROR__INSTRUCTION__EXPECTED_TO_HAVE_DATA]: "The instruction does not have any data.",
    [SOLANA_ERROR__INSTRUCTION__PROGRAM_ID_MISMATCH]: "Expected instruction to have progress address $expectedProgramAddress, got $actualProgramAddress.",
    [SOLANA_ERROR__INVALID_BLOCKHASH_BYTE_LENGTH]: "Expected base58 encoded blockhash to decode to a byte array of length 32. Actual length: $actualLength.",
    [SOLANA_ERROR__INVALID_NONCE]: "The nonce `$expectedNonceValue` is no longer valid. It has advanced to `$actualNonceValue`",
    [SOLANA_ERROR__INVARIANT_VIOLATION__CACHED_ABORTABLE_ITERABLE_CACHE_ENTRY_MISSING]: "Invariant violation: Found no abortable iterable cache entry for key `$cacheKey`. It should be impossible to hit this error; please file an issue at https://sola.na/web3invariant",
    [SOLANA_ERROR__INVARIANT_VIOLATION__DATA_PUBLISHER_CHANNEL_UNIMPLEMENTED]: "Invariant violation: This data publisher does not publish to the channel named `$channelName`. Supported channels include $supportedChannelNames.",
    [SOLANA_ERROR__INVARIANT_VIOLATION__SUBSCRIPTION_ITERATOR_MUST_NOT_POLL_BEFORE_RESOLVING_EXISTING_MESSAGE_PROMISE]: "Invariant violation: WebSocket message iterator state is corrupt; iterated without first resolving existing message promise. It should be impossible to hit this error; please file an issue at https://sola.na/web3invariant",
    [SOLANA_ERROR__INVARIANT_VIOLATION__SUBSCRIPTION_ITERATOR_STATE_MISSING]: "Invariant violation: WebSocket message iterator is missing state storage. It should be impossible to hit this error; please file an issue at https://sola.na/web3invariant",
    [SOLANA_ERROR__INVARIANT_VIOLATION__SWITCH_MUST_BE_EXHAUSTIVE]: "Invariant violation: Switch statement non-exhaustive. Received unexpected value `$unexpectedValue`. It should be impossible to hit this error; please file an issue at https://sola.na/web3invariant",
    [SOLANA_ERROR__JSON_RPC__INTERNAL_ERROR]: "JSON-RPC error: Internal JSON-RPC error ($__serverMessage)",
    [SOLANA_ERROR__JSON_RPC__INVALID_PARAMS]: "JSON-RPC error: Invalid method parameter(s) ($__serverMessage)",
    [SOLANA_ERROR__JSON_RPC__INVALID_REQUEST]: "JSON-RPC error: The JSON sent is not a valid `Request` object ($__serverMessage)",
    [SOLANA_ERROR__JSON_RPC__METHOD_NOT_FOUND]: "JSON-RPC error: The method does not exist / is not available ($__serverMessage)",
    [SOLANA_ERROR__JSON_RPC__PARSE_ERROR]: "JSON-RPC error: An error occurred on the server while parsing the JSON text ($__serverMessage)",
    [SOLANA_ERROR__JSON_RPC__SCAN_ERROR]: "$__serverMessage",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_BLOCK_CLEANED_UP]: "$__serverMessage",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_BLOCK_NOT_AVAILABLE]: "$__serverMessage",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_BLOCK_STATUS_NOT_AVAILABLE_YET]: "$__serverMessage",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_KEY_EXCLUDED_FROM_SECONDARY_INDEX]: "$__serverMessage",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_LONG_TERM_STORAGE_SLOT_SKIPPED]: "$__serverMessage",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_MIN_CONTEXT_SLOT_NOT_REACHED]: "Minimum context slot has not been reached",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_NODE_UNHEALTHY]: "Node is unhealthy; behind by $numSlotsBehind slots",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_NO_SNAPSHOT]: "No snapshot",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_SEND_TRANSACTION_PREFLIGHT_FAILURE]: "Transaction simulation failed",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_SLOT_SKIPPED]: "$__serverMessage",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_TRANSACTION_HISTORY_NOT_AVAILABLE]: "Transaction history is not available from this node",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_TRANSACTION_PRECOMPILE_VERIFICATION_FAILURE]: "$__serverMessage",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_TRANSACTION_SIGNATURE_LEN_MISMATCH]: "Transaction signature length mismatch",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_TRANSACTION_SIGNATURE_VERIFICATION_FAILURE]: "Transaction signature verification failure",
    [SOLANA_ERROR__JSON_RPC__SERVER_ERROR_UNSUPPORTED_TRANSACTION_VERSION]: "$__serverMessage",
    [SOLANA_ERROR__KEYS__INVALID_KEY_PAIR_BYTE_LENGTH]: "Key pair bytes must be of length 64, got $byteLength.",
    [SOLANA_ERROR__KEYS__INVALID_PRIVATE_KEY_BYTE_LENGTH]: "Expected private key bytes with length 32. Actual length: $actualLength.",
    [SOLANA_ERROR__KEYS__INVALID_SIGNATURE_BYTE_LENGTH]: "Expected base58-encoded signature to decode to a byte array of length 64. Actual length: $actualLength.",
    [SOLANA_ERROR__KEYS__PUBLIC_KEY_MUST_MATCH_PRIVATE_KEY]: "The provided private key does not match the provided public key.",
    [SOLANA_ERROR__KEYS__SIGNATURE_STRING_LENGTH_OUT_OF_RANGE]: "Expected base58-encoded signature string of length in the range [64, 88]. Actual length: $actualLength.",
    [SOLANA_ERROR__LAMPORTS_OUT_OF_RANGE]: "Lamports value must be in the range [0, 2e64-1]",
    [SOLANA_ERROR__MALFORMED_BIGINT_STRING]: "`$value` cannot be parsed as a `BigInt`",
    [SOLANA_ERROR__MALFORMED_NUMBER_STRING]: "`$value` cannot be parsed as a `Number`",
    [SOLANA_ERROR__NONCE_ACCOUNT_NOT_FOUND]: "No nonce account could be found at address `$nonceAccountAddress`",
    [SOLANA_ERROR__RPC_SUBSCRIPTIONS__CANNOT_CREATE_SUBSCRIPTION_PLAN]: "The notification name must end in 'Notifications' and the API must supply a subscription plan creator function for the notification '$notificationName'.",
    [SOLANA_ERROR__RPC_SUBSCRIPTIONS__CHANNEL_CLOSED_BEFORE_MESSAGE_BUFFERED]: "WebSocket was closed before payload could be added to the send buffer",
    [SOLANA_ERROR__RPC_SUBSCRIPTIONS__CHANNEL_CONNECTION_CLOSED]: "WebSocket connection closed",
    [SOLANA_ERROR__RPC_SUBSCRIPTIONS__CHANNEL_FAILED_TO_CONNECT]: "WebSocket failed to connect",
    [SOLANA_ERROR__RPC_SUBSCRIPTIONS__EXPECTED_SERVER_SUBSCRIPTION_ID]: "Failed to obtain a subscription id from the server",
    [SOLANA_ERROR__RPC__API_PLAN_MISSING_FOR_RPC_METHOD]: "Could not find an API plan for RPC method: `$method`",
    [SOLANA_ERROR__RPC__INTEGER_OVERFLOW]: "The $argumentLabel argument to the `$methodName` RPC method$optionalPathLabel was `$value`. This number is unsafe for use with the Solana JSON-RPC because it exceeds `Number.MAX_SAFE_INTEGER`.",
    [SOLANA_ERROR__RPC__TRANSPORT_HTTP_ERROR]: "HTTP error ($statusCode): $message",
    [SOLANA_ERROR__RPC__TRANSPORT_HTTP_HEADER_FORBIDDEN]: "HTTP header(s) forbidden: $headers. Learn more at https://developer.mozilla.org/en-US/docs/Glossary/Forbidden_header_name.",
    [SOLANA_ERROR__SIGNER__ADDRESS_CANNOT_HAVE_MULTIPLE_SIGNERS]: "Multiple distinct signers were identified for address `$address`. Please ensure that you are using the same signer instance for each address.",
    [SOLANA_ERROR__SIGNER__EXPECTED_KEY_PAIR_SIGNER]: "The provided value does not implement the `KeyPairSigner` interface",
    [SOLANA_ERROR__SIGNER__EXPECTED_MESSAGE_MODIFYING_SIGNER]: "The provided value does not implement the `MessageModifyingSigner` interface",
    [SOLANA_ERROR__SIGNER__EXPECTED_MESSAGE_PARTIAL_SIGNER]: "The provided value does not implement the `MessagePartialSigner` interface",
    [SOLANA_ERROR__SIGNER__EXPECTED_MESSAGE_SIGNER]: "The provided value does not implement any of the `MessageSigner` interfaces",
    [SOLANA_ERROR__SIGNER__EXPECTED_TRANSACTION_MODIFYING_SIGNER]: "The provided value does not implement the `TransactionModifyingSigner` interface",
    [SOLANA_ERROR__SIGNER__EXPECTED_TRANSACTION_PARTIAL_SIGNER]: "The provided value does not implement the `TransactionPartialSigner` interface",
    [SOLANA_ERROR__SIGNER__EXPECTED_TRANSACTION_SENDING_SIGNER]: "The provided value does not implement the `TransactionSendingSigner` interface",
    [SOLANA_ERROR__SIGNER__EXPECTED_TRANSACTION_SIGNER]: "The provided value does not implement any of the `TransactionSigner` interfaces",
    [SOLANA_ERROR__SIGNER__TRANSACTION_CANNOT_HAVE_MULTIPLE_SENDING_SIGNERS]: "More than one `TransactionSendingSigner` was identified.",
    [SOLANA_ERROR__SIGNER__TRANSACTION_SENDING_SIGNER_MISSING]: "No `TransactionSendingSigner` was identified. Please provide a valid `ITransactionWithSingleSendingSigner` transaction.",
    [SOLANA_ERROR__SIGNER__WALLET_MULTISIGN_UNIMPLEMENTED]: "Wallet account signers do not support signing multiple messages/transactions in a single operation",
    [SOLANA_ERROR__SUBTLE_CRYPTO__CANNOT_EXPORT_NON_EXTRACTABLE_KEY]: "Cannot export a non-extractable key.",
    [SOLANA_ERROR__SUBTLE_CRYPTO__DIGEST_UNIMPLEMENTED]: "No digest implementation could be found.",
    [SOLANA_ERROR__SUBTLE_CRYPTO__DISALLOWED_IN_INSECURE_CONTEXT]: "Cryptographic operations are only allowed in secure browser contexts. Read more here: https://developer.mozilla.org/en-US/docs/Web/Security/Secure_Contexts.",
    [SOLANA_ERROR__SUBTLE_CRYPTO__ED25519_ALGORITHM_UNIMPLEMENTED]: "This runtime does not support the generation of Ed25519 key pairs.\n\nInstall @solana/webcrypto-ed25519-polyfill and call its `install` function before generating keys in environments that do not support Ed25519.\n\nFor a list of runtimes that currently support Ed25519 operations, visit https://github.com/WICG/webcrypto-secure-curves/issues/20.",
    [SOLANA_ERROR__SUBTLE_CRYPTO__EXPORT_FUNCTION_UNIMPLEMENTED]: "No signature verification implementation could be found.",
    [SOLANA_ERROR__SUBTLE_CRYPTO__GENERATE_FUNCTION_UNIMPLEMENTED]: "No key generation implementation could be found.",
    [SOLANA_ERROR__SUBTLE_CRYPTO__SIGN_FUNCTION_UNIMPLEMENTED]: "No signing implementation could be found.",
    [SOLANA_ERROR__SUBTLE_CRYPTO__VERIFY_FUNCTION_UNIMPLEMENTED]: "No key export implementation could be found.",
    [SOLANA_ERROR__TIMESTAMP_OUT_OF_RANGE]: "Timestamp value must be in the range [-(2n ** 63n), (2n ** 63n) - 1]. `$value` given",
    [SOLANA_ERROR__TRANSACTION_ERROR__ACCOUNT_BORROW_OUTSTANDING]: "Transaction processing left an account with an outstanding borrowed reference",
    [SOLANA_ERROR__TRANSACTION_ERROR__ACCOUNT_IN_USE]: "Account in use",
    [SOLANA_ERROR__TRANSACTION_ERROR__ACCOUNT_LOADED_TWICE]: "Account loaded twice",
    [SOLANA_ERROR__TRANSACTION_ERROR__ACCOUNT_NOT_FOUND]: "Attempt to debit an account but found no record of a prior credit.",
    [SOLANA_ERROR__TRANSACTION_ERROR__ADDRESS_LOOKUP_TABLE_NOT_FOUND]: "Transaction loads an address table account that doesn't exist",
    [SOLANA_ERROR__TRANSACTION_ERROR__ALREADY_PROCESSED]: "This transaction has already been processed",
    [SOLANA_ERROR__TRANSACTION_ERROR__BLOCKHASH_NOT_FOUND]: "Blockhash not found",
    [SOLANA_ERROR__TRANSACTION_ERROR__CALL_CHAIN_TOO_DEEP]: "Loader call chain is too deep",
    [SOLANA_ERROR__TRANSACTION_ERROR__CLUSTER_MAINTENANCE]: "Transactions are currently disabled due to cluster maintenance",
    [SOLANA_ERROR__TRANSACTION_ERROR__DUPLICATE_INSTRUCTION]: "Transaction contains a duplicate instruction ($index) that is not allowed",
    [SOLANA_ERROR__TRANSACTION_ERROR__INSUFFICIENT_FUNDS_FOR_FEE]: "Insufficient funds for fee",
    [SOLANA_ERROR__TRANSACTION_ERROR__INSUFFICIENT_FUNDS_FOR_RENT]: "Transaction results in an account ($accountIndex) with insufficient funds for rent",
    [SOLANA_ERROR__TRANSACTION_ERROR__INVALID_ACCOUNT_FOR_FEE]: "This account may not be used to pay transaction fees",
    [SOLANA_ERROR__TRANSACTION_ERROR__INVALID_ACCOUNT_INDEX]: "Transaction contains an invalid account reference",
    [SOLANA_ERROR__TRANSACTION_ERROR__INVALID_ADDRESS_LOOKUP_TABLE_DATA]: "Transaction loads an address table account with invalid data",
    [SOLANA_ERROR__TRANSACTION_ERROR__INVALID_ADDRESS_LOOKUP_TABLE_INDEX]: "Transaction address table lookup uses an invalid index",
    [SOLANA_ERROR__TRANSACTION_ERROR__INVALID_ADDRESS_LOOKUP_TABLE_OWNER]: "Transaction loads an address table account with an invalid owner",
    [SOLANA_ERROR__TRANSACTION_ERROR__INVALID_LOADED_ACCOUNTS_DATA_SIZE_LIMIT]: "LoadedAccountsDataSizeLimit set for transaction must be greater than 0.",
    [SOLANA_ERROR__TRANSACTION_ERROR__INVALID_PROGRAM_FOR_EXECUTION]: "This program may not be used for executing instructions",
    [SOLANA_ERROR__TRANSACTION_ERROR__INVALID_RENT_PAYING_ACCOUNT]: "Transaction leaves an account with a lower balance than rent-exempt minimum",
    [SOLANA_ERROR__TRANSACTION_ERROR__INVALID_WRITABLE_ACCOUNT]: "Transaction loads a writable account that cannot be written",
    [SOLANA_ERROR__TRANSACTION_ERROR__MAX_LOADED_ACCOUNTS_DATA_SIZE_EXCEEDED]: "Transaction exceeded max loaded accounts data size cap",
    [SOLANA_ERROR__TRANSACTION_ERROR__MISSING_SIGNATURE_FOR_FEE]: "Transaction requires a fee but has no signature present",
    [SOLANA_ERROR__TRANSACTION_ERROR__PROGRAM_ACCOUNT_NOT_FOUND]: "Attempt to load a program that does not exist",
    [SOLANA_ERROR__TRANSACTION_ERROR__PROGRAM_EXECUTION_TEMPORARILY_RESTRICTED]: "Execution of the program referenced by account at index $accountIndex is temporarily restricted.",
    [SOLANA_ERROR__TRANSACTION_ERROR__RESANITIZATION_NEEDED]: "ResanitizationNeeded",
    [SOLANA_ERROR__TRANSACTION_ERROR__SANITIZE_FAILURE]: "Transaction failed to sanitize accounts offsets correctly",
    [SOLANA_ERROR__TRANSACTION_ERROR__SIGNATURE_FAILURE]: "Transaction did not pass signature verification",
    [SOLANA_ERROR__TRANSACTION_ERROR__TOO_MANY_ACCOUNT_LOCKS]: "Transaction locked too many accounts",
    [SOLANA_ERROR__TRANSACTION_ERROR__UNBALANCED_TRANSACTION]: "Sum of account balances before and after transaction do not match",
    [SOLANA_ERROR__TRANSACTION_ERROR__UNKNOWN]: "The transaction failed with the error `$errorName`",
    [SOLANA_ERROR__TRANSACTION_ERROR__UNSUPPORTED_VERSION]: "Transaction version is unsupported",
    [SOLANA_ERROR__TRANSACTION_ERROR__WOULD_EXCEED_ACCOUNT_DATA_BLOCK_LIMIT]: "Transaction would exceed account data limit within the block",
    [SOLANA_ERROR__TRANSACTION_ERROR__WOULD_EXCEED_ACCOUNT_DATA_TOTAL_LIMIT]: "Transaction would exceed total account data limit",
    [SOLANA_ERROR__TRANSACTION_ERROR__WOULD_EXCEED_MAX_ACCOUNT_COST_LIMIT]: "Transaction would exceed max account limit within the block",
    [SOLANA_ERROR__TRANSACTION_ERROR__WOULD_EXCEED_MAX_BLOCK_COST_LIMIT]: "Transaction would exceed max Block Cost Limit",
    [SOLANA_ERROR__TRANSACTION_ERROR__WOULD_EXCEED_MAX_VOTE_COST_LIMIT]: "Transaction would exceed max Vote Cost Limit",
    [SOLANA_ERROR__TRANSACTION__ADDRESSES_CANNOT_SIGN_TRANSACTION]: "Attempted to sign a transaction with an address that is not a signer for it",
    [SOLANA_ERROR__TRANSACTION__ADDRESS_MISSING]: "Transaction is missing an address at index: $index.",
    [SOLANA_ERROR__TRANSACTION__CANNOT_ENCODE_WITH_EMPTY_SIGNATURES]: "Transaction has no expected signers therefore it cannot be encoded",
    [SOLANA_ERROR__TRANSACTION__EXPECTED_BLOCKHASH_LIFETIME]: "Transaction does not have a blockhash lifetime",
    [SOLANA_ERROR__TRANSACTION__EXPECTED_NONCE_LIFETIME]: "Transaction is not a durable nonce transaction",
    [SOLANA_ERROR__TRANSACTION__FAILED_TO_DECOMPILE_ADDRESS_LOOKUP_TABLE_CONTENTS_MISSING]: "Contents of these address lookup tables unknown: $lookupTableAddresses",
    [SOLANA_ERROR__TRANSACTION__FAILED_TO_DECOMPILE_ADDRESS_LOOKUP_TABLE_INDEX_OUT_OF_RANGE]: "Lookup of address at index $highestRequestedIndex failed for lookup table `$lookupTableAddress`. Highest known index is $highestKnownIndex. The lookup table may have been extended since its contents were retrieved",
    [SOLANA_ERROR__TRANSACTION__FAILED_TO_DECOMPILE_FEE_PAYER_MISSING]: "No fee payer set in CompiledTransaction",
    [SOLANA_ERROR__TRANSACTION__FAILED_TO_DECOMPILE_INSTRUCTION_PROGRAM_ADDRESS_NOT_FOUND]: "Could not find program address at index $index",
    [SOLANA_ERROR__TRANSACTION__FAILED_TO_ESTIMATE_COMPUTE_LIMIT]: "Failed to estimate the compute unit consumption for this transaction message. This is likely because simulating the transaction failed. Inspect the `cause` property of this error to learn more",
    [SOLANA_ERROR__TRANSACTION__FAILED_WHEN_SIMULATING_TO_ESTIMATE_COMPUTE_LIMIT]: "Transaction failed when it was simulated in order to estimate the compute unit consumption. The compute unit estimate provided is for a transaction that failed when simulated and may not be representative of the compute units this transaction would consume if successful. Inspect the `cause` property of this error to learn more",
    [SOLANA_ERROR__TRANSACTION__FEE_PAYER_MISSING]: "Transaction is missing a fee payer.",
    [SOLANA_ERROR__TRANSACTION__FEE_PAYER_SIGNATURE_MISSING]: "Could not determine this transaction's signature. Make sure that the transaction has been signed by its fee payer.",
    [SOLANA_ERROR__TRANSACTION__INVALID_NONCE_TRANSACTION_FIRST_INSTRUCTION_MUST_BE_ADVANCE_NONCE]: "Transaction first instruction is not advance nonce account instruction.",
    [SOLANA_ERROR__TRANSACTION__INVALID_NONCE_TRANSACTION_INSTRUCTIONS_MISSING]: "Transaction with no instructions cannot be durable nonce transaction.",
    [SOLANA_ERROR__TRANSACTION__INVOKED_PROGRAMS_CANNOT_PAY_FEES]: "This transaction includes an address (`$programAddress`) which is both invoked and set as the fee payer. Program addresses may not pay fees",
    [SOLANA_ERROR__TRANSACTION__INVOKED_PROGRAMS_MUST_NOT_BE_WRITABLE]: "This transaction includes an address (`$programAddress`) which is both invoked and marked writable. Program addresses may not be writable",
    [SOLANA_ERROR__TRANSACTION__MESSAGE_SIGNATURES_MISMATCH]: "The transaction message expected the transaction to have $signerAddressesLength signatures, got $signaturesLength.",
    [SOLANA_ERROR__TRANSACTION__SIGNATURES_MISSING]: "Transaction is missing signatures for addresses: $addresses.",
    [SOLANA_ERROR__TRANSACTION__VERSION_NUMBER_OUT_OF_RANGE]: "Transaction version must be in the range [0, 127]. `$actualVersion` given"
}, START_INDEX = "i", TYPE = "t";

function getHumanReadableErrorMessage(code, context = {}) {
    const messageFormatString = SolanaErrorMessages[code];
    if (0 === messageFormatString.length) return "";
    let state;
    function commitStateUpTo(endIndex) {
        if (2 === state[TYPE]) {
            const variableName = messageFormatString.slice(state[START_INDEX] + 1, endIndex);
            fragments.push(variableName in context ? `${context[variableName]}` : `$${variableName}`);
        } else 1 === state[TYPE] && fragments.push(messageFormatString.slice(state[START_INDEX], endIndex));
    }
    const fragments = [];
    return messageFormatString.split("").forEach(((char, ii) => {
        if (0 === ii) return void (state = {
            [START_INDEX]: 0,
            [TYPE]: "\\" === messageFormatString[0] ? 0 : "$" === messageFormatString[0] ? 2 : 1
        });
        let nextState;
        switch (state[TYPE]) {
          case 0:
            nextState = {
                [START_INDEX]: ii,
                [TYPE]: 1
            };
            break;

          case 1:
            "\\" === char ? nextState = {
                [START_INDEX]: ii,
                [TYPE]: 0
            } : "$" === char && (nextState = {
                [START_INDEX]: ii,
                [TYPE]: 2
            });
            break;

          case 2:
            "\\" === char ? nextState = {
                [START_INDEX]: ii,
                [TYPE]: 0
            } : "$" === char ? nextState = {
                [START_INDEX]: ii,
                [TYPE]: 2
            } : char.match(/\w/) || (nextState = {
                [START_INDEX]: ii,
                [TYPE]: 1
            });
        }
        nextState && (state !== nextState && commitStateUpTo(ii), state = nextState);
    })), commitStateUpTo(), fragments.join("");
}

function getErrorMessage(code, context = {}) {
    if ("production" !== process.env.NODE_ENV) return getHumanReadableErrorMessage(code, context);
    {
        let decodingAdviceMessage = `Solana error #${code}; Decode this error by running \`npx @solana/errors decode -- ${code}`;
        return Object.keys(context).length && (decodingAdviceMessage += ` '${encodeContextObject(context)}'`), 
        `${decodingAdviceMessage}\``;
    }
}

var SolanaError = class extends Error {
    cause=this.cause;
    context;
    constructor(...[code, contextAndErrorOptions]) {
        let context, errorOptions;
        if (contextAndErrorOptions) {
            const {cause: cause, ...contextRest} = contextAndErrorOptions;
            cause && (errorOptions = {
                cause: cause
            }), Object.keys(contextRest).length > 0 && (context = contextRest);
        }
        super(getErrorMessage(code, context), errorOptions), this.context = {
            __code: code,
            ...context
        }, this.name = "SolanaError";
    }
};

function safeCaptureStackTrace(...args) {
    "captureStackTrace" in Error && "function" == typeof Error.captureStackTrace && Error.captureStackTrace(...args);
}

function getSolanaErrorFromRpcError({errorCodeBaseOffset: errorCodeBaseOffset, getErrorContext: getErrorContext, orderedErrorNames: orderedErrorNames, rpcEnumError: rpcEnumError}, constructorOpt) {
    let rpcErrorName, rpcErrorContext;
    "string" == typeof rpcEnumError ? rpcErrorName = rpcEnumError : (rpcErrorName = Object.keys(rpcEnumError)[0], 
    rpcErrorContext = rpcEnumError[rpcErrorName]);
    const errorCode = errorCodeBaseOffset + orderedErrorNames.indexOf(rpcErrorName), errorContext = getErrorContext(errorCode, rpcErrorName, rpcErrorContext), err = new SolanaError(errorCode, errorContext);
    return safeCaptureStackTrace(err, constructorOpt), err;
}

var ORDERED_ERROR_NAMES = [ "GenericError", "InvalidArgument", "InvalidInstructionData", "InvalidAccountData", "AccountDataTooSmall", "InsufficientFunds", "IncorrectProgramId", "MissingRequiredSignature", "AccountAlreadyInitialized", "UninitializedAccount", "UnbalancedInstruction", "ModifiedProgramId", "ExternalAccountLamportSpend", "ExternalAccountDataModified", "ReadonlyLamportChange", "ReadonlyDataModified", "DuplicateAccountIndex", "ExecutableModified", "RentEpochModified", "NotEnoughAccountKeys", "AccountDataSizeChanged", "AccountNotExecutable", "AccountBorrowFailed", "AccountBorrowOutstanding", "DuplicateAccountOutOfSync", "Custom", "InvalidError", "ExecutableDataModified", "ExecutableLamportChange", "ExecutableAccountNotRentExempt", "UnsupportedProgramId", "CallDepth", "MissingAccount", "ReentrancyNotAllowed", "MaxSeedLengthExceeded", "InvalidSeeds", "InvalidRealloc", "ComputationalBudgetExceeded", "PrivilegeEscalation", "ProgramEnvironmentSetupFailure", "ProgramFailedToComplete", "ProgramFailedToCompile", "Immutable", "IncorrectAuthority", "BorshIoError", "AccountNotRentExempt", "InvalidAccountOwner", "ArithmeticOverflow", "UnsupportedSysvar", "IllegalOwner", "MaxAccountsDataAllocationsExceeded", "MaxAccountsExceeded", "MaxInstructionTraceLengthExceeded", "BuiltinProgramsMustConsumeComputeUnits" ];

function getSolanaErrorFromInstructionError(index, instructionError) {
    const numberIndex = Number(index);
    return getSolanaErrorFromRpcError({
        errorCodeBaseOffset: 4615001,
        getErrorContext: (errorCode, rpcErrorName, rpcErrorContext) => errorCode === SOLANA_ERROR__INSTRUCTION_ERROR__UNKNOWN ? {
            errorName: rpcErrorName,
            index: numberIndex,
            ...void 0 !== rpcErrorContext ? {
                instructionErrorContext: rpcErrorContext
            } : null
        } : errorCode === SOLANA_ERROR__INSTRUCTION_ERROR__CUSTOM ? {
            code: Number(rpcErrorContext),
            index: numberIndex
        } : errorCode === SOLANA_ERROR__INSTRUCTION_ERROR__BORSH_IO_ERROR ? {
            encodedData: rpcErrorContext,
            index: numberIndex
        } : {
            index: numberIndex
        },
        orderedErrorNames: ORDERED_ERROR_NAMES,
        rpcEnumError: instructionError
    }, getSolanaErrorFromInstructionError);
}

var ORDERED_ERROR_NAMES2 = [ "AccountInUse", "AccountLoadedTwice", "AccountNotFound", "ProgramAccountNotFound", "InsufficientFundsForFee", "InvalidAccountForFee", "AlreadyProcessed", "BlockhashNotFound", "CallChainTooDeep", "MissingSignatureForFee", "InvalidAccountIndex", "SignatureFailure", "InvalidProgramForExecution", "SanitizeFailure", "ClusterMaintenance", "AccountBorrowOutstanding", "WouldExceedMaxBlockCostLimit", "UnsupportedVersion", "InvalidWritableAccount", "WouldExceedMaxAccountCostLimit", "WouldExceedAccountDataBlockLimit", "TooManyAccountLocks", "AddressLookupTableNotFound", "InvalidAddressLookupTableOwner", "InvalidAddressLookupTableData", "InvalidAddressLookupTableIndex", "InvalidRentPayingAccount", "WouldExceedMaxVoteCostLimit", "WouldExceedAccountDataTotalLimit", "DuplicateInstruction", "InsufficientFundsForRent", "MaxLoadedAccountsDataSizeExceeded", "InvalidLoadedAccountsDataSizeLimit", "ResanitizationNeeded", "ProgramExecutionTemporarilyRestricted", "UnbalancedTransaction" ];

function getSolanaErrorFromTransactionError(transactionError) {
    return "object" == typeof transactionError && "InstructionError" in transactionError ? getSolanaErrorFromInstructionError(...transactionError.InstructionError) : getSolanaErrorFromRpcError({
        errorCodeBaseOffset: 7050001,
        getErrorContext: (errorCode, rpcErrorName, rpcErrorContext) => errorCode === SOLANA_ERROR__TRANSACTION_ERROR__UNKNOWN ? {
            errorName: rpcErrorName,
            ...void 0 !== rpcErrorContext ? {
                transactionErrorContext: rpcErrorContext
            } : null
        } : errorCode === SOLANA_ERROR__TRANSACTION_ERROR__DUPLICATE_INSTRUCTION ? {
            index: Number(rpcErrorContext)
        } : errorCode === SOLANA_ERROR__TRANSACTION_ERROR__INSUFFICIENT_FUNDS_FOR_RENT || errorCode === SOLANA_ERROR__TRANSACTION_ERROR__PROGRAM_EXECUTION_TEMPORARILY_RESTRICTED ? {
            accountIndex: Number(rpcErrorContext.account_index)
        } : void 0,
        orderedErrorNames: ORDERED_ERROR_NAMES2,
        rpcEnumError: transactionError
    }, getSolanaErrorFromTransactionError);
}

function getSolanaErrorFromJsonRpcError({code: rawCode, data: data, message: message}) {
    let out;
    const code = Number(rawCode);
    if (code === SOLANA_ERROR__JSON_RPC__SERVER_ERROR_SEND_TRANSACTION_PREFLIGHT_FAILURE) {
        const {err: err, ...preflightErrorContext} = data, causeObject = err ? {
            cause: getSolanaErrorFromTransactionError(err)
        } : null;
        out = new SolanaError(SOLANA_ERROR__JSON_RPC__SERVER_ERROR_SEND_TRANSACTION_PREFLIGHT_FAILURE, {
            ...preflightErrorContext,
            ...causeObject
        });
    } else {
        let errorContext;
        switch (code) {
          case SOLANA_ERROR__JSON_RPC__INTERNAL_ERROR:
          case SOLANA_ERROR__JSON_RPC__INVALID_PARAMS:
          case SOLANA_ERROR__JSON_RPC__INVALID_REQUEST:
          case SOLANA_ERROR__JSON_RPC__METHOD_NOT_FOUND:
          case SOLANA_ERROR__JSON_RPC__PARSE_ERROR:
          case SOLANA_ERROR__JSON_RPC__SCAN_ERROR:
          case SOLANA_ERROR__JSON_RPC__SERVER_ERROR_BLOCK_CLEANED_UP:
          case SOLANA_ERROR__JSON_RPC__SERVER_ERROR_BLOCK_NOT_AVAILABLE:
          case SOLANA_ERROR__JSON_RPC__SERVER_ERROR_BLOCK_STATUS_NOT_AVAILABLE_YET:
          case SOLANA_ERROR__JSON_RPC__SERVER_ERROR_KEY_EXCLUDED_FROM_SECONDARY_INDEX:
          case SOLANA_ERROR__JSON_RPC__SERVER_ERROR_LONG_TERM_STORAGE_SLOT_SKIPPED:
          case SOLANA_ERROR__JSON_RPC__SERVER_ERROR_SLOT_SKIPPED:
          case SOLANA_ERROR__JSON_RPC__SERVER_ERROR_TRANSACTION_PRECOMPILE_VERIFICATION_FAILURE:
          case SOLANA_ERROR__JSON_RPC__SERVER_ERROR_UNSUPPORTED_TRANSACTION_VERSION:
            errorContext = {
                __serverMessage: message
            };
            break;

          default:
            "object" != typeof data || Array.isArray(data) || (errorContext = data);
        }
        out = new SolanaError(code, errorContext);
    }
    return safeCaptureStackTrace(out, getSolanaErrorFromJsonRpcError), out;
}

var padBytes = (bytes, length) => {
    if (bytes.length >= length) return bytes;
    const paddedBytes = new Uint8Array(length).fill(0);
    return paddedBytes.set(bytes), paddedBytes;
}, fixBytes = (bytes, length) => padBytes(bytes.length <= length ? bytes : bytes.slice(0, length), length);

function containsBytes(data, bytes, offset) {
    const slice = data.length === bytes.length ? data : data.slice(offset, offset + bytes.length);
    return slice.length === bytes.length && bytes.every(((b, i) => b === slice[i]));
}

function getEncodedSize(value, encoder) {
    return "fixedSize" in encoder ? encoder.fixedSize : encoder.getSizeFromValue(value);
}

function createEncoder(encoder) {
    return Object.freeze({
        ...encoder,
        encode: value => {
            const bytes = new Uint8Array(getEncodedSize(value, encoder));
            return encoder.write(value, bytes, 0), bytes;
        }
    });
}

function createDecoder(decoder) {
    return Object.freeze({
        ...decoder,
        decode: (bytes, offset = 0) => decoder.read(bytes, offset)[0]
    });
}

function isFixedSize(codec) {
    return "fixedSize" in codec && "number" == typeof codec.fixedSize;
}

function assertByteArrayIsNotEmptyForCodec(codecDescription, bytes, offset = 0) {
    if (bytes.length - offset <= 0) throw new SolanaError(SOLANA_ERROR__CODECS__CANNOT_DECODE_EMPTY_BYTE_ARRAY, {
        codecDescription: codecDescription
    });
}

function assertByteArrayHasEnoughBytesForCodec(codecDescription, expected, bytes, offset = 0) {
    const bytesLength = bytes.length - offset;
    if (bytesLength < expected) throw new SolanaError(SOLANA_ERROR__CODECS__INVALID_BYTE_LENGTH, {
        bytesLength: bytesLength,
        codecDescription: codecDescription,
        expected: expected
    });
}

function fixDecoderSize(decoder, fixedBytes) {
    return createDecoder({
        fixedSize: fixedBytes,
        read: (bytes, offset) => {
            assertByteArrayHasEnoughBytesForCodec("fixCodecSize", fixedBytes, bytes, offset), 
            (offset > 0 || bytes.length > fixedBytes) && (bytes = bytes.slice(offset, offset + fixedBytes)), 
            isFixedSize(decoder) && (bytes = fixBytes(bytes, decoder.fixedSize));
            const [value] = decoder.read(bytes, 0);
            return [ value, offset + fixedBytes ];
        }
    });
}

function assertValidBaseString(alphabet4, testValue, givenValue = testValue) {
    if (!testValue.match(new RegExp(`^[${alphabet4}]*$`))) throw new SolanaError(SOLANA_ERROR__CODECS__INVALID_STRING_FOR_BASE, {
        alphabet: alphabet4,
        base: alphabet4.length,
        value: givenValue
    });
}

var getBaseXEncoder = alphabet4 => createEncoder({
    getSizeFromValue: value => {
        const [leadingZeroes, tailChars] = partitionLeadingZeroes(value, alphabet4[0]);
        if (!tailChars) return value.length;
        const base10Number = getBigIntFromBaseX(tailChars, alphabet4);
        return leadingZeroes.length + Math.ceil(base10Number.toString(16).length / 2);
    },
    write(value, bytes, offset) {
        if (assertValidBaseString(alphabet4, value), "" === value) return offset;
        const [leadingZeroes, tailChars] = partitionLeadingZeroes(value, alphabet4[0]);
        if (!tailChars) return bytes.set(new Uint8Array(leadingZeroes.length).fill(0), offset), 
        offset + leadingZeroes.length;
        let base10Number = getBigIntFromBaseX(tailChars, alphabet4);
        const tailBytes = [];
        for (;base10Number > 0n; ) tailBytes.unshift(Number(base10Number % 256n)), base10Number /= 256n;
        const bytesToAdd = [ ...Array(leadingZeroes.length).fill(0), ...tailBytes ];
        return bytes.set(bytesToAdd, offset), offset + bytesToAdd.length;
    }
}), getBaseXDecoder = alphabet4 => createDecoder({
    read(rawBytes, offset) {
        const bytes = 0 === offset ? rawBytes : rawBytes.slice(offset);
        if (0 === bytes.length) return [ "", 0 ];
        let trailIndex = bytes.findIndex((n => 0 !== n));
        trailIndex = -1 === trailIndex ? bytes.length : trailIndex;
        const leadingZeroes = alphabet4[0].repeat(trailIndex);
        if (trailIndex === bytes.length) return [ leadingZeroes, rawBytes.length ];
        return [ leadingZeroes + getBaseXFromBigInt(bytes.slice(trailIndex).reduce(((sum, byte) => 256n * sum + BigInt(byte)), 0n), alphabet4), rawBytes.length ];
    }
});

function partitionLeadingZeroes(value, zeroCharacter) {
    const [leadingZeros, tailChars] = value.split(new RegExp(`((?!${zeroCharacter}).*)`));
    return [ leadingZeros, tailChars ];
}

function getBigIntFromBaseX(value, alphabet4) {
    const base = BigInt(alphabet4.length);
    let sum = 0n;
    for (const char of value) sum *= base, sum += BigInt(alphabet4.indexOf(char));
    return sum;
}

function getBaseXFromBigInt(value, alphabet4) {
    const base = BigInt(alphabet4.length), tailChars = [];
    for (;value > 0n; ) tailChars.unshift(alphabet4[Number(value % base)]), value /= base;
    return tailChars.join("");
}

var alphabet2 = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz", getBase58Encoder = () => getBaseXEncoder(alphabet2), getBase58Decoder = () => getBaseXDecoder(alphabet2), memoizedBase58Encoder, memoizedBase58Decoder;

function assertKeyExporterIsAvailable() {
    if (void 0 === globalThis.crypto || "function" != typeof globalThis.crypto.subtle?.exportKey) throw new SolanaError(SOLANA_ERROR__SUBTLE_CRYPTO__EXPORT_FUNCTION_UNIMPLEMENTED);
}

function assertSigningCapabilityIsAvailable() {
    if (void 0 === globalThis.crypto || "function" != typeof globalThis.crypto.subtle?.sign) throw new SolanaError(SOLANA_ERROR__SUBTLE_CRYPTO__SIGN_FUNCTION_UNIMPLEMENTED);
}

function assertVerificationCapabilityIsAvailable() {
    if (void 0 === globalThis.crypto || "function" != typeof globalThis.crypto.subtle?.verify) throw new SolanaError(SOLANA_ERROR__SUBTLE_CRYPTO__VERIFY_FUNCTION_UNIMPLEMENTED);
}

function assertPRNGIsAvailable() {
    if (void 0 === globalThis.crypto || "function" != typeof globalThis.crypto.getRandomValues) throw new SolanaError(SOLANA_ERROR__CRYPTO__RANDOM_VALUES_FUNCTION_UNIMPLEMENTED);
}

function getMemoizedBase58Encoder() {
    return memoizedBase58Encoder || (memoizedBase58Encoder = getBase58Encoder()), memoizedBase58Encoder;
}

function getMemoizedBase58Decoder() {
    return memoizedBase58Decoder || (memoizedBase58Decoder = getBase58Decoder()), memoizedBase58Decoder;
}

function isAddress(putativeAddress) {
    if (putativeAddress.length < 32 || putativeAddress.length > 44) return !1;
    const base58Encoder = getMemoizedBase58Encoder();
    try {
        return 32 === base58Encoder.encode(putativeAddress).byteLength;
    } catch {
        return !1;
    }
}

function assertIsAddress(putativeAddress) {
    if (putativeAddress.length < 32 || putativeAddress.length > 44) throw new SolanaError(SOLANA_ERROR__ADDRESSES__STRING_LENGTH_OUT_OF_RANGE, {
        actualLength: putativeAddress.length
    });
    const numBytes = getMemoizedBase58Encoder().encode(putativeAddress).byteLength;
    if (32 !== numBytes) throw new SolanaError(SOLANA_ERROR__ADDRESSES__INVALID_BYTE_LENGTH, {
        actualLength: numBytes
    });
}

function address(putativeAddress) {
    return assertIsAddress(putativeAddress), putativeAddress;
}

function getAddressDecoder() {
    return fixDecoderSize(getMemoizedBase58Decoder(), 32);
}

async function getAddressFromPublicKey(publicKey) {
    if (assertKeyExporterIsAvailable(), "public" !== publicKey.type || "Ed25519" !== publicKey.algorithm.name) throw new SolanaError(SOLANA_ERROR__ADDRESSES__INVALID_ED25519_PUBLIC_KEY);
    const publicKeyBytes = await crypto.subtle.exportKey("raw", publicKey);
    return getAddressDecoder().decode(new Uint8Array(publicKeyBytes));
}

function assertNumberIsBetweenForCodec(codecDescription, min, max, value) {
    if (value < min || value > max) throw new SolanaError(SOLANA_ERROR__CODECS__NUMBER_OUT_OF_RANGE, {
        codecDescription: codecDescription,
        max: max,
        min: min,
        value: value
    });
}

function isLittleEndian(config) {
    return 1 !== config?.endian;
}

function numberEncoderFactory(input) {
    return createEncoder({
        fixedSize: input.size,
        write(value, bytes, offset) {
            input.range && assertNumberIsBetweenForCodec(input.name, input.range[0], input.range[1], value);
            const arrayBuffer = new ArrayBuffer(input.size);
            return input.set(new DataView(arrayBuffer), value, isLittleEndian(input.config)), 
            bytes.set(new Uint8Array(arrayBuffer), offset), offset + input.size;
        }
    });
}

function numberDecoderFactory(input) {
    return createDecoder({
        fixedSize: input.size,
        read(bytes, offset = 0) {
            assertByteArrayIsNotEmptyForCodec(input.name, bytes, offset), assertByteArrayHasEnoughBytesForCodec(input.name, input.size, bytes, offset);
            const view = new DataView(toArrayBuffer(bytes, offset, input.size));
            return [ input.get(view, isLittleEndian(input.config)), offset + input.size ];
        }
    });
}

function toArrayBuffer(bytes, offset, length) {
    const bytesOffset = bytes.byteOffset + (offset ?? 0), bytesLength = length ?? bytes.byteLength;
    return bytes.buffer.slice(bytesOffset, bytesOffset + bytesLength);
}

var getU32Decoder = (config = {}) => numberDecoderFactory({
    config: config,
    get: (view, le) => view.getUint32(0, le),
    name: "u32",
    size: 4
}), getU64Decoder = (config = {}) => numberDecoderFactory({
    config: config,
    get: (view, le) => view.getBigUint64(0, le),
    name: "u64",
    size: 8
}), getU8Encoder = () => numberEncoderFactory({
    name: "u8",
    range: [ 0, Number("0xff") ],
    set: (view, value) => view.setUint8(0, Number(value)),
    size: 1
}), getU8Decoder = () => numberDecoderFactory({
    get: view => view.getUint8(0),
    name: "u8",
    size: 1
}), base58Encoder;

function sumCodecSizes(sizes) {
    return sizes.reduce(((all, size) => null === all || null === size ? null : all + size), 0);
}

function getFixedSize(codec) {
    return isFixedSize(codec) ? codec.fixedSize : null;
}

function getMaxSize(codec) {
    return isFixedSize(codec) ? codec.fixedSize : codec.maxSize ?? null;
}

function getStructDecoder(fields) {
    const fieldCodecs = fields.map((([, codec]) => codec)), fixedSize = sumCodecSizes(fieldCodecs.map(getFixedSize)), maxSize = sumCodecSizes(fieldCodecs.map(getMaxSize)) ?? void 0;
    return createDecoder({
        ...null === fixedSize ? {
            maxSize: maxSize
        } : {
            fixedSize: fixedSize
        },
        read: (bytes, offset) => {
            const struct = {};
            return fields.forEach((([key, codec]) => {
                const [value, newOffset] = codec.read(bytes, offset);
                offset = newOffset, struct[key] = value;
            })), [ struct, offset ];
        }
    });
}

function pipe$1(init, ...fns) {
    return fns.reduce(((acc, fn) => fn(acc)), init);
}

function addPkcs8Header(bytes) {
    return new Uint8Array([ 48, 46, 2, 1, 0, 48, 5, 6, 3, 43, 101, 112, 4, 34, 4, 32, ...bytes ]);
}

async function createPrivateKeyFromBytes(bytes, extractable) {
    const actualLength = bytes.byteLength;
    if (32 !== actualLength) throw new SolanaError(SOLANA_ERROR__KEYS__INVALID_PRIVATE_KEY_BYTE_LENGTH, {
        actualLength: actualLength
    });
    const privateKeyBytesPkcs8 = addPkcs8Header(bytes);
    return await crypto.subtle.importKey("pkcs8", privateKeyBytesPkcs8, "Ed25519", !1, [ "sign" ]);
}

function assertIsSignature(putativeSignature) {
    if (base58Encoder || (base58Encoder = getBase58Encoder()), putativeSignature.length < 64 || putativeSignature.length > 88) throw new SolanaError(SOLANA_ERROR__KEYS__SIGNATURE_STRING_LENGTH_OUT_OF_RANGE, {
        actualLength: putativeSignature.length
    });
    const numBytes = base58Encoder.encode(putativeSignature).byteLength;
    if (64 !== numBytes) throw new SolanaError(SOLANA_ERROR__KEYS__INVALID_SIGNATURE_BYTE_LENGTH, {
        actualLength: numBytes
    });
}

function isSignature(putativeSignature) {
    if (base58Encoder || (base58Encoder = getBase58Encoder()), putativeSignature.length < 64 || putativeSignature.length > 88) return !1;
    return 64 === base58Encoder.encode(putativeSignature).byteLength;
}

async function signBytes(key, data) {
    assertSigningCapabilityIsAvailable();
    const signedData = await crypto.subtle.sign("Ed25519", key, data);
    return new Uint8Array(signedData);
}

function signature(putativeSignature) {
    return assertIsSignature(putativeSignature), putativeSignature;
}

async function verifySignature(key, signature2, data) {
    return assertVerificationCapabilityIsAvailable(), await crypto.subtle.verify("Ed25519", key, signature2, data);
}

async function createKeyPairFromBytes(bytes, extractable) {
    if (assertPRNGIsAvailable(), 64 !== bytes.byteLength) throw new SolanaError(SOLANA_ERROR__KEYS__INVALID_KEY_PAIR_BYTE_LENGTH, {
        byteLength: bytes.byteLength
    });
    const [publicKey, privateKey] = await Promise.all([ crypto.subtle.importKey("raw", bytes.slice(32), "Ed25519", !0, [ "verify" ]), createPrivateKeyFromBytes(bytes.slice(0, 32)) ]), randomBytes = new Uint8Array(32);
    crypto.getRandomValues(randomBytes);
    const signedData = await signBytes(privateKey, randomBytes);
    if (!await verifySignature(publicKey, signedData, randomBytes)) throw new SolanaError(SOLANA_ERROR__KEYS__PUBLIC_KEY_MUST_MATCH_PRIVATE_KEY);
    return {
        privateKey: privateKey,
        publicKey: publicKey
    };
}

function parseJsonWithBigInts(json) {
    return JSON.parse(wrapIntegersInBigIntValueObject(json), ((_, value) => isBigIntValueObject(value) ? unwrapBigIntValueObject(value) : value));
}

function wrapIntegersInBigIntValueObject(json) {
    const out = [];
    let inQuote = !1;
    for (let ii = 0; ii < json.length; ii++) {
        let isEscaped = !1;
        if ("\\" === json[ii] && (out.push(json[ii++]), isEscaped = !isEscaped), '"' !== json[ii]) {
            if (!inQuote) {
                const consumedNumber = consumeNumber(json, ii);
                if (consumedNumber?.length) {
                    ii += consumedNumber.length - 1, consumedNumber.match(/\.|[eE]-/) ? out.push(consumedNumber) : out.push(wrapBigIntValueObject(consumedNumber));
                    continue;
                }
            }
            out.push(json[ii]);
        } else out.push(json[ii]), isEscaped || (inQuote = !inQuote);
    }
    return out.join("");
}

function consumeNumber(json, ii) {
    if (!json[ii]?.match(/[-\d]/)) return null;
    const numberMatch = json.slice(ii).match(/^-?(?:0|[1-9]\d*)(?:\.\d+)?(?:[eE][+-]?\d+)?/);
    return numberMatch ? numberMatch[0] : null;
}

function wrapBigIntValueObject(value) {
    return `{"$n":"${value}"}`;
}

function unwrapBigIntValueObject({$n: $n}) {
    if ($n.match(/[eE]/)) {
        const [units, exponent] = $n.split(/[eE]/);
        return BigInt(units) * BigInt(10) ** BigInt(exponent);
    }
    return BigInt($n);
}

function isBigIntValueObject(value) {
    return !!value && "object" == typeof value && "$n" in value && "string" == typeof value.$n;
}

var _nextMessageId = 0n;

function getNextMessageId() {
    const id = _nextMessageId;
    return _nextMessageId++, id.toString();
}

function createRpcMessage(request) {
    return {
        id: getNextMessageId(),
        jsonrpc: "2.0",
        method: request.methodName,
        params: request.params
    };
}

function stringifyJsonWithBigints(value, space) {
    return unwrapBigIntValueObject2(JSON.stringify(value, ((_, v) => "bigint" == typeof v ? wrapBigIntValueObject2(v) : v), space));
}

function wrapBigIntValueObject2(value) {
    return {
        $n: `${value}`
    };
}

function unwrapBigIntValueObject2(value) {
    return value.replace(/\{\s*"\$n"\s*:\s*"(-?\d+)"\s*\}/g, "$1");
}

function createRpc(rpcConfig) {
    return makeProxy(rpcConfig);
}

function makeProxy(rpcConfig) {
    return new Proxy(rpcConfig.api, {
        defineProperty: () => !1,
        deleteProperty: () => !1,
        get: (target, p, receiver) => function(...rawParams) {
            const methodName = p.toString(), getApiPlan = Reflect.get(target, methodName, receiver);
            if (!getApiPlan) throw new SolanaError(SOLANA_ERROR__RPC__API_PLAN_MISSING_FOR_RPC_METHOD, {
                method: methodName,
                params: rawParams
            });
            const apiPlan = getApiPlan(...rawParams);
            return createPendingRpcRequest(rpcConfig, apiPlan);
        }
    });
}

function createPendingRpcRequest({transport: transport}, plan) {
    return {
        send: async options => await plan.execute({
            signal: options?.abortSignal,
            transport: transport
        })
    };
}

function createJsonRpcApi(config) {
    return new Proxy({}, {
        defineProperty: () => !1,
        deleteProperty: () => !1,
        get(...args) {
            const [_, p] = args, methodName = p.toString();
            return function(...rawParams) {
                const rawRequest = Object.freeze({
                    methodName: methodName,
                    params: rawParams
                }), request = config?.requestTransformer ? config?.requestTransformer(rawRequest) : rawRequest;
                return Object.freeze({
                    execute: async ({signal: signal, transport: transport}) => {
                        const payload = createRpcMessage(request), response = await transport({
                            payload: payload,
                            signal: signal
                        });
                        return config?.responseTransformer ? config.responseTransformer(response, request) : response;
                    }
                });
            };
        }
    });
}

function isJsonRpcPayload(payload) {
    return null != payload && "object" == typeof payload && !Array.isArray(payload) && ("jsonrpc" in payload && "2.0" === payload.jsonrpc && "method" in payload && "string" == typeof payload.method && "params" in payload);
}

var KEYPATH_WILDCARD = {};

function getTreeWalker(visitors) {
    return function traverse(node, state) {
        if (Array.isArray(node)) return node.map(((element, ii) => {
            const nextState = {
                ...state,
                keyPath: [ ...state.keyPath, ii ]
            };
            return traverse(element, nextState);
        }));
        if ("object" == typeof node && null !== node) {
            const out = {};
            for (const propName in node) {
                if (!Object.prototype.hasOwnProperty.call(node, propName)) continue;
                const nextState = {
                    ...state,
                    keyPath: [ ...state.keyPath, propName ]
                };
                out[propName] = traverse(node[propName], nextState);
            }
            return out;
        }
        return visitors.reduce(((acc, visitNode) => visitNode(acc, state)), node);
    };
}

function getTreeWalkerRequestTransformer(visitors, initialState) {
    return request => {
        const traverse = getTreeWalker(visitors);
        return Object.freeze({
            ...request,
            params: traverse(request.params, initialState)
        });
    };
}

function getTreeWalkerResponseTransformer(visitors, initialState) {
    return json => getTreeWalker(visitors)(json, initialState);
}

function getBigIntDowncastRequestTransformer() {
    return getTreeWalkerRequestTransformer([ downcastNodeToNumberIfBigint ], {
        keyPath: []
    });
}

function downcastNodeToNumberIfBigint(value) {
    return "bigint" == typeof value ? Number(value) : value;
}

function getDefaultCommitmentRequestTransformer({defaultCommitment: defaultCommitment, optionsObjectPositionByMethod: optionsObjectPositionByMethod}) {
    return request => {
        const {params: params, methodName: methodName} = request;
        if (!Array.isArray(params)) return request;
        const optionsObjectPositionInParams = optionsObjectPositionByMethod[methodName];
        return null == optionsObjectPositionInParams ? request : Object.freeze({
            methodName: methodName,
            params: applyDefaultCommitment({
                commitmentPropertyName: "sendTransaction" === methodName ? "preflightCommitment" : "commitment",
                optionsObjectPositionInParams: optionsObjectPositionInParams,
                overrideCommitment: defaultCommitment,
                params: params
            })
        });
    };
}

function applyDefaultCommitment({commitmentPropertyName: commitmentPropertyName, params: params, optionsObjectPositionInParams: optionsObjectPositionInParams, overrideCommitment: overrideCommitment}) {
    const paramInTargetPosition = params[optionsObjectPositionInParams];
    if (void 0 === paramInTargetPosition || paramInTargetPosition && "object" == typeof paramInTargetPosition && !Array.isArray(paramInTargetPosition)) if (paramInTargetPosition && commitmentPropertyName in paramInTargetPosition) {
        if (!paramInTargetPosition[commitmentPropertyName] || "finalized" === paramInTargetPosition[commitmentPropertyName]) {
            const nextParams = [ ...params ], {[commitmentPropertyName]: _, ...rest} = paramInTargetPosition;
            return Object.keys(rest).length > 0 ? nextParams[optionsObjectPositionInParams] = rest : optionsObjectPositionInParams === nextParams.length - 1 ? nextParams.length-- : nextParams[optionsObjectPositionInParams] = void 0, 
            nextParams;
        }
    } else if ("finalized" !== overrideCommitment) {
        const nextParams = [ ...params ];
        return nextParams[optionsObjectPositionInParams] = {
            ...paramInTargetPosition,
            [commitmentPropertyName]: overrideCommitment
        }, nextParams;
    }
    return params;
}

function getIntegerOverflowRequestTransformer(onIntegerOverflow) {
    return request => getTreeWalkerRequestTransformer([ getIntegerOverflowNodeVisitor(((...args) => onIntegerOverflow(request, ...args))) ], {
        keyPath: []
    })(request);
}

function getIntegerOverflowNodeVisitor(onIntegerOverflow) {
    return (value, {keyPath: keyPath}) => ("bigint" == typeof value && onIntegerOverflow && (value > Number.MAX_SAFE_INTEGER || value < -Number.MAX_SAFE_INTEGER) && onIntegerOverflow(keyPath, value), 
    value);
}

var OPTIONS_OBJECT_POSITION_BY_METHOD = {
    accountNotifications: 1,
    blockNotifications: 1,
    getAccountInfo: 1,
    getBalance: 1,
    getBlock: 1,
    getBlockHeight: 0,
    getBlockProduction: 0,
    getBlocks: 2,
    getBlocksWithLimit: 2,
    getEpochInfo: 0,
    getFeeForMessage: 1,
    getInflationGovernor: 0,
    getInflationReward: 1,
    getLargestAccounts: 0,
    getLatestBlockhash: 0,
    getLeaderSchedule: 1,
    getMinimumBalanceForRentExemption: 1,
    getMultipleAccounts: 1,
    getProgramAccounts: 1,
    getSignaturesForAddress: 1,
    getSlot: 0,
    getSlotLeader: 0,
    getStakeMinimumDelegation: 0,
    getSupply: 0,
    getTokenAccountBalance: 1,
    getTokenAccountsByDelegate: 2,
    getTokenAccountsByOwner: 2,
    getTokenLargestAccounts: 1,
    getTokenSupply: 1,
    getTransaction: 1,
    getTransactionCount: 0,
    getVoteAccounts: 0,
    isBlockhashValid: 1,
    logsNotifications: 1,
    programNotifications: 1,
    requestAirdrop: 2,
    sendTransaction: 1,
    signatureNotifications: 1,
    simulateTransaction: 1
};

function getDefaultRequestTransformerForSolanaRpc(config) {
    const handleIntegerOverflow = config?.onIntegerOverflow;
    return request => pipe$1(request, handleIntegerOverflow ? getIntegerOverflowRequestTransformer(handleIntegerOverflow) : r => r, getBigIntDowncastRequestTransformer(), getDefaultCommitmentRequestTransformer({
        defaultCommitment: config?.defaultCommitment,
        optionsObjectPositionByMethod: OPTIONS_OBJECT_POSITION_BY_METHOD
    }));
}

function getBigIntUpcastResponseTransformer(allowedNumericKeyPaths) {
    return getTreeWalkerResponseTransformer([ getBigIntUpcastVisitor(allowedNumericKeyPaths) ], {
        keyPath: []
    });
}

function getBigIntUpcastVisitor(allowedNumericKeyPaths) {
    return function(value, {keyPath: keyPath}) {
        return "number" == typeof value && Number.isInteger(value) || "bigint" == typeof value ? keyPathIsAllowedToBeNumeric(keyPath, allowedNumericKeyPaths) ? Number(value) : BigInt(value) : value;
    };
}

function keyPathIsAllowedToBeNumeric(keyPath, allowedNumericKeyPaths) {
    return allowedNumericKeyPaths.some((prohibitedKeyPath => {
        if (prohibitedKeyPath.length !== keyPath.length) return !1;
        for (let ii = keyPath.length - 1; ii >= 0; ii--) {
            const keyPathPart = keyPath[ii], prohibitedKeyPathPart = prohibitedKeyPath[ii];
            if (prohibitedKeyPathPart !== keyPathPart && (prohibitedKeyPathPart !== KEYPATH_WILDCARD || "number" != typeof keyPathPart)) return !1;
        }
        return !0;
    }));
}

function getResultResponseTransformer() {
    return json => json.result;
}

function getThrowSolanaErrorResponseTransformer() {
    return json => {
        const jsonRpcResponse = json;
        if ("error" in jsonRpcResponse) throw getSolanaErrorFromJsonRpcError(jsonRpcResponse.error);
        return jsonRpcResponse;
    };
}

function getDefaultResponseTransformerForSolanaRpc(config) {
    return (response, request) => {
        const methodName = request.methodName, keyPaths = config?.allowedNumericKeyPaths && methodName ? config.allowedNumericKeyPaths[methodName] : void 0;
        return pipe$1(response, (r => getThrowSolanaErrorResponseTransformer()(r, request)), (r => getResultResponseTransformer()(r, request)), (r => getBigIntUpcastResponseTransformer(keyPaths ?? [])(r, request)));
    };
}

var jsonParsedTokenAccountsConfigs = [ [ "data", "parsed", "info", "tokenAmount", "decimals" ], [ "data", "parsed", "info", "tokenAmount", "uiAmount" ], [ "data", "parsed", "info", "rentExemptReserve", "decimals" ], [ "data", "parsed", "info", "rentExemptReserve", "uiAmount" ], [ "data", "parsed", "info", "delegatedAmount", "decimals" ], [ "data", "parsed", "info", "delegatedAmount", "uiAmount" ], [ "data", "parsed", "info", "extensions", KEYPATH_WILDCARD, "state", "olderTransferFee", "transferFeeBasisPoints" ], [ "data", "parsed", "info", "extensions", KEYPATH_WILDCARD, "state", "newerTransferFee", "transferFeeBasisPoints" ], [ "data", "parsed", "info", "extensions", KEYPATH_WILDCARD, "state", "preUpdateAverageRate" ], [ "data", "parsed", "info", "extensions", KEYPATH_WILDCARD, "state", "currentRate" ] ], jsonParsedAccountsConfigs = [ ...jsonParsedTokenAccountsConfigs, [ "data", "parsed", "info", "lastExtendedSlotStartIndex" ], [ "data", "parsed", "info", "slashPenalty" ], [ "data", "parsed", "info", "warmupCooldownRate" ], [ "data", "parsed", "info", "decimals" ], [ "data", "parsed", "info", "numRequiredSigners" ], [ "data", "parsed", "info", "numValidSigners" ], [ "data", "parsed", "info", "stake", "delegation", "warmupCooldownRate" ], [ "data", "parsed", "info", "exemptionThreshold" ], [ "data", "parsed", "info", "burnPercent" ], [ "data", "parsed", "info", "commission" ], [ "data", "parsed", "info", "votes", KEYPATH_WILDCARD, "confirmationCount" ] ], innerInstructionsConfigs = [ [ "index" ], [ "instructions", KEYPATH_WILDCARD, "accounts", KEYPATH_WILDCARD ], [ "instructions", KEYPATH_WILDCARD, "programIdIndex" ], [ "instructions", KEYPATH_WILDCARD, "stackHeight" ] ], messageConfig = [ [ "addressTableLookups", KEYPATH_WILDCARD, "writableIndexes", KEYPATH_WILDCARD ], [ "addressTableLookups", KEYPATH_WILDCARD, "readonlyIndexes", KEYPATH_WILDCARD ], [ "header", "numReadonlySignedAccounts" ], [ "header", "numReadonlyUnsignedAccounts" ], [ "header", "numRequiredSignatures" ], [ "instructions", KEYPATH_WILDCARD, "accounts", KEYPATH_WILDCARD ], [ "instructions", KEYPATH_WILDCARD, "programIdIndex" ], [ "instructions", KEYPATH_WILDCARD, "stackHeight" ] ], memoizedKeypaths;

function createSolanaRpcApi(config) {
    return createJsonRpcApi({
        requestTransformer: getDefaultRequestTransformerForSolanaRpc(config),
        responseTransformer: getDefaultResponseTransformerForSolanaRpc({
            allowedNumericKeyPaths: getAllowedNumericKeypaths()
        })
    });
}

function getAllowedNumericKeypaths() {
    return memoizedKeypaths || (memoizedKeypaths = {
        getAccountInfo: jsonParsedAccountsConfigs.map((c => [ "value", ...c ])),
        getBlock: [ [ "transactions", KEYPATH_WILDCARD, "meta", "preTokenBalances", KEYPATH_WILDCARD, "accountIndex" ], [ "transactions", KEYPATH_WILDCARD, "meta", "preTokenBalances", KEYPATH_WILDCARD, "uiTokenAmount", "decimals" ], [ "transactions", KEYPATH_WILDCARD, "meta", "postTokenBalances", KEYPATH_WILDCARD, "accountIndex" ], [ "transactions", KEYPATH_WILDCARD, "meta", "postTokenBalances", KEYPATH_WILDCARD, "uiTokenAmount", "decimals" ], [ "transactions", KEYPATH_WILDCARD, "meta", "rewards", KEYPATH_WILDCARD, "commission" ], ...innerInstructionsConfigs.map((c => [ "transactions", KEYPATH_WILDCARD, "meta", "innerInstructions", KEYPATH_WILDCARD, ...c ])), ...messageConfig.map((c => [ "transactions", KEYPATH_WILDCARD, "transaction", "message", ...c ])), [ "rewards", KEYPATH_WILDCARD, "commission" ] ],
        getClusterNodes: [ [ KEYPATH_WILDCARD, "featureSet" ], [ KEYPATH_WILDCARD, "shredVersion" ] ],
        getInflationGovernor: [ [ "initial" ], [ "foundation" ], [ "foundationTerm" ], [ "taper" ], [ "terminal" ] ],
        getInflationRate: [ [ "foundation" ], [ "total" ], [ "validator" ] ],
        getInflationReward: [ [ KEYPATH_WILDCARD, "commission" ] ],
        getMultipleAccounts: jsonParsedAccountsConfigs.map((c => [ "value", KEYPATH_WILDCARD, ...c ])),
        getProgramAccounts: jsonParsedAccountsConfigs.flatMap((c => [ [ "value", KEYPATH_WILDCARD, "account", ...c ], [ KEYPATH_WILDCARD, "account", ...c ] ])),
        getRecentPerformanceSamples: [ [ KEYPATH_WILDCARD, "samplePeriodSecs" ] ],
        getTokenAccountBalance: [ [ "value", "decimals" ], [ "value", "uiAmount" ] ],
        getTokenAccountsByDelegate: jsonParsedTokenAccountsConfigs.map((c => [ "value", KEYPATH_WILDCARD, "account", ...c ])),
        getTokenAccountsByOwner: jsonParsedTokenAccountsConfigs.map((c => [ "value", KEYPATH_WILDCARD, "account", ...c ])),
        getTokenLargestAccounts: [ [ "value", KEYPATH_WILDCARD, "decimals" ], [ "value", KEYPATH_WILDCARD, "uiAmount" ] ],
        getTokenSupply: [ [ "value", "decimals" ], [ "value", "uiAmount" ] ],
        getTransaction: [ [ "meta", "preTokenBalances", KEYPATH_WILDCARD, "accountIndex" ], [ "meta", "preTokenBalances", KEYPATH_WILDCARD, "uiTokenAmount", "decimals" ], [ "meta", "postTokenBalances", KEYPATH_WILDCARD, "accountIndex" ], [ "meta", "postTokenBalances", KEYPATH_WILDCARD, "uiTokenAmount", "decimals" ], [ "meta", "rewards", KEYPATH_WILDCARD, "commission" ], ...innerInstructionsConfigs.map((c => [ "meta", "innerInstructions", KEYPATH_WILDCARD, ...c ])), ...messageConfig.map((c => [ "transaction", "message", ...c ])) ],
        getVersion: [ [ "feature-set" ] ],
        getVoteAccounts: [ [ "current", KEYPATH_WILDCARD, "commission" ], [ "delinquent", KEYPATH_WILDCARD, "commission" ] ],
        simulateTransaction: [ ...jsonParsedAccountsConfigs.map((c => [ "value", "accounts", KEYPATH_WILDCARD, ...c ])), ...innerInstructionsConfigs.map((c => [ "value", "innerInstructions", KEYPATH_WILDCARD, ...c ])) ]
    }), memoizedKeypaths;
}

var DISALLOWED_HEADERS = {
    accept: !0,
    "content-length": !0,
    "content-type": !0
}, FORBIDDEN_HEADERS = Object.assign({
    "accept-charset": !0,
    "access-control-request-headers": !0,
    "access-control-request-method": !0,
    connection: !0,
    "content-length": !0,
    cookie: !0,
    date: !0,
    dnt: !0,
    expect: !0,
    host: !0,
    "keep-alive": !0,
    origin: !0,
    "permissions-policy": !0,
    referer: !0,
    te: !0,
    trailer: !0,
    "transfer-encoding": !0,
    upgrade: !0,
    via: !0
}, void 0);

function assertIsAllowedHttpRequestHeaders(headers) {
    const badHeaders = Object.keys(headers).filter((headerName => {
        const lowercaseHeaderName = headerName.toLowerCase();
        return !0 === DISALLOWED_HEADERS[headerName.toLowerCase()] || !0 === FORBIDDEN_HEADERS[headerName.toLowerCase()] || lowercaseHeaderName.startsWith("proxy-") || lowercaseHeaderName.startsWith("sec-");
    }));
    if (badHeaders.length > 0) throw new SolanaError(SOLANA_ERROR__RPC__TRANSPORT_HTTP_HEADER_FORBIDDEN, {
        headers: badHeaders
    });
}

function normalizeHeaders$1(headers) {
    const out = {};
    for (const headerName in headers) out[headerName.toLowerCase()] = headers[headerName];
    return out;
}

function createHttpTransport(config) {
    process.env.NODE_ENV;
    const {fromJson: fromJson, headers: headers, toJson: toJson, url: url} = config;
    let dispatcherConfig;
    "production" !== process.env.NODE_ENV && headers && assertIsAllowedHttpRequestHeaders(headers), 
    "dispatcher_NODE_ONLY" in config && (dispatcherConfig = {
        dispatcher: config.dispatcher_NODE_ONLY
    });
    const customHeaders = headers && normalizeHeaders$1(headers);
    return async function({payload: payload, signal: signal}) {
        const body = toJson ? toJson(payload) : JSON.stringify(payload), requestInfo = {
            ...dispatcherConfig,
            body: body,
            headers: {
                ...customHeaders,
                accept: "application/json",
                "content-length": body.length.toString(),
                "content-type": "application/json; charset=utf-8"
            },
            method: "POST",
            signal: signal
        }, response = await fetch(url, requestInfo);
        if (!response.ok) throw new SolanaError(SOLANA_ERROR__RPC__TRANSPORT_HTTP_ERROR, {
            message: response.statusText,
            statusCode: response.status
        });
        return fromJson ? fromJson(await response.text(), payload) : await response.json();
    };
}

var SOLANA_RPC_METHODS = [ "getAccountInfo", "getBalance", "getBlock", "getBlockCommitment", "getBlockHeight", "getBlockProduction", "getBlocks", "getBlocksWithLimit", "getBlockTime", "getClusterNodes", "getEpochInfo", "getEpochSchedule", "getFeeForMessage", "getFirstAvailableBlock", "getGenesisHash", "getHealth", "getHighestSnapshotSlot", "getIdentity", "getInflationGovernor", "getInflationRate", "getInflationReward", "getLargestAccounts", "getLatestBlockhash", "getLeaderSchedule", "getMaxRetransmitSlot", "getMaxShredInsertSlot", "getMinimumBalanceForRentExemption", "getMultipleAccounts", "getProgramAccounts", "getRecentPerformanceSamples", "getRecentPrioritizationFees", "getSignaturesForAddress", "getSignatureStatuses", "getSlot", "getSlotLeader", "getSlotLeaders", "getStakeMinimumDelegation", "getSupply", "getTokenAccountBalance", "getTokenAccountsByDelegate", "getTokenAccountsByOwner", "getTokenLargestAccounts", "getTokenSupply", "getTransaction", "getTransactionCount", "getVersion", "getVoteAccounts", "index", "isBlockhashValid", "minimumLedgerSlot", "requestAirdrop", "sendTransaction", "simulateTransaction" ];

function isSolanaRequest(payload) {
    return isJsonRpcPayload(payload) && SOLANA_RPC_METHODS.includes(payload.method);
}

function createHttpTransportForSolanaRpc(config) {
    return createHttpTransport({
        ...config,
        fromJson: (rawResponse, payload) => isSolanaRequest(payload) ? parseJsonWithBigInts(rawResponse) : JSON.parse(rawResponse),
        toJson: payload => isSolanaRequest(payload) ? stringifyJsonWithBigints(payload) : JSON.stringify(payload)
    });
}

var objToString = Object.prototype.toString, objKeys = Object.keys || function(obj) {
    const keys = [];
    for (const name in obj) keys.push(name);
    return keys;
};

function stringify(val, isArrayProp) {
    let i, max, str, keys, key, propVal, toStr;
    if (!0 === val) return "true";
    if (!1 === val) return "false";
    switch (typeof val) {
      case "object":
        if (null === val) return null;
        if ("toJSON" in val && "function" == typeof val.toJSON) return stringify(val.toJSON(), isArrayProp);
        if (toStr = objToString.call(val), "[object Array]" === toStr) {
            for (str = "[", max = val.length - 1, i = 0; i < max; i++) str += stringify(val[i], !0) + ",";
            return max > -1 && (str += stringify(val[i], !0)), str + "]";
        }
        if ("[object Object]" === toStr) {
            for (keys = objKeys(val).sort(), max = keys.length, str = "", i = 0; i < max; ) key = keys[i], 
            propVal = stringify(val[key], !1), void 0 !== propVal && (str && (str += ","), str += JSON.stringify(key) + ":" + propVal), 
            i++;
            return "{" + str + "}";
        }
        return JSON.stringify(val);

      case "function":
      case "undefined":
        return isArrayProp ? null : void 0;

      case "bigint":
        return `${val.toString()}n`;

      case "string":
        return JSON.stringify(val);

      default:
        return isFinite(val) ? val : null;
    }
}

function src_default(val) {
    const returnVal = stringify(val, !1);
    if (void 0 !== returnVal) return "" + returnVal;
}

function createSolanaJsonRpcIntegerOverflowError(methodName, keyPath, value) {
    let argumentLabel = "";
    if ("number" == typeof keyPath[0]) {
        const argPosition = keyPath[0] + 1, lastDigit = argPosition % 10, lastTwoDigits = argPosition % 100;
        argumentLabel = 1 == lastDigit && 11 != lastTwoDigits ? argPosition + "st" : 2 == lastDigit && 12 != lastTwoDigits ? argPosition + "nd" : 3 == lastDigit && 13 != lastTwoDigits ? argPosition + "rd" : argPosition + "th";
    } else argumentLabel = `\`${keyPath[0].toString()}\``;
    const path = keyPath.length > 1 ? keyPath.slice(1).map((pathPart => "number" == typeof pathPart ? `[${pathPart}]` : pathPart)).join(".") : void 0, error = new SolanaError(SOLANA_ERROR__RPC__INTEGER_OVERFLOW, {
        argumentLabel: argumentLabel,
        keyPath: keyPath,
        methodName: methodName,
        optionalPathLabel: path ? ` at path \`${path}\`` : "",
        value: value,
        ...void 0 !== path ? {
            path: path
        } : void 0
    });
    return safeCaptureStackTrace(error, createSolanaJsonRpcIntegerOverflowError), error;
}

var DEFAULT_RPC_CONFIG = {
    defaultCommitment: "confirmed",
    onIntegerOverflow(request, keyPath, value) {
        throw createSolanaJsonRpcIntegerOverflowError(request.methodName, keyPath, value);
    }
}, EXPLICIT_ABORT_TOKEN;

function createExplicitAbortToken() {
    return "production" !== process.env.NODE_ENV ? {
        EXPLICIT_ABORT_TOKEN: "This object is thrown from the request that underlies a series of coalesced requests when the last request in that series aborts"
    } : {};
}

function getRpcTransportWithRequestCoalescing(transport, getDeduplicationKey) {
    let coalescedRequestsByDeduplicationKey;
    return async function(request) {
        const {payload: payload, signal: signal} = request, deduplicationKey = getDeduplicationKey(payload);
        if (void 0 === deduplicationKey) return await transport(request);
        if (coalescedRequestsByDeduplicationKey || (queueMicrotask((() => {
            coalescedRequestsByDeduplicationKey = void 0;
        })), coalescedRequestsByDeduplicationKey = {}), null == coalescedRequestsByDeduplicationKey[deduplicationKey]) {
            const abortController = new AbortController, responsePromise = (async () => {
                try {
                    return await transport({
                        ...request,
                        signal: abortController.signal
                    });
                } catch (e) {
                    if (e === (EXPLICIT_ABORT_TOKEN ||= createExplicitAbortToken())) return;
                    throw e;
                }
            })();
            coalescedRequestsByDeduplicationKey[deduplicationKey] = {
                abortController: abortController,
                numConsumers: 0,
                responsePromise: responsePromise
            };
        }
        const coalescedRequest = coalescedRequestsByDeduplicationKey[deduplicationKey];
        if (coalescedRequest.numConsumers++, signal) {
            const responsePromise = coalescedRequest.responsePromise;
            return await new Promise(((resolve, reject) => {
                const handleAbort = e => {
                    signal.removeEventListener("abort", handleAbort), coalescedRequest.numConsumers -= 1, 
                    queueMicrotask((() => {
                        if (0 === coalescedRequest.numConsumers) {
                            coalescedRequest.abortController.abort(EXPLICIT_ABORT_TOKEN ||= createExplicitAbortToken());
                        }
                    })), reject(e.target.reason);
                };
                signal.addEventListener("abort", handleAbort), responsePromise.then(resolve).catch(reject).finally((() => {
                    signal.removeEventListener("abort", handleAbort);
                }));
            }));
        }
        return await coalescedRequest.responsePromise;
    };
}

function getSolanaRpcPayloadDeduplicationKey(payload) {
    return isJsonRpcPayload(payload) ? src_default([ payload.method, payload.params ]) : void 0;
}

function normalizeHeaders(headers) {
    const out = {};
    for (const headerName in headers) out[headerName.toLowerCase()] = headers[headerName];
    return out;
}

function createDefaultRpcTransport(config) {
    return pipe$1(createHttpTransportForSolanaRpc({
        ...config,
        headers: {
            "accept-encoding": "br,gzip,deflate",
            ...config.headers ? normalizeHeaders(config.headers) : void 0,
            "solana-client": "js/2.0.0"
        }
    }), (transport => getRpcTransportWithRequestCoalescing(transport, getSolanaRpcPayloadDeduplicationKey)));
}

function createSolanaRpc(clusterUrl, config) {
    return createSolanaRpcFromTransport(createDefaultRpcTransport({
        url: clusterUrl,
        ...config
    }));
}

function createSolanaRpcFromTransport(transport) {
    return createRpc({
        api: createSolanaRpcApi(DEFAULT_RPC_CONFIG),
        transport: transport
    });
}

function mainnet(putativeString) {
    return putativeString;
}

function devnet(putativeString) {
    return putativeString;
}

function testnet(putativeString) {
    return putativeString;
}

function isStringifiedNumber(putativeNumber) {
    return !Number.isNaN(Number(putativeNumber));
}

function uint8ArraysEqual(arr1, arr2) {
    return arr1.length === arr2.length && arr1.every(((value, index) => value === arr2[index]));
}

async function partiallySignTransaction(keyPairs, transaction) {
    let newSignatures, unexpectedSigners;
    if (await Promise.all(keyPairs.map((async keyPair => {
        const address = await getAddressFromPublicKey(keyPair.publicKey), existingSignature = transaction.signatures[address];
        if (void 0 === existingSignature) return unexpectedSigners ||= new Set, void unexpectedSigners.add(address);
        if (unexpectedSigners) return;
        const newSignature = await signBytes(keyPair.privateKey, transaction.messageBytes);
        null !== existingSignature && uint8ArraysEqual(newSignature, existingSignature) || (newSignatures ||= {}, 
        newSignatures[address] = newSignature);
    }))), unexpectedSigners && unexpectedSigners.size > 0) {
        const expectedSigners = Object.keys(transaction.signatures);
        throw new SolanaError(SOLANA_ERROR__TRANSACTION__ADDRESSES_CANNOT_SIGN_TRANSACTION, {
            expectedAddresses: expectedSigners,
            unexpectedAddresses: [ ...unexpectedSigners ]
        });
    }
    return newSignatures ? Object.freeze({
        ...transaction,
        signatures: Object.freeze({
            ...transaction.signatures,
            ...newSignatures
        })
    }) : transaction;
}

async function createSignerFromKeyPair(keyPair) {
    const address = await getAddressFromPublicKey(keyPair.publicKey), out = {
        address: address,
        keyPair: keyPair,
        signMessages: messages => Promise.all(messages.map((async message => Object.freeze({
            [address]: await signBytes(keyPair.privateKey, message.content)
        })))),
        signTransactions: transactions => Promise.all(transactions.map((async transaction => {
            const signedTransaction = await partiallySignTransaction([ keyPair ], transaction);
            return Object.freeze({
                [address]: signedTransaction.signatures[address]
            });
        })))
    };
    return Object.freeze(out);
}

async function createKeyPairSignerFromBytes(bytes, extractable) {
    return await createSignerFromKeyPair(await createKeyPairFromBytes(bytes));
}

function loadKeypairFromFile(filePath = DEFAULT_KEYPAIR_PATH) {
    if (!doesFileExist(filePath)) return null;
    const jsonBytes = loadJsonFile(filePath);
    return createKeyPairSignerFromBytes(Buffer.from(jsonBytes));
}

function parseRpcUrlOrMoniker(input, includeBetaLabel = !0, allowUrl = !0) {
    if (allowUrl && input.match(/^http?s/i)) {
        try {
            return new URL(input).toString();
        } catch (err) {
            console.error("Unable to parse 'url':", input), process.exit(1);
        }
        return input;
    }
    return input.startsWith("local") || input.startsWith("l") ? "localhost" : input.startsWith("t") ? "testnet" : input.startsWith("d") ? "devnet" : (input.startsWith("m") || warnMessage("Unable to parse url or moniker. Falling back to mainnet"), 
    includeBetaLabel ? "mainnet-beta" : "mainnet");
}

function getSafeClusterMoniker(cluster, labels) {
    switch (labels || (labels = {
        devnet: {},
        localnet: {},
        mainnet: {},
        testnet: {}
    }), cluster = parseRpcUrlOrMoniker(cluster, !0, !1)) {
      case "localhost":
      case "localnet":
        cluster = "localnet";
        break;

      case "mainnet":
      case "mainnet-beta":
        cluster = "mainnet";
    }
    return !!Object.hasOwn(labels, cluster) && cluster;
}

function getPlatformToolsVersions() {
    const res = getCommandOutputSync("cargo build-sbf --version"), tools = {};
    return res ? (res.split("\n").map((line => {
        if (!(line = line.trim().toLowerCase())) return;
        const version = VERSION_REGEX.exec(line)?.[1];
        line.startsWith("rustc") && (tools.rustc = version), line.startsWith("platform-tools") && (tools["platform-tools"] = version), 
        line.startsWith("solana-cargo-build-") && (tools["build-sbf"] = version);
    })), tools) : tools;
}

function cliProgramRoot() {
    return (new Command).name("mucho").configureOutput(cliOutputConfig).description("mucho tools, one cli").option("--version", "output the version number of this tool", (async () => {
        if (console.log("mucho", getAppInfo().version), 2 === process.argv.indexOf("--version")) {
            console.log(getCommandOutputSync(TOOL_CONFIG.rust.version) || "rustc not found"), 
            console.log("node", getCommandOutputSync(TOOL_CONFIG.node.version) || "not found (odd if you are running this node tool...)"), 
            console.log(getCommandOutputSync(TOOL_CONFIG.solana.version) || "solana cli not found");
            const solanaTools = getPlatformToolsVersions();
            console.log("solana platform tools:");
            for (const key in solanaTools) console.log("  " + key + ": " + solanaTools[key]);
            console.log("\nFor more info run: 'npx mucho info'");
        }
        process.exit(0);
    }));
}

const ANSI_BACKGROUND_OFFSET = 10, wrapAnsi16 = (offset = 0) => code => `[${code + offset}m`, wrapAnsi256 = (offset = 0) => code => `[${38 + offset};5;${code}m`, wrapAnsi16m = (offset = 0) => (red, green, blue) => `[${38 + offset};2;${red};${green};${blue}m`, styles$2 = {
    modifier: {
        reset: [ 0, 0 ],
        bold: [ 1, 22 ],
        dim: [ 2, 22 ],
        italic: [ 3, 23 ],
        underline: [ 4, 24 ],
        overline: [ 53, 55 ],
        inverse: [ 7, 27 ],
        hidden: [ 8, 28 ],
        strikethrough: [ 9, 29 ]
    },
    color: {
        black: [ 30, 39 ],
        red: [ 31, 39 ],
        green: [ 32, 39 ],
        yellow: [ 33, 39 ],
        blue: [ 34, 39 ],
        magenta: [ 35, 39 ],
        cyan: [ 36, 39 ],
        white: [ 37, 39 ],
        blackBright: [ 90, 39 ],
        gray: [ 90, 39 ],
        grey: [ 90, 39 ],
        redBright: [ 91, 39 ],
        greenBright: [ 92, 39 ],
        yellowBright: [ 93, 39 ],
        blueBright: [ 94, 39 ],
        magentaBright: [ 95, 39 ],
        cyanBright: [ 96, 39 ],
        whiteBright: [ 97, 39 ]
    },
    bgColor: {
        bgBlack: [ 40, 49 ],
        bgRed: [ 41, 49 ],
        bgGreen: [ 42, 49 ],
        bgYellow: [ 43, 49 ],
        bgBlue: [ 44, 49 ],
        bgMagenta: [ 45, 49 ],
        bgCyan: [ 46, 49 ],
        bgWhite: [ 47, 49 ],
        bgBlackBright: [ 100, 49 ],
        bgGray: [ 100, 49 ],
        bgGrey: [ 100, 49 ],
        bgRedBright: [ 101, 49 ],
        bgGreenBright: [ 102, 49 ],
        bgYellowBright: [ 103, 49 ],
        bgBlueBright: [ 104, 49 ],
        bgMagentaBright: [ 105, 49 ],
        bgCyanBright: [ 106, 49 ],
        bgWhiteBright: [ 107, 49 ]
    }
};

Object.keys(styles$2.modifier);

const foregroundColorNames = Object.keys(styles$2.color), backgroundColorNames = Object.keys(styles$2.bgColor);

function assembleStyles() {
    const codes = new Map;
    for (const [groupName, group] of Object.entries(styles$2)) {
        for (const [styleName, style] of Object.entries(group)) styles$2[styleName] = {
            open: `[${style[0]}m`,
            close: `[${style[1]}m`
        }, group[styleName] = styles$2[styleName], codes.set(style[0], style[1]);
        Object.defineProperty(styles$2, groupName, {
            value: group,
            enumerable: !1
        });
    }
    return Object.defineProperty(styles$2, "codes", {
        value: codes,
        enumerable: !1
    }), styles$2.color.close = "[39m", styles$2.bgColor.close = "[49m", styles$2.color.ansi = wrapAnsi16(), 
    styles$2.color.ansi256 = wrapAnsi256(), styles$2.color.ansi16m = wrapAnsi16m(), 
    styles$2.bgColor.ansi = wrapAnsi16(ANSI_BACKGROUND_OFFSET), styles$2.bgColor.ansi256 = wrapAnsi256(ANSI_BACKGROUND_OFFSET), 
    styles$2.bgColor.ansi16m = wrapAnsi16m(ANSI_BACKGROUND_OFFSET), Object.defineProperties(styles$2, {
        rgbToAnsi256: {
            value: (red, green, blue) => red === green && green === blue ? red < 8 ? 16 : red > 248 ? 231 : Math.round((red - 8) / 247 * 24) + 232 : 16 + 36 * Math.round(red / 255 * 5) + 6 * Math.round(green / 255 * 5) + Math.round(blue / 255 * 5),
            enumerable: !1
        },
        hexToRgb: {
            value(hex) {
                const matches = /[a-f\d]{6}|[a-f\d]{3}/i.exec(hex.toString(16));
                if (!matches) return [ 0, 0, 0 ];
                let [colorString] = matches;
                3 === colorString.length && (colorString = [ ...colorString ].map((character => character + character)).join(""));
                const integer = Number.parseInt(colorString, 16);
                return [ integer >> 16 & 255, integer >> 8 & 255, 255 & integer ];
            },
            enumerable: !1
        },
        hexToAnsi256: {
            value: hex => styles$2.rgbToAnsi256(...styles$2.hexToRgb(hex)),
            enumerable: !1
        },
        ansi256ToAnsi: {
            value(code) {
                if (code < 8) return 30 + code;
                if (code < 16) return code - 8 + 90;
                let red, green, blue;
                if (code >= 232) red = (10 * (code - 232) + 8) / 255, green = red, blue = red; else {
                    const remainder = (code -= 16) % 36;
                    red = Math.floor(code / 36) / 5, green = Math.floor(remainder / 6) / 5, blue = remainder % 6 / 5;
                }
                const value = 2 * Math.max(red, green, blue);
                if (0 === value) return 30;
                let result = 30 + (Math.round(blue) << 2 | Math.round(green) << 1 | Math.round(red));
                return 2 === value && (result += 60), result;
            },
            enumerable: !1
        },
        rgbToAnsi: {
            value: (red, green, blue) => styles$2.ansi256ToAnsi(styles$2.rgbToAnsi256(red, green, blue)),
            enumerable: !1
        },
        hexToAnsi: {
            value: hex => styles$2.ansi256ToAnsi(styles$2.hexToAnsi256(hex)),
            enumerable: !1
        }
    }), styles$2;
}

const ansiStyles$1 = assembleStyles();

function hasFlag$1(flag, argv = (globalThis.Deno ? globalThis.Deno.args : process$2.argv)) {
    const prefix = flag.startsWith("-") ? "" : 1 === flag.length ? "-" : "--", position = argv.indexOf(prefix + flag), terminatorPosition = argv.indexOf("--");
    return -1 !== position && (-1 === terminatorPosition || position < terminatorPosition);
}

const {env: env} = process$2;

let flagForceColor;

function envForceColor() {
    if ("FORCE_COLOR" in env) return "true" === env.FORCE_COLOR ? 1 : "false" === env.FORCE_COLOR ? 0 : 0 === env.FORCE_COLOR.length ? 1 : Math.min(Number.parseInt(env.FORCE_COLOR, 10), 3);
}

function translateLevel(level) {
    return 0 !== level && {
        level: level,
        hasBasic: !0,
        has256: level >= 2,
        has16m: level >= 3
    };
}

function _supportsColor(haveStream, {streamIsTTY: streamIsTTY, sniffFlags: sniffFlags = !0} = {}) {
    const noFlagForceColor = envForceColor();
    void 0 !== noFlagForceColor && (flagForceColor = noFlagForceColor);
    const forceColor = sniffFlags ? flagForceColor : noFlagForceColor;
    if (0 === forceColor) return 0;
    if (sniffFlags) {
        if (hasFlag$1("color=16m") || hasFlag$1("color=full") || hasFlag$1("color=truecolor")) return 3;
        if (hasFlag$1("color=256")) return 2;
    }
    if ("TF_BUILD" in env && "AGENT_NAME" in env) return 1;
    if (haveStream && !streamIsTTY && void 0 === forceColor) return 0;
    const min = forceColor || 0;
    if ("dumb" === env.TERM) return min;
    if ("win32" === process$2.platform) {
        const osRelease = os.release().split(".");
        return Number(osRelease[0]) >= 10 && Number(osRelease[2]) >= 10586 ? Number(osRelease[2]) >= 14931 ? 3 : 2 : 1;
    }
    if ("CI" in env) return "GITHUB_ACTIONS" in env || "GITEA_ACTIONS" in env ? 3 : [ "TRAVIS", "CIRCLECI", "APPVEYOR", "GITLAB_CI", "BUILDKITE", "DRONE" ].some((sign => sign in env)) || "codeship" === env.CI_NAME ? 1 : min;
    if ("TEAMCITY_VERSION" in env) return /^(9\.(0*[1-9]\d*)\.|\d{2,}\.)/.test(env.TEAMCITY_VERSION) ? 1 : 0;
    if ("truecolor" === env.COLORTERM) return 3;
    if ("xterm-kitty" === env.TERM) return 3;
    if ("TERM_PROGRAM" in env) {
        const version = Number.parseInt((env.TERM_PROGRAM_VERSION || "").split(".")[0], 10);
        switch (env.TERM_PROGRAM) {
          case "iTerm.app":
            return version >= 3 ? 3 : 2;

          case "Apple_Terminal":
            return 2;
        }
    }
    return /-256(color)?$/i.test(env.TERM) ? 2 : /^screen|^xterm|^vt100|^vt220|^rxvt|color|ansi|cygwin|linux/i.test(env.TERM) || "COLORTERM" in env ? 1 : min;
}

function createSupportsColor(stream, options = {}) {
    return translateLevel(_supportsColor(stream, {
        streamIsTTY: stream && stream.isTTY,
        ...options
    }));
}

hasFlag$1("no-color") || hasFlag$1("no-colors") || hasFlag$1("color=false") || hasFlag$1("color=never") ? flagForceColor = 0 : (hasFlag$1("color") || hasFlag$1("colors") || hasFlag$1("color=true") || hasFlag$1("color=always")) && (flagForceColor = 1);

const supportsColor = {
    stdout: createSupportsColor({
        isTTY: tty.isatty(1)
    }),
    stderr: createSupportsColor({
        isTTY: tty.isatty(2)
    })
};

function stringReplaceAll(string, substring, replacer) {
    let index = string.indexOf(substring);
    if (-1 === index) return string;
    const substringLength = substring.length;
    let endIndex = 0, returnValue = "";
    do {
        returnValue += string.slice(endIndex, index) + substring + replacer, endIndex = index + substringLength, 
        index = string.indexOf(substring, endIndex);
    } while (-1 !== index);
    return returnValue += string.slice(endIndex), returnValue;
}

function stringEncaseCRLFWithFirstIndex(string, prefix, postfix, index) {
    let endIndex = 0, returnValue = "";
    do {
        const gotCR = "\r" === string[index - 1];
        returnValue += string.slice(endIndex, gotCR ? index - 1 : index) + prefix + (gotCR ? "\r\n" : "\n") + postfix, 
        endIndex = index + 1, index = string.indexOf("\n", endIndex);
    } while (-1 !== index);
    return returnValue += string.slice(endIndex), returnValue;
}

const {stdout: stdoutColor, stderr: stderrColor} = supportsColor, GENERATOR = Symbol("GENERATOR"), STYLER = Symbol("STYLER"), IS_EMPTY = Symbol("IS_EMPTY"), levelMapping = [ "ansi", "ansi", "ansi256", "ansi16m" ], styles$1 = Object.create(null), applyOptions = (object, options = {}) => {
    if (options.level && !(Number.isInteger(options.level) && options.level >= 0 && options.level <= 3)) throw new Error("The `level` option should be an integer from 0 to 3");
    const colorLevel = stdoutColor ? stdoutColor.level : 0;
    object.level = void 0 === options.level ? colorLevel : options.level;
}, chalkFactory = options => {
    const chalk = (...strings) => strings.join(" ");
    return applyOptions(chalk, options), Object.setPrototypeOf(chalk, createChalk.prototype), 
    chalk;
};

function createChalk(options) {
    return chalkFactory(options);
}

Object.setPrototypeOf(createChalk.prototype, Function.prototype);

for (const [styleName, style] of Object.entries(ansiStyles$1)) styles$1[styleName] = {
    get() {
        const builder = createBuilder(this, createStyler(style.open, style.close, this[STYLER]), this[IS_EMPTY]);
        return Object.defineProperty(this, styleName, {
            value: builder
        }), builder;
    }
};

styles$1.visible = {
    get() {
        const builder = createBuilder(this, this[STYLER], !0);
        return Object.defineProperty(this, "visible", {
            value: builder
        }), builder;
    }
};

const getModelAnsi = (model, level, type, ...arguments_) => "rgb" === model ? "ansi16m" === level ? ansiStyles$1[type].ansi16m(...arguments_) : "ansi256" === level ? ansiStyles$1[type].ansi256(ansiStyles$1.rgbToAnsi256(...arguments_)) : ansiStyles$1[type].ansi(ansiStyles$1.rgbToAnsi(...arguments_)) : "hex" === model ? getModelAnsi("rgb", level, type, ...ansiStyles$1.hexToRgb(...arguments_)) : ansiStyles$1[type][model](...arguments_), usedModels = [ "rgb", "hex", "ansi256" ];

for (const model of usedModels) {
    styles$1[model] = {
        get() {
            const {level: level} = this;
            return function(...arguments_) {
                const styler = createStyler(getModelAnsi(model, levelMapping[level], "color", ...arguments_), ansiStyles$1.color.close, this[STYLER]);
                return createBuilder(this, styler, this[IS_EMPTY]);
            };
        }
    };
    const bgModel = "bg" + model[0].toUpperCase() + model.slice(1);
    styles$1[bgModel] = {
        get() {
            const {level: level} = this;
            return function(...arguments_) {
                const styler = createStyler(getModelAnsi(model, levelMapping[level], "bgColor", ...arguments_), ansiStyles$1.bgColor.close, this[STYLER]);
                return createBuilder(this, styler, this[IS_EMPTY]);
            };
        }
    };
}

const proto = Object.defineProperties((() => {}), {
    ...styles$1,
    level: {
        enumerable: !0,
        get() {
            return this[GENERATOR].level;
        },
        set(level) {
            this[GENERATOR].level = level;
        }
    }
}), createStyler = (open, close, parent) => {
    let openAll, closeAll;
    return void 0 === parent ? (openAll = open, closeAll = close) : (openAll = parent.openAll + open, 
    closeAll = close + parent.closeAll), {
        open: open,
        close: close,
        openAll: openAll,
        closeAll: closeAll,
        parent: parent
    };
}, createBuilder = (self, _styler, _isEmpty) => {
    const builder = (...arguments_) => applyStyle(builder, 1 === arguments_.length ? "" + arguments_[0] : arguments_.join(" "));
    return Object.setPrototypeOf(builder, proto), builder[GENERATOR] = self, builder[STYLER] = _styler, 
    builder[IS_EMPTY] = _isEmpty, builder;
}, applyStyle = (self, string) => {
    if (self.level <= 0 || !string) return self[IS_EMPTY] ? "" : string;
    let styler = self[STYLER];
    if (void 0 === styler) return string;
    const {openAll: openAll, closeAll: closeAll} = styler;
    if (string.includes("")) for (;void 0 !== styler; ) string = stringReplaceAll(string, styler.close, styler.open), 
    styler = styler.parent;
    const lfIndex = string.indexOf("\n");
    return -1 !== lfIndex && (string = stringEncaseCRLFWithFirstIndex(string, closeAll, openAll, lfIndex)), 
    openAll + string + closeAll;
};

Object.defineProperties(createChalk.prototype, styles$1);

const chalk = createChalk();

createChalk({
    level: stderrColor ? stderrColor.level : 0
});

const copyProperty = (to, from, property, ignoreNonConfigurable) => {
    if ("length" === property || "prototype" === property) return;
    if ("arguments" === property || "caller" === property) return;
    const toDescriptor = Object.getOwnPropertyDescriptor(to, property), fromDescriptor = Object.getOwnPropertyDescriptor(from, property);
    !canCopyProperty(toDescriptor, fromDescriptor) && ignoreNonConfigurable || Object.defineProperty(to, property, fromDescriptor);
}, canCopyProperty = function(toDescriptor, fromDescriptor) {
    return void 0 === toDescriptor || toDescriptor.configurable || toDescriptor.writable === fromDescriptor.writable && toDescriptor.enumerable === fromDescriptor.enumerable && toDescriptor.configurable === fromDescriptor.configurable && (toDescriptor.writable || toDescriptor.value === fromDescriptor.value);
}, changePrototype = (to, from) => {
    const fromPrototype = Object.getPrototypeOf(from);
    fromPrototype !== Object.getPrototypeOf(to) && Object.setPrototypeOf(to, fromPrototype);
}, wrappedToString = (withName, fromBody) => `/* Wrapped ${withName}*/\n${fromBody}`, toStringDescriptor = Object.getOwnPropertyDescriptor(Function.prototype, "toString"), toStringName = Object.getOwnPropertyDescriptor(Function.prototype.toString, "name"), changeToString = (to, from, name) => {
    const withName = "" === name ? "" : `with ${name.trim()}() `, newToString = wrappedToString.bind(null, withName, from.toString());
    Object.defineProperty(newToString, "name", toStringName);
    const {writable: writable, enumerable: enumerable, configurable: configurable} = toStringDescriptor;
    Object.defineProperty(to, "toString", {
        value: newToString,
        writable: writable,
        enumerable: enumerable,
        configurable: configurable
    });
};

function mimicFunction(to, from, {ignoreNonConfigurable: ignoreNonConfigurable = !1} = {}) {
    const {name: name} = to;
    for (const property of Reflect.ownKeys(from)) copyProperty(to, from, property, ignoreNonConfigurable);
    return changePrototype(to, from), changeToString(to, from, name), to;
}

const calledFunctions = new WeakMap, onetime = (function_, options = {}) => {
    if ("function" != typeof function_) throw new TypeError("Expected a function");
    let returnValue, callCount = 0;
    const functionName = function_.displayName || function_.name || "<anonymous>", onetime = function(...arguments_) {
        if (calledFunctions.set(onetime, ++callCount), 1 === callCount) returnValue = function_.apply(this, arguments_), 
        function_ = void 0; else if (!0 === options.throw) throw new Error(`Function \`${functionName}\` can only be called once`);
        return returnValue;
    };
    return mimicFunction(onetime, function_), calledFunctions.set(onetime, callCount), 
    onetime;
};

onetime.callCount = function_ => {
    if (!calledFunctions.has(function_)) throw new Error(`The given function \`${function_.name}\` is not wrapped by the \`onetime\` package`);
    return calledFunctions.get(function_);
};

const signals = [];

signals.push("SIGHUP", "SIGINT", "SIGTERM"), "win32" !== process.platform && signals.push("SIGALRM", "SIGABRT", "SIGVTALRM", "SIGXCPU", "SIGXFSZ", "SIGUSR2", "SIGTRAP", "SIGSYS", "SIGQUIT", "SIGIOT"), 
"linux" === process.platform && signals.push("SIGIO", "SIGPOLL", "SIGPWR", "SIGSTKFLT");

const processOk = process => !!process && "object" == typeof process && "function" == typeof process.removeListener && "function" == typeof process.emit && "function" == typeof process.reallyExit && "function" == typeof process.listeners && "function" == typeof process.kill && "number" == typeof process.pid && "function" == typeof process.on, kExitEmitter = Symbol.for("signal-exit emitter"), global$1 = globalThis, ObjectDefineProperty = Object.defineProperty.bind(Object);

class Emitter {
    emitted={
        afterExit: !1,
        exit: !1
    };
    listeners={
        afterExit: [],
        exit: []
    };
    count=0;
    id=Math.random();
    constructor() {
        if (global$1[kExitEmitter]) return global$1[kExitEmitter];
        ObjectDefineProperty(global$1, kExitEmitter, {
            value: this,
            writable: !1,
            enumerable: !1,
            configurable: !1
        });
    }
    on(ev, fn) {
        this.listeners[ev].push(fn);
    }
    removeListener(ev, fn) {
        const list = this.listeners[ev], i = list.indexOf(fn);
        -1 !== i && (0 === i && 1 === list.length ? list.length = 0 : list.splice(i, 1));
    }
    emit(ev, code, signal) {
        if (this.emitted[ev]) return !1;
        this.emitted[ev] = !0;
        let ret = !1;
        for (const fn of this.listeners[ev]) ret = !0 === fn(code, signal) || ret;
        return "exit" === ev && (ret = this.emit("afterExit", code, signal) || ret), ret;
    }
}

class SignalExitBase {}

const signalExitWrap = handler => ({
    onExit: (cb, opts) => handler.onExit(cb, opts),
    load: () => handler.load(),
    unload: () => handler.unload()
});

class SignalExitFallback extends SignalExitBase {
    onExit() {
        return () => {};
    }
    load() {}
    unload() {}
}

class SignalExit extends SignalExitBase {
    #hupSig="win32" === process$1.platform ? "SIGINT" : "SIGHUP";
    #emitter=new Emitter;
    #process;
    #originalProcessEmit;
    #originalProcessReallyExit;
    #sigListeners={};
    #loaded=!1;
    constructor(process) {
        super(), this.#process = process, this.#sigListeners = {};
        for (const sig of signals) this.#sigListeners[sig] = () => {
            const listeners = this.#process.listeners(sig);
            let {count: count} = this.#emitter;
            const p = process;
            if ("object" == typeof p.__signal_exit_emitter__ && "number" == typeof p.__signal_exit_emitter__.count && (count += p.__signal_exit_emitter__.count), 
            listeners.length === count) {
                this.unload();
                const ret = this.#emitter.emit("exit", null, sig), s = "SIGHUP" === sig ? this.#hupSig : sig;
                ret || process.kill(process.pid, s);
            }
        };
        this.#originalProcessReallyExit = process.reallyExit, this.#originalProcessEmit = process.emit;
    }
    onExit(cb, opts) {
        if (!processOk(this.#process)) return () => {};
        !1 === this.#loaded && this.load();
        const ev = opts?.alwaysLast ? "afterExit" : "exit";
        return this.#emitter.on(ev, cb), () => {
            this.#emitter.removeListener(ev, cb), 0 === this.#emitter.listeners.exit.length && 0 === this.#emitter.listeners.afterExit.length && this.unload();
        };
    }
    load() {
        if (!this.#loaded) {
            this.#loaded = !0, this.#emitter.count += 1;
            for (const sig of signals) try {
                const fn = this.#sigListeners[sig];
                fn && this.#process.on(sig, fn);
            } catch (_) {}
            this.#process.emit = (ev, ...a) => this.#processEmit(ev, ...a), this.#process.reallyExit = code => this.#processReallyExit(code);
        }
    }
    unload() {
        this.#loaded && (this.#loaded = !1, signals.forEach((sig => {
            const listener = this.#sigListeners[sig];
            if (!listener) throw new Error("Listener not defined for signal: " + sig);
            try {
                this.#process.removeListener(sig, listener);
            } catch (_) {}
        })), this.#process.emit = this.#originalProcessEmit, this.#process.reallyExit = this.#originalProcessReallyExit, 
        this.#emitter.count -= 1);
    }
    #processReallyExit(code) {
        return processOk(this.#process) ? (this.#process.exitCode = code || 0, this.#emitter.emit("exit", this.#process.exitCode, null), 
        this.#originalProcessReallyExit.call(this.#process, this.#process.exitCode)) : 0;
    }
    #processEmit(ev, ...args) {
        const og = this.#originalProcessEmit;
        if ("exit" === ev && processOk(this.#process)) {
            "number" == typeof args[0] && (this.#process.exitCode = args[0]);
            const ret = og.call(this.#process, ev, ...args);
            return this.#emitter.emit("exit", this.#process.exitCode, null), ret;
        }
        return og.call(this.#process, ev, ...args);
    }
}

const process$1 = globalThis.process, {onExit: onExit, load: load, unload: unload} = signalExitWrap(processOk(process$1) ? new SignalExit(process$1) : new SignalExitFallback), terminal = process$2.stderr.isTTY ? process$2.stderr : process$2.stdout.isTTY ? process$2.stdout : void 0, restoreCursor = terminal ? onetime((() => {
    onExit((() => {
        terminal.write("[?25h");
    }), {
        alwaysLast: !0
    });
})) : () => {};

let isHidden = !1;

const cliCursor = {
    show: (writableStream = process$2.stderr) => {
        writableStream.isTTY && (isHidden = !1, writableStream.write("[?25h"));
    },
    hide: (writableStream = process$2.stderr) => {
        writableStream.isTTY && (restoreCursor(), isHidden = !0, writableStream.write("[?25l"));
    },
    toggle: (force, writableStream) => {
        void 0 !== force && (isHidden = force), isHidden ? cliCursor.show(writableStream) : cliCursor.hide(writableStream);
    }
};

var dots = {
    interval: 80,
    frames: [ "⠋", "⠙", "⠹", "⠸", "⠼", "⠴", "⠦", "⠧", "⠇", "⠏" ]
}, dots2 = {
    interval: 80,
    frames: [ "⣾", "⣽", "⣻", "⢿", "⡿", "⣟", "⣯", "⣷" ]
}, dots3 = {
    interval: 80,
    frames: [ "⠋", "⠙", "⠚", "⠞", "⠖", "⠦", "⠴", "⠲", "⠳", "⠓" ]
}, dots4 = {
    interval: 80,
    frames: [ "⠄", "⠆", "⠇", "⠋", "⠙", "⠸", "⠰", "⠠", "⠰", "⠸", "⠙", "⠋", "⠇", "⠆" ]
}, dots5 = {
    interval: 80,
    frames: [ "⠋", "⠙", "⠚", "⠒", "⠂", "⠂", "⠒", "⠲", "⠴", "⠦", "⠖", "⠒", "⠐", "⠐", "⠒", "⠓", "⠋" ]
}, dots6 = {
    interval: 80,
    frames: [ "⠁", "⠉", "⠙", "⠚", "⠒", "⠂", "⠂", "⠒", "⠲", "⠴", "⠤", "⠄", "⠄", "⠤", "⠴", "⠲", "⠒", "⠂", "⠂", "⠒", "⠚", "⠙", "⠉", "⠁" ]
}, dots7 = {
    interval: 80,
    frames: [ "⠈", "⠉", "⠋", "⠓", "⠒", "⠐", "⠐", "⠒", "⠖", "⠦", "⠤", "⠠", "⠠", "⠤", "⠦", "⠖", "⠒", "⠐", "⠐", "⠒", "⠓", "⠋", "⠉", "⠈" ]
}, dots8 = {
    interval: 80,
    frames: [ "⠁", "⠁", "⠉", "⠙", "⠚", "⠒", "⠂", "⠂", "⠒", "⠲", "⠴", "⠤", "⠄", "⠄", "⠤", "⠠", "⠠", "⠤", "⠦", "⠖", "⠒", "⠐", "⠐", "⠒", "⠓", "⠋", "⠉", "⠈", "⠈" ]
}, dots9 = {
    interval: 80,
    frames: [ "⢹", "⢺", "⢼", "⣸", "⣇", "⡧", "⡗", "⡏" ]
}, dots10 = {
    interval: 80,
    frames: [ "⢄", "⢂", "⢁", "⡁", "⡈", "⡐", "⡠" ]
}, dots11 = {
    interval: 100,
    frames: [ "⠁", "⠂", "⠄", "⡀", "⢀", "⠠", "⠐", "⠈" ]
}, dots12 = {
    interval: 80,
    frames: [ "⢀⠀", "⡀⠀", "⠄⠀", "⢂⠀", "⡂⠀", "⠅⠀", "⢃⠀", "⡃⠀", "⠍⠀", "⢋⠀", "⡋⠀", "⠍⠁", "⢋⠁", "⡋⠁", "⠍⠉", "⠋⠉", "⠋⠉", "⠉⠙", "⠉⠙", "⠉⠩", "⠈⢙", "⠈⡙", "⢈⠩", "⡀⢙", "⠄⡙", "⢂⠩", "⡂⢘", "⠅⡘", "⢃⠨", "⡃⢐", "⠍⡐", "⢋⠠", "⡋⢀", "⠍⡁", "⢋⠁", "⡋⠁", "⠍⠉", "⠋⠉", "⠋⠉", "⠉⠙", "⠉⠙", "⠉⠩", "⠈⢙", "⠈⡙", "⠈⠩", "⠀⢙", "⠀⡙", "⠀⠩", "⠀⢘", "⠀⡘", "⠀⠨", "⠀⢐", "⠀⡐", "⠀⠠", "⠀⢀", "⠀⡀" ]
}, dots13 = {
    interval: 80,
    frames: [ "⣼", "⣹", "⢻", "⠿", "⡟", "⣏", "⣧", "⣶" ]
}, dots8Bit = {
    interval: 80,
    frames: [ "⠀", "⠁", "⠂", "⠃", "⠄", "⠅", "⠆", "⠇", "⡀", "⡁", "⡂", "⡃", "⡄", "⡅", "⡆", "⡇", "⠈", "⠉", "⠊", "⠋", "⠌", "⠍", "⠎", "⠏", "⡈", "⡉", "⡊", "⡋", "⡌", "⡍", "⡎", "⡏", "⠐", "⠑", "⠒", "⠓", "⠔", "⠕", "⠖", "⠗", "⡐", "⡑", "⡒", "⡓", "⡔", "⡕", "⡖", "⡗", "⠘", "⠙", "⠚", "⠛", "⠜", "⠝", "⠞", "⠟", "⡘", "⡙", "⡚", "⡛", "⡜", "⡝", "⡞", "⡟", "⠠", "⠡", "⠢", "⠣", "⠤", "⠥", "⠦", "⠧", "⡠", "⡡", "⡢", "⡣", "⡤", "⡥", "⡦", "⡧", "⠨", "⠩", "⠪", "⠫", "⠬", "⠭", "⠮", "⠯", "⡨", "⡩", "⡪", "⡫", "⡬", "⡭", "⡮", "⡯", "⠰", "⠱", "⠲", "⠳", "⠴", "⠵", "⠶", "⠷", "⡰", "⡱", "⡲", "⡳", "⡴", "⡵", "⡶", "⡷", "⠸", "⠹", "⠺", "⠻", "⠼", "⠽", "⠾", "⠿", "⡸", "⡹", "⡺", "⡻", "⡼", "⡽", "⡾", "⡿", "⢀", "⢁", "⢂", "⢃", "⢄", "⢅", "⢆", "⢇", "⣀", "⣁", "⣂", "⣃", "⣄", "⣅", "⣆", "⣇", "⢈", "⢉", "⢊", "⢋", "⢌", "⢍", "⢎", "⢏", "⣈", "⣉", "⣊", "⣋", "⣌", "⣍", "⣎", "⣏", "⢐", "⢑", "⢒", "⢓", "⢔", "⢕", "⢖", "⢗", "⣐", "⣑", "⣒", "⣓", "⣔", "⣕", "⣖", "⣗", "⢘", "⢙", "⢚", "⢛", "⢜", "⢝", "⢞", "⢟", "⣘", "⣙", "⣚", "⣛", "⣜", "⣝", "⣞", "⣟", "⢠", "⢡", "⢢", "⢣", "⢤", "⢥", "⢦", "⢧", "⣠", "⣡", "⣢", "⣣", "⣤", "⣥", "⣦", "⣧", "⢨", "⢩", "⢪", "⢫", "⢬", "⢭", "⢮", "⢯", "⣨", "⣩", "⣪", "⣫", "⣬", "⣭", "⣮", "⣯", "⢰", "⢱", "⢲", "⢳", "⢴", "⢵", "⢶", "⢷", "⣰", "⣱", "⣲", "⣳", "⣴", "⣵", "⣶", "⣷", "⢸", "⢹", "⢺", "⢻", "⢼", "⢽", "⢾", "⢿", "⣸", "⣹", "⣺", "⣻", "⣼", "⣽", "⣾", "⣿" ]
}, sand = {
    interval: 80,
    frames: [ "⠁", "⠂", "⠄", "⡀", "⡈", "⡐", "⡠", "⣀", "⣁", "⣂", "⣄", "⣌", "⣔", "⣤", "⣥", "⣦", "⣮", "⣶", "⣷", "⣿", "⡿", "⠿", "⢟", "⠟", "⡛", "⠛", "⠫", "⢋", "⠋", "⠍", "⡉", "⠉", "⠑", "⠡", "⢁" ]
}, line = {
    interval: 130,
    frames: [ "-", "\\", "|", "/" ]
}, line2 = {
    interval: 100,
    frames: [ "⠂", "-", "–", "—", "–", "-" ]
}, pipe = {
    interval: 100,
    frames: [ "┤", "┘", "┴", "└", "├", "┌", "┬", "┐" ]
}, simpleDots = {
    interval: 400,
    frames: [ ".  ", ".. ", "...", "   " ]
}, simpleDotsScrolling = {
    interval: 200,
    frames: [ ".  ", ".. ", "...", " ..", "  .", "   " ]
}, star = {
    interval: 70,
    frames: [ "✶", "✸", "✹", "✺", "✹", "✷" ]
}, star2 = {
    interval: 80,
    frames: [ "+", "x", "*" ]
}, flip = {
    interval: 70,
    frames: [ "_", "_", "_", "-", "`", "`", "'", "´", "-", "_", "_", "_" ]
}, hamburger = {
    interval: 100,
    frames: [ "☱", "☲", "☴" ]
}, growVertical = {
    interval: 120,
    frames: [ "▁", "▃", "▄", "▅", "▆", "▇", "▆", "▅", "▄", "▃" ]
}, growHorizontal = {
    interval: 120,
    frames: [ "▏", "▎", "▍", "▌", "▋", "▊", "▉", "▊", "▋", "▌", "▍", "▎" ]
}, balloon = {
    interval: 140,
    frames: [ " ", ".", "o", "O", "@", "*", " " ]
}, balloon2 = {
    interval: 120,
    frames: [ ".", "o", "O", "°", "O", "o", "." ]
}, noise = {
    interval: 100,
    frames: [ "▓", "▒", "░" ]
}, bounce = {
    interval: 120,
    frames: [ "⠁", "⠂", "⠄", "⠂" ]
}, boxBounce = {
    interval: 120,
    frames: [ "▖", "▘", "▝", "▗" ]
}, boxBounce2 = {
    interval: 100,
    frames: [ "▌", "▀", "▐", "▄" ]
}, triangle = {
    interval: 50,
    frames: [ "◢", "◣", "◤", "◥" ]
}, binary = {
    interval: 80,
    frames: [ "010010", "001100", "100101", "111010", "111101", "010111", "101011", "111000", "110011", "110101" ]
}, arc = {
    interval: 100,
    frames: [ "◜", "◠", "◝", "◞", "◡", "◟" ]
}, circle = {
    interval: 120,
    frames: [ "◡", "⊙", "◠" ]
}, squareCorners = {
    interval: 180,
    frames: [ "◰", "◳", "◲", "◱" ]
}, circleQuarters = {
    interval: 120,
    frames: [ "◴", "◷", "◶", "◵" ]
}, circleHalves = {
    interval: 50,
    frames: [ "◐", "◓", "◑", "◒" ]
}, squish = {
    interval: 100,
    frames: [ "╫", "╪" ]
}, toggle = {
    interval: 250,
    frames: [ "⊶", "⊷" ]
}, toggle2 = {
    interval: 80,
    frames: [ "▫", "▪" ]
}, toggle3 = {
    interval: 120,
    frames: [ "□", "■" ]
}, toggle4 = {
    interval: 100,
    frames: [ "■", "□", "▪", "▫" ]
}, toggle5 = {
    interval: 100,
    frames: [ "▮", "▯" ]
}, toggle6 = {
    interval: 300,
    frames: [ "ဝ", "၀" ]
}, toggle7 = {
    interval: 80,
    frames: [ "⦾", "⦿" ]
}, toggle8 = {
    interval: 100,
    frames: [ "◍", "◌" ]
}, toggle9 = {
    interval: 100,
    frames: [ "◉", "◎" ]
}, toggle10 = {
    interval: 100,
    frames: [ "㊂", "㊀", "㊁" ]
}, toggle11 = {
    interval: 50,
    frames: [ "⧇", "⧆" ]
}, toggle12 = {
    interval: 120,
    frames: [ "☗", "☖" ]
}, toggle13 = {
    interval: 80,
    frames: [ "=", "*", "-" ]
}, arrow = {
    interval: 100,
    frames: [ "←", "↖", "↑", "↗", "→", "↘", "↓", "↙" ]
}, arrow2 = {
    interval: 80,
    frames: [ "⬆️ ", "↗️ ", "➡️ ", "↘️ ", "⬇️ ", "↙️ ", "⬅️ ", "↖️ " ]
}, arrow3 = {
    interval: 120,
    frames: [ "▹▹▹▹▹", "▸▹▹▹▹", "▹▸▹▹▹", "▹▹▸▹▹", "▹▹▹▸▹", "▹▹▹▹▸" ]
}, bouncingBar = {
    interval: 80,
    frames: [ "[    ]", "[=   ]", "[==  ]", "[=== ]", "[====]", "[ ===]", "[  ==]", "[   =]", "[    ]", "[   =]", "[  ==]", "[ ===]", "[====]", "[=== ]", "[==  ]", "[=   ]" ]
}, bouncingBall = {
    interval: 80,
    frames: [ "( ●    )", "(  ●   )", "(   ●  )", "(    ● )", "(     ●)", "(    ● )", "(   ●  )", "(  ●   )", "( ●    )", "(●     )" ]
}, smiley = {
    interval: 200,
    frames: [ "😄 ", "😝 " ]
}, monkey = {
    interval: 300,
    frames: [ "🙈 ", "🙈 ", "🙉 ", "🙊 " ]
}, hearts = {
    interval: 100,
    frames: [ "💛 ", "💙 ", "💜 ", "💚 ", "❤️ " ]
}, clock = {
    interval: 100,
    frames: [ "🕛 ", "🕐 ", "🕑 ", "🕒 ", "🕓 ", "🕔 ", "🕕 ", "🕖 ", "🕗 ", "🕘 ", "🕙 ", "🕚 " ]
}, earth = {
    interval: 180,
    frames: [ "🌍 ", "🌎 ", "🌏 " ]
}, material = {
    interval: 17,
    frames: [ "█▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁", "██▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁", "███▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁", "████▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁", "██████▁▁▁▁▁▁▁▁▁▁▁▁▁▁", "██████▁▁▁▁▁▁▁▁▁▁▁▁▁▁", "███████▁▁▁▁▁▁▁▁▁▁▁▁▁", "████████▁▁▁▁▁▁▁▁▁▁▁▁", "█████████▁▁▁▁▁▁▁▁▁▁▁", "█████████▁▁▁▁▁▁▁▁▁▁▁", "██████████▁▁▁▁▁▁▁▁▁▁", "███████████▁▁▁▁▁▁▁▁▁", "█████████████▁▁▁▁▁▁▁", "██████████████▁▁▁▁▁▁", "██████████████▁▁▁▁▁▁", "▁██████████████▁▁▁▁▁", "▁██████████████▁▁▁▁▁", "▁██████████████▁▁▁▁▁", "▁▁██████████████▁▁▁▁", "▁▁▁██████████████▁▁▁", "▁▁▁▁█████████████▁▁▁", "▁▁▁▁██████████████▁▁", "▁▁▁▁██████████████▁▁", "▁▁▁▁▁██████████████▁", "▁▁▁▁▁██████████████▁", "▁▁▁▁▁██████████████▁", "▁▁▁▁▁▁██████████████", "▁▁▁▁▁▁██████████████", "▁▁▁▁▁▁▁█████████████", "▁▁▁▁▁▁▁█████████████", "▁▁▁▁▁▁▁▁████████████", "▁▁▁▁▁▁▁▁████████████", "▁▁▁▁▁▁▁▁▁███████████", "▁▁▁▁▁▁▁▁▁███████████", "▁▁▁▁▁▁▁▁▁▁██████████", "▁▁▁▁▁▁▁▁▁▁██████████", "▁▁▁▁▁▁▁▁▁▁▁▁████████", "▁▁▁▁▁▁▁▁▁▁▁▁▁███████", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁██████", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁█████", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁█████", "█▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁████", "██▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁███", "██▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁███", "███▁▁▁▁▁▁▁▁▁▁▁▁▁▁███", "████▁▁▁▁▁▁▁▁▁▁▁▁▁▁██", "█████▁▁▁▁▁▁▁▁▁▁▁▁▁▁█", "█████▁▁▁▁▁▁▁▁▁▁▁▁▁▁█", "██████▁▁▁▁▁▁▁▁▁▁▁▁▁█", "████████▁▁▁▁▁▁▁▁▁▁▁▁", "█████████▁▁▁▁▁▁▁▁▁▁▁", "█████████▁▁▁▁▁▁▁▁▁▁▁", "█████████▁▁▁▁▁▁▁▁▁▁▁", "█████████▁▁▁▁▁▁▁▁▁▁▁", "███████████▁▁▁▁▁▁▁▁▁", "████████████▁▁▁▁▁▁▁▁", "████████████▁▁▁▁▁▁▁▁", "██████████████▁▁▁▁▁▁", "██████████████▁▁▁▁▁▁", "▁██████████████▁▁▁▁▁", "▁██████████████▁▁▁▁▁", "▁▁▁█████████████▁▁▁▁", "▁▁▁▁▁████████████▁▁▁", "▁▁▁▁▁████████████▁▁▁", "▁▁▁▁▁▁███████████▁▁▁", "▁▁▁▁▁▁▁▁█████████▁▁▁", "▁▁▁▁▁▁▁▁█████████▁▁▁", "▁▁▁▁▁▁▁▁▁█████████▁▁", "▁▁▁▁▁▁▁▁▁█████████▁▁", "▁▁▁▁▁▁▁▁▁▁█████████▁", "▁▁▁▁▁▁▁▁▁▁▁████████▁", "▁▁▁▁▁▁▁▁▁▁▁████████▁", "▁▁▁▁▁▁▁▁▁▁▁▁███████▁", "▁▁▁▁▁▁▁▁▁▁▁▁███████▁", "▁▁▁▁▁▁▁▁▁▁▁▁▁███████", "▁▁▁▁▁▁▁▁▁▁▁▁▁███████", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁█████", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁████", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁████", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁████", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁███", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁███", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁██", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁██", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁██", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁█", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁█", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁█", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁", "▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁▁" ]
}, moon = {
    interval: 80,
    frames: [ "🌑 ", "🌒 ", "🌓 ", "🌔 ", "🌕 ", "🌖 ", "🌗 ", "🌘 " ]
}, runner = {
    interval: 140,
    frames: [ "🚶 ", "🏃 " ]
}, pong = {
    interval: 80,
    frames: [ "▐⠂       ▌", "▐⠈       ▌", "▐ ⠂      ▌", "▐ ⠠      ▌", "▐  ⡀     ▌", "▐  ⠠     ▌", "▐   ⠂    ▌", "▐   ⠈    ▌", "▐    ⠂   ▌", "▐    ⠠   ▌", "▐     ⡀  ▌", "▐     ⠠  ▌", "▐      ⠂ ▌", "▐      ⠈ ▌", "▐       ⠂▌", "▐       ⠠▌", "▐       ⡀▌", "▐      ⠠ ▌", "▐      ⠂ ▌", "▐     ⠈  ▌", "▐     ⠂  ▌", "▐    ⠠   ▌", "▐    ⡀   ▌", "▐   ⠠    ▌", "▐   ⠂    ▌", "▐  ⠈     ▌", "▐  ⠂     ▌", "▐ ⠠      ▌", "▐ ⡀      ▌", "▐⠠       ▌" ]
}, shark = {
    interval: 120,
    frames: [ "▐|\\____________▌", "▐_|\\___________▌", "▐__|\\__________▌", "▐___|\\_________▌", "▐____|\\________▌", "▐_____|\\_______▌", "▐______|\\______▌", "▐_______|\\_____▌", "▐________|\\____▌", "▐_________|\\___▌", "▐__________|\\__▌", "▐___________|\\_▌", "▐____________|\\▌", "▐____________/|▌", "▐___________/|_▌", "▐__________/|__▌", "▐_________/|___▌", "▐________/|____▌", "▐_______/|_____▌", "▐______/|______▌", "▐_____/|_______▌", "▐____/|________▌", "▐___/|_________▌", "▐__/|__________▌", "▐_/|___________▌", "▐/|____________▌" ]
}, dqpb = {
    interval: 100,
    frames: [ "d", "q", "p", "b" ]
}, weather = {
    interval: 100,
    frames: [ "☀️ ", "☀️ ", "☀️ ", "🌤 ", "⛅️ ", "🌥 ", "☁️ ", "🌧 ", "🌨 ", "🌧 ", "🌨 ", "🌧 ", "🌨 ", "⛈ ", "🌨 ", "🌧 ", "🌨 ", "☁️ ", "🌥 ", "⛅️ ", "🌤 ", "☀️ ", "☀️ " ]
}, christmas = {
    interval: 400,
    frames: [ "🌲", "🎄" ]
}, grenade = {
    interval: 80,
    frames: [ "،  ", "′  ", " ´ ", " ‾ ", "  ⸌", "  ⸊", "  |", "  ⁎", "  ⁕", " ෴ ", "  ⁓", "   ", "   ", "   " ]
}, point = {
    interval: 125,
    frames: [ "∙∙∙", "●∙∙", "∙●∙", "∙∙●", "∙∙∙" ]
}, layer = {
    interval: 150,
    frames: [ "-", "=", "≡" ]
}, betaWave = {
    interval: 80,
    frames: [ "ρββββββ", "βρβββββ", "ββρββββ", "βββρβββ", "ββββρββ", "βββββρβ", "ββββββρ" ]
}, fingerDance = {
    interval: 160,
    frames: [ "🤘 ", "🤟 ", "🖖 ", "✋ ", "🤚 ", "👆 " ]
}, fistBump = {
    interval: 80,
    frames: [ "🤜　　　　🤛 ", "🤜　　　　🤛 ", "🤜　　　　🤛 ", "　🤜　　🤛　 ", "　　🤜🤛　　 ", "　🤜✨🤛　　 ", "🤜　✨　🤛　 " ]
}, soccerHeader = {
    interval: 80,
    frames: [ " 🧑⚽️       🧑 ", "🧑  ⚽️      🧑 ", "🧑   ⚽️     🧑 ", "🧑    ⚽️    🧑 ", "🧑     ⚽️   🧑 ", "🧑      ⚽️  🧑 ", "🧑       ⚽️🧑  ", "🧑      ⚽️  🧑 ", "🧑     ⚽️   🧑 ", "🧑    ⚽️    🧑 ", "🧑   ⚽️     🧑 ", "🧑  ⚽️      🧑 " ]
}, mindblown = {
    interval: 160,
    frames: [ "😐 ", "😐 ", "😮 ", "😮 ", "😦 ", "😦 ", "😧 ", "😧 ", "🤯 ", "💥 ", "✨ ", "　 ", "　 ", "　 " ]
}, speaker = {
    interval: 160,
    frames: [ "🔈 ", "🔉 ", "🔊 ", "🔉 " ]
}, orangePulse = {
    interval: 100,
    frames: [ "🔸 ", "🔶 ", "🟠 ", "🟠 ", "🔶 " ]
}, bluePulse = {
    interval: 100,
    frames: [ "🔹 ", "🔷 ", "🔵 ", "🔵 ", "🔷 " ]
}, orangeBluePulse = {
    interval: 100,
    frames: [ "🔸 ", "🔶 ", "🟠 ", "🟠 ", "🔶 ", "🔹 ", "🔷 ", "🔵 ", "🔵 ", "🔷 " ]
}, timeTravel = {
    interval: 100,
    frames: [ "🕛 ", "🕚 ", "🕙 ", "🕘 ", "🕗 ", "🕖 ", "🕕 ", "🕔 ", "🕓 ", "🕒 ", "🕑 ", "🕐 " ]
}, aesthetic = {
    interval: 80,
    frames: [ "▰▱▱▱▱▱▱", "▰▰▱▱▱▱▱", "▰▰▰▱▱▱▱", "▰▰▰▰▱▱▱", "▰▰▰▰▰▱▱", "▰▰▰▰▰▰▱", "▰▰▰▰▰▰▰", "▰▱▱▱▱▱▱" ]
}, dwarfFortress = {
    interval: 80,
    frames: [ " ██████£££  ", "☺██████£££  ", "☺██████£££  ", "☺▓█████£££  ", "☺▓█████£££  ", "☺▒█████£££  ", "☺▒█████£££  ", "☺░█████£££  ", "☺░█████£££  ", "☺ █████£££  ", " ☺█████£££  ", " ☺█████£££  ", " ☺▓████£££  ", " ☺▓████£££  ", " ☺▒████£££  ", " ☺▒████£££  ", " ☺░████£££  ", " ☺░████£££  ", " ☺ ████£££  ", "  ☺████£££  ", "  ☺████£££  ", "  ☺▓███£££  ", "  ☺▓███£££  ", "  ☺▒███£££  ", "  ☺▒███£££  ", "  ☺░███£££  ", "  ☺░███£££  ", "  ☺ ███£££  ", "   ☺███£££  ", "   ☺███£££  ", "   ☺▓██£££  ", "   ☺▓██£££  ", "   ☺▒██£££  ", "   ☺▒██£££  ", "   ☺░██£££  ", "   ☺░██£££  ", "   ☺ ██£££  ", "    ☺██£££  ", "    ☺██£££  ", "    ☺▓█£££  ", "    ☺▓█£££  ", "    ☺▒█£££  ", "    ☺▒█£££  ", "    ☺░█£££  ", "    ☺░█£££  ", "    ☺ █£££  ", "     ☺█£££  ", "     ☺█£££  ", "     ☺▓£££  ", "     ☺▓£££  ", "     ☺▒£££  ", "     ☺▒£££  ", "     ☺░£££  ", "     ☺░£££  ", "     ☺ £££  ", "      ☺£££  ", "      ☺£££  ", "      ☺▓££  ", "      ☺▓££  ", "      ☺▒££  ", "      ☺▒££  ", "      ☺░££  ", "      ☺░££  ", "      ☺ ££  ", "       ☺££  ", "       ☺££  ", "       ☺▓£  ", "       ☺▓£  ", "       ☺▒£  ", "       ☺▒£  ", "       ☺░£  ", "       ☺░£  ", "       ☺ £  ", "        ☺£  ", "        ☺£  ", "        ☺▓  ", "        ☺▓  ", "        ☺▒  ", "        ☺▒  ", "        ☺░  ", "        ☺░  ", "        ☺   ", "        ☺  &", "        ☺ ☼&", "       ☺ ☼ &", "       ☺☼  &", "      ☺☼  & ", "      ‼   & ", "     ☺   &  ", "    ‼    &  ", "   ☺    &   ", "  ‼     &   ", " ☺     &    ", "‼      &    ", "      &     ", "      &     ", "     &   ░  ", "     &   ▒  ", "    &    ▓  ", "    &    £  ", "   &    ░£  ", "   &    ▒£  ", "  &     ▓£  ", "  &     ££  ", " &     ░££  ", " &     ▒££  ", "&      ▓££  ", "&      £££  ", "      ░£££  ", "      ▒£££  ", "      ▓£££  ", "      █£££  ", "     ░█£££  ", "     ▒█£££  ", "     ▓█£££  ", "     ██£££  ", "    ░██£££  ", "    ▒██£££  ", "    ▓██£££  ", "    ███£££  ", "   ░███£££  ", "   ▒███£££  ", "   ▓███£££  ", "   ████£££  ", "  ░████£££  ", "  ▒████£££  ", "  ▓████£££  ", "  █████£££  ", " ░█████£££  ", " ▒█████£££  ", " ▓█████£££  ", " ██████£££  ", " ██████£££  " ]
}, require$$0 = {
    dots: dots,
    dots2: dots2,
    dots3: dots3,
    dots4: dots4,
    dots5: dots5,
    dots6: dots6,
    dots7: dots7,
    dots8: dots8,
    dots9: dots9,
    dots10: dots10,
    dots11: dots11,
    dots12: dots12,
    dots13: dots13,
    dots8Bit: dots8Bit,
    sand: sand,
    line: line,
    line2: line2,
    pipe: pipe,
    simpleDots: simpleDots,
    simpleDotsScrolling: simpleDotsScrolling,
    star: star,
    star2: star2,
    flip: flip,
    hamburger: hamburger,
    growVertical: growVertical,
    growHorizontal: growHorizontal,
    balloon: balloon,
    balloon2: balloon2,
    noise: noise,
    bounce: bounce,
    boxBounce: boxBounce,
    boxBounce2: boxBounce2,
    triangle: triangle,
    binary: binary,
    arc: arc,
    circle: circle,
    squareCorners: squareCorners,
    circleQuarters: circleQuarters,
    circleHalves: circleHalves,
    squish: squish,
    toggle: toggle,
    toggle2: toggle2,
    toggle3: toggle3,
    toggle4: toggle4,
    toggle5: toggle5,
    toggle6: toggle6,
    toggle7: toggle7,
    toggle8: toggle8,
    toggle9: toggle9,
    toggle10: toggle10,
    toggle11: toggle11,
    toggle12: toggle12,
    toggle13: toggle13,
    arrow: arrow,
    arrow2: arrow2,
    arrow3: arrow3,
    bouncingBar: bouncingBar,
    bouncingBall: bouncingBall,
    smiley: smiley,
    monkey: monkey,
    hearts: hearts,
    clock: clock,
    earth: earth,
    material: material,
    moon: moon,
    runner: runner,
    pong: pong,
    shark: shark,
    dqpb: dqpb,
    weather: weather,
    christmas: christmas,
    grenade: grenade,
    point: point,
    layer: layer,
    betaWave: betaWave,
    fingerDance: fingerDance,
    fistBump: fistBump,
    soccerHeader: soccerHeader,
    mindblown: mindblown,
    speaker: speaker,
    orangePulse: orangePulse,
    bluePulse: bluePulse,
    orangeBluePulse: orangeBluePulse,
    timeTravel: timeTravel,
    aesthetic: aesthetic,
    dwarfFortress: dwarfFortress
}, cliSpinners$1, hasRequiredCliSpinners;

function requireCliSpinners() {
    if (hasRequiredCliSpinners) return cliSpinners$1;
    hasRequiredCliSpinners = 1;
    const spinners = Object.assign({}, require$$0), spinnersList = Object.keys(spinners);
    return Object.defineProperty(spinners, "random", {
        get() {
            const randomIndex = Math.floor(Math.random() * spinnersList.length), spinnerName = spinnersList[randomIndex];
            return spinners[spinnerName];
        }
    }), cliSpinners$1 = spinners;
}

var cliSpinnersExports = requireCliSpinners(), cliSpinners = getDefaultExportFromCjs(cliSpinnersExports);

function isUnicodeSupported$2() {
    return "win32" !== process$2.platform ? "linux" !== process$2.env.TERM : Boolean(process$2.env.CI) || Boolean(process$2.env.WT_SESSION) || Boolean(process$2.env.TERMINUS_SUBLIME) || "{cmd::Cmder}" === process$2.env.ConEmuTask || "Terminus-Sublime" === process$2.env.TERM_PROGRAM || "vscode" === process$2.env.TERM_PROGRAM || "xterm-256color" === process$2.env.TERM || "alacritty" === process$2.env.TERM || "JetBrains-JediTerm" === process$2.env.TERMINAL_EMULATOR;
}

const main$1 = {
    info: chalk.blue("ℹ"),
    success: chalk.green("✔"),
    warning: chalk.yellow("⚠"),
    error: chalk.red("✖")
}, fallback = {
    info: chalk.blue("i"),
    success: chalk.green("√"),
    warning: chalk.yellow("‼"),
    error: chalk.red("×")
}, logSymbols = isUnicodeSupported$2() ? main$1 : fallback;

function ansiRegex$1({onlyFirst: onlyFirst = !1} = {}) {
    const pattern = [ "[\\u001B\\u009B][[\\]()#;?]*(?:(?:(?:(?:;[-a-zA-Z\\d\\/#&.:=?%@~_]+)*|[a-zA-Z\\d]+(?:;[-a-zA-Z\\d\\/#&.:=?%@~_]*)*)?(?:\\u0007|\\u001B\\u005C|\\u009C))", "(?:(?:\\d{1,4}(?:;\\d{0,4})*)?[\\dA-PR-TZcf-nq-uy=><~]))" ].join("|");
    return new RegExp(pattern, onlyFirst ? void 0 : "g");
}

const regex = ansiRegex$1();

function stripAnsi$2(string) {
    if ("string" != typeof string) throw new TypeError(`Expected a \`string\`, got \`${typeof string}\``);
    return string.replace(regex, "");
}

function isAmbiguous(x) {
    return 161 === x || 164 === x || 167 === x || 168 === x || 170 === x || 173 === x || 174 === x || x >= 176 && x <= 180 || x >= 182 && x <= 186 || x >= 188 && x <= 191 || 198 === x || 208 === x || 215 === x || 216 === x || x >= 222 && x <= 225 || 230 === x || x >= 232 && x <= 234 || 236 === x || 237 === x || 240 === x || 242 === x || 243 === x || x >= 247 && x <= 250 || 252 === x || 254 === x || 257 === x || 273 === x || 275 === x || 283 === x || 294 === x || 295 === x || 299 === x || x >= 305 && x <= 307 || 312 === x || x >= 319 && x <= 322 || 324 === x || x >= 328 && x <= 331 || 333 === x || 338 === x || 339 === x || 358 === x || 359 === x || 363 === x || 462 === x || 464 === x || 466 === x || 468 === x || 470 === x || 472 === x || 474 === x || 476 === x || 593 === x || 609 === x || 708 === x || 711 === x || x >= 713 && x <= 715 || 717 === x || 720 === x || x >= 728 && x <= 731 || 733 === x || 735 === x || x >= 768 && x <= 879 || x >= 913 && x <= 929 || x >= 931 && x <= 937 || x >= 945 && x <= 961 || x >= 963 && x <= 969 || 1025 === x || x >= 1040 && x <= 1103 || 1105 === x || 8208 === x || x >= 8211 && x <= 8214 || 8216 === x || 8217 === x || 8220 === x || 8221 === x || x >= 8224 && x <= 8226 || x >= 8228 && x <= 8231 || 8240 === x || 8242 === x || 8243 === x || 8245 === x || 8251 === x || 8254 === x || 8308 === x || 8319 === x || x >= 8321 && x <= 8324 || 8364 === x || 8451 === x || 8453 === x || 8457 === x || 8467 === x || 8470 === x || 8481 === x || 8482 === x || 8486 === x || 8491 === x || 8531 === x || 8532 === x || x >= 8539 && x <= 8542 || x >= 8544 && x <= 8555 || x >= 8560 && x <= 8569 || 8585 === x || x >= 8592 && x <= 8601 || 8632 === x || 8633 === x || 8658 === x || 8660 === x || 8679 === x || 8704 === x || 8706 === x || 8707 === x || 8711 === x || 8712 === x || 8715 === x || 8719 === x || 8721 === x || 8725 === x || 8730 === x || x >= 8733 && x <= 8736 || 8739 === x || 8741 === x || x >= 8743 && x <= 8748 || 8750 === x || x >= 8756 && x <= 8759 || 8764 === x || 8765 === x || 8776 === x || 8780 === x || 8786 === x || 8800 === x || 8801 === x || x >= 8804 && x <= 8807 || 8810 === x || 8811 === x || 8814 === x || 8815 === x || 8834 === x || 8835 === x || 8838 === x || 8839 === x || 8853 === x || 8857 === x || 8869 === x || 8895 === x || 8978 === x || x >= 9312 && x <= 9449 || x >= 9451 && x <= 9547 || x >= 9552 && x <= 9587 || x >= 9600 && x <= 9615 || x >= 9618 && x <= 9621 || 9632 === x || 9633 === x || x >= 9635 && x <= 9641 || 9650 === x || 9651 === x || 9654 === x || 9655 === x || 9660 === x || 9661 === x || 9664 === x || 9665 === x || x >= 9670 && x <= 9672 || 9675 === x || x >= 9678 && x <= 9681 || x >= 9698 && x <= 9701 || 9711 === x || 9733 === x || 9734 === x || 9737 === x || 9742 === x || 9743 === x || 9756 === x || 9758 === x || 9792 === x || 9794 === x || 9824 === x || 9825 === x || x >= 9827 && x <= 9829 || x >= 9831 && x <= 9834 || 9836 === x || 9837 === x || 9839 === x || 9886 === x || 9887 === x || 9919 === x || x >= 9926 && x <= 9933 || x >= 9935 && x <= 9939 || x >= 9941 && x <= 9953 || 9955 === x || 9960 === x || 9961 === x || x >= 9963 && x <= 9969 || 9972 === x || x >= 9974 && x <= 9977 || 9979 === x || 9980 === x || 9982 === x || 9983 === x || 10045 === x || x >= 10102 && x <= 10111 || x >= 11094 && x <= 11097 || x >= 12872 && x <= 12879 || x >= 57344 && x <= 63743 || x >= 65024 && x <= 65039 || 65533 === x || x >= 127232 && x <= 127242 || x >= 127248 && x <= 127277 || x >= 127280 && x <= 127337 || x >= 127344 && x <= 127373 || 127375 === x || 127376 === x || x >= 127387 && x <= 127404 || x >= 917760 && x <= 917999 || x >= 983040 && x <= 1048573 || x >= 1048576 && x <= 1114109;
}

function isFullWidth(x) {
    return 12288 === x || x >= 65281 && x <= 65376 || x >= 65504 && x <= 65510;
}

function isWide(x) {
    return x >= 4352 && x <= 4447 || 8986 === x || 8987 === x || 9001 === x || 9002 === x || x >= 9193 && x <= 9196 || 9200 === x || 9203 === x || 9725 === x || 9726 === x || 9748 === x || 9749 === x || x >= 9800 && x <= 9811 || 9855 === x || 9875 === x || 9889 === x || 9898 === x || 9899 === x || 9917 === x || 9918 === x || 9924 === x || 9925 === x || 9934 === x || 9940 === x || 9962 === x || 9970 === x || 9971 === x || 9973 === x || 9978 === x || 9981 === x || 9989 === x || 9994 === x || 9995 === x || 10024 === x || 10060 === x || 10062 === x || x >= 10067 && x <= 10069 || 10071 === x || x >= 10133 && x <= 10135 || 10160 === x || 10175 === x || 11035 === x || 11036 === x || 11088 === x || 11093 === x || x >= 11904 && x <= 11929 || x >= 11931 && x <= 12019 || x >= 12032 && x <= 12245 || x >= 12272 && x <= 12287 || x >= 12289 && x <= 12350 || x >= 12353 && x <= 12438 || x >= 12441 && x <= 12543 || x >= 12549 && x <= 12591 || x >= 12593 && x <= 12686 || x >= 12688 && x <= 12771 || x >= 12783 && x <= 12830 || x >= 12832 && x <= 12871 || x >= 12880 && x <= 19903 || x >= 19968 && x <= 42124 || x >= 42128 && x <= 42182 || x >= 43360 && x <= 43388 || x >= 44032 && x <= 55203 || x >= 63744 && x <= 64255 || x >= 65040 && x <= 65049 || x >= 65072 && x <= 65106 || x >= 65108 && x <= 65126 || x >= 65128 && x <= 65131 || x >= 94176 && x <= 94180 || 94192 === x || 94193 === x || x >= 94208 && x <= 100343 || x >= 100352 && x <= 101589 || x >= 101632 && x <= 101640 || x >= 110576 && x <= 110579 || x >= 110581 && x <= 110587 || 110589 === x || 110590 === x || x >= 110592 && x <= 110882 || 110898 === x || x >= 110928 && x <= 110930 || 110933 === x || x >= 110948 && x <= 110951 || x >= 110960 && x <= 111355 || 126980 === x || 127183 === x || 127374 === x || x >= 127377 && x <= 127386 || x >= 127488 && x <= 127490 || x >= 127504 && x <= 127547 || x >= 127552 && x <= 127560 || 127568 === x || 127569 === x || x >= 127584 && x <= 127589 || x >= 127744 && x <= 127776 || x >= 127789 && x <= 127797 || x >= 127799 && x <= 127868 || x >= 127870 && x <= 127891 || x >= 127904 && x <= 127946 || x >= 127951 && x <= 127955 || x >= 127968 && x <= 127984 || 127988 === x || x >= 127992 && x <= 128062 || 128064 === x || x >= 128066 && x <= 128252 || x >= 128255 && x <= 128317 || x >= 128331 && x <= 128334 || x >= 128336 && x <= 128359 || 128378 === x || 128405 === x || 128406 === x || 128420 === x || x >= 128507 && x <= 128591 || x >= 128640 && x <= 128709 || 128716 === x || x >= 128720 && x <= 128722 || x >= 128725 && x <= 128727 || x >= 128732 && x <= 128735 || 128747 === x || 128748 === x || x >= 128756 && x <= 128764 || x >= 128992 && x <= 129003 || 129008 === x || x >= 129292 && x <= 129338 || x >= 129340 && x <= 129349 || x >= 129351 && x <= 129535 || x >= 129648 && x <= 129660 || x >= 129664 && x <= 129672 || x >= 129680 && x <= 129725 || x >= 129727 && x <= 129733 || x >= 129742 && x <= 129755 || x >= 129760 && x <= 129768 || x >= 129776 && x <= 129784 || x >= 131072 && x <= 196605 || x >= 196608 && x <= 262141;
}

function validate(codePoint) {
    if (!Number.isSafeInteger(codePoint)) throw new TypeError(`Expected a code point, got \`${typeof codePoint}\`.`);
}

function eastAsianWidth(codePoint, {ambiguousAsWide: ambiguousAsWide = !1} = {}) {
    return validate(codePoint), isFullWidth(codePoint) || isWide(codePoint) || ambiguousAsWide && isAmbiguous(codePoint) ? 2 : 1;
}

var emojiRegex$1 = () => /[#*0-9]\uFE0F?\u20E3|[\xA9\xAE\u203C\u2049\u2122\u2139\u2194-\u2199\u21A9\u21AA\u231A\u231B\u2328\u23CF\u23ED-\u23EF\u23F1\u23F2\u23F8-\u23FA\u24C2\u25AA\u25AB\u25B6\u25C0\u25FB\u25FC\u25FE\u2600-\u2604\u260E\u2611\u2614\u2615\u2618\u2620\u2622\u2623\u2626\u262A\u262E\u262F\u2638-\u263A\u2640\u2642\u2648-\u2653\u265F\u2660\u2663\u2665\u2666\u2668\u267B\u267E\u267F\u2692\u2694-\u2697\u2699\u269B\u269C\u26A0\u26A7\u26AA\u26B0\u26B1\u26BD\u26BE\u26C4\u26C8\u26CF\u26D1\u26E9\u26F0-\u26F5\u26F7\u26F8\u26FA\u2702\u2708\u2709\u270F\u2712\u2714\u2716\u271D\u2721\u2733\u2734\u2744\u2747\u2757\u2763\u27A1\u2934\u2935\u2B05-\u2B07\u2B1B\u2B1C\u2B55\u3030\u303D\u3297\u3299]\uFE0F?|[\u261D\u270C\u270D](?:\uD83C[\uDFFB-\uDFFF]|\uFE0F)?|[\u270A\u270B](?:\uD83C[\uDFFB-\uDFFF])?|[\u23E9-\u23EC\u23F0\u23F3\u25FD\u2693\u26A1\u26AB\u26C5\u26CE\u26D4\u26EA\u26FD\u2705\u2728\u274C\u274E\u2753-\u2755\u2795-\u2797\u27B0\u27BF\u2B50]|\u26D3\uFE0F?(?:\u200D\uD83D\uDCA5)?|\u26F9(?:\uD83C[\uDFFB-\uDFFF]|\uFE0F)?(?:\u200D[\u2640\u2642]\uFE0F?)?|\u2764\uFE0F?(?:\u200D(?:\uD83D\uDD25|\uD83E\uDE79))?|\uD83C(?:[\uDC04\uDD70\uDD71\uDD7E\uDD7F\uDE02\uDE37\uDF21\uDF24-\uDF2C\uDF36\uDF7D\uDF96\uDF97\uDF99-\uDF9B\uDF9E\uDF9F\uDFCD\uDFCE\uDFD4-\uDFDF\uDFF5\uDFF7]\uFE0F?|[\uDF85\uDFC2\uDFC7](?:\uD83C[\uDFFB-\uDFFF])?|[\uDFC4\uDFCA](?:\uD83C[\uDFFB-\uDFFF])?(?:\u200D[\u2640\u2642]\uFE0F?)?|[\uDFCB\uDFCC](?:\uD83C[\uDFFB-\uDFFF]|\uFE0F)?(?:\u200D[\u2640\u2642]\uFE0F?)?|[\uDCCF\uDD8E\uDD91-\uDD9A\uDE01\uDE1A\uDE2F\uDE32-\uDE36\uDE38-\uDE3A\uDE50\uDE51\uDF00-\uDF20\uDF2D-\uDF35\uDF37-\uDF43\uDF45-\uDF4A\uDF4C-\uDF7C\uDF7E-\uDF84\uDF86-\uDF93\uDFA0-\uDFC1\uDFC5\uDFC6\uDFC8\uDFC9\uDFCF-\uDFD3\uDFE0-\uDFF0\uDFF8-\uDFFF]|\uDDE6\uD83C[\uDDE8-\uDDEC\uDDEE\uDDF1\uDDF2\uDDF4\uDDF6-\uDDFA\uDDFC\uDDFD\uDDFF]|\uDDE7\uD83C[\uDDE6\uDDE7\uDDE9-\uDDEF\uDDF1-\uDDF4\uDDF6-\uDDF9\uDDFB\uDDFC\uDDFE\uDDFF]|\uDDE8\uD83C[\uDDE6\uDDE8\uDDE9\uDDEB-\uDDEE\uDDF0-\uDDF7\uDDFA-\uDDFF]|\uDDE9\uD83C[\uDDEA\uDDEC\uDDEF\uDDF0\uDDF2\uDDF4\uDDFF]|\uDDEA\uD83C[\uDDE6\uDDE8\uDDEA\uDDEC\uDDED\uDDF7-\uDDFA]|\uDDEB\uD83C[\uDDEE-\uDDF0\uDDF2\uDDF4\uDDF7]|\uDDEC\uD83C[\uDDE6\uDDE7\uDDE9-\uDDEE\uDDF1-\uDDF3\uDDF5-\uDDFA\uDDFC\uDDFE]|\uDDED\uD83C[\uDDF0\uDDF2\uDDF3\uDDF7\uDDF9\uDDFA]|\uDDEE\uD83C[\uDDE8-\uDDEA\uDDF1-\uDDF4\uDDF6-\uDDF9]|\uDDEF\uD83C[\uDDEA\uDDF2\uDDF4\uDDF5]|\uDDF0\uD83C[\uDDEA\uDDEC-\uDDEE\uDDF2\uDDF3\uDDF5\uDDF7\uDDFC\uDDFE\uDDFF]|\uDDF1\uD83C[\uDDE6-\uDDE8\uDDEE\uDDF0\uDDF7-\uDDFB\uDDFE]|\uDDF2\uD83C[\uDDE6\uDDE8-\uDDED\uDDF0-\uDDFF]|\uDDF3\uD83C[\uDDE6\uDDE8\uDDEA-\uDDEC\uDDEE\uDDF1\uDDF4\uDDF5\uDDF7\uDDFA\uDDFF]|\uDDF4\uD83C\uDDF2|\uDDF5\uD83C[\uDDE6\uDDEA-\uDDED\uDDF0-\uDDF3\uDDF7-\uDDF9\uDDFC\uDDFE]|\uDDF6\uD83C\uDDE6|\uDDF7\uD83C[\uDDEA\uDDF4\uDDF8\uDDFA\uDDFC]|\uDDF8\uD83C[\uDDE6-\uDDEA\uDDEC-\uDDF4\uDDF7-\uDDF9\uDDFB\uDDFD-\uDDFF]|\uDDF9\uD83C[\uDDE6\uDDE8\uDDE9\uDDEB-\uDDED\uDDEF-\uDDF4\uDDF7\uDDF9\uDDFB\uDDFC\uDDFF]|\uDDFA\uD83C[\uDDE6\uDDEC\uDDF2\uDDF3\uDDF8\uDDFE\uDDFF]|\uDDFB\uD83C[\uDDE6\uDDE8\uDDEA\uDDEC\uDDEE\uDDF3\uDDFA]|\uDDFC\uD83C[\uDDEB\uDDF8]|\uDDFD\uD83C\uDDF0|\uDDFE\uD83C[\uDDEA\uDDF9]|\uDDFF\uD83C[\uDDE6\uDDF2\uDDFC]|\uDF44(?:\u200D\uD83D\uDFEB)?|\uDF4B(?:\u200D\uD83D\uDFE9)?|\uDFC3(?:\uD83C[\uDFFB-\uDFFF])?(?:\u200D(?:[\u2640\u2642]\uFE0F?(?:\u200D\u27A1\uFE0F?)?|\u27A1\uFE0F?))?|\uDFF3\uFE0F?(?:\u200D(?:\u26A7\uFE0F?|\uD83C\uDF08))?|\uDFF4(?:\u200D\u2620\uFE0F?|\uDB40\uDC67\uDB40\uDC62\uDB40(?:\uDC65\uDB40\uDC6E\uDB40\uDC67|\uDC73\uDB40\uDC63\uDB40\uDC74|\uDC77\uDB40\uDC6C\uDB40\uDC73)\uDB40\uDC7F)?)|\uD83D(?:[\uDC3F\uDCFD\uDD49\uDD4A\uDD6F\uDD70\uDD73\uDD76-\uDD79\uDD87\uDD8A-\uDD8D\uDDA5\uDDA8\uDDB1\uDDB2\uDDBC\uDDC2-\uDDC4\uDDD1-\uDDD3\uDDDC-\uDDDE\uDDE1\uDDE3\uDDE8\uDDEF\uDDF3\uDDFA\uDECB\uDECD-\uDECF\uDEE0-\uDEE5\uDEE9\uDEF0\uDEF3]\uFE0F?|[\uDC42\uDC43\uDC46-\uDC50\uDC66\uDC67\uDC6B-\uDC6D\uDC72\uDC74-\uDC76\uDC78\uDC7C\uDC83\uDC85\uDC8F\uDC91\uDCAA\uDD7A\uDD95\uDD96\uDE4C\uDE4F\uDEC0\uDECC](?:\uD83C[\uDFFB-\uDFFF])?|[\uDC6E\uDC70\uDC71\uDC73\uDC77\uDC81\uDC82\uDC86\uDC87\uDE45-\uDE47\uDE4B\uDE4D\uDE4E\uDEA3\uDEB4\uDEB5](?:\uD83C[\uDFFB-\uDFFF])?(?:\u200D[\u2640\u2642]\uFE0F?)?|[\uDD74\uDD90](?:\uD83C[\uDFFB-\uDFFF]|\uFE0F)?|[\uDC00-\uDC07\uDC09-\uDC14\uDC16-\uDC25\uDC27-\uDC3A\uDC3C-\uDC3E\uDC40\uDC44\uDC45\uDC51-\uDC65\uDC6A\uDC79-\uDC7B\uDC7D-\uDC80\uDC84\uDC88-\uDC8E\uDC90\uDC92-\uDCA9\uDCAB-\uDCFC\uDCFF-\uDD3D\uDD4B-\uDD4E\uDD50-\uDD67\uDDA4\uDDFB-\uDE2D\uDE2F-\uDE34\uDE37-\uDE41\uDE43\uDE44\uDE48-\uDE4A\uDE80-\uDEA2\uDEA4-\uDEB3\uDEB7-\uDEBF\uDEC1-\uDEC5\uDED0-\uDED2\uDED5-\uDED7\uDEDC-\uDEDF\uDEEB\uDEEC\uDEF4-\uDEFC\uDFE0-\uDFEB\uDFF0]|\uDC08(?:\u200D\u2B1B)?|\uDC15(?:\u200D\uD83E\uDDBA)?|\uDC26(?:\u200D(?:\u2B1B|\uD83D\uDD25))?|\uDC3B(?:\u200D\u2744\uFE0F?)?|\uDC41\uFE0F?(?:\u200D\uD83D\uDDE8\uFE0F?)?|\uDC68(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D\uD83D(?:\uDC8B\u200D\uD83D)?\uDC68|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D(?:[\uDC68\uDC69]\u200D\uD83D(?:\uDC66(?:\u200D\uD83D\uDC66)?|\uDC67(?:\u200D\uD83D[\uDC66\uDC67])?)|[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uDC66(?:\u200D\uD83D\uDC66)?|\uDC67(?:\u200D\uD83D[\uDC66\uDC67])?)|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]))|\uD83C(?:\uDFFB(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D\uD83D(?:\uDC8B\u200D\uD83D)?\uDC68\uD83C[\uDFFB-\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83D\uDC68\uD83C[\uDFFC-\uDFFF])))?|\uDFFC(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D\uD83D(?:\uDC8B\u200D\uD83D)?\uDC68\uD83C[\uDFFB-\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83D\uDC68\uD83C[\uDFFB\uDFFD-\uDFFF])))?|\uDFFD(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D\uD83D(?:\uDC8B\u200D\uD83D)?\uDC68\uD83C[\uDFFB-\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83D\uDC68\uD83C[\uDFFB\uDFFC\uDFFE\uDFFF])))?|\uDFFE(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D\uD83D(?:\uDC8B\u200D\uD83D)?\uDC68\uD83C[\uDFFB-\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83D\uDC68\uD83C[\uDFFB-\uDFFD\uDFFF])))?|\uDFFF(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D\uD83D(?:\uDC8B\u200D\uD83D)?\uDC68\uD83C[\uDFFB-\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83D\uDC68\uD83C[\uDFFB-\uDFFE])))?))?|\uDC69(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D\uD83D(?:\uDC8B\u200D\uD83D)?[\uDC68\uDC69]|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D(?:[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uDC66(?:\u200D\uD83D\uDC66)?|\uDC67(?:\u200D\uD83D[\uDC66\uDC67])?|\uDC69\u200D\uD83D(?:\uDC66(?:\u200D\uD83D\uDC66)?|\uDC67(?:\u200D\uD83D[\uDC66\uDC67])?))|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]))|\uD83C(?:\uDFFB(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D\uD83D(?:[\uDC68\uDC69]|\uDC8B\u200D\uD83D[\uDC68\uDC69])\uD83C[\uDFFB-\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83D[\uDC68\uDC69]\uD83C[\uDFFC-\uDFFF])))?|\uDFFC(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D\uD83D(?:[\uDC68\uDC69]|\uDC8B\u200D\uD83D[\uDC68\uDC69])\uD83C[\uDFFB-\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83D[\uDC68\uDC69]\uD83C[\uDFFB\uDFFD-\uDFFF])))?|\uDFFD(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D\uD83D(?:[\uDC68\uDC69]|\uDC8B\u200D\uD83D[\uDC68\uDC69])\uD83C[\uDFFB-\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83D[\uDC68\uDC69]\uD83C[\uDFFB\uDFFC\uDFFE\uDFFF])))?|\uDFFE(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D\uD83D(?:[\uDC68\uDC69]|\uDC8B\u200D\uD83D[\uDC68\uDC69])\uD83C[\uDFFB-\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83D[\uDC68\uDC69]\uD83C[\uDFFB-\uDFFD\uDFFF])))?|\uDFFF(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D\uD83D(?:[\uDC68\uDC69]|\uDC8B\u200D\uD83D[\uDC68\uDC69])\uD83C[\uDFFB-\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83D[\uDC68\uDC69]\uD83C[\uDFFB-\uDFFE])))?))?|\uDC6F(?:\u200D[\u2640\u2642]\uFE0F?)?|\uDD75(?:\uD83C[\uDFFB-\uDFFF]|\uFE0F)?(?:\u200D[\u2640\u2642]\uFE0F?)?|\uDE2E(?:\u200D\uD83D\uDCA8)?|\uDE35(?:\u200D\uD83D\uDCAB)?|\uDE36(?:\u200D\uD83C\uDF2B\uFE0F?)?|\uDE42(?:\u200D[\u2194\u2195]\uFE0F?)?|\uDEB6(?:\uD83C[\uDFFB-\uDFFF])?(?:\u200D(?:[\u2640\u2642]\uFE0F?(?:\u200D\u27A1\uFE0F?)?|\u27A1\uFE0F?))?)|\uD83E(?:[\uDD0C\uDD0F\uDD18-\uDD1F\uDD30-\uDD34\uDD36\uDD77\uDDB5\uDDB6\uDDBB\uDDD2\uDDD3\uDDD5\uDEC3-\uDEC5\uDEF0\uDEF2-\uDEF8](?:\uD83C[\uDFFB-\uDFFF])?|[\uDD26\uDD35\uDD37-\uDD39\uDD3D\uDD3E\uDDB8\uDDB9\uDDCD\uDDCF\uDDD4\uDDD6-\uDDDD](?:\uD83C[\uDFFB-\uDFFF])?(?:\u200D[\u2640\u2642]\uFE0F?)?|[\uDDDE\uDDDF](?:\u200D[\u2640\u2642]\uFE0F?)?|[\uDD0D\uDD0E\uDD10-\uDD17\uDD20-\uDD25\uDD27-\uDD2F\uDD3A\uDD3F-\uDD45\uDD47-\uDD76\uDD78-\uDDB4\uDDB7\uDDBA\uDDBC-\uDDCC\uDDD0\uDDE0-\uDDFF\uDE70-\uDE7C\uDE80-\uDE89\uDE8F-\uDEC2\uDEC6\uDECE-\uDEDC\uDEDF-\uDEE9]|\uDD3C(?:\u200D[\u2640\u2642]\uFE0F?|\uD83C[\uDFFB-\uDFFF])?|\uDDCE(?:\uD83C[\uDFFB-\uDFFF])?(?:\u200D(?:[\u2640\u2642]\uFE0F?(?:\u200D\u27A1\uFE0F?)?|\u27A1\uFE0F?))?|\uDDD1(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\uD83C[\uDF3E\uDF73\uDF7C\uDF84\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83E\uDDD1|\uDDD1\u200D\uD83E\uDDD2(?:\u200D\uD83E\uDDD2)?|\uDDD2(?:\u200D\uD83E\uDDD2)?))|\uD83C(?:\uDFFB(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D(?:\uD83D\uDC8B\u200D)?\uD83E\uDDD1\uD83C[\uDFFC-\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF84\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83E\uDDD1\uD83C[\uDFFB-\uDFFF])))?|\uDFFC(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D(?:\uD83D\uDC8B\u200D)?\uD83E\uDDD1\uD83C[\uDFFB\uDFFD-\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF84\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83E\uDDD1\uD83C[\uDFFB-\uDFFF])))?|\uDFFD(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D(?:\uD83D\uDC8B\u200D)?\uD83E\uDDD1\uD83C[\uDFFB\uDFFC\uDFFE\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF84\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83E\uDDD1\uD83C[\uDFFB-\uDFFF])))?|\uDFFE(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D(?:\uD83D\uDC8B\u200D)?\uD83E\uDDD1\uD83C[\uDFFB-\uDFFD\uDFFF]|\uD83C[\uDF3E\uDF73\uDF7C\uDF84\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83E\uDDD1\uD83C[\uDFFB-\uDFFF])))?|\uDFFF(?:\u200D(?:[\u2695\u2696\u2708]\uFE0F?|\u2764\uFE0F?\u200D(?:\uD83D\uDC8B\u200D)?\uD83E\uDDD1\uD83C[\uDFFB-\uDFFE]|\uD83C[\uDF3E\uDF73\uDF7C\uDF84\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E(?:[\uDDAF\uDDBC\uDDBD](?:\u200D\u27A1\uFE0F?)?|[\uDDB0-\uDDB3]|\uDD1D\u200D\uD83E\uDDD1\uD83C[\uDFFB-\uDFFF])))?))?|\uDEF1(?:\uD83C(?:\uDFFB(?:\u200D\uD83E\uDEF2\uD83C[\uDFFC-\uDFFF])?|\uDFFC(?:\u200D\uD83E\uDEF2\uD83C[\uDFFB\uDFFD-\uDFFF])?|\uDFFD(?:\u200D\uD83E\uDEF2\uD83C[\uDFFB\uDFFC\uDFFE\uDFFF])?|\uDFFE(?:\u200D\uD83E\uDEF2\uD83C[\uDFFB-\uDFFD\uDFFF])?|\uDFFF(?:\u200D\uD83E\uDEF2\uD83C[\uDFFB-\uDFFE])?))?)/g;

const segmenter = new Intl.Segmenter, defaultIgnorableCodePointRegex = /^\p{Default_Ignorable_Code_Point}$/u;

function stringWidth$1(string, options = {}) {
    if ("string" != typeof string || 0 === string.length) return 0;
    const {ambiguousIsNarrow: ambiguousIsNarrow = !0, countAnsiEscapeCodes: countAnsiEscapeCodes = !1} = options;
    if (countAnsiEscapeCodes || (string = stripAnsi$2(string)), 0 === string.length) return 0;
    let width = 0;
    const eastAsianWidthOptions = {
        ambiguousAsWide: !ambiguousIsNarrow
    };
    for (const {segment: character} of segmenter.segment(string)) {
        const codePoint = character.codePointAt(0);
        codePoint <= 31 || codePoint >= 127 && codePoint <= 159 || (codePoint >= 8203 && codePoint <= 8207 || 65279 === codePoint || codePoint >= 768 && codePoint <= 879 || codePoint >= 6832 && codePoint <= 6911 || codePoint >= 7616 && codePoint <= 7679 || codePoint >= 8400 && codePoint <= 8447 || codePoint >= 65056 && codePoint <= 65071 || codePoint >= 55296 && codePoint <= 57343 || codePoint >= 65024 && codePoint <= 65039 || defaultIgnorableCodePointRegex.test(character) || (emojiRegex$1().test(character) ? width += 2 : width += eastAsianWidth(codePoint, eastAsianWidthOptions)));
    }
    return width;
}

function isInteractive({stream: stream = process.stdout} = {}) {
    return Boolean(stream && stream.isTTY && "dumb" !== process.env.TERM && !("CI" in process.env));
}

function isUnicodeSupported$1() {
    const {env: env} = process$2, {TERM: TERM, TERM_PROGRAM: TERM_PROGRAM} = env;
    return "win32" !== process$2.platform ? "linux" !== TERM : Boolean(env.WT_SESSION) || Boolean(env.TERMINUS_SUBLIME) || "{cmd::Cmder}" === env.ConEmuTask || "Terminus-Sublime" === TERM_PROGRAM || "vscode" === TERM_PROGRAM || "xterm-256color" === TERM || "alacritty" === TERM || "rxvt-unicode" === TERM || "rxvt-unicode-256color" === TERM || "JetBrains-JediTerm" === env.TERMINAL_EMULATOR;
}

const ASCII_ETX_CODE = 3;

class StdinDiscarder {
    #activeCount=0;
    start() {
        this.#activeCount++, 1 === this.#activeCount && this.#realStart();
    }
    stop() {
        if (this.#activeCount <= 0) throw new Error("`stop` called more times than `start`");
        this.#activeCount--, 0 === this.#activeCount && this.#realStop();
    }
    #realStart() {
        "win32" !== process$2.platform && process$2.stdin.isTTY && (process$2.stdin.setRawMode(!0), 
        process$2.stdin.on("data", this.#handleInput), process$2.stdin.resume());
    }
    #realStop() {
        process$2.stdin.isTTY && (process$2.stdin.off("data", this.#handleInput), process$2.stdin.pause(), 
        process$2.stdin.setRawMode(!1));
    }
    #handleInput(chunk) {
        chunk[0] === ASCII_ETX_CODE && process$2.emit("SIGINT");
    }
}

const stdinDiscarder = new StdinDiscarder;

class Ora {
    #linesToClear=0;
    #isDiscardingStdin=!1;
    #lineCount=0;
    #frameIndex=0;
    #options;
    #spinner;
    #stream;
    #id;
    #initialInterval;
    #isEnabled;
    #isSilent;
    #indent;
    #text;
    #prefixText;
    #suffixText;
    color;
    constructor(options) {
        "string" == typeof options && (options = {
            text: options
        }), this.#options = {
            color: "cyan",
            stream: process$2.stderr,
            discardStdin: !0,
            hideCursor: !0,
            ...options
        }, this.color = this.#options.color, this.spinner = this.#options.spinner, this.#initialInterval = this.#options.interval, 
        this.#stream = this.#options.stream, this.#isEnabled = "boolean" == typeof this.#options.isEnabled ? this.#options.isEnabled : isInteractive({
            stream: this.#stream
        }), this.#isSilent = "boolean" == typeof this.#options.isSilent && this.#options.isSilent, 
        this.text = this.#options.text, this.prefixText = this.#options.prefixText, this.suffixText = this.#options.suffixText, 
        this.indent = this.#options.indent, "test" === process$2.env.NODE_ENV && (this._stream = this.#stream, 
        this._isEnabled = this.#isEnabled, Object.defineProperty(this, "_linesToClear", {
            get() {
                return this.#linesToClear;
            },
            set(newValue) {
                this.#linesToClear = newValue;
            }
        }), Object.defineProperty(this, "_frameIndex", {
            get() {
                return this.#frameIndex;
            }
        }), Object.defineProperty(this, "_lineCount", {
            get() {
                return this.#lineCount;
            }
        }));
    }
    get indent() {
        return this.#indent;
    }
    set indent(indent = 0) {
        if (!(indent >= 0 && Number.isInteger(indent))) throw new Error("The `indent` option must be an integer from 0 and up");
        this.#indent = indent, this.#updateLineCount();
    }
    get interval() {
        return this.#initialInterval ?? this.#spinner.interval ?? 100;
    }
    get spinner() {
        return this.#spinner;
    }
    set spinner(spinner) {
        if (this.#frameIndex = 0, this.#initialInterval = void 0, "object" == typeof spinner) {
            if (void 0 === spinner.frames) throw new Error("The given spinner must have a `frames` property");
            this.#spinner = spinner;
        } else if (isUnicodeSupported$1()) if (void 0 === spinner) this.#spinner = cliSpinners.dots; else {
            if ("default" === spinner || !cliSpinners[spinner]) throw new Error(`There is no built-in spinner named '${spinner}'. See https://github.com/sindresorhus/cli-spinners/blob/main/spinners.json for a full list.`);
            this.#spinner = cliSpinners[spinner];
        } else this.#spinner = cliSpinners.line;
    }
    get text() {
        return this.#text;
    }
    set text(value = "") {
        this.#text = value, this.#updateLineCount();
    }
    get prefixText() {
        return this.#prefixText;
    }
    set prefixText(value = "") {
        this.#prefixText = value, this.#updateLineCount();
    }
    get suffixText() {
        return this.#suffixText;
    }
    set suffixText(value = "") {
        this.#suffixText = value, this.#updateLineCount();
    }
    get isSpinning() {
        return void 0 !== this.#id;
    }
    #getFullPrefixText(prefixText = this.#prefixText, postfix = " ") {
        return "string" == typeof prefixText && "" !== prefixText ? prefixText + postfix : "function" == typeof prefixText ? prefixText() + postfix : "";
    }
    #getFullSuffixText(suffixText = this.#suffixText, prefix = " ") {
        return "string" == typeof suffixText && "" !== suffixText ? prefix + suffixText : "function" == typeof suffixText ? prefix + suffixText() : "";
    }
    #updateLineCount() {
        const columns = this.#stream.columns ?? 80, fullPrefixText = this.#getFullPrefixText(this.#prefixText, "-"), fullSuffixText = this.#getFullSuffixText(this.#suffixText, "-"), fullText = " ".repeat(this.#indent) + fullPrefixText + "--" + this.#text + "--" + fullSuffixText;
        this.#lineCount = 0;
        for (const line of stripAnsi$2(fullText).split("\n")) this.#lineCount += Math.max(1, Math.ceil(stringWidth$1(line, {
            countAnsiEscapeCodes: !0
        }) / columns));
    }
    get isEnabled() {
        return this.#isEnabled && !this.#isSilent;
    }
    set isEnabled(value) {
        if ("boolean" != typeof value) throw new TypeError("The `isEnabled` option must be a boolean");
        this.#isEnabled = value;
    }
    get isSilent() {
        return this.#isSilent;
    }
    set isSilent(value) {
        if ("boolean" != typeof value) throw new TypeError("The `isSilent` option must be a boolean");
        this.#isSilent = value;
    }
    frame() {
        const {frames: frames} = this.#spinner;
        let frame = frames[this.#frameIndex];
        this.color && (frame = chalk[this.color](frame)), this.#frameIndex = ++this.#frameIndex % frames.length;
        return ("string" == typeof this.#prefixText && "" !== this.#prefixText ? this.#prefixText + " " : "") + frame + ("string" == typeof this.text ? " " + this.text : "") + ("string" == typeof this.#suffixText && "" !== this.#suffixText ? " " + this.#suffixText : "");
    }
    clear() {
        if (!this.#isEnabled || !this.#stream.isTTY) return this;
        this.#stream.cursorTo(0);
        for (let index = 0; index < this.#linesToClear; index++) index > 0 && this.#stream.moveCursor(0, -1), 
        this.#stream.clearLine(1);
        return (this.#indent || this.lastIndent !== this.#indent) && this.#stream.cursorTo(this.#indent), 
        this.lastIndent = this.#indent, this.#linesToClear = 0, this;
    }
    render() {
        return this.#isSilent || (this.clear(), this.#stream.write(this.frame()), this.#linesToClear = this.#lineCount), 
        this;
    }
    start(text) {
        return text && (this.text = text), this.#isSilent ? this : this.#isEnabled ? (this.isSpinning || (this.#options.hideCursor && cliCursor.hide(this.#stream), 
        this.#options.discardStdin && process$2.stdin.isTTY && (this.#isDiscardingStdin = !0, 
        stdinDiscarder.start()), this.render(), this.#id = setInterval(this.render.bind(this), this.interval)), 
        this) : (this.text && this.#stream.write(`- ${this.text}\n`), this);
    }
    stop() {
        return this.#isEnabled ? (clearInterval(this.#id), this.#id = void 0, this.#frameIndex = 0, 
        this.clear(), this.#options.hideCursor && cliCursor.show(this.#stream), this.#options.discardStdin && process$2.stdin.isTTY && this.#isDiscardingStdin && (stdinDiscarder.stop(), 
        this.#isDiscardingStdin = !1), this) : this;
    }
    succeed(text) {
        return this.stopAndPersist({
            symbol: logSymbols.success,
            text: text
        });
    }
    fail(text) {
        return this.stopAndPersist({
            symbol: logSymbols.error,
            text: text
        });
    }
    warn(text) {
        return this.stopAndPersist({
            symbol: logSymbols.warning,
            text: text
        });
    }
    info(text) {
        return this.stopAndPersist({
            symbol: logSymbols.info,
            text: text
        });
    }
    stopAndPersist(options = {}) {
        if (this.#isSilent) return this;
        const prefixText = options.prefixText ?? this.#prefixText, fullPrefixText = this.#getFullPrefixText(prefixText, " "), symbolText = options.symbol ?? " ", text = options.text ?? this.text, fullText = "string" == typeof text ? " " + text : "", suffixText = options.suffixText ?? this.#suffixText, textToWrite = fullPrefixText + symbolText + fullText + this.#getFullSuffixText(suffixText, " ") + "\n";
        return this.stop(), this.#stream.write(textToWrite), this;
    }
}

function ora(options) {
    return new Ora(options);
}

async function installMucho({updateAvailable: updateAvailable} = {}) {
    const spinner = ora("Checking the mucho cli...").start();
    try {
        let installedVersion = await getCurrentNpmPackageVersion("mucho", !0);
        if (installedVersion) {
            if (!updateAvailable) return spinner.info(`mucho ${installedVersion} is already installed`), 
            !0;
            spinner.text = "Updating the mucho cli";
        } else spinner.text = "Installing the mucho cli";
        return await src_default$1("npm install -gy --force mucho@latest"), spinner.text = "Verifying mucho was installed", 
        installedVersion = await getCurrentNpmPackageVersion("mucho", !0), installedVersion ? (spinner.succeed(`mucho ${installedVersion} installed`), 
        installedVersion) : (spinner.fail(picocolors.red("mucho cli failed to install")), 
        !1);
    } catch (err) {
        spinner.fail(picocolors.red("Unable to install the mucho cli")), "string" == typeof err ? console.error(err) : (Error, 
        console.error(err.message));
    }
    return !1;
}

async function checkDebianDependenciesInstalled() {
    const deps = [ "build-essential", "pkg-config", "libudev-dev", "llvm", "libclang-dev", "protobuf-compiler", "libssl-dev" ], statuses = await Promise.allSettled(deps.map((dep => checkCommand(`apt list ${dep} | grep [installed]`, {
        exit: !1
    })))), missingDeps = [];
    if (statuses.map(((res, index) => {
        let isInstalled = !1;
        "rejected" != res.status && res?.value ? res.value.split("\n").map((data => {
            new RegExp(`^${deps[index]}/[^\\s]+.+\\[installed\\]`, "m").test(data) && (isInstalled = !0);
        })) : console.error("Unable to detect dependency:", deps[index]), isInstalled || missingDeps.push(deps[index]);
    })), 0 != missingDeps.length) return missingDeps;
}

async function installRust({version: version} = {}) {
    const spinner = ora("Installing the rust toolchain using rustup").start();
    try {
        appendPathToRCFiles(TOOL_CONFIG.rust.pathSource, "rust");
        if ("linux" == detectOperatingSystem() && await checkCommand("apt --version")) {
            spinner.text = "Checking for required linux dependencies";
            const missingDeps = await checkDebianDependenciesInstalled();
            missingDeps && missingDeps.length > 0 && (spinner.fail(picocolors.red(`Missing dependencies: ${missingDeps.join(" ")}`)), 
            console.log("Install the missing dependencies using the following command:"), console.log("sudo apt update && sudo apt install -y", missingDeps.join(" ")), 
            process.exit(0));
        }
        let installedVersion = await installedToolVersion("rust");
        if (installedVersion) return spinner.info(`rust ${installedVersion} is already installed`), 
        !0;
        spinner.text = "Installing the rust toolchain using rustup";
        const result = await src_default$1("curl --proto '=https' --tlsv1.2 -sSf https://sh.rustup.rs | sh -s -- -y");
        if (result && 0 != result.code) {
            throw result.stderr.trim().split("\n").join("\n");
        }
        return spinner.text = "Verifying rust was installed", installedVersion = await installedToolVersion("rust"), 
        installedVersion ? (spinner.succeed(`rust ${installedVersion} installed`), installedVersion) : (spinner.fail(picocolors.red("rust failed to install")), 
        !1);
    } catch (err) {
        spinner.fail(picocolors.red("Unable to install rust")), "string" == typeof err ? console.error(err) : (Error, 
        console.error(err.message));
    }
    return !1;
}

async function installSolana({version: version = "stable"} = {}) {
    const spinner = ora("Installing the Solana CLI tool suite...").start();
    try {
        appendPathToRCFiles(TOOL_CONFIG.solana.pathSource, "solana");
        let installedVersion = await installedToolVersion("solana");
        if (installedVersion) return spinner.info(`solana ${installedVersion} is already installed`), 
        !0;
        if ("stable" != (version = version.toLowerCase()) && !version.startsWith("v")) throw `Invalid version: '${version}'`;
        spinner.text = "Installing the Solana CLI tool suite...";
        const result = await src_default$1(`sh -c "$(curl -sSfL https://release.anza.xyz/${version}/install)"`);
        if (result && 0 != result.code) {
            throw result.stderr.trim().split("\n").join("\n");
        }
        return spinner.text = "Verifying solana was installed", installedVersion = await installedToolVersion("solana"), 
        installedVersion ? (spinner.succeed(`solana ${installedVersion} installed`), installedVersion) : (spinner.fail(picocolors.red("solana failed to install")), 
        !1);
    } catch (err) {
        spinner.fail(picocolors.red("Unable to install the Solana CLI tool suite")), "string" == typeof err ? console.error(err) : (Error, 
        console.error(err.message));
    }
    return !1;
}

async function installAnchorVersionManager({version: version = "latest", updateAvailable: updateAvailable} = {}) {
    const spinner = ora("Installing avm (anchor version manager)").start();
    try {
        let installedVersion = await installedToolVersion("avm");
        if (installedVersion) {
            let message = `avm ${installedVersion} is already installed`;
            return updateAvailable && (message = picocolors.yellow(message + " - update available")), 
            spinner.info(message), !0;
        }
        spinner.text = "Installing avm (anchor version manager)";
        const result = await src_default$1("cargo install --git https://github.com/coral-xyz/anchor avm --locked --force");
        if (result && 0 != result.code) {
            throw result.stderr.trim().split("\n").join("\n");
        }
        return spinner.text = "Verifying avm was installed", installedVersion = await installedToolVersion("avm"), 
        installedVersion ? (spinner.succeed(`avm ${installedVersion} installed`), installedVersion) : (spinner.fail(picocolors.red("avm failed to install")), 
        !1);
    } catch (err) {
        spinner.fail(picocolors.red("Unable to install avm")), "string" == typeof err ? console.error(err) : (Error, 
        console.error(err.message));
    }
    return !1;
}

async function installAnchorUsingAvm({verifyParentCommand: verifyParentCommand = !0, version: version = "latest", updateAvailable: updateAvailable} = {}) {
    const spinner = ora("Installing anchor using avm...").start();
    try {
        let result, installedVersion = await installedToolVersion("anchor");
        if (installedVersion) {
            let message = `anchor ${installedVersion} is already installed`;
            return updateAvailable && (message = picocolors.yellow(message + ` - v${updateAvailable.latest} available`)), 
            spinner.info(message), !0;
        }
        if (verifyParentCommand) {
            spinner.text = "Verifying avm is installed";
            const avmVersion = await installedToolVersion("avm");
            if (!avmVersion) throw "avm is NOT already installed";
            version = avmVersion;
        }
        if (installedVersion && installedVersion == version) return spinner.info(`anchor ${installedVersion} is already installed`), 
        !0;
        try {
            spinner.text = `Installing anchor version '${version}'. This may take a few moments...`, 
            result = await src_default$1(`avm install ${version}`);
        } catch (err) {
            throw "Unable to execute `avm install`";
        }
        if (result && 0 != result.code) {
            throw result.stderr.trim().split("\n").join("\n");
        }
        try {
            spinner.text = "Setting anchor version with avm", result = await src_default$1(`avm use ${version}`);
        } catch (err) {
            throw "Unable to execute `avm use`";
        }
        return spinner.text = "Verifying anchor was installed", installedVersion = await installedToolVersion("anchor"), 
        installedVersion ? (spinner.succeed(`anchor ${installedVersion} installed using avm`), 
        installedVersion) : (spinner.fail(picocolors.red("anchor failed to install")), !1);
    } catch (err) {
        spinner.fail(picocolors.red("Unable to install anchor using avm")), "string" == typeof err ? console.error(err) : (Error, 
        console.error(err.message));
    }
    return !1;
}

async function installYarn({} = {}) {
    const spinner = ora("Installing yarn package manager...").start();
    try {
        let installedVersion = await installedToolVersion("yarn");
        return installedVersion ? (spinner.info(`yarn ${installedVersion} is already installed`), 
        !0) : (spinner.text = "Installing yarn package manager", await src_default$1("npm install -gy yarn"), 
        spinner.text = "Verifying yarn was installed", installedVersion = await installedToolVersion("yarn"), 
        installedVersion ? (spinner.succeed(`yarn ${installedVersion} installed`), installedVersion) : (spinner.fail(picocolors.red("yarn package manager failed to install")), 
        !1));
    } catch (err) {
        spinner.fail(picocolors.red("Unable to install yarn package manager")), "string" == typeof err ? console.error(err) : (Error, 
        console.error(err.message));
    }
    return !1;
}

async function installTrident({version: version = "latest", verifyParentCommand: verifyParentCommand = !0, updateAvailable: updateAvailable} = {}) {
    const spinner = ora("Installing trident (fuzzer)").start();
    try {
        let installedVersion = await installedToolVersion("trident");
        if (installedVersion) {
            let message = `trident ${installedVersion} is already installed`;
            return updateAvailable && (message = picocolors.yellow(message + ` - ${updateAvailable.latest} available`)), 
            spinner.info(message), !0;
        }
        if (verifyParentCommand) {
            if (!await installedToolVersion("rust")) throw spinner.fail(picocolors.red("Rust/cargo was not found")), 
            "parent command not found";
        }
        spinner.text = "Installing trident (fuzzer)";
        const result = await src_default$1("cargo install honggfuzz trident-cli");
        if (result && 0 != result.code) {
            throw result.stderr.trim().split("\n").join("\n");
        }
        return spinner.text = "Verifying trident was installed", installedVersion = await installedToolVersion("trident"), 
        installedVersion ? (spinner.succeed(`trident ${installedVersion} installed`), installedVersion) : (spinner.fail(picocolors.red("trident failed to install")), 
        !1);
    } catch (err) {
        spinner.fail(picocolors.red("Unable to install the trident fuzzer")), "string" == typeof err ? console.error(err) : (Error, 
        console.error(err.message));
    }
    return !1;
}

async function installZest({version: version = "latest", verifyParentCommand: verifyParentCommand = !0, updateAvailable: updateAvailable} = {}) {
    const spinner = ora("Installing zest (code coverage)").start();
    try {
        let result, installedVersion = await installedToolVersion("zest");
        if (installedVersion) {
            let message = `zest ${installedVersion} is already installed`;
            return updateAvailable && (message = picocolors.yellow(message + " - update available")), 
            spinner.info(message), !0;
        }
        if (verifyParentCommand) {
            if (!await installedToolVersion("rust")) throw "rustc/cargo was not found but is required";
        }
        try {
            spinner.text = "Installing zest (code coverage)...", result = await src_default$1("cargo install --git https://github.com/LimeChain/zest zest --force");
        } catch (err) {
            throw "Unable to execute the zest installer";
        }
        if (result && 0 != result.code) {
            const error = result.stderr.trim().split("\n");
            let parsed = null;
            if (parsed = error.slice(-1)[0].match(/(?<=\bit requires rustc\s)\d+\.\d+(\.\d+)?(?=\sor newer\b)/gi)[0]) throw `Zest requires a minimum rustc version of ${parsed}. To set your rustc version, run the following command:\nrustup default ${parsed}`;
            throw error.join("\n");
        }
        try {
            spinner.text = "Installing the 'llvm-tools-preview' component in rustup", result = await src_default$1("rustup component add llvm-tools-preview");
        } catch (err) {
            throw "Unable to execute 'rustup component add llvm-tools-preview'. Try running manually.";
        }
        return spinner.text = "Verifying zest was installed", installedVersion = await installedToolVersion("zest"), 
        installedVersion ? (spinner.succeed(`zest ${installedVersion} installed`), installedVersion) : (spinner.fail(picocolors.red("zest failed to install")), 
        !1);
    } catch (err) {
        spinner.fail(picocolors.red("Unable to install zest")), "string" == typeof err ? console.error(err) : (Error, 
        console.error(err.message));
    }
    return !1;
}

async function installCargoUpdate({version: version = "latest", verifyParentCommand: verifyParentCommand = !0} = {}) {
    const spinner = ora("Installing cargo-update").start();
    try {
        let result, installedVersion = await installedToolVersion("cargo-update");
        if (installedVersion) return spinner.info(`cargo-update ${installedVersion} is already installed`), 
        !0;
        if (verifyParentCommand) {
            if (!await installedToolVersion("rust")) throw "rustc/cargo was not found but is required";
        }
        try {
            spinner.text = "Installing cargo-update...", result = await src_default$1("cargo install cargo-update");
        } catch (err) {
            throw "Unable to execute the cargo-update installer";
        }
        if (result && 0 != result.code) {
            throw result.stderr.trim().split("\n").join("\n");
        }
        return spinner.text = "Verifying cargo-update was installed", installedVersion = await installedToolVersion("cargo-update"), 
        installedVersion ? (spinner.succeed(`cargo-update ${installedVersion} installed`), 
        installedVersion) : (spinner.fail(picocolors.red("cargo-update failed to install")), 
        !1);
    } catch (err) {
        spinner.fail(picocolors.red("Unable to install cargo-update")), "string" == typeof err ? console.error(err) : (Error, 
        console.error(err.message));
    }
    return !1;
}

async function installSolanaVerify({version: version = "latest", verifyParentCommand: verifyParentCommand = !0, updateAvailable: updateAvailable} = {}) {
    const spinner = ora("Installing solana-verify").start();
    try {
        let result, installedVersion = await installedToolVersion("verify");
        if (installedVersion) {
            let message = `solana-verify ${installedVersion} is already installed`;
            return updateAvailable && (message = picocolors.yellow(message + ` - ${updateAvailable.latest} available`)), 
            spinner.info(message), await isDockerInstalled(), !0;
        }
        if (verifyParentCommand) {
            if (!await installedToolVersion("rust")) throw "rustc/cargo was not found but is required";
        }
        try {
            spinner.text = "Installing solana-verify...", result = await src_default$1("cargo install solana-verify");
        } catch (err) {
            throw "Unable to execute the solana-verify installer";
        }
        if (result && 0 != result.code) {
            const error = result.stderr.trim().split("\n");
            throw error.join("\n");
        }
        return spinner.text = "Verifying solana-verify was installed", installedVersion = await installedToolVersion("verify"), 
        installedVersion ? (spinner.succeed(`solana-verify ${installedVersion} installed`), 
        await isDockerInstalled(), installedVersion) : (spinner.fail(picocolors.red("solana-verify failed to install")), 
        !1);
    } catch (err) {
        spinner.fail(picocolors.red("Unable to install solana-verify")), "string" == typeof err ? console.error(err) : (Error, 
        console.error(err.message));
    }
    return !1;
}

async function isDockerInstalled() {
    return checkCommand("docker --version", {
        exit: !1,
        onError: () => {
            console.error("Unable to detect Docker (which is required for 'solana-verify'). Do you have it installed?"), 
            console.error("To install Docker, follow the instructions in the official Docker documentation:", "\nhttps://docs.docker.com/engine/install/");
        }
    });
}

async function checkInstalledTools({outputToolStatus: outputToolStatus = !1} = {}) {
    let allInstalled = !0, status = {
        rust: !1,
        solana: !1,
        avm: !1,
        anchor: !1,
        node: !1,
        yarn: !1,
        trident: !1,
        zest: !1,
        "cargo-update": !1,
        verify: !1
    };
    if (await Promise.all(Object.keys(status).map((async tool => {
        const version = await installedToolVersion(tool);
        status[tool] = version, status[tool] || (allInstalled = !1);
    }))), outputToolStatus) {
        let noteContent = "";
        for (const command in status) Object.prototype.hasOwnProperty.call(status, command) && (noteContent += "- ", 
        status[command] ? noteContent += picocolors.green(command) : noteContent += picocolors.red(command), 
        noteContent += ` ${status[command] || "(not installed)"}\n`);
        console.log(noteContent.trim(), "\n");
    }
    return {
        allInstalled: allInstalled,
        status: status
    };
}

async function checkShellPathSource(cmd, pathSource) {
    const [withoutPath, withPath] = await Promise.allSettled([ src_default$1(cmd), src_default$1(`export PATH="${pathSource}:$PATH" && ${cmd}`) ]);
    if ("fulfilled" == withoutPath.status && "fulfilled" == withPath.status) {
        if (0 === withoutPath.value.code) return withoutPath.value.stdout.toLowerCase() == withPath.value.stdout.toLowerCase() ? PathSourceStatus.SUCCESS : PathSourceStatus.OUTPUT_MISMATCH;
        if (0 === withPath.value.code) return PathSourceStatus.MISSING_PATH;
    }
    return PathSourceStatus.FAILED;
}

async function getNpmPackageUpdates(packageName, global = !1) {
    const updates = [];
    const current = await getCurrentNpmPackageVersion(packageName, true), registry = await getNpmRegistryPackageVersion(packageName);
    return (!current || registry.latest && isVersionNewer(registry.latest, current)) && updates.push({
        installed: current,
        needsUpdate: !0,
        latest: registry.latest,
        name: packageName
    }), updates;
}

async function getCargoUpdateOutput() {
    const res = await checkCommand("cargo install-update --git --list", {
        exit: !0,
        onError: async () => {
            warnMessage("Unable to detect the 'cargo install-update' command. Installing..."), 
            await installCargoUpdate();
        },
        doubleCheck: !0
    });
    if (!res) return [];
    const results = [], linesToParse = ("Package" + res.split("\nPackage").slice(1).join("Package")).split("\n").filter((line => line.trim().length > 0));
    let currentChunk = [];
    for (const line of linesToParse) line.startsWith("Package") ? (currentChunk.length > 1 && processChunk(currentChunk, results), 
    currentChunk = [ line ]) : currentChunk.push(line);
    return currentChunk.length > 1 && processChunk(currentChunk, results), results;
}

function processChunk(chunk, results) {
    const headerLine = chunk[0], dataLines = chunk.slice(1);
    headerLine.toLowerCase().includes("installed") && headerLine.split(/\s+/).length;
    for (const line of dataLines) {
        const parts = line.split(/\s+/).filter((part => part.length > 0));
        results.push({
            name: parts[0],
            installed: parts[1],
            latest: parts[2],
            needsUpdate: "yes" === parts[3].toLowerCase()
        });
    }
}

const ANCHOR_TOML = "Anchor.toml";

function loadAnchorToml(configPath, isConfigRequired = !1) {
    configPath.endsWith(ANCHOR_TOML) || (configPath = dirname(configPath)), directoryExists(configPath) && (configPath = join(configPath, ANCHOR_TOML));
    let anchor = {
        configPath: configPath
    };
    return doesFileExist(configPath, !0) ? (anchor = loadTomlFile(configPath) || anchor, 
    anchor.configPath = configPath, anchor) : (isConfigRequired && warningOutro("No Anchor.toml config file found. Operation canceled."), 
    !1);
}

function deconflictAnchorTomlWithConfig(anchorToml, config) {
    if (anchorToml.test?.validator?.clone) for (const cloner in anchorToml.test.validator.clone) Object.prototype.hasOwnProperty.call(anchorToml.test.validator.clone, cloner) && !config.clone.program[anchorToml.test.validator.clone[cloner].address] && (anchorToml.test.validator?.url && (anchorToml.test.validator.clone[cloner].cluster = anchorToml.test.validator.url), 
    config.clone.program[anchorToml.test.validator.clone[cloner].address] = anchorToml.test.validator.clone[cloner]);
    if (anchorToml.test?.validator?.account) for (const cloner in anchorToml.test.validator.account) Object.prototype.hasOwnProperty.call(anchorToml.test.validator.account, cloner) && !config.clone.account[anchorToml.test.validator.account[cloner].address] && (anchorToml.test.validator?.url && (anchorToml.test.validator.account[cloner].cluster = anchorToml.test.validator.url), 
    config.clone.account[anchorToml.test.validator.account[cloner].address] = anchorToml.test.validator.account[cloner]);
    return config;
}

async function getAvailableAnchorVersions() {
    const res = await checkCommand("avm list", {
        exit: !0,
        onError: async () => {
            warnMessage("Unable to detect the 'avm' command. Installing..."), await installAnchorVersionManager();
        },
        doubleCheck: !0
    }), data = {
        latest: null,
        current: null,
        installed: [],
        available: []
    };
    return res ? (res.split("\n").map((line => {
        if (!(line = line.trim().toLowerCase())) return;
        const version = VERSION_REGEX.exec(line)[1];
        version && (line.includes("current") && (data.current = version), line.includes("latest") && (data.latest = version), 
        line.includes("installed") && data.installed.push(version), data.available.push(version));
    })), data) : data;
}

const toolNames = [ "rust", "solana", "avm", "anchor", "trident", "zest", "yarn", "verify" ];

function installCommand() {
    return new Command("install").configureOutput(cliOutputConfig).description("install Solana development tooling").addArgument(new Argument("<tool>", "tool to install (default: all)").choices(toolNames).argOptional()).addArgument(new Argument("<version>", "desired tool version to install (default: stable)").argOptional()).action((async (toolName, version, options) => {
        titleMessage("Install Solana development tooling");
        const os = detectOperatingSystem();
        "windows" == os && warnMessage("Windows is not yet natively supported for the rust based tooling.\nWe recommend using WSL inside your Windows terminal.");
        const pathsToRefresh = [], updatesAvailable = await getNpmPackageUpdates("mucho", !0);
        await installMucho({
            os: os,
            version: version,
            updateAvailable: updatesAvailable.filter((data => "mucho" == data.name.toLowerCase()))[0]
        }), toolName && "rust" != toolName || (await installRust({
            os: os,
            version: version
        }), await checkShellPathSource(TOOL_CONFIG.rust.version, TOOL_CONFIG.rust.pathSource).then((status => status != PathSourceStatus.MISSING_PATH || pathsToRefresh.push(TOOL_CONFIG.rust.pathSource))));
        const anchorVersions = await getAvailableAnchorVersions();
        anchorVersions.current && anchorVersions.latest && isVersionNewer(anchorVersions.latest, anchorVersions.current) && updatesAvailable.push({
            installed: anchorVersions.current,
            latest: anchorVersions.latest,
            name: "anchor",
            needsUpdate: !0
        }), updatesAvailable.push(...await getCargoUpdateOutput()), toolName && "solana" != toolName || (await installSolana({
            os: os,
            version: version
        }), await checkShellPathSource(TOOL_CONFIG.solana.version, TOOL_CONFIG.solana.pathSource).then((status => status != PathSourceStatus.MISSING_PATH || pathsToRefresh.push(TOOL_CONFIG.solana.pathSource)))), 
        toolName && "avm" != toolName || await installAnchorVersionManager({
            os: os,
            version: version,
            updateAvailable: updatesAvailable.filter((data => "avm" == data.name.toLowerCase()))[0]
        }), toolName && "anchor" != toolName || await installAnchorUsingAvm({
            os: os,
            version: version,
            updateAvailable: updatesAvailable.filter((data => "anchor" == data.name.toLowerCase()))[0]
        }), toolName && "trident" != toolName || await installTrident({
            os: os,
            version: version,
            updateAvailable: updatesAvailable.filter((data => "trident-cli" == data.name.toLowerCase()))[0]
        }), toolName && "zest" != toolName || await installZest({
            os: os,
            version: version,
            updateAvailable: updatesAvailable.filter((data => "zest" == data.name.toLowerCase()))[0]
        }), toolName && "verify" != toolName || await installSolanaVerify({
            os: os,
            version: version,
            updateAvailable: updatesAvailable.filter((data => "solana-verify" == data.name.toLowerCase()))[0]
        }), toolName && "yarn" != toolName || await installYarn({
            os: os,
            version: version
        }), pathsToRefresh.length > 0 && (console.log("\nClose and reopen your terminal to apply the required", "PATH changes \nor run the following in your existing shell:", "\n"), 
        console.log(`export PATH="${pathsToRefresh.join(":")}:$PATH"`, "\n"));
    }));
}

function infoCommand() {
    return new Command("info").configureOutput(cliOutputConfig).description("gather helpful troubleshooting info about your setup").action((async () => {
        titleMessage("Gather troubleshooting info"), console.log();
        const spinner = ora("Gathering mucho info").start(), separator = "------------------------", info = [];
        info.push("--- start mucho info ---"), info.push("System Info:");
        const sysInfo = await getCommandOutput("uname -a");
        sysInfo ? info.push(sysInfo.toString()) : info.push(`platform: ${os.platform()}, arch: ${os.arch()}`);
        const tools = await checkInstalledTools();
        info.push(separator), info.push("Installed tooling:");
        for (const key in tools.status) info.push("  " + key + ": " + tools.status[key]);
        info.push(separator), info.push("Platform tools:");
        const solanaTools = getPlatformToolsVersions();
        for (const key in solanaTools) info.push("  " + key + ": " + solanaTools[key]);
        info.push(separator), info.push("Solana CLI info:");
        const solanaConfig = await getCommandOutput("solana config get") || "Unable to get 'solana config'";
        info.push(solanaConfig.split("\n").map((line => `  ${line}`)).join("\n")), info.push(`Does CLI config.yml exist: ${await doesFileExist(DEFAULT_CLI_YAML_PATH)}`);
        const testValidatorChecker = await getCommandOutput("ps aux | awk '/solana-test-validator/ && !/grep/ {$1=$2=$3=$4=$5=$6=$7=$8=$9=$10=\"\"; print substr($0,11)}'");
        let localnetClusterUrl = "http://127.0.0.1:8899";
        if (testValidatorChecker && testValidatorChecker.includes("--rpc-port") && (localnetClusterUrl = `http://127.0.0.1:${testValidatorChecker.match(/--rpc-port\s+(\d+)/)[1] || 8899}`), 
        await doesFileExist(DEFAULT_KEYPAIR_PATH)) {
            const address = await getCommandOutput("solana address");
            info.push("Address: " + address);
            const balances = {
                devnet: await getCommandOutput("solana balance -ud") || !1,
                mainnet: await getCommandOutput("solana balance -um") || !1,
                testnet: await getCommandOutput("solana balance -ut") || !1,
                localnet: await getCommandOutput(`solana balance --url ${localnetClusterUrl}`) || !1
            };
            info.push("Balances for address:");
            for (const key in balances) info.push("  " + key + ": " + balances[key]);
        } else info.push(`Default keypair file NOT found at: ${DEFAULT_KEYPAIR_PATH}`);
        info.push("Is test-validator running? " + !!testValidatorChecker), info.push("Localnet url: " + localnetClusterUrl), 
        info.push("---- end mucho info ----"), spinner.stop(), console.log(info.join("\n")), 
        console.log(), console.log("Looking for troubleshooting help? Post a question on the Solana StackExchange:"), 
        console.log("https://solana.stackexchange.com");
    }));
}

async function cloneAccount({address: address, saveDir: saveDir = DEFAULT_ACCOUNTS_DIR, url: url}) {
    let command = [ `solana account ${address}`, "--output json" ];
    url && command.push(`--url ${parseRpcUrlOrMoniker(url, !0)}`);
    const saveFile = path.resolve(saveDir, `${address}.json`);
    return createFolders(saveFile), command.push(`--output-file ${saveFile}`), await src_default$1(command.join(" ")), 
    doesFileExist(saveFile) && loadJsonFile(saveFile) || !1;
}

async function cloneProgram({address: address, saveDir: saveDir = "programs", url: url}) {
    let command = [ "solana program dump" ];
    const saveFile = path.resolve(saveDir, `${address}.so`);
    return createFolders(saveDir), command.push(address, saveFile), url && command.push(`--url ${parseRpcUrlOrMoniker(url, !0)}`), 
    await src_default$1(command.join(" ")), !!doesFileExist(saveFile);
}

async function cloneProgramsFromConfig(config, settings, currentAccounts) {
    if (!config?.clone?.program) return null;
    for (const key in config.clone.program) {
        if (!config.clone.program.hasOwnProperty(key)) continue;
        const program = config.clone.program[key];
        if (program?.name || (program.name = key), settings.autoClone && warnMessage(`Auto clone program: ${program.address}`), 
        "always" === program.frequency) console.log("Always clone program:", program.address); else if (!0 === settings.force) console.log("Force clone program:", program.address); else {
            if (currentAccounts.has(program.address)) {
                console.log("Skipping clone program:", program.address);
                continue;
            }
            console.log("Clone program:", program.address);
        }
        await cloneProgram({
            address: program.address,
            saveDir: DEFAULT_ACCOUNTS_DIR_TEMP,
            url: program.cluster || config?.settings?.cluster
        }) || console.error("Failed to clone program:", program.address);
    }
}

async function cloneTokensFromConfig(config, settings, currentAccounts) {
    if (!config?.clone?.token) return null;
    for (const key in config.clone.token) {
        if (!config.clone.token.hasOwnProperty(key)) continue;
        const token = config.clone.token[key];
        if (token?.name || (token.name = key), "always" === token.frequency) console.log("Always clone token:", token.address); else if (!0 === settings.force) console.log("Force clone token:", token.address); else {
            if (currentAccounts.has(token.address)) {
                console.log("Skipping clone token:", token.address);
                continue;
            }
            console.log("Clone token:", token.address);
        }
        const newAccount = await cloneAccount({
            saveDir: DEFAULT_ACCOUNTS_DIR_TEMP,
            address: token.address,
            url: token.cluster || config.settings.cluster
        });
        if (newAccount) {
            if (1 == settings.force) ; else if (doesFileExist(path.join(config.settings.accountDir, `${token.address}.json`))) {
                const oldAccount = loadJsonFile(path.resolve(config.settings.accountDir, `${token.address}.json`));
                if (JSON.stringify(newAccount) !== JSON.stringify(oldAccount)) {
                    warnMessage(`${token.address} has changed`), settings.prompt;
                    continue;
                }
            }
        } else console.error("Failed to clone token:", token.address);
    }
}

async function cloneAccountsFromConfig(config, settings, currentAccounts) {
    if (!config?.clone?.account) return !1;
    const owners = new Map, changedAccounts = new Map;
    let newAccount = !1;
    for (const key in config.clone.account) {
        if (!config.clone.account.hasOwnProperty(key)) continue;
        const account = config.clone.account[key];
        if (account?.name || (account.name = key), "always" === account.frequency) console.log("Always clone account:", account.address); else if (!0 === settings.force) console.log("Force clone account:", account.address); else {
            if (currentAccounts.has(account.address)) {
                newAccount = loadJsonFile(path.join(config.settings.accountDir, currentAccounts.get(account.address))), 
                owners.set(newAccount.account.owner, account.cluster || config.settings.cluster), 
                console.log("Skipping clone account:", account.address);
                continue;
            }
            console.log("Clone account:", account.address);
        }
        if (newAccount = await cloneAccount({
            saveDir: DEFAULT_ACCOUNTS_DIR_TEMP,
            address: account.address,
            url: account.cluster || config.settings.cluster
        }), newAccount) {
            if (owners.set(newAccount.account.owner, account.cluster || config.settings.cluster), 
            1 == settings.force) ; else if (doesFileExist(path.join(config.settings.accountDir, `${account.address}.json`))) {
                const oldAccount = loadJsonFile(path.resolve(config.settings.accountDir, `${account.address}.json`));
                if (JSON.stringify(newAccount) !== JSON.stringify(oldAccount)) {
                    warnMessage(`${account.address} has changed`), settings.prompt && changedAccounts.set(newAccount.account.owner, newAccount);
                    continue;
                }
            }
        } else console.error("Failed to clone account:", account.address);
    }
    return {
        owners: owners,
        changedAccounts: changedAccounts
    };
}

function mergeOwnersMapWithConfig(owners, config = {}) {
    return owners ? (owners.delete("11111111111111111111111111111111"), owners.delete("TokenkegQfeZyiNwAJbNbGKPFXCWuBvf9Ss623VQ5DA"), 
    owners.delete("TokenzQdBNbLqP5VEhdkAS6EPFLC1PHnBqCXEpPxuEb"), 0 == owners.size || owners.forEach(((cluster, address) => {
        config[address] = {
            address: address,
            cluster: cluster
        };
    })), config) : config;
}

function validateExpectedCloneCounts(accountDir, clone) {
    const clonedAccounts = loadFileNamesToMap(accountDir = path.resolve(accountDir), ".json"), clonedPrograms = loadFileNamesToMap(accountDir, ".so"), actual = clonedAccounts.size + clonedPrograms.size, autoCloned = new Map;
    clonedAccounts.forEach(((filename, key) => {
        autoCloned.set(loadJsonFile(path.join(accountDir, filename)).account.owner, "");
    }));
    let expected = 0;
    return clone && (clone.account && (expected += Object.keys(clone.account).length), 
    clone.token && (expected += Object.keys(clone.token).length), clone.program && (clone.program = mergeOwnersMapWithConfig(autoCloned, clone?.program || {}) || {}, 
    expected += Object.keys(clone.program).length)), {
        actual: actual,
        expected: expected
    };
}

function isGitRepo(targetFolder) {
    try {
        return "true" === execSync$1(`git -C ${targetFolder} rev-parse --is-inside-work-tree`, {
            stdio: "pipe"
        }).toString().trim();
    } catch (error) {
        return !1;
    }
}

function initGitRepo(targetFolder, commitMessage = "init") {
    try {
        createFolders(targetFolder, !0);
        const commands = [];
        return commands.push(`git init ${targetFolder}`, `git add ${join(targetFolder, ".")} --force`, `git -C ${targetFolder} commit -m "${commitMessage}"`), 
        execSync$1(commands.join(" && "), {
            stdio: "ignore"
        }), !0;
    } catch (error) {
        return console.error("[initGitRepo]", "Unable to execute 'git init'"), !1;
    }
}

const isUpKey = key => "up" === key.name || "k" === key.name || key.ctrl && "p" === key.name, isDownKey = key => "down" === key.name || "j" === key.name || key.ctrl && "n" === key.name, isBackspaceKey = key => "backspace" === key.name, isNumberKey = key => "123456789".includes(key.name), isEnterKey = key => "enter" === key.name || "return" === key.name;

class AbortPromptError extends Error {
    name="AbortPromptError";
    message="Prompt was aborted";
    constructor(options) {
        super(), this.cause = options?.cause;
    }
}

class CancelPromptError extends Error {
    name="CancelPromptError";
    message="Prompt was canceled";
}

class ExitPromptError extends Error {
    name="ExitPromptError";
}

class HookError extends Error {
    name="HookError";
}

class ValidationError extends Error {
    name="ValidationError";
}

const hookStorage = new AsyncLocalStorage;

function createStore(rl) {
    return {
        rl: rl,
        hooks: [],
        hooksCleanup: [],
        hooksEffect: [],
        index: 0,
        handleChange() {}
    };
}

function withHooks(rl, cb) {
    const store = createStore(rl);
    return hookStorage.run(store, (() => cb((function(render) {
        store.handleChange = () => {
            store.index = 0, render();
        }, store.handleChange();
    }))));
}

function getStore() {
    const store = hookStorage.getStore();
    if (!store) throw new HookError("[Inquirer] Hook functions can only be called from within a prompt");
    return store;
}

function readline() {
    return getStore().rl;
}

function withUpdates(fn) {
    return AsyncResource.bind(((...args) => {
        const store = getStore();
        let shouldUpdate = !1;
        const oldHandleChange = store.handleChange;
        store.handleChange = () => {
            shouldUpdate = !0;
        };
        const returnValue = fn(...args);
        return shouldUpdate && oldHandleChange(), store.handleChange = oldHandleChange, 
        returnValue;
    }));
}

function withPointer(cb) {
    const store = getStore(), {index: index} = store, returnValue = cb({
        get: () => store.hooks[index],
        set(value) {
            store.hooks[index] = value;
        },
        initialized: index in store.hooks
    });
    return store.index++, returnValue;
}

function handleChange() {
    getStore().handleChange();
}

const effectScheduler = {
    queue(cb) {
        const store = getStore(), {index: index} = store;
        store.hooksEffect.push((() => {
            store.hooksCleanup[index]?.();
            const cleanFn = cb(readline());
            if (null != cleanFn && "function" != typeof cleanFn) throw new ValidationError("useEffect return value must be a cleanup function or nothing.");
            store.hooksCleanup[index] = cleanFn;
        }));
    },
    run() {
        const store = getStore();
        withUpdates((() => {
            store.hooksEffect.forEach((effect => {
                effect();
            })), store.hooksEffect.length = 0;
        }))();
    },
    clearAll() {
        const store = getStore();
        store.hooksCleanup.forEach((cleanFn => {
            cleanFn?.();
        })), store.hooksEffect.length = 0, store.hooksCleanup.length = 0;
    }
};

function useState(defaultValue) {
    return withPointer((pointer => {
        const setFn = newValue => {
            pointer.get() !== newValue && (pointer.set(newValue), handleChange());
        };
        if (pointer.initialized) return [ pointer.get(), setFn ];
        const value = "function" == typeof defaultValue ? defaultValue() : defaultValue;
        return pointer.set(value), [ value, setFn ];
    }));
}

function useEffect(cb, depArray) {
    withPointer((pointer => {
        const oldDeps = pointer.get();
        (!Array.isArray(oldDeps) || depArray.some(((dep, i) => !Object.is(dep, oldDeps[i])))) && effectScheduler.queue(cb), 
        pointer.set(depArray);
    }));
}

var yoctocolorsCjs, hasRequiredYoctocolorsCjs;

function requireYoctocolorsCjs() {
    if (hasRequiredYoctocolorsCjs) return yoctocolorsCjs;
    hasRequiredYoctocolorsCjs = 1;
    const tty$1 = tty, hasColors = tty$1?.WriteStream?.prototype?.hasColors?.() ?? !1, format = (open, close) => {
        if (!hasColors) return input => input;
        const openCode = `[${open}m`, closeCode = `[${close}m`;
        return input => {
            const string = input + "";
            let index = string.indexOf(closeCode);
            if (-1 === index) return openCode + string + closeCode;
            let result = openCode, lastIndex = 0;
            for (;-1 !== index; ) result += string.slice(lastIndex, index) + openCode, lastIndex = index + closeCode.length, 
            index = string.indexOf(closeCode, lastIndex);
            return result += string.slice(lastIndex) + closeCode, result;
        };
    }, colors = {};
    return colors.reset = format(0, 0), colors.bold = format(1, 22), colors.dim = format(2, 22), 
    colors.italic = format(3, 23), colors.underline = format(4, 24), colors.overline = format(53, 55), 
    colors.inverse = format(7, 27), colors.hidden = format(8, 28), colors.strikethrough = format(9, 29), 
    colors.black = format(30, 39), colors.red = format(31, 39), colors.green = format(32, 39), 
    colors.yellow = format(33, 39), colors.blue = format(34, 39), colors.magenta = format(35, 39), 
    colors.cyan = format(36, 39), colors.white = format(37, 39), colors.gray = format(90, 39), 
    colors.bgBlack = format(40, 49), colors.bgRed = format(41, 49), colors.bgGreen = format(42, 49), 
    colors.bgYellow = format(43, 49), colors.bgBlue = format(44, 49), colors.bgMagenta = format(45, 49), 
    colors.bgCyan = format(46, 49), colors.bgWhite = format(47, 49), colors.bgGray = format(100, 49), 
    colors.redBright = format(91, 39), colors.greenBright = format(92, 39), colors.yellowBright = format(93, 39), 
    colors.blueBright = format(94, 39), colors.magentaBright = format(95, 39), colors.cyanBright = format(96, 39), 
    colors.whiteBright = format(97, 39), colors.bgRedBright = format(101, 49), colors.bgGreenBright = format(102, 49), 
    colors.bgYellowBright = format(103, 49), colors.bgBlueBright = format(104, 49), 
    colors.bgMagentaBright = format(105, 49), colors.bgCyanBright = format(106, 49), 
    colors.bgWhiteBright = format(107, 49), yoctocolorsCjs = colors;
}

var yoctocolorsCjsExports = requireYoctocolorsCjs(), colors$1 = getDefaultExportFromCjs(yoctocolorsCjsExports);

function isUnicodeSupported() {
    return "win32" !== process$2.platform ? "linux" !== process$2.env.TERM : Boolean(process$2.env.WT_SESSION) || Boolean(process$2.env.TERMINUS_SUBLIME) || "{cmd::Cmder}" === process$2.env.ConEmuTask || "Terminus-Sublime" === process$2.env.TERM_PROGRAM || "vscode" === process$2.env.TERM_PROGRAM || "xterm-256color" === process$2.env.TERM || "alacritty" === process$2.env.TERM || "JetBrains-JediTerm" === process$2.env.TERMINAL_EMULATOR;
}

const common = {
    circleQuestionMark: "(?)",
    questionMarkPrefix: "(?)",
    square: "█",
    squareDarkShade: "▓",
    squareMediumShade: "▒",
    squareLightShade: "░",
    squareTop: "▀",
    squareBottom: "▄",
    squareLeft: "▌",
    squareRight: "▐",
    squareCenter: "■",
    bullet: "●",
    dot: "․",
    ellipsis: "…",
    pointerSmall: "›",
    triangleUp: "▲",
    triangleUpSmall: "▴",
    triangleDown: "▼",
    triangleDownSmall: "▾",
    triangleLeftSmall: "◂",
    triangleRightSmall: "▸",
    home: "⌂",
    heart: "♥",
    musicNote: "♪",
    musicNoteBeamed: "♫",
    arrowUp: "↑",
    arrowDown: "↓",
    arrowLeft: "←",
    arrowRight: "→",
    arrowLeftRight: "↔",
    arrowUpDown: "↕",
    almostEqual: "≈",
    notEqual: "≠",
    lessOrEqual: "≤",
    greaterOrEqual: "≥",
    identical: "≡",
    infinity: "∞",
    subscriptZero: "₀",
    subscriptOne: "₁",
    subscriptTwo: "₂",
    subscriptThree: "₃",
    subscriptFour: "₄",
    subscriptFive: "₅",
    subscriptSix: "₆",
    subscriptSeven: "₇",
    subscriptEight: "₈",
    subscriptNine: "₉",
    oneHalf: "½",
    oneThird: "⅓",
    oneQuarter: "¼",
    oneFifth: "⅕",
    oneSixth: "⅙",
    oneEighth: "⅛",
    twoThirds: "⅔",
    twoFifths: "⅖",
    threeQuarters: "¾",
    threeFifths: "⅗",
    threeEighths: "⅜",
    fourFifths: "⅘",
    fiveSixths: "⅚",
    fiveEighths: "⅝",
    sevenEighths: "⅞",
    line: "─",
    lineBold: "━",
    lineDouble: "═",
    lineDashed0: "┄",
    lineDashed1: "┅",
    lineDashed2: "┈",
    lineDashed3: "┉",
    lineDashed4: "╌",
    lineDashed5: "╍",
    lineDashed6: "╴",
    lineDashed7: "╶",
    lineDashed8: "╸",
    lineDashed9: "╺",
    lineDashed10: "╼",
    lineDashed11: "╾",
    lineDashed12: "−",
    lineDashed13: "–",
    lineDashed14: "‐",
    lineDashed15: "⁃",
    lineVertical: "│",
    lineVerticalBold: "┃",
    lineVerticalDouble: "║",
    lineVerticalDashed0: "┆",
    lineVerticalDashed1: "┇",
    lineVerticalDashed2: "┊",
    lineVerticalDashed3: "┋",
    lineVerticalDashed4: "╎",
    lineVerticalDashed5: "╏",
    lineVerticalDashed6: "╵",
    lineVerticalDashed7: "╷",
    lineVerticalDashed8: "╹",
    lineVerticalDashed9: "╻",
    lineVerticalDashed10: "╽",
    lineVerticalDashed11: "╿",
    lineDownLeft: "┐",
    lineDownLeftArc: "╮",
    lineDownBoldLeftBold: "┓",
    lineDownBoldLeft: "┒",
    lineDownLeftBold: "┑",
    lineDownDoubleLeftDouble: "╗",
    lineDownDoubleLeft: "╖",
    lineDownLeftDouble: "╕",
    lineDownRight: "┌",
    lineDownRightArc: "╭",
    lineDownBoldRightBold: "┏",
    lineDownBoldRight: "┎",
    lineDownRightBold: "┍",
    lineDownDoubleRightDouble: "╔",
    lineDownDoubleRight: "╓",
    lineDownRightDouble: "╒",
    lineUpLeft: "┘",
    lineUpLeftArc: "╯",
    lineUpBoldLeftBold: "┛",
    lineUpBoldLeft: "┚",
    lineUpLeftBold: "┙",
    lineUpDoubleLeftDouble: "╝",
    lineUpDoubleLeft: "╜",
    lineUpLeftDouble: "╛",
    lineUpRight: "└",
    lineUpRightArc: "╰",
    lineUpBoldRightBold: "┗",
    lineUpBoldRight: "┖",
    lineUpRightBold: "┕",
    lineUpDoubleRightDouble: "╚",
    lineUpDoubleRight: "╙",
    lineUpRightDouble: "╘",
    lineUpDownLeft: "┤",
    lineUpBoldDownBoldLeftBold: "┫",
    lineUpBoldDownBoldLeft: "┨",
    lineUpDownLeftBold: "┥",
    lineUpBoldDownLeftBold: "┩",
    lineUpDownBoldLeftBold: "┪",
    lineUpDownBoldLeft: "┧",
    lineUpBoldDownLeft: "┦",
    lineUpDoubleDownDoubleLeftDouble: "╣",
    lineUpDoubleDownDoubleLeft: "╢",
    lineUpDownLeftDouble: "╡",
    lineUpDownRight: "├",
    lineUpBoldDownBoldRightBold: "┣",
    lineUpBoldDownBoldRight: "┠",
    lineUpDownRightBold: "┝",
    lineUpBoldDownRightBold: "┡",
    lineUpDownBoldRightBold: "┢",
    lineUpDownBoldRight: "┟",
    lineUpBoldDownRight: "┞",
    lineUpDoubleDownDoubleRightDouble: "╠",
    lineUpDoubleDownDoubleRight: "╟",
    lineUpDownRightDouble: "╞",
    lineDownLeftRight: "┬",
    lineDownBoldLeftBoldRightBold: "┳",
    lineDownLeftBoldRightBold: "┯",
    lineDownBoldLeftRight: "┰",
    lineDownBoldLeftBoldRight: "┱",
    lineDownBoldLeftRightBold: "┲",
    lineDownLeftRightBold: "┮",
    lineDownLeftBoldRight: "┭",
    lineDownDoubleLeftDoubleRightDouble: "╦",
    lineDownDoubleLeftRight: "╥",
    lineDownLeftDoubleRightDouble: "╤",
    lineUpLeftRight: "┴",
    lineUpBoldLeftBoldRightBold: "┻",
    lineUpLeftBoldRightBold: "┷",
    lineUpBoldLeftRight: "┸",
    lineUpBoldLeftBoldRight: "┹",
    lineUpBoldLeftRightBold: "┺",
    lineUpLeftRightBold: "┶",
    lineUpLeftBoldRight: "┵",
    lineUpDoubleLeftDoubleRightDouble: "╩",
    lineUpDoubleLeftRight: "╨",
    lineUpLeftDoubleRightDouble: "╧",
    lineUpDownLeftRight: "┼",
    lineUpBoldDownBoldLeftBoldRightBold: "╋",
    lineUpDownBoldLeftBoldRightBold: "╈",
    lineUpBoldDownLeftBoldRightBold: "╇",
    lineUpBoldDownBoldLeftRightBold: "╊",
    lineUpBoldDownBoldLeftBoldRight: "╉",
    lineUpBoldDownLeftRight: "╀",
    lineUpDownBoldLeftRight: "╁",
    lineUpDownLeftBoldRight: "┽",
    lineUpDownLeftRightBold: "┾",
    lineUpBoldDownBoldLeftRight: "╂",
    lineUpDownLeftBoldRightBold: "┿",
    lineUpBoldDownLeftBoldRight: "╃",
    lineUpBoldDownLeftRightBold: "╄",
    lineUpDownBoldLeftBoldRight: "╅",
    lineUpDownBoldLeftRightBold: "╆",
    lineUpDoubleDownDoubleLeftDoubleRightDouble: "╬",
    lineUpDoubleDownDoubleLeftRight: "╫",
    lineUpDownLeftDoubleRightDouble: "╪",
    lineCross: "╳",
    lineBackslash: "╲",
    lineSlash: "╱"
}, specialMainSymbols = {
    tick: "✔",
    info: "ℹ",
    warning: "⚠",
    cross: "✘",
    squareSmall: "◻",
    squareSmallFilled: "◼",
    circle: "◯",
    circleFilled: "◉",
    circleDotted: "◌",
    circleDouble: "◎",
    circleCircle: "ⓞ",
    circleCross: "ⓧ",
    circlePipe: "Ⓘ",
    radioOn: "◉",
    radioOff: "◯",
    checkboxOn: "☒",
    checkboxOff: "☐",
    checkboxCircleOn: "ⓧ",
    checkboxCircleOff: "Ⓘ",
    pointer: "❯",
    triangleUpOutline: "△",
    triangleLeft: "◀",
    triangleRight: "▶",
    lozenge: "◆",
    lozengeOutline: "◇",
    hamburger: "☰",
    smiley: "㋡",
    mustache: "෴",
    star: "★",
    play: "▶",
    nodejs: "⬢",
    oneSeventh: "⅐",
    oneNinth: "⅑",
    oneTenth: "⅒"
}, specialFallbackSymbols = {
    tick: "√",
    info: "i",
    warning: "‼",
    cross: "×",
    squareSmall: "□",
    squareSmallFilled: "■",
    circle: "( )",
    circleFilled: "(*)",
    circleDotted: "( )",
    circleDouble: "( )",
    circleCircle: "(○)",
    circleCross: "(×)",
    circlePipe: "(│)",
    radioOn: "(*)",
    radioOff: "( )",
    checkboxOn: "[×]",
    checkboxOff: "[ ]",
    checkboxCircleOn: "(×)",
    checkboxCircleOff: "( )",
    pointer: ">",
    triangleUpOutline: "∆",
    triangleLeft: "◄",
    triangleRight: "►",
    lozenge: "♦",
    lozengeOutline: "◊",
    hamburger: "≡",
    smiley: "☺",
    mustache: "┌─┐",
    star: "✶",
    play: "►",
    nodejs: "♦",
    oneSeventh: "1/7",
    oneNinth: "1/9",
    oneTenth: "1/10"
}, mainSymbols = {
    ...common,
    ...specialMainSymbols
}, fallbackSymbols = {
    ...common,
    ...specialFallbackSymbols
}, shouldUseMain = isUnicodeSupported(), figures = shouldUseMain ? mainSymbols : fallbackSymbols, defaultTheme = {
    prefix: {
        idle: colors$1.blue("?"),
        done: colors$1.green(figures.tick)
    },
    spinner: {
        interval: 80,
        frames: [ "⠋", "⠙", "⠹", "⠸", "⠼", "⠴", "⠦", "⠧", "⠇", "⠏" ].map((frame => colors$1.yellow(frame)))
    },
    style: {
        answer: colors$1.cyan,
        message: colors$1.bold,
        error: text => colors$1.red(`> ${text}`),
        defaultAnswer: text => colors$1.dim(`(${text})`),
        help: colors$1.dim,
        highlight: colors$1.cyan,
        key: text => colors$1.cyan(colors$1.bold(`<${text}>`))
    }
};

function isPlainObject(value) {
    if ("object" != typeof value || null === value) return !1;
    let proto = value;
    for (;null !== Object.getPrototypeOf(proto); ) proto = Object.getPrototypeOf(proto);
    return Object.getPrototypeOf(value) === proto;
}

function deepMerge(...objects) {
    const output = {};
    for (const obj of objects) for (const [key, value] of Object.entries(obj)) {
        const prevValue = output[key];
        output[key] = isPlainObject(prevValue) && isPlainObject(value) ? deepMerge(prevValue, value) : value;
    }
    return output;
}

function makeTheme(...themes) {
    return deepMerge(...[ defaultTheme, ...themes.filter((theme => null != theme)) ]);
}

function usePrefix({status: status = "idle", theme: theme}) {
    const [showLoader, setShowLoader] = useState(!1), [tick, setTick] = useState(0), {prefix: prefix, spinner: spinner} = makeTheme(theme);
    if (useEffect((() => {
        if ("loading" === status) {
            let tickInterval, inc = -1;
            const delayTimeout = setTimeout(AsyncResource.bind((() => {
                setShowLoader(!0), tickInterval = setInterval(AsyncResource.bind((() => {
                    inc += 1, setTick(inc % spinner.frames.length);
                })), spinner.interval);
            })), 300);
            return () => {
                clearTimeout(delayTimeout), clearInterval(tickInterval);
            };
        }
        setShowLoader(!1);
    }), [ status ]), showLoader) return spinner.frames[tick];
    return "string" == typeof prefix ? prefix : prefix["loading" === status ? "idle" : status];
}

function useMemo(fn, dependencies) {
    return withPointer((pointer => {
        const prev = pointer.get();
        if (!prev || prev.dependencies.length !== dependencies.length || prev.dependencies.some(((dep, i) => dep !== dependencies[i]))) {
            const value = fn();
            return pointer.set({
                value: value,
                dependencies: dependencies
            }), value;
        }
        return prev.value;
    }));
}

function useRef(val) {
    return useState({
        current: val
    })[0];
}

function useKeypress(userHandler) {
    const signal = useRef(userHandler);
    signal.current = userHandler, useEffect((rl => {
        let ignore = !1;
        const handler = withUpdates(((_input, event) => {
            ignore || signal.current(event, rl);
        }));
        return rl.input.on("keypress", handler), () => {
            ignore = !0, rl.input.removeListener("keypress", handler);
        };
    }), []);
}

var cliWidth_1, hasRequiredCliWidth;

function requireCliWidth() {
    if (hasRequiredCliWidth) return cliWidth_1;
    return hasRequiredCliWidth = 1, cliWidth_1 = function(options) {
        const opts = function(options) {
            const defaultOpts = {
                defaultWidth: 0,
                output: process.stdout,
                tty: require$$0$3
            };
            if (!options) return defaultOpts;
            return Object.keys(defaultOpts).forEach((function(key) {
                options[key] || (options[key] = defaultOpts[key]);
            })), options;
        }(options);
        if (opts.output.getWindowSize) return opts.output.getWindowSize()[0] || opts.defaultWidth;
        if (opts.tty.getWindowSize) return opts.tty.getWindowSize()[1] || opts.defaultWidth;
        if (opts.output.columns) return opts.output.columns;
        if (process.env.CLI_WIDTH) {
            const width = parseInt(process.env.CLI_WIDTH, 10);
            if (!isNaN(width) && 0 !== width) return width;
        }
        return opts.defaultWidth;
    };
}

var cliWidthExports = requireCliWidth(), cliWidth = getDefaultExportFromCjs(cliWidthExports), stringWidth = {
    exports: {}
}, ansiRegex, hasRequiredAnsiRegex, stripAnsi$1, hasRequiredStripAnsi;

function requireAnsiRegex() {
    return hasRequiredAnsiRegex ? ansiRegex : (hasRequiredAnsiRegex = 1, ansiRegex = ({onlyFirst: onlyFirst = !1} = {}) => {
        const pattern = [ "[\\u001B\\u009B][[\\]()#;?]*(?:(?:(?:(?:;[-a-zA-Z\\d\\/#&.:=?%@~_]+)*|[a-zA-Z\\d]+(?:;[-a-zA-Z\\d\\/#&.:=?%@~_]*)*)?\\u0007)", "(?:(?:\\d{1,4}(?:;\\d{0,4})*)?[\\dA-PR-TZcf-ntqry=><~]))" ].join("|");
        return new RegExp(pattern, onlyFirst ? void 0 : "g");
    });
}

function requireStripAnsi() {
    if (hasRequiredStripAnsi) return stripAnsi$1;
    hasRequiredStripAnsi = 1;
    const ansiRegex = requireAnsiRegex();
    return stripAnsi$1 = string => "string" == typeof string ? string.replace(ansiRegex(), "") : string, 
    stripAnsi$1;
}

var isFullwidthCodePoint = {
    exports: {}
}, hasRequiredIsFullwidthCodePoint, emojiRegex, hasRequiredEmojiRegex, hasRequiredStringWidth;

function requireIsFullwidthCodePoint() {
    if (hasRequiredIsFullwidthCodePoint) return isFullwidthCodePoint.exports;
    hasRequiredIsFullwidthCodePoint = 1;
    const isFullwidthCodePoint$1 = codePoint => !Number.isNaN(codePoint) && (codePoint >= 4352 && (codePoint <= 4447 || 9001 === codePoint || 9002 === codePoint || 11904 <= codePoint && codePoint <= 12871 && 12351 !== codePoint || 12880 <= codePoint && codePoint <= 19903 || 19968 <= codePoint && codePoint <= 42182 || 43360 <= codePoint && codePoint <= 43388 || 44032 <= codePoint && codePoint <= 55203 || 63744 <= codePoint && codePoint <= 64255 || 65040 <= codePoint && codePoint <= 65049 || 65072 <= codePoint && codePoint <= 65131 || 65281 <= codePoint && codePoint <= 65376 || 65504 <= codePoint && codePoint <= 65510 || 110592 <= codePoint && codePoint <= 110593 || 127488 <= codePoint && codePoint <= 127569 || 131072 <= codePoint && codePoint <= 262141));
    return isFullwidthCodePoint.exports = isFullwidthCodePoint$1, isFullwidthCodePoint.exports.default = isFullwidthCodePoint$1, 
    isFullwidthCodePoint.exports;
}

function requireEmojiRegex() {
    return hasRequiredEmojiRegex ? emojiRegex : (hasRequiredEmojiRegex = 1, emojiRegex = function() {
        return /\uD83C\uDFF4\uDB40\uDC67\uDB40\uDC62(?:\uDB40\uDC65\uDB40\uDC6E\uDB40\uDC67|\uDB40\uDC73\uDB40\uDC63\uDB40\uDC74|\uDB40\uDC77\uDB40\uDC6C\uDB40\uDC73)\uDB40\uDC7F|\uD83D\uDC68(?:\uD83C\uDFFC\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68\uD83C\uDFFB|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFF\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB-\uDFFE])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFE\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB-\uDFFD])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFD\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB\uDFFC])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\u200D(?:\u2764\uFE0F\u200D(?:\uD83D\uDC8B\u200D)?\uD83D\uDC68|(?:\uD83D[\uDC68\uDC69])\u200D(?:\uD83D\uDC66\u200D\uD83D\uDC66|\uD83D\uDC67\u200D(?:\uD83D[\uDC66\uDC67]))|\uD83D\uDC66\u200D\uD83D\uDC66|\uD83D\uDC67\u200D(?:\uD83D[\uDC66\uDC67])|(?:\uD83D[\uDC68\uDC69])\u200D(?:\uD83D[\uDC66\uDC67])|[\u2695\u2696\u2708]\uFE0F|\uD83D[\uDC66\uDC67]|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|(?:\uD83C\uDFFB\u200D[\u2695\u2696\u2708]|\uD83C\uDFFF\u200D[\u2695\u2696\u2708]|\uD83C\uDFFE\u200D[\u2695\u2696\u2708]|\uD83C\uDFFD\u200D[\u2695\u2696\u2708]|\uD83C\uDFFC\u200D[\u2695\u2696\u2708])\uFE0F|\uD83C\uDFFB\u200D(?:\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C[\uDFFB-\uDFFF])|(?:\uD83E\uDDD1\uD83C\uDFFB\u200D\uD83E\uDD1D\u200D\uD83E\uDDD1|\uD83D\uDC69\uD83C\uDFFC\u200D\uD83E\uDD1D\u200D\uD83D\uDC69)\uD83C\uDFFB|\uD83E\uDDD1(?:\uD83C\uDFFF\u200D\uD83E\uDD1D\u200D\uD83E\uDDD1(?:\uD83C[\uDFFB-\uDFFF])|\u200D\uD83E\uDD1D\u200D\uD83E\uDDD1)|(?:\uD83E\uDDD1\uD83C\uDFFE\u200D\uD83E\uDD1D\u200D\uD83E\uDDD1|\uD83D\uDC69\uD83C\uDFFF\u200D\uD83E\uDD1D\u200D(?:\uD83D[\uDC68\uDC69]))(?:\uD83C[\uDFFB-\uDFFE])|(?:\uD83E\uDDD1\uD83C\uDFFC\u200D\uD83E\uDD1D\u200D\uD83E\uDDD1|\uD83D\uDC69\uD83C\uDFFD\u200D\uD83E\uDD1D\u200D\uD83D\uDC69)(?:\uD83C[\uDFFB\uDFFC])|\uD83D\uDC69(?:\uD83C\uDFFE\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB-\uDFFD\uDFFF])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFC\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB\uDFFD-\uDFFF])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFB\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFC-\uDFFF])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFD\u200D(?:\uD83E\uDD1D\u200D\uD83D\uDC68(?:\uD83C[\uDFFB\uDFFC\uDFFE\uDFFF])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\u200D(?:\u2764\uFE0F\u200D(?:\uD83D\uDC8B\u200D(?:\uD83D[\uDC68\uDC69])|\uD83D[\uDC68\uDC69])|\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD])|\uD83C\uDFFF\u200D(?:\uD83C[\uDF3E\uDF73\uDF93\uDFA4\uDFA8\uDFEB\uDFED]|\uD83D[\uDCBB\uDCBC\uDD27\uDD2C\uDE80\uDE92]|\uD83E[\uDDAF-\uDDB3\uDDBC\uDDBD]))|\uD83D\uDC69\u200D\uD83D\uDC69\u200D(?:\uD83D\uDC66\u200D\uD83D\uDC66|\uD83D\uDC67\u200D(?:\uD83D[\uDC66\uDC67]))|(?:\uD83E\uDDD1\uD83C\uDFFD\u200D\uD83E\uDD1D\u200D\uD83E\uDDD1|\uD83D\uDC69\uD83C\uDFFE\u200D\uD83E\uDD1D\u200D\uD83D\uDC69)(?:\uD83C[\uDFFB-\uDFFD])|\uD83D\uDC69\u200D\uD83D\uDC66\u200D\uD83D\uDC66|\uD83D\uDC69\u200D\uD83D\uDC69\u200D(?:\uD83D[\uDC66\uDC67])|(?:\uD83D\uDC41\uFE0F\u200D\uD83D\uDDE8|\uD83D\uDC69(?:\uD83C\uDFFF\u200D[\u2695\u2696\u2708]|\uD83C\uDFFE\u200D[\u2695\u2696\u2708]|\uD83C\uDFFC\u200D[\u2695\u2696\u2708]|\uD83C\uDFFB\u200D[\u2695\u2696\u2708]|\uD83C\uDFFD\u200D[\u2695\u2696\u2708]|\u200D[\u2695\u2696\u2708])|(?:(?:\u26F9|\uD83C[\uDFCB\uDFCC]|\uD83D\uDD75)\uFE0F|\uD83D\uDC6F|\uD83E[\uDD3C\uDDDE\uDDDF])\u200D[\u2640\u2642]|(?:\u26F9|\uD83C[\uDFCB\uDFCC]|\uD83D\uDD75)(?:\uD83C[\uDFFB-\uDFFF])\u200D[\u2640\u2642]|(?:\uD83C[\uDFC3\uDFC4\uDFCA]|\uD83D[\uDC6E\uDC71\uDC73\uDC77\uDC81\uDC82\uDC86\uDC87\uDE45-\uDE47\uDE4B\uDE4D\uDE4E\uDEA3\uDEB4-\uDEB6]|\uD83E[\uDD26\uDD37-\uDD39\uDD3D\uDD3E\uDDB8\uDDB9\uDDCD-\uDDCF\uDDD6-\uDDDD])(?:(?:\uD83C[\uDFFB-\uDFFF])\u200D[\u2640\u2642]|\u200D[\u2640\u2642])|\uD83C\uDFF4\u200D\u2620)\uFE0F|\uD83D\uDC69\u200D\uD83D\uDC67\u200D(?:\uD83D[\uDC66\uDC67])|\uD83C\uDFF3\uFE0F\u200D\uD83C\uDF08|\uD83D\uDC15\u200D\uD83E\uDDBA|\uD83D\uDC69\u200D\uD83D\uDC66|\uD83D\uDC69\u200D\uD83D\uDC67|\uD83C\uDDFD\uD83C\uDDF0|\uD83C\uDDF4\uD83C\uDDF2|\uD83C\uDDF6\uD83C\uDDE6|[#\*0-9]\uFE0F\u20E3|\uD83C\uDDE7(?:\uD83C[\uDDE6\uDDE7\uDDE9-\uDDEF\uDDF1-\uDDF4\uDDF6-\uDDF9\uDDFB\uDDFC\uDDFE\uDDFF])|\uD83C\uDDF9(?:\uD83C[\uDDE6\uDDE8\uDDE9\uDDEB-\uDDED\uDDEF-\uDDF4\uDDF7\uDDF9\uDDFB\uDDFC\uDDFF])|\uD83C\uDDEA(?:\uD83C[\uDDE6\uDDE8\uDDEA\uDDEC\uDDED\uDDF7-\uDDFA])|\uD83E\uDDD1(?:\uD83C[\uDFFB-\uDFFF])|\uD83C\uDDF7(?:\uD83C[\uDDEA\uDDF4\uDDF8\uDDFA\uDDFC])|\uD83D\uDC69(?:\uD83C[\uDFFB-\uDFFF])|\uD83C\uDDF2(?:\uD83C[\uDDE6\uDDE8-\uDDED\uDDF0-\uDDFF])|\uD83C\uDDE6(?:\uD83C[\uDDE8-\uDDEC\uDDEE\uDDF1\uDDF2\uDDF4\uDDF6-\uDDFA\uDDFC\uDDFD\uDDFF])|\uD83C\uDDF0(?:\uD83C[\uDDEA\uDDEC-\uDDEE\uDDF2\uDDF3\uDDF5\uDDF7\uDDFC\uDDFE\uDDFF])|\uD83C\uDDED(?:\uD83C[\uDDF0\uDDF2\uDDF3\uDDF7\uDDF9\uDDFA])|\uD83C\uDDE9(?:\uD83C[\uDDEA\uDDEC\uDDEF\uDDF0\uDDF2\uDDF4\uDDFF])|\uD83C\uDDFE(?:\uD83C[\uDDEA\uDDF9])|\uD83C\uDDEC(?:\uD83C[\uDDE6\uDDE7\uDDE9-\uDDEE\uDDF1-\uDDF3\uDDF5-\uDDFA\uDDFC\uDDFE])|\uD83C\uDDF8(?:\uD83C[\uDDE6-\uDDEA\uDDEC-\uDDF4\uDDF7-\uDDF9\uDDFB\uDDFD-\uDDFF])|\uD83C\uDDEB(?:\uD83C[\uDDEE-\uDDF0\uDDF2\uDDF4\uDDF7])|\uD83C\uDDF5(?:\uD83C[\uDDE6\uDDEA-\uDDED\uDDF0-\uDDF3\uDDF7-\uDDF9\uDDFC\uDDFE])|\uD83C\uDDFB(?:\uD83C[\uDDE6\uDDE8\uDDEA\uDDEC\uDDEE\uDDF3\uDDFA])|\uD83C\uDDF3(?:\uD83C[\uDDE6\uDDE8\uDDEA-\uDDEC\uDDEE\uDDF1\uDDF4\uDDF5\uDDF7\uDDFA\uDDFF])|\uD83C\uDDE8(?:\uD83C[\uDDE6\uDDE8\uDDE9\uDDEB-\uDDEE\uDDF0-\uDDF5\uDDF7\uDDFA-\uDDFF])|\uD83C\uDDF1(?:\uD83C[\uDDE6-\uDDE8\uDDEE\uDDF0\uDDF7-\uDDFB\uDDFE])|\uD83C\uDDFF(?:\uD83C[\uDDE6\uDDF2\uDDFC])|\uD83C\uDDFC(?:\uD83C[\uDDEB\uDDF8])|\uD83C\uDDFA(?:\uD83C[\uDDE6\uDDEC\uDDF2\uDDF3\uDDF8\uDDFE\uDDFF])|\uD83C\uDDEE(?:\uD83C[\uDDE8-\uDDEA\uDDF1-\uDDF4\uDDF6-\uDDF9])|\uD83C\uDDEF(?:\uD83C[\uDDEA\uDDF2\uDDF4\uDDF5])|(?:\uD83C[\uDFC3\uDFC4\uDFCA]|\uD83D[\uDC6E\uDC71\uDC73\uDC77\uDC81\uDC82\uDC86\uDC87\uDE45-\uDE47\uDE4B\uDE4D\uDE4E\uDEA3\uDEB4-\uDEB6]|\uD83E[\uDD26\uDD37-\uDD39\uDD3D\uDD3E\uDDB8\uDDB9\uDDCD-\uDDCF\uDDD6-\uDDDD])(?:\uD83C[\uDFFB-\uDFFF])|(?:\u26F9|\uD83C[\uDFCB\uDFCC]|\uD83D\uDD75)(?:\uD83C[\uDFFB-\uDFFF])|(?:[\u261D\u270A-\u270D]|\uD83C[\uDF85\uDFC2\uDFC7]|\uD83D[\uDC42\uDC43\uDC46-\uDC50\uDC66\uDC67\uDC6B-\uDC6D\uDC70\uDC72\uDC74-\uDC76\uDC78\uDC7C\uDC83\uDC85\uDCAA\uDD74\uDD7A\uDD90\uDD95\uDD96\uDE4C\uDE4F\uDEC0\uDECC]|\uD83E[\uDD0F\uDD18-\uDD1C\uDD1E\uDD1F\uDD30-\uDD36\uDDB5\uDDB6\uDDBB\uDDD2-\uDDD5])(?:\uD83C[\uDFFB-\uDFFF])|(?:[\u231A\u231B\u23E9-\u23EC\u23F0\u23F3\u25FD\u25FE\u2614\u2615\u2648-\u2653\u267F\u2693\u26A1\u26AA\u26AB\u26BD\u26BE\u26C4\u26C5\u26CE\u26D4\u26EA\u26F2\u26F3\u26F5\u26FA\u26FD\u2705\u270A\u270B\u2728\u274C\u274E\u2753-\u2755\u2757\u2795-\u2797\u27B0\u27BF\u2B1B\u2B1C\u2B50\u2B55]|\uD83C[\uDC04\uDCCF\uDD8E\uDD91-\uDD9A\uDDE6-\uDDFF\uDE01\uDE1A\uDE2F\uDE32-\uDE36\uDE38-\uDE3A\uDE50\uDE51\uDF00-\uDF20\uDF2D-\uDF35\uDF37-\uDF7C\uDF7E-\uDF93\uDFA0-\uDFCA\uDFCF-\uDFD3\uDFE0-\uDFF0\uDFF4\uDFF8-\uDFFF]|\uD83D[\uDC00-\uDC3E\uDC40\uDC42-\uDCFC\uDCFF-\uDD3D\uDD4B-\uDD4E\uDD50-\uDD67\uDD7A\uDD95\uDD96\uDDA4\uDDFB-\uDE4F\uDE80-\uDEC5\uDECC\uDED0-\uDED2\uDED5\uDEEB\uDEEC\uDEF4-\uDEFA\uDFE0-\uDFEB]|\uD83E[\uDD0D-\uDD3A\uDD3C-\uDD45\uDD47-\uDD71\uDD73-\uDD76\uDD7A-\uDDA2\uDDA5-\uDDAA\uDDAE-\uDDCA\uDDCD-\uDDFF\uDE70-\uDE73\uDE78-\uDE7A\uDE80-\uDE82\uDE90-\uDE95])|(?:[#\*0-9\xA9\xAE\u203C\u2049\u2122\u2139\u2194-\u2199\u21A9\u21AA\u231A\u231B\u2328\u23CF\u23E9-\u23F3\u23F8-\u23FA\u24C2\u25AA\u25AB\u25B6\u25C0\u25FB-\u25FE\u2600-\u2604\u260E\u2611\u2614\u2615\u2618\u261D\u2620\u2622\u2623\u2626\u262A\u262E\u262F\u2638-\u263A\u2640\u2642\u2648-\u2653\u265F\u2660\u2663\u2665\u2666\u2668\u267B\u267E\u267F\u2692-\u2697\u2699\u269B\u269C\u26A0\u26A1\u26AA\u26AB\u26B0\u26B1\u26BD\u26BE\u26C4\u26C5\u26C8\u26CE\u26CF\u26D1\u26D3\u26D4\u26E9\u26EA\u26F0-\u26F5\u26F7-\u26FA\u26FD\u2702\u2705\u2708-\u270D\u270F\u2712\u2714\u2716\u271D\u2721\u2728\u2733\u2734\u2744\u2747\u274C\u274E\u2753-\u2755\u2757\u2763\u2764\u2795-\u2797\u27A1\u27B0\u27BF\u2934\u2935\u2B05-\u2B07\u2B1B\u2B1C\u2B50\u2B55\u3030\u303D\u3297\u3299]|\uD83C[\uDC04\uDCCF\uDD70\uDD71\uDD7E\uDD7F\uDD8E\uDD91-\uDD9A\uDDE6-\uDDFF\uDE01\uDE02\uDE1A\uDE2F\uDE32-\uDE3A\uDE50\uDE51\uDF00-\uDF21\uDF24-\uDF93\uDF96\uDF97\uDF99-\uDF9B\uDF9E-\uDFF0\uDFF3-\uDFF5\uDFF7-\uDFFF]|\uD83D[\uDC00-\uDCFD\uDCFF-\uDD3D\uDD49-\uDD4E\uDD50-\uDD67\uDD6F\uDD70\uDD73-\uDD7A\uDD87\uDD8A-\uDD8D\uDD90\uDD95\uDD96\uDDA4\uDDA5\uDDA8\uDDB1\uDDB2\uDDBC\uDDC2-\uDDC4\uDDD1-\uDDD3\uDDDC-\uDDDE\uDDE1\uDDE3\uDDE8\uDDEF\uDDF3\uDDFA-\uDE4F\uDE80-\uDEC5\uDECB-\uDED2\uDED5\uDEE0-\uDEE5\uDEE9\uDEEB\uDEEC\uDEF0\uDEF3-\uDEFA\uDFE0-\uDFEB]|\uD83E[\uDD0D-\uDD3A\uDD3C-\uDD45\uDD47-\uDD71\uDD73-\uDD76\uDD7A-\uDDA2\uDDA5-\uDDAA\uDDAE-\uDDCA\uDDCD-\uDDFF\uDE70-\uDE73\uDE78-\uDE7A\uDE80-\uDE82\uDE90-\uDE95])\uFE0F|(?:[\u261D\u26F9\u270A-\u270D]|\uD83C[\uDF85\uDFC2-\uDFC4\uDFC7\uDFCA-\uDFCC]|\uD83D[\uDC42\uDC43\uDC46-\uDC50\uDC66-\uDC78\uDC7C\uDC81-\uDC83\uDC85-\uDC87\uDC8F\uDC91\uDCAA\uDD74\uDD75\uDD7A\uDD90\uDD95\uDD96\uDE45-\uDE47\uDE4B-\uDE4F\uDEA3\uDEB4-\uDEB6\uDEC0\uDECC]|\uD83E[\uDD0F\uDD18-\uDD1F\uDD26\uDD30-\uDD39\uDD3C-\uDD3E\uDDB5\uDDB6\uDDB8\uDDB9\uDDBB\uDDCD-\uDDCF\uDDD1-\uDDDD])/g;
    });
}

function requireStringWidth() {
    if (hasRequiredStringWidth) return stringWidth.exports;
    hasRequiredStringWidth = 1;
    const stripAnsi = requireStripAnsi(), isFullwidthCodePoint = requireIsFullwidthCodePoint(), emojiRegex = requireEmojiRegex(), stringWidth$1 = string => {
        if ("string" != typeof string || 0 === string.length) return 0;
        if (0 === (string = stripAnsi(string)).length) return 0;
        string = string.replace(emojiRegex(), "  ");
        let width = 0;
        for (let i = 0; i < string.length; i++) {
            const code = string.codePointAt(i);
            code <= 31 || code >= 127 && code <= 159 || (code >= 768 && code <= 879 || (code > 65535 && i++, 
            width += isFullwidthCodePoint(code) ? 2 : 1));
        }
        return width;
    };
    return stringWidth.exports = stringWidth$1, stringWidth.exports.default = stringWidth$1, 
    stringWidth.exports;
}

var ansiStyles = {
    exports: {}
}, colorName, hasRequiredColorName, conversions, hasRequiredConversions, route, hasRequiredRoute, colorConvert, hasRequiredColorConvert, hasRequiredAnsiStyles, wrapAnsi_1, hasRequiredWrapAnsi;

function requireColorName() {
    return hasRequiredColorName ? colorName : (hasRequiredColorName = 1, colorName = {
        aliceblue: [ 240, 248, 255 ],
        antiquewhite: [ 250, 235, 215 ],
        aqua: [ 0, 255, 255 ],
        aquamarine: [ 127, 255, 212 ],
        azure: [ 240, 255, 255 ],
        beige: [ 245, 245, 220 ],
        bisque: [ 255, 228, 196 ],
        black: [ 0, 0, 0 ],
        blanchedalmond: [ 255, 235, 205 ],
        blue: [ 0, 0, 255 ],
        blueviolet: [ 138, 43, 226 ],
        brown: [ 165, 42, 42 ],
        burlywood: [ 222, 184, 135 ],
        cadetblue: [ 95, 158, 160 ],
        chartreuse: [ 127, 255, 0 ],
        chocolate: [ 210, 105, 30 ],
        coral: [ 255, 127, 80 ],
        cornflowerblue: [ 100, 149, 237 ],
        cornsilk: [ 255, 248, 220 ],
        crimson: [ 220, 20, 60 ],
        cyan: [ 0, 255, 255 ],
        darkblue: [ 0, 0, 139 ],
        darkcyan: [ 0, 139, 139 ],
        darkgoldenrod: [ 184, 134, 11 ],
        darkgray: [ 169, 169, 169 ],
        darkgreen: [ 0, 100, 0 ],
        darkgrey: [ 169, 169, 169 ],
        darkkhaki: [ 189, 183, 107 ],
        darkmagenta: [ 139, 0, 139 ],
        darkolivegreen: [ 85, 107, 47 ],
        darkorange: [ 255, 140, 0 ],
        darkorchid: [ 153, 50, 204 ],
        darkred: [ 139, 0, 0 ],
        darksalmon: [ 233, 150, 122 ],
        darkseagreen: [ 143, 188, 143 ],
        darkslateblue: [ 72, 61, 139 ],
        darkslategray: [ 47, 79, 79 ],
        darkslategrey: [ 47, 79, 79 ],
        darkturquoise: [ 0, 206, 209 ],
        darkviolet: [ 148, 0, 211 ],
        deeppink: [ 255, 20, 147 ],
        deepskyblue: [ 0, 191, 255 ],
        dimgray: [ 105, 105, 105 ],
        dimgrey: [ 105, 105, 105 ],
        dodgerblue: [ 30, 144, 255 ],
        firebrick: [ 178, 34, 34 ],
        floralwhite: [ 255, 250, 240 ],
        forestgreen: [ 34, 139, 34 ],
        fuchsia: [ 255, 0, 255 ],
        gainsboro: [ 220, 220, 220 ],
        ghostwhite: [ 248, 248, 255 ],
        gold: [ 255, 215, 0 ],
        goldenrod: [ 218, 165, 32 ],
        gray: [ 128, 128, 128 ],
        green: [ 0, 128, 0 ],
        greenyellow: [ 173, 255, 47 ],
        grey: [ 128, 128, 128 ],
        honeydew: [ 240, 255, 240 ],
        hotpink: [ 255, 105, 180 ],
        indianred: [ 205, 92, 92 ],
        indigo: [ 75, 0, 130 ],
        ivory: [ 255, 255, 240 ],
        khaki: [ 240, 230, 140 ],
        lavender: [ 230, 230, 250 ],
        lavenderblush: [ 255, 240, 245 ],
        lawngreen: [ 124, 252, 0 ],
        lemonchiffon: [ 255, 250, 205 ],
        lightblue: [ 173, 216, 230 ],
        lightcoral: [ 240, 128, 128 ],
        lightcyan: [ 224, 255, 255 ],
        lightgoldenrodyellow: [ 250, 250, 210 ],
        lightgray: [ 211, 211, 211 ],
        lightgreen: [ 144, 238, 144 ],
        lightgrey: [ 211, 211, 211 ],
        lightpink: [ 255, 182, 193 ],
        lightsalmon: [ 255, 160, 122 ],
        lightseagreen: [ 32, 178, 170 ],
        lightskyblue: [ 135, 206, 250 ],
        lightslategray: [ 119, 136, 153 ],
        lightslategrey: [ 119, 136, 153 ],
        lightsteelblue: [ 176, 196, 222 ],
        lightyellow: [ 255, 255, 224 ],
        lime: [ 0, 255, 0 ],
        limegreen: [ 50, 205, 50 ],
        linen: [ 250, 240, 230 ],
        magenta: [ 255, 0, 255 ],
        maroon: [ 128, 0, 0 ],
        mediumaquamarine: [ 102, 205, 170 ],
        mediumblue: [ 0, 0, 205 ],
        mediumorchid: [ 186, 85, 211 ],
        mediumpurple: [ 147, 112, 219 ],
        mediumseagreen: [ 60, 179, 113 ],
        mediumslateblue: [ 123, 104, 238 ],
        mediumspringgreen: [ 0, 250, 154 ],
        mediumturquoise: [ 72, 209, 204 ],
        mediumvioletred: [ 199, 21, 133 ],
        midnightblue: [ 25, 25, 112 ],
        mintcream: [ 245, 255, 250 ],
        mistyrose: [ 255, 228, 225 ],
        moccasin: [ 255, 228, 181 ],
        navajowhite: [ 255, 222, 173 ],
        navy: [ 0, 0, 128 ],
        oldlace: [ 253, 245, 230 ],
        olive: [ 128, 128, 0 ],
        olivedrab: [ 107, 142, 35 ],
        orange: [ 255, 165, 0 ],
        orangered: [ 255, 69, 0 ],
        orchid: [ 218, 112, 214 ],
        palegoldenrod: [ 238, 232, 170 ],
        palegreen: [ 152, 251, 152 ],
        paleturquoise: [ 175, 238, 238 ],
        palevioletred: [ 219, 112, 147 ],
        papayawhip: [ 255, 239, 213 ],
        peachpuff: [ 255, 218, 185 ],
        peru: [ 205, 133, 63 ],
        pink: [ 255, 192, 203 ],
        plum: [ 221, 160, 221 ],
        powderblue: [ 176, 224, 230 ],
        purple: [ 128, 0, 128 ],
        rebeccapurple: [ 102, 51, 153 ],
        red: [ 255, 0, 0 ],
        rosybrown: [ 188, 143, 143 ],
        royalblue: [ 65, 105, 225 ],
        saddlebrown: [ 139, 69, 19 ],
        salmon: [ 250, 128, 114 ],
        sandybrown: [ 244, 164, 96 ],
        seagreen: [ 46, 139, 87 ],
        seashell: [ 255, 245, 238 ],
        sienna: [ 160, 82, 45 ],
        silver: [ 192, 192, 192 ],
        skyblue: [ 135, 206, 235 ],
        slateblue: [ 106, 90, 205 ],
        slategray: [ 112, 128, 144 ],
        slategrey: [ 112, 128, 144 ],
        snow: [ 255, 250, 250 ],
        springgreen: [ 0, 255, 127 ],
        steelblue: [ 70, 130, 180 ],
        tan: [ 210, 180, 140 ],
        teal: [ 0, 128, 128 ],
        thistle: [ 216, 191, 216 ],
        tomato: [ 255, 99, 71 ],
        turquoise: [ 64, 224, 208 ],
        violet: [ 238, 130, 238 ],
        wheat: [ 245, 222, 179 ],
        white: [ 255, 255, 255 ],
        whitesmoke: [ 245, 245, 245 ],
        yellow: [ 255, 255, 0 ],
        yellowgreen: [ 154, 205, 50 ]
    });
}

function requireConversions() {
    if (hasRequiredConversions) return conversions;
    hasRequiredConversions = 1;
    const cssKeywords = requireColorName(), reverseKeywords = {};
    for (const key of Object.keys(cssKeywords)) reverseKeywords[cssKeywords[key]] = key;
    const convert = {
        rgb: {
            channels: 3,
            labels: "rgb"
        },
        hsl: {
            channels: 3,
            labels: "hsl"
        },
        hsv: {
            channels: 3,
            labels: "hsv"
        },
        hwb: {
            channels: 3,
            labels: "hwb"
        },
        cmyk: {
            channels: 4,
            labels: "cmyk"
        },
        xyz: {
            channels: 3,
            labels: "xyz"
        },
        lab: {
            channels: 3,
            labels: "lab"
        },
        lch: {
            channels: 3,
            labels: "lch"
        },
        hex: {
            channels: 1,
            labels: [ "hex" ]
        },
        keyword: {
            channels: 1,
            labels: [ "keyword" ]
        },
        ansi16: {
            channels: 1,
            labels: [ "ansi16" ]
        },
        ansi256: {
            channels: 1,
            labels: [ "ansi256" ]
        },
        hcg: {
            channels: 3,
            labels: [ "h", "c", "g" ]
        },
        apple: {
            channels: 3,
            labels: [ "r16", "g16", "b16" ]
        },
        gray: {
            channels: 1,
            labels: [ "gray" ]
        }
    };
    conversions = convert;
    for (const model of Object.keys(convert)) {
        if (!("channels" in convert[model])) throw new Error("missing channels property: " + model);
        if (!("labels" in convert[model])) throw new Error("missing channel labels property: " + model);
        if (convert[model].labels.length !== convert[model].channels) throw new Error("channel and label counts mismatch: " + model);
        const {channels: channels, labels: labels} = convert[model];
        delete convert[model].channels, delete convert[model].labels, Object.defineProperty(convert[model], "channels", {
            value: channels
        }), Object.defineProperty(convert[model], "labels", {
            value: labels
        });
    }
    return convert.rgb.hsl = function(rgb) {
        const r = rgb[0] / 255, g = rgb[1] / 255, b = rgb[2] / 255, min = Math.min(r, g, b), max = Math.max(r, g, b), delta = max - min;
        let h, s;
        max === min ? h = 0 : r === max ? h = (g - b) / delta : g === max ? h = 2 + (b - r) / delta : b === max && (h = 4 + (r - g) / delta), 
        h = Math.min(60 * h, 360), h < 0 && (h += 360);
        const l = (min + max) / 2;
        return s = max === min ? 0 : l <= .5 ? delta / (max + min) : delta / (2 - max - min), 
        [ h, 100 * s, 100 * l ];
    }, convert.rgb.hsv = function(rgb) {
        let rdif, gdif, bdif, h, s;
        const r = rgb[0] / 255, g = rgb[1] / 255, b = rgb[2] / 255, v = Math.max(r, g, b), diff = v - Math.min(r, g, b), diffc = function(c) {
            return (v - c) / 6 / diff + .5;
        };
        return 0 === diff ? (h = 0, s = 0) : (s = diff / v, rdif = diffc(r), gdif = diffc(g), 
        bdif = diffc(b), r === v ? h = bdif - gdif : g === v ? h = 1 / 3 + rdif - bdif : b === v && (h = 2 / 3 + gdif - rdif), 
        h < 0 ? h += 1 : h > 1 && (h -= 1)), [ 360 * h, 100 * s, 100 * v ];
    }, convert.rgb.hwb = function(rgb) {
        const r = rgb[0], g = rgb[1];
        let b = rgb[2];
        const h = convert.rgb.hsl(rgb)[0], w = 1 / 255 * Math.min(r, Math.min(g, b));
        return b = 1 - 1 / 255 * Math.max(r, Math.max(g, b)), [ h, 100 * w, 100 * b ];
    }, convert.rgb.cmyk = function(rgb) {
        const r = rgb[0] / 255, g = rgb[1] / 255, b = rgb[2] / 255, k = Math.min(1 - r, 1 - g, 1 - b);
        return [ 100 * ((1 - r - k) / (1 - k) || 0), 100 * ((1 - g - k) / (1 - k) || 0), 100 * ((1 - b - k) / (1 - k) || 0), 100 * k ];
    }, convert.rgb.keyword = function(rgb) {
        const reversed = reverseKeywords[rgb];
        if (reversed) return reversed;
        let currentClosestKeyword, currentClosestDistance = 1 / 0;
        for (const keyword of Object.keys(cssKeywords)) {
            const value = cssKeywords[keyword], distance = (y = value, ((x = rgb)[0] - y[0]) ** 2 + (x[1] - y[1]) ** 2 + (x[2] - y[2]) ** 2);
            distance < currentClosestDistance && (currentClosestDistance = distance, currentClosestKeyword = keyword);
        }
        var x, y;
        return currentClosestKeyword;
    }, convert.keyword.rgb = function(keyword) {
        return cssKeywords[keyword];
    }, convert.rgb.xyz = function(rgb) {
        let r = rgb[0] / 255, g = rgb[1] / 255, b = rgb[2] / 255;
        r = r > .04045 ? ((r + .055) / 1.055) ** 2.4 : r / 12.92, g = g > .04045 ? ((g + .055) / 1.055) ** 2.4 : g / 12.92, 
        b = b > .04045 ? ((b + .055) / 1.055) ** 2.4 : b / 12.92;
        return [ 100 * (.4124 * r + .3576 * g + .1805 * b), 100 * (.2126 * r + .7152 * g + .0722 * b), 100 * (.0193 * r + .1192 * g + .9505 * b) ];
    }, convert.rgb.lab = function(rgb) {
        const xyz = convert.rgb.xyz(rgb);
        let x = xyz[0], y = xyz[1], z = xyz[2];
        x /= 95.047, y /= 100, z /= 108.883, x = x > .008856 ? x ** (1 / 3) : 7.787 * x + 16 / 116, 
        y = y > .008856 ? y ** (1 / 3) : 7.787 * y + 16 / 116, z = z > .008856 ? z ** (1 / 3) : 7.787 * z + 16 / 116;
        return [ 116 * y - 16, 500 * (x - y), 200 * (y - z) ];
    }, convert.hsl.rgb = function(hsl) {
        const h = hsl[0] / 360, s = hsl[1] / 100, l = hsl[2] / 100;
        let t2, t3, val;
        if (0 === s) return val = 255 * l, [ val, val, val ];
        t2 = l < .5 ? l * (1 + s) : l + s - l * s;
        const t1 = 2 * l - t2, rgb = [ 0, 0, 0 ];
        for (let i = 0; i < 3; i++) t3 = h + 1 / 3 * -(i - 1), t3 < 0 && t3++, t3 > 1 && t3--, 
        val = 6 * t3 < 1 ? t1 + 6 * (t2 - t1) * t3 : 2 * t3 < 1 ? t2 : 3 * t3 < 2 ? t1 + (t2 - t1) * (2 / 3 - t3) * 6 : t1, 
        rgb[i] = 255 * val;
        return rgb;
    }, convert.hsl.hsv = function(hsl) {
        const h = hsl[0];
        let s = hsl[1] / 100, l = hsl[2] / 100, smin = s;
        const lmin = Math.max(l, .01);
        l *= 2, s *= l <= 1 ? l : 2 - l, smin *= lmin <= 1 ? lmin : 2 - lmin;
        return [ h, 100 * (0 === l ? 2 * smin / (lmin + smin) : 2 * s / (l + s)), 100 * ((l + s) / 2) ];
    }, convert.hsv.rgb = function(hsv) {
        const h = hsv[0] / 60, s = hsv[1] / 100;
        let v = hsv[2] / 100;
        const hi = Math.floor(h) % 6, f = h - Math.floor(h), p = 255 * v * (1 - s), q = 255 * v * (1 - s * f), t = 255 * v * (1 - s * (1 - f));
        switch (v *= 255, hi) {
          case 0:
            return [ v, t, p ];

          case 1:
            return [ q, v, p ];

          case 2:
            return [ p, v, t ];

          case 3:
            return [ p, q, v ];

          case 4:
            return [ t, p, v ];

          case 5:
            return [ v, p, q ];
        }
    }, convert.hsv.hsl = function(hsv) {
        const h = hsv[0], s = hsv[1] / 100, v = hsv[2] / 100, vmin = Math.max(v, .01);
        let sl, l;
        l = (2 - s) * v;
        const lmin = (2 - s) * vmin;
        return sl = s * vmin, sl /= lmin <= 1 ? lmin : 2 - lmin, sl = sl || 0, l /= 2, [ h, 100 * sl, 100 * l ];
    }, convert.hwb.rgb = function(hwb) {
        const h = hwb[0] / 360;
        let wh = hwb[1] / 100, bl = hwb[2] / 100;
        const ratio = wh + bl;
        let f;
        ratio > 1 && (wh /= ratio, bl /= ratio);
        const i = Math.floor(6 * h), v = 1 - bl;
        f = 6 * h - i, 1 & i && (f = 1 - f);
        const n = wh + f * (v - wh);
        let r, g, b;
        switch (i) {
          default:
          case 6:
          case 0:
            r = v, g = n, b = wh;
            break;

          case 1:
            r = n, g = v, b = wh;
            break;

          case 2:
            r = wh, g = v, b = n;
            break;

          case 3:
            r = wh, g = n, b = v;
            break;

          case 4:
            r = n, g = wh, b = v;
            break;

          case 5:
            r = v, g = wh, b = n;
        }
        return [ 255 * r, 255 * g, 255 * b ];
    }, convert.cmyk.rgb = function(cmyk) {
        const c = cmyk[0] / 100, m = cmyk[1] / 100, y = cmyk[2] / 100, k = cmyk[3] / 100;
        return [ 255 * (1 - Math.min(1, c * (1 - k) + k)), 255 * (1 - Math.min(1, m * (1 - k) + k)), 255 * (1 - Math.min(1, y * (1 - k) + k)) ];
    }, convert.xyz.rgb = function(xyz) {
        const x = xyz[0] / 100, y = xyz[1] / 100, z = xyz[2] / 100;
        let r, g, b;
        return r = 3.2406 * x + -1.5372 * y + -.4986 * z, g = -.9689 * x + 1.8758 * y + .0415 * z, 
        b = .0557 * x + -.204 * y + 1.057 * z, r = r > .0031308 ? 1.055 * r ** (1 / 2.4) - .055 : 12.92 * r, 
        g = g > .0031308 ? 1.055 * g ** (1 / 2.4) - .055 : 12.92 * g, b = b > .0031308 ? 1.055 * b ** (1 / 2.4) - .055 : 12.92 * b, 
        r = Math.min(Math.max(0, r), 1), g = Math.min(Math.max(0, g), 1), b = Math.min(Math.max(0, b), 1), 
        [ 255 * r, 255 * g, 255 * b ];
    }, convert.xyz.lab = function(xyz) {
        let x = xyz[0], y = xyz[1], z = xyz[2];
        x /= 95.047, y /= 100, z /= 108.883, x = x > .008856 ? x ** (1 / 3) : 7.787 * x + 16 / 116, 
        y = y > .008856 ? y ** (1 / 3) : 7.787 * y + 16 / 116, z = z > .008856 ? z ** (1 / 3) : 7.787 * z + 16 / 116;
        return [ 116 * y - 16, 500 * (x - y), 200 * (y - z) ];
    }, convert.lab.xyz = function(lab) {
        let x, y, z;
        y = (lab[0] + 16) / 116, x = lab[1] / 500 + y, z = y - lab[2] / 200;
        const y2 = y ** 3, x2 = x ** 3, z2 = z ** 3;
        return y = y2 > .008856 ? y2 : (y - 16 / 116) / 7.787, x = x2 > .008856 ? x2 : (x - 16 / 116) / 7.787, 
        z = z2 > .008856 ? z2 : (z - 16 / 116) / 7.787, x *= 95.047, y *= 100, z *= 108.883, 
        [ x, y, z ];
    }, convert.lab.lch = function(lab) {
        const l = lab[0], a = lab[1], b = lab[2];
        let h;
        h = 360 * Math.atan2(b, a) / 2 / Math.PI, h < 0 && (h += 360);
        return [ l, Math.sqrt(a * a + b * b), h ];
    }, convert.lch.lab = function(lch) {
        const l = lch[0], c = lch[1], hr = lch[2] / 360 * 2 * Math.PI;
        return [ l, c * Math.cos(hr), c * Math.sin(hr) ];
    }, convert.rgb.ansi16 = function(args, saturation = null) {
        const [r, g, b] = args;
        let value = null === saturation ? convert.rgb.hsv(args)[2] : saturation;
        if (value = Math.round(value / 50), 0 === value) return 30;
        let ansi = 30 + (Math.round(b / 255) << 2 | Math.round(g / 255) << 1 | Math.round(r / 255));
        return 2 === value && (ansi += 60), ansi;
    }, convert.hsv.ansi16 = function(args) {
        return convert.rgb.ansi16(convert.hsv.rgb(args), args[2]);
    }, convert.rgb.ansi256 = function(args) {
        const r = args[0], g = args[1], b = args[2];
        if (r === g && g === b) return r < 8 ? 16 : r > 248 ? 231 : Math.round((r - 8) / 247 * 24) + 232;
        return 16 + 36 * Math.round(r / 255 * 5) + 6 * Math.round(g / 255 * 5) + Math.round(b / 255 * 5);
    }, convert.ansi16.rgb = function(args) {
        let color = args % 10;
        if (0 === color || 7 === color) return args > 50 && (color += 3.5), color = color / 10.5 * 255, 
        [ color, color, color ];
        const mult = .5 * (1 + ~~(args > 50));
        return [ (1 & color) * mult * 255, (color >> 1 & 1) * mult * 255, (color >> 2 & 1) * mult * 255 ];
    }, convert.ansi256.rgb = function(args) {
        if (args >= 232) {
            const c = 10 * (args - 232) + 8;
            return [ c, c, c ];
        }
        let rem;
        args -= 16;
        return [ Math.floor(args / 36) / 5 * 255, Math.floor((rem = args % 36) / 6) / 5 * 255, rem % 6 / 5 * 255 ];
    }, convert.rgb.hex = function(args) {
        const string = (((255 & Math.round(args[0])) << 16) + ((255 & Math.round(args[1])) << 8) + (255 & Math.round(args[2]))).toString(16).toUpperCase();
        return "000000".substring(string.length) + string;
    }, convert.hex.rgb = function(args) {
        const match = args.toString(16).match(/[a-f0-9]{6}|[a-f0-9]{3}/i);
        if (!match) return [ 0, 0, 0 ];
        let colorString = match[0];
        3 === match[0].length && (colorString = colorString.split("").map((char => char + char)).join(""));
        const integer = parseInt(colorString, 16);
        return [ integer >> 16 & 255, integer >> 8 & 255, 255 & integer ];
    }, convert.rgb.hcg = function(rgb) {
        const r = rgb[0] / 255, g = rgb[1] / 255, b = rgb[2] / 255, max = Math.max(Math.max(r, g), b), min = Math.min(Math.min(r, g), b), chroma = max - min;
        let grayscale, hue;
        return grayscale = chroma < 1 ? min / (1 - chroma) : 0, hue = chroma <= 0 ? 0 : max === r ? (g - b) / chroma % 6 : max === g ? 2 + (b - r) / chroma : 4 + (r - g) / chroma, 
        hue /= 6, hue %= 1, [ 360 * hue, 100 * chroma, 100 * grayscale ];
    }, convert.hsl.hcg = function(hsl) {
        const s = hsl[1] / 100, l = hsl[2] / 100, c = l < .5 ? 2 * s * l : 2 * s * (1 - l);
        let f = 0;
        return c < 1 && (f = (l - .5 * c) / (1 - c)), [ hsl[0], 100 * c, 100 * f ];
    }, convert.hsv.hcg = function(hsv) {
        const s = hsv[1] / 100, v = hsv[2] / 100, c = s * v;
        let f = 0;
        return c < 1 && (f = (v - c) / (1 - c)), [ hsv[0], 100 * c, 100 * f ];
    }, convert.hcg.rgb = function(hcg) {
        const h = hcg[0] / 360, c = hcg[1] / 100, g = hcg[2] / 100;
        if (0 === c) return [ 255 * g, 255 * g, 255 * g ];
        const pure = [ 0, 0, 0 ], hi = h % 1 * 6, v = hi % 1, w = 1 - v;
        let mg = 0;
        switch (Math.floor(hi)) {
          case 0:
            pure[0] = 1, pure[1] = v, pure[2] = 0;
            break;

          case 1:
            pure[0] = w, pure[1] = 1, pure[2] = 0;
            break;

          case 2:
            pure[0] = 0, pure[1] = 1, pure[2] = v;
            break;

          case 3:
            pure[0] = 0, pure[1] = w, pure[2] = 1;
            break;

          case 4:
            pure[0] = v, pure[1] = 0, pure[2] = 1;
            break;

          default:
            pure[0] = 1, pure[1] = 0, pure[2] = w;
        }
        return mg = (1 - c) * g, [ 255 * (c * pure[0] + mg), 255 * (c * pure[1] + mg), 255 * (c * pure[2] + mg) ];
    }, convert.hcg.hsv = function(hcg) {
        const c = hcg[1] / 100, v = c + hcg[2] / 100 * (1 - c);
        let f = 0;
        return v > 0 && (f = c / v), [ hcg[0], 100 * f, 100 * v ];
    }, convert.hcg.hsl = function(hcg) {
        const c = hcg[1] / 100, l = hcg[2] / 100 * (1 - c) + .5 * c;
        let s = 0;
        return l > 0 && l < .5 ? s = c / (2 * l) : l >= .5 && l < 1 && (s = c / (2 * (1 - l))), 
        [ hcg[0], 100 * s, 100 * l ];
    }, convert.hcg.hwb = function(hcg) {
        const c = hcg[1] / 100, v = c + hcg[2] / 100 * (1 - c);
        return [ hcg[0], 100 * (v - c), 100 * (1 - v) ];
    }, convert.hwb.hcg = function(hwb) {
        const w = hwb[1] / 100, v = 1 - hwb[2] / 100, c = v - w;
        let g = 0;
        return c < 1 && (g = (v - c) / (1 - c)), [ hwb[0], 100 * c, 100 * g ];
    }, convert.apple.rgb = function(apple) {
        return [ apple[0] / 65535 * 255, apple[1] / 65535 * 255, apple[2] / 65535 * 255 ];
    }, convert.rgb.apple = function(rgb) {
        return [ rgb[0] / 255 * 65535, rgb[1] / 255 * 65535, rgb[2] / 255 * 65535 ];
    }, convert.gray.rgb = function(args) {
        return [ args[0] / 100 * 255, args[0] / 100 * 255, args[0] / 100 * 255 ];
    }, convert.gray.hsl = function(args) {
        return [ 0, 0, args[0] ];
    }, convert.gray.hsv = convert.gray.hsl, convert.gray.hwb = function(gray) {
        return [ 0, 100, gray[0] ];
    }, convert.gray.cmyk = function(gray) {
        return [ 0, 0, 0, gray[0] ];
    }, convert.gray.lab = function(gray) {
        return [ gray[0], 0, 0 ];
    }, convert.gray.hex = function(gray) {
        const val = 255 & Math.round(gray[0] / 100 * 255), string = ((val << 16) + (val << 8) + val).toString(16).toUpperCase();
        return "000000".substring(string.length) + string;
    }, convert.rgb.gray = function(rgb) {
        return [ (rgb[0] + rgb[1] + rgb[2]) / 3 / 255 * 100 ];
    }, conversions;
}

function requireRoute() {
    if (hasRequiredRoute) return route;
    hasRequiredRoute = 1;
    const conversions = requireConversions();
    function deriveBFS(fromModel) {
        const graph = function() {
            const graph = {}, models = Object.keys(conversions);
            for (let len = models.length, i = 0; i < len; i++) graph[models[i]] = {
                distance: -1,
                parent: null
            };
            return graph;
        }(), queue = [ fromModel ];
        for (graph[fromModel].distance = 0; queue.length; ) {
            const current = queue.pop(), adjacents = Object.keys(conversions[current]);
            for (let len = adjacents.length, i = 0; i < len; i++) {
                const adjacent = adjacents[i], node = graph[adjacent];
                -1 === node.distance && (node.distance = graph[current].distance + 1, node.parent = current, 
                queue.unshift(adjacent));
            }
        }
        return graph;
    }
    function link(from, to) {
        return function(args) {
            return to(from(args));
        };
    }
    function wrapConversion(toModel, graph) {
        const path = [ graph[toModel].parent, toModel ];
        let fn = conversions[graph[toModel].parent][toModel], cur = graph[toModel].parent;
        for (;graph[cur].parent; ) path.unshift(graph[cur].parent), fn = link(conversions[graph[cur].parent][cur], fn), 
        cur = graph[cur].parent;
        return fn.conversion = path, fn;
    }
    return route = function(fromModel) {
        const graph = deriveBFS(fromModel), conversion = {}, models = Object.keys(graph);
        for (let len = models.length, i = 0; i < len; i++) {
            const toModel = models[i];
            null !== graph[toModel].parent && (conversion[toModel] = wrapConversion(toModel, graph));
        }
        return conversion;
    };
}

function requireColorConvert() {
    if (hasRequiredColorConvert) return colorConvert;
    hasRequiredColorConvert = 1;
    const conversions = requireConversions(), route = requireRoute(), convert = {};
    return Object.keys(conversions).forEach((fromModel => {
        convert[fromModel] = {}, Object.defineProperty(convert[fromModel], "channels", {
            value: conversions[fromModel].channels
        }), Object.defineProperty(convert[fromModel], "labels", {
            value: conversions[fromModel].labels
        });
        const routes = route(fromModel);
        Object.keys(routes).forEach((toModel => {
            const fn = routes[toModel];
            convert[fromModel][toModel] = function(fn) {
                const wrappedFn = function(...args) {
                    const arg0 = args[0];
                    if (null == arg0) return arg0;
                    arg0.length > 1 && (args = arg0);
                    const result = fn(args);
                    if ("object" == typeof result) for (let len = result.length, i = 0; i < len; i++) result[i] = Math.round(result[i]);
                    return result;
                };
                return "conversion" in fn && (wrappedFn.conversion = fn.conversion), wrappedFn;
            }(fn), convert[fromModel][toModel].raw = function(fn) {
                const wrappedFn = function(...args) {
                    const arg0 = args[0];
                    return null == arg0 ? arg0 : (arg0.length > 1 && (args = arg0), fn(args));
                };
                return "conversion" in fn && (wrappedFn.conversion = fn.conversion), wrappedFn;
            }(fn);
        }));
    })), colorConvert = convert;
}

function requireAnsiStyles() {
    return hasRequiredAnsiStyles || (hasRequiredAnsiStyles = 1, function(module) {
        const wrapAnsi16 = (fn, offset) => (...args) => `[${fn(...args) + offset}m`, wrapAnsi256 = (fn, offset) => (...args) => {
            const code = fn(...args);
            return `[${38 + offset};5;${code}m`;
        }, wrapAnsi16m = (fn, offset) => (...args) => {
            const rgb = fn(...args);
            return `[${38 + offset};2;${rgb[0]};${rgb[1]};${rgb[2]}m`;
        }, ansi2ansi = n => n, rgb2rgb = (r, g, b) => [ r, g, b ], setLazyProperty = (object, property, get) => {
            Object.defineProperty(object, property, {
                get: () => {
                    const value = get();
                    return Object.defineProperty(object, property, {
                        value: value,
                        enumerable: !0,
                        configurable: !0
                    }), value;
                },
                enumerable: !0,
                configurable: !0
            });
        };
        let colorConvert;
        const makeDynamicStyles = (wrap, targetSpace, identity, isBackground) => {
            void 0 === colorConvert && (colorConvert = requireColorConvert());
            const offset = isBackground ? 10 : 0, styles = {};
            for (const [sourceSpace, suite] of Object.entries(colorConvert)) {
                const name = "ansi16" === sourceSpace ? "ansi" : sourceSpace;
                sourceSpace === targetSpace ? styles[name] = wrap(identity, offset) : "object" == typeof suite && (styles[name] = wrap(suite[targetSpace], offset));
            }
            return styles;
        };
        Object.defineProperty(module, "exports", {
            enumerable: !0,
            get: function() {
                const codes = new Map, styles = {
                    modifier: {
                        reset: [ 0, 0 ],
                        bold: [ 1, 22 ],
                        dim: [ 2, 22 ],
                        italic: [ 3, 23 ],
                        underline: [ 4, 24 ],
                        inverse: [ 7, 27 ],
                        hidden: [ 8, 28 ],
                        strikethrough: [ 9, 29 ]
                    },
                    color: {
                        black: [ 30, 39 ],
                        red: [ 31, 39 ],
                        green: [ 32, 39 ],
                        yellow: [ 33, 39 ],
                        blue: [ 34, 39 ],
                        magenta: [ 35, 39 ],
                        cyan: [ 36, 39 ],
                        white: [ 37, 39 ],
                        blackBright: [ 90, 39 ],
                        redBright: [ 91, 39 ],
                        greenBright: [ 92, 39 ],
                        yellowBright: [ 93, 39 ],
                        blueBright: [ 94, 39 ],
                        magentaBright: [ 95, 39 ],
                        cyanBright: [ 96, 39 ],
                        whiteBright: [ 97, 39 ]
                    },
                    bgColor: {
                        bgBlack: [ 40, 49 ],
                        bgRed: [ 41, 49 ],
                        bgGreen: [ 42, 49 ],
                        bgYellow: [ 43, 49 ],
                        bgBlue: [ 44, 49 ],
                        bgMagenta: [ 45, 49 ],
                        bgCyan: [ 46, 49 ],
                        bgWhite: [ 47, 49 ],
                        bgBlackBright: [ 100, 49 ],
                        bgRedBright: [ 101, 49 ],
                        bgGreenBright: [ 102, 49 ],
                        bgYellowBright: [ 103, 49 ],
                        bgBlueBright: [ 104, 49 ],
                        bgMagentaBright: [ 105, 49 ],
                        bgCyanBright: [ 106, 49 ],
                        bgWhiteBright: [ 107, 49 ]
                    }
                };
                styles.color.gray = styles.color.blackBright, styles.bgColor.bgGray = styles.bgColor.bgBlackBright, 
                styles.color.grey = styles.color.blackBright, styles.bgColor.bgGrey = styles.bgColor.bgBlackBright;
                for (const [groupName, group] of Object.entries(styles)) {
                    for (const [styleName, style] of Object.entries(group)) styles[styleName] = {
                        open: `[${style[0]}m`,
                        close: `[${style[1]}m`
                    }, group[styleName] = styles[styleName], codes.set(style[0], style[1]);
                    Object.defineProperty(styles, groupName, {
                        value: group,
                        enumerable: !1
                    });
                }
                return Object.defineProperty(styles, "codes", {
                    value: codes,
                    enumerable: !1
                }), styles.color.close = "[39m", styles.bgColor.close = "[49m", setLazyProperty(styles.color, "ansi", (() => makeDynamicStyles(wrapAnsi16, "ansi16", ansi2ansi, !1))), 
                setLazyProperty(styles.color, "ansi256", (() => makeDynamicStyles(wrapAnsi256, "ansi256", ansi2ansi, !1))), 
                setLazyProperty(styles.color, "ansi16m", (() => makeDynamicStyles(wrapAnsi16m, "rgb", rgb2rgb, !1))), 
                setLazyProperty(styles.bgColor, "ansi", (() => makeDynamicStyles(wrapAnsi16, "ansi16", ansi2ansi, !0))), 
                setLazyProperty(styles.bgColor, "ansi256", (() => makeDynamicStyles(wrapAnsi256, "ansi256", ansi2ansi, !0))), 
                setLazyProperty(styles.bgColor, "ansi16m", (() => makeDynamicStyles(wrapAnsi16m, "rgb", rgb2rgb, !0))), 
                styles;
            }
        });
    }(ansiStyles)), ansiStyles.exports;
}

function requireWrapAnsi() {
    if (hasRequiredWrapAnsi) return wrapAnsi_1;
    hasRequiredWrapAnsi = 1;
    const stringWidth = requireStringWidth(), stripAnsi = requireStripAnsi(), ansiStyles = requireAnsiStyles(), ESCAPES = new Set([ "", "" ]), wrapAnsi = code => `${ESCAPES.values().next().value}[${code}m`, wrapWord = (rows, word, columns) => {
        const characters = [ ...word ];
        let isInsideEscape = !1, visible = stringWidth(stripAnsi(rows[rows.length - 1]));
        for (const [index, character] of characters.entries()) {
            const characterLength = stringWidth(character);
            if (visible + characterLength <= columns ? rows[rows.length - 1] += character : (rows.push(character), 
            visible = 0), ESCAPES.has(character)) isInsideEscape = !0; else if (isInsideEscape && "m" === character) {
                isInsideEscape = !1;
                continue;
            }
            isInsideEscape || (visible += characterLength, visible === columns && index < characters.length - 1 && (rows.push(""), 
            visible = 0));
        }
        !visible && rows[rows.length - 1].length > 0 && rows.length > 1 && (rows[rows.length - 2] += rows.pop());
    }, stringVisibleTrimSpacesRight = str => {
        const words = str.split(" ");
        let last = words.length;
        for (;last > 0 && !(stringWidth(words[last - 1]) > 0); ) last--;
        return last === words.length ? str : words.slice(0, last).join(" ") + words.slice(last).join("");
    }, exec = (string, columns, options = {}) => {
        if (!1 !== options.trim && "" === string.trim()) return "";
        let escapeCode, pre = "", ret = "";
        const lengths = (string => string.split(" ").map((character => stringWidth(character))))(string);
        let rows = [ "" ];
        for (const [index, word] of string.split(" ").entries()) {
            !1 !== options.trim && (rows[rows.length - 1] = rows[rows.length - 1].trimLeft());
            let rowLength = stringWidth(rows[rows.length - 1]);
            if (0 !== index && (rowLength >= columns && (!1 === options.wordWrap || !1 === options.trim) && (rows.push(""), 
            rowLength = 0), (rowLength > 0 || !1 === options.trim) && (rows[rows.length - 1] += " ", 
            rowLength++)), options.hard && lengths[index] > columns) {
                const remainingColumns = columns - rowLength, breaksStartingThisLine = 1 + Math.floor((lengths[index] - remainingColumns - 1) / columns);
                Math.floor((lengths[index] - 1) / columns) < breaksStartingThisLine && rows.push(""), 
                wrapWord(rows, word, columns);
            } else {
                if (rowLength + lengths[index] > columns && rowLength > 0 && lengths[index] > 0) {
                    if (!1 === options.wordWrap && rowLength < columns) {
                        wrapWord(rows, word, columns);
                        continue;
                    }
                    rows.push("");
                }
                rowLength + lengths[index] > columns && !1 === options.wordWrap ? wrapWord(rows, word, columns) : rows[rows.length - 1] += word;
            }
        }
        !1 !== options.trim && (rows = rows.map(stringVisibleTrimSpacesRight)), pre = rows.join("\n");
        for (const [index, character] of [ ...pre ].entries()) {
            if (ret += character, ESCAPES.has(character)) {
                const code = parseFloat(/\d[^m]*/.exec(pre.slice(index, index + 4)));
                escapeCode = 39 === code ? null : code;
            }
            const code = ansiStyles.codes.get(Number(escapeCode));
            escapeCode && code && ("\n" === pre[index + 1] ? ret += wrapAnsi(code) : "\n" === character && (ret += wrapAnsi(escapeCode)));
        }
        return ret;
    };
    return wrapAnsi_1 = (string, columns, options) => String(string).normalize().replace(/\r\n/g, "\n").split("\n").map((line => exec(line, columns, options))).join("\n"), 
    wrapAnsi_1;
}

ansiStyles.exports;

var wrapAnsiExports = requireWrapAnsi(), wrapAnsi = getDefaultExportFromCjs(wrapAnsiExports), lib, hasRequiredLib;

function breakLines(content, width) {
    return content.split("\n").flatMap((line => wrapAnsi(line, width, {
        trim: !1,
        hard: !0
    }).split("\n").map((str => str.trimEnd())))).join("\n");
}

function readlineWidth() {
    return cliWidth({
        defaultWidth: 80,
        output: readline().output
    });
}

function split(content, width) {
    return breakLines(content, width).split("\n");
}

function rotate(count, items) {
    const max = items.length, offset = (count % max + max) % max;
    return [ ...items.slice(offset), ...items.slice(0, offset) ];
}

function lines({items: items, width: width, renderItem: renderItem, active: active, position: requested, pageSize: pageSize}) {
    const layouts = items.map(((item, index) => ({
        item: item,
        index: index,
        isActive: index === active
    }))), layoutsInPage = rotate(active - requested, layouts).slice(0, pageSize), renderItemAt = index => null == layoutsInPage[index] ? [] : split(renderItem(layoutsInPage[index]), width), pageBuffer = Array.from({
        length: pageSize
    }), activeItem = renderItemAt(requested).slice(0, pageSize), position = requested + activeItem.length <= pageSize ? requested : pageSize - activeItem.length;
    pageBuffer.splice(position, activeItem.length, ...activeItem);
    let bufferPointer = position + activeItem.length, layoutPointer = requested + 1;
    for (;bufferPointer < pageSize && layoutPointer < layoutsInPage.length; ) {
        for (const line of renderItemAt(layoutPointer)) if (pageBuffer[bufferPointer++] = line, 
        bufferPointer >= pageSize) break;
        layoutPointer++;
    }
    for (bufferPointer = position - 1, layoutPointer = requested - 1; bufferPointer >= 0 && layoutPointer >= 0; ) {
        for (const line of renderItemAt(layoutPointer).reverse()) if (pageBuffer[bufferPointer--] = line, 
        bufferPointer < 0) break;
        layoutPointer--;
    }
    return pageBuffer.filter((line => "string" == typeof line));
}

function finite({active: active, pageSize: pageSize, total: total}) {
    const middle = Math.floor(pageSize / 2);
    return total <= pageSize || active < middle ? active : active >= total - middle ? active + pageSize - total : middle;
}

function infinite({active: active, lastActive: lastActive, total: total, pageSize: pageSize, pointer: pointer}) {
    return total <= pageSize ? active : lastActive < active && active - lastActive < pageSize ? Math.min(Math.floor(pageSize / 2), pointer + active - lastActive) : pointer;
}

function usePagination({items: items, active: active, renderItem: renderItem, pageSize: pageSize, loop: loop = !0}) {
    const state = useRef({
        position: 0,
        lastActive: 0
    }), position = loop ? infinite({
        active: active,
        lastActive: state.current.lastActive,
        total: items.length,
        pageSize: pageSize,
        pointer: state.current.position
    }) : finite({
        active: active,
        total: items.length,
        pageSize: pageSize
    });
    return state.current.position = position, state.current.lastActive = active, lines({
        items: items,
        width: readlineWidth(),
        renderItem: renderItem,
        active: active,
        position: position,
        pageSize: pageSize
    }).join("\n");
}

function requireLib() {
    if (hasRequiredLib) return lib;
    hasRequiredLib = 1;
    const Stream = require$$0$2;
    return lib = class extends Stream {
        #isTTY=null;
        constructor(opts = {}) {
            super(opts), this.writable = this.readable = !0, this.muted = !1, this.on("pipe", this._onpipe), 
            this.replace = opts.replace, this._prompt = opts.prompt || null, this._hadControl = !1;
        }
        #destSrc(key, def) {
            return this._dest ? this._dest[key] : this._src ? this._src[key] : def;
        }
        #proxy(method, ...args) {
            "function" == typeof this._dest?.[method] && this._dest[method](...args), "function" == typeof this._src?.[method] && this._src[method](...args);
        }
        get isTTY() {
            return null !== this.#isTTY ? this.#isTTY : this.#destSrc("isTTY", !1);
        }
        set isTTY(val) {
            this.#isTTY = val;
        }
        get rows() {
            return this.#destSrc("rows");
        }
        get columns() {
            return this.#destSrc("columns");
        }
        mute() {
            this.muted = !0;
        }
        unmute() {
            this.muted = !1;
        }
        _onpipe(src) {
            this._src = src;
        }
        pipe(dest, options) {
            return this._dest = dest, super.pipe(dest, options);
        }
        pause() {
            if (this._src) return this._src.pause();
        }
        resume() {
            if (this._src) return this._src.resume();
        }
        write(c) {
            if (this.muted) {
                if (!this.replace) return !0;
                if (c.match(/^\u001b/)) return 0 === c.indexOf(this._prompt) && (c = (c = c.slice(this._prompt.length)).replace(/./g, this.replace), 
                c = this._prompt + c), this._hadControl = !0, this.emit("data", c);
                this._prompt && this._hadControl && 0 === c.indexOf(this._prompt) && (this._hadControl = !1, 
                this.emit("data", this._prompt), c = c.slice(this._prompt.length)), c = c.toString().replace(/./g, this.replace);
            }
            this.emit("data", c);
        }
        end(c) {
            this.muted && (c = c && this.replace ? c.toString().replace(/./g, this.replace) : null), 
            c && this.emit("data", c), this.emit("end");
        }
        destroy(...args) {
            return this.#proxy("destroy", ...args);
        }
        destroySoon(...args) {
            return this.#proxy("destroySoon", ...args);
        }
        close(...args) {
            return this.#proxy("close", ...args);
        }
    };
}

var libExports = requireLib(), MuteStream = getDefaultExportFromCjs(libExports), stripAnsiExports = requireStripAnsi(), stripAnsi = getDefaultExportFromCjs(stripAnsiExports), ansiEscapes$1 = {
    exports: {}
}, hasRequiredAnsiEscapes;

function requireAnsiEscapes() {
    return hasRequiredAnsiEscapes || (hasRequiredAnsiEscapes = 1, function(module) {
        const ansiEscapes = module.exports;
        module.exports.default = ansiEscapes;
        const ESC = "[", OSC = "]", BEL = "", SEP = ";", isTerminalApp = "Apple_Terminal" === process.env.TERM_PROGRAM;
        ansiEscapes.cursorTo = (x, y) => {
            if ("number" != typeof x) throw new TypeError("The `x` argument is required");
            return "number" != typeof y ? ESC + (x + 1) + "G" : ESC + (y + 1) + ";" + (x + 1) + "H";
        }, ansiEscapes.cursorMove = (x, y) => {
            if ("number" != typeof x) throw new TypeError("The `x` argument is required");
            let ret = "";
            return x < 0 ? ret += ESC + -x + "D" : x > 0 && (ret += ESC + x + "C"), y < 0 ? ret += ESC + -y + "A" : y > 0 && (ret += ESC + y + "B"), 
            ret;
        }, ansiEscapes.cursorUp = (count = 1) => ESC + count + "A", ansiEscapes.cursorDown = (count = 1) => ESC + count + "B", 
        ansiEscapes.cursorForward = (count = 1) => ESC + count + "C", ansiEscapes.cursorBackward = (count = 1) => ESC + count + "D", 
        ansiEscapes.cursorLeft = "[G", ansiEscapes.cursorSavePosition = isTerminalApp ? "7" : "[s", 
        ansiEscapes.cursorRestorePosition = isTerminalApp ? "8" : "[u", ansiEscapes.cursorGetPosition = "[6n", 
        ansiEscapes.cursorNextLine = "[E", ansiEscapes.cursorPrevLine = "[F", ansiEscapes.cursorHide = "[?25l", 
        ansiEscapes.cursorShow = "[?25h", ansiEscapes.eraseLines = count => {
            let clear = "";
            for (let i = 0; i < count; i++) clear += ansiEscapes.eraseLine + (i < count - 1 ? ansiEscapes.cursorUp() : "");
            return count && (clear += ansiEscapes.cursorLeft), clear;
        }, ansiEscapes.eraseEndLine = "[K", ansiEscapes.eraseStartLine = "[1K", ansiEscapes.eraseLine = "[2K", 
        ansiEscapes.eraseDown = "[J", ansiEscapes.eraseUp = "[1J", ansiEscapes.eraseScreen = "[2J", 
        ansiEscapes.scrollUp = "[S", ansiEscapes.scrollDown = "[T", ansiEscapes.clearScreen = "c", 
        ansiEscapes.clearTerminal = "win32" === process.platform ? `${ansiEscapes.eraseScreen}${ESC}0f` : `${ansiEscapes.eraseScreen}${ESC}3J${ESC}H`, 
        ansiEscapes.beep = BEL, ansiEscapes.link = (text, url) => [ OSC, "8", SEP, SEP, url, BEL, text, OSC, "8", SEP, SEP, BEL ].join(""), 
        ansiEscapes.image = (buffer, options = {}) => {
            let ret = `${OSC}1337;File=inline=1`;
            return options.width && (ret += `;width=${options.width}`), options.height && (ret += `;height=${options.height}`), 
            !1 === options.preserveAspectRatio && (ret += ";preserveAspectRatio=0"), ret + ":" + buffer.toString("base64") + BEL;
        }, ansiEscapes.iTerm = {
            setCwd: (cwd = process.cwd()) => `${OSC}50;CurrentDir=${cwd}${BEL}`,
            annotation: (message, options = {}) => {
                let ret = `${OSC}1337;`;
                const hasX = void 0 !== options.x, hasY = void 0 !== options.y;
                if ((hasX || hasY) && (!hasX || !hasY || void 0 === options.length)) throw new Error("`x`, `y` and `length` must be defined when `x` or `y` is defined");
                return message = message.replace(/\|/g, ""), ret += options.isHidden ? "AddHiddenAnnotation=" : "AddAnnotation=", 
                options.length > 0 ? ret += (hasX ? [ message, options.length, options.x, options.y ] : [ options.length, message ]).join("|") : ret += message, 
                ret + BEL;
            }
        };
    }(ansiEscapes$1)), ansiEscapes$1.exports;
}

var ansiEscapesExports = requireAnsiEscapes(), ansiEscapes = getDefaultExportFromCjs(ansiEscapesExports);

const height = content => content.split("\n").length, lastLine = content => content.split("\n").pop() ?? "";

function cursorDown(n) {
    return n > 0 ? ansiEscapes.cursorDown(n) : "";
}

class ScreenManager {
    rl;
    height=0;
    extraLinesUnderPrompt=0;
    cursorPos;
    constructor(rl) {
        this.rl = rl, this.rl = rl, this.cursorPos = rl.getCursorPos();
    }
    write(content) {
        this.rl.output.unmute(), this.rl.output.write(content), this.rl.output.mute();
    }
    render(content, bottomContent = "") {
        const promptLine = lastLine(content), rawPromptLine = stripAnsi(promptLine);
        let prompt = rawPromptLine;
        this.rl.line.length > 0 && (prompt = prompt.slice(0, -this.rl.line.length)), this.rl.setPrompt(prompt), 
        this.cursorPos = this.rl.getCursorPos();
        const width = readlineWidth();
        content = breakLines(content, width), bottomContent = breakLines(bottomContent, width), 
        rawPromptLine.length % width == 0 && (content += "\n");
        let output = content + (bottomContent ? "\n" + bottomContent : "");
        const bottomContentHeight = Math.floor(rawPromptLine.length / width) - this.cursorPos.rows + (bottomContent ? height(bottomContent) : 0);
        bottomContentHeight > 0 && (output += ansiEscapes.cursorUp(bottomContentHeight)), 
        output += ansiEscapes.cursorTo(this.cursorPos.cols), this.write(cursorDown(this.extraLinesUnderPrompt) + ansiEscapes.eraseLines(this.height) + output), 
        this.extraLinesUnderPrompt = bottomContentHeight, this.height = height(output);
    }
    checkCursorPos() {
        const cursorPos = this.rl.getCursorPos();
        cursorPos.cols !== this.cursorPos.cols && (this.write(ansiEscapes.cursorTo(cursorPos.cols)), 
        this.cursorPos = cursorPos);
    }
    done({clearContent: clearContent}) {
        this.rl.setPrompt("");
        let output = cursorDown(this.extraLinesUnderPrompt);
        output += clearContent ? ansiEscapes.eraseLines(this.height) : "\n", output += ansiEscapes.cursorShow, 
        this.write(output), this.rl.close();
    }
}

class PromisePolyfill extends Promise {
    static withResolver() {
        let resolve, reject;
        return {
            promise: new Promise(((res, rej) => {
                resolve = res, reject = rej;
            })),
            resolve: resolve,
            reject: reject
        };
    }
}

function createPrompt(view) {
    return (config, context = {}) => {
        const {input: input = process.stdin, signal: signal} = context, cleanups = new Set, output = new MuteStream;
        output.pipe(context.output ?? process.stdout);
        const rl = readline$1.createInterface({
            terminal: !0,
            input: input,
            output: output
        }), screen = new ScreenManager(rl), {promise: promise, resolve: resolve, reject: reject} = PromisePolyfill.withResolver(), cancel = () => reject(new CancelPromptError);
        if (signal) {
            const abort = () => reject(new AbortPromptError({
                cause: signal.reason
            }));
            if (signal.aborted) return abort(), Object.assign(promise, {
                cancel: cancel
            });
            signal.addEventListener("abort", abort), cleanups.add((() => signal.removeEventListener("abort", abort)));
        }
        cleanups.add(onExit(((code, signal) => {
            reject(new ExitPromptError(`User force closed the prompt with ${code} ${signal}`));
        })));
        const checkCursorPos = () => screen.checkCursorPos();
        return rl.input.on("keypress", checkCursorPos), cleanups.add((() => rl.input.removeListener("keypress", checkCursorPos))), 
        withHooks(rl, (cycle => {
            const hooksCleanup = AsyncResource.bind((() => effectScheduler.clearAll()));
            return rl.on("close", hooksCleanup), cleanups.add((() => rl.removeListener("close", hooksCleanup))), 
            cycle((() => {
                try {
                    const nextView = view(config, (value => {
                        setImmediate((() => resolve(value)));
                    })), [content, bottomContent] = "string" == typeof nextView ? [ nextView ] : nextView;
                    screen.render(content, bottomContent), effectScheduler.run();
                } catch (error) {
                    reject(error);
                }
            })), Object.assign(promise.then((answer => (effectScheduler.clearAll(), answer)), (error => {
                throw effectScheduler.clearAll(), error;
            })).finally((() => {
                cleanups.forEach((cleanup => cleanup())), screen.done({
                    clearContent: Boolean(context?.clearPromptOnDone)
                }), output.end();
            })).then((() => promise)), {
                cancel: cancel
            });
        }));
    };
}

class Separator {
    separator=colors$1.dim(Array.from({
        length: 15
    }).join(figures.line));
    type="separator";
    constructor(separator) {
        separator && (this.separator = separator);
    }
    static isSeparator(choice) {
        return Boolean(choice && "object" == typeof choice && "type" in choice && "separator" === choice.type);
    }
}

const selectTheme = {
    icon: {
        cursor: figures.pointer
    },
    style: {
        disabled: text => colors$1.dim(`- ${text}`),
        description: text => colors$1.cyan(text)
    },
    helpMode: "auto"
};

function isSelectable(item) {
    return !Separator.isSeparator(item) && !item.disabled;
}

function normalizeChoices(choices) {
    return choices.map((choice => {
        if (Separator.isSeparator(choice)) return choice;
        if ("string" == typeof choice) return {
            value: choice,
            name: choice,
            short: choice,
            disabled: !1
        };
        const name = choice.name ?? String(choice.value);
        return {
            value: choice.value,
            name: name,
            description: choice.description,
            short: choice.short ?? name,
            disabled: choice.disabled ?? !1
        };
    }));
}

var select = createPrompt(((config, done) => {
    const {loop: loop = !0, pageSize: pageSize = 7} = config, firstRender = useRef(!0), theme = makeTheme(selectTheme, config.theme), [status, setStatus] = useState("idle"), prefix = usePrefix({
        status: status,
        theme: theme
    }), searchTimeoutRef = useRef(), items = useMemo((() => normalizeChoices(config.choices)), [ config.choices ]), bounds = useMemo((() => {
        const first = items.findIndex(isSelectable), last = items.findLastIndex(isSelectable);
        if (-1 === first) throw new ValidationError("[select prompt] No selectable choices. All choices are disabled.");
        return {
            first: first,
            last: last
        };
    }), [ items ]), defaultItemIndex = useMemo((() => "default" in config ? items.findIndex((item => isSelectable(item) && item.value === config.default)) : -1), [ config.default, items ]), [active, setActive] = useState(-1 === defaultItemIndex ? bounds.first : defaultItemIndex), selectedChoice = items[active];
    useKeypress(((key, rl) => {
        if (clearTimeout(searchTimeoutRef.current), isEnterKey(key)) setStatus("done"), 
        done(selectedChoice.value); else if (isUpKey(key) || isDownKey(key)) {
            if (rl.clearLine(0), loop || isUpKey(key) && active !== bounds.first || isDownKey(key) && active !== bounds.last) {
                const offset = isUpKey(key) ? -1 : 1;
                let next = active;
                do {
                    next = (next + offset + items.length) % items.length;
                } while (!isSelectable(items[next]));
                setActive(next);
            }
        } else if (isNumberKey(key)) {
            rl.clearLine(0);
            const position = Number(key.name) - 1, item = items[position];
            null != item && isSelectable(item) && setActive(position);
        } else if (isBackspaceKey(key)) rl.clearLine(0); else {
            const searchTerm = rl.line.toLowerCase(), matchIndex = items.findIndex((item => !(Separator.isSeparator(item) || !isSelectable(item)) && item.name.toLowerCase().startsWith(searchTerm)));
            -1 !== matchIndex && setActive(matchIndex), searchTimeoutRef.current = setTimeout((() => {
                rl.clearLine(0);
            }), 700);
        }
    })), useEffect((() => () => {
        clearTimeout(searchTimeoutRef.current);
    }), []);
    const message = theme.style.message(config.message, status);
    let helpTipTop = "", helpTipBottom = "";
    ("always" === theme.helpMode || "auto" === theme.helpMode && firstRender.current) && (firstRender.current = !1, 
    items.length > pageSize ? helpTipBottom = `\n${theme.style.help("(Use arrow keys to reveal more choices)")}` : helpTipTop = theme.style.help("(Use arrow keys)"));
    const page = usePagination({
        items: items,
        active: active,
        renderItem({item: item, isActive: isActive}) {
            if (Separator.isSeparator(item)) return ` ${item.separator}`;
            if (item.disabled) {
                const disabledLabel = "string" == typeof item.disabled ? item.disabled : "(disabled)";
                return theme.style.disabled(`${item.name} ${disabledLabel}`);
            }
            return (isActive ? theme.style.highlight : x => x)(`${isActive ? theme.icon.cursor : " "} ${item.name}`);
        },
        pageSize: pageSize,
        loop: loop
    });
    if ("done" === status) return `${prefix} ${message} ${theme.style.answer(selectedChoice.short)}`;
    const choiceDescription = selectedChoice.description ? `\n${theme.style.description(selectedChoice.description)}` : "";
    return `${[ prefix, message, helpTipTop ].filter(Boolean).join(" ")}\n${page}${helpTipBottom}${choiceDescription}${ansiEscapes.cursorHide}`;
}));

async function promptToInitGitRepo(defaultGitDir, exitOnFailToCreate = !0) {
    return select({
        message: "Would you like to initialize a git repo?",
        default: "y",
        choices: [ {
            name: "(y) Yes",
            short: "y",
            value: !0,
            description: `Default repo directory: ${defaultGitDir}`
        }, {
            name: "(n) No",
            short: "n",
            value: !1,
            description: "Skip it"
        } ]
    }).then((answer => (1 == answer && (initGitRepo(defaultGitDir), !isGitRepo(defaultGitDir) && exitOnFailToCreate && warningOutro(`Unable to initialize a new git repo at: ${defaultGitDir}`)), 
    !0))).catch((err => !1));
}

function cloneCommand() {
    return new Command("clone").configureOutput(cliOutputConfig).description("clone all the accounts and programs listed in the Solana.toml file").addOption(new Option("--force", "force clone all fixtures, even if they exist")).addOption(new Option("--no-prompt", "skip the prompts to override any existing fixtures")).addOption(COMMON_OPTIONS.accountDir).addOption(COMMON_OPTIONS.config).addOption(new Option(COMMON_OPTIONS.url.flags, COMMON_OPTIONS.url.description).default("mainnet")).action((async options => {
        titleMessage("Clone fixtures (accounts and programs)"), await checkCommand("solana account --help", {
            exit: !0,
            message: "Unable to detect the 'solana account' command. Do you have it installed?"
        });
        let targetGitDir = process.cwd();
        isGitRepo(targetGitDir) || (warnMessage("Cloning fixtures without tracking changes via git is not recommended"), 
        await promptToInitGitRepo(targetGitDir));
        let config = loadConfigToml(options.config, options, !0);
        const anchorToml = loadAnchorToml(config.configPath);
        anchorToml && (config = deconflictAnchorTomlWithConfig(anchorToml, config)), updateGitignore([ DEFAULT_CACHE_DIR, DEFAULT_TEST_LEDGER_DIR ]), 
        rmSync(DEFAULT_ACCOUNTS_DIR_TEMP, {
            recursive: !0,
            force: !0
        });
        const currentAccounts = loadFileNamesToMap(config.settings.accountDir), accounts = await cloneAccountsFromConfig(config, options, currentAccounts);
        await cloneTokensFromConfig(config, options, currentAccounts);
        let detectedPrograms = {};
        accounts && (detectedPrograms = mergeOwnersMapWithConfig(accounts.owners), await cloneProgramsFromConfig({
            settings: config.settings,
            clone: {
                program: detectedPrograms
            }
        }, {
            ...options,
            autoClone: !0
        }, currentAccounts)), await cloneProgramsFromConfig(config, options, currentAccounts), 
        createFolders(DEFAULT_ACCOUNTS_DIR_TEMP), moveFiles(DEFAULT_ACCOUNTS_DIR_TEMP, config.settings.accountDir, !0), 
        rmSync(DEFAULT_ACCOUNTS_DIR_TEMP, {
            recursive: !0,
            force: !0
        });
        const cloneCounts = validateExpectedCloneCounts(config.settings.accountDir, config.clone);
        cloneCounts.actual === cloneCounts.expected ? console.log(`Completed cloning ${cloneCounts.actual} ${1 == cloneCounts.actual ? "account" : "accounts"}`) : warnMessage(`Completed cloning fixtures. Expected ${cloneCounts.expected} fixtures, but only ${cloneCounts.actual} found`);
    }));
}

function buildTestValidatorCommand({reset: reset = !1, verbose: verbose = !1, accountDir: accountDir = DEFAULT_ACCOUNTS_DIR, ledgerDir: ledgerDir = DEFAULT_TEST_LEDGER_DIR, authority: authority, localPrograms: localPrograms} = {}) {
    const command = [ "solana-test-validator" ], stagingDir = resolve(DEFAULT_ACCOUNTS_DIR_LOADED);
    if (reset && (rmSync(stagingDir, {
        recursive: !0,
        force: !0
    }), command.push("--reset")), ledgerDir && (createFolders(ledgerDir), command.push(`--ledger ${ledgerDir}`)), 
    accountDir) if (directoryExists(accountDir = resolve(accountDir))) {
        createFolders(stagingDir, !1), moveFiles(accountDir, stagingDir, !0), command.push(`--account-dir ${stagingDir}`);
        loadFileNamesToMap(accountDir, ".so").forEach(((_value, address) => {
            authority ? command.push(`--upgradeable-program ${address}`, resolve(accountDir, `${address}.so`), authority) : command.push(`--bpf-program ${address}`, resolve(accountDir, `${address}.so`));
        }));
    } else verbose && (warnMessage(`Accounts directory does not exist: ${accountDir}`), 
    warnMessage("Skipping cloning of fixtures"));
    if (localPrograms) for (const key in localPrograms) Object.prototype.hasOwnProperty.call(localPrograms, key) && command.push(`--upgradeable-program ${localPrograms[key].address}`, localPrograms[key].filePath, authority);
    return command.join(" ");
}

async function promptToAutoClone() {
    return console.log(), select({
        message: "Would you like to perform the 'clone' command now?",
        default: "y",
        choices: [ {
            name: "(y) Yes",
            short: "y",
            value: !0,
            description: "Yes, clone all the accounts and programs in Solana.toml"
        }, {
            name: "(n) No",
            short: "n",
            value: !1,
            description: "Do not run the 'clone' command now"
        } ]
    }).then((async answer => {
        if (!0 !== answer) return !1;
        await cloneCommand().parseAsync([]);
    })).catch((() => {
        console.log("Operation canceled."), process.exit();
    }));
}

function listLocalPrograms({labels: labels = {}, buildDir: buildDir = DEFAULT_BUILD_DIR, basePath: basePath, configPath: configPath, cluster: cluster = "localnet"} = {}) {
    let allFound = !1, locatedPrograms = {}, buildDirListing = new Map;
    if (!directoryExists(buildDir = resolve(join(basePath || (configPath ? dirname(configPath) : process.cwd()), buildDir)))) return warnMessage(`Unable to locate build output directory: ${buildDir}`), 
    {
        locatedPrograms: locatedPrograms,
        buildDirListing: buildDirListing,
        allFound: allFound
    };
    if (buildDirListing = loadFileNamesToMap(buildDir, ".so"), !Object.prototype.hasOwnProperty.call(labels, cluster)) return {
        locatedPrograms: locatedPrograms,
        buildDirListing: buildDirListing,
        allFound: allFound
    };
    let missingCounter = 0;
    return buildDirListing.forEach(((binaryName, programName) => {
        Object.prototype.hasOwnProperty.call(labels[cluster], programName) && !Object.hasOwn(locatedPrograms, programName) ? locatedPrograms[programName] = {
            address: labels[cluster][programName],
            filePath: join(buildDir, binaryName)
        } : (missingCounter++, Object.prototype.hasOwnProperty.call(labels[cluster], programName) ? warnMessage(`Unable to locate compiled program '${programName}' from config`) : warnMessage(`Compiled program '${programName}' was found with no config info`));
    })), 0 == missingCounter && (allFound = !0), {
        locatedPrograms: locatedPrograms,
        buildDirListing: buildDirListing,
        allFound: allFound
    };
}

function validatorCommand() {
    return 4 === process.argv.length && "validator" == process.argv[2].toLowerCase() && "--version" == process.argv[3].toLowerCase() && (console.log("mucho", getAppInfo().version), 
    console.log(getCommandOutputSync("solana-test-validator --version") || "solana-test-validator not found"), 
    process.exit()), new Command("validator").allowUnknownOption(!0).addHelpText("afterAll", "Additional Options:\n  see 'solana-test-validator --help'").configureOutput(cliOutputConfig).description("run the Solana test validator on your local machine").addOption(new Option("--reset", "reset the test validator to genesis, reloading all preloaded fixtures")).addOption(new Option("-l, --ledger <LEDGER_DIR>", "location for the local ledger").default(DEFAULT_TEST_LEDGER_DIR)).addOption(new Option("--output", "output the generated test validator command while executing it")).addOption(COMMON_OPTIONS.outputOnly).addOption(COMMON_OPTIONS.accountDir).addOption(COMMON_OPTIONS.config).addOption(COMMON_OPTIONS.keypair).addOption(COMMON_OPTIONS.url).action((async (options, {args: passThroughArgs}) => {
        options.outputOnly ? options.output = options.outputOnly : titleMessage("solana-test-validator"), 
        await checkCommand("solana-test-validator --version", {
            exit: !0,
            message: "Unable to detect the 'solana-test-validator'. Do you have it installed?"
        });
        let config = loadConfigToml(options.config, options);
        updateGitignore([ DEFAULT_CACHE_DIR, DEFAULT_TEST_LEDGER_DIR ]);
        let authorityAddress = null;
        config.settings.keypair && (doesFileExist(config.settings.keypair) ? authorityAddress = (await loadKeypairFromFile(config.settings.keypair)).address : (warnMessage(`Unable to locate keypair file: ${config.settings.keypair}`), 
        warnMessage("Skipping auto creation and setting authorities")));
        let locatedPrograms = {};
        const anchorToml = loadAnchorToml(config.configPath);
        anchorToml && (config = deconflictAnchorTomlWithConfig(anchorToml, config), config.programs = deepMerge$1(config.programs, anchorToml.programs)), 
        Object.assign(locatedPrograms, listLocalPrograms({
            configPath: config.configPath,
            labels: config.programs,
            cluster: "localnet"
        }).locatedPrograms), options.reset && await cloneCommand().parseAsync([]);
        const cloneCounts = validateExpectedCloneCounts(config.settings.accountDir, config.clone);
        cloneCounts.actual !== cloneCounts.expected && (warnMessage(`Expected ${cloneCounts.expected} fixtures, but only ${cloneCounts.actual} found.`), 
        options.outputOnly || await promptToAutoClone());
        const command = buildTestValidatorCommand({
            verbose: !options.output,
            reset: options.reset || !1,
            accountDir: config.settings.accountDir,
            ledgerDir: config.settings.ledgerDir,
            authority: authorityAddress,
            localPrograms: locatedPrograms
        });
        options.output && console.log(`\n${command}\n`), options.outputOnly && process.exit(), 
        options.reset && (console.log("Loaded", loadFileNamesToMap(config.settings.accountDir, ".json").size, "accounts into the local validator"), 
        console.log("Loaded", loadFileNamesToMap(config.settings.accountDir, ".so").size, "programs into the local validator"));
        const explorerUrl = new URL("https://explorer.solana.com/?cluster=custom");
        explorerUrl.searchParams.set("customUrl", "http://localhost:8899"), console.log("\nSolana Explorer for your local test validator:"), 
        console.log("(on Brave Browser, you may need to turn Shields down for the Explorer website)"), 
        console.log(explorerUrl.toString()), shellExecInSession({
            command: command,
            args: passThroughArgs,
            outputOnly: options.outputOnly
        });
    }));
}

const DEFAULT_CARGO_TOML_FILE = "Cargo.toml";

function loadCargoToml(manifestPath = DEFAULT_CARGO_TOML_FILE, settings = {}, isManifestRequired = !1) {
    return directoryExists(manifestPath) && (manifestPath = join(manifestPath, DEFAULT_CARGO_TOML_FILE)), 
    !!doesFileExist(manifestPath, !0) && loadTomlFile(manifestPath);
}

function findAllCargoToml(startDir, whitelist = [], blacklist = [ "node_modules", "dist", "target" ], maxDepth = 3) {
    const cargoTomlPaths = [], whitelistPatterns = whitelist.map((pattern => new RegExp("^" + pattern.replace(/\*/g, ".*") + "$")));
    function isWhitelisted(relativeDir) {
        return 0 === whitelistPatterns.length || whitelistPatterns.some((regex => regex.test(relativeDir)));
    }
    return function searchDir(dir, depth) {
        if (depth > maxDepth) return;
        const items = readdirSync(dir);
        for (const item of items) {
            const itemPath = join(dir, item), stats = statSync(itemPath);
            if (stats.isFile() && "Cargo.toml" === item && cargoTomlPaths.push(itemPath), stats.isDirectory()) {
                const relativeDir = relative(startDir, itemPath);
                if (blacklist.includes(relativeDir) && !isWhitelisted(relativeDir)) continue;
                searchDir(itemPath, depth + 1);
            }
        }
    }(startDir, 0), cargoTomlPaths;
}

function getProgramPathsInWorkspace(startDir, workspaceDirs) {
    const programPaths = new Map;
    let tempToml = !1;
    doesFileExist(startDir) && (startDir = dirname(startDir));
    return findAllCargoToml(startDir, workspaceDirs).map((progPath => {
        if (!doesFileExist(progPath)) return;
        if (tempToml = loadCargoToml(progPath), !tempToml || tempToml.workspace) return;
        const name = tempToml?.lib?.name || tempToml?.package?.name;
        name && programPaths.set(name, tempToml.configPath);
    })), programPaths;
}

function autoLocateProgramsInWorkspace(manifestPath = join(process.cwd(), "Cargo.toml"), workspaceDirs = [ "temp", "programs/*", "program" ]) {
    let cargoToml = loadCargoToml(manifestPath);
    cargoToml || workspaceDirs.some((workspace => {
        const filePath = join(process.cwd(), workspace.replace(/\*+$/, ""), "Cargo.toml");
        !doesFileExist(filePath) || (cargoToml = loadCargoToml(filePath));
    }));
    let programs = new Map;
    return cargoToml && (cargoToml.configPath && (manifestPath = cargoToml.configPath), 
    cargoToml.workspace?.members && (workspaceDirs = cargoToml.workspace.members), programs = getProgramPathsInWorkspace(manifestPath, workspaceDirs)), 
    {
        programs: programs,
        cargoToml: cargoToml
    };
}

function buildProgramCommand({verbose: verbose = !1, manifestPath: manifestPath, workspace: workspace = !1, toolsVersion: toolsVersion}) {
    const command = [ "cargo build-sbf" ];
    return manifestPath && command.push(`--manifest-path ${manifestPath}`), workspace && command.push("--workspace"), 
    toolsVersion && (toolsVersion.startsWith("v") || (toolsVersion = `v${toolsVersion}`), 
    command.push(`--tools-version ${toolsVersion}`)), command.join(" ");
}

function buildCommand() {
    return new Command("build").configureOutput(cliOutputConfig).description("build your Solana programs").usage("[options] [-- <CARGO_ARGS>...]").addOption(new Option("-- <CARGO_ARGS>", "arguments to pass to the underlying 'cargo build-sbf' command")).addOption(new Option("-p --program-name <PROGRAM_NAME>", "name of the program to build")).addOption(COMMON_OPTIONS.manifestPath).addOption(COMMON_OPTIONS.config).addOption(COMMON_OPTIONS.outputOnly).action((async (options, {args: passThroughArgs}) => {
        options.outputOnly || titleMessage("Build your Solana programs"), await checkCommand("cargo build-sbf --help", {
            exit: !0,
            message: "Unable to detect the 'cargo build-sbf' command. Do you have it installed?"
        });
        let {programs: programs, cargoToml: cargoToml} = autoLocateProgramsInWorkspace(options.manifestPath);
        if (options.programName && (programs.has(options.programName) && doesFileExist(programs.get(options.programName)) ? cargoToml = loadCargoToml(programs.get(options.programName)) : (warnMessage(`Unable to locate program '${options.programName}' in this workspace`), 
        console.log("The following programs were located:"), programs.forEach(((_programPath, programName) => console.log(" -", programName))), 
        process.exit())), !cargoToml) return warningOutro("Unable to locate Cargo.toml file. Operation canceled.");
        let buildCommand = null, toolsVersion = null;
        const solanaVersion = await installedToolVersion("solana"), solanaTools = getPlatformToolsVersions();
        if (isVersionNewer(solanaVersion || "", "2.0") && !isVersionNewer(solanaTools["platform-tools"], "1.43") && (warnMessage("cargo build-sbf versions >=2.X requires building with platform tools version >=1.43"), 
        toolsVersion = "1.43", warnMessage(`Auto setting platform tools to ${toolsVersion} for this build`)), 
        cargoToml.workspace) console.log("Building all programs in the workspace"), buildCommand = buildProgramCommand({
            manifestPath: cargoToml.configPath,
            workspace: !0,
            toolsVersion: toolsVersion
        }); else {
            if (!cargoToml.package || !cargoToml.lib["crate-type"].includes("lib")) return warningOutro("Unable to locate any program's Cargo.toml file");
            console.log(`Building program '${cargoToml.lib.name || cargoToml.package.name || "[unknown]"}' only`), 
            buildCommand = buildProgramCommand({
                manifestPath: cargoToml.configPath,
                toolsVersion: toolsVersion
            });
        }
        if (!buildCommand) return warningOutro("Unable to create build command");
        shellExecInSession({
            command: buildCommand,
            args: passThroughArgs,
            outputOnly: options.outputOnly
        });
    }));
}

async function promptToInstall(toolName) {
    return select({
        message: `Would you like to install '${toolName}' now?`,
        default: "y",
        choices: [ {
            name: "(y) Yes",
            short: "y",
            value: !0
        }, {
            name: "(n) No",
            short: "n",
            value: !1
        } ]
    }).then((async answer => !0 === answer)).catch((() => {
        console.log("Operation canceled."), process.exit();
    }));
}

const command = "zest coverage";

function coverageCommand() {
    return new Command("coverage").configureOutput(cliOutputConfig).description("run code coverage on a Solana program").usage("[options] [-- <ZEST_ARGS>...]").addOption(new Option("-- <ZEST_ARGS>", `arguments to pass to the underlying ${command} command`)).addOption(COMMON_OPTIONS.outputOnly).action((async (options, {args: passThroughArgs}) => {
        options.outputOnly || titleMessage("Zest code coverage"), await checkCommand("zest --help", {
            exit: !0,
            onError: async () => {
                warnMessage("Unable to detect the 'zest' command.");
                await promptToInstall("zest") && await installZest();
            },
            doubleCheck: !0
        }), shellExecInSession({
            command: command,
            args: passThroughArgs,
            outputOnly: options.outputOnly
        });
    }));
}

function buildDeployProgramCommand({programPath: programPath, programId: programId, verbose: verbose = !1, manifestPath: manifestPath, workspace: workspace = !1, url: url, keypair: keypair, upgradeAuthority: upgradeAuthority}) {
    const command = [ "solana program deploy" ];
    return url && command.push(`--url ${parseRpcUrlOrMoniker(url, !0)}`), keypair && command.push(`--keypair ${keypair}`), 
    upgradeAuthority && command.push(`--upgrade-authority ${upgradeAuthority}`), programId && command.push(`--program-id ${programId}`), 
    command.push(programPath), command.join(" ");
}

async function getDeployedProgramInfo(programId, cluster) {
    const command = [ "solana program show", "--output json", `--url ${parseRpcUrlOrMoniker(cluster, !0)}`, programId ];
    let {stderr: stderr, stdout: stdout} = await src_default$1(command.join(" "));
    if (stderr) return stderr.trim().split("\n"), !1;
    if (!stdout.trim()) return !1;
    const programInfo = parseJson(stdout);
    return "none" == programInfo.authority && (programInfo.authority = !1), programInfo;
}

async function promptToSelectCluster(message = "Select a cluster?", defaultValue = "mainnet") {
    return console.log(), select({
        message: message,
        theme: {},
        default: defaultValue,
        choices: [ {
            short: "m",
            name: "m) mainnet",
            value: "mainnet"
        }, {
            short: "d",
            name: "d) devnet",
            value: "devnet"
        }, {
            short: "t",
            name: "t) testnet",
            value: "testnet"
        }, {
            short: "l",
            name: "l) localnet",
            value: "localnet"
        } ]
    }).then((async answer => (answer.startsWith("m") && (answer = "mainnet"), answer))).catch((() => (console.log("Operation canceled."), 
    process.exit(), defaultValue)));
}

function deployCommand() {
    return new Command("deploy").configureOutput(cliOutputConfig).description("deploy a Solana program").usage("[options] [-- <DEPLOY_ARGS>...]").addOption(new Option("-- <DEPLOY_ARGS>", "arguments to pass to the underlying 'solana program' command")).addOption(new Option("-p --program-name <PROGRAM_NAME>", "name of the program to deploy")).addOption(COMMON_OPTIONS.url).addOption(COMMON_OPTIONS.manifestPath).addOption(COMMON_OPTIONS.keypair).addOption(COMMON_OPTIONS.config).addOption(COMMON_OPTIONS.outputOnly).action((async (options, {args: passThroughArgs}) => {
        options.outputOnly || titleMessage("Deploy a Solana program"), await checkCommand("solana program --help", {
            exit: !0,
            message: "Unable to detect the 'solana program' command"
        });
        const {programs: programs, cargoToml: cargoToml} = autoLocateProgramsInWorkspace(options.manifestPath);
        if (options.programName || 1 != programs.size || (options.programName = programs.entries().next().value[0]), 
        !cargoToml) return warnMessage("Unable to locate Cargo.toml");
        if (programs.has(options.programName) && options.programName || (options.programName ? programs.has(options.programName) || warnMessage(`Unable to locate program '${options.programName}' in this workspace`) : warnMessage("You must select a program to deploy. See --help."), 
        console.log("The following programs were located:"), programs.forEach(((_programPath, programName) => console.log(" -", programName))), 
        process.exit()), !options.url) {
            const cluster = await promptToSelectCluster("Select the cluster to deploy your program on?", getSafeClusterMoniker(cliConfig?.json_rpc_url) || void 0);
            options.url = parseRpcUrlOrMoniker(cluster);
        }
        if (!options.url) return warnMessage("You must select cluster to deploy to. See --help");
        let selectedCluster = getSafeClusterMoniker(options.url);
        if (!selectedCluster) {
            if (selectedCluster = getSafeClusterMoniker(await promptToSelectCluster("Unable to auto detect the cluster to deploy too. Select a cluster?")), 
            !selectedCluster) return warnMessage("Unable to detect cluster to deploy to. Operation canceled.");
        }
        let config = loadConfigToml(options.config, options, !1);
        const buildDir = path.join(path.dirname(cargoToml.configPath), "target", "deploy");
        if (!directoryExists(buildDir)) return warnMessage(`Unable to locate your build dir: ${buildDir}`), 
        warnMessage("Have you built your programs?");
        const binaryPath = path.join(buildDir, `${options.programName}.so`);
        if (!doesFileExist(binaryPath)) return warnMessage(`Unable to locate program binary:\n${binaryPath}`), 
        warnMessage("Have you built your programs?");
        let programId = null, programIdPath = path.join(buildDir, `${options.programName}-keypair.json`);
        const programKeypair = await loadKeypairFromFile(programIdPath);
        if (config?.programs ? (getSafeClusterMoniker(selectedCluster, config.programs) || (warnMessage(`Unable to locate '${selectedCluster}' programs in your Solana.toml`), 
        console.log("The following programs are declared:"), Object.keys(config.programs).forEach((cl => {
            console.log(` - ${cl}:`), Object.keys(config.programs[cl]).forEach((name => {
                console.log(`    - ${name}`);
            }));
        })), process.exit()), config?.programs?.[selectedCluster] && Object.hasOwn(config.programs[selectedCluster], options.programName) || (warnMessage(`Program '${options.programName}' not found in 'programs.${selectedCluster}'`), 
        process.exit()), programId = config.programs[selectedCluster][options.programName]) : programKeypair ? (programId = programKeypair.address, 
        warnMessage("Auto detected default program keypair file:"), console.log(` - keypair path: ${programIdPath}`), 
        console.log(` - program id: ${programId}`)) : (warnMessage("Unable to locate any program id or program keypair."), 
        process.exit()), !programId) return warnMessage(`Unable to locate program id for '${options.programName}'. Do you have it declared?`);
        let programInfo = await getDeployedProgramInfo(programId, options.url);
        if (!programInfo) {
            if (!programKeypair) return warnMessage(`Unable to locate program keypair: ${programIdPath}`);
            const programIdFromKeypair = programKeypair.address;
            programIdFromKeypair !== programId && (warnMessage("The loaded program keypair does NOT match the configured program id"), 
            console.log(` - program keypair: ${programIdFromKeypair}`), console.log(` - declared program id: ${programId}`), 
            warnMessage("Unable to perform initial program deployment. Operation cancelled."), 
            process.exit()), programId = programIdFromKeypair, programInfo = await getDeployedProgramInfo(programId, options.url);
        }
        const authorityKeypair = await loadKeypairFromFile(config.settings.keypair);
        if (programInfo) {
            if (!programInfo.authority) return cancelMessage(`Program ${programInfo.programId} is no longer upgradeable`);
            if (programInfo.authority !== authorityKeypair.address) return cancelMessage(`Your keypair (${authorityKeypair.address}) is not the upgrade authority for program ${programId}`);
            programId = programInfo.programId;
        }
        shellExecInSession({
            command: buildDeployProgramCommand({
                programPath: binaryPath,
                programId: programIdPath || programId,
                url: options.url,
                keypair: options.keypair
            }),
            args: passThroughArgs,
            outputOnly: options.outputOnly
        });
    }));
}

const DOCS_INFO = {
    mucho: {
        url: "https://github.com/solana-developers/mucho?tab=readme-ov-file#mucho",
        desc: "A superset of popular Solana developer tools to simplify Solana program development and testing"
    },
    solana: {
        url: "https://solana.com/docs",
        desc: "Official Solana blockchain technical documentation"
    },
    "solana-cli": {
        url: "https://solana.com/docs/intro/installation#install-the-solana-cli",
        desc: "The Agave CLI tool suite required to build and deploy Solana programs (formerly known as Solana CLI tool suite)"
    },
    rust: {
        url: "https://www.rust-lang.org/learn",
        desc: "Programming language for writing Solana smart contracts"
    },
    node: {
        url: "https://nodejs.org/en/learn",
        desc: "Runtime environment for Solana client applications"
    },
    typescript: {
        url: "https://www.typescriptlang.org/docs",
        desc: "TypeScript language for writing Solana client applications"
    },
    anchor: {
        url: "https://www.anchor-lang.com",
        desc: "Solana's most popular smart contract framework"
    },
    avm: {
        url: "https://www.anchor-lang.com/docs/installation#installing-using-anchor-version-manager-avm-recommended",
        desc: "Anchor Version Manager (AVM) to manage multiple versions of Anchor"
    },
    spl: {
        url: "https://spl.solana.com",
        desc: "The Solana Program Library (SPL) is a collection of standard smart contracts for Solana"
    },
    metaplex: {
        url: "https://developers.metaplex.com",
        desc: "Metaplex NFT standard documentation"
    },
    "solana-verify": {
        url: "https://github.com/Ellipsis-Labs/solana-verifiable-build",
        desc: "CLI tool for verifying Solana program builds match on-chain deployments"
    },
    trident: {
        url: "https://ackee.xyz/trident/docs/latest",
        desc: "Rust-based fuzzing framework for testing Solana programs"
    },
    zest: {
        url: "https://github.com/LimeChain/zest",
        desc: "Code coverage CLI tool for Solana programs"
    },
    nvm: {
        url: "https://github.com/nvm-sh/nvm",
        desc: "Node Version Manager for managing multiple versions of Node.js"
    },
    npm: {
        url: "https://docs.npmjs.com/cli",
        desc: "Default package manager for Node.js - manages dependencies and project scripts"
    },
    pnpm: {
        url: "https://pnpm.io/installation",
        desc: "Fast, disk space efficient package manager for Node.js - alternative to npm"
    },
    yarn: {
        url: "https://yarnpkg.com/getting-started",
        desc: "Alternative Node.js package manager - dependency for Anchor"
    }
}, OPEN_COMMANDS = {
    win32: "start",
    darwin: "open",
    linux: "xdg-open"
};

function docsCommand() {
    const openCommand = OPEN_COMMANDS[process.platform] ?? "xdg-open", TOOLS = Object.keys(DOCS_INFO);
    return new Command("docs").configureOutput(cliOutputConfig).description("Open documentation websites for Solana development tools").usage("[options] [tool]").addArgument(new Argument("[tool]", "tool to open docs for").choices(TOOLS).default("mucho")).addHelpText("after", `\nAvailable documentation:\n${Object.entries(DOCS_INFO).map((([name, {desc: desc, url: url}]) => `  ${name.padEnd(14)}${desc}\n  ${" ".repeat(14)}${url}`)).join("\n\n")}\n\nExamples:\n  $ mucho docs         # open Mucho documentation\n  $ mucho docs solana  # open Solana documentation`).action((async (tool, options) => {
        console.log(picocolors.dim(`Opening: ${DOCS_INFO[tool].url}`)), await src_default$1(`${openCommand} ${DOCS_INFO[tool].url}`);
    }));
}

var debug_1, hasRequiredDebug, utils, hasRequiredUtils;

function requireDebug() {
    if (hasRequiredDebug) return debug_1;
    hasRequiredDebug = 1;
    let messages = [], level = 0;
    const debug = (msg, min) => {
        level >= min && messages.push(msg);
    };
    return debug.WARN = 1, debug.INFO = 2, debug.DEBUG = 3, debug.reset = () => {
        messages = [];
    }, debug.setDebugLevel = v => {
        level = v;
    }, debug.warn = msg => debug(msg, debug.WARN), debug.info = msg => debug(msg, debug.INFO), 
    debug.debug = msg => debug(msg, debug.DEBUG), debug.debugMessages = () => messages, 
    debug_1 = debug;
}

function requireUtils() {
    if (hasRequiredUtils) return utils;
    hasRequiredUtils = 1;
    const stringWidth = requireStringWidth();
    function codeRegex(capture) {
        return capture ? /\u001b\[((?:\d*;){0,5}\d*)m/g : /\u001b\[(?:\d*;){0,5}\d*m/g;
    }
    function strlen(str) {
        let code = codeRegex();
        return ("" + str).replace(code, "").split("\n").reduce((function(memo, s) {
            return stringWidth(s) > memo ? stringWidth(s) : memo;
        }), 0);
    }
    function repeat(str, times) {
        return Array(times + 1).join(str);
    }
    let codeCache = {};
    function addToCodeCache(name, on, off) {
        off = "[" + off + "m", codeCache[on = "[" + on + "m"] = {
            set: name,
            to: !0
        }, codeCache[off] = {
            set: name,
            to: !1
        }, codeCache[name] = {
            on: on,
            off: off
        };
    }
    function updateState(state, controlChars) {
        let controlCode = controlChars[1] ? parseInt(controlChars[1].split(";")[0]) : 0;
        if (controlCode >= 30 && controlCode <= 39 || controlCode >= 90 && controlCode <= 97) return void (state.lastForegroundAdded = controlChars[0]);
        if (controlCode >= 40 && controlCode <= 49 || controlCode >= 100 && controlCode <= 107) return void (state.lastBackgroundAdded = controlChars[0]);
        if (0 === controlCode) {
            for (let i in state) Object.prototype.hasOwnProperty.call(state, i) && delete state[i];
            return;
        }
        let info = codeCache[controlChars[0]];
        info && (state[info.set] = info.to);
    }
    function readState(line) {
        let code = codeRegex(!0), controlChars = code.exec(line), state = {};
        for (;null !== controlChars; ) updateState(state, controlChars), controlChars = code.exec(line);
        return state;
    }
    function unwindState(state, ret) {
        let lastBackgroundAdded = state.lastBackgroundAdded, lastForegroundAdded = state.lastForegroundAdded;
        return delete state.lastBackgroundAdded, delete state.lastForegroundAdded, Object.keys(state).forEach((function(key) {
            state[key] && (ret += codeCache[key].off);
        })), lastBackgroundAdded && "[49m" != lastBackgroundAdded && (ret += "[49m"), 
        lastForegroundAdded && "[39m" != lastForegroundAdded && (ret += "[39m"), ret;
    }
    function rewindState(state, ret) {
        let lastBackgroundAdded = state.lastBackgroundAdded, lastForegroundAdded = state.lastForegroundAdded;
        return delete state.lastBackgroundAdded, delete state.lastForegroundAdded, Object.keys(state).forEach((function(key) {
            state[key] && (ret = codeCache[key].on + ret);
        })), lastBackgroundAdded && "[49m" != lastBackgroundAdded && (ret = lastBackgroundAdded + ret), 
        lastForegroundAdded && "[39m" != lastForegroundAdded && (ret = lastForegroundAdded + ret), 
        ret;
    }
    function truncateWidth(str, desiredLength) {
        if (str.length === strlen(str)) return str.substr(0, desiredLength);
        for (;strlen(str) > desiredLength; ) str = str.slice(0, -1);
        return str;
    }
    function wordWrap(maxLength, input) {
        let whitespace, lines = [], split = input.split(/(\s+)/g), line = [], lineLength = 0;
        for (let i = 0; i < split.length; i += 2) {
            let word = split[i], newLength = lineLength + strlen(word);
            lineLength > 0 && whitespace && (newLength += whitespace.length), newLength > maxLength ? (0 !== lineLength && lines.push(line.join("")), 
            line = [ word ], lineLength = strlen(word)) : (line.push(whitespace || "", word), 
            lineLength = newLength), whitespace = split[i + 1];
        }
        return lineLength && lines.push(line.join("")), lines;
    }
    function textWrap(maxLength, input) {
        let lines = [], line = "";
        function pushLine(str, ws) {
            for (line.length && ws && (line += ws), line += str; line.length > maxLength; ) lines.push(line.slice(0, maxLength)), 
            line = line.slice(maxLength);
        }
        let split = input.split(/(\s+)/g);
        for (let i = 0; i < split.length; i += 2) pushLine(split[i], i && split[i - 1]);
        return line.length && lines.push(line), lines;
    }
    return addToCodeCache("bold", 1, 22), addToCodeCache("italics", 3, 23), addToCodeCache("underline", 4, 24), 
    addToCodeCache("inverse", 7, 27), addToCodeCache("strikethrough", 9, 29), utils = {
        strlen: strlen,
        repeat: repeat,
        pad: function(str, len, pad, dir) {
            let length = strlen(str);
            if (len + 1 >= length) {
                let padlen = len - length;
                switch (dir) {
                  case "right":
                    str = repeat(pad, padlen) + str;
                    break;

                  case "center":
                    {
                        let right = Math.ceil(padlen / 2);
                        str = repeat(pad, padlen - right) + str + repeat(pad, right);
                        break;
                    }

                  default:
                    str += repeat(pad, padlen);
                }
            }
            return str;
        },
        truncate: function(str, desiredLength, truncateChar) {
            if (truncateChar = truncateChar || "…", strlen(str) <= desiredLength) return str;
            let ret = function(str, desiredLength) {
                let myArray, code = codeRegex(!0), split = str.split(codeRegex()), splitIndex = 0, retLen = 0, ret = "", state = {};
                for (;retLen < desiredLength; ) {
                    myArray = code.exec(str);
                    let toAdd = split[splitIndex];
                    if (splitIndex++, retLen + strlen(toAdd) > desiredLength && (toAdd = truncateWidth(toAdd, desiredLength - retLen)), 
                    ret += toAdd, retLen += strlen(toAdd), retLen < desiredLength) {
                        if (!myArray) break;
                        ret += myArray[0], updateState(state, myArray);
                    }
                }
                return unwindState(state, ret);
            }(str, desiredLength -= strlen(truncateChar));
            return ret += truncateChar, str.includes("]8;;") && !ret.includes("]8;;") && (ret += "]8;;"), 
            ret;
        },
        mergeOptions: function(options, defaults) {
            options = options || {}, defaults = defaults || {
                chars: {
                    top: "─",
                    "top-mid": "┬",
                    "top-left": "┌",
                    "top-right": "┐",
                    bottom: "─",
                    "bottom-mid": "┴",
                    "bottom-left": "└",
                    "bottom-right": "┘",
                    left: "│",
                    "left-mid": "├",
                    mid: "─",
                    "mid-mid": "┼",
                    right: "│",
                    "right-mid": "┤",
                    middle: "│"
                },
                truncate: "…",
                colWidths: [],
                rowHeights: [],
                colAligns: [],
                rowAligns: [],
                style: {
                    "padding-left": 1,
                    "padding-right": 1,
                    head: [ "red" ],
                    border: [ "grey" ],
                    compact: !1
                },
                head: []
            };
            let ret = Object.assign({}, defaults, options);
            return ret.chars = Object.assign({}, defaults.chars, options.chars), ret.style = Object.assign({}, defaults.style, options.style), 
            ret;
        },
        wordWrap: function(maxLength, input, wrapOnWordBoundary = !0) {
            let output = [];
            input = input.split("\n");
            const handler = wrapOnWordBoundary ? wordWrap : textWrap;
            for (let i = 0; i < input.length; i++) output.push.apply(output, handler(maxLength, input[i]));
            return output;
        },
        colorizeLines: function(input) {
            let state = {}, output = [];
            for (let i = 0; i < input.length; i++) {
                let line = rewindState(state, input[i]);
                state = readState(line);
                let temp = Object.assign({}, state);
                output.push(unwindState(temp, line));
            }
            return output;
        },
        hyperlink: function(url, text) {
            const OSC = "]", BEL = "", SEP = ";";
            return [ OSC, "8", SEP, SEP, url || text, BEL, text, OSC, "8", SEP, SEP, BEL ].join("");
        }
    }, utils;
}

var layoutManager = {
    exports: {}
}, cell = {
    exports: {}
}, safe = {
    exports: {}
}, colors = {
    exports: {}
}, styles = {
    exports: {}
}, hasRequiredStyles, hasFlag, hasRequiredHasFlag, supportsColors, hasRequiredSupportsColors;

function requireStyles() {
    return hasRequiredStyles || (hasRequiredStyles = 1, function(module) {
        var styles = {};
        module.exports = styles;
        var codes = {
            reset: [ 0, 0 ],
            bold: [ 1, 22 ],
            dim: [ 2, 22 ],
            italic: [ 3, 23 ],
            underline: [ 4, 24 ],
            inverse: [ 7, 27 ],
            hidden: [ 8, 28 ],
            strikethrough: [ 9, 29 ],
            black: [ 30, 39 ],
            red: [ 31, 39 ],
            green: [ 32, 39 ],
            yellow: [ 33, 39 ],
            blue: [ 34, 39 ],
            magenta: [ 35, 39 ],
            cyan: [ 36, 39 ],
            white: [ 37, 39 ],
            gray: [ 90, 39 ],
            grey: [ 90, 39 ],
            brightRed: [ 91, 39 ],
            brightGreen: [ 92, 39 ],
            brightYellow: [ 93, 39 ],
            brightBlue: [ 94, 39 ],
            brightMagenta: [ 95, 39 ],
            brightCyan: [ 96, 39 ],
            brightWhite: [ 97, 39 ],
            bgBlack: [ 40, 49 ],
            bgRed: [ 41, 49 ],
            bgGreen: [ 42, 49 ],
            bgYellow: [ 43, 49 ],
            bgBlue: [ 44, 49 ],
            bgMagenta: [ 45, 49 ],
            bgCyan: [ 46, 49 ],
            bgWhite: [ 47, 49 ],
            bgGray: [ 100, 49 ],
            bgGrey: [ 100, 49 ],
            bgBrightRed: [ 101, 49 ],
            bgBrightGreen: [ 102, 49 ],
            bgBrightYellow: [ 103, 49 ],
            bgBrightBlue: [ 104, 49 ],
            bgBrightMagenta: [ 105, 49 ],
            bgBrightCyan: [ 106, 49 ],
            bgBrightWhite: [ 107, 49 ],
            blackBG: [ 40, 49 ],
            redBG: [ 41, 49 ],
            greenBG: [ 42, 49 ],
            yellowBG: [ 43, 49 ],
            blueBG: [ 44, 49 ],
            magentaBG: [ 45, 49 ],
            cyanBG: [ 46, 49 ],
            whiteBG: [ 47, 49 ]
        };
        Object.keys(codes).forEach((function(key) {
            var val = codes[key], style = styles[key] = [];
            style.open = "[" + val[0] + "m", style.close = "[" + val[1] + "m";
        }));
    }(styles)), styles.exports;
}

function requireHasFlag() {
    return hasRequiredHasFlag ? hasFlag : (hasRequiredHasFlag = 1, hasFlag = function(flag, argv) {
        var terminatorPos = (argv = argv || process.argv).indexOf("--"), prefix = /^-{1,2}/.test(flag) ? "" : "--", pos = argv.indexOf(prefix + flag);
        return -1 !== pos && (-1 === terminatorPos || pos < terminatorPos);
    });
}

function requireSupportsColors() {
    if (hasRequiredSupportsColors) return supportsColors;
    hasRequiredSupportsColors = 1;
    var os = require$$2$1, hasFlag = requireHasFlag(), env = process.env, forceColor = void 0;
    function getSupportLevel(stream) {
        var level = function(stream) {
            if (!1 === forceColor) return 0;
            if (hasFlag("color=16m") || hasFlag("color=full") || hasFlag("color=truecolor")) return 3;
            if (hasFlag("color=256")) return 2;
            if (stream && !stream.isTTY && !0 !== forceColor) return 0;
            var min = forceColor ? 1 : 0;
            if ("win32" === process.platform) {
                var osRelease = os.release().split(".");
                return Number(process.versions.node.split(".")[0]) >= 8 && Number(osRelease[0]) >= 10 && Number(osRelease[2]) >= 10586 ? Number(osRelease[2]) >= 14931 ? 3 : 2 : 1;
            }
            if ("CI" in env) return [ "TRAVIS", "CIRCLECI", "APPVEYOR", "GITLAB_CI" ].some((function(sign) {
                return sign in env;
            })) || "codeship" === env.CI_NAME ? 1 : min;
            if ("TEAMCITY_VERSION" in env) return /^(9\.(0*[1-9]\d*)\.|\d{2,}\.)/.test(env.TEAMCITY_VERSION) ? 1 : 0;
            if ("TERM_PROGRAM" in env) {
                var version = parseInt((env.TERM_PROGRAM_VERSION || "").split(".")[0], 10);
                switch (env.TERM_PROGRAM) {
                  case "iTerm.app":
                    return version >= 3 ? 3 : 2;

                  case "Hyper":
                    return 3;

                  case "Apple_Terminal":
                    return 2;
                }
            }
            return /-256(color)?$/i.test(env.TERM) ? 2 : /^screen|^xterm|^vt100|^rxvt|color|ansi|cygwin|linux/i.test(env.TERM) || "COLORTERM" in env ? 1 : (env.TERM, 
            min);
        }(stream);
        return function(level) {
            return 0 !== level && {
                level: level,
                hasBasic: !0,
                has256: level >= 2,
                has16m: level >= 3
            };
        }(level);
    }
    return hasFlag("no-color") || hasFlag("no-colors") || hasFlag("color=false") ? forceColor = !1 : (hasFlag("color") || hasFlag("colors") || hasFlag("color=true") || hasFlag("color=always")) && (forceColor = !0), 
    "FORCE_COLOR" in env && (forceColor = 0 === env.FORCE_COLOR.length || 0 !== parseInt(env.FORCE_COLOR, 10)), 
    supportsColors = {
        supportsColor: getSupportLevel,
        stdout: getSupportLevel(process.stdout),
        stderr: getSupportLevel(process.stderr)
    };
}

var trap = {
    exports: {}
}, hasRequiredTrap;

function requireTrap() {
    return hasRequiredTrap || (hasRequiredTrap = 1, trap.exports = function(text, options) {
        var result = "";
        text = (text = text || "Run the trap, drop the bass").split("");
        var trap = {
            a: [ "@", "Ą", "Ⱥ", "Ʌ", "Δ", "Λ", "Д" ],
            b: [ "ß", "Ɓ", "Ƀ", "ɮ", "β", "฿" ],
            c: [ "©", "Ȼ", "Ͼ" ],
            d: [ "Ð", "Ɗ", "Ԁ", "ԁ", "Ԃ", "ԃ" ],
            e: [ "Ë", "ĕ", "Ǝ", "ɘ", "Σ", "ξ", "Ҽ", "੬" ],
            f: [ "Ӻ" ],
            g: [ "ɢ" ],
            h: [ "Ħ", "ƕ", "Ң", "Һ", "Ӈ", "Ԋ" ],
            i: [ "༏" ],
            j: [ "Ĵ" ],
            k: [ "ĸ", "Ҡ", "Ӄ", "Ԟ" ],
            l: [ "Ĺ" ],
            m: [ "ʍ", "Ӎ", "ӎ", "Ԡ", "ԡ", "൩" ],
            n: [ "Ñ", "ŋ", "Ɲ", "Ͷ", "Π", "Ҋ" ],
            o: [ "Ø", "õ", "ø", "Ǿ", "ʘ", "Ѻ", "ם", "۝", "๏" ],
            p: [ "Ƿ", "Ҏ" ],
            q: [ "্" ],
            r: [ "®", "Ʀ", "Ȑ", "Ɍ", "ʀ", "Я" ],
            s: [ "§", "Ϟ", "ϟ", "Ϩ" ],
            t: [ "Ł", "Ŧ", "ͳ" ],
            u: [ "Ʊ", "Ս" ],
            v: [ "ט" ],
            w: [ "Ш", "Ѡ", "Ѽ", "൰" ],
            x: [ "Ҳ", "Ӿ", "Ӽ", "ӽ" ],
            y: [ "¥", "Ұ", "Ӌ" ],
            z: [ "Ƶ", "ɀ" ]
        };
        return text.forEach((function(c) {
            c = c.toLowerCase();
            var chars = trap[c] || [ " " ], rand = Math.floor(Math.random() * chars.length);
            result += void 0 !== trap[c] ? trap[c][rand] : c;
        })), result;
    }), trap.exports;
}

var zalgo = {
    exports: {}
}, hasRequiredZalgo;

function requireZalgo() {
    return hasRequiredZalgo || (hasRequiredZalgo = 1, zalgo.exports = function(text, options) {
        text = text || "   he is here   ";
        var soul = {
            up: [ "̍", "̎", "̄", "̅", "̿", "̑", "̆", "̐", "͒", "͗", "͑", "̇", "̈", "̊", "͂", "̓", "̈", "͊", "͋", "͌", "̃", "̂", "̌", "͐", "̀", "́", "̋", "̏", "̒", "̓", "̔", "̽", "̉", "ͣ", "ͤ", "ͥ", "ͦ", "ͧ", "ͨ", "ͩ", "ͪ", "ͫ", "ͬ", "ͭ", "ͮ", "ͯ", "̾", "͛", "͆", "̚" ],
            down: [ "̖", "̗", "̘", "̙", "̜", "̝", "̞", "̟", "̠", "̤", "̥", "̦", "̩", "̪", "̫", "̬", "̭", "̮", "̯", "̰", "̱", "̲", "̳", "̹", "̺", "̻", "̼", "ͅ", "͇", "͈", "͉", "͍", "͎", "͓", "͔", "͕", "͖", "͙", "͚", "̣" ],
            mid: [ "̕", "̛", "̀", "́", "͘", "̡", "̢", "̧", "̨", "̴", "̵", "̶", "͜", "͝", "͞", "͟", "͠", "͢", "̸", "̷", "͡", " ҉" ]
        }, all = [].concat(soul.up, soul.down, soul.mid);
        function randomNumber(range) {
            return Math.floor(Math.random() * range);
        }
        function isChar(character) {
            var bool = !1;
            return all.filter((function(i) {
                bool = i === character;
            })), bool;
        }
        return function(text, options) {
            var counts, l, result = "";
            for (l in (options = options || {}).up = void 0 === options.up || options.up, options.mid = void 0 === options.mid || options.mid, 
            options.down = void 0 === options.down || options.down, options.size = void 0 !== options.size ? options.size : "maxi", 
            text = text.split("")) if (!isChar(l)) {
                switch (result += text[l], counts = {
                    up: 0,
                    down: 0,
                    mid: 0
                }, options.size) {
                  case "mini":
                    counts.up = randomNumber(8), counts.mid = randomNumber(2), counts.down = randomNumber(8);
                    break;

                  case "maxi":
                    counts.up = randomNumber(16) + 3, counts.mid = randomNumber(4) + 1, counts.down = randomNumber(64) + 3;
                    break;

                  default:
                    counts.up = randomNumber(8) + 1, counts.mid = randomNumber(6) / 2, counts.down = randomNumber(8) + 1;
                }
                var arr = [ "up", "mid", "down" ];
                for (var d in arr) for (var index = arr[d], i = 0; i <= counts[index]; i++) options[index] && (result += soul[index][randomNumber(soul[index].length)]);
            }
            return result;
        }(text, options);
    }), zalgo.exports;
}

var america = {
    exports: {}
}, hasRequiredAmerica;

function requireAmerica() {
    return hasRequiredAmerica || (hasRequiredAmerica = 1, america.exports = function(colors) {
        return function(letter, i, exploded) {
            if (" " === letter) return letter;
            switch (i % 3) {
              case 0:
                return colors.red(letter);

              case 1:
                return colors.white(letter);

              case 2:
                return colors.blue(letter);
            }
        };
    }), america.exports;
}

var zebra = {
    exports: {}
}, hasRequiredZebra;

function requireZebra() {
    return hasRequiredZebra || (hasRequiredZebra = 1, zebra.exports = function(colors) {
        return function(letter, i, exploded) {
            return i % 2 == 0 ? letter : colors.inverse(letter);
        };
    }), zebra.exports;
}

var rainbow = {
    exports: {}
}, hasRequiredRainbow;

function requireRainbow() {
    return hasRequiredRainbow || (hasRequiredRainbow = 1, rainbow.exports = function(colors) {
        var rainbowColors = [ "red", "yellow", "green", "blue", "magenta" ];
        return function(letter, i, exploded) {
            return " " === letter ? letter : colors[rainbowColors[i++ % rainbowColors.length]](letter);
        };
    }), rainbow.exports;
}

var random = {
    exports: {}
}, hasRequiredRandom, hasRequiredColors, hasRequiredSafe, hasRequiredCell, hasRequiredLayoutManager, table, hasRequiredTable, cliTable3, hasRequiredCliTable3;

function requireRandom() {
    return hasRequiredRandom || (hasRequiredRandom = 1, random.exports = function(colors) {
        var available = [ "underline", "inverse", "grey", "yellow", "red", "green", "blue", "white", "cyan", "magenta", "brightYellow", "brightRed", "brightGreen", "brightBlue", "brightWhite", "brightCyan", "brightMagenta" ];
        return function(letter, i, exploded) {
            return " " === letter ? letter : colors[available[Math.round(Math.random() * (available.length - 2))]](letter);
        };
    }), random.exports;
}

function requireColors() {
    return hasRequiredColors || (hasRequiredColors = 1, function(module) {
        var colors = {};
        module.exports = colors, colors.themes = {};
        var util = require$$0$4, ansiStyles = colors.styles = requireStyles(), defineProps = Object.defineProperties, newLineRegex = new RegExp(/[\r\n]+/g);
        colors.supportsColor = requireSupportsColors().supportsColor, void 0 === colors.enabled && (colors.enabled = !1 !== colors.supportsColor()), 
        colors.enable = function() {
            colors.enabled = !0;
        }, colors.disable = function() {
            colors.enabled = !1;
        }, colors.stripColors = colors.strip = function(str) {
            return ("" + str).replace(/\x1B\[\d+m/g, "");
        }, colors.stylize = function(str, style) {
            if (!colors.enabled) return str + "";
            var styleMap = ansiStyles[style];
            return !styleMap && style in colors ? colors[style](str) : styleMap.open + str + styleMap.close;
        };
        var matchOperatorsRe = /[|\\{}()[\]^$+*?.]/g;
        function build(_styles) {
            var builder = function builder() {
                return applyStyle.apply(builder, arguments);
            };
            return builder._styles = _styles, builder.__proto__ = proto, builder;
        }
        var ret, styles = (ret = {}, ansiStyles.grey = ansiStyles.gray, Object.keys(ansiStyles).forEach((function(key) {
            ansiStyles[key].closeRe = new RegExp(function(str) {
                if ("string" != typeof str) throw new TypeError("Expected a string");
                return str.replace(matchOperatorsRe, "\\$&");
            }(ansiStyles[key].close), "g"), ret[key] = {
                get: function() {
                    return build(this._styles.concat(key));
                }
            };
        })), ret), proto = defineProps((function() {}), styles);
        function applyStyle() {
            var str = Array.prototype.slice.call(arguments).map((function(arg) {
                return null != arg && arg.constructor === String ? arg : util.inspect(arg);
            })).join(" ");
            if (!colors.enabled || !str) return str;
            for (var newLinesPresent = -1 != str.indexOf("\n"), nestedStyles = this._styles, i = nestedStyles.length; i--; ) {
                var code = ansiStyles[nestedStyles[i]];
                str = code.open + str.replace(code.closeRe, code.open) + code.close, newLinesPresent && (str = str.replace(newLineRegex, (function(match) {
                    return code.close + match + code.open;
                })));
            }
            return str;
        }
        colors.setTheme = function(theme) {
            if ("string" != typeof theme) for (var style in theme) !function(style) {
                colors[style] = function(str) {
                    if ("object" == typeof theme[style]) {
                        var out = str;
                        for (var i in theme[style]) out = colors[theme[style][i]](out);
                        return out;
                    }
                    return colors[theme[style]](str);
                };
            }(style); else console.log("colors.setTheme now only accepts an object, not a string.  If you are trying to set a theme from a file, it is now your (the caller's) responsibility to require the file.  The old syntax looked like colors.setTheme(__dirname + '/../themes/generic-logging.js'); The new syntax looks like colors.setTheme(require(__dirname + '/../themes/generic-logging.js'));");
        };
        var sequencer = function(map, str) {
            var exploded = str.split("");
            return (exploded = exploded.map(map)).join("");
        };
        for (var map in colors.trap = requireTrap(), colors.zalgo = requireZalgo(), colors.maps = {}, 
        colors.maps.america = requireAmerica()(colors), colors.maps.zebra = requireZebra()(colors), 
        colors.maps.rainbow = requireRainbow()(colors), colors.maps.random = requireRandom()(colors), 
        colors.maps) !function(map) {
            colors[map] = function(str) {
                return sequencer(colors.maps[map], str);
            };
        }(map);
        defineProps(colors, function() {
            var ret = {};
            return Object.keys(styles).forEach((function(name) {
                ret[name] = {
                    get: function() {
                        return build([ name ]);
                    }
                };
            })), ret;
        }());
    }(colors)), colors.exports;
}

function requireSafe() {
    return hasRequiredSafe || (hasRequiredSafe = 1, function(module) {
        var colors = requireColors();
        module.exports = colors;
    }(safe)), safe.exports;
}

function requireCell() {
    if (hasRequiredCell) return cell.exports;
    hasRequiredCell = 1;
    const {info: info, debug: debug} = requireDebug(), utils = requireUtils();
    class Cell {
        constructor(options) {
            this.setOptions(options), this.x = null, this.y = null;
        }
        setOptions(options) {
            -1 !== [ "boolean", "number", "bigint", "string" ].indexOf(typeof options) && (options = {
                content: "" + options
            }), options = options || {}, this.options = options;
            let content = options.content;
            if (-1 !== [ "boolean", "number", "bigint", "string" ].indexOf(typeof content)) this.content = String(content); else {
                if (content) throw new Error("Content needs to be a primitive, got: " + typeof content);
                this.content = this.options.href || "";
            }
            this.colSpan = options.colSpan || 1, this.rowSpan = options.rowSpan || 1, this.options.href && Object.defineProperty(this, "href", {
                get() {
                    return this.options.href;
                }
            });
        }
        mergeTableOptions(tableOptions, cells) {
            this.cells = cells;
            let optionsChars = this.options.chars || {}, tableChars = tableOptions.chars, chars = this.chars = {};
            CHAR_NAMES.forEach((function(name) {
                setOption(optionsChars, tableChars, name, chars);
            })), this.truncate = this.options.truncate || tableOptions.truncate;
            let style = this.options.style = this.options.style || {}, tableStyle = tableOptions.style;
            setOption(style, tableStyle, "padding-left", this), setOption(style, tableStyle, "padding-right", this), 
            this.head = style.head || tableStyle.head, this.border = style.border || tableStyle.border, 
            this.fixedWidth = tableOptions.colWidths[this.x], this.lines = this.computeLines(tableOptions), 
            this.desiredWidth = utils.strlen(this.content) + this.paddingLeft + this.paddingRight, 
            this.desiredHeight = this.lines.length;
        }
        computeLines(tableOptions) {
            const tableWordWrap = tableOptions.wordWrap || tableOptions.textWrap, {wordWrap: wordWrap = tableWordWrap} = this.options;
            if (this.fixedWidth && wordWrap) {
                if (this.fixedWidth -= this.paddingLeft + this.paddingRight, this.colSpan) {
                    let i = 1;
                    for (;i < this.colSpan; ) this.fixedWidth += tableOptions.colWidths[this.x + i], 
                    i++;
                }
                const {wrapOnWordBoundary: tableWrapOnWordBoundary = !0} = tableOptions, {wrapOnWordBoundary: wrapOnWordBoundary = tableWrapOnWordBoundary} = this.options;
                return this.wrapLines(utils.wordWrap(this.fixedWidth, this.content, wrapOnWordBoundary));
            }
            return this.wrapLines(this.content.split("\n"));
        }
        wrapLines(computedLines) {
            const lines = utils.colorizeLines(computedLines);
            return this.href ? lines.map((line => utils.hyperlink(this.href, line))) : lines;
        }
        init(tableOptions) {
            let x = this.x, y = this.y;
            this.widths = tableOptions.colWidths.slice(x, x + this.colSpan), this.heights = tableOptions.rowHeights.slice(y, y + this.rowSpan), 
            this.width = this.widths.reduce(sumPlusOne, -1), this.height = this.heights.reduce(sumPlusOne, -1), 
            this.hAlign = this.options.hAlign || tableOptions.colAligns[x], this.vAlign = this.options.vAlign || tableOptions.rowAligns[y], 
            this.drawRight = x + this.colSpan == tableOptions.colWidths.length;
        }
        draw(lineNum, spanningCell) {
            if ("top" == lineNum) return this.drawTop(this.drawRight);
            if ("bottom" == lineNum) return this.drawBottom(this.drawRight);
            let content = utils.truncate(this.content, 10, this.truncate);
            lineNum || info(`${this.y}-${this.x}: ${this.rowSpan - lineNum}x${this.colSpan} Cell ${content}`);
            let padTop, padLen = Math.max(this.height - this.lines.length, 0);
            switch (this.vAlign) {
              case "center":
                padTop = Math.ceil(padLen / 2);
                break;

              case "bottom":
                padTop = padLen;
                break;

              default:
                padTop = 0;
            }
            if (lineNum < padTop || lineNum >= padTop + this.lines.length) return this.drawEmpty(this.drawRight, spanningCell);
            let forceTruncation = this.lines.length > this.height && lineNum + 1 >= this.height;
            return this.drawLine(lineNum - padTop, this.drawRight, forceTruncation, spanningCell);
        }
        drawTop(drawRight) {
            let content = [];
            return this.cells ? this.widths.forEach((function(width, index) {
                content.push(this._topLeftChar(index)), content.push(utils.repeat(this.chars[0 == this.y ? "top" : "mid"], width));
            }), this) : (content.push(this._topLeftChar(0)), content.push(utils.repeat(this.chars[0 == this.y ? "top" : "mid"], this.width))), 
            drawRight && content.push(this.chars[0 == this.y ? "topRight" : "rightMid"]), this.wrapWithStyleColors("border", content.join(""));
        }
        _topLeftChar(offset) {
            let leftChar, x = this.x + offset;
            if (0 == this.y) leftChar = 0 == x ? "topLeft" : 0 == offset ? "topMid" : "top"; else if (0 == x) leftChar = "leftMid"; else if (leftChar = 0 == offset ? "midMid" : "bottomMid", 
            this.cells) {
                if (this.cells[this.y - 1][x] instanceof Cell.ColSpanCell && (leftChar = 0 == offset ? "topMid" : "mid"), 
                0 == offset) {
                    let i = 1;
                    for (;this.cells[this.y][x - i] instanceof Cell.ColSpanCell; ) i++;
                    this.cells[this.y][x - i] instanceof Cell.RowSpanCell && (leftChar = "leftMid");
                }
            }
            return this.chars[leftChar];
        }
        wrapWithStyleColors(styleProperty, content) {
            if (!this[styleProperty] || !this[styleProperty].length) return content;
            try {
                let colors = requireSafe();
                for (let i = this[styleProperty].length - 1; i >= 0; i--) colors = colors[this[styleProperty][i]];
                return colors(content);
            } catch (e) {
                return content;
            }
        }
        drawLine(lineNum, drawRight, forceTruncationSymbol, spanningCell) {
            let left = this.chars[0 == this.x ? "left" : "middle"];
            if (this.x && spanningCell && this.cells) {
                let cellLeft = this.cells[this.y + spanningCell][this.x - 1];
                for (;cellLeft instanceof ColSpanCell; ) cellLeft = this.cells[cellLeft.y][cellLeft.x - 1];
                cellLeft instanceof RowSpanCell || (left = this.chars.rightMid);
            }
            let leftPadding = utils.repeat(" ", this.paddingLeft), right = drawRight ? this.chars.right : "", rightPadding = utils.repeat(" ", this.paddingRight), line = this.lines[lineNum], len = this.width - (this.paddingLeft + this.paddingRight);
            forceTruncationSymbol && (line += this.truncate || "…");
            let content = utils.truncate(line, len, this.truncate);
            return content = utils.pad(content, len, " ", this.hAlign), content = leftPadding + content + rightPadding, 
            this.stylizeLine(left, content, right);
        }
        stylizeLine(left, content, right) {
            return left = this.wrapWithStyleColors("border", left), right = this.wrapWithStyleColors("border", right), 
            0 === this.y && (content = this.wrapWithStyleColors("head", content)), left + content + right;
        }
        drawBottom(drawRight) {
            let left = this.chars[0 == this.x ? "bottomLeft" : "bottomMid"], content = utils.repeat(this.chars.bottom, this.width), right = drawRight ? this.chars.bottomRight : "";
            return this.wrapWithStyleColors("border", left + content + right);
        }
        drawEmpty(drawRight, spanningCell) {
            let left = this.chars[0 == this.x ? "left" : "middle"];
            if (this.x && spanningCell && this.cells) {
                let cellLeft = this.cells[this.y + spanningCell][this.x - 1];
                for (;cellLeft instanceof ColSpanCell; ) cellLeft = this.cells[cellLeft.y][cellLeft.x - 1];
                cellLeft instanceof RowSpanCell || (left = this.chars.rightMid);
            }
            let right = drawRight ? this.chars.right : "", content = utils.repeat(" ", this.width);
            return this.stylizeLine(left, content, right);
        }
    }
    class ColSpanCell {
        constructor() {}
        draw(lineNum) {
            return "number" == typeof lineNum && debug(`${this.y}-${this.x}: 1x1 ColSpanCell`), 
            "";
        }
        init() {}
        mergeTableOptions() {}
    }
    class RowSpanCell {
        constructor(originalCell) {
            this.originalCell = originalCell;
        }
        init(tableOptions) {
            let y = this.y, originalY = this.originalCell.y;
            this.cellOffset = y - originalY, this.offset = function(dimensionTable, startingIndex, span) {
                let ret = dimensionTable[startingIndex];
                for (let i = 1; i < span; i++) ret += 1 + dimensionTable[startingIndex + i];
                return ret;
            }(tableOptions.rowHeights, originalY, this.cellOffset);
        }
        draw(lineNum) {
            return "top" == lineNum ? this.originalCell.draw(this.offset, this.cellOffset) : "bottom" == lineNum ? this.originalCell.draw("bottom") : (debug(`${this.y}-${this.x}: 1x${this.colSpan} RowSpanCell for ${this.originalCell.content}`), 
            this.originalCell.draw(this.offset + 1 + lineNum));
        }
        mergeTableOptions() {}
    }
    function firstDefined(...args) {
        return args.filter((v => null != v)).shift();
    }
    function setOption(objA, objB, nameB, targetObj) {
        let nameA = nameB.split("-");
        nameA.length > 1 ? (nameA[1] = nameA[1].charAt(0).toUpperCase() + nameA[1].substr(1), 
        nameA = nameA.join(""), targetObj[nameA] = firstDefined(objA[nameA], objA[nameB], objB[nameA], objB[nameB])) : targetObj[nameB] = firstDefined(objA[nameB], objB[nameB]);
    }
    function sumPlusOne(a, b) {
        return a + b + 1;
    }
    let CHAR_NAMES = [ "top", "top-mid", "top-left", "top-right", "bottom", "bottom-mid", "bottom-left", "bottom-right", "left", "left-mid", "mid", "mid-mid", "right", "right-mid", "middle" ];
    return cell.exports = Cell, cell.exports.ColSpanCell = ColSpanCell, cell.exports.RowSpanCell = RowSpanCell, 
    cell.exports;
}

function requireLayoutManager() {
    if (hasRequiredLayoutManager) return layoutManager.exports;
    hasRequiredLayoutManager = 1;
    const {warn: warn, debug: debug} = requireDebug(), Cell = requireCell(), {ColSpanCell: ColSpanCell, RowSpanCell: RowSpanCell} = Cell;
    function makeComputeWidths(colSpan, desiredWidth, x, forcedMin) {
        return function(vals, table) {
            let result = [], spanners = [], auto = {};
            table.forEach((function(row) {
                row.forEach((function(cell) {
                    (cell[colSpan] || 1) > 1 ? spanners.push(cell) : result[cell[x]] = Math.max(result[cell[x]] || 0, cell[desiredWidth] || 0, forcedMin);
                }));
            })), vals.forEach((function(val, index) {
                "number" == typeof val && (result[index] = val);
            }));
            for (let k = spanners.length - 1; k >= 0; k--) {
                let cell = spanners[k], span = cell[colSpan], col = cell[x], existingWidth = result[col], editableCols = "number" == typeof vals[col] ? 0 : 1;
                if ("number" == typeof existingWidth) for (let i = 1; i < span; i++) existingWidth += 1 + result[col + i], 
                "number" != typeof vals[col + i] && editableCols++; else existingWidth = "desiredWidth" === desiredWidth ? cell.desiredWidth - 1 : 1, 
                (!auto[col] || auto[col] < existingWidth) && (auto[col] = existingWidth);
                if (cell[desiredWidth] > existingWidth) {
                    let i = 0;
                    for (;editableCols > 0 && cell[desiredWidth] > existingWidth; ) {
                        if ("number" != typeof vals[col + i]) {
                            let dif = Math.round((cell[desiredWidth] - existingWidth) / editableCols);
                            existingWidth += dif, result[col + i] += dif, editableCols--;
                        }
                        i++;
                    }
                }
            }
            Object.assign(vals, result, auto);
            for (let j = 0; j < vals.length; j++) vals[j] = Math.max(forcedMin, vals[j] || 0);
        };
    }
    return function() {
        function next(alloc, col) {
            return alloc[col] > 0 ? next(alloc, col + 1) : col;
        }
        function layoutTable(table) {
            let alloc = {};
            table.forEach((function(row, rowIndex) {
                let col = 0;
                row.forEach((function(cell) {
                    cell.y = rowIndex, cell.x = rowIndex ? next(alloc, col) : col;
                    const rowSpan = cell.rowSpan || 1, colSpan = cell.colSpan || 1;
                    if (rowSpan > 1) for (let cs = 0; cs < colSpan; cs++) alloc[cell.x + cs] = rowSpan;
                    col = cell.x + colSpan;
                })), Object.keys(alloc).forEach((idx => {
                    alloc[idx]--, alloc[idx] < 1 && delete alloc[idx];
                }));
            }));
        }
        function maxWidth(table) {
            let mw = 0;
            return table.forEach((function(row) {
                row.forEach((function(cell) {
                    mw = Math.max(mw, cell.x + (cell.colSpan || 1));
                }));
            })), mw;
        }
        function cellsConflict(cell1, cell2) {
            let yMin1 = cell1.y, yMax1 = cell1.y - 1 + (cell1.rowSpan || 1), yMin2 = cell2.y, yConflict = !(yMin1 > cell2.y - 1 + (cell2.rowSpan || 1) || yMin2 > yMax1), xMin1 = cell1.x, xMax1 = cell1.x - 1 + (cell1.colSpan || 1), xMin2 = cell2.x, xMax2 = cell2.x - 1 + (cell2.colSpan || 1);
            return yConflict && !(xMin1 > xMax2 || xMin2 > xMax1);
        }
        function conflictExists(rows, x, y) {
            let i_max = Math.min(rows.length - 1, y), cell = {
                x: x,
                y: y
            };
            for (let i = 0; i <= i_max; i++) {
                let row = rows[i];
                for (let j = 0; j < row.length; j++) if (cellsConflict(cell, row[j])) return !0;
            }
            return !1;
        }
        function allBlank(rows, y, xMin, xMax) {
            for (let x = xMin; x < xMax; x++) if (conflictExists(rows, x, y)) return !1;
            return !0;
        }
        function addRowSpanCells(table) {
            table.forEach((function(row, rowIndex) {
                row.forEach((function(cell) {
                    for (let i = 1; i < cell.rowSpan; i++) {
                        let rowSpanCell = new RowSpanCell(cell);
                        rowSpanCell.x = cell.x, rowSpanCell.y = cell.y + i, rowSpanCell.colSpan = cell.colSpan, 
                        insertCell(rowSpanCell, table[rowIndex + i]);
                    }
                }));
            }));
        }
        function insertCell(cell, row) {
            let x = 0;
            for (;x < row.length && row[x].x < cell.x; ) x++;
            row.splice(x, 0, cell);
        }
        function fillInTable(table) {
            let h_max = function(table) {
                return table.length;
            }(table), w_max = maxWidth(table);
            debug(`Max rows: ${h_max}; Max cols: ${w_max}`);
            for (let y = 0; y < h_max; y++) for (let x = 0; x < w_max; x++) if (!conflictExists(table, x, y)) {
                let opts = {
                    x: x,
                    y: y,
                    colSpan: 1,
                    rowSpan: 1
                };
                for (x++; x < w_max && !conflictExists(table, x, y); ) opts.colSpan++, x++;
                let y2 = y + 1;
                for (;y2 < h_max && allBlank(table, y2, opts.x, opts.x + opts.colSpan); ) opts.rowSpan++, 
                y2++;
                let cell = new Cell(opts);
                cell.x = opts.x, cell.y = opts.y, warn(`Missing cell at ${cell.y}-${cell.x}.`), 
                insertCell(cell, table[y]);
            }
        }
        layoutManager.exports = {
            makeTableLayout: function(rows) {
                let cellRows = function(rows) {
                    return rows.map((function(row) {
                        if (!Array.isArray(row)) {
                            let key = Object.keys(row)[0];
                            row = row[key], Array.isArray(row) ? (row = row.slice()).unshift(key) : row = [ key, row ];
                        }
                        return row.map((function(cell) {
                            return new Cell(cell);
                        }));
                    }));
                }(rows);
                return layoutTable(cellRows), fillInTable(cellRows), addRowSpanCells(cellRows), 
                function(cellRows) {
                    for (let rowIndex = cellRows.length - 1; rowIndex >= 0; rowIndex--) {
                        let cellColumns = cellRows[rowIndex];
                        for (let columnIndex = 0; columnIndex < cellColumns.length; columnIndex++) {
                            let cell = cellColumns[columnIndex];
                            for (let k = 1; k < cell.colSpan; k++) {
                                let colSpanCell = new ColSpanCell;
                                colSpanCell.x = cell.x + k, colSpanCell.y = cell.y, cellColumns.splice(columnIndex + 1, 0, colSpanCell);
                            }
                        }
                    }
                }(cellRows), cellRows;
            },
            layoutTable: layoutTable,
            addRowSpanCells: addRowSpanCells,
            maxWidth: maxWidth,
            fillInTable: fillInTable,
            computeWidths: makeComputeWidths("colSpan", "desiredWidth", "x", 1),
            computeHeights: makeComputeWidths("rowSpan", "desiredHeight", "y", 1)
        };
    }(), layoutManager.exports;
}

function requireTable() {
    if (hasRequiredTable) return table;
    hasRequiredTable = 1;
    const debug = requireDebug(), utils = requireUtils(), tableLayout = requireLayoutManager();
    class Table extends Array {
        constructor(opts) {
            super();
            const options = utils.mergeOptions(opts);
            if (Object.defineProperty(this, "options", {
                value: options,
                enumerable: options.debug
            }), options.debug) {
                switch (typeof options.debug) {
                  case "boolean":
                    debug.setDebugLevel(debug.WARN);
                    break;

                  case "number":
                    debug.setDebugLevel(options.debug);
                    break;

                  case "string":
                    debug.setDebugLevel(parseInt(options.debug, 10));
                    break;

                  default:
                    debug.setDebugLevel(debug.WARN), debug.warn("Debug option is expected to be boolean, number, or string. Received a " + typeof options.debug);
                }
                Object.defineProperty(this, "messages", {
                    get: () => debug.debugMessages()
                });
            }
        }
        toString() {
            let array = this, headersPresent = this.options.head && this.options.head.length;
            headersPresent ? (array = [ this.options.head ], this.length && array.push.apply(array, this)) : this.options.style.head = [];
            let cells = tableLayout.makeTableLayout(array);
            cells.forEach((function(row) {
                row.forEach((function(cell) {
                    cell.mergeTableOptions(this.options, cells);
                }), this);
            }), this), tableLayout.computeWidths(this.options.colWidths, cells), tableLayout.computeHeights(this.options.rowHeights, cells), 
            cells.forEach((function(row) {
                row.forEach((function(cell) {
                    cell.init(this.options);
                }), this);
            }), this);
            let result = [];
            for (let rowIndex = 0; rowIndex < cells.length; rowIndex++) {
                let row = cells[rowIndex], heightOfRow = this.options.rowHeights[rowIndex];
                (0 === rowIndex || !this.options.style.compact || 1 == rowIndex && headersPresent) && doDraw(row, "top", result);
                for (let lineNum = 0; lineNum < heightOfRow; lineNum++) doDraw(row, lineNum, result);
                rowIndex + 1 == cells.length && doDraw(row, "bottom", result);
            }
            return result.join("\n");
        }
        get width() {
            return this.toString().split("\n")[0].length;
        }
    }
    function doDraw(row, lineNum, result) {
        let line = [];
        row.forEach((function(cell) {
            line.push(cell.draw(lineNum));
        }));
        let str = line.join("");
        str.length && result.push(str);
    }
    return Table.reset = () => debug.reset(), table = Table;
}

function requireCliTable3() {
    return hasRequiredCliTable3 ? cliTable3 : (hasRequiredCliTable3 = 1, cliTable3 = requireTable());
}

var cliTable3Exports = requireCliTable3(), CliTable3 = getDefaultExportFromCjs(cliTable3Exports), ComputeBudgetInstruction = (ComputeBudgetInstruction2 => (ComputeBudgetInstruction2[ComputeBudgetInstruction2.RequestUnits = 0] = "RequestUnits", 
ComputeBudgetInstruction2[ComputeBudgetInstruction2.RequestHeapFrame = 1] = "RequestHeapFrame", 
ComputeBudgetInstruction2[ComputeBudgetInstruction2.SetComputeUnitLimit = 2] = "SetComputeUnitLimit", 
ComputeBudgetInstruction2[ComputeBudgetInstruction2.SetComputeUnitPrice = 3] = "SetComputeUnitPrice", 
ComputeBudgetInstruction2[ComputeBudgetInstruction2.SetLoadedAccountsDataSizeLimit = 4] = "SetLoadedAccountsDataSizeLimit", 
ComputeBudgetInstruction2))(ComputeBudgetInstruction || {});

function identifyComputeBudgetInstruction(instruction) {
    const data = "data" in instruction ? instruction.data : instruction;
    if (containsBytes(data, getU8Encoder().encode(0), 0)) return 0;
    if (containsBytes(data, getU8Encoder().encode(1), 0)) return 1;
    if (containsBytes(data, getU8Encoder().encode(2), 0)) return 2;
    if (containsBytes(data, getU8Encoder().encode(3), 0)) return 3;
    if (containsBytes(data, getU8Encoder().encode(4), 0)) return 4;
    throw new Error("The provided instruction could not be identified as a computeBudget instruction.");
}

function getRequestHeapFrameInstructionDataDecoder() {
    return getStructDecoder([ [ "discriminator", getU8Decoder() ], [ "bytes", getU32Decoder() ] ]);
}

function parseRequestHeapFrameInstruction(instruction) {
    return {
        programAddress: instruction.programAddress,
        data: getRequestHeapFrameInstructionDataDecoder().decode(instruction.data)
    };
}

function getRequestUnitsInstructionDataDecoder() {
    return getStructDecoder([ [ "discriminator", getU8Decoder() ], [ "units", getU32Decoder() ], [ "additionalFee", getU32Decoder() ] ]);
}

function parseRequestUnitsInstruction(instruction) {
    return {
        programAddress: instruction.programAddress,
        data: getRequestUnitsInstructionDataDecoder().decode(instruction.data)
    };
}

function getSetComputeUnitLimitInstructionDataDecoder() {
    return getStructDecoder([ [ "discriminator", getU8Decoder() ], [ "units", getU32Decoder() ] ]);
}

function parseSetComputeUnitLimitInstruction(instruction) {
    return {
        programAddress: instruction.programAddress,
        data: getSetComputeUnitLimitInstructionDataDecoder().decode(instruction.data)
    };
}

function getSetComputeUnitPriceInstructionDataDecoder() {
    return getStructDecoder([ [ "discriminator", getU8Decoder() ], [ "microLamports", getU64Decoder() ] ]);
}

function parseSetComputeUnitPriceInstruction(instruction) {
    return {
        programAddress: instruction.programAddress,
        data: getSetComputeUnitPriceInstructionDataDecoder().decode(instruction.data)
    };
}

function getSetLoadedAccountsDataSizeLimitInstructionDataDecoder() {
    return getStructDecoder([ [ "discriminator", getU8Decoder() ], [ "accountDataSizeLimit", getU32Decoder() ] ]);
}

function parseSetLoadedAccountsDataSizeLimitInstruction(instruction) {
    return {
        programAddress: instruction.programAddress,
        data: getSetLoadedAccountsDataSizeLimitInstructionDataDecoder().decode(instruction.data)
    };
}

function getPublicSolanaRpcUrl(cluster) {
    switch (cluster) {
      case "devnet":
        return devnet("https://api.devnet.solana.com");

      case "testnet":
        return testnet("https://api.testnet.solana.com");

      case "mainnet":
      case "mainnet-beta":
        return mainnet("https://api.mainnet-beta.solana.com");

      case "localnet":
      case "localhost":
        return "http://127.0.0.1:8899";

      default:
        throw new Error("Invalid cluster moniker");
    }
}

const GENESIS_HASH = {
    mainnet: "5eykt4UsFv8P8NJdTREpY1vzqKqZKvdpKuc147dw2N9d",
    devnet: "EtWTRABZaYq6iMfeYKouRu166VU2xqa1wcaWoxPkrZBG",
    testnet: "4uhcVJyU9pJkvQyS88uRDiswHXSCkY3zQawwpjk2NsNY"
};

async function getMonikerFromGenesisHash(args) {
    if ("rpc" in args) {
        const hash = await args.rpc.getGenesisHash().send();
        args = {
            hash: hash
        };
    }
    if ("hash" in args) switch (args.hash) {
      case GENESIS_HASH.mainnet:
        return "mainnet-beta";

      case GENESIS_HASH.devnet:
        return "devnet";

      case GENESIS_HASH.testnet:
        return "testnet";

      default:
        return "localnet";
    }
    throw Error("Unable to process genesis hash or rpc connection");
}

function lamportsToSol(lamports) {
    return new Intl.NumberFormat("en-US", {
        maximumFractionDigits: 9
    }).format(Number(lamports) / 1e9);
}

function getExplorerLink(props) {
    let url = null;
    if (props.cluster || (props.cluster = "mainnet-beta"), "address" in props ? url = new URL(`https://explorer.solana.com/address/${props.address}`) : "transaction" in props ? url = new URL(`https://explorer.solana.com/tx/${props.transaction}`) : "block" in props && (url = new URL(`https://explorer.solana.com/block/${props.block}`)), 
    !url) throw new Error("Invalid Solana Explorer URL created");
    return "mainnet-beta" !== props.cluster && ("localnet" === props.cluster ? (url.searchParams.set("cluster", "custom"), 
    url.searchParams.set("customUrl", "http://localhost:8899")) : url.searchParams.set("cluster", props.cluster)), 
    url;
}

function unixTimestampToDate(blockTime) {
    return new Date(1e3 * Number(blockTime));
}

const COMPUTE_BUDGET_PROGRAM_ID = address("ComputeBudget111111111111111111111111111111"), VOTE_PROGRAM_ID = address("Vote111111111111111111111111111111111111111");

function getComputeBudgetDataFromTransaction(tx) {
    const budget = {
        unitsConsumed: Number(tx.meta.computeUnitsConsumed),
        unitsRequested: null,
        unitLimit: null,
        unitPrice: null,
        accountDataSizeLimit: null,
        heapFrameSize: null
    }, computeBudgetIndex = tx.transaction.message.accountKeys.findIndex((address => address == COMPUTE_BUDGET_PROGRAM_ID));
    return tx.transaction.message.instructions.filter((ix => ix.programIdIndex == computeBudgetIndex)).map((ix => {
        const data = getBase58Encoder().encode(ix.data);
        switch (identifyComputeBudgetInstruction(data)) {
          case ComputeBudgetInstruction.SetComputeUnitPrice:
            {
                const {data: {microLamports: microLamports}} = parseSetComputeUnitPriceInstruction({
                    data: data,
                    programAddress: COMPUTE_BUDGET_PROGRAM_ID
                });
                return void (budget.unitPrice = Number(microLamports));
            }

          case ComputeBudgetInstruction.SetComputeUnitLimit:
            {
                const {data: {units: units}} = parseSetComputeUnitLimitInstruction({
                    data: data,
                    programAddress: COMPUTE_BUDGET_PROGRAM_ID
                });
                return void (budget.unitLimit = units);
            }

          case ComputeBudgetInstruction.RequestUnits:
            {
                const {data: {units: units}} = parseRequestUnitsInstruction({
                    data: data,
                    programAddress: COMPUTE_BUDGET_PROGRAM_ID
                });
                return void (budget.unitsRequested = units);
            }

          case ComputeBudgetInstruction.SetLoadedAccountsDataSizeLimit:
            {
                const {data: {accountDataSizeLimit: accountDataSizeLimit}} = parseSetLoadedAccountsDataSizeLimitInstruction({
                    data: data,
                    programAddress: COMPUTE_BUDGET_PROGRAM_ID
                });
                return void (budget.accountDataSizeLimit = accountDataSizeLimit);
            }

          case ComputeBudgetInstruction.RequestHeapFrame:
            {
                const {data: {bytes: bytes}} = parseRequestHeapFrameInstruction({
                    data: data,
                    programAddress: COMPUTE_BUDGET_PROGRAM_ID
                });
                return void (budget.heapFrameSize = bytes);
            }
        }
    })), budget;
}

async function inspectSignature({rpc: rpc, cluster: cluster, signature: signature}) {
    const spinner = ora("Fetching transaction").start();
    try {
        const explorerUrl = getExplorerLink({
            cluster: cluster,
            transaction: signature
        }).toString(), tx = await rpc.getTransaction(signature, {
            commitment: "confirmed",
            maxSupportedTransactionVersion: 0
        }).send();
        if (!tx) throw "Transaction not found. Try the Solana Explorer:\n" + explorerUrl;
        const allAccounts = tx.transaction.message.accountKeys.concat(tx.meta.loadedAddresses.writable).concat(tx.meta.loadedAddresses.readonly), indexToProgramIds = new Map;
        tx.transaction.message.instructions.map((ix => indexToProgramIds.set(ix.programIdIndex, allAccounts[ix.programIdIndex])));
        const overviewTable = buildTransactionOverview({
            tx: tx,
            allAccounts: allAccounts,
            indexToProgramIds: indexToProgramIds
        }), accountsTable = buildAccountsTable({
            tx: tx,
            allAccounts: allAccounts,
            indexToProgramIds: indexToProgramIds
        }), ixsTable = buildInstructionsTable({
            tx: tx,
            allAccounts: allAccounts,
            indexToProgramIds: indexToProgramIds
        });
        spinner.stop(), console.log(accountsTable.toString()), console.log(ixsTable.toString()), 
        console.log(overviewTable.toString()), console.log("Open on Solana Explorer:"), 
        console.log(explorerUrl);
    } finally {
        spinner.stop();
    }
}

function buildTransactionOverview({tx: tx}) {
    const table = new CliTable3({
        head: [ "Transaction Overview", tx.meta.err ? picocolors.red("FAILED") : picocolors.green("SUCCESS") ],
        style: {
            head: [ tx.meta.err ? "red" : "green" ]
        }
    }), blockTime = unixTimestampToDate(tx.blockTime);
    table.push([ "Timestamp", blockTime.toLocaleDateString(void 0, {
        dateStyle: "medium"
    }) + " " + blockTime.toLocaleTimeString(void 0, {
        timeZoneName: "short"
    }) + `\n(${timeAgo(blockTime)})` ]), table.push([ "Version", tx.version ]), table.push([ "Slot", (new Intl.NumberFormat).format(tx.slot) ]), 
    table.push([ "Fee (SOL)", "~" + lamportsToSol(tx.meta.fee) ]);
    const budget = getComputeBudgetDataFromTransaction(tx);
    return table.push([ "Compute units consumed", (new Intl.NumberFormat).format(budget.unitsConsumed) ]), 
    table.push([ "Compute units requested", budget.unitsRequested ? (new Intl.NumberFormat).format(budget.unitsRequested) : picocolors.red(`NONE SET - fallback to ${(new Intl.NumberFormat).format(2e5 * tx.transaction.message.instructions.length)}`) ]), 
    table.push([ "Compute unit price (in microLamports)", budget.unitPrice ? (new Intl.NumberFormat).format(budget.unitPrice) : picocolors.red("NONE") ]), 
    table;
}

function buildInstructionsTable({tx: tx, allAccounts: allAccounts, indexToProgramIds: indexToProgramIds}) {
    const logs = parseProgramLogs(tx.meta.logMessages, tx.meta.err), failedIx = logs.findIndex((ix => ix.failed)), table = new CliTable3({
        head: [ "#", "Instruction & Logs" ],
        style: {
            head: [ tx.meta.err ? "red" : "green" ]
        },
        wordWrap: !0,
        colWidths: [ 3, 90 ]
    });
    return tx.transaction.message.instructions.map(((ix, index) => {
        let content = [ indexToProgramIds.get(ix.programIdIndex) + " Program Instruction" ];
        logs[index].logs.find((log => "error" == log.style)) && (content[0] = picocolors.red(content[0])), 
        logs[index].logs.map((log => {
            const text = log.prefix + log.text;
            "error" == log.style ? content.push(picocolors.red(text)) : "success" == log.style ? content.push(picocolors.green(text)) : content.push(text);
        })), failedIx == index && (content.push(`Accounts Used (${ix.accounts.length}):`), 
        ix.accounts.map((index => {
            content.push("  #" + (1 == index.toString().length ? `${index} ` : index) + " - " + allAccounts[index]);
        }))), table.push([ index, content.join("\n") ]);
    })), table;
}

function buildAccountsTable({tx: tx}) {
    const table = new CliTable3({
        head: [ "#", "Address", "Details" ],
        style: {
            head: [ tx.meta.err ? "red" : "green" ]
        }
    });
    let counter = 0;
    const {numRequiredSignatures: numRequiredSignatures, numReadonlySignedAccounts: numReadonlySignedAccounts, numReadonlyUnsignedAccounts: numReadonlyUnsignedAccounts} = tx.transaction.message.header, indexToProgramIds = new Map;
    function isInFailedIx(index) {
        return !(!tx.meta.err || !tx.transaction.message.instructions[tx.transaction.message.instructions.length - 1].accounts.includes(index));
    }
    return tx.transaction.message.instructions.map((ix => indexToProgramIds.set(ix.programIdIndex, tx.transaction.message.accountKeys[ix.programIdIndex]))), 
    tx.transaction.message.accountKeys.map(((address, index) => {
        let details = [];
        const numUnsignedAccounts = index - numRequiredSignatures;
        0 == index && details.push("Fee Payer"), index < numRequiredSignatures && details.push("Signer"), 
        (index >= numRequiredSignatures && numUnsignedAccounts < tx.transaction.message.accountKeys.length - numRequiredSignatures - numReadonlyUnsignedAccounts || index < numRequiredSignatures - numReadonlySignedAccounts) && details.push("Writable"), 
        indexToProgramIds.has(index) && details.push("Program"), table.push([ counter, isInFailedIx(index) ? picocolors.red(address) : address, details.join(", ").trim() ]), 
        counter++;
    })), tx.meta.loadedAddresses.writable.map(((address, index) => {
        table.push([ counter, isInFailedIx(index) ? picocolors.red(address) : address, [ "Address Lookup Table", "Writable" ].join(", ").trim() ]), 
        counter++;
    })), tx.meta.loadedAddresses.readonly.map(((address, index) => {
        table.push([ counter, isInFailedIx(index) ? picocolors.red(address) : address, [ "Address Lookup Table" ].join(", ").trim() ]), 
        counter++;
    })), table;
}

function parseProgramLogs(logs, error) {
    let depth = 0;
    const prettyLogs = [];
    function prefixBuilder(indentLevel) {
        let prefix;
        return prefix = indentLevel <= 0 ? "" : new Array(indentLevel - 1).fill("  ").join(""), 
        prefix + "> ";
    }
    let prettyError;
    if (error && (prettyError = getTransactionInstructionError(error)), logs.forEach((log => {
        if (log.startsWith("Program data:")) ; else if (log.startsWith("Program log:")) log = log.replace(/Program log: (.*)/g, ((match, p1) => `Program logged: "${p1}"`)), 
        prettyLogs[prettyLogs.length - 1].logs.push({
            prefix: prefixBuilder(depth),
            style: "muted",
            text: log
        }); else if (log.startsWith("Log truncated")) prettyLogs[prettyLogs.length - 1].truncated = !0; else {
            const regex = /Program (\w*) invoke \[(\d)\]/g, matches = Array.from(log.matchAll(regex));
            if (matches.length > 0) {
                const programAddress = matches[0][1];
                0 === depth ? prettyLogs.push({
                    computeUnits: 0,
                    failed: !1,
                    invokedProgram: programAddress,
                    logs: [],
                    truncated: !1
                }) : prettyLogs[prettyLogs.length - 1].logs.push({
                    prefix: prefixBuilder(depth),
                    style: "info",
                    text: `Program invoked: ${programAddress}`
                }), depth++;
            } else if (log.includes("success")) prettyLogs[prettyLogs.length - 1].logs.push({
                prefix: prefixBuilder(depth),
                style: "success",
                text: "Program returned success"
            }), depth--; else if (log.includes("failed")) {
                const instructionLog = prettyLogs[prettyLogs.length - 1];
                instructionLog.failed = !0;
                let currText = `Program returned error: "${log.slice(log.indexOf(": ") + 2)}"`;
                log.startsWith("failed") && (depth++, currText = log.charAt(0).toUpperCase() + log.slice(1)), 
                instructionLog.logs.push({
                    prefix: prefixBuilder(depth),
                    style: "error",
                    text: currText
                }), depth--;
            } else 0 === depth && (prettyLogs.push({
                computeUnits: 0,
                failed: !1,
                invokedProgram: null,
                logs: [],
                truncated: !1
            }), depth++), log = log.replace(/Program \w* consumed (\d*) (.*)/g, ((match, p1, p2) => (1 === depth && (prettyLogs[prettyLogs.length - 1].computeUnits += Number.parseInt(p1)), 
            `Program consumed: ${p1} ${p2}`))), prettyLogs[prettyLogs.length - 1].logs.push({
                prefix: prefixBuilder(depth),
                style: "muted",
                text: log
            });
        }
    })), prettyError && 0 === prettyLogs.length && prettyLogs.push({
        computeUnits: 0,
        failed: !0,
        invokedProgram: null,
        logs: [],
        truncated: !1
    }), prettyError && prettyError.index === prettyLogs.length - 1) {
        const failedIx = prettyLogs[prettyError.index];
        failedIx.failed || (failedIx.failed = !0, failedIx.logs.push({
            prefix: prefixBuilder(1),
            style: "error",
            text: `Runtime error: ${prettyError.message}`
        }));
    }
    return prettyLogs;
}

const instructionErrorMessage = new Map([ [ "GenericError", "generic instruction error" ], [ "InvalidArgument", "invalid program argument" ], [ "InvalidInstructionData", "invalid instruction data" ], [ "InvalidAccountData", "invalid account data for instruction" ], [ "AccountDataTooSmall", "account data too small for instruction" ], [ "InsufficientFunds", "insufficient funds for instruction" ], [ "IncorrectProgramId", "incorrect program id for instruction" ], [ "MissingRequiredSignature", "missing required signature for instruction" ], [ "AccountAlreadyInitialized", "instruction requires an uninitialized account" ], [ "UninitializedAccount", "instruction requires an initialized account" ], [ "UnbalancedInstruction", "sum of account balances before and after instruction do not match" ], [ "ModifiedProgramId", "instruction modified the program id of an account" ], [ "ExternalAccountLamportSpend", "instruction spent from the balance of an account it does not own" ], [ "ExternalAccountDataModified", "instruction modified data of an account it does not own" ], [ "ReadonlyLamportChange", "instruction changed the balance of a read-only account" ], [ "ReadonlyDataModified", "instruction modified data of a read-only account" ], [ "DuplicateAccountIndex", "instruction contains duplicate accounts" ], [ "ExecutableModified", "instruction changed executable bit of an account" ], [ "RentEpochModified", "instruction modified rent epoch of an account" ], [ "NotEnoughAccountKeys", "insufficient account keys for instruction" ], [ "AccountDataSizeChanged", "non-system instruction changed account size" ], [ "AccountNotExecutable", "instruction expected an executable account" ], [ "AccountBorrowFailed", "instruction tries to borrow reference for an account which is already borrowed" ], [ "AccountBorrowOutstanding", "instruction left account with an outstanding borrowed reference" ], [ "DuplicateAccountOutOfSync", "instruction modifications of multiply-passed account differ" ], [ "Custom", "custom program error: {0}" ], [ "InvalidError", "program returned invalid error code" ], [ "ExecutableDataModified", "instruction changed executable accounts data" ], [ "ExecutableLamportChange", "instruction changed the balance of a executable account" ], [ "ExecutableAccountNotRentExempt", "executable accounts must be rent exempt" ], [ "UnsupportedProgramId", "Unsupported program id" ], [ "CallDepth", "Cross-program invocation call depth too deep" ], [ "MissingAccount", "An account required by the instruction is missing" ], [ "ReentrancyNotAllowed", "Cross-program invocation reentrancy not allowed for this instruction" ], [ "MaxSeedLengthExceeded", "Length of the seed is too long for address generation" ], [ "InvalidSeeds", "Provided seeds do not result in a valid address" ], [ "InvalidRealloc", "Failed to reallocate account data" ], [ "ComputationalBudgetExceeded", "Computational budget exceeded" ], [ "PrivilegeEscalation", "Cross-program invocation with unauthorized signer or writable account" ], [ "ProgramEnvironmentSetupFailure", "Failed to create program execution environment" ], [ "ProgramFailedToComplete", "Program failed to complete" ], [ "ProgramFailedToCompile", "Program failed to compile" ], [ "Immutable", "Account is immutable" ], [ "IncorrectAuthority", "Incorrect authority provided" ], [ "BorshIoError", "Failed to serialize or deserialize account data: {0}" ], [ "AccountNotRentExempt", "An account does not have enough lamports to be rent-exempt" ], [ "InvalidAccountOwner", "Invalid account owner" ], [ "ArithmeticOverflow", "Program arithmetic overflowed" ], [ "UnsupportedSysvar", "Unsupported sysvar" ], [ "IllegalOwner", "Provided owner is not allowed" ] ]);

function getTransactionInstructionError(error) {
    if (error && "object" == typeof error && "InstructionError" in error) {
        const innerError = error.InstructionError;
        return {
            index: innerError[0],
            message: getInstructionError(innerError[1])
        };
    }
}

function getInstructionError(error) {
    let out, value;
    if ("string" == typeof error) {
        const message = instructionErrorMessage.get(error);
        if (message) return message;
    } else "Custom" in error ? (out = instructionErrorMessage.get("Custom"), value = error.Custom) : "BorshIoError" in error && (out = instructionErrorMessage.get("BorshIoError"), 
    value = error.BorshIoError);
    return out && value ? out.replace("{0}", value) : "Unknown instruction error";
}

async function inspectAddress({rpc: rpc, cluster: cluster, address: address, commitment: commitment = "confirmed"}) {
    const spinner = ora("Fetching account").start();
    try {
        const explorerUrl = getExplorerLink({
            cluster: cluster,
            address: address
        }).toString(), account = await rpc.getAccountInfo(address, {
            commitment: commitment,
            encoding: "base64"
        }).send();
        if (!account) throw "Account not found. Try the Solana Explorer:\n" + explorerUrl;
        const overviewTable = buildAccountOverview({
            account: account
        });
        spinner.stop(), console.log(overviewTable.toString()), console.log("Open on Solana Explorer:"), 
        console.log(explorerUrl);
    } finally {
        spinner.stop();
    }
}

function buildAccountOverview({account: {value: account}}) {
    const table = new CliTable3({
        head: [ "Account Overview" ],
        style: {
            head: [ "green" ]
        }
    });
    return table.push([ "Owner", account.owner ]), table.push([ "Executable", account.executable ]), 
    table.push([ "Balance (lamports)", (new Intl.NumberFormat).format(account.lamports) ]), 
    table.push([ "Balance (sol)", lamportsToSol(account.lamports) ]), table.push([ "Space", (new Intl.NumberFormat).format(account.space) + " bytes" ]), 
    table;
}

async function inspectBlock({rpc: rpc, cluster: cluster, block: blockNumber, commitment: commitment = "confirmed"}) {
    const spinner = ora("Fetching block, this could take a few moments").start();
    try {
        if ("string" == typeof blockNumber) try {
            blockNumber = parseInt(numberStringToNumber(blockNumber).toString());
        } catch (err) {
            throw "Invalid block number provided";
        }
        if ("bigint" != typeof blockNumber && "number" != typeof blockNumber) throw "Provided block number is not an actual number";
        const explorerUrl = getExplorerLink({
            cluster: cluster,
            block: blockNumber.toString()
        }).toString(), [block, leaders] = await Promise.all([ await rpc.getBlock(BigInt(blockNumber), {
            commitment: commitment,
            maxSupportedTransactionVersion: 0,
            rewards: !0
        }).send(), rpc.getSlotLeaders(BigInt(blockNumber), 1).send() ]);
        if (!block) throw "Block not found. Try the Solana Explorer:\n" + explorerUrl;
        const overviewTable = buildBlockOverview({
            block: block,
            leader: leaders[0]
        });
        spinner.stop(), console.log(overviewTable.toString()), console.log("Open on Solana Explorer:"), 
        console.log(explorerUrl);
    } finally {
        spinner.stop();
    }
}

function buildBlockOverview({block: block, leader: leader}) {
    const table = new CliTable3({
        head: [ "Block Overview" ],
        style: {
            head: [ "green" ]
        }
    }), blockTime = unixTimestampToDate(block.blockTime), successfulTxs = block.transactions.filter((tx => null === tx.meta?.err)), voteTxs = block.transactions.filter((tx => tx.transaction.message.accountKeys.find((account => account == VOTE_PROGRAM_ID)))), failedTxCount = block.transactions.length - successfulTxs.length, nonVoteTxCount = block.transactions.length - voteTxs.length, computeBudgetTxs = block.transactions.filter((tx => tx.transaction.message.accountKeys.find((account => account == COMPUTE_BUDGET_PROGRAM_ID))));
    return table.push([ "Timestamp", blockTime.toLocaleDateString(void 0, {
        dateStyle: "medium"
    }) + " " + blockTime.toLocaleTimeString(void 0, {
        timeZoneName: "short"
    }) + `\n(${timeAgo(blockTime)})` ]), table.push([ "Leader", leader ]), table.push([ "Blockhash", block.blockhash ]), 
    table.push([ "Previous blockhash", block.previousBlockhash ]), table.push([ "Block height", (new Intl.NumberFormat).format(block.blockHeight) ]), 
    table.push([ "Parent slot", (new Intl.NumberFormat).format(block.parentSlot) ]), 
    table.push([ "Total transactions", (new Intl.NumberFormat).format(block.transactions.length) ]), 
    table.push([ "Successful transactions", (new Intl.NumberFormat).format(successfulTxs.length) + ` (${new Intl.NumberFormat(void 0, {
        style: "percent"
    }).format(successfulTxs.length / block.transactions.length)})` ]), table.push([ "Failed transactions", (new Intl.NumberFormat).format(failedTxCount) + ` (${new Intl.NumberFormat(void 0, {
        style: "percent"
    }).format(failedTxCount / block.transactions.length)})` ]), table.push([ "Vote transactions", (new Intl.NumberFormat).format(voteTxs.length) + ` (${new Intl.NumberFormat(void 0, {
        style: "percent"
    }).format(voteTxs.length / block.transactions.length)})` ]), table.push([ "Non-vote transactions", (new Intl.NumberFormat).format(nonVoteTxCount) + ` (${new Intl.NumberFormat(void 0, {
        style: "percent"
    }).format(nonVoteTxCount / block.transactions.length)})` ]), table.push([ "Compute budget transactions", (new Intl.NumberFormat).format(computeBudgetTxs.length) + ` (${new Intl.NumberFormat(void 0, {
        style: "percent"
    }).format(computeBudgetTxs.length / block.transactions.length)})` ]), table;
}

const helpText = [ "Examples:", "  Account:", "    mucho inspect GQuioVe2yA6KZfstgmirAvugfUZBcdxSi7sHK7JGk3gk", "    mucho inspect --url devnet nicktrLHhYzLmoVbuZQzHUTicd2sfP571orwo9jfc8c", "  Transaction:", "    mucho inspect --url mainnet 5tb4d17U4ZsBa2gkNFEmVsrnaQ6wCzQ51i4iPkh2p9S2mnfZezcmWQUogVMK3mBZBtgMxKSqe242vAxuV6FyTYPf", "  Block:", "    mucho inspect 315667873", `    mucho inspect --url mainnet ${Intl.NumberFormat().format(315667873)} (with your locale formatted numbers)` ];

function inspectCommand() {
    return new Command("inspect").configureOutput(cliOutputConfig).description("inspect transactions, accounts, etc (like a block explorer)").addHelpText("after", helpText.join("\n")).addArgument(new Argument("<INPUT>", "transaction signature, account address, or block number to inspect").argRequired()).addOption(COMMON_OPTIONS.url).addOption(COMMON_OPTIONS.config).action((async (input, options) => {
        titleMessage("Inspector");
        const cliConfig = loadSolanaCliConfig();
        let clusterUrl = parseRpcUrlOrMoniker(options.url || cliConfig.json_rpc_url);
        if (input.match(/^https?:\/\//gi)) try {
            const url = new URL(input);
            if ("explorer.solana.com" != url.hostname.toLowerCase()) return errorMessage("Only the https://explorer.solana.com is supported");
            let clusterFromUrl = (url.searchParams.get("cluster") || "mainnet").toLowerCase();
            if ("custom" == clusterFromUrl && (clusterFromUrl = url.searchParams.has("customUrl") ? url.searchParams.get("customUrl") : "localhost"), 
            clusterUrl = parseRpcUrlOrMoniker(clusterFromUrl), url.pathname.match(/^\/address\//gi)) input = url.pathname.match(/^\/address\/(.*)\/?/i)[1]; else if (url.pathname.match(/^\/(tx|transaction)\//gi)) input = url.pathname.match(/^\/(tx|transaction)\/(.*)\/?/i)[2]; else {
                if (!url.pathname.match(/^\/(block)\//gi)) return warningOutro("Unsupported explorer URL");
                input = url.pathname.match(/^\/block\/(.*)\/?/i)[1];
            }
        } catch (err) {
            return errorMessage("Unable to parse inspector input as valid URL");
        }
        clusterUrl = clusterUrl.startsWith("http") ? clusterUrl : getPublicSolanaRpcUrl(clusterUrl);
        const rpc = createSolanaRpc(clusterUrl);
        try {
            const selectedCluster = await getMonikerFromGenesisHash({
                rpc: rpc
            });
            isAddress(input) ? await inspectAddress({
                rpc: rpc,
                cluster: selectedCluster,
                address: address(input)
            }) : isSignature(input) ? await inspectSignature({
                rpc: rpc,
                cluster: selectedCluster,
                signature: signature(input)
            }) : isStringifiedNumber(numberStringToNumber(input).toString()) ? await inspectBlock({
                rpc: rpc,
                cluster: selectedCluster,
                block: input
            }) : warnMessage("Unable to determine your 'INPUT'. Check it's formatting and try again :)");
        } catch (err) {
            err instanceof TypeError ? warnMessage("A network error occurred while connecting to your configured RPC endpoint.\nIs your RPC connection available at: " + clusterUrl) : warnMessage(err);
        }
    }));
}

async function main() {
    try {
        await checkForSelfUpdate();
        try {
            const program = cliProgramRoot();
            program.addCommand(installCommand()).addCommand(validatorCommand()).addCommand(inspectCommand()).addCommand(cloneCommand()).addCommand(buildCommand()).addCommand(deployCommand()).addCommand(coverageCommand()).addCommand(infoCommand()).addCommand(docsCommand()), 
            2 === process.argv.length && process.argv.push("--help"), await program.parseAsync();
        } catch (err) {
            errorOutro(err.toString());
        }
    } catch (err) {
        errorOutro(err, "[mucho - unhandled error]");
    }
    console.log();
}

assertRuntimeVersion(), main();
